# Cuckoo search

From Wikipedia, the free encyclopedia

Jump to: [navigation](Special:Random.html#mw-navigation), [search](Special:Random.html#p-search)

This article is about the search algorithm. For the hashing algorithm, see [cuckoo hashing](http://en.wikipedia.org/wiki/Cuckoo_hashing).

**Cuckoo search (CS)** is an [optimization](http://en.wikipedia.org/wiki/Optimization_\(mathematics\)) [algorithm](http://en.wikipedia.org/wiki/Algorithm) developed by [Xin-she Yang](http://en.wikipedia.org/wiki/Xin-she_Yang) and Suash Deb in 2009.[[1]](Special:Random.html#cite_note-NaBIC-1)[[2]](Special:Random.html#cite_note-2) It was inspired by the [obligate brood parasitism](http://en.wikipedia.org/wiki/Obligate_parasite) of some [cuckoo](http://en.wikipedia.org/wiki/Cuckoo) species by laying their eggs in the nests of other host birds (of other species). Some host birds can engage direct conflict with the intruding cuckoos. For example, if a host bird discovers the eggs are not their own, it will either throw these alien eggs away or simply abandon its nest and build a new nest elsewhere. Some cuckoo species such as the [New World](http://en.wikipedia.org/wiki/New_World) brood-parasitic [Tapera](http://en.wikipedia.org/wiki/Tapera) have evolved in such a way that female parasitic cuckoos are often very specialized in the mimicry in colors and pattern of the eggs of a few chosen host species [[3]](Special:Random.html#cite_note-3)

Cuckoo search idealized such breeding behavior, and thus can be applied for various optimization problems. It seems that it can outperform other metaheuristic algorithms in applications.[[4]](Special:Random.html#cite_note-4)

Cuckoo search (CS) uses the following representations:

Each egg in a nest represents a solution, and a cuckoo egg represents a new solution. The aim is to use the new and potentially better solutions (cuckoos) to replace a not-so-good solution in the nests. In the simplest form, each nest has one egg. The algorithm can be extended to more complicated cases in which each nest has multiple eggs representing a set of solutions.

CS is based on three idealized rules:

  1. Each cuckoo lays one egg at a time, and dumps its egg in a randomly chosen nest;
  2. The best nests with high quality of eggs will carry over to the next generation;
  3. The number of available hosts nests is fixed, and the egg laid by a cuckoo is discovered by the host bird with a probability ![p_a \\in \(0,1\)](../../upload.wikimedia.org/math/a/9/4/a94a9fd82e15e64856944d67d58b1837.png). Discovering operate on some set of worst nests, and discovered solutions dumped from farther calculations.

In addition, Yang and Deb discovered that the random-walk style search is better performed by [Lévy flights](http://en.wikipedia.org/wiki/L%C3%A9vy_flight) rather than simple [random walk](http://en.wikipedia.org/wiki/Random_walk).

The [pseudo-code](http://en.wikipedia.org/wiki/Pseudo-code) can be summarized as:
    
    
    Objective function: ![f\(\\mathbf{x}\), \\quad \\mathbf{x}=\(x_1,x_2,\\dots,x_d\); \\, ](../../upload.wikimedia.org/math/6/c/2/6c24672cb606fad4a730557cfd60be50.png)
    Generate an initial population of ![ n ](../../upload.wikimedia.org/math/7/b/8/7b8b965ad4bca0e41ab51de7b31363a1.png) host nests; 
    While (t<MaxGeneration) or (stop criterion)
       Get a cuckoo randomly (say, i) and replace its solution by performing Lévy flights;
       Evaluate its quality/fitness ![F_i ](../../upload.wikimedia.org/math/2/8/4/284b3567801d1127ea5db52b3fe71319.png)
             [For maximization, ![F_i \\propto f\(\\mathbf{x}_i\)](../../upload.wikimedia.org/math/9/a/4/9a4b946f605bed2f3f1df95b92d7c3e7.png) ];
       Choose a nest among n (say, j) randomly;
       if (![F_i>F_j ](../../upload.wikimedia.org/math/b/d/a/bda5125924a81e23d589db83f02c47e2.png)),
              Replace j by the new solution;
       end if
       A fraction (![p_a](../../upload.wikimedia.org/math/9/7/b/97b9a09ef13c661f91546e1f342126cf.png)) of the worse nests are abandoned and new ones are built;
       Keep the best solutions/nests;
       Rank the solutions/nests and find the current best;
       Pass the current best solutions to the next generation;
    end while
    

An important advantage of this algorithm is its simplicity. In fact, comparing with other population- or agent-based [metaheuristic](http://en.wikipedia.org/wiki/Metaheuristic) algorithms such as [particle swarm optimization](http://en.wikipedia.org/wiki/Particle_swarm_optimization) and [harmony search](http://en.wikipedia.org/wiki/Harmony_search), there is essentially only a single parameter ![p_a](../../upload.wikimedia.org/math/9/7/b/97b9a09ef13c661f91546e1f342126cf.png) in CS (apart from the population size ![n](../../upload.wikimedia.org/math/7/b/8/7b8b965ad4bca0e41ab51de7b31363a1.png)). Therefore, it is very easy to implement.

## Contents

  * [1 Random walks and the step size](Special:Random.html#Random_walks_and_the_step_size)
  * [2 Implementation](Special:Random.html#Implementation)
  * [3 Modified cuckoo search](Special:Random.html#Modified_cuckoo_search)
  * [4 Multiobjective cuckoo search (MOCS)](Special:Random.html#Multiobjective_cuckoo_search_.28MOCS.29)
  * [5 Hybridization](Special:Random.html#Hybridization)
  * [6 Applications](Special:Random.html#Applications)
  * [7 References](Special:Random.html#References)

## Random walks and the step size[[edit](http://en.wikipedia.org/w/index.php?title=Cuckoo_search&action=edit&section=1)]

An important issue is the applications of Lévy flights and random walks in the generic equation for generating new solutions

    ![ \\mathbf{x}_{t+1}=\\mathbf{x}_t + s E_t, ](../../upload.wikimedia.org/math/b/6/3/b6382fd715f4f553282de33d956fb3d5.png)

where ![ E_t ](../../upload.wikimedia.org/math/7/2/6/7269bb1cf7bc648860efd8bb9d0e65de.png) is drawn from a standard normal distribution with zero mean and unity standard deviation for random walks, or drawn from Lévy distribution for Lévy flights. Obviously, the random walks can also be linked with the similarity between a cuckoo's egg and the host's egg which can be tricky in implementation. Here the step size ![s](../../upload.wikimedia.org/math/0/3/c/03c7c0ace395d80182db07ae2c30f034.png) determines how far a random walker can go for a fixed number of iterations. The generation of Lévy step size is often tricky, and a good algorithm is Mantegna's algorithm.[[5]](Special:Random.html#cite_note-5)

If s is too large, then the new solution generated will be too far away from the old solution (or even jump out side of the bounds). Then, such a move is unlikely to be accepted. If s is too small, the change is too small to be significant, and consequently such search is not efficient. So a proper step size is important to maintain the search as efficient as possible.

As an example, for simple isotropic random walks, we know that the average distance ![ r ](../../upload.wikimedia.org/math/4/b/4/4b43b0aee35624cd95b910189b3dc231.png) traveled in the d-dimension space is

    ![ r^2=2 d D t, ](../../upload.wikimedia.org/math/1/0/b/10bdcaea6218ffe986eb7b4d05e515fc.png)

where ![D=s^2/2\\tau](../../upload.wikimedia.org/math/6/c/1/6c117aa6c0e977756977df69370166b1.png) is the effective diffusion coefficient. Here ![ s](../../upload.wikimedia.org/math/0/3/c/03c7c0ace395d80182db07ae2c30f034.png) is the step size or distance traveled at each jump, and ![ \\tau ](../../upload.wikimedia.org/math/8/1/a/81a69207104f00baaabd6f84cafd15a0.png) is the time taken for each jump. The above equation implies that[[6]](Special:Random.html#cite_note-6)

    ![ s^2=\\frac{\\tau \\; r^2}{t \\; d}. ](../../upload.wikimedia.org/math/6/9/a/69aaec36f771663e4ce95fc6ee540aff.png)

For a typical length scale L of a dimension of interest, the local search is typically limited in a region of ![r=L/10 ](../../upload.wikimedia.org/math/c/7/d/c7d65b799ae31a71717391245d1fcead.png). For ![\\tau=1 ](../../upload.wikimedia.org/math/5/1/e/51ee9fe0cc0da469625af68b03c2ce45.png) and t=100 to 1000, we have ![s\\approx 0.01L ](../../upload.wikimedia.org/math/1/f/9/1f9734d268251e4f3ac6e3c6bca990e4.png) for d=1, and ![s \\approx 0.001L ](../../upload.wikimedia.org/math/2/a/1/2a1d041c39c72b18cfc65e3c68248fb5.png) for d=10. Therefore, we can use s/L=0.001 to 0.01 for most problems. Though the exact derivation may require detailed analysis of the behaviour of Lévy flights.[[7]](Special:Random.html#cite_note-7)

Algorithm and convergence analysis will be fruitful, because there are many open problems related to metaheuristics[[8]](Special:Random.html#cite_note-8)

## Implementation[[edit](http://en.wikipedia.org/w/index.php?title=Cuckoo_search&action=edit&section=2)]

The pseudo code was given in a sequential form, but Yang and Deb provides a vectorized implementation which is very efficient.[[9]](Special:Random.html#cite_note-9) In the real world, if a cuckoo's egg is very similar to a host's eggs, then this cuckoo's egg is less likely to be discovered, thus the fitness should be related to the difference in solutions. Therefore, it is a good idea to do a random walk in a biased way with some random step sizes. For Matlab implementation, this biased random walk can partly be achieved by

    stepsize=rand*(nest(randperm(n),:)-nest(randperm(n),:));
    new_nest=nest+stepsize.*K;

where K=rand(size(nest))>pa and pa is the discovery rate.

A standard CS matlab can be found here <http://www.mathworks.com/matlabcentral/fileexchange/29809-cuckoo-search-cs-algorithm>.[[10]](Special:Random.html#cite_note-10) However, there is some controversy on whether or not this implementation truly reflects the originally published cuckoo search algorithm. The biased walk described above closely resembles another optimization algorithm, [differential evolution](http://en.wikipedia.org/wiki/Differential_evolution). It was shown that this biased walk has a larger impact on the performance of the algorithm than the cuckoo search algorithm itself.[[11]](Special:Random.html#cite_note-11)

An object-oriented software implementation of cuckoo search was provided by Bacanin[[12]](Special:Random.html#cite_note-12) On the other hand, a modified cuckoo search algorithm is also implemented for unconstrained optimization problems.[[13]](Special:Random.html#cite_note-13)

An open source implementation of Modified Cuckoo Search can be found here <http://code.google.com/p/modified-cs/>

## Modified cuckoo search[[edit](http://en.wikipedia.org/w/index.php?title=Cuckoo_search&action=edit&section=3)]

A modification of the standard Cuckoo Search was made by Walton et al.[[14]](Special:Random.html#cite_note-14) with the aim to speed up convergence. The modification involves the additional step of information exchange between the top eggs. It was shown that Modified Cuckoo Search (MCS) outperforms the standard cuckoo search and other algorithms, in terms of convergence rates, when applied to a series of standard optimization benchmark objective functions.

Subsequently, the modified cuckoo search has been applied to optimize unstructured mesh which reduces computational cost significantly.[[15]](Special:Random.html#cite_note-15)[[16]](Special:Random.html#cite_note-16) In addition, another interesting improvement to cuckoo search is the so-called quantum-inspired cuckoo search with convincing results [[17]](Special:Random.html#cite_note-17)

## Multiobjective cuckoo search (MOCS)[[edit](http://en.wikipedia.org/w/index.php?title=Cuckoo_search&action=edit&section=4)]

A multiobjective cuckoo search (MOCS) method has been formulated to deal with multi-criteria optimization problems.[[18]](Special:Random.html#cite_note-18) This approach uses random weights to combine multiple objectives to a single objective. As the weights vary randomly, Pareto fronts can be found so the points can distributed diversely over the fronts.

## Hybridization[[edit](http://en.wikipedia.org/w/index.php?title=Cuckoo_search&action=edit&section=5)]

Though cuckoo search is a swarm-intelligence-based algorithm, it can be still hybridized with other swarm-based algorithms such as PSO. For example, a hybrid CS-PSO algorithm seems to remedy the defect of PSO.[[19]](Special:Random.html#cite_note-19)

## Applications[[edit](http://en.wikipedia.org/w/index.php?title=Cuckoo_search&action=edit&section=6)]

The applications of Cuckoo Search into engineering optimization problems[[20]](Special:Random.html#cite_note-20) have shown its promising efficiency. For example, for both spring design and welded beam design problems, CS obtained better solutions than existing solutions in literature. A promising discrete cuckoo search algorithm is recently proposed to solve nurse scheduling problem.[[21]](Special:Random.html#cite_note-21) An efficient computation approach based on cuckoo search has been proposed for data fusion in wireless sensor networks.[[22]](Special:Random.html#cite_note-22)[[23]](Special:Random.html#cite_note-23) Cuckoo search is adapted to solve NP-hard combinatorial optimization problems like [travelling salesman problem](http://en.wikipedia.org/wiki/Travelling_salesman_problem).[[24]](Special:Random.html#cite_note-24) A new quantum-inspired cuckoo search was developed to solve Knapsack problems, which shows its effectiveness.[[25]](Special:Random.html#cite_note-25) Cuckoo search can also be used to efficiently generate independent test paths for structural software testing [[26]](Special:Random.html#cite_note-26) and test data generation.[[27]](Special:Random.html#cite_note-27)

A conceptual comparison of the cuckoo search with [Particle swarm optimization](http://en.wikipedia.org/wiki/Particle_swarm_optimization), [Differential evolution](http://en.wikipedia.org/wiki/Differential_evolution) and [Artificial bee colony algorithm](http://en.wikipedia.org/wiki/Artificial_bee_colony_algorithm) suggest that Cuckoo search and Differential evolution algorithms provide more robust results than Particle swarm optimization and Artificial bee colony algorithm.[[28]](Special:Random.html#cite_note-28) An extensive detailed study of various structural optimization problems suggests that cuckoo search obtains better results than other algorithms.[[29]](Special:Random.html#cite_note-29) In addition, a new software testing approach has been developed based on cuckoo search.[[30]](Special:Random.html#cite_note-30) In addition, cuckoo search is particularly suitable for large scale problems, as shown in a recent study.[[31]](Special:Random.html#cite_note-31) Cuckoo search has been applied to train neural networks with improved performance.[[32]](Special:Random.html#cite_note-32) Furthermore, CS is successfully applied to train spiking neural models.[[33]](Special:Random.html#cite_note-33) Cuckoo search has also been used to optimize web service composition process and planning graphs. [[34]](Special:Random.html#cite_note-34)

Cuckoo search is a reliable approach for embedded system design[[35]](Special:Random.html#cite_note-35) and design optimization[[36]](Special:Random.html#cite_note-36) including optimum design of steel frames.[[37]](Special:Random.html#cite_note-37)

More recent studies suggest that cuckoo search can outperform other algorithms in milling applications,[[38]](Special:Random.html#cite_note-38) manufacturing scheduling,[[39]](Special:Random.html#cite_note-39) and others.[[40]](Special:Random.html#cite_note-40) An interesting application of cuckoo search is to solve boundary value problems.[[41]](Special:Random.html#cite_note-41)

## References[[edit](http://en.wikipedia.org/w/index.php?title=Cuckoo_search&action=edit&section=7)]

  1. **[^](Special:Random.html#cite_ref-NaBIC_1-0)** X.-S. Yang; S. Deb (December 2009). "Cuckoo search via Lévy flights". World Congress on Nature & Biologically Inspired Computing (NaBIC 2009). IEEE Publications. pp. 210–214. [arXiv](http://en.wikipedia.org/wiki/ArXiv):[1003.1594v1](http://arxiv.org/abs/1003.1594v1).  Cite uses deprecated parameters ([help](http://en.wikipedia.org/wiki/Help:CS1_errors#deprecated_params)); `|accessdate=` requires `|url=` ([help](http://en.wikipedia.org/wiki/Help:CS1_errors#accessdate_missing_url)) [Papercore summary http://papercore.org/Yang2009](http://papercore.org/Yang2009)
  2. **[^](Special:Random.html#cite_ref-2)** Inderscience (27 May 2010). ["Cuckoo designs spring"](http://www.alphagalileo.org/ViewItem.aspx?ItemId=76985&CultureCode=en). Alphagalileo.org. Retrieved 2010-05-27. 
  3. **[^](Special:Random.html#cite_ref-3)** R. B. Payne, M. D. Sorenson, and K. Klitz, The Cuckoos, Oxford University Press, (2005).
  4. **[^](Special:Random.html#cite_ref-4)** <http://www.scientificcomputing.com/news-DA-Novel-Cuckoo-Search-Algorithm-Beats-Particle-Swarm-Optimization-060110.aspx>
  5. **[^](Special:Random.html#cite_ref-5)** R. N. Mantegna, Fast, accurate algorithm for numerical simulation of Lévy stable stochastic processes, Physical Review E, Vol.49, 4677-4683 (1994).
  6. **[^](Special:Random.html#cite_ref-6)** X.-S. Yang, Nature-Inspired Metaheuristic Algorithms, 2nd Edition, Luniver Press, (2010).
  7. **[^](Special:Random.html#cite_ref-7)** M. Gutowski, Lévy flights as an underlying mechanism for global optimization algorithms, ArXiv Mathematical Physics e-Prints, June, (2001).
  8. **[^](Special:Random.html#cite_ref-8)** X. S. Yang, Metaheuristic optimization: algorithm analysis and open problems, in: Experimental Algorithms (SEA2011), Eds (P. M. Pardalos and S. Rebennack), LNCS 6630, pp.21-32 (2011).
  9. **[^](Special:Random.html#cite_ref-9)** X.-S. Yang and S. Deb, "Engineering optimisation by cuckoo search", Int. J. Mathematical Modelling and Numerical Optimisation", Vol. 1, No. 4, 330-343 (2010). <http://arxiv.org/abs/1005.2908>
  10. **[^](Special:Random.html#cite_ref-10)** [Matlab demo code](http://www.mathworks.com/matlabcentral/fileexchange/29809-cuckoo-search-cs-algorithm)
  11. **[^](Special:Random.html#cite_ref-11)** S. Walton; O. Hassan, M.R. Brown and K. Morgan (24 May 2013). "Comment on Paper by Bhargava, Fateen and Bonilla-Petriciolet". _Fluid Phase Equilibria_. [doi](http://en.wikipedia.org/wiki/Digital_object_identifier):[10.1016/j.fluid.2013.05.011](http://dx.doi.org/10.1016%2Fj.fluid.2013.05.011).  Cite uses deprecated parameters ([help](http://en.wikipedia.org/wiki/Help:CS1_errors#deprecated_params))
  12. **[^](Special:Random.html#cite_ref-12)** N. Bacanin, An object-oriented software implementation of a novel cuckoo search algorithm, Proc. of the 5th European Conference on European Computing Conference (ECC'11), pp. 245-250 (2011).
  13. **[^](Special:Random.html#cite_ref-13)** M. Tuba, M. Subotic, and N. Stanarevic, Modified cuckoo search algorithm for unconstrained optimization problems, Proc. of the 5th European Conference on European Computing Conference (ECC'11), pp. 263-268 (2011).
  14. **[^](Special:Random.html#cite_ref-14)** S. Walton; O. Hassan, K. Morgan and M.R. Brown (30 June 2011). "Modified cuckoo search: A new gradient free optimisation algorithm". _Chaos, Solitons & Fractals_. [doi](http://en.wikipedia.org/wiki/Digital_object_identifier):[10.1016/j.chaos.2011.06.004](http://dx.doi.org/10.1016%2Fj.chaos.2011.06.004).  Cite uses deprecated parameters ([help](http://en.wikipedia.org/wiki/Help:CS1_errors#deprecated_params))
  15. **[^](Special:Random.html#cite_ref-15)** S. Walton, O. Hassan, K. Morgan, Using proper orthogonal decomposition to reduce the order of optimization problems, in: Proc. 16th Int. Conf. on Finite Elrments in Flow Problems (Eds. Wall W.A. and Gvravemeier V.), Munich, p.90 (2011).
  16. **[^](Special:Random.html#cite_ref-16)** Walton, S., Hassan, O. and Morgan, K. (2012), Reduced order mesh optimisation using proper orthogonal decomposition and a modified cuckoo search. Int. J. Numer. Meth. Engng.. doi: 10.1002/nme.4400
  17. **[^](Special:Random.html#cite_ref-17)** A. Layeb, A novel quantum inspired cuckoo search for knapsack problems, Int. J. Bio-Inspired Computation, Vol. 3, 297-305 (2011).
  18. **[^](Special:Random.html#cite_ref-18)** X. S. Yang and S. Deb, Multiobjective cuckoo search for design optimization, Computers and Operations Research, October (2011). [doi](http://en.wikipedia.org/wiki/Digital_object_identifier):[10.1016/j.cor.2011.09.026](http://dx.doi.org/10.1016%2Fj.cor.2011.09.026)
  19. **[^](Special:Random.html#cite_ref-19)** F. Wang, L. Lou, X. He, Y. Wang, Hybrid optimization algorithm of PSO and Cuckoo Search, in: Proc. of 2nd Int. Conference on Artificial Intelligence, Management Science and Electronic Commerce (AIMSEC'11), pp. 1172-1175 (2011).
  20. **[^](Special:Random.html#cite_ref-20)** Sean Walton, Oubay Hassan, Kenneth Morgan, Selected Engineering Applications of Gradient Free Optimisation Using Cuckoo Search and Proper Orthogonal Decomposition, Archives of Computational Methods in EngineeringJune 2013, Volume 20, Issue 2, pp 123-154, <http://link.springer.com/article/10.1007/s11831-013-9083-7>
  21. **[^](Special:Random.html#cite_ref-21)** Tein L. H. and Ramli R., Recent advancements of nurse scheduling models and a potential path, in: Proc. 6th IMT-GT Conference on Mathematics, Statistics and its Applications (ICMSA 2010), pp. 395-409 (2010). <http://research.utar.edu.my/CMS/ICMSA2010/ICMSA2010_Proceedings/files/statistics/ST-Lim.pdf>
  22. **[^](Special:Random.html#cite_ref-22)** M. Dhivya, M. Sundarambal, L. N. Anand, Energy Efficient Computation of Data Fusion in Wireless Sensor Networks Using Cuckoo Based Particle Approach (CBPA), Int. J. of Communications, Network and System Sciences, Vol. 4, No. 4, 249-255 (2011).
  23. **[^](Special:Random.html#cite_ref-23)** M. Dhivya and M. Sundarambal, Cuckoo search for data gathering in wireless sensor networks,Int. J. Mobile Communications, Vol. 9, pp. 642-656 (2011).
  24. **[^](Special:Random.html#cite_ref-24)** A. Ouaarab, B. Ahiod, and X.-S. Yang, Discrete cuckoo search algorithm for the travelling salesman problem, Neural Computing and Applications, (2013). doi:10.1007/s00521-013-1402-2.
  25. **[^](Special:Random.html#cite_ref-25)** A. Layeb, A novel quantum-inspired cuckoo search for Knapsack problems, Int. J. Bio-inspired Computation, Vol. 4, (2011).
  26. **[^](Special:Random.html#cite_ref-26)** P. R. Srivastava, M. Chis, S. Deb and X. S. Yang, An efficient optimization algorithm for structural software testing, Int. J. Artificial Intelligence, Vol. 9 (S12), 68-77(2012)
  27. **[^](Special:Random.html#cite_ref-27)** K. Perumal, J. M. Ungati, G. Kumar, N. Jain, R. Gaurav and P. R. Srivastava, Test data generation: a hybrid approach using cuckoo and tabu search, Swarm, Evolutionary, and Memetic Computing (SEMCCO2011), Lecture Notes in Computer Sciences, Vol. 7077, 46-54 (2011)
  28. **[^](Special:Random.html#cite_ref-28)** P. Civicioglu and E. Besdok, A conception comparison of the cuckoo search, particle swarm optimization, differential evolution and artificial bee colony algorithms, Artificial Intelligence Review, [doi](http://en.wikipedia.org/wiki/Digital_object_identifier):[10.1007/s10462-011-92760](http://dx.doi.org/10.1007%2Fs10462-011-92760), 6 July (2011).
  29. **[^](Special:Random.html#cite_ref-29)** A. H. Gandomi, X. S. Yang, A. H. Alavi, Cuckoo search algorithm: a metaheuristic approach to solve structural optimization problems,Engineering with Computers, Vol. 27, July (2011).
  30. **[^](Special:Random.html#cite_ref-30)** K. Choudhary and G. N. Purohit, A new testing approach using cuckoo search to achieve multi-objective genetic algorithm, J. of Computing, Vol. 3, No. 4, 117-119 (2011).
  31. **[^](Special:Random.html#cite_ref-31)** E. R. Speed, Evolving a Mario agent using cuckoo search and softmax heuristics, Games Innovations Conference (ICE-GIC), pp. 1-7 (2010). [doi](http://en.wikipedia.org/wiki/Digital_object_identifier):[10.1109/ICEGIC.2010.5716893](http://dx.doi.org/10.1109%2FICEGIC.2010.5716893)
  32. **[^](Special:Random.html#cite_ref-32)** E. Valian, S. Mohanna and S. Tavakoli, Improved cuckoo search algorithm for feedforward neural network training, Int. J. Artificial Intelligence and Applications, Vol. 2, No. 3, 36-43(2011).
  33. **[^](Special:Random.html#cite_ref-33)** R. A. Vazquez, Training spiking neural models using cuckoo search algorithm, 2011 IEEE Congress on Evolutionary Computation (CEC'11), pp.679-686 (2011).
  34. **[^](Special:Random.html#cite_ref-34)** V. R. Chifu, C. B. Pop, I. Salomie, D> S. Suia and A. N. Niculici, Optimizing the semantic web service composition process using cuckoo search, in: Intelligent Distributed Computing V, Studies in Computational Intelligence, Vol. 382, pp. 93-102 (2012).
  35. **[^](Special:Random.html#cite_ref-35)** A. Kumar and S. Chakarverty, Design optimization for reliable embedded system using Cuckoo Search,in: Proc. of 3rd Int. Conference on Electronics Computer Technology (ICECT2011), pp. 564-268 (2011).
  36. **[^](Special:Random.html#cite_ref-36)** A. Kumar and S. Chakarverty, Design optimization using genetic algorithm and Cuckoo Search, in: Proc. of IEEE Int. Conference on Electro/Information Technology (EIT), pp. 1-5 (2011).
  37. **[^](Special:Random.html#cite_ref-37)** A. Kaveh, T. Bakhshpoori, Optimum design of steel frames using cuckoo search algorithm with Lévy flights,Structural Design of Tall and Special Buildings, Vol. 21, online first 28 Nov 2011.<http://onlinelibrary.wiley.com/doi/10.1002/tal.754/abstract>
  38. **[^](Special:Random.html#cite_ref-38)** A. R. Yildiz, Cuckoo search algorithm for the selection of optimal machine parameters in milling operations, Int. J. Adv. Manuf. Technol., (2012). [doi](http://en.wikipedia.org/wiki/Digital_object_identifier):[10.1007/s00170-012-4013-7](http://dx.doi.org/10.1007%2Fs00170-012-4013-7)
  39. **[^](Special:Random.html#cite_ref-39)** S. Burnwal, S. Deb, Scheduling optimization of flexible manufacturing system using cuckoo search-based approach, Int. J. Adv Manuf Technol, (2012).
  40. **[^](Special:Random.html#cite_ref-40)** H. Q. Zheng and Y. Zhou, A novel cuckoo search optimization algorithm based on Gauss distribution, J. Computational Information Systems, Vol. 8, 4193-4200 (2012).
  41. **[^](Special:Random.html#cite_ref-41)** A. Noghrehabadi, M. Ghalambaz and A. Vosough, A hybrid power series -- Cuckoo search optimization algorithm to electrostatic deflection of micro fixed-fixed actuators, Int. J. Multidisciplinary Science and Engineering, Vol. 2, No. 4,22-26 (2011).

  * [v](http://en.wikipedia.org/wiki/Template:Swarming)
  * [t](http://en.wikipedia.org/wiki/Template_talk:Swarming)
  * [e](http://en.wikipedia.org/w/index.php?title=Template:Swarming&action=edit)

[Swarming](http://en.wikipedia.org/wiki/Swarm_behaviour)

[Biological swarming](http://en.wikipedia.org/wiki/Swarm_behaviour#Biological_swarming)

  * [Agent-based model in biology](http://en.wikipedia.org/wiki/Agent-based_model_in_biology)
  * [Bait ball](http://en.wikipedia.org/wiki/Bait_ball)
  * [Collective animal behavior](http://en.wikipedia.org/wiki/Collective_animal_behavior)
  * [Feeding frenzy](http://en.wikipedia.org/wiki/Feeding_frenzy)
  * [Flock](http://en.wikipedia.org/wiki/Flock_\(birds\))
  * [Flocking](http://en.wikipedia.org/wiki/Flocking_\(behavior\))
  * [Herd](http://en.wikipedia.org/wiki/Herd)
  * [Herd behavior](http://en.wikipedia.org/wiki/Herd_behavior)
  * [Mixed-species foraging flock](http://en.wikipedia.org/wiki/Mixed-species_foraging_flock)
  * [Mobbing behavior](http://en.wikipedia.org/wiki/Mobbing_\(animal_behavior\))
  * [Pack hunter](http://en.wikipedia.org/wiki/Pack_hunter)
  * [Patterns of self-organization in ants](http://en.wikipedia.org/wiki/Patterns_of_self-organization_in_ants)
  * [Shoaling and schooling](http://en.wikipedia.org/wiki/Shoaling_and_schooling)
  * [Sort sol](http://en.wikipedia.org/wiki/Sort_sol_\(bird_flock\))
  * [Swarming behaviour](http://en.wikipedia.org/wiki/Swarm_behaviour)
  * [Swarming (honey bee)](http://en.wikipedia.org/wiki/Swarming_\(honey_bee\))
  * [Swarming motility](http://en.wikipedia.org/wiki/Swarming_motility)

![Auklet flock Shumagins 1986.jpg](../../upload.wikimedia.org/wikipedia/commons/thumb/5/5e/Auklet_flock_Shumagins_1986.jpg/140px-Auklet_flock_Shumagins_1986.jpg)

[Animal migration](http://en.wikipedia.org/wiki/Animal_migration)

  * [Animal migration](http://en.wikipedia.org/wiki/Animal_migration)
  * [Bird migration](http://en.wikipedia.org/wiki/Bird_migration)
  * [Bird migration flyways](http://en.wikipedia.org/wiki/Category:Bird_migration_flyways)
  * [Diel vertical migration](http://en.wikipedia.org/wiki/Diel_vertical_migration)
  * [Fish migration](http://en.wikipedia.org/wiki/Fish_migration)
  * [Homing (biology)](http://en.wikipedia.org/wiki/Homing_\(biology\))
  * [Insect migration](http://en.wikipedia.org/wiki/Insect_migration)
  * [Lessepsian migration](http://en.wikipedia.org/wiki/Lessepsian_migration)
  * [Lepidoptera migration](http://en.wikipedia.org/wiki/Lepidoptera_migration)
  * [Reverse migration](http://en.wikipedia.org/wiki/Reverse_migration_\(birds\))
  * [Salmon run](http://en.wikipedia.org/wiki/Salmon_run)
  * [Sardine run](http://en.wikipedia.org/wiki/Sardine_run)
  * [Sea turtle migration](http://en.wikipedia.org/wiki/Sea_turtle_migration)
  * [Animal migration tracking](http://en.wikipedia.org/wiki/Animal_migration_tracking)

[Swarm algorithms](http://en.wikipedia.org/wiki/Swarm_behaviour#Models)

  * [Agent-based models](http://en.wikipedia.org/wiki/Agent-based_model)
  * [Ant colony optimization](http://en.wikipedia.org/wiki/Ant_colony_optimization_algorithms)
  * [Ant robotics](http://en.wikipedia.org/wiki/Ant_robotics)
  * [Artificial ants](http://en.wikipedia.org/wiki/Artificial_Ants)
  * [Bat algorithm](http://en.wikipedia.org/wiki/Bat_algorithm)
  * [Bees algorithm](http://en.wikipedia.org/wiki/Bees_algorithm)
  * [Boids](http://en.wikipedia.org/wiki/Boids)
  * [Crowd simulation](http://en.wikipedia.org/wiki/Crowd_simulation)
  * [Firefly algorithm](http://en.wikipedia.org/wiki/Firefly_algorithm)
  * [Glowworm swarm optimization](http://en.wikipedia.org/wiki/Glowworm_swarm_optimization)
  * [Particle swarm optimization](http://en.wikipedia.org/wiki/Particle_swarm_optimization)
  * [Self-propelled particles](http://en.wikipedia.org/wiki/Self-propelled_particles)
  * [Swarm intelligence](http://en.wikipedia.org/wiki/Swarm_intelligence)
  * [Swarm (simulation)](http://en.wikipedia.org/wiki/Swarm_\(simulation\))

[Swarm robotics](http://en.wikipedia.org/wiki/Swarm_robotics)

  * [I-Swarm](http://en.wikipedia.org/wiki/Intelligent_Small_World_Autonomous_Robots_for_Micro-manipulation)
  * [Microbotics](http://en.wikipedia.org/wiki/Microbotics)
  * [Swarm robotics](http://en.wikipedia.org/wiki/Swarm_robotics)
  * [Symbrion](http://en.wikipedia.org/wiki/Symbrion)

Related topics

  * [Allee effect](http://en.wikipedia.org/wiki/Allee_effect)
  * [Animal navigation](http://en.wikipedia.org/wiki/Animal_navigation)
  * [Collective intelligence](http://en.wikipedia.org/wiki/Collective_intelligence)
  * [Eusociality](http://en.wikipedia.org/wiki/Eusociality)
  * [Group size measures](http://en.wikipedia.org/wiki/Group_size_measures)
  * [Microbial intelligence](http://en.wikipedia.org/wiki/Microbial_intelligence)
  * [Mutualism](http://en.wikipedia.org/wiki/Mutualism_\(biology\))
  * [Predator satiation](http://en.wikipedia.org/wiki/Predator_satiation)
  * [Quorum sensing](http://en.wikipedia.org/wiki/Quorum_sensing)
  * [Spatial organization](http://en.wikipedia.org/wiki/Spatial_organization)
  * [Stigmergy](http://en.wikipedia.org/wiki/Stigmergy)
  * [Swarming (military)](http://en.wikipedia.org/wiki/Swarming_\(military\))
  * [Task allocation and partitioning of social insects](http://en.wikipedia.org/wiki/Task_allocation_and_partitioning_of_social_insects)

  * [v](http://en.wikipedia.org/wiki/Template:Optimization_algorithms)
  * [t](http://en.wikipedia.org/wiki/Template_talk:Optimization_algorithms)
  * [e](http://en.wikipedia.org/w/index.php?title=Template:Optimization_algorithms&action=edit)

[Optimization](http://en.wikipedia.org/wiki/Mathematical_optimization): [Algorithms](http://en.wikipedia.org/wiki/Optimization_algorithm), [methods](http://en.wikipedia.org/wiki/Iterative_method), and [heuristics](http://en.wikipedia.org/wiki/Heuristic_algorithm)

 

[Unconstrained nonlinear](http://en.wikipedia.org/wiki/Nonlinear_programming): [Methods](http://en.wikipedia.org/wiki/Iterative_method) [calling …](http://en.wikipedia.org/wiki/Subroutine)

… [functions](http://en.wikipedia.org/wiki/Function_\(mathematics\))

  * [Golden section search](http://en.wikipedia.org/wiki/Golden_section_search)
  * [Interpolation methods](http://en.wikipedia.org/wiki/Powell%27s_method)
  * [Line search](http://en.wikipedia.org/wiki/Line_search)
  * [Nelder–Mead method](http://en.wikipedia.org/wiki/Nelder%E2%80%93Mead_method)
  * [Successive parabolic interpolation](http://en.wikipedia.org/wiki/Successive_parabolic_interpolation)

… and [gradients](http://en.wikipedia.org/wiki/Gradient)

[Convergence](http://en.wikipedia.org/wiki/Local_convergence)

  * [Trust region](http://en.wikipedia.org/wiki/Trust_region)
  * [Wolfe conditions](http://en.wikipedia.org/wiki/Wolfe_conditions)

[Quasi–Newton](http://en.wikipedia.org/wiki/Quasi-Newton_method)

  * [BFGS](http://en.wikipedia.org/wiki/BFGS_method) and [L-BFGS](http://en.wikipedia.org/wiki/Limited-memory_BFGS)
  * [DFP](http://en.wikipedia.org/wiki/Davidon%E2%80%93Fletcher%E2%80%93Powell_formula)
  * [Symmetric rank-one (SR1)](http://en.wikipedia.org/wiki/SR1_formula)

[Other methods](http://en.wikipedia.org/wiki/Iterative_method)

  * [Gauss–Newton](http://en.wikipedia.org/wiki/Gauss%E2%80%93Newton_algorithm)
  * [Gradient](http://en.wikipedia.org/wiki/Gradient_descent)
  * [Levenberg–Marquardt](http://en.wikipedia.org/wiki/Levenberg%E2%80%93Marquardt_algorithm)
  * [Conjugate gradient](http://en.wikipedia.org/wiki/Nonlinear_conjugate_gradient_method)

… and [Hessians](http://en.wikipedia.org/wiki/Hessian_matrix)

  * [Newton's method](http://en.wikipedia.org/wiki/Newton%27s_method_in_optimization)

![The graph of a strictly concave quadratic function is shown in blue, with its unique maximum shown as a red dot. Below the graph appears the contours of the function: The level sets are nested ellipses.](../../upload.wikimedia.org/wikipedia/commons/thumb/6/62/MaximumParaboloid.png/100px-MaximumParaboloid.png)

 

[Constrained nonlinear](http://en.wikipedia.org/wiki/Nonlinear_programming)

General

  * [Barrier methods](http://en.wikipedia.org/wiki/Barrier_function)
  * [Penalty methods](http://en.wikipedia.org/wiki/Penalty_method)

Differentiable

  * [Augmented Lagrangian methods](http://en.wikipedia.org/wiki/Augmented_Lagrangian_method)
  * [Sequential quadratic programming](http://en.wikipedia.org/wiki/Sequential_quadratic_programming)
  * [Successive linear programming](http://en.wikipedia.org/wiki/Successive_linear_programming)

 

[Convex optimization](http://en.wikipedia.org/wiki/Convex_optimization)

[Convex  
minimization](http://en.wikipedia.org/wiki/Convex_minimization)

  * [Cutting-plane method](http://en.wikipedia.org/wiki/Cutting-plane_method)
  * [Reduced gradient (Frank–Wolfe)](http://en.wikipedia.org/wiki/Frank%E2%80%93Wolfe_algorithm)
  * [Subgradient method](http://en.wikipedia.org/wiki/Subgradient_method)

[Linear](http://en.wikipedia.org/wiki/Linear_programming) and  
[quadratic](http://en.wikipedia.org/wiki/Quadratic_programming)

[Interior point](http://en.wikipedia.org/wiki/Linear_programming#Interior_point)

  * [Ellipsoid algorithm of Khachiyan](http://en.wikipedia.org/wiki/Ellipsoid_method)
  * [Projective algorithm of Karmarkar](http://en.wikipedia.org/wiki/Karmarkar%27s_algorithm)

[Basis-](http://en.wikipedia.org/wiki/Matroid)[Exchange](http://en.wikipedia.org/wiki/Exchange_algorithm)

  * [Simplex algorithm of Dantzig](http://en.wikipedia.org/wiki/Simplex_algorithm)
  * [Revised simplex algorithm](http://en.wikipedia.org/wiki/Revised_simplex_algorithm)
  * [Criss-cross algorithm](http://en.wikipedia.org/wiki/Criss-cross_algorithm)
  * [Principal pivoting algorithm of Lemke](http://en.wikipedia.org/wiki/Lemke%27s_algorithm)

 

[Combinatorial](http://en.wikipedia.org/wiki/Combinatorial_optimization)

Paradigms

  * [Approximation algorithm](http://en.wikipedia.org/wiki/Approximation_algorithm)
  * [Dynamic programming](http://en.wikipedia.org/wiki/Dynamic_programming)
  * [Greedy algorithm](http://en.wikipedia.org/wiki/Greedy_algorithm)
  * [Integer programming](http://en.wikipedia.org/wiki/Integer_programming)
    * [Branch & bound](http://en.wikipedia.org/wiki/Branch_and_bound) or [cut](http://en.wikipedia.org/wiki/Branch_and_cut)

[Graph  
algorithms](http://en.wikipedia.org/wiki/Graph_algorithm)

[Minimum  
spanning tree](http://en.wikipedia.org/wiki/Minimum_spanning_tree)

  * [Bellman–Ford](http://en.wikipedia.org/wiki/Bellman%E2%80%93Ford_algorithm)
  * [Borůvka](http://en.wikipedia.org/wiki/Bor%C5%AFvka%27s_algorithm)
  * [Dijkstra](http://en.wikipedia.org/wiki/Dijkstra%27s_algorithm)
  * [Floyd–Warshall](http://en.wikipedia.org/wiki/Floyd%E2%80%93Warshall_algorithm)
  * [Johnson](http://en.wikipedia.org/wiki/Johnson%27s_algorithm)
  * [Kruskal](http://en.wikipedia.org/wiki/Kruskal%27s_algorithm)

[Network flows](http://en.wikipedia.org/wiki/Flow_network)

  * [Dinic](http://en.wikipedia.org/wiki/Dinic%27s_algorithm)
  * [Edmonds–Karp](http://en.wikipedia.org/wiki/Edmonds%E2%80%93Karp_algorithm)
  * [Ford–Fulkerson](http://en.wikipedia.org/wiki/Ford%E2%80%93Fulkerson_algorithm)
  * [Push-relabel maximum flow](http://en.wikipedia.org/wiki/Push-relabel_maximum_flow_algorithm)

 

[Metaheuristics](http://en.wikipedia.org/wiki/Metaheuristic)

  * [Evolutionary algorithm](http://en.wikipedia.org/wiki/Evolutionary_algorithm)
  * [Hill climbing](http://en.wikipedia.org/wiki/Hill_climbing)
  * [Local search](http://en.wikipedia.org/wiki/Local_search_\(optimization\))
  * [Simulated annealing](http://en.wikipedia.org/wiki/Simulated_annealing)
  * [Tabu search](http://en.wikipedia.org/wiki/Tabu_search)

  * **Categories**
    * [Algorithms and methods](http://en.wikipedia.org/wiki/Category:Optimization_algorithms_and_methods)
    * [Heuristics](http://en.wikipedia.org/wiki/Category:Heuristic_algorithms)
  * **[Software](http://en.wikipedia.org/wiki/Comparison_of_optimization_software)**

L. Sivakumar, R. Kotteeswaran Soft Computing Based Partial-Retuning of Decentralised PI Controller of Nonlinear Multivariable Process, ICT and Critical Infrastructure: Proceedings of the 48th Annual Convention of Computer Society of India- Vol I Advances in Intelligent Systems and Computing Volume 248, 2014, pp 117-124, <http://link.springer.com/chapter/10.1007/978-3-319-03107-1_14>

![](Special:CentralAutoLogin/start%3Ftype=1x1)

Retrieved from "[http://en.wikipedia.org/w/index.php?title=Cuckoo_search&oldid=601017275](http://en.wikipedia.org/w/index.php?title=Cuckoo_search&oldid=601017275)" 

[Categories](http://en.wikipedia.org/wiki/Help:Category): 

  * [Optimization algorithms and methods](http://en.wikipedia.org/wiki/Category:Optimization_algorithms_and_methods)
  * [Evolutionary algorithms](http://en.wikipedia.org/wiki/Category:Evolutionary_algorithms)

Hidden categories: 

  * [Pages containing cite templates with deprecated parameters](http://en.wikipedia.org/wiki/Category:Pages_containing_cite_templates_with_deprecated_parameters)
  * [Pages using citations with accessdate and no URL](http://en.wikipedia.org/wiki/Category:Pages_using_citations_with_accessdate_and_no_URL)

## Navigation menu

### Personal tools

  * [Create account](http://en.wikipedia.org/w/index.php?title=Special:UserLogin&returnto=Cuckoo+search&type=signup)
  * [Log in](http://en.wikipedia.org/w/index.php?title=Special:UserLogin&returnto=Cuckoo+search)

### Namespaces

  * [Article](Special:Random.html)
  * [Talk](http://en.wikipedia.org/wiki/Talk:Cuckoo_search)

### 

### Variants

### Views

  * [Read](Special:Random.html)
  * [Edit](http://en.wikipedia.org/w/index.php?title=Cuckoo_search&action=edit)
  * [View history](http://en.wikipedia.org/w/index.php?title=Cuckoo_search&action=history)

### Actions

### Search

### Navigation

<../../upload.wikimedia.org/wikipedia/en/b/bc/Wiki.png);" href="/wiki/Main_Page" title="Visit the main page">

### Navigation

  * [Main page](http://en.wikipedia.org/wiki/Main_Page)
  * [Contents](http://en.wikipedia.org/wiki/Portal:Contents)
  * [Featured content](http://en.wikipedia.org/wiki/Portal:Featured_content)
  * [Current events](http://en.wikipedia.org/wiki/Portal:Current_events)
  * [Random article](Special:Random.html)
  * [Donate to Wikipedia](https://donate.wikimedia.org/wiki/Special:FundraiserRedirector?utm_source=donate&utm_medium=sidebar&utm_campaign=C13_en.wikipedia.org&uselang=en)
  * [Wikimedia Shop](http://shop.wikimedia.org/)

### Interaction

  * [Help](http://en.wikipedia.org/wiki/Help:Contents)
  * [About Wikipedia](http://en.wikipedia.org/wiki/Wikipedia:About)
  * [Community portal](http://en.wikipedia.org/wiki/Wikipedia:Community_portal)
  * [Recent changes](http://en.wikipedia.org/wiki/Special:RecentChanges)
  * [Contact page](http://en.wikipedia.org/wiki/Wikipedia:Contact_us)

### Tools

  * [What links here](http://en.wikipedia.org/wiki/Special:WhatLinksHere/Cuckoo_search)
  * [Related changes](http://en.wikipedia.org/wiki/Special:RecentChangesLinked/Cuckoo_search)
  * [Upload file](http://en.wikipedia.org/wiki/Wikipedia:File_Upload_Wizard)
  * [Special pages](http://en.wikipedia.org/wiki/Special:SpecialPages)
  * [Permanent link](http://en.wikipedia.org/w/index.php?title=Cuckoo_search&oldid=601017275)
  * [Page information](http://en.wikipedia.org/w/index.php?title=Cuckoo_search&action=info)
  * [Data item](http://www.wikidata.org/wiki/Q5192379)
  * [Cite this page](http://en.wikipedia.org/w/index.php?title=Special:Cite&page=Cuckoo_search&id=601017275)

### Print/export

  * [Create a book](http://en.wikipedia.org/w/index.php?title=Special:Book&bookcmd=book_creator&referer=Cuckoo+search)
  * [Download as PDF](http://en.wikipedia.org/w/index.php?title=Special:Book&bookcmd=render_article&arttitle=Cuckoo+search&oldid=601017275&writer=rl)
  * [Printable version](http://en.wikipedia.org/w/index.php?title=Cuckoo_search&printable=yes)

### Languages

  * [فارسی](http://fa.wikipedia.org/wiki/%D8%AC%D8%B3%D8%AA%D8%AC%D9%88%DB%8C_%DA%A9%D9%88%DA%A9%D9%88)
  * [Українська](http://uk.wikipedia.org/wiki/%D0%90%D0%BB%D0%B3%D0%BE%D1%80%D0%B8%D1%82%D0%BC_%D0%B7%D0%BE%D0%B7%D1%83%D0%BB%D1%96)
  * [中文](http://zh.wikipedia.org/wiki/%E5%B8%83%E8%B0%B7%E9%B8%9F%E6%90%9C%E7%B4%A2%E7%AE%97%E6%B3%95)
  *   * [Edit links](http://www.wikidata.org/wiki/Q5192379#sitelinks-wikipedia)

  * This page was last modified on 24 March 2014 at 11:24.  

  * Text is available under the [Creative Commons Attribution-ShareAlike License](http://en.wikipedia.org/wiki/Wikipedia:Text_of_Creative_Commons_Attribution-ShareAlike_3.0_Unported_License); additional terms may apply. By using this site, you agree to the [Terms of Use](http://wikimediafoundation.org/wiki/Terms_of_Use) and [Privacy Policy](http://wikimediafoundation.org/wiki/Privacy_policy). Wikipedia® is a registered trademark of the [Wikimedia Foundation, Inc.](http://www.wikimediafoundation.org/), a non-profit organization.
  * [Privacy policy](http://wikimediafoundation.org/wiki/Privacy_policy)
  * [About Wikipedia](http://en.wikipedia.org/wiki/Wikipedia:About)
  * [Disclaimers](http://en.wikipedia.org/wiki/Wikipedia:General_disclaimer)
  * [Contact Wikipedia](http://en.wikipedia.org/wiki/Wikipedia:Contact_us)
  * [Developers](https://www.mediawiki.org/wiki/Special:MyLanguage/How_to_contribute)
  * [Mobile view](http://en.m.wikipedia.org/wiki/Cuckoo_search)
  * ![Wikimedia Foundation](../../bits.wikimedia.org/images/wikimedia-button.png)
  * ![Powered by MediaWiki](../../bits.wikimedia.org/static-1.24wmf1/skins/common/images/poweredby_mediawiki_88x31.png)
