# Interval (graph theory)

From Wikipedia, the free encyclopedia

Jump to: [navigation](Special:Random.html#mw-navigation), [search](Special:Random.html#p-search)

This article is about a concept in graph-theoretic data flow analysis. For graphs defined from intersections of intervals, see [Interval graph](http://en.wikipedia.org/wiki/Interval_graph). For the set of elements in a DAG that are reachable from one vertex and can reach another vertex, see [Partially ordered set#Interval](http://en.wikipedia.org/wiki/Partially_ordered_set#Interval).

In [graph theory](http://en.wikipedia.org/wiki/Graph_theory), an **interval** _I_(_h_) in a [directed graph](http://en.wikipedia.org/wiki/Directed_graph) is a maximal, single entry subgraph in which _h_ is the only entry to _I_(_h_) and all closed [paths](http://en.wikipedia.org/wiki/Path_\(graph_theory\)) in _I_(_h_) contain _h_. Intervals were described in 1976 by F. E. Allen and J. Cooke.[[1]](Special:Random.html#cite_note-1) Interval graphs are integral to some algorithms used in [compilers](http://en.wikipedia.org/wiki/Compilers), specifically data flow analyses.

The following algorithm finds all the intervals in a graph consisting of vertices _N_ and the entry vertex _n_0, and with the functions `pred(n)` and `succ(n)` which return the list of predecessors and successors of a given node _n_, respectively.
    
    
       H = { n0 }                               // Initialize work list
       while H is not empty
           remove next h from H     
           create the interval I(h)
           I(h) += { h }
           while ∃n ∈ { succ(I(h)) — I(h) } such that pred(n) ⊆ I(h)
               I(h) += { n }
           while ∃n ∈ N such that n ∉ I(h) and    // find next headers
                 ∃m ∈ pred(n) such that m ∈ I(h)
               H += n  
    

The algorithm effectively [partitions](http://en.wikipedia.org/wiki/Partition_of_a_set) the graph into its intervals.

Each interval can in turn be replaced with a single node, while all edges between nodes in different intervals in the original graph become edges between their corresponding nodes in the new graph. This new graph is called an **interval derived graph**. The process of creating derived graphs can be repeated until the resulting graph can't be reduced further. If the final graph consists of a single node, then the original graph is said to be **reducible**.

## References[[edit](http://en.wikipedia.org/w/index.php?title=Interval_\(graph_theory\)&action=edit&section=1)]

  1. **[^](Special:Random.html#cite_ref-1)** F.E. Allen, J. Cooke (March 1976). "A Program Data Flow Analysis Procedure". _Comm. ACM_ **19** (3): 137–147. 

![](Special:CentralAutoLogin/start%3Ftype=1x1)

Retrieved from "[http://en.wikipedia.org/w/index.php?title=Interval_(graph_theory)&oldid=603506728](http://en.wikipedia.org/w/index.php?title=Interval_\(graph_theory\)&oldid=603506728)" 

[Categories](http://en.wikipedia.org/wiki/Help:Category): 

  * [Graph theory](http://en.wikipedia.org/wiki/Category:Graph_theory)
  * [Control-flow analysis](http://en.wikipedia.org/wiki/Category:Control-flow_analysis)

## Navigation menu

### Personal tools

  * [Create account](http://en.wikipedia.org/w/index.php?title=Special:UserLogin&returnto=Interval+%28graph+theory%29&type=signup)
  * [Log in](http://en.wikipedia.org/w/index.php?title=Special:UserLogin&returnto=Interval+%28graph+theory%29)

### Namespaces

  * [Article](Special:Random.html)
  * [Talk](http://en.wikipedia.org/w/index.php?title=Talk:Interval_\(graph_theory\)&action=edit&redlink=1)

### 

### Variants

### Views

  * [Read](Special:Random.html)
  * [Edit](http://en.wikipedia.org/w/index.php?title=Interval_\(graph_theory\)&action=edit)
  * [View history](http://en.wikipedia.org/w/index.php?title=Interval_\(graph_theory\)&action=history)

### Actions

### Search

### Navigation

  * [Main page](/wiki/Main_Page)
  * [Contents](/wiki/Portal:Contents)
  * [Featured content](/wiki/Portal:Featured_content)
  * [Current events](/wiki/Portal:Current_events)
  * [Random article](/wiki/Special:Random)
  * [Donate to Wikipedia](https://donate.wikimedia.org/wiki/Special:FundraiserRedirector?utm_source=donate&utm_medium=sidebar&utm_campaign=C13_en.wikipedia.org&uselang=en)
  * [Wikimedia Shop](//shop.wikimedia.org)

### Interaction

  * [Help](/wiki/Help:Contents)
  * [About Wikipedia](/wiki/Wikipedia:About)
  * [Community portal](/wiki/Wikipedia:Community_portal)
  * ### Navigation

    * [Main page](http://en.wikipedia.org/wiki/Main_Page)
    * [Contents](http://en.wikipedia.org/wiki/Portal:Contents)
    * [Featured content](http://en.wikipedia.org/wiki/Portal:Featured_content)
    * [Current events](http://en.wikipedia.org/wiki/Portal:Current_events)
    * [Random article](Special:Random.html)
    * [Donate to Wikipedia](https://donate.wikimedia.org/wiki/Special:FundraiserRedirector?utm_source=donate&utm_medium=sidebar&utm_campaign=C13_en.wikipedia.org&uselang=en)
    * [Wikimedia Shop](http://shop.wikimedia.org/)

### Interaction

    * [Help](http://en.wikipedia.org/wiki/Help:Contents)
    * [About Wikipedia](http://en.wikipedia.org/wiki/Wikipedia:About)
    * [Community portal](http://en.wikipedia.org/wiki/Wikipedia:Community_portal)
    * [Recent changes](http://en.wikipedia.org/wiki/Special:RecentChanges)
    * [Contact page](http://en.wikipedia.org/wiki/Wikipedia:Contact_us)

### Tools

    * [What links here](http://en.wikipedia.org/wiki/Special:WhatLinksHere/Interval_\(graph_theory\))
    * [Related changes](http://en.wikipedia.org/wiki/Special:RecentChangesLinked/Interval_\(graph_theory\))
    * [Upload file](http://en.wikipedia.org/wiki/Wikipedia:File_Upload_Wizard)
    * [Special pages](http://en.wikipedia.org/wiki/Special:SpecialPages)
    * [Permanent link](http://en.wikipedia.org/w/index.php?title=Interval_\(graph_theory\)&oldid=603506728)
    * [Page information](http://en.wikipedia.org/w/index.php?title=Interval_\(graph_theory\)&action=info)
    * [Cite this page](http://en.wikipedia.org/w/index.php?title=Special:Cite&page=Interval_%28graph_theory%29&id=603506728)

### Print/export

    * [Create a book](http://en.wikipedia.org/w/index.php?title=Special:Book&bookcmd=book_creator&referer=Interval+%28graph+theory%29)
    * [Download as PDF](http://en.wikipedia.org/w/index.php?title=Special:Book&bookcmd=render_article&arttitle=Interval+%28graph+theory%29&oldid=603506728&writer=rl)
    * [Printable version](http://en.wikipedia.org/w/index.php?title=Interval_\(graph_theory\)&printable=yes)

### Languages

    *     * This page was last modified on 9 April 2014 at 21:22.  

    * Text is available under the [Creative Commons Attribution-ShareAlike License](http://en.wikipedia.org/wiki/Wikipedia:Text_of_Creative_Commons_Attribution-ShareAlike_3.0_Unported_License); additional terms may apply. By using this site, you agree to the [Terms of Use](http://wikimediafoundation.org/wiki/Terms_of_Use) and [Privacy Policy](http://wikimediafoundation.org/wiki/Privacy_policy). Wikipedia® is a registered trademark of the [Wikimedia Foundation, Inc.](http://www.wikimediafoundation.org/), a non-profit organization.
    * [Privacy policy](http://wikimediafoundation.org/wiki/Privacy_policy)
    * [About Wikipedia](http://en.wikipedia.org/wiki/Wikipedia:About)
    * [Disclaimers](http://en.wikipedia.org/wiki/Wikipedia:General_disclaimer)
    * [Contact Wikipedia](http://en.wikipedia.org/wiki/Wikipedia:Contact_us)
    * [Developers](https://www.mediawiki.org/wiki/Special:MyLanguage/How_to_contribute)
    * [Mobile view](http://en.m.wikipedia.org/wiki/Interval_\(graph_theory\))
    * ![Wikimedia Foundation](../../bits.wikimedia.org/images/wikimedia-button.png)
    * ![Powered by MediaWiki](../../bits.wikimedia.org/static-1.24wmf1/skins/common/images/poweredby_mediawiki_88x31.png)
