# Quadratic Gauss sum

From Wikipedia, the free encyclopedia

Jump to: [navigation](Special:Random.html#mw-navigation), [search](Special:Random.html#p-search)

In [number theory](http://en.wikipedia.org/wiki/Number_theory), **quadratic Gauss sums** are certain finite sums of roots of unity. A quadratic Gauss sum can be interpreted as a linear combination of the values of the complex [exponential function](http://en.wikipedia.org/wiki/Exponential_function) with coefficients given by a quadratic character; for a general character, one obtains a more general [Gauss sum](http://en.wikipedia.org/wiki/Gauss_sum). These objects are named after [Carl Friedrich Gauss](http://en.wikipedia.org/wiki/Carl_Friedrich_Gauss), who studied them extensively and applied them to [quadratic](http://en.wikipedia.org/wiki/Quadratic_reciprocity), [cubic](http://en.wikipedia.org/wiki/Cubic_reciprocity), and [biquadratic](http://en.wikipedia.org/wiki/Biquadratic_reciprocity) reciprocity laws.

## Contents

  * [1 Definition](Special:Random.html#Definition)
    * [1.1 Properties](Special:Random.html#Properties)
  * [2 Generalized quadratic Gauss sums](Special:Random.html#Generalized_quadratic_Gauss_sums)
    * [2.1 Properties](Special:Random.html#Properties_2)
  * [3 See also](Special:Random.html#See_also)
  * [4 References](Special:Random.html#References)

## Definition[[edit](http://en.wikipedia.org/w/index.php?title=Quadratic_Gauss_sum&action=edit&section=1)]

Let _p_ be an odd [prime number](http://en.wikipedia.org/wiki/Prime_number) and _a_ an integer. Then the **Gauss sum** mod _p_, _g_(_a_;_p_), is the following sum of the _p_th [roots of unity](http://en.wikipedia.org/wiki/Root_of_unity):

    ![ g\(a;p\) =\\sum_{n=0}^{p-1}e^{2{\\pi}ian^2/p}=\\sum_{n=0}^{p-1}\\zeta_p^{an^2}, 
\\quad \\zeta_p=e^{2{\\pi}i/p}. ](../../upload.wikimedia.org/math/6/a/1/6a1fef816543e5207c2e42945d121597.png)

If _a_ is not divisible by _p_, an alternative expression for the Gauss sum (with the same value) is

    ![G\(a,\\chi\)=\\sum_{n=1}^{p-1}\\left\(\\frac{n}{p}\\right\)e^{2{\\pi}ian/p}. ](../../upload.wikimedia.org/math/b/c/7/bc7e1381012e84e5b4ff168259b18653.png)

Here ![\\chi\(n\)=\\left\(\\frac{n}{p}\\right\)](../../upload.wikimedia.org/math/2/2/5/225ac082f789dbafeff57fc4776c884a.png) is the [Legendre symbol](http://en.wikipedia.org/wiki/Legendre_symbol), which is a quadratic character mod _p_. An analogous formula with a general character _χ_ in place of the Legendre symbol defines the [Gauss sum](http://en.wikipedia.org/wiki/Gauss_sum) _G_(_χ_).

### Properties[[edit](http://en.wikipedia.org/w/index.php?title=Quadratic_Gauss_sum&action=edit&section=2)]

  * The value of the Gauss sum is an [algebraic integer](http://en.wikipedia.org/wiki/Algebraic_integer) in the _p_th [cyclotomic field](http://en.wikipedia.org/wiki/Cyclotomic_field) **Q**(_ζ__p_).
  * The evaluation of the Gauss sum can be reduced to the case _a_ = 1:

    ![ g\(a;p\)=\\left\(\\frac{a}{p}\\right\)g\(1;p\). ](../../upload.wikimedia.org/math/1/5/a/15ace789cb82d93ec211df76ab845894.png)

  * The exact value of the Gauss sum, computed by Gauss, is given by the formula

    

    ![ g\(1;p\) =\\sum_{n=0}^{p-1}e^{2{\\pi}in^2/p}=
\\begin{cases} 
\\sqrt{p} & p\\equiv 1\\mod 4 \\\\ i\\sqrt{p} & p\\equiv 3\\mod 4 
\\end{cases}.](../../upload.wikimedia.org/math/8/e/4/8e4429f7a113be16bb7ffcd4c72cb55f.png)
    The fact that ![g\(a;p\)^2=\\left\(\\frac{-1}{p}\\right\)p](../../upload.wikimedia.org/math/1/0/c/10c8f1e68f38d066d285c9a595c0e44e.png) was easy to prove and led to one of Gauss's [proofs of quadratic reciprocity](http://en.wikipedia.org/wiki/Proofs_of_quadratic_reciprocity). However, the determination of the _sign_ of the Gauss sum turned out to be considerably more difficult: Gauss could only establish it after several years' work. Later, [Peter Gustav Lejeune Dirichlet](http://en.wikipedia.org/wiki/Peter_Gustav_Lejeune_Dirichlet), [Leopold Kronecker](http://en.wikipedia.org/wiki/Leopold_Kronecker), [Issai Schur](http://en.wikipedia.org/wiki/Issai_Schur) and other mathematicians found different proofs.

## Generalized quadratic Gauss sums[[edit](http://en.wikipedia.org/w/index.php?title=Quadratic_Gauss_sum&action=edit&section=3)]

Let _a_,_b_,_c_ be [natural numbers](http://en.wikipedia.org/wiki/Natural_numbers). The **generalized Gauss sum** _G_(_a_,_b_,_c_) is defined by

    ![G\(a,b,c\)=\\sum_{n=0}^{c-1} e\\left\(\\frac{a n^2+bn}{c}\\right\),](../../upload.wikimedia.org/math/3/0/3/3035e88066768c5c25613d21d93fc841.png)

where _e_(_x_) is the exponential function exp(2πi_x_). The classical Gauss sum is the sum ![G\(a,c\)=G\(a,0,c\)](../../upload.wikimedia.org/math/8/e/7/8e75ed63ecc7eba0dcdae9eaf0ba6e56.png).

### Properties[[edit](http://en.wikipedia.org/w/index.php?title=Quadratic_Gauss_sum&action=edit&section=4)]

  * The Gauss sum _G_(_a_,_b_,_c_) depends only on the [residue class](http://en.wikipedia.org/wiki/Residue_class) of _a_,_b_ modulo _c_.
  * Gauss sums are [multiplicative](http://en.wikipedia.org/wiki/Multiplicative_function), i.e. given natural numbers _a_, _b_, _c_ and _d_ with [gcd](http://en.wikipedia.org/wiki/Greatest_common_divisor)(_c_,_d_) =1 one has

    _G_(_a_,_b_,_cd_)=_G_(_ac_,_b_,_d_)_G_(_ad_,_b_,_c_).

This is a direct consequence of the [Chinese remainder theorem](http://en.wikipedia.org/wiki/Chinese_remainder_theorem).

  * One has _G_(_a_,_b_,_c_)=_0_ if gcd(_a_,_c_)>1 except if gcd(_a_,_c_) divides _b_ in which case one has

    ![
G\(a,b,c\)= \\gcd\(a,c\) \\cdot G\\left\(\\frac{a}{\\gcd\(a,c\)},\\frac{b}{\\gcd\(a,c\)},\\frac{c}{\\gcd\(a,c\)}\\right\)
](../../upload.wikimedia.org/math/4/9/6/496b092114bc63098f5b8ae27f614676.png)

Thus in the evaluation of quadratic Gauss sums one may always assume gcd(_a_,_c_)=_1_.

  * Let _a_,_b_ and _c_ be integers with ![ac\\neq 0](../../upload.wikimedia.org/math/4/8/b/48b5d16114ab0a71d8c43cebe80ae414.png) and _ac+b_ even. One has the following analogue of the [quadratic reciprocity](http://en.wikipedia.org/wiki/Quadratic_reciprocity) law for (even more general) Gauss sums

    ![
\\sum_{n=0}^{|c|-1} e^{\\pi i \(a n^2+bn\)/c} = |c/a|^{1/2} e^{\\pi i \(|ac|-b^2\)/\(4ac\)} \\sum_{n=0}^{|a|-1} e^{-\\pi i \(c n^2+b n\)/a}. 
](../../upload.wikimedia.org/math/d/3/e/d3e907526e28f148e78ee09e21a24529.png)

  * Define ![ \\varepsilon_m = \\begin{cases} 1 & m\\equiv 1\\mod 4 \\\\ i & m\\equiv 3\\mod 4 \\end{cases}](../../upload.wikimedia.org/math/e/b/4/eb484dd6a68e2334d82e901cb53bdf6e.png) for every odd integer _m_.

The values of Gauss sums with _b=0_ and gcd(_a_,_c_)=_1_ are explicitly given by

    ![
G\(a,c\) = G\(a,0,c\) = \\begin{cases} 0 & c\\equiv 2\\mod 4 \\\\ \\varepsilon_c \\sqrt{c} \\left\(\\frac{a}{c}\\right\) & c\\ \\text{odd} \\\\
\(1+i\) \\varepsilon_a^{-1} \\sqrt{c} \\left\(\\frac{c}{a}\\right\) & a\\ \\text{odd}, 4\\mid c.\\end{cases}
](../../upload.wikimedia.org/math/9/a/7/9a76a5f6b5ee6c28408ee0f13f5e1e4e.png)

Here ![ \\left\(\\frac{a}{c}\\right\)](../../upload.wikimedia.org/math/0/d/8/0d850ba20a03048ad6431bc478b5d6c8.png) is the [Jacobi symbol](http://en.wikipedia.org/wiki/Jacobi_symbol). This is the famous formula of [Carl Friedrich Gauß](http://en.wikipedia.org/wiki/Carl_Friedrich_Gau%C3%9F).

  * For _b_>_0_ the Gauss sums can easily be computed by [completing the square](http://en.wikipedia.org/wiki/Completing_the_square) in most cases. This fails however in some cases (for example _c_ even and _b_ odd) which can be computed relatively easy by other means. For example if _c_ is odd and gcd(_a_,_c_)=_1_ one has

    ![
G\(a,b,c\) =  \\varepsilon_c \\sqrt{c} \\cdot \\left\(\\frac{a}{c}\\right\) e^{-2\\pi i \\psi\(a\) b^2/c} 
](../../upload.wikimedia.org/math/b/7/f/b7faf4a91ccac5ce173616e59f94137a.png)

where ![\\psi\(a\) ](../../upload.wikimedia.org/math/1/9/9/19963447e2347dc2d53adc0b5a38c13c.png) is some number with ![4\\psi\(a\)a \\equiv 1\\ \\text{mod}\\ c ](../../upload.wikimedia.org/math/3/7/a/37a530b91aeea5727ae086d1881e7542.png). As another example, if _4_ divides _c_ and _b_ is odd and as always gcd(_a_,_c_)=_1_ then _G_(_a_,_b_,_c_)=_0_. This can, for example, be proven as follows: Because of the multiplicative property of Gauss sums we only have to show that ![ G\(a,b,2^n\)=0 ](../../upload.wikimedia.org/math/8/1/3/813ba1364aa953d0771039109cdcf640.png) if _n_>_1_ and _a,b_ are odd with gcd(_a_,_c_)=1. If _b_ is odd then ![ a n^2+bn](../../upload.wikimedia.org/math/e/a/c/eac952d973dcabe9eb007f3f63df758d.png) is even for all ![ 0\\leq n < c-1 ](../../upload.wikimedia.org/math/f/7/c/f7cb089c5fa9eb769e244a1d30816738.png). By [Hensel's lemma](http://en.wikipedia.org/wiki/Hensel%27s_lemma), for every _q_, the equation ![ an^2+bn+q=0 ](../../upload.wikimedia.org/math/5/b/f/5bf62b4321b0937110ead183239067bd.png) has at most two solutions in ![ \\mathbb{Z}/2^n \\mathbb{Z} ](../../upload.wikimedia.org/math/5/1/1/511fec194410b409968ff974e685ed22.png). Because of a counting argument ![ an^2+bn](../../upload.wikimedia.org/math/e/a/c/eac952d973dcabe9eb007f3f63df758d.png) runs through all even residue classes modulo _c_ exactly two times. The [geometric sum](http://en.wikipedia.org/wiki/Geometric_sum) formula then shows that ![ G\(a,b,2^n\)=0 ](../../upload.wikimedia.org/math/8/1/3/813ba1364aa953d0771039109cdcf640.png).

  * If _c_ is odd and [squarefree](http://en.wikipedia.org/wiki/Square-free_integer) and gcd(_a_,_c_)=_1_ then

    ![
G\(a,0,c\) = \\sum_{n=0}^{c-1} \\left\(\\frac{n}{c}\\right\) e^{2\\pi i a n/c}.
](../../upload.wikimedia.org/math/5/b/4/5b48a3488d033f8782bd8e06fb960403.png)

If _c_ is not squarefree then the right side vanishes while the left side does not. Often the right sum is also called a quadratic Gauss sum.

  * Another useful formula is

    _G_(_n_,_pk_)=_pG_(_n_,_p__k_-2)

if _k_≥2 and _p_ is an odd prime number or if _k_≥4 and _p_=2.

## See also[[edit](http://en.wikipedia.org/w/index.php?title=Quadratic_Gauss_sum&action=edit&section=5)]

  * [Gaussian period](http://en.wikipedia.org/wiki/Gaussian_period)
  * [Kummer sum](http://en.wikipedia.org/wiki/Kummer_sum)
  * [Landsberg-Schaar relation](http://en.wikipedia.org/wiki/Landsberg-Schaar_relation)

## References[[edit](http://en.wikipedia.org/w/index.php?title=Quadratic_Gauss_sum&action=edit&section=6)]

  * Ireland and Rosen (1990). _A Classical Introduction to Modern Number Theory_. Springer-Verlag. [ISBN](http://en.wikipedia.org/wiki/International_Standard_Book_Number) [0-387-97329-X](http://en.wikipedia.org/wiki/Special:BookSources/0-387-97329-X). 
  * Bruce C. Berndt, Ronald J. Evans and Kenneth S. Williams (1998). _Gauss and Jacobi Sums_. Wiley and Sons, Inc. [ISBN](http://en.wikipedia.org/wiki/International_Standard_Book_Number) [0-471-12807-4](http://en.wikipedia.org/wiki/Special:BookSources/0-471-12807-4). 
  * Henryk Iwaniec, Emmanuel Kowalski (2004). _Analytic number theory_. American Mathematical Society. [ISBN](http://en.wikipedia.org/wiki/International_Standard_Book_Number) [0-8218-3633-1](http://en.wikipedia.org/wiki/Special:BookSources/0-8218-3633-1). 
![](Special:CentralAutoLogin/start%3Ftype=1x1)

Retrieved from "[http://en.wikipedia.org/w/index.php?title=Quadratic_Gauss_sum&oldid=554731477](http://en.wikipedia.org/w/index.php?title=Quadratic_Gauss_sum&oldid=554731477)" 

[Categories](http://en.wikipedia.org/wiki/Help:Category): 

  * [Cyclotomic fields](http://en.wikipedia.org/wiki/Category:Cyclotomic_fields)

## Navigation menu

### Personal tools

  * [Create account](http://en.wikipedia.org/w/index.php?title=Special:UserLogin&returnto=Quadratic+Gauss+sum&type=signup)
  * [Log in](http://en.wikipedia.org/w/index.php?title=Special:UserLogin&returnto=Quadratic+Gauss+sum)

### Namespaces

  * [Article](Special:Random.html)
  * [Talk](http://en.wikipedia.org/wiki/Talk:Quadratic_Gauss_sum)

### 

### Variants

### Views

  * [Read](Special:Random.html)
  * [Edit](http://en.wikipedia.org/w/index.php?title=Quadratic_Gauss_sum&action=edit)
  * [View history](http://en.wikipedia.org/w/index.php?title=Quadratic_Gauss_sum&action=history)

### Actions

### Search

### Navigation

  * [Main page](/wiki/Main_Page)
  * ### Navigation

    * [Main page](http://en.wikipedia.org/wiki/Main_Page)
    * [Contents](http://en.wikipedia.org/wiki/Portal:Contents)
    * [Featured content](http://en.wikipedia.org/wiki/Portal:Featured_content)
    * [Current events](http://en.wikipedia.org/wiki/Portal:Current_events)
    * [Random article](Special:Random.html)
    * [Donate to Wikipedia](https://donate.wikimedia.org/wiki/Special:FundraiserRedirector?utm_source=donate&utm_medium=sidebar&utm_campaign=C13_en.wikipedia.org&uselang=en)
    * [Wikimedia Shop](http://shop.wikimedia.org/)

### Interaction

    * [Help](http://en.wikipedia.org/wiki/Help:Contents)
    * [About Wikipedia](http://en.wikipedia.org/wiki/Wikipedia:About)
    * [Community portal](http://en.wikipedia.org/wiki/Wikipedia:Community_portal)
    * [Recent changes](http://en.wikipedia.org/wiki/Special:RecentChanges)
    * [Contact page](http://en.wikipedia.org/wiki/Wikipedia:Contact_us)

### Tools

    * [What links here](http://en.wikipedia.org/wiki/Special:WhatLinksHere/Quadratic_Gauss_sum)
    * [Related changes](http://en.wikipedia.org/wiki/Special:RecentChangesLinked/Quadratic_Gauss_sum)
    * [Upload file](http://en.wikipedia.org/wiki/Wikipedia:File_Upload_Wizard)
    * [Special pages](http://en.wikipedia.org/wiki/Special:SpecialPages)
    * [Permanent link](http://en.wikipedia.org/w/index.php?title=Quadratic_Gauss_sum&oldid=554731477)
    * [Page information](http://en.wikipedia.org/w/index.php?title=Quadratic_Gauss_sum&action=info)
    * [Data item](http://www.wikidata.org/wiki/Q7268351)
    * [Cite this page](http://en.wikipedia.org/w/index.php?title=Special:Cite&page=Quadratic_Gauss_sum&id=554731477)

### Print/export

    * [Create a book](http://en.wikipedia.org/w/index.php?title=Special:Book&bookcmd=book_creator&referer=Quadratic+Gauss+sum)
    * [Download as PDF](http://en.wikipedia.org/w/index.php?title=Special:Book&bookcmd=render_article&arttitle=Quadratic+Gauss+sum&oldid=554731477&writer=rl)
    * [Printable version](http://en.wikipedia.org/w/index.php?title=Quadratic_Gauss_sum&printable=yes)

### Languages

    * [Svenska](http://sv.wikipedia.org/wiki/Kvadratisk_Gaussumma)
    *     * [Edit links](http://www.wikidata.org/wiki/Q7268351#sitelinks-wikipedia)

    * This page was last modified on 12 May 2013 at 13:40.  

    * Text is available under the [Creative Commons Attribution-ShareAlike License](http://en.wikipedia.org/wiki/Wikipedia:Text_of_Creative_Commons_Attribution-ShareAlike_3.0_Unported_License); additional terms may apply. By using this site, you agree to the [Terms of Use](http://wikimediafoundation.org/wiki/Terms_of_Use) and [Privacy Policy](http://wikimediafoundation.org/wiki/Privacy_policy). Wikipedia® is a registered trademark of the [Wikimedia Foundation, Inc.](http://www.wikimediafoundation.org/), a non-profit organization.
    * [Privacy policy](http://wikimediafoundation.org/wiki/Privacy_policy)
    * [About Wikipedia](http://en.wikipedia.org/wiki/Wikipedia:About)
    * [Disclaimers](http://en.wikipedia.org/wiki/Wikipedia:General_disclaimer)
    * [Contact Wikipedia](http://en.wikipedia.org/wiki/Wikipedia:Contact_us)
    * [Developers](https://www.mediawiki.org/wiki/Special:MyLanguage/How_to_contribute)
    * [Mobile view](http://en.m.wikipedia.org/wiki/Quadratic_Gauss_sum)
    * ![Wikimedia Foundation](../../bits.wikimedia.org/images/wikimedia-button.png)
    * ![Powered by MediaWiki](../../bits.wikimedia.org/static-1.24wmf1/skins/common/images/poweredby_mediawiki_88x31.png)
