# Vickrey auction

From Wikipedia, the free encyclopedia

Jump to: [navigation](Special:Random.html#mw-navigation), [search](Special:Random.html#p-search)

A **Vickrey auction** is a type of sealed-bid [auction](http://en.wikipedia.org/wiki/Auction). Bidders submit written bids without knowing the bid of the other people in the auction. The highest bidder wins but the price paid is the second-highest bid. The auction was first described academically by [Columbia University](http://en.wikipedia.org/wiki/Columbia_University) [professor](http://en.wikipedia.org/wiki/Professor) [William Vickrey](http://en.wikipedia.org/wiki/William_Vickrey) in 1961[[1]](Special:Random.html#cite_note-1) though it had been used by [stamp collectors](http://en.wikipedia.org/wiki/Stamp_collecting) since 1893.[[2]](Special:Random.html#cite_note-2) This type of auction is strategically similar to an [English auction](http://en.wikipedia.org/wiki/English_auction) and gives bidders an [incentive to bid their true value](http://en.wikipedia.org/wiki/Incentive_compatibility).

Vickrey's original paper mainly considered auctions where only a single, indivisible good is being sold. The terms _Vickrey auction_ and _second-price sealed-bid auction_ are, in this case only, equivalent and used interchangeably. When either a divisible good or multiple identical goods are sold in a single auction, however, these terms are used differently.

Vickrey auctions are much studied in economic literature but uncommon in practice. [eBay](http://en.wikipedia.org/wiki/EBay)'s system of [proxy bidding](http://en.wikipedia.org/wiki/Proxy_bid) is similar. A slightly generalized variant of a Vickrey auction, named [generalized second-price auction](http://en.wikipedia.org/wiki/Generalized_second-price_auction), is used in Google's and Yahoo!'s online advertisement programmes.[[3]](Special:Random.html#cite_note-3)[[4]](Special:Random.html#cite_note-4) [NYU Law School](http://en.wikipedia.org/wiki/NYU_Law_School) uses an iterated version of the Vickrey auction model for its course registration lottery.[_[citation needed](http://en.wikipedia.org/wiki/Wikipedia:Citation_needed)_] [[5]](Special:Random.html#cite_note-5)

## Contents

  * [1 Properties](Special:Random.html#Properties)
    * [1.1 Self-revelation/incentive compatibility](Special:Random.html#Self-revelation.2Fincentive_compatibility)
    * [1.2 Ex-post efficiency](Special:Random.html#Ex-post_efficiency)
    * [1.3 Weaknesses](Special:Random.html#Weaknesses)
  * [2 Proof of dominance of truthful bidding](Special:Random.html#Proof_of_dominance_of_truthful_bidding)
  * [3 Revenue equivalence of the Vickrey auction and sealed first price auction](Special:Random.html#Revenue_equivalence_of_the_Vickrey_auction_and_sealed_first_price_auction)
  * [4 Use in network routing](Special:Random.html#Use_in_network_routing)
  * [5 Generalizations](Special:Random.html#Generalizations)
  * [6 See also](Special:Random.html#See_also)
  * [7 References](Special:Random.html#References)
  * [8 Notes](Special:Random.html#Notes)

## Properties[[edit](http://en.wikipedia.org/w/index.php?title=Vickrey_auction&action=edit&section=1)]

### Self-revelation/incentive compatibility[[edit](http://en.wikipedia.org/w/index.php?title=Vickrey_auction&action=edit&section=2)]

In a Vickrey auction with private values each bidder maximizes their [expected utility](http://en.wikipedia.org/wiki/Expected_utility) by bidding (revealing) their valuation of the item for sale.

### Ex-post efficiency[[edit](http://en.wikipedia.org/w/index.php?title=Vickrey_auction&action=edit&section=3)]

A Vickrey auction is decision efficient (the winner is the bidder with the highest valuation) under the most general circumstances;[_[citation needed](http://en.wikipedia.org/wiki/Wikipedia:Citation_needed)_] it thus provides a baseline model against which the efficiency properties of other types of auctions can be posited. It is only ex-post efficient (sum of transfers equal to zero) if the seller is included as "player zero," whose transfer equals the negative of the sum of the other players' transfers (i.e. the bids).

### Weaknesses[[edit](http://en.wikipedia.org/w/index.php?title=Vickrey_auction&action=edit&section=4)]

Despite the Vickrey auction's strengths, it has shortcomings:

  * It does not allow for [price discovery](http://en.wikipedia.org/wiki/Price_discovery), that is, discovery of the market price if the buyers are unsure of their own valuations, without sequential auctions.
  * Sellers may use [shill](http://en.wikipedia.org/wiki/Shill) bids to increase profit.

The [Vickrey–Clarke–Groves](http://en.wikipedia.org/wiki/Vickrey%E2%80%93Clarke%E2%80%93Groves) (VCG) mechanism has the additional shortcomings:

  * It is vulnerable to bidder [collusion](http://en.wikipedia.org/wiki/Collusion). If all bidders in Vickrey auction reveal their valuations to each other, they can lower some or all of their valuations, while preserving who wins the auction. [[1]](http://www.maxi-pedia.com/vickrey+auction)
  * It is vulnerable to a version of shill biding in which a buyer uses multiple identities in the auction in order to maximize its profit. [[6]](Special:Random.html#cite_note-6)
  * It does not necessarily maximize seller revenues; seller revenues may even be zero in VCG auctions. If the purpose of holding the auction is to maximize profit for the seller rather than just allocate resources among buyers, then VCG may be a poor choice.
  * The seller's revenues are non-[monotonic](http://en.wikipedia.org/wiki/Monotonic) with regard to the sets of bidders and offers.

The non-monotonicity of seller's revenues with respect to bids (without introducing the VCG opportunity-cost mechanism described at the bottom of this article) can be shown by the following example. Consider 3 bidders A, B, and C, and two homogeneous items bid upon, Y and Z.

  * A wants both items and bids $2 for the package of Y and Z.
  * B and C both bid $2 each for a single item (bid $2 for Y or Z), as they really want one item but don't care if they have the second.

Now, Y and Z are allocated to B and C, but the price is $0, as can be found by removing either B or C respectively. If C bid $0 instead of $2, then the seller would make $2 instead of $0. Because the seller's revenue can go up when bids are either increased or decreased, the seller's revenues are non-monotonic with respect to bids.

## Proof of dominance of truthful bidding[[edit](http://en.wikipedia.org/w/index.php?title=Vickrey_auction&action=edit&section=5)]

The dominant strategy in a Vickrey auction with a single, indivisible item is for each bidder to bid their true value of the item.[[7]](Special:Random.html#cite_note-7)

Let ![v_i](../../upload.wikimedia.org/math/f/0/e/f0e66f55342ef85ba8be3415dd92d8e2.png) be bidder i's value for the item. Let ![b_i](../../upload.wikimedia.org/math/c/9/f/c9f6d8557ce40f989fa727b5c0bb1ddf.png) be bidder i's bid for the item.

The payoff for bidder i is ![
  \\begin{cases}
 v_i-\\max_{j\\neq i} b_j & \\text{if } b_i > \\max_{j\\neq i} b_j \\\\
0 & \\text{otherwise}
 \\end{cases}
 ](../../upload.wikimedia.org/math/f/d/0/fd0823efe4e51d7aa7d52b781d3782fa.png)

The strategy of overbidding is dominated by bidding truthfully. Assume that bidder i bids ![ b_i > v_i ](../../upload.wikimedia.org/math/1/f/a/1fa2da8e513e8425649cdaae8bdeef8a.png).

If ![\\max_{j\\neq i} b_j < v_i ](../../upload.wikimedia.org/math/0/0/6/006258981c14c92630fff27ebd965eab.png) then the bidder would win the item with a truthful bid as well as an overbid. The bid's amount does not change the payoff so the two strategies have equal payoffs in this case.

If ![\\max_{j\\neq i} b_j > b_i ](../../upload.wikimedia.org/math/1/7/9/1791bc233aa439bc5fcec752b1f1def1.png) then the bidder would lose the item either way so the strategies have equal payoffs in this case.

If ![v_i < \\max_{j\\neq i} b_j < b_i ](../../upload.wikimedia.org/math/d/b/c/dbc086abe135298efd57ae9bdb8a4dc6.png) then only the strategy of overbidding would win the auction. The payoff would be negative for the strategy of overbidding because they paid more than their value of the item, while the payoff for a truthful bid would be zero. Thus the strategy of bidding higher than one's true valuation is dominated by the strategy of truthfully bidding.

The strategy of underbidding is dominated by bidding truthfully. Assume that bidder i bids ![ b_i < v_i ](../../upload.wikimedia.org/math/5/4/4/5440a3249a3b4aed77e17ae12e2ec91d.png).

If ![\\max_{j\\neq i} b_j > v_i ](../../upload.wikimedia.org/math/d/f/6/df6d8d1d75435fac8f2e64365c68fbc7.png) then the bidder would lose the item with a truthful bid as well as an underbid, so the strategies have equal payoffs for this case.

If ![\\max_{j\\neq i} b_j < b_i ](../../upload.wikimedia.org/math/a/0/6/a06aa08de0a646a44ad420806d9ee9bf.png) then the bidder would win the item either way so the strategies have equal payoffs in this case.

If ![b_i < \\max_{j\\neq i} b_j < v_i ](../../upload.wikimedia.org/math/7/9/b/79b408520b4fa794a26a78534b745dbd.png) then only the strategy of truthfully bidding would win the auction. The payoff for the truthful strategy would be positive as they paid less than their value of the item, while the payoff for an underbid bid would be zero. Thus the strategy of underbidding is dominated by the strategy of truthfully bidding.

Truthful bidding dominates the other possible strategies (underbidding and overbidding) so it is an optimal strategy.

## Revenue equivalence of the Vickrey auction and sealed first price auction[[edit](http://en.wikipedia.org/w/index.php?title=Vickrey_auction&action=edit&section=6)]

The two most common auctions are the sealed first price (or high-bid) auction and the open ascending price (or English) auction. In the former each buyer submits a sealed bid. The high bidder is awarded the item and pays his or her bid. In the latter, the auctioneer announces successively higher asking prices and continues until no one is willing to accept a higher price. Suppose that a buyer's value is v and the current asking price is b. If v-b is negative, then the buyer loses by raising his hand. If v-b is positive and the buyer is not the current high bidder, it is more profitable to bid than to let someone else be the winner. Thus it is a dominant strategy for a buyer to drop out of the bidding when the asking price reaches his or her valuation. Thus, just as in the Vickrey sealed second price auction, the price paid by the buyer with the highest valuation is equal to the second highest value.

Consider then the expected payment in the sealed second-price auction. Vickrey considered the case of two buyers and assumed that each buyer's value was an independent draw from a uniform distribution with support [0,1]. With buyers bidding according to their dominant strategies, a buyer with value v wins if his opponent's value x < v. Suppose that v is the high value. Then the winning payment is uniformly distributed on the interval [0,v] and so the expected payment of the winner is

    ![e\(v\)=\\tfrac{1}{2}v](../../upload.wikimedia.org/math/e/1/d/e1d8dd9372f2a4244f4528a5d0afccb2.png).

We now argue that in the sealed first price auction the equilibrium bid of a buyer with value v is

    ![B\(v\)=e\(v\)=\\tfrac{1}{2}v](../../upload.wikimedia.org/math/e/1/a/e1a6e7be86081d7ea2d027ec4148cfe3.png).

That is, the payment of the winner in the sealed first-price auction is equal to the expected revenue in the sealed second-price auction.

**Proof of revenue equivalence**

Suppose that buyer 2 bids according to the strategy B(v) = v/2. We need to show that buyer 1's best response is to use the same strategy.

Note first that if uses the strategy B(v) = v/2, then buyer 2's maximum bid is B(1) = 1/2 and so buyer 1 wins with probability 1 with any bid of 1/2 or more. Consider then a bid b on the interval [0,1/2]. Let buyer 2's value be x. Then buyer 1 wins if B(x) = x/2 < b, that is if x < 2b. Under Vickrey's assumption of uniformly distributed values, the win probability is w(b) = 2b. Buyer 1's expected payoff is therefore

    ![U\(b\)=w\(b\)\(v-b\)=2b\(v-b\)=\\tfrac{1}{2}\[{{v}^{2}}-{{\(v-2b\)}^{2}}\]](../../upload.wikimedia.org/math/9/2/4/924f621d60ef5faf68f9b2b2aa3c5115.png)

Note that U(b) takes on its maximum at b = v/2 = B(v).

## Use in network routing[[edit](http://en.wikipedia.org/w/index.php?title=Vickrey_auction&action=edit&section=7)]

![](../../upload.wikimedia.org/wikipedia/en/thumb/9/99/Question_book-new.svg/50px-Question_book-new.svg.png)

This section **needs additional citations for [verification](http://en.wikipedia.org/wiki/Wikipedia:Verifiability)**. Please help [improve this article](http://en.wikipedia.org/w/index.php?title=Vickrey_auction&action=edit) by [adding citations to reliable sources](http://en.wikipedia.org/wiki/Help:Introduction_to_referencing/1). Unsourced material may be challenged and removed. _(May 2009)_

In [network routing](http://en.wikipedia.org/wiki/Network_routing), VCG mechanisms are a family of [payment](http://en.wikipedia.org/wiki/Payment) schemes based on the [added value](http://en.wikipedia.org/wiki/Added_value) concept. The basic idea of a VCG mechanism in network routing is to pay the owner of each link or node (depending on the network model) that is part of the solution, its declared cost _plus_ its added value. In many routing problems, this mechanism is not only [strategyproof](http://en.wikipedia.org/wiki/Strategyproof), but also the minimum among all strategyproof mechanisms.

In the case of network flows, [Unicast](http://en.wikipedia.org/wiki/Unicast) or [Multicast](http://en.wikipedia.org/wiki/Multicast), a minimum cost flow (MCF) in graph _G_ is calculated based on the declared costs _d__k_ of each of the links and payment is calculated as follows:

Each link (or node) ![\\scriptstyle e_k](../../upload.wikimedia.org/math/5/4/c/54ce1ac5c8011954a08a6452b46228d0.png) in the MCF is paid

    ![p_k = d_k + MCF\(G - e_k\) - MCF\(G\)](../../upload.wikimedia.org/math/c/d/e/cdef24b59d1cca80cd29ae0586899a71.png),

where MCF(_G_) indicates the cost of the minimum cost flow in graph _G_ and _G_ − _e__k_ indicates graph _G_ without the link _e__k_. Links not in the MCF are paid nothing. This routing problem is one of the cases for which VCG is strategyproof and minimum.

In 2004, it was shown that the expected VCG overpayment of an [Erdős–Rényi random graph](http://en.wikipedia.org/wiki/Erd%C5%91s%E2%80%93R%C3%A9nyi_model) with _n_ nodes and edge probability _p_, ![\\scriptstyle G \\in G\(n, p\)](../../upload.wikimedia.org/math/1/2/4/124e55c394498ed295ef335e244a9b6e.png) approaches

    ![ \\frac{p}{2-p} ](../../upload.wikimedia.org/math/d/b/e/dbeeb85051faeb69cb01c8b1ac2bb360.png)

as _n_, approaches ![\\scriptstyle \\infty ](../../upload.wikimedia.org/math/f/3/5/f352e235d35e218d377e0111fc3885c2.png), for ![n p = \\omega\(\\sqrt{n \\log n}\)](../../upload.wikimedia.org/math/f/e/a/fea8fa3fa2d8df715542a13ff5824b23.png). Prior to this result, it was known that VCG overpayment in _G_(_n_, _p_) is

    ![\\Omega\\left\(\\frac{1}{np}\\right\)](../../upload.wikimedia.org/math/2/4/7/2471111129bd427d2ddc4c5889377ab0.png)

and

    ![O\(1\)\\,](../../upload.wikimedia.org/math/3/c/a/3cafa487bd35dc5de745b73b42e41f8c.png)

with high probability given

    ![np=\\omega\(\\log n\).\\,](../../upload.wikimedia.org/math/3/a/d/3ad58e5ebd27d3fb6cee3c4248856210.png)

## Generalizations[[edit](http://en.wikipedia.org/w/index.php?title=Vickrey_auction&action=edit&section=8)]

The most obvious generalization to multiple or divisible goods is to have all winning bidders pay the amount of the highest non-winning bid. This is known as a _[uniform price auction](http://en.wikipedia.org/wiki/Uniform_price_auction)_. The uniform-price auction does not, however, result in bidders bidding their true valuations as they do in a second-price auction unless each bidder has demand for only a single unit. A generalization of the Vickrey auction that maintains the incentive to bid truthfully is known as the [Vickrey–Clarke–Groves](http://en.wikipedia.org/wiki/Vickrey%E2%80%93Clarke%E2%80%93Groves_auction) (VCG) mechanism. The idea in VCG is that items are assigned to maximize the sum of utilities; then each bidder pays the "opportunity cost" that their presence introduces to all the other players. This opportunity cost for a bidder is defined as the total bids of all the other bidders that would have won if the first bidder didn't bid, minus the total bids of all the other actual winning bidders.

For example, suppose two apples are being auctioned among three bidders.

  * Bidder A wants one apple and bids $5 for that apple.
  * Bidder B wants one apple and is willing to pay $2 for it.
  * Bidder C wants two apples and is willing to pay $6 to have both of them but is uninterested in buying only one without the other.

First, the outcome of the auction is determined by maximizing bids: the apples go to bidder A and bidder B. Next, the formula for deciding payments gives:

  * A: B and C have total utility $2 (the amount they pay together: $2 + $0) - if A were removed, the optimal allocation would give B and C total utility $6 ($0 + $6). So A pays $4 ($6 − $2).
  * B: A and C have total utility $5 ($5 + $0) - if B were removed, the optimal allocation would give A and C total utility $(0 + 6). So B pays $1 ($6 − $5).
  * similarly, C pays $0 (5 + 2) − (5 + 2) = $0.

## See also[[edit](http://en.wikipedia.org/w/index.php?title=Vickrey_auction&action=edit&section=9)]

  * [Auction theory](http://en.wikipedia.org/wiki/Auction_theory)

## References[[edit](http://en.wikipedia.org/w/index.php?title=Vickrey_auction&action=edit&section=10)]

  * Vijay Krishna, _Auction Theory_, Academic Press, 2002.
  * Peter Cramton, Yoav Shoham, Richard Steinberg (Eds), _Combinatorial Auctions_, MIT Press, 2006, Chapter 1. [ISBN 0-262-03342-9](http://en.wikipedia.org/wiki/Special:BookSources/0262033429).
  * Paul Milgrom, _Putting Auction Theory to Work_, Cambridge University Press, 2004.
  * Teck Ho, "Consumption and Production" UC Berkeley, Haas Class of 2010.

## Notes[[edit](http://en.wikipedia.org/w/index.php?title=Vickrey_auction&action=edit&section=11)]

  1. **[^](Special:Random.html#cite_ref-1)** Vickrey, William (1961). "Counterspeculation, Auctions, and Competitive Sealed Tenders". _The Journal of Finance_ **16** (1): 8–37. [doi](http://en.wikipedia.org/wiki/Digital_object_identifier):[10.1111/j.1540-6261.1961.tb02789.x](http://dx.doi.org/10.1111%2Fj.1540-6261.1961.tb02789.x). 
  2. **[^](Special:Random.html#cite_ref-2)** Lucking-Reiley, David (2000). "Vickrey Auctions in Practice: From Nineteenth-Century Philately to Twenty-First-Century E-Commerce". _Journal of Economic Perspectives_ **14** (3): 183–192. [doi](http://en.wikipedia.org/wiki/Digital_object_identifier):[10.1257/jep.14.3.183](http://dx.doi.org/10.1257%2Fjep.14.3.183). 
  3. **[^](Special:Random.html#cite_ref-3)** Benjamin Edelman, Michael Ostrovsky, and Michael Schwarz: "Internet Advertising and the Generalized Second-Price Auction: Selling Billions of Dollars Worth of Keywords". American Economic Review 97(1), 2007 pp 242–259.
  4. **[^](Special:Random.html#cite_ref-4)** Hal R. Varian: "Position Auctions". International Journal of Industrial Organization, 2006, [doi](http://en.wikipedia.org/wiki/Digital_object_identifier):[10.1016/j.ijindorg.2006.10.002](http://dx.doi.org/10.1016%2Fj.ijindorg.2006.10.002) .
  5. **[^](Special:Random.html#cite_ref-5)** Memorandum from the Office of the Vice Dean of NYU School of Law. <http://www.law.nyu.edu/ecm_dlv/groups/public/@nyu_law_website__academics/documents/web_copytext/ecm_pro_061262.pdf>
  6. **[^](Special:Random.html#cite_ref-6)** Lawrence M. Ausubel, and Paul Milgrom . [The Lovely but Lonely Vickrey Auction](http://www.stanford.edu/~milgrom/publishedarticles/Lovely%20but%20Lonely%20Vickrey%20Auction-072404a.pdf). Combinatorial Auctions, MIT Press, 2006, Chapter 1, p. 12, .
  7. **[^](Special:Random.html#cite_ref-7)** [von Ahn, Luis](http://en.wikipedia.org/wiki/Luis_von_Ahn) (2008-09-30). ["Auctions"](http://www.scienceoftheweb.org/15-396/lectures/lecture09.pdf) (PDF). _15–396: Science of the Web Course Notes_. Carnegie Mellon University. Retrieved 2008-11-06. 

![](Special:CentralAutoLogin/start%3Ftype=1x1)

Retrieved from "[http://en.wikipedia.org/w/index.php?title=Vickrey_auction&oldid=594424816](http://en.wikipedia.org/w/index.php?title=Vickrey_auction&oldid=594424816)" 

[Categories](http://en.wikipedia.org/wiki/Help:Category): 

  * [Auctioneering](http://en.wikipedia.org/wiki/Category:Auctioneering)
  * [Networks](http://en.wikipedia.org/wiki/Category:Networks)
  * [Graph theory](http://en.wikipedia.org/wiki/Category:Graph_theory)
  * [Auction theory](http://en.wikipedia.org/wiki/Category:Auction_theory)

Hidden categories: 

  * [All articles with unsourced statements](http://en.wikipedia.org/wiki/Category:All_articles_with_unsourced_statements)
  * [Articles with unsourced statements from January 2014](http://en.wikipedia.org/wiki/Category:Articles_with_unsourced_statements_from_January_2014)
  * [Articles with unsourced statements from November 2012](http://en.wikipedia.org/wiki/Category:Articles_with_unsourced_statements_from_November_2012)
  * [Articles needing additional references from May 2009](http://en.wikipedia.org/wiki/Category:Articles_needing_additional_references_from_May_2009)
  * [All articles needing additional references](http://en.wikipedia.org/wiki/Category:All_articles_needing_additional_references)

## Navigation menu

### Personal tools

  * [Create account](http://en.wikipedia.org/w/index.php?title=Special:UserLogin&returnto=Vickrey+auction&type=signup)
  * [Log in](http://en.wikipedia.org/w/index.php?title=Special:UserLogin&returnto=Vickrey+auction)

### Namespaces

  * [Article](Special:Random.html)
  * [Talk](http://en.wikipedia.org/wiki/Talk:Vickrey_auction)

### 

### Variants

### Views

  * [Read](Special:Random.html)
  * [Edit](http://en.wikipedia.org/w/index.php?title=Vickrey_auction&action=edit)
  * [View history](http://en.wikipedia.org/w/index.php?title=Vickrey_auction&action=history)

### Actions

### Search

### Navigation

  * ### Navigation

    * [Main page](http://en.wikipedia.org/wiki/Main_Page)
    * [Contents](http://en.wikipedia.org/wiki/Portal:Contents)
    * [Featured content](http://en.wikipedia.org/wiki/Portal:Featured_content)
    * [Current events](http://en.wikipedia.org/wiki/Portal:Current_events)
    * [Random article](Special:Random.html)
    * [Donate to Wikipedia](https://donate.wikimedia.org/wiki/Special:FundraiserRedirector?utm_source=donate&utm_medium=sidebar&utm_campaign=C13_en.wikipedia.org&uselang=en)
    * [Wikimedia Shop](http://shop.wikimedia.org/)

### Interaction

    * [Help](http://en.wikipedia.org/wiki/Help:Contents)
    * [About Wikipedia](http://en.wikipedia.org/wiki/Wikipedia:About)
    * [Community portal](http://en.wikipedia.org/wiki/Wikipedia:Community_portal)
    * [Recent changes](http://en.wikipedia.org/wiki/Special:RecentChanges)
    * [Contact page](http://en.wikipedia.org/wiki/Wikipedia:Contact_us)

### Tools

    * [What links here](http://en.wikipedia.org/wiki/Special:WhatLinksHere/Vickrey_auction)
    * [Related changes](http://en.wikipedia.org/wiki/Special:RecentChangesLinked/Vickrey_auction)
    * [Upload file](http://en.wikipedia.org/wiki/Wikipedia:File_Upload_Wizard)
    * [Special pages](http://en.wikipedia.org/wiki/Special:SpecialPages)
    * [Permanent link](http://en.wikipedia.org/w/index.php?title=Vickrey_auction&oldid=594424816)
    * [Page information](http://en.wikipedia.org/w/index.php?title=Vickrey_auction&action=info)
    * [Data item](http://www.wikidata.org/wiki/Q243758)
    * [Cite this page](http://en.wikipedia.org/w/index.php?title=Special:Cite&page=Vickrey_auction&id=594424816)

### Print/export

    * [Create a book](http://en.wikipedia.org/w/index.php?title=Special:Book&bookcmd=book_creator&referer=Vickrey+auction)
    * [Download as PDF](http://en.wikipedia.org/w/index.php?title=Special:Book&bookcmd=render_article&arttitle=Vickrey+auction&oldid=594424816&writer=rl)
    * [Printable version](http://en.wikipedia.org/w/index.php?title=Vickrey_auction&printable=yes)

### Languages

    * [Čeština](http://cs.wikipedia.org/wiki/Vickreyova_aukce)
    * [Deutsch](http://de.wikipedia.org/wiki/Zweitpreisauktion)
    * [Español](http://es.wikipedia.org/wiki/Subasta_Vickrey)
    * [Français](http://fr.wikipedia.org/wiki/Ench%C3%A8re_de_Vickrey)
    * [Italiano](http://it.wikipedia.org/wiki/Asta_di_Vickrey)
    * [עברית](http://he.wikipedia.org/wiki/%D7%9E%D7%9B%D7%A8%D7%96_%D7%9E%D7%97%D7%99%D7%A8_%D7%A9%D7%A0%D7%99)
    * [Nederlands](http://nl.wikipedia.org/wiki/Vickreyveiling)
    * [Русский](http://ru.wikipedia.org/wiki/%D0%90%D1%83%D0%BA%D1%86%D0%B8%D0%BE%D0%BD_%D0%92%D0%B8%D0%BA%D1%80%D0%B8)
    * [Svenska](http://sv.wikipedia.org/wiki/Andraprisauktion)
    * [中文](http://zh.wikipedia.org/wiki/%E7%B6%AD%E5%85%8B%E9%87%8C%E6%8B%8D%E8%B3%A3)
    *     * [Edit links](http://www.wikidata.org/wiki/Q243758#sitelinks-wikipedia)

    * This page was last modified on 7 February 2014 at 21:33.  

    * Text is available under the [Creative Commons Attribution-ShareAlike License](http://en.wikipedia.org/wiki/Wikipedia:Text_of_Creative_Commons_Attribution-ShareAlike_3.0_Unported_License); additional terms may apply. By using this site, you agree to the [Terms of Use](http://wikimediafoundation.org/wiki/Terms_of_Use) and [Privacy Policy](http://wikimediafoundation.org/wiki/Privacy_policy). Wikipedia® is a registered trademark of the [Wikimedia Foundation, Inc.](http://www.wikimediafoundation.org/), a non-profit organization.
    * [Privacy policy](http://wikimediafoundation.org/wiki/Privacy_policy)
    * [About Wikipedia](http://en.wikipedia.org/wiki/Wikipedia:About)
    * [Disclaimers](http://en.wikipedia.org/wiki/Wikipedia:General_disclaimer)
    * [Contact Wikipedia](http://en.wikipedia.org/wiki/Wikipedia:Contact_us)
    * [Developers](https://www.mediawiki.org/wiki/Special:MyLanguage/How_to_contribute)
    * [Mobile view](http://en.m.wikipedia.org/wiki/Vickrey_auction)
    * ![Wikimedia Foundation](../../bits.wikimedia.org/images/wikimedia-button.png)
    * ![Powered by MediaWiki](../../bits.wikimedia.org/static-1.24wmf1/skins/common/images/poweredby_mediawiki_88x31.png)
