# Human Physiology/Print Version

From Wikibooks, open books for an open world

< [Human Physiology](http://en.wikibooks.org/wiki/Human_Physiology)

Jump to: [navigation](Print_Version.html#mw-navigation), [search](Print_Version.html#p-search)

  


![Printer.svg](../../../upload.wikimedia.org/wikipedia/commons/thumb/2/23/Printer.svg/40px-Printer.svg.png)

**This is the [print version](http://en.wikibooks.org/wiki/Help:Print_versions) of [Human Physiology](http://en.wikibooks.org/wiki/Human_Physiology)**  
You won't see this message or any elements not part of the book's content when you print or [preview](http://en.wikibooks.org/w/index.php?title=Human_Physiology/Print_Version&action=purge&printable=yes) this page.

  


# Homeostasis

## Overview

The human organism consists of trillions of cells all working together for the maintenance of the entire organism. While cells may perform very different functions, all the cells are quite similar in their metabolic requirements. Maintaining a constant internal environment with all that the cells need to survive (oxygen, glucose, mineral ions, waste removal, and so forth) is necessary for the well-being of individual cells and the well-being of the entire body. The varied processes by which the body regulates its internal environment are collectively referred to as homeostasis.

### What is Homeostasis?

**Homeostasis** in a general sense refers to stability, balance or equilibrium. It is the body's attempt to maintain a constant internal environment. Maintaining a stable internal environment requires constant monitoring and adjustments as conditions change. This adjusting of physiological systems within the body is called _homeostatic regulation_.

Homeostatic regulation involves three parts or mechanisms: 1) the _**receptor**_, 2) the _**control center**_ and 3) the _**effector**_.

The _**receptor**_ receives information that something in the environment is changing. The _**control center**_ or _**integration center**_ receives and processes information from the _**receptor**_. And lastly, the _**effector**_ responds to the commands of the _**control center**_ by either opposing or enhancing the stimulus. This is an ongoing process that continually works to restore and maintain homeostasis. For example, in regulating body temperature there are temperature _receptors_ in the skin, which communicate information to the brain, which is the _control center,_ and the _effector_ is our blood vessels and sweat glands in our skin.

Because the internal and external environment of the body are constantly changing and adjustments must be made continuously to stay at or near the set point, homeostasis can be thought of as a _synthetic equilibrium._

Since homeostasis is an attempt to maintain the internal conditions of an environment by limiting fluctuations, it must involve a series of negative feedback loops.

### Positive and Negative Feedback

When a change of variable occurs, there are two main types of feedback to which the system reacts:

  * _**Negative feedback**_: a reaction in which the system responds in such a way as to reverse the direction of change. Since this tends to keep things constant, it allows the maintenance of homeostasis. For instance, when the concentration of carbon dioxide in the human body increases, the lungs are signaled to increase their activity and expel more carbon dioxide. Thermoregulation is another example of negative feedback. When body temperature rises, receptors in the skin and the hypothalamus sense a change, triggering a command from the brain. This command, in turn, effects the correct response, in this case a decrease in body temperature.

    

    **Home Heating System Vs. Negative Feedback**
    When you are at home, you set your thermostat to a desired temperature. Let's say today you set it at 70 degrees. The thermometer in the thermostat waits to sense a temperature change either too high above or too far below the 70 degree set point. When this change happens the thermometer will send a message to to the "Control Center", or thermostat, Which in turn will then send a message to the furnace to either shut off if the temperature is too high or kick back on if the temperature is too low. In the home-heating example the air temperature is the "NEGATIVE FEEDBACK." When the Control Center receives negative feedback it triggers a chain reaction in order to maintain room temperature.

  * _**Positive feedback**_: a response is to amplify the change in the variable. This has a destabilizing effect, so does not result in homeostasis. Positive feedback is less common in naturally occurring systems than negative feedback, but it has its applications. For example, in nerves, a threshold electric potential triggers the generation of a much larger action potential. Blood clotting in which the platelets process mechanisms to transform blood liquid to solidify is an example of positive feedback loop. Another example is the secretion of oxytocin which provides a pathway for the uterus to contract, leading to child birth.

    

    **Harmful Positive Feedback**
    Although Positive Feedback is needed within Homeostasis it also can be harmful at times. When you have a high fever it causes a metabolic change that can push the fever higher and higher. In rare occurrences the body temperature reaches 113 degrees Fahrenheit / 45 degrees Celsius and the cellular proteins stop working and the metabolism stops, resulting in death.

**Summary**: Sustainable systems require combinations of both kinds of feedback. Generally with the recognition of divergence from the homeostatic condition, positive feedbacks are called into play, whereas once the homeostatic condition is approached, negative feedback is used for "fine tuning" responses. This creates a situation of "metastability," in which homeostatic conditions are maintained within fixed limits, but once these limits are exceeded, the system can shift wildly to a wholly new (and possibly less desirable) situation of homeostasis.

  
_**Homeostatic systems have several properties**_

  * They are ultra-stable, meaning the system is capable of testing which way its variables should be adjusted.
  * Their whole organization (internal, structural, and functional) contributes to the maintenance of balance.
  * Physiology is largely a study of processes related to homeostasis. Some of the functions you will learn about in this book are not specifically about homeostasis (e.g. how muscles contract), but in order for all bodily processes to function there must be a suitable internal environment. Homeostasis is, therefore, a fitting framework for the introductory study of physiology.

_**Where did the term "Homeostasis" come from?**_

The concept of homeostasis was first articulated by the French scientist Claude Bernard (1813-1878) in his studies of the maintenance of stability in the "milieu interior." He said, "All the vital mechanisms, varied as they are, have only one object, that of preserving constant the conditions of life in the internal environment" (from _Leçons sur les Phénonèmes de la Vie Commune aux Animaux et aux Végétaux_, 1879). The term itself was coined by American physiologist Walter Cannon, author of _The Wisdom of the Body_ (1932). The word comes from the Greek _homoios_ (same, like, resembling) and _stasis_ (to stand, posture).

### Cruise Control on a car as a simple metaphor for homeostasis

When a car is put on cruise control it has a set speed limit that it will travel. At times this speed may vary by a few miles per hour but in general the system will maintain the set speed. If the car starts to go up a hill, the systems will automatically increase the amount of fuel given to maintain the set speed. If the car starts to come down a hill, the car will automatically decrease the amount of fuel given in order to maintain the set speed. It is the same with homeostasis- the body has a set limit on each environment. If one of these limits increases or decreases, the body will sense and automatically try to fix the problem in order to maintain the pre-set limits. This is a simple metaphor of how the body operates—constant monitoring of levels, and automatic small adjustments when those levels fall below (or rise above) a set point.

## Pathways That Alter Homeostasis

A variety of homeostatic mechanisms maintain the internal environment within tolerable limits. Either homeostasis is maintained through a series of control mechanisms, or the body suffers various illnesses or disease. When the cells in the body begin to malfunction, the homeostatic balance becomes disrupted. Eventually this leads to disease or cell malfunction. Disease and cellular malfunction can be caused in two basic ways: either, _deficiency_ (cells not getting all they need) or _toxicity_ (cells being poisoned by things they do not need). When homeostasis is interrupted in your cells, there are _pathways_ to correct or worsen the problem. In addition to the internal control mechanisms, there are external influences based primarily on lifestyle choices and environmental exposures that influence our body's ability to maintain cellular health.

  * **Nutrition:** If your diet is lacking in a specific vitamin or mineral your cells will function poorly, possibly resulting in a disease condition. For example, a menstruating woman with inadequate dietary intake of iron will become anemic. Lack of hemoglobin, a molecule that requires iron, will result in reduced oxygen-carrying capacity. In mild cases symptoms may be vague (e.g. fatigue), but if the anemia (British English: _anaemia_) is severe the body will try to compensate by increasing cardiac output, leading to palpitations and sweatiness, and possibly to heart failure.
  * **Toxins:** Any substance that interferes with cellular function, causing cellular malfunction. This is done through a variety of ways; chemical, plant, insecticides, and/or bites. A commonly seen example of this is drug overdoses. When a person takes too much of a drug their vital signs begin to waver; either increasing or decreasing, these vital signs can cause problems including coma, brain damage and even death.
  * **Psychological:** Your physical health and mental health are inseparable. Our thoughts and emotions cause chemical changes to take place either for better as with meditation, or worse as with stress.
  * **Physical:** Physical maintenance is essential for our cells and bodies. Adequate rest, sunlight, and exercise are examples of physical mechanisms for influencing homeostasis. Lack of sleep is related to a number of ailments such as irregular cardiac rhythms, fatigue, anxiety and headaches.
  * **Genetic/Reproductive:** Inheriting strengths and weaknesses is part of our genetic makeup. Genes are sometimes turned off or on due to external factors which we can have some control over, but at other times little can be done to correct or improve genetic diseases. Beginning at the cellular level a variety of diseases come from mutated genes. For example, cancer can be genetically inherited or can be caused due to a mutation from an external source such as radiation or genes altered in a fetus when the mother uses drugs.
  * **Medical:** Because of genetic differences some bodies need help in gaining or maintaining homeostasis. Through modern medicine our bodies can be given different aids, from anti-bodies to help fight infections, or chemotherapy to kill harmful cancer cells. Traditional and alternative medical practices have many benefits, but like any medical practice the potential for harmful effects is present. Whether by nosocomial infections, or wrong dosage of medication, homeostasis can be altered by that which is trying to fix it. Trial and error with medications can cause potential harmful reactions and possibly death if not caught soon enough.

The factors listed above all have their effects at the cellular level, whether harmful or beneficial. Inadequate beneficial pathways (deficiency) will almost always result in a harmful waver in homeostasis. Too much toxicity also causes homeostatic imbalance, resulting in cellular malfunction. By removing negative health influences, and providing adequate positive health influences, your body is better able to self-regulate and self-repair, thus maintaining homeostasis.

## Homeostasis Throughout the Body

Each body system contributes to the homeostasis of other systems and of the entire organism. No system of the body works in isolation, and the well-being of the person depends upon the well-being of all the interacting body systems. A disruption within one system generally has consequences for several additional body systems. Here are some brief explanations of how various body systems contribute to the maintenance of homeostasis:

### Nervous System

Since the nervous system does not store nutrients, it must receive a continuous supply from blood. Any interruption to the flow of blood may bring brain damage or death. The nervous system maintains homeostasis by controlling and regulating the other parts of the body. A deviation from a normal set point acts as a stimulus to a receptor, which sends nerve impulses to a regulating center in the brain. The brain directs an effector to act in such a way that an adaptive response takes place. If, for example, the deviation was a lowering of body temperature, the effector acts to increase body temperature. The adaptive response returns the body to a state of normalcy and the receptor, the regulating center, and the effector temporarily cease their activities. Since the effector is regulated by the very conditions it produced, this process is called control by negative feedback. This manner of regulating normalcy results in a fluctuation between two extreme levels. Not until body temperature drops below normal do receptors stimulate the regulating center and effectors act to raise body temperature. Regulating centers are located in the central nervous system, consisting of the brain and spinal cord. The hypothalamus is a portion of the brain particularly concerned with homeostasis; it influences the action of the medulla oblongata, a lower part of the brain, the autonomic nervous system, and the pituitary gland.

The nervous system has two major portions: the central nervous system and the peripheral nervous system. The peripheral nervous system consists of the cranial and spinal nerves. The autonomic nervous system is a part of peripheral nervous system and contains motor neurons that control internal organs. It operates at the subconscious level and has two divisions, the sympathetic and parasympathetic systems. In general, the sympathetic system brings about those results we associate with emergency situations, often called fight or flight reactions, and the parasympathetic system produces those effects necessary to our everyday existence.

### Endocrine System

The endocrine system consists of glands which secrete hormones into the bloodstream. Each hormone has an effect on one or more target tissues. In this way the endocrine system regulates the metabolism and development of most body cells and body systems. To be more specific, the Endocrine system has sex hormones that can activate sebaceous glands, development of mammary glands, alter dermal blood flow and release lipids from adipocytes. MSH can stimulate melanocytes on our skin. Our bone growth is regulated by several hormones, and the endocrine system helps with the mobilization of calcitonin and calcium. In the muscular system, hormones adjust muscle metabolism, energy production, and growth. In the nervous system, hormones affect neural metabolism, regulate fluid/electrolyte balance and help with reproductive hormones that influence CNS development and behaviors. In the Cardiovascular system, we need hormones that regulate the production of RBC's (red blood cells), which elevate and lower blood pressure. Hormones also have anti-inflammatory effects and stimulate the lymphatic system. In summary, the endocrine system has a regulatory effect on basically every other body system.

### Integumentary System

The integumentary system (the skin) is involved in protecting the body from invading microbes (mainly by forming a thick impenetrable layer), regulating body temperature through sweating and vasodilation/vasoconstriction, or shivering and piloerection (goose bumps), and regulating ion balances in the blood. Stimulation of mast cells also produce changes in blood flow and capillary permeability which can effect the blood flow in the body and how it is regulated. It also helps synthesize vitamin D which interacts with calcium and phosphorus absorption needed for bone growth, maintenance, and repair. Hair on the skin guards entrance into the nasal cavity or other orifices, preventing invaders from getting further into our bodies. Our skin also helps maintain balance by excretion of water and other solutes (i.e.) the keratinized epidermis limits fluid loss through skin. It also provides mechanical protection against environmental hazards. We need to remember that our skin is integumentary; it is our first line of defense.

### Skeletal System

As the structural framework for the human body, the skeletal system consists mainly of the 206 or so bones of the skeletal system but also includes cartilages, ligaments, and other connective tissues that stabilize and interconnect them. Bones work in conjunction with the muscular system to aid in posture and locomotion. Many bones of the skeleton function as levers, which change the magnitude and direction of forces generated by skeletal muscle. Protection is a pivotal role occupied by the skeletal system, as many vital organs are encased within the skeletal cavities (eg. cranial and spinal), and bones form much of the structural basis for other body cavities (ex: thoracic and pelvic cavities). The skeletal system also serves as an important mineral reserve. For example, if blood levels of calcium or magnesium are low and the minerals are not available in the diet, they will be taken from the bones. Also, the skeletal system provides calcium needed for all muscular contraction. Finally, red blood cells, lymphocytes and other cells relating to the immune response are produced and stored in the bone marrow.

### Muscular System

The muscular system is one of the most versatile systems in the body. The muscular system contains the heart, which constantly pumps blood through the body. The muscular system is also responsible for involuntary (e.g. goosebumps, digestion, breathing) and voluntary (e.g. walking, picking up objects) actions. Muscles also help protect organs in the body's cavities. The muscles in your body contract, which increases your body heat when you're cold. The act of shivering occurs when internal temperature drops. Muscles around vital organs contract, breaking down ATP, and thereby expending heat, which is then distributed to the rest of the body.

### Cardiovascular System

The cardiovascular system, in addition to needing to maintain itself within certain levels, plays a role in maintenance of other body systems by transporting hormones (heart secretes Atrial Natriuretic Peptide and Brain Natriuretic Peptide, or ANP and BNP, respectively) and nutrients (oxygen, EPO to bones,etc.), taking away waste products, and providing all living body cells with a fresh supply of oxygen and removing carbon dioxide. Homeostasis is disturbed if the cardiovascular or lymphatic systems are not functioning correctly. Our skin, bones, muscles, lungs, digestive tract, and nervous, endocrine, lymphatic, urinary and reproductive systems use the cardiovascular system as its "road" or "highway" as far as distribution of things such as nutrients, oxygen, waste products, hormones, drugs, etc. There are many risk factors for an unhealthy cardiovascular system. Some diseases associated are typically labeled "uncontrollable" or "controllable." The main uncontrollable risk factors are age, gender, and a family history of heart disease, especially at an early age.

The cardiovascular system also contains sensors to monitor blood pressure, called baroreceptors, that work by detecting how stretched a blood vessel is. This information is relayed to the Medulla Oblongata in the brain where action is taken to raise or lower blood pressure via the autonomic nervous system.

### Lymphatic System

The lymphatic system has three principal roles. First is the maintenance of blood and tissue volume. Excess fluid that leaves the capillaries when under pressure would build up and cause edema. Secondly, the lymphatic system absorbs fatty acids and triglycerides from fat digestion so that these components of digestion do not enter directly into the blood stream. Third, the lymphatic system is involved in defending the body against invading microbes, and the immune response. This system assists in maintenance, such as bone and muscle repair after injuries. Another defense is maintaining the acidic pH of urine to fight infections in the urinary system. The tonsils are our bodies "helpers" to defend us against infections and toxins absorbed from the digestive tract. The tonsils also protect against infections entering into our lungs.

### Respiratory System

The respiratory system works in conjunction with the cardiovascular system to provide oxygen to cells within every body system for cellular metabolism. The respiratory system also removes carbon dioxide. Since CO2 is mainly transported in the plasma as bicarbonate ions, which act as a chemical buffer, the respiratory system also helps maintain proper blood pH levels, a fact that is very important for homeostasis. As a result of hyperventilation, CO2 is decreased in blood levels. This causes the pH of body fluids to increase. If acid levels rise above 7.45, the result is respiratory alkalosis. On the other hand, too much CO2 causes pH to fall below 7.35 which results in respiratory acidosis. The respiratory system also helps the lymphatic system by trapping pathogens and protecting deeper tissues within. Note that when you have increased thoracic space it can provide abdominal pressure through the contraction of respiratory muscles. This can assist in defecation. Remember the lungs are the gateway for our breath of life.

or

The organ of the respiratory system includes the nose, pharynx, larynx, trachea, bronchi and lungs. Together these organs permit the movement of air into the tiny, thin walled sacs of the lungs called alveoli. It is in the alveoli that oxygen from the air is exchanged for the waste product carbon dioxide, which is carried to lungs by the blood so that it can be eliminated from the body.

### Digestive System

Without a regular supply of energy and nutrients from the digestive system, all body systems would soon suffer. The digestive system absorbs organic substances, vitamins, ions, and water that are needed all over the body. In the skin, the digestive tract provides lipids for storage in the subcutaneous layer. Note that food undergoes three types of processes in the body: digestion, absorption, and elimination. If one of these is not working, you will have problems that will be extremely noticeable. Mechanics of digestion can include chemical digestion, movements, ingestion absorption, and elimination. In order to maintain a healthy and efficient digestive system, we have to remember the components involved. If these are disturbed, digestive health may be compromised.

### Urinary System

Toxic nitrogenous wastes accumulate as proteins and nucleic acids are broken down and used for other purposes. The urinary system rids the body of these wastes. The urinary system is also directly involved in maintaining proper blood volume (and indirectly blood pressure) and ion concentration within the blood. One other contribution is that the kidneys produce a hormone (erythropoietin) that stimulates red blood cell production. The kidneys also play an important role in maintaining the correct water content of the body and the correct salt composition of extracellular fluid. External changes that lead to excess fluid loss trigger feedback mechanisms that act to inhibit fluid loss.

### Reproductive System

The Reproductive System is unique in that it does little to contribute to the homeostasis of the organism. Rather than being tied to the maintenance of the organism, the reproductive system relates to the maintenance of the species. Having said that, the sex hormones do have an effect on other body systems, and an imbalance can lead to various disorders (e.g. a woman whose ovaries are removed early in life is at much higher risk of osteoporosis).

### Excretory System

Excretory System is responsible for removing wastes, excess water and salt in the urine. Regulates the volume and pH of the internal environment. The human excretory system maintains homeostasis by removing metabolic waste such as water, salt and metabolite concentrations in the blood. The kidneys, which are the primary excretory organs, are major organs of homeostasis because they excrete nitrogenous wastes, and regulate water-salt balance and acid base balance. This section will examine the kidney in details.

## Thermoregulation

The living bodies have been characterized with a number of automated processes, which make them self-sustainable in the natural environment. Among these many processes are that of reproduction, adjustment with external environment, and instinct to live, which are gifted by nature to living beings.

The survival of living beings greatly depends on their capability to maintain a stable body temperature irrespective of temperature of surrounding environment. This capability of maintaining body temperature is called thermoregulation. Cold blooded animals, such as reptiles, have somewhat different means of temperature regulation than warm blooded (or homeothermic) animals, such as humans and other mammals. This section is most relevant when considering warm blooded organisms.

Body temperature depends on the heat produced minus the heat lost. Heat is lost by radiation, convection, and conduction, but the net loss by all three processes depends on a gradient between the body and the outside. Thus, when the external temperature is low, radiation is the most important form of heat loss. When there is a high external temperature, evaporation is the most important form of heat loss. The balance of heat produced and heat lost maintains a constant body temperature. However, temperature does vary during the day, and this set point is controlled by the hypothalamus.

Body temperature is usually about 37.4°C, but does vary during the day by about 0.8°C. The lowest daily temperature is when the person is asleep. Temperature receptors are found in the skin, the great veins, the abdominal organs and the hypothalamus. While the ones in the skin provide the sensation of coldness, the hypothalamic (central core) temperature receptors are the most important. The core body temperature is usually about 0.7-1.0°C higher than axillary or oral temperature.

When body temperature drops due to external cold, an important component of protection is vaso-constriction of skin and limb blood vessels. This drops the surface temperature, providing an insulating layer (such as the fat cell layer) between the core temperature and the external environment. Likewise, if the temperature rises, blood flow to the skin increases, maximizing the potential for loss by radiation and evaporation. Thus, if you dilated the skin blood vessels by alcohol ingestion this might give a nice warm glow, but it would increase heat loss (if the external temperature was still low). The major adjustments in cold are to shiver to increase heat production, and constrict blood vessels in the periphery and skin. This helps to minimize heat loss through the skin, and directs blood to the vital internal organs.

Besides the daily variation in body temperature, there are other cyclic variations. In women, body temperature falls prior to ovulation and rises by about 1°C at ovulation, largely due to progesterone increasing the set point. Thyroid hormone and pyrogens also increase the set point. The basal metabolic rate (BMR) is about 30 calories/sq m/h. It is higher in children than in adults, partly as a result of different surface area to body mass ratio. Due to this relationship, young children are more likely to drop their temperature rapidly; there is greater temperature variation in children than in adults. It is increased by thyroid hormone and decreased by thyroid hormone lack. Different foods can affect BMR and the Respiratory Quotient of foods differ. Carbohydrate 1.0; Protein = 1.0; Fats = 0.7

## Body Composition

  


Extracellular Fluid

Cellular Fluid

Volume

plasma – 3 litres

interstitial – 10 litres

30 litres

Osmolality (mOsm)

290

290

Na + (mmol/l)

140

15

Ca 2+ (mmol/l)

2.2

< 10 -6

Cl - (mmol/l)

110

10

HCO3 - (mmol/l)

30

10

K + (mmol/l)

4

150

Mg 2+ (mmol/l)

1.5

15

PO4 3+ (mmol/l)

2

40

pH

7.4

7.1

Potential Difference (mV)

  


-70

Blood pressure is expressed as two different numbers. The first number is called the "systolic" blood pressure, and the second is the "diastolic" blood pressure. Systolic blood pressure is the pressure at the time of the cardiac cycle when the heart contracts, forcing blood out(called systole). This is the time of greatest pressure. The Diastolic number comes from the time in the cardiac cycle when pressure is at its lowest, while the heart is refilling with blood. This phase is called Diastole. The blood pressure in large arteries is about 120/80 mmHg. By the time this comes to the capillaries it has partly lost its pulsatile nature and has a pressure of about 35 mmHg. The pressure falls rapidly along the capillary to 15 mmHg at the venous end. This hydrostatic pressure tends to force fluid out of the capillary into the interstitium (the fluid between cells) but balance is maintained by the colloid osmotic pressure (due to protein, principally albumin) of 26 mmHg. Net water movement is small (about 2%) and thus colloid osmotic pressure is the same at the arterial and venous end of the capillary.

At the arterial end of the capillary there is a net outward force of about 11 mmHg while at the venous end the net inward force is about 9 mmHg (ie. -9). There is an imbalance between water movement out and movement back in which leads to an imbalance of about 3 litres/day, which is removed as lymph. There is some albumin in the interstitial tissue and it varies in different organs but the concentration may be up to 10 or 20% of plasma. This gives an interstitial oncotic pressure which causes movement of fluid into the interstitium. However the bulk movement of water is not the way nutrients get to cells. Nutrients diffuse down their concentration gradient as the capillary is very permeable to all small molecules.

The extracellular volume is approximately thirteen litres in a seventy kg person. Ten litres are in the interstitial space and three litres in plasma. The capillaries are the interface between the two compartments and are permeable to most substances with a molecular weight less than 20,000. Thus nutrients can readily diffuse across the wall and go from blood to cell. Despite the high permeability of the capillary water is maintained inside due to the oncotic pressure and only about 2% of the plasma flowing through the capillary moves across the wall.

The blood volume is about 5 litres of which about 3 litres are plasma and about 2 litres red blood cells. The red blood cell volume (haematocrit) is about 43% and the relationship between plasma and blood volume and haematocrit is Blood Volume = Plasma Volume 100/(100 - Ht). Most of the blood is usually in the veins (70%).

Capillaries differ in their permeability throughout the body. Brain capillaries are relatively impermeable due to tight junctions between endothelial cells lining the blood vessels. This is known as the blood brain barrier, or BBB, and helps prevent toxins from entering the brain.

In order of less permeability:

Brain < Muscle < Glomerulus < Liver sinusoids.

The capillaries, while having a large surface area, only contain about 7% of the blood volume. The arteries and arterioles contain about 15%. Most of the blood is in the veins.

## Body Fluid Distribution

The cell membrane is a bilipid layer that is permeable to water and lipid soluble particles. However, it is impermeable to charged particles. It is the osmolality controlling factor. Osmolality in the cell and interstitial fluid are the same but the anionic and cationic compositions differ. Made of albumin, the capillary membrane is permeable to everything except proteins. The membranes in different tissues differ. There are fenestrae (or pores) to promote better flow of fluids. Particles weighing over 40,000 have low permeability. It is the oncotic pressure controlling factor. Capillaries in the brain are relatively impermeable while capillaries in liver sinusoids and glomeruli are extremely permeable.

  


Water (litres)

Sodium (mmol)

Potassium (mmol)

Total

43

3700

4000

Intracellular

30

400

  


Bone

-

1500

300

Extracellular

13

1820

52

Plasma

3

420

12

Interstitial

10

1400

40

Usual Intake

1.5

180

70

Range

0.7-5

5-400

50-400

## Dehydration and Volume Depletion

Plasma osmolality is about 290 mosmol/l contributed mainly by sodium (140 mmol/l) and it's accompanying anions. In dehydration water is lost from the body. The rise in osmolality that occurs in the plasma (also sodium rises) causes water to initially move out of the cells along the osmotic gradient. Thus cell volume is initially reduced but cell homeostatic processes subsequently return it towards normal by taking up solute.

In dehydration water is removed from the plasma and thus haematocrit and albumin which have not been lost will have a higher concentration. In volume depletion water and electrolytes are both lost and thus there will be little effect on either sodium concentration or osmolality. As osmolality is not altered there will be no force to pull water out of the cells and cell volume is not affected.

In volume depletion due to blood loss the haematocrit acutely is the same but the resultant fall in blood pressure causes fluid to come out of the interstitium into the vascular compartment and albumin and haematocrit both decrease. When there is volume depletion due to electrolyte and water loss by vomiting or diarrhoea there will be little or no effect on plasma osmolality or sodium concentration. However there will be a small increase in haematocrit and plasma albumin because the volume is lost from the extracellular space and as blood cells and albumin are not lost this increases the concentration.

In volume depletion forces are activated that retain sodium and water in the body. The sodium retention works to a major extent by the renin-angiotensin-aldosterone system which is activated by a fall in blood pressure caused by volume depletion. In dehydration, the high osmolality activates ADH secretion which causes water retention. As there is also volume depletion, this activates the renin-angiotensin-aldosterone system which causes sodium to be retained. This retention would tend to cause a rise in sodium concentration which is already high but the water retention would correct this. There is no effective receptor that monitors and controls Na concentration by altering sodium excretion. Sodium retaining hormones are predominantly regulated by the volume and blood pressure. Initially in blood loss the haematocrit is not altered but falls as fluid comes in from the interstitial space.

## Water Balance

Vasopressin, also called Antidiuretic Hormone (ADH), is the principal compound controlling water balance by decreasing water output by the kidney, and thus decreased urination. It perceives the need by monitoring plasma osmolality and if this is high, vasopressin is secreted. Vasopressin is formed in the hypothalamus and travels down axons to the posterior pituitary where it is stored.

Plasma osmolality is the usual factor regulating vasopressin release but other factors alter the release. Pain and emotion release vasopressin together with the other posterior pituitary hormone oxytocin. Alcohol inhibits the release of vasopressin and thus causes a diuresis. A low plasma volume also releases vasopressin which in high concentration can cause vasoconstriction. These different factors can overcome the usual physiological control of osmolality.

Osmoreceptors in the hypothalamus monitor the plasma osmolality and send a signal down the axon that releases vasopressin from the posterior pituitary gland. Vasopressin travels by the blood to the kidney and binds to a receptor on the basolateral membrane and by a series of cellular events alters the permeability of the luminal membrane to water, thereby increasing the water permeability of the collecting duct and due to osmotic gradients created in the kidney causes water to be retained by the body (ie. an antidiuresis) which provides the other name for vasopressin of antidiuretic hormone.

Vasopressin released by the pituitary binds to a receptor on the basolateral membrane and activates adenyl cyclase which increases cyclic AMP levels in the kidney. This by a series of reactions, some of which involve calcium, cause microfilaments to contract and insert preformed water channels (aquaporins) into the luminal membrane increasing water permeability.

A high plasma osmolality is the important physiological stimulus causing vasopressin release. Urea in plasma in a normal person only has a concentration of 6 mmol/l and thus contributes to only a small part of plasma osmolality. Even if plasma urea is elevated to 30 mmol/l it would not have a significant effect on vasopressin release as membranes (including those of the osmoreceptor cells) are permeable to urea. If there is excessive ADH water is retained and the osmolality and sodium concentration would fall (hyponatraemia). If there is no ADH water is lost and osmolality and sodium concentration would rise (hypernatraemia). While ADH is released if the plasma volume falls the most important factor to restore volume is retention of sodium by the renin-angiotensin-aldosterone and other salt retaining systems.

## Sodium Balance

  


Amount

Concentration

Amount in body

3700 mmol

  


Intracellular

400 mmol

15 mmol/l

Extracellular

1800 mmol

140 mmol/l

Plasma

420 mmol

140 mmol/l

Interstitial

1400 mmol

140 mmol/l

Bone

1500 mmol

  


Amount in diet

  


Hunter Gatherer

20 mmol/day

  


Western

180 mmol/day

  


Japanese

300 mmol/day

  


Obligatory Need

< 5 mmol/day

  


Sodium is an important cation distributed primarily outside the cell. The cell sodium concentration is about 15 mmol/l but varies in different organs and with an intracellular volume of 30 litres about 400 mmol are inside the cell. The plasma and interstitial sodium is about 140 mmol/l with an extracellular volume of about 13 litres, 1800 mmol are in the extracellular space. The total body sodium, however, is about 3700 mmol as there is about 1500 mmol stored in bones.

The usual sodium intake of an Australian diet is about 180 mmol/d but varies widely (50-400 mmol/day) depending on habit and cultural influences. The body has potent sodium retaining mechanisms and even if a person is on 5 mmol Na+/day they can maintain sodium balance. Extra sodium is lost from the body by reducing the activity of the renin angiotensin aldosterone system which leads to increased sodium loss from the body. Sodium is lost through the kidney, sweat and faeces. In states of sodium depletion aldosterone levels increase and in states of sodium excess aldosterone levels decrease. The major physiological controller of aldosterone secretion is the plasma angiotensin II level which increases aldosterone secretion. A high plasma potassium also increases aldosterone secretion because besides retaining Na+ high plasma aldosterone causes K+ loss by the kidney. Plasma Na+ levels have little effect on aldosterone secretion.

A low renal perfusion pressure stimulates the release of renin, which forms angiotensin I which is converted to angiotensin II. Angiotensin II will correct the low perfusion pressure by causing constriction of blood vessels and by increasing sodium retention by a direct effect on the proximal renal tubule and by an effect operated through aldosterone. The perfusion pressure to the adrenal gland has little direct effect on aldosterone secretion and the low blood pressure operates to control aldosterone via the renin angiotensin system.

In addition to aldosterone and angiotensin II other factors influence sodium excretion. Thus in high sodium states due either to excess intake or cardiac disease (+ others) atrial peptide is secreted from the heart and by a series of actions causes loss of sodium by the kidney. Elevated blood pressure will also tend to cause Na+ loss and a low blood pressure usually leads to sodium retention. Aldosterone also acts on the sweat ducts and colonic epithelium to conserve sodium. When aldosterone has been activated to retain sodium the plasma sodium tends to rise. This immediately causes release of ADH which causes water to be retained, thus retaining Na+ and H2O in the right proportion to restore plasma volume.

## Potassium Balance

  


Amount

Concentration

Amount in body

4000 mmol

  


Intracellular

3000 + mmol

110 mmol/l

Extracellular

53 mmol

4 mmol/l

Plasma

12 mmol

4 mmol/l

Interstitial

40 mmol

4 mmol/l

Bone

300 mmol

  


Amount in diet

  


  


Hunter Gatherer

200 – 400 mmol/day

  


Western

50 – 100 mmol/day

  


Obligatory Need

30 – 50 mmol/day

  


Potassium is predominantly an intracellular ion and most of the total body potassium of about 4000 mmol is inside the cells and the next largest proportion (300-500 mmol) is in the bones. Cell K+ concentration is about 150 mmol/l but varies in different organs. Extracellular potassium is about 4.0 mmol/l and with an extracellular value of about 13 litres, 52 mmol (ie. less than 1.5%) is present here and only 12 mmol in the plasma.

In an unprocessed diet potassium is much more plentiful than sodium and is present as an organic salt while sodium is added as NaCl. In a hunter gatherer K+ intake may be as much as 400 mmol/d while in the Western diet it is 70 mmol/d or less if a person has a minimal amount of fresh fruit and vegetables. Processing of foods replaces K+ with NaCl. While the body can excrete a large K+ load it is unable to conserve K+. On a zero K+ intake or in a person with K+ depletion there will still be a loss of K+ of 30-50 mmol/d in the urine and faeces.

If there is a high potassium intake, eg. 100 mmol, this would potentially increase the extracellular K+ level 2 times before the kidney could excrete the extra potassium. The body buffers the extra potassium by equilibrating it within the cells. The acid base status controls the distribution between plasma and cells. A high pH (ie. alkalosis >7.4) favours movement of K+ into the cells whilst a low pH (ie. acidosis) causes movement out of the cell. A high plasma potassium increases aldosterone secretion and this increases the potassium loss from the body, restoring balance. This change of distribution with the acid base status means that the plasma K+ may not reflect the total body content. Thus a person with an acidosis (pH 7.1) and a plasma K+ of 6.5 mmol/l could be depleted of total body potassium. This occurs in diabetic acidosis. Conversely a person who is alkalotic with a plasma K+ of 3.4 mmol/l may have normal total body potassium.

## Calcium and Phosphate Balance

  


Amount

Concentration

Amount in body

  


  


Interstitial (0.9%)

270 mmol

9 mmol/l

Cytoplasm

<1 mmol

10-6 mmol/l

Cell organelles

270 mmol

9 mmol/l

Extracellular (0.1%)

30 mmol

2.2 mmol/l

Plasma

7 mmol

2.2 mmol/l

Interstitial

23 mmol

2.2 mmol/l

Bone (99%)

27.5 mol (1.1 kg)

  


Amount in diet

1200 mg/day

40 mmol/day

Amount absorbed

300 mg/day

10 mmol/day

Amount excreted

300 mg/day

10 mmol/day

Obligatory Need

100 mg/day

3 mmol/day

Bone => Plasma

500 mmol/day

  


Calcium is a very important electrolyte. 99% or more is deposited in bone but the remainder is importantly associated with nerve conduction, muscle contraction, hormone release and cell signalling. The plasma concentration of Ca++ is 2.2 mmol/l and phosphate 1.0 mmol/l. The solubility product of Ca and P is close to saturation in plasma. The concentration of Ca++ in the cytoplasm is < 10-6 mmol/l but the concentration of Ca++ in the cell is much higher as calcium is taken up (and is able to be released from) cell organelles.

In the Australian diet there is about 1200 mg/d of calcium. Even if it was all soluble it is not all absorbed as it combines with phosphates in the intestinal secretions. In addition absorption is regulated by active Vitamin D and increased amounts increase Ca++ absorption. Absorption is controlled by Vitamin D while excretion is controlled by parathyroid hormones. However, the distribution from bone to plasma is controlled by both the parathyroid hormones and vitamin D. There is a constant loss of calcium by the kidney even if there was none in the diet. The excretion of calcium by the kidney and its distribution between bone and the rest of the body is primarily controlled by parathyroid hormone.

Calcium in plasma exists in 3 forms. Ionized, non ionized and protein bound. It is the ionized calcium concentration that is monitored by the parathyroid gland and if low, parathyroid hormone secretion is increased. This acts to increase ionized calcium levels by increasing bone re-absorption, decreasing renal excretion and acting on the kidney to increase the rate of formation of active Vitamin D, and thereby increase gut absorption of calcium.

The usual amount of phosphate in the diet is about 1 g/d but not all is absorbed. Any excess is excreted by the kidney and this excretion is increased by parathyroid hormone. Parathyroid hormone also causes phosphate to come out of bone. Plasma phosphate has no direct effect on parathyroid hormone secretion. However if it is elevated it combines with Ca++ decreasing the ionized Ca++ in plasma, thereby increasing parathyroid hormone secretion.

## Case Study

**Heat stroke and Heat exhaustion**

If you have ever performed heavy manual labor or competed in an athletic event on a very hot day, you may have experienced symptoms of heat exhaustion. Typically these include an elevated core body temperature (above 104F or 40C), profuse sweating, pale color, muscle cramps, dizziness, and in some extreme circumstances, fainting or loss of consciousness.

Heat exhaustion occurs as a consequence of disruption of the body's own system of thermoregulation, the means by which it adjusts temperature. Sweating is the principal means through which the body cools itself down, but diverting blood from other regions toward the skin also serves this purpose. Although sweat allows excess heat to dissipate as the moisture reaches the skin surface, it can also have dangerous implications for blood pressure and volume. As sweating increases, blood volume can drop precipitously, meaning that the brain and other body systems are at risk for insufficient oxygen and nutrient supplies. Furthermore, diverting blood away from other systems and towards the skin compounds the changes in blood volume and blood pressure induced through sweating.

Heat stroke is a far more serious condition. This happens when the body's temperature rises out of control due to the failure of the thermoregulating system. If the body is unable to reduce its temperature due to outside or physical influences, the brain will start to malfunction. Delirium and loss of consciousness set in. The center of the brain controlling the sweat glands will stop functioning, halting the production of sweat. This causes the body's temperature to rise even faster. Furthermore, with the increase of the body's temperature, the metabolic process will speed up causing even more heat in the body. If left untreated this will result in death. One of the easiest ways to spot heat stroke is the skin. If it is flushed due to the increase of blood flow but dry because the sweat glands have stopped secreting, the individual will need prompt medical attention.

## Other Examples

  * Thermoregulation 
    * The skeletal muscles can shiver to produce heat if the body temperature is too low.
    * Non-shivering thermogenesis involves the decomposition of fat to produce heat.
    * Sweating cools the body with the use of evaporation.
  * Chemical regulation 
    * The pancreas produces insulin and glucagon to control blood-sugar concentration.
    * The lungs take in oxygen and give off carbon dioxide, which regulates pH in the blood.
    * The kidneys remove urea, and adjust the concentrations of water and a wide variety of ions.

Main examples of homeostasis in mammals are as follows:

  * The regulation of the amounts of water and minerals in the body. This is known as osmoregulation. This happens primarily in the kidneys.
  * The removal of metabolic waste. This is known as excretion. This is done by the excretory organs such as the kidneys and lungs.
  * The regulation of body temperature. This is mainly done by the skin.
  * The regulation of blood glucose level. This is mainly done by the liver and the insulin and glucagon secreted by the pancreas in the body.

Most of these organs are controlled by hormones secreted from the pituitary gland, which in turn is directed by the hypothalamus.

## Review Questions

Answers for these questions can be found [here](http://en.wikibooks.org/wiki/Human_Physiology/Appendix_1:_answers_to_review_questions#Homeostasis)

1\. Meaning of Homeostasis:

A) contributor and provider

B) expand

C) same or constant

D) receiver

  
2\. What is the normal pH value for body fluid?

A) 7.15-7.25

B) 7.35-7.45

C) 7.55- 7.65

D) 7.00-7.35

E) 6.5-7.5

  
3\. An example of the urinary system working with the respiratory system to regulate blood pH would be

A) When you hold your breath the kidneys will remove CO2 from your blood

B) If you exercise a lot your urine will become more acidic

C) If you have emphysema the kidneys will remove fewer bicarbonate ions from circulation

D) If you hyperventilate the kidneys will counteract the alkalinity by adding hydrogen ions into the blood stream

E) None of the above-the urinary system never works with the respiratory system

  
4\. The urge to breathe comes in direct response to:

A) How long it has been since you last took a breath

B) The oxygen concentration of your surrounding environment

C) The build-up of nitrogen within your blood stream

D) The pH of your blood

E) The build-up of blood pressure that occurs when you don't breathe

  
5\. In response to a bacterial infection my body's thermostat is raised. I start to shiver and produce more body heat. When my body temperature reaches 101 degrees, I stop shivering and my body temperature stops going up. This is an example of:

A) Negative feedback

B) A malfunctioning control system

C) Positive feedback

D) A negative impact

  
6\. Which of the following is an example of a positive feedback?

A) Shivering to warm up in a cold winter storm

B) A cruise control set on your car applies more gas when going up a hill

C) You sweat on a hot summer's day and the blood vessels in your skin vasodilate

D) You get cut and platelets form a clot. This in turn activates the fibrin clotting system and more blood forms clots

  
7\. Where is the body's "thermostat" found?

A) Within the nervous system, in the Hypothalamus

B) Within the integumentary system, in the skin

C) Within the brain, in the corpus callosum

D) Within the Urinary system, in the kidneys

  
8\. What system has little to contribute to the homeostasis of the organism?

A) Urinary System

B) Reproductive System

C) Respiratory System

D) Nervous System

  
9\. Select the _phrase(s)_ below that best describe(s) homeostasis.

A) Fluctuating within a homeostatic range

B) Maintaining a constant internal environment

C) Dynamic equilibrium

D) Deviating

### Review Answers

  * 1=C
  * 2=B
  * 3=C
  * 4=D
  * 5=A
  * 6=D
  * 7=A
  * 8=B
  * 9=B

## Glossary

  * **Control Center or Integration Center:** receives and processes information from the receptor
  * **Effector:** responds to the commands of the control center by either opposing or enhancing the stimulus
  * **Homeostasis:** refers to stability, balance or equilibrium
  * **Negative Feedback:** a reaction in which the system responds in such a way as to reverse the direction of change
  * **Positive Feedback:** a response is to amplify the change in the variable
  * **Receptor:** receives information that something in the environment is changing

  


# Cell Physiology

## Cell Structure and Function

### What is a Cell?

A cell is a structure as well as a functional unit of life-Every living thing has cells: bacteria, protozoans, fungi, plants, and animals are the main group of living things. Some organisms are made up of just one cell are called uni cellular. (e.g. bacteria and protozoans), but animals, including human beings, are multi-cellular. An adult human body is composed of about 100,000,000,000,000 cells! Each cell has basic requirements to sustain it, and the body's organ systems are largely built around providing the many trillions of cells with those basic needs (such as oxygen, food, and waste removal).

There are about 200 different kinds of specialized cells in the human body. When many identical cells are organized together it is called a tissue (such as muscle tissue, nervous tissue, etc). Various tissues organized together for a common purpose are called organs (e.g. the stomach is an organ, and so is the skin, the brain, and the uterus).

Ideas about cell structure have changed considerably over the years. Early biologists saw cells as simple membranous sacs containing fluid and a few floating particles. Today's biologists know that cells are inconceivably more complex than this. Therefore, a strong knowledge of the various cellular organelles and their functions is important to any physiologist. If a person's cells are healthy, then that person is healthy. All physiological processes, disease, growth and development can be described at the cellular level.

### Specialized Cells of the Human Body

Although there are specialized cells - both in structure and function - within the body, all cells have similarities in their structural organization and metabolic needs (such as maintaining energy levels via conversion of carbohydrate to ATP and using genes to create and maintain proteins).

Here are some of the different types of specialized cells within the human body.

  * **Nerve Cells**: Also called Neurons, these cells are in the nervous system and function to process and transmit information (it is hypothesized). They are the core components of the brain, spinal cord and peripheral nerves. They use chemical synapses that can evoke electrical signals, called action potentials, to relay signals throughout the body.
  * **Epithelial cells**: Functions of epithelial cells include secretion, absorption, protection, transcellular transport, sensation detection, and selective permeability. Epithelium lines both the outside (skin) and the inside cavities and lumen of bodies.
  * **Exocrine cells**: These cells secrete products through ducts, such as mucus, sweat, or digestive enzymes. The products of these cells go directly to the target organ through the ducts. For example, the bile from the gall bladder is carried directly into the duodenum via the bile duct.
  * **Endocrine cells**: These cells are similar to exocrine cells, but secrete their products directly into the bloodstream instead of through a duct. Endocrine cells are found throughout the body but are concentrated in hormone-secreting glands such as the pituitary. The products of the endocrine cells go throughout the body in the blood stream but act on specific organs by receptors on the cells of the target organs. For example, the hormone estrogen acts specifically on the uterus and breasts of females because there are estrogen receptors in the cells of these target organs.
  * **Blood Cells**: The most common types of blood cells are: 
    * **red blood cells (erythrocytes)**. The main function of red blood cells is to collect oxygen in the lungs and deliver it through the blood to the body tissues. Gas exchange is carried out by simple diffusion.
    * various types of **white blood cells (leukocytes)**. They are produced in the bone marrow and help the body to fight infectious disease and foreign objects in the immune system. White cells are found in the circulatory system, lymphatic system, spleen, and other body tissues.

### Cell Size

Cells are the smallest structural & functional living units within our body, but play a big role in making our body function properly. Many cells never have a large increase in size like eggs, after they are first formed from a parental cell. Typical stem cells reproduce, double in size, then reproduce again. Most Cytosolic contents such as the endomembrane system and the cytoplasm easily scale to larger sizes in larger cells. If a cell becomes too large, the normal cellular amount of DNA may not be adequate to keep the cell supplied with RNA. Large cells often replicate their chromosomes to an abnormally high amount or become multinucleated. Large cells that are primarily for nutrient storage can have a smooth surface membrane, but metabolically active large cells often have some sort of folding of the cell surface membrane in order to increase the surface area available for transport functions.

### Cellular Organization

Several different molecules interact to form organelles within our body. Each type of organelle has a specific function. Organelles perform the vital functions that keep our cells alive.

#### Cell Membranes

The boundary of the cell, sometimes called the plasma membrane, separates internal metabolic events from the external environment and controls the movement of materials into and out of the cell. This membrane is very selective about what it allows to pass through; this characteristic is referred to as "selective permeability." For example, it allows oxygen and nutrients to enter the cell while keeping toxins and waste products out. The plasma membrane is a double phospholipid membrane, or a lipid bilayer, with the nonpolar hydrophobic tails pointing toward the inside of the membrane and the polar hydrophilic heads forming the inner and outer surfaces of the membrane.

![](../../../upload.wikimedia.org/wikipedia/commons/e/ee/CellMembraneDrawing.jpg)

The molecular structure of the cell membrane.

22

#### Protein and Cholesterol

Proteins and cholesterol molecules are scattered throughout the flexible phospholipid membrane. Peripheral proteins attach loosely to the inner or outer surface of the plasma membrane. Integral proteins lie across the membrane, extending from inside to outside. A variety of proteins are scattered throughout the flexible matrix of phospholipid molecules, somewhat like icebergs floating in the ocean, and this is termed the _fluid mosaic model_ of the cell membrane.

The phospholipid bilayer is selectively permeable. Only small, uncharged polar molecules can pass freely across the membrane. Some of these molecules are H2O and CO2, hydrophobic (nonpolar) molecules like O2, and lipid soluble molecules such as hydrocarbons. Other molecules need the help of a membrane protein to get across. There are a variety of membrane proteins that serve various functions:

  * **Channel proteins**: Proteins that provide passageways through the membranes for certain hydrophilic or water-soluble substances such as polar and charged molecules. No energy is used during transport, hence this type of movement is called facilitated diffusion.
  * **Transport proteins**: Proteins that spend energy (ATP) to transfer materials across the membrane. When energy is used to provide passageway for materials, the process is called active transport.
  * **Recognition proteins**: Proteins that distinguish the identity of neighboring cells. These proteins have oligosaccharide or short polysaccharide chains extending out from their cell surface.
  * **Adhesion proteins**: Proteins that attach cells to neighboring cells or provide anchors for the internal filaments and tubules that give stability to the cell.
  * **Receptor proteins**: Proteins that initiate specific cell responses once hormones or other trigger molecules bind to them.
  * **Electron transfer proteins**: Proteins that are involved in moving electrons from one molecule to another during chemical reactions.

#### Passive Transport Across the Cell Membrane

**Passive transport** describes the movement of substances down a concentration gradient and does not require energy use.

  * **Bulk flow** is the collective movement of substances in the same direction in response to a force, such as pressure. Blood moving through a vessel is an example of bulk flow.
  * **Simple diffusion**, or diffusion, is the net movement of substances from an area of higher concentration to an area of lower concentration. This movement occurs as a result of the random and constant motion characteristic of all molecules, (atoms or ions) and is independent from the motion of other molecules. Since, at any one time, some molecules may be moving against the gradient and some molecules may be moving down the gradient, although the motion is random, the word "net" is used to indicate the overall, eventual end result of the movement.
  * **Facilitated diffusion** is the diffusion of solutes through channel proteins in the plasma membrane. Water can pass freely through the plasma membrane without the aid of specialized proteins (though facilitated by aquaporins).
  * **Osmosis** is the diffusion of water molecules across a selectively permeable membrane. When water moves into a body by osmosis, hydrostatic pressure or osmotic pressure may build up inside the body.
  * **Dialysis** is the diffusion of solutes across a selectively permeable membrane.

#### Active Transport Across the Cell Membrane

**Active transport** is the movement of solutes against a gradient and requires the expenditure of energy, usually in the form of ATP. Active transport is achieved through one of these two mechanisms:

##### Protein Pumps

  * Transport proteins in the plasma membrane transfer solutes such as small ions (Na+, K+, Cl-, H+), amino acids, and monosaccharides.
  * The proteins involved with active transport are also known as **ion pumps**.
  * The protein binds to a molecule of the substance to be transported on one side of the membrane, then it uses the released energy (ATP) to change its shape, and releases it on the other side.
  * The protein pumps are specific, there is a different pump for each molecule to be transported.
  * Protein pumps are catalysts in the splitting of ATP → ADP + phosphate, so they are called **ATPase enzymes**. 
    * The sodium-potassium pump (also called the Na+/K+-ATPase enzyme) actively moves sodium out of the cell and potassium into the cell. These pumps are found in the membrane of virtually every cell, and are essential in transmission of nerve impulses and in muscular contractions.

**Cystic fibrosis** is a genetic disorder that results in a mutated chloride ion channel. By not regulating chloride secretion properly, water flow across the airway surface is reduced and the mucus becomes dehydrated and thick.

##### Vesicular Transport

  * Vesicles or other bodies in the cytoplasm move macromolecules or large particles across the plasma membrane. Types of **vesicular** **transport** include:
  1. **Exocytosis**, which describes the process of vesicles fusing with the plasma membrane and releasing their contents to the outside of the cell. This process is common when a cell produces substances for export.
  2. **Endocytosis**, which describes the capture of a substance outside the cell when the plasma membrane merges to engulf it. The substance subsequently enters the cytoplasm enclosed in a vesicle.

    There are three kinds of endocytosis: 

  * **Phagocytosis** or cellular eating, occurs when the dissolved materials enter the cell. The plasma membrane engulfs the solid material, forming a phagocytic vesicle.
  * **Pinocytosis** or cellular drinking occurs when the plasma membrane folds inward to form a channel allowing dissolved substances to enter the cell. When the channel is closed, the liquid is encircled within a pinocytic vesicle.
  * **Receptor-mediated endocytosis** occurs when specific molecules in the fluid surrounding the cell bind to specialized receptors in the plasma membrane. As in pinocytosis, the plasma membrane folds inward and the formation of a vesicle follows.

    **Note:** Certain hormones are able to target specific cells by receptor-mediated endocytosis.

## Parts of the Cell

![Animal cell structure en.svg](../../../upload.wikimedia.org/wikipedia/commons/thumb/4/48/Animal_cell_structure_en.svg/724px-Animal_cell_structure_en.svg.png)

### Cytoplasm

The gel-like material within the cell membrane is referred to as the cytoplasm. It is a fluid matrix, the cytosol, which consists of 80% to 90% water, salts, organic molecules and many enzymes that catalyze reactions, along with dissolved substances such as proteins and nutrients. The cytoplasm plays an important role in a cell, serving as a "molecular soup" in which organelles are suspended and held together by a fatty membrane.

Within the plasma membrane of a cell, the cytoplasm surrounds the nuclear envelope and the cytoplasmic organelles. It plays a mechanical role by moving around inside the membrane and pushing against the cell membrane helping to maintain the shape and consistency of the cell and again, to provide suspension to the organelles. It is also a storage space for chemical substances indispensable to life, which are involved in vital metabolic reactions, such as anaerobic glycolysis and protein synthesis.

The cell membrane keeps the cytoplasm from leaking out. It contains many different organelles which are considered the insoluble constituents of the cytoplasm, such as the mitochondria, lysosomes, peroxysomes, ribosomes, several vacuoles and cytoskeletons, as well as complex cell membrane structures such as the endoplasmic reticulum and the Golgi apparatus that each have specific functions within the cell.

  * **Cytoskeleton**

Threadlike proteins that make up the cytoskeleton continually reconstruct to adapt to the cell's constantly changing needs. It helps cells maintain their shape and allows cells and their contents to move. The cytoskeleton allows certain cells such as neutrophils and macrophages to make amoeboid movements.

The network is composed of three elements: microtubules, actin filaments, and intermediate fibers.

  


  * **Microtubules**

Microtubules function as the framework along which organelles and vesicles move within a cell. They are the thickest of the cytoskeleton structures. They are long hollow cylinders, composed of protein subunits, called tubulin. Microtubules form mitotic spindles, the machinery that partitions chromosomes between two cells in the process of cell division. Without mitotic spindles cells could not reproduce.

Microtubules, intermediate filaments, and microfilaments are three protein fibers of decreasing diameter, respectively. All are involved in establishing the shape or movements of the cytoskeleton, the internal structure of the cell.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/e/e9/MEF_microfilaments.jpg/220px-MEF_microfilaments.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

A photograph of microfilaments.

  * **Microfilaments**

Microfilaments provide mechanical support for the cell, determine the cell shape, and in some cases enable cell movements. They have an arrow-like appearance, with a fast growing plus or barbed end and a slow growing minus or pointed end. They are made of the protein actin and are involved in cell motility. They are found in almost every cell, but are predominant in muscle cells and in the cells that move by changing shape, such as phagocytes (white blood cells that scour the body for bacteria and other foreign invaders).

### Organelles

Organelles are bodies embedded in the cytoplasm that serve to physically separate the various metabolic activities that occur within cells. The organelles are each like separate little factories, each organelle is responsible for producing a certain product that is used elsewhere in the cell or body.

Cells of all living things are divided into two broad categories: prokaryotes and eukaryotes. Bacteria (and archea) are prokaryotes, which means they lack a nucleus or other membrane-bound organelles. Eukaryotes include all protozoans, fungi, plants, and animals (including humans), and these cells are characterized by a nucleus (which houses the chromosomes) as well as a variety of other organelles. Human cells vary considerably (consider the differences between a bone cell, a blood cell, and a nerve cell), but most cells have the features described below.

![](../../../upload.wikimedia.org/wikipedia/commons/d/d4/Celltypes.png)

A comparison of Eukaryote and Prokaryote cells.

#### Nucleus

Controls the cell; houses the genetic material (DNA). The nucleus is the largest of the cells organelles. Cells can have more than one nucleus or lack a nucleus all together. Skeletal muscle cells contain more than one nucleus whereas red blood cells do not contain a nucleus at all. The nucleus is bounded by the nuclear envelope, a phospholipid bilayer similar to the plasma membrane. The space between these two layers is the nucleolemma Cisterna.

The nucleus contains the DNA, as mentioned above, the hereditary information in the cell. Normally the DNA is spread out within the nucleus as a threadlike matrix called chromatin. When the cell begins to divide, the chromatin condenses into rod-shaped bodies called chromosomes, each of which, before dividing, is made up of two long DNA molecules and various histone molecules. The histones serve to organize the lengthy DNA, coiling it into bundles called nucleosomes. Also visible within the nucleus are one or more nucleoli, each consisting of DNA in the process of manufacturing the components of ribosomes. Ribosomes are shipped to the cytoplasm where they assemble amino acids into proteins. The nucleus also serves as the site for the separation of the chromosomes during cell division.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/3/38/Diagram_human_cell_nucleus.svg/220px-Diagram_human_cell_nucleus.svg.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

A cross-sectional diagram of a cell.

  * **Chromosomes**

![](../../../upload.wikimedia.org/wikipedia/commons/5/54/Chromosome.png)

A rough sketch of a chromosome.

Inside each cell nucleus are chromosomes. Chromosomes are made up of chromatin, which is made up of protein and deoxyribonucleic acid strands. Deoxyribonucleic acid is DNA, the genetic material that is in the shape of a twisted ladder, also called the double helix. Humans have 23 pairs of chromosomes. Down Syndrome and Cri du Chat Syndrome result from having an abnormal number of chromosomes.

#### Centrioles

Centrioles are rod like structures composed of 9 bundles which contain three microtubules each. Two perpendicularly placed centrioles surrounded by proteins make up the centrosome. Centrioles are very important in cellular division, where they arrange the mitotic spindles that pull the chromosome apart.

Centrioles and basal bodies act as microtubule organizing centers. A pair of centrioles (enclosed in a centrosome) located outside the nuclear envelope gives rise to the microtubules that make up the spindle apparatus used during cell division. Basal bodies are at the base of each flagellum and cilium and appear to organize their development.

#### Ribosomes

![Ribosome structure.png](../../../upload.wikimedia.org/wikipedia/commons/8/82/Ribosome_structure.png)

Ribosomes play an active role in the complex process of protein synthesis, where they serve as the structures that facilitate the joining of amino acids. Each ribosome is composed of a large and small subunit which are made up of ribosomal proteins and ribosomal RNAs. They can either be found in groups called polyribosomes within the cytoplasm or found alone. Occasionally they are attached to the endoplasmic reticulum.

  


![](../../../upload.wikimedia.org/wikipedia/commons/thumb/7/75/Diagram_of_a_human_mitochondrion.png/220px-Diagram_of_a_human_mitochondrion.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

A cutaway view inside a mitochondria.

#### Mitochondria

Mitochondria are the organelles that function as the cell "powerhouse", generating ATP, the universal form of energy used by all cells. It converts food nutrients such as glucose, to a fuel (ATP) that the cells of the body can use. Mitochondria are tiny sac-like structures found near the nucleus. Little shelves called cristae are formed from folds in the inner membrane. Cells that are metabolically active such as muscle, liver and kidney cells have high energy requirements and therefore have more mitochondria.

Mitochondria are unique in that they have their own mitochondrial DNA (separate from the DNA that is in the nucleus). It is believed that eukaryotes evolved from one cell living inside another cell, and mitochondria share many traits with free-living bacteria (similar chromosome, similar ribosomes, etc).

#### Endoplasmic Reticulum

    **Endoplasmic** means "within the plasm" and **reticulum** means "network".

A complex three dimensional internal membrane system of flattened sheets, sacs and tubes, that play an important role in making proteins and shuttling cellular products; also involved in metabolisms of fats, and the production of various materials. In cross-section, they appear as a series of maze-like channels, often closely associated with the nucleus. When ribosomes are present, the rough ER connects polysaccharide groups to the polypeptides as they are assembled by the ribosomes. Smooth ER, without ribosomes, is responsible for various activities, including the synthesis of lipids and hormones, especially in cells that produce these substances for export from the cell.

Rough endoplasmic reticulum has characteristic bumpy appearance due to the multitude of ribosomes coating it. It is the site where proteins not destined for the cytoplasm are synthesized.

Smooth endoplasmic reticulum provides a variety of functions, including lipid synthesis and degradation, and calcium ion storage. In liver cells, the smooth ER is involved in the breakdown of toxins, drugs, and toxic byproducts from cellular reactions.

#### Golgi Apparatus

"Packages" cellular products in sacs called vesicles so that the products can cross the cell membrane and exit the cell. The **Golgi apparatus** is the central delivery system for the cell. It is a group of flattened sacs arranged much like a stack of bowls. They function to modify and package proteins and lipids into vesicles, small spherically shaped sacs that bud from the ends of a Golgi apparatus. Vesicles often migrate to and merge with the plasma membrane, releasing their contents outside the cell. The Golgi apparatus also transports lipids and creates lysosomes and organelles involved in digestion.

#### Vacuoles

Spaces in the cytoplasm that sometimes serve to carry materials to the cell membrane for discharge to the outside of the cell. **Vacuoles** are formed during endocytosis when portions of the cell membrane are pinched off.

#### Lysosomes

Lysosomes are sac-like compartments that contain a number of powerful degradative enzymes. They are built in the Golgi apparatus. They break down harmful cell products and waste materials, cellular debris, and foreign invaders such as bacteria, and then force them out of the cell. Tay-Sachs disease and Pompe's disease are just two of the malfunctions of lysosomes or their digestive proteins.

#### Peroxisomes

Organelles in which oxygen is used to oxidize substances, breaking down lipids and detoxifying certain chemicals. Peroxisomes self replicate by enlarging and then dividing. They are common in liver and kidney cells that break down potentially harmful substances. Peroxisomes can convert hydrogen peroxide, a toxin made of H2O2 to H2O.

### Extracellular structures

  * **Extracellular matrix**

Human cells, like other animal cells, do not have a rigid cell wall. Human cells do have an important and variable structure outside of their cell membrane called the extracellular matrix. Sometimes this matrix can be extensive and solid (examples = calcified bone matrix, cartilage matrix), while other times it consists of a layer of extracellular proteins and carbohydrates. This matrix is responsible for cells binding to each other and is incredibly important in how cells physically and physiologically interact with each other.

  * **Flagella**

Many prokaryotes have flagella, allowing, for example, an _E. coli_ bacteria to propel its way up the urethra to cause a UTI (Urinary Tract Infection). Human cells, however (and in fact most eukaryotic cells) lack flagella. This makes sense since humans are multicellular, and individual cells do not need to swim around. The obvious exception to this is with sperm, and indeed each sperm is propelled by a single flagellum. The flagellum of sperm is composed of microtubules.

  * **Cilia**

Cilia are especially notable on the single-celled protozoans, where they beat in synchrony to move the cells nimbly through the water. They are composed of extensions of the cell membrane that contain microtubules. When present in humans they are typically found in large numbers on a single surface of the cells, where rather than moving cells, they move materials. The _mucociliary escalator_ of the respiratory system consists of mucus-secreting cells lining the trachea and bronchi, and ciliated epithelial cells that move the mucus ever-upward. In this manner mold spores, bacteria, and debris are caught in the mucus, removed from the trachea, and pushed into the esophagus (to be swallowed into a pit of acid). In the oviducts cilia move the ovum from the ovary to the uterus, a journey which takes a few days.

![](../../../upload.wikimedia.org/wikipedia/commons/5/50/Cilia.jpg)

A magnified view of several cells, with visible cilia.

## Cell Junctions

The plasma membranes of adjacent cells are usually separated by extracellular fluids that allow transport of nutrients and wastes to and from the bloodstream. In certain tissues, however, the membranes of adjacent cells may join and form a junction. Three kinds of cell junctions are recognized:

  * **Desmosomes** are protein attachments between adjacent cells. Inside the plasma membrane, a desmosome bears a disk shaped structure from which protein fibers extend into the cytoplasm. Desmosomes act like spot welds to hold together tissues that undergo considerable stress, such as our skin or heart muscle.
  * **Tight junctions** are tightly stitched seams between cells. The junction completely encircles each cell, preventing the movement of material between the cell. Tight junctions are characteristic of cells lining the digestive tract, where materials are required to pass through cells,rather than intercellular spaces, to penetrate the bloodstream.
  * **Gap junctions** are narrow tunnels that directly connect the cytoplasm of two neighbouring cells, consisting of proteins called connexons. These proteins allow only the passage of ions and small molecules. In this manner, gap junctions allow communication between cells through the exchange of materials or the transmission of electrical impulses.

  


## Cell Metabolism

**Cell metabolism** is the total energy released and consumed by a cell. Metabolism describes all of the chemical reactions that are happening in the body. Some reactions, called anabolic reactions, create needed products. Other reactions, called catabolic reactions, break down products. Your body is performing both anabolic and catabolic reactions at the same time and around the clock, twenty four hours a day, to keep your body alive and functioning. Even while you sleep, your cells are busy metabolizing.

  * **Catabolism**: The energy releasing process in which a chemical or food is used (broken down) by degradation or decomposition, into smaller pieces.
  * **Anabolism**: Anabolism is just the opposite of catabolism. In this portion of metabolism, the cell consumes energy to produce larger molecules via smaller ones.

### Energy Rich Molecules

#### Adenosine Triphosphate (ATP)

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/2/2d/Adenosintriphosphat.svg/220px-Adenosintriphosphat.svg.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Chemical diagram of an ATP molecule.

ATP is the currency of the cell. When the cell needs to use energy such as when it needs to move substances across the cell membrane via the active transport system, it "pays" with molecules of ATP. The total quantity of ATP in the human body at any one time is about 0.1 Mole. The energy used by human cells requires the hydrolysis of 200 to 300 moles of ATP daily. This means that each ATP molecule is recycled 2000 to 3000 times during a single day. ATP cannot be stored, hence its consumption must closely follow its synthesis. On a per-hour basis, 1 kilogram of ATP is created, processed and then recycled in the body. Looking at it another way, a single cell uses about 10 million ATP molecules per second to meet its metabolic needs, and recycles all of its ATP molecules about every 20-30 seconds.

#### Flavin Adenine Dinucleotide (FAD)

When two hydrogen atoms are bonded, FAD is reduced to FADH2 and is turned into an energy-carrying molecule. FAD accommodates two equivalents of Hydrogen; both the hydride and the proton ions. This is used by organisms to carry out energy requiring processes. FAD is reduced in the citric acid cycle during aerobic respiration

#### Nicotinamide Adenine Dinucleotide (NADH)

Nicotinamide adenine dinucleotide (NAD+) and nicotinamide adenine dinucleotide phosphate (NADP) are two important cofactors found in cells. NADH is the reduced form of NAD+, and NAD+ is the oxidized form of NADH. It forms NADP with the addition of a phosphate group to the 2' position of the adenosyl nucleotide through an ester linkage.

NAD is used extensively in glycolysis and the citric acid cycle of cellular respiration. The reducing potential stored in NADH can be converted to ATP through the electron transport chain or used for anabolic metabolism. ATP "energy" is necessary for an organism to live. Green plants obtain ATP through photosynthesis, while other organisms obtain it by cellular respiration.

NADP is used in anabolic reactions, such as fat acid and nucleic acid synthesis, that require NADPH as a reducing agent. In chloroplasts, NADP is an oxidising agent important in the preliminary reactions of photosynthesis. The NADPH produced by photosynthesis is then used as reducing power for the biosynthetic reactions in the Calvin cycle of photosynthesis.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/c/c6/NADH.GIF/220px-NADH.GIF)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Chemical diagram of an NADH molecule.

MH2 \+ NAD+ → NADH + H+ \+ M: + energy, where M is a metabolite. Two hydrogen ions (a hydride ion and an H+ ion) are transferred from the metabolite. One electron is transferred to the positively-charged nitrogen, and one hydrogen attaches to the carbon atom opposite to the nitrogen.

The human body synthesizes NAD from the vitamin niacin in the form of nicotinic acid or nicotinamide.

#### Cellular Respiration

Cellular respiration is the energy releasing process by which sugar molecules are broken down by a series of reactions and the chemical energy gets converted to energy stored in ATP molecules. The reactions that convert the fuel (glucose) to usable cellular energy (ATP) are glycolysis, the Krebs cycle (sometimes called the citric acid cycle), and the electron transport chain. Altogether these reactions are referred to as "cellular respiration" or "aerobic respiration." Oxygen is needed as the final electron acceptor, and carrying out cellular respiration is the very reason we breathe and the reason we eat.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/0/01/Cellular_respiration_flowchart_\(en\).svg/248px-Cellular_respiration_flowchart_\(en\).svg.png)

Flowchart of cellular respiration.

### Glycolysis

The glycolytic pathway (glycolysis) is where glucose, the smallest molecule that a carbohydrate can be broken into during digestion, gets oxidized and broken into two 3-carbon molecules (pyruvates), which are then fed into the Kreb's Cycle. Glycolysis is the beginning of cellular respiration and takes place in the cytoplasm. Two molecules of ATP are required for glycolysis, but four are produced so there is a net gain of two ATP per glucose molecule. Two NADH molecules transfer electrons (in the form of hydrogen ions) to the electron transport chain in the mitochondria, where they will be used to generate additional ATP. During physical exertion when the mitochondria are already producing the maximum ATP possible with the amount of oxygen available, glycolysis can continue to produce an additional 2 ATP per glucose molecule without sending the electrons to the mitochondria. However, during this _anaerobic respiration_ lactic acid is produced, which may accumulate and lead to temporary muscle cramping.

### Krebs Cycle

The Krebs cycle was named after [Sir Hans Krebs](http://en.wikipedia.org/wiki/Hans_Adolf_Krebs) (1900-1981), who proposed the key elements of this pathway in 1937 and was awarded the Nobel Prize in Medicine for its discovery in 1953.

Two molecules of pyruvate enter the Krebs cycle, which is called the aerobic pathway because it requires the presence of oxygen in order to occur. This cycle is a major biological pathway that occurs in humans and every plant and animal.

After glycolysis takes place in the cell's cytoplasm, the pyruvic acid molecules travel into the interior of the mitochondrion. Once the pyruvic acid is inside, carbon dioxide is enzymatically removed from each three-carbon pyruvic acid molecule to form acetic acid. The enzyme then combines the acetic acid with an enzyme, coenzyme A, to produce acetyl coenzyme A, also known as acetyl CoA.

Once acetyl CoA is formed, the Krebs cycle begins. The cycle is split into eight steps, each of which will be explained below.

  * Step 1: The acetic acid subunit of acetyl CoA is combined with oxaloacetate to form a molecule of citrate. The acetyl coenzyme A acts only as a transporter of acetic acid from one enzyme to another. After Step 1, the coenzyme is released by hydrolysis so that it may combine with another acetic acid molecule to begin the Krebs cycle again.
  * Step 2: The citric acid molecule undergoes an isomerization. A hydroxyl group and a hydrogen molecule are removed from the citrate structure in the form of water. The two carbons form a double bond until the water molecule is added back. Only now, the hydroxyl group and hydrogen molecule are reversed with respect to the original structure of the citrate molecule. Thus, isocitrate is formed.
  * Step 3: In this step, the isocitrate molecule is oxidized by a NAD molecule. The NAD molecule is reduced by the hydrogen atom and the hydroxyl group. The NAD binds with a hydrogen atom and carries off the other hydrogen atom leaving a carbonyl group. This structure is very unstable, so a molecule of CO2 is released creating alpha-ketoglutarate.
  * Step 4: In this step, our friend, coenzyme A, returns to oxidize the alpha-ketoglutarate molecule. A molecule of NAD is reduced again to form NADH and leaves with another hydrogen. This instability causes a carbonyl group to be released as carbon dioxide and a thioester bond is formed in its place between the former alpha-ketoglutarate and coenzyme A to create a molecule of succinyl-coenzyme A complex.
  * Step 5: A water molecule sheds its hydrogen atoms to coenzyme A. Then, a free-floating phosphate group displaces coenzyme A and forms a bond with the succinyl complex. The phosphate is then transferred to a molecule of GDP to produce an energy molecule of GTP. It leaves behind a molecule of succinate.
  * Step 6: In this step, succinate is oxidized by a molecule of FAD (Flavin adenine dinucleotide). The FAD removes two hydrogen atoms from the succinate and forces a double bond to form between the two carbon atoms, thus creating fumarate.
  * Step 7: An enzyme adds water to the fumarate molecule to form malate. The malate is created by adding one hydrogen atom to a carbon atom and then adding a hydroxyl group to a carbon next to a terminal carbonyl group.
  * Step 8: In this final step, the malate molecule is oxidized by a NAD molecule. The carbon that carried the hydroxyl group is now converted into a carbonyl group. The end product is oxaloacetate which can then combine with acetyl-coenzyme A and begin the Krebs cycle all over again.
  * Summary: In summary, three major events occur during the Krebs cycle. One GTP (guanosine triphosphate) is produced which eventually donates a phosphate group to ADP to form one ATP; three molecules of NAD are reduced; and one molecule of FAD is reduced. Although one molecule of GTP leads to the production of one ATP, the production of the reduced NAD and FAD are far more significant in the cell's energy-generating process. This is because NADH and FADH2 donate their electrons to an electron transport system that generates large amounts of energy by forming many molecules of ATP.

To see a visual summary of "Kreb Cycle" please [click here](http://homepage.smc.edu/hodson_kent/Energetics/Krebs2.htm).

### Electron Transport System

The most complicated system of all. In the respiration chain, oxidation and reduction reactions occur repeatedly as a way of transporting energy. The respiratory chain is also called the electron transport chain. At the end of the chain, oxygen accepts the electron and water is produced.

#### Redox Reaction

This is a simultaneous oxidation-reduction process whereby cellular metabolism occurs, such as the oxidation of sugar in the human body, through a series of very complex electron transfer processes.

The chemical way to look at redox processes is that the substance being oxidized transfers electrons to the substance being reduced. Thus, in the reaction, the substance being oxidized (aka. the reducing agent) loses electrons, while the substance being reduced (aka. the oxidizing agent) gains electrons. Remember: LEO (Losing Electrons is Oxidation) the lion says GER (Gaining Electrons is Reduction); or alternatively: OIL (Oxidation is Loss) RIG (Reduction is Gain).

The term redox state is often used to describe the balance of NAD+/NADH and NADP+/NADPH in a biological system such as a cell or organ. The redox state is reflected in the balance of several sets of metabolites (e.g., lactate and pyruvate, β-hydroxybutyrate and acetoacetate) whose interconversion is dependent on these ratios. An abnormal redox state can develop in a variety of deleterious situations, such as hypoxia, shock, and sepsis.

## Cell Building Blocks

What major classes of molecules are found within cells?

### Lipids

The term is more-specifically used to refer to fatty-acids and their derivatives (including tri-, di-, and mono-glycerides and phospholipids) as well as other fat-soluble sterol-containing metabolites such as cholesterol. Lipids serve many functions in living organisms including energy storage, serve as structural components of cell membranes, and constitute important signaling molecules. Although the term lipid is sometimes used as a synonym for fat, the latter is in fact a subgroup of lipids called triglycerides and should not be confused with the term fatty acid.

### Carbohydrates

Carbohydrate molecules consist of carbon, hydrogen, and oxygen. They have a general formula Cn(H2O)n. There are several sub-families based on molecular size.

Carbohydrates are chemical compounds that contain oxygen, hydrogen, and carbon atoms, and no other elements. They consist of monosaccharide sugars of varying chain lengths.

Certain carbohydrates are an important storage and transport form of energy in most organisms, including plants and animals. Carbohydrates are classified by their number of sugar units: monosaccharides (such as glucose and fructose), disaccharides (such as sucrose and lactose), oligosaccharides, and polysaccharides (such as starch, glycogen, and cellulose).

The simplest carbohydrates are monosaccharides, which are small straight-chain aldehydes and ketones with many hydroxyl groups added, usually one on each carbon except the functional group. Other carbohydrates are composed of monosaccharide units and break down under hydrolysis. These may be classified as disaccharides, oligosaccharides, or polysaccharides, depending on whether they have two, several, or many monosaccharide units.

![Protein-structure.png](../../../upload.wikimedia.org/wikipedia/commons/a/a6/Protein-structure.png)

### Proteins

All proteins contain carbon, hydrogen, oxygen and nitrogen. Some also contain phosphorus and sulfur. The building blocks of proteins are amino acids. There are 20 different kinds of amino acids used by the human body. They unite by peptide bonds to form long molecules called polypeptides. Polypeptides are assembled into proteins. Proteins have four levels of structure

  * **Primary**

Primary structure is the sequence of amino acids bonded in the polypeptide.

  * **Secondary**

The secondary structure is formed by hydrogen bonds between amino acids. The polypeptide can coil into a helix or form a pleated sheet.

  * **Tertiary**

The tertiary structure refers to the three-dimensional folding of the helix or pleated sheet.

  * **Quaternary**

The quaternary structure refers to the spatial relationship among the polypeptide in the protein.

  * **Hexagonary**

The hexagonary structure refers to the carpal relationship among the bipeptide in the person.

### Enzymes

A biological molecule that catalyzes a chemical reaction. Enzymes are essential for life because most chemical reactions in living cells would occur too slowly or would lead to different products without enzymes. Most enzymes are proteins and the word "enzyme" is often used to mean a protein enzyme. Some RNA molecules also have a catalytic activity, and to differentiate them from protein enzymes, they are referred to as RNA enzymes or ribozymes.

## Review Questions

Answers for these questions can be found [here](http://en.wikibooks.org/wiki/Human_Physiology/Appendix_1:_answers_to_review_questions#Cell_physiology)

1\. List 2 functions of the cell membrane:

Questions 2 - 6 Match the following organelles with their function: 2. Mitochondria 3. Vacuoles 4. Cilia 5. Smooth ER 6. Golgi Apparatus

    A. Movement of the cell
    B. Lipid synthesis and transport
    C. "Powerhouse" of the cell, makes ATP
    D. Storage areas, mainly found in plant cells
    E. Packages and distributes cellular products

7\. The diffusion of H2O across a semi permeable or selectively permeable membrane is termed

    A. Active transport
    B. Diffusion
    C. Osmosis
    D. Endocytosis

8\. Oxygen enters a cell via?

    a. Diffusion
    b. Filtration
    c. Osmosis
    d. Active transport

9\. The term used to describe, "cell eating" is?

    a. Exocytosis
    b. Phagocytosis
    c. Pinocytosis
    d. Diffusion

10\. Which of the following requires energy?

    a. Diffusion
    b. Osmosis
    c. Active transport
    d. Facilitated diffusion

11\. Protein synthesis occurs at the

    a. Mitochondria
    b. Lysosomes
    c. Within the nucleus
    d. Ribosomes

12\. Which of the following is not found in the cell membrane?

    a. Cholesterol
    b. Phospholipids
    c. Proteins
    d. Galactose
    e. Nucleic acids

13\. What is a cell?

    a. The largest living units within our bodies.
    b. Enzymes that "eat" bacteria
    c. Microscopic fundamental units of all living things.
    d. All of the above.

## Glossary

**Active Transport**: the movement of solutes against a gradient and requires the expenditure of energy

**Adenosine Triphosphate (ATP):** a cell’s source of energy

**Bulk Flow:** the collective movement of substances in the same direction in response to a force

**Cells:** the microscopic fundamental unit that makes up all living things

**Cell Membrane:** boundary of the cell, sometimes called the plasma membrane

**Cytoplasm:** a water-like substance that fills cells. The cytoplasm consists of cytosol and the cellular organelles, except the cell nucleus. The cytosol is made up of water, salts, organic molecules and many enzymes that catalyze reactions. The cytoplasm holds all of the cellular organelles outside of the nucleus, maintains the shape and consistency of the cell, and serves as a storage place for chemical substances.

**Cytoskeleton:** made of threadlike proteins, helps cells maintain their shape and allows cells and their contents to move

**Dialysis:** the diffusion of solutes across a selectively permeable membrane. Most commonly heard of when a patient has had renal failure. In medicine, dialysis is a type of renal replacement therapy which is used to provide an artificial replacement for lost kidney function due to renal failure. It is a life support treatment and does not treat any kidney diseases.

**Endocrine cells:** similar to exocrine cells, but secrete their products directly into the bloodstream instead of through a duct

**Endocytosis:** the capture of a substance outside the cell when the plasma membrane merges to engulf it

**Endoplasmic Reticulum:** organelle that play an important role in making proteins and shuttling cellular products; also involved in metabolisms of fats, and the production of various materials

**Epithelial Cells:** cells that aid in secretion, absorption, protection, trans-cellular transport, sensation detection, and selective permeability

**Exocrine Cells:** cells that secrete products through ducts, such as mucus, sweat, or digestive enzymes

**Exocytosis:** the process of vesicles fusing with the plasma membrane and releasing their contents to the outside of the cell

**Facilitated Diffusion:** the diffusion of solutes through channel proteins in the plasma membrane

**Golgi Apparatus:** "packages" cellular products in sacs called vesicles so that the products can cross the cell membrane and exit the cell

**Glycolysis:** process in which sugars (glucose) are converted to acid

**Lysosomes:** sac-like compartments that contain a number of powerful degradative enzymes

**Microfilaments:** provide mechanical support for the cell, determine the cell shape, and in some cases enable cell movements

**Microtubules:** function as the framework along which organelles and vesicles move within a cell

**Mitochondria:** the organelles that function as the cell "powerhouse", generating ATP

**Nucleus:** controls the cell; houses the genetic material

**Organelles:** bodies embedded in the cytoplasm that serve to physically separate the various metabolic activities that occur within cells

**Osmosis:** the diffusion of water molecules across a selectively permeable membrane from an area of high solute concentration to an area of low solute concentration.

**Passive Transport:** the movement of substances down a concentration gradient and does not require energy use

**Peroxisomes:** organelles in which oxygen is used to oxidize substances, breaking down lipids and detoxifying certain chemicals

**Phagocytosis:** a form of endocytosis wherein large particles are enveloped by the cell membrane of a (usually larger) cell and internalized to form a phagosome, or "food vacuole." In animals, phagocytosis is performed by specialized cells called phagocytes, which serve to remove foreign bodies and thus fight infection. In vertebrates, these include larger macrophages and smaller granulocytes, types of blood cells. Bacteria, dead tissue cells, and small mineral particles are all examples of objects that may be phagocytosed.

**Pinocytosis:** also called cellular drinking, is a form of endocytosis, a process in which small particles are taken in by a cell by splitting into smaller particles. The particles then form small vesicles which subsequently fuse with lysosomes to hydrolyze, or to break down, the particles. This process requires adenosine triphosphate (ATP).

**Receptor-mediated Endocytosis:** occurs when specific molecules in the fluid surrounding the cell bind to specialized receptors in the plasma membrane

**Red Blood Cells (erythrocytes):** cells that collect oxygen in the lungs and deliver it through the blood to the body tissues

**Ribosomes:** play an active role in the complex process of protein synthesis, where they serve as the structures that facilitate the joining of amino acids

**Simple Diffusion:** the net movement of substances from an area of higher concentration to an area of lower concentration

**Vacuoles:** spaces in the cytoplasm that sometimes serve to carry materials to the cell membrane for discharge to the outside of the cell

**White Blood Cells (leukocytes):** produced in the bone marrow and help the body to fight infectious disease and foreign objects in the immune system  


# The Integumentary System

## Introduction

The **integumentary system** consists of the skin, hair, nails, the subcutaneous tissue below the skin,and assorted glands.The most obvious function of the integumentary system is the protection that the skin gives to underlying tissues. The skin not only keeps most harmful substances out, but also prevents the loss of fluids.

A major function of the subcutaneous tissue is to connect the skin to underlying tissues such as muscles. Hair on the scalp provides insulation from cold for the head. The hair of eyelashes and eyebrows helps keep dust and perspiration out of the eyes, and the hair in our nostrils helps keep dust out of the nasal cavities. Any other hair on our bodies no longer serves a function, but is an evolutionary remnant. Nails protect the tips of fingers and toes from mechanical injury. Fingernails give the fingers greater ability to pick up small objects.

There are four types of glands in the integumentary system: Sudoriferous glands, Sebaceous glands, Ceruminous glands, and Mammary glands. Sudoriferous glands are sweat producing glands. These are important to help maintain body temperature. Sebaceous glands are oil producing glands which help inhibit bacteria, keep us waterproof and prevent our hair and skin from drying out. Ceruminous glands produce earwax which keeps the outer surface of the eardrum pliable and prevents drying. Mammary glands produce milk.

## Skin

In zoology and dermatology, skin is an organ of the integumentary system made up of a layer of tissues that guard underlying muscles and organs. As the interface with the surroundings, it plays the most important role in protecting against pathogens. Its other main functions are insulation and temperature regulation, sensation and vitamin D and B synthesis. Skin is considered one of the most important parts of the body.

Skin has pigmentation, melanin, provided by melanocytes, which absorbs some of the potentially dangerous radiation in sunlight. It also contains DNA repair enzymes which reverse UV damage, and people who lack the genes for these enzymes suffer high rates of skin cancer. One form predominantly produced by UV light, malignant melanoma, is particularly invasive, causing it to spread quickly, and can often be deadly. Human skin pigmentation varies among populations in a striking manner. This has sometimes led to the classification of people(s) on the basis of skin color.

Damaged skin will try to heal by forming scar tissue, often giving rise to discoloration and depigmentation of the skin.

The skin is often known as "the largest organ in the human body". This applies to exterior surface, as it covers the body, appearing to have the largest surface area of all the organs. Moreover, it applies to weight, as it weighs more than any single internal organ, accounting for about 15 percent of body weight. For the average adult human, the skin has a surface area of between 1.5-2.0 square meters, most of it is between 2-3 mm thick. The average square inch of skin holds 650 sweat glands, 20 blood vessels, 60,000 melanocytes, and more than a thousand nerve endings.

  
The use of natural or synthetic cosmetics to treat the appearance of the face and condition of the skin (such as pore control and black head cleansing) is common among many cultures.

### Layers

The skin has two major layers which are made of different tissues and have very different functions.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/2/27/Skin.png/400px-Skin.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Diagram of the layers of human skin

Skin is composed of the _epidermis_ and the _dermis_. Below these layers lies the _hypodermis or subcutaneous adipose layer_, which is not usually classified as a layer of skin.

The outermost epidermis consists of stratified squamous keratinizing epithelium with an underlying basement membrane. It contains no blood vessels, and is nourished by diffusion from the dermis. The main type of cells which make up the epidermis are keratinocytes, with melanocytes and Langerhans cells also present. The epidermis can be further subdivided into the following _strata_ (beginning with the outermost layer): corneum, lucidum, granulosum, spinosum, basale. Cells are formed through mitosis at the innermost layers. They move up the strata changing shape and composition as they differentiate, inducing expression of new types of keratin genes. They eventually reach the corneum and become sloughed off (desquamation). This process is called _keratinization_ and takes place within about 30 days. This layer of skin is responsible for keeping water in the body and keeping other harmful chemicals and pathogens out.

Blood capillaries are found beneath the dermis, and are linked to an arteriole and a venule. Arterial shunt vessels may bypass the network in ears, the nose and fingertips.

The dermis lies below the epidermis and contains a number of structures including blood vessels, nerves, hair follicles, smooth muscle, glands and lymphatic tissue. It consists of loose connective tissue otherwise called areolar connective tissue - collagen, elastin and reticular fibers are present. Erector muscles, attached between the hair papilla and epidermis, can contract, resulting in the hair fiber pulled upright and consequentially goose bumps. The main cell types are fibroblasts, adipocytes (fat storage) and macrophages. Sebaceous glands are exocrine glands which produce, a mixture of lipids and waxy substances: lubrication, water-proofing, softening and antibactericidal actions are among the many functions of sebum. Sweat glands open up via a duct onto the skin by a pore.

The dermis is made of an irregular type of fibrous connective tissue consisting of collagen and elastin fibers. It can be split into the _papillary_ and _reticular_ layers. The papillary layer is outermost and extends into the epidermis to supply it with vessels. It is composed of loosely arranged fibers. Papillary ridges make up the lines of the hands giving us fingerprints. The reticular layer is more dense and is continuous with the hypodermis. It contains the bulk of the structures (such as sweat glands). The reticular layer is composed of irregularly arranged fibers and resists stretching.

The hypodermis is not part of the skin, and lies below the dermis. Its purpose is to attach the skin to underlying bone and muscle as well as supplying it with blood vessels and nerves. It consists of loose connective tissue and elastin. The main cell types are fibroblasts, macrophages and adipocytes (the hypodermis contains 95% of body fat). Fat serves as padding and insulation for the body.

**Clinical Application:**  
The patch drug delivery system. The transdermal patch is an increasingly popular drug delivery system. These patches are designed so that the drug molecules diffuse through the epidermis to the blood vessels in the dermis layer. A typical patch works well for small lipid-soluble molecules (for example, estrogen, nitroglycerin, and nicotine) that can make their way between epidermal cells.

### Functions

![Pili.JPG](../../../upload.wikimedia.org/wikibooks/en/f/fe/Pili.JPG)

  1. Protection: Skin gives an anatomical barrier between the internal and external environment in bodily defense; Langerhans cells in the skin are part of the immune system
  2. Sensation: Skin contains a variety of nerve endings that react to heat, cold, touch, pressure, vibration, and tissue injury; see [somatosensory](http://en.wikipedia.org/wiki/Somatosensory_system) system and [touch](http://en.wikipedia.org/wiki/Touch).
  3. Heat regulation: The skin contains a blood supply far greater than its requirements which allows precise control of energy loss by radiation, convection and conduction. Dilated blood vessels increase perfusion and heat loss while constricted vessels greatly reduce cutaneous blood flow and conserve heat. Erector pili muscles are significant in animals.

  


#### Tumors

  * Benign tumors of the skin: Squamous cell papilloma
  * [Skin cancer](http://en.wikipedia.org/wiki/Skin_cancer)
  * [Acne](http://en.wikipedia.org/wiki/Acne)
  * [Keratosis pilaris](http://en.wikipedia.org/wiki/Keratosis_pilaris)
  * [Fungal](http://en.wikipedia.org/wiki/Fungus) infections such as [athlete's foot](http://en.wikipedia.org/wiki/Athlete%27s_foot)
  * [microbial infections](http://en.wikipedia.org/wiki/Microbe)
  * [calcinosis cutis](http://en.wikipedia.org/wiki/Calcinosis_cutis)
  * [ulcer](http://en.wikipedia.org/wiki/Ulcer)

## Hair

![Wikipedia-logo.png](../../../upload.wikimedia.org/wikipedia/commons/thumb/6/63/Wikipedia-logo.png/40px-Wikipedia-logo.png)

[Wikipedia](http://en.wikipedia.org/wiki/) has related information at _**[Hair**_](http://en.wikipedia.org/wiki/Hair)

  


  


### Types of hair

Humans have three different types of hair:

  * Lanugo, the fine, unpigmented hair that covers nearly the entire body of a fetus, although most has been replaced with vellus by the time of the baby's birth
  * Vellus hair, the short, downy, "peach fuzz" body hair (also unpigmented) that grows in most places on the human body. While it occurs in both sexes, and makes up much of the hair in children, men have a much smaller percentage (around 10%) vellus whereas 2/3 of a female's hair is vellus.
  * Terminal hair, the fully developed hair, which is generally longer, coarser, thicker, and darker than vellus hair, and often is found in regions such as the axillary, male beard, and pubic.

### Pathological impacts on hair

Drugs used in cancer chemotherapy frequently cause a temporary loss of hair, noticeable on the head and eyebrows, because they kill all rapidly dividing cells, not just the cancerous ones. Other diseases and traumas can cause temporary or permanent loss of hair, either generally or in patches.

The hair shafts may also store certain poisons for years, even decades, after death. In the case of Col. Lafayette Baker, who died July 3, 1868, use of an atomic absorption spectrophotometer showed the man was killed by white arsenic. The prime suspect was Wallace Pollock, Baker's brother-in-law. According to Dr. Ray A. Neff, Pollack had laced Baker's beer with it over a period of months, and a century or so later minute traces of arsenic showed up in the dead man's hair. Mrs. Baker's diary seems to confirm that it was indeed arsenic, as she writes of how she found some vials of it inside her brother's suit coat one day.

## Nails

  


### Parts of the fingernail

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/9f/Fingernail_label.jpg/200px-Fingernail_label.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

The parts of a finger nail

The fingernail is an important structure made of keratin. The fingernail generally serve two purposes. It serves as a protective plate and enhances sensation of the fingertip. The protection function of the fingernail is commonly known, but the sensation function is equally important. The fingertip has many nerve endings in it allowing us to receive volumes of information about objects we touch. The nail acts as a counterforce to the fingertip providing even more sensory input when an object is touched.

### Nail Structure

The structure we know of as the nail is divided into six specific parts - the root, nail bed, nail plate, eponychium (cuticle), perionychium, and hyponychium.

**Root** The root of the fingernail is also known as the germinal matrix. This portion of the nail is actually beneath the skin behind the fingernail and extends several millimeters into the finger. The fingernail root produces most of the volume of the nail and the nail bed. This portion of the nail does not have any melanocytes, or melanin producing cells. The edge of the germinal matrix is seen as a white, crescent shaped structure called the lunula.

**Nail Bed** The nail bed is part of the nail matrix called the sterile matrix. It extends from the edge of the germinal matrix, or lunula, to the hyponychium. The nail bed contains the blood vessels, nerves, and melanocytes, or melanin-producing cells. As the nail is produced by the root, it streams down along the nail bed, which adds material to the undersurface of the nail making it thicker. It is important for normal nail growth that the nail bed be smooth. If it is not, the nail may split or develop grooves that can be cosmetically unappealing.

**Nail Plate** The nail plate is the actual fingernail, made of translucent keratin. The pink appearance of the nail comes from the blood vessels underneath the nail. The underneath surface of the nail plate has grooves along the length of the nail that help anchor it to the nail bed.

**eponychium** The cuticle of the fingernail is also called the eponychium. The cuticle is situated between the skin of the finger and the nail plate fusing these structures together and providing a waterproof barrier.

**Perionychium** The perioncyhium is the skin that overlies the nail plate on its sides. It is also known as the paronychial edge. The perionychium is the site of hangnails, ingrown nails, and an infection of the skin called paronychia.

**Hyponychium** The hyponychium is the area between the nail plate and the fingertip. It is the junction between the free edge of the nail and the skin of the fingertip, also providing a waterproof barrier.

  


![](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Fingernails.jpg/200px-Fingernails.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Nails: left hand, adult human male

### Nail Diseases

Nail diseases are in a separate category from diseases of the skin. Although nails are a skin appendage, they have their own signs and symptoms which may relate to other medical conditions. Nail conditions that show signs of infection or inflammation require medical assistance and cannot be treated at a beauty parlor. Deformity or disease of the nails may be referred to as **onychosis**.

There are many disease that can occur with the fingernails and toenails. The most common of these diseases are ingrown nails and fungal infections.

#### Ingrown Nails

**Onychocryptosis**, commonly known as "ingrown nails" (unguis incarnatus), can affect either the fingers or the toes. In this condition, the nail cuts into one or both sides of the nail bed, resulting in inflammation and possibly infection. The relative rarity of this condition in the fingers suggests that pressure from the ground or shoe against the toe is a prime factor. The movements involved in walking or other physical disturbances can contribute to the problem. Mild onychocryptosis, particularly in the absence of infection, can be treated by trimming and rounding the nail. More advanced cases, which usually include infection, are treated by surgically excising the ingrowing portion of the nail down to its bony origin and cauterizing the matrix, or 'root', to prevent recurrence. This surgery is called matricectomy. The best results are achieved by cauterizing the matrix with phenol. Another method, which is much less effective, is excision of the matrix, sometimes called a 'cold steel procedure'

#### Nail Fungus

An infection of nail fungus (onychomycosis) occurs when fungi infect one or more of your nails. **Onychomycosis** generally begins as a white or yellow spot under the tip of the fingernail or toenail. As the nail fungus spreads deeper into the nail, it may cause the nail to discolor, thicken and develop crumbling edges — an unsightly and potentially painful problem.

Infections of nail fungus account for about half of all nail disorders. These infections usually develop on nails continually exposed to warm, moist environments, such as sweaty shoes or shower floors. Nail fungus isn't the same as athlete's foot, which primarily affects the skin of the feet, but at times the two may coexist and can be caused by the same type of fungus.

An infection with nail fungus may be difficult to treat, and infections may recur. But medications are available to help clear up nail fungus permanently.

### Clinical Application

Nail inspection can give a great deal of information about the internal working of the body as well, and like tongue or iris inspection, has a long history of diagnostic use in cantraditional medical practices such as Chinese medicine.

**Pliability:**

Brittleness is associated with iron deficiency, thyroid problems, impaired kidney function, circulation problems[2], and biotin deficiency[3] Splitting and fraying are associated with psoriasis, folic acid, protein and/or Vitamin C deficiency. Unusual thickness is associated with circulation problems. Thinning nails and itchy skin are associated with lichen planus[4].

**Shape and texture:**

Clubbing, or nails that curve down around the fingertips with nail beds that bulge is associated with oxygen deprivation and lung, heart, or liver disease. Spooning, or nails that grow upwards is associated with iron or B12 deficiency. Flatness can indicate a B12 vitamin deficiency[5] or Raynaud's disease[6] Pitting of the nails is associated with Psoriasis. Horizontal ridges indicate stress, and Beau's lines are associated with many serious conditions. Vertical ridges are associated with arthritis[7]. Vertical grooves are associated with kidney disorders, aging, and iron deficiency[8]. Beading is associated with rheumatoid arthritis[9]. Nails that resemble hammered brass are associated with (or portend) hair loss[10]. Short small beds are associated with heart disease[11]. Coloration of the nail bed:

Mee's lines are associated with arsenic or thallium poisoning, and renal failure. White lines across the nail are associated with heart disease, liver disease, or a history of a recent high fever[12]. Opaque white nails with a dark band at the fingertip are associated with cancer, cirrhosis, congestive heart failure, diabetes and aging[13]. Paleness or whitening is associated with liver or kidney disease and anemia[14]. Yellowing of the nail bed is associated with chronic bronchitis, lymphatic problems, diabetes, and liver disorders. Brown or copper nail beds are associated with arsenic or copper poisoning, and local fungal infection. Grey nail beds are associated with arthritis, edema, malnutrition, post-operative effects, glaucoma and cardio-pulmonary disease[15]. redness is associated with heart conditions. dark nails are associated with B12 deficiency. Stains of the nail plate (not the nail bed) are associated with nail polish[16], smoking, and henna use.

**Markings:**

Pink and white nails are associated with kidney disease[17]. Parallel white lines in the nails are associated with hypoalbuminemia. red skin at the base of the nail is associated with connective tissue disorders[18]. blue lunulae are associated with silver poisoning or lung disorder[19]. blue nail beds are (much like blue skin) associated with poor oxygenation of the blood (asthma, emphysema, etc)[20]. small white patches are associated with zinc or calcium deficiency or malabsorption, parasites, or local injury[21]. receded lunulae (fewer than 8) are associated with poor circulation[22], shallow breathing habits or thyroid mysfunction[23]. large lunulae (more than 25% of the thumb nail) is associated with high blood pressure.

## Glands

### Sweat Glands

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/1/18/Gray940.png/350px-Gray940.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

A diagrammatic sectional view of the skin (magnified). Sweat gland labeled as "sudoriferous gland" at center right.

In humans, there are two kinds of sweat glands which differ greatly in both the composition of the sweat and its purpose: Also "**click**" here["How our body Sweats"](http://health.howstuffworks.com/adam-200101.htm%7C) to see a short movie on sweat glands.

#### Eccrine (a.k.a. merocrine)

**Eccrine sweat glands** are exocrine glands distributed over the entire body surface but are particularly abundant on the palms of hands, soles of feet, and on the forehead. These produce sweat that is composed chiefly of water (99%) with various salts. The primary function is body temperature regulation.

Eccrine sweat glands are coiled tubular glands derived leading directly to the most superficial layer of the epidermis (out layer of skin) but extending into the inner layer of the skin (dermis layer). They are distributed over almost the entire surface of the body in humans and many other species, but are lacking in some marine and fur-bearing species. The sweat glands are controlled by sympathetic cholinergic nerves which are controlled by a center in the hypothalamus. The hypothalamus senses core temperature directly, and also has input from temperature receptors in the skin and modifies the sweat output, along with other thermoregulatory processes.

Human eccrine sweat is composed chiefly of water with various salts and organic compounds in solution. It contains minute amounts of fatty materials, urea, and other wastes. The concentration of sodium varies from 35–65 mmol/l and is lower in people acclimatised to a hot environment. The sweat of other species generally differ in composition.

#### Apocrine

Apocrine sweat glands only develop during early- to mid-puberty (approximately age 15) and release more than normal amounts of sweat for approximately a month and subsequently regulate and release normal amounts of sweat after a certain period of time. **Apocrine sweat glands** produce sweat that contains fatty materials. These glands are mainly present in the armpits and around the genital area and their activity is the main cause of sweat odor, due to the bacteria that break down the organic compounds in the sweat from these glands. Emotional stress increases the production of sweat from the apocrine glands, or more precisely: the sweat already present in the tubule is squeezed out. Apocrine sweat glands essentially serve as scent glands.

In some areas of the body, these sweat glands are modified to produce wholly different secretions, including the cerumen ("wax") of the outer ear. Other glands, such as Mammary glands, are greatly enlarged and modified to produce milk.

### Sebaceous Glands

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/d/d6/HairFollicle.jpg/200px-HairFollicle.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Schematic view of a hair follicle with sebaceous gland.

The **sebaceous glands** are glands found in the skin of mammals. They secrete an oily substance called **sebum** (Latin, meaning _fat_ or _tallow_) that is made of fat (lipids) and the debris of dead fat-producing cells. These glands exist in humans throughout the skin except in the palms of the hands and soles of the feet. Sebum acts to protect and waterproof hair and skin, and keep them from becoming dry, brittle, and cracked. It can also inhibit the growth of microorganisms on skin.

Sebaceous glands can usually be found in hair-covered areas where they are connected to hair follicles to deposit sebum on the hairs, and bring it to the skin surface along the hair shaft. The structure consisting of hair, hair follicle and sebaceous gland is also known as **pilosebaceous unit**. Sebaceous glands are also found in non haired areas of lips, eyelids, penis, labia minora and nipples; here the sebum reaches the surface through ducts. In the glands, sebum is produced within specialized cells and is released as these cells burst; sebaceous glands are thus classified as holocrine glands.

Sebum is odorless, but its bacterial breakdown can produce odors. Sebum is the cause of some people experiencing "oily" hair if it is not washed for several days. Earwax is partly sebum, as is mucopurulent discharge, the dry substance accumulating in the corners of the eye after sleeping.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/5/5e/Gray944.png/360px-Gray944.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

A hair follicle with associated structures.

The composition of sebum varies from species to species; in humans, the lipid content consists of about 25% wax monoesters, 41% triglycerides, 16% free fatty acids, and 12% squalene.

The activity of the sebaceous glands increases during puberty because of heightened levels of androgens.

Sebaceous glands are involved in skin problems such as acne and keratosis pilaris. A blocked sebaceous gland can result in a sebaceous cyst. The prescription drug isotretinoin significantly reduces the amount of sebum produced by the sebaceous glands, and is used to treat acne. The extreme use (up to 10 times doctor prescribed amounts) of anabolic steroids by bodybuilders to prevent weight loss tend to stimulate the sebaceous glands which can cause acne.

The sebaceous glands of a human fetus _in utero_ secrete a substance called Vernix caseosa, a "waxy" or "cheesy" white substance coating the skin of newborns.

The preputial glands of mice and rats are large modified sebaceous glands that produce pheromones.

### Ceruminous glands

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/5/5f/Earwax_on_swab.jpg/200px-Earwax_on_swab.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Wet-type human earwax on a cotton swab.

**Earwax**, also known by the medical term **cerumen**, is a yellowish, waxy substance secreted in the ear canal of humans and many other mammals. It plays a vital role in the human ear canal, assisting in cleaning and lubrication, and also provides some protection from bacteria, fungus, and insects. A comprehensive review of the physiology and pathophysiology of cerumen can be found in Roeser and Ballachanda. Excess or impacted cerumen can press against the eardrum and/or occlude the external auditory canal and impair hearing.

#### Production, composition, and different types

Cerumen is produced in the outer third of the cartilaginous portion of the human ear canal. It is a mixture of viscous secretions from sebaceous glands and less-viscous ones from modified apocrine sweat glands.

Two distinct genetically determined types of earwax are distinguished -- the wet-type which is dominant, and the dry type which is recessive. Asians and Native Americans are more likely to have the dry type of cerumen (grey and flaky), whereas Caucasians and Africans are more likely to have the wet type (honey-brown to dark-brown and moist). Cerumen type has been used by anthropologists to track human migratory patterns, such as those of the Inuit.

The difference in cerumen type has been tracked to a single base change (an single nucleotide polymorphism) in a gene known as "ATP-binding cassette C11 gene". In addition to affecting cerumen type, this mutation also reduces sweat production. The researchers conjecture that the reduction in sweat was beneficial to the ancestors of East Asians and Native Americans who are thought to have lived in cold climates.

#### Function

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/d/d4/Earwax_on_swab_\(uv\).jpg/200px-Earwax_on_swab_\(uv\).jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Wet-type earwax fluoresces weakly under ultraviolet light.

**Cleaning.** Cleaning of the ear canal occurs as a result of the "conveyor belt" process of epithelial migration, aided by jaw movement. Cells formed in the center of the tympanic membrane migrate outwards from the umbo (at a rate equivalent to that of fingernail growth) to the walls of the ear canal, and accelerate towards the entrance of the ear canal. The cerumen in the canal is also carried outwards, taking with it any dirt, dust, and particulate matter that may have gathered in the canal. Jaw movement assists this process by dislodging debris attached to the walls of the ear canal, increasing the likelihood of its extrusion.

**Lubrication.** Lubrication prevents desiccation and itching of the skin within the ear canal (known as _asteatosis_). The lubricative properties arise from the high lipid content of the sebum produced by the sebaceous glands. In wet-type cerumen at least, these lipids include cholesterol, squalene, and many long-chain fatty acids and alcohols.

**Antibacterial and antifungal roles.** While studies conducted up until the 1960s found little evidence supporting an antibacterial role for cerumen, more recent studies have found that cerumen provides some bactericidal protection against some strains of bacteria. Cerumen has been found to be effective in reducing the viability of a wide range of bacteria (sometimes by up to 99%), including _Haemophilus influenzae_, _Staphylococcus aureus_, and many variants of _Escherichia coli_. The growth of two fungi commonly present in otomycosis was also significantly inhibited by human cerumen. These antimicrobial properties are due principally to the presence of saturated fatty acids, lysozyme and, especially, to the relatively low pH of cerumen (typically around 6.1 in normal individuals).

### Mammary Glands

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/6/64/Illu_breast_anatomy.jpg/200px-Illu_breast_anatomy.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Cross section of the breast of a human female.

**Mammary glands** are the organs that, in the female mammal, produce milk for the sustenance of the young. These exocrine glands are enlarged and modified sweat glands and are the characteristic of mammals which gave the class its name.

  


#### Structure

The basic components of the mammary gland are the _alveoli_ (hollow cavities, a few millimetres large) lined with milk-secreting epithelial cells and surrounded by myoepithelial cells. These alveoli join up to form groups known as _lobules_, and each lobule has a _lactiferous duct_ that drains into openings in the nipple. The myoepithelial cells can contract, similar to muscle cells, and thereby push the milk from the alveoli through the lactiferous ducts towards the nipple, where it collects in widenings (_sinuses_) of the ducts. A suckling baby essentially squeezes the milk out of these sinuses.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/6/65/Dissected_lactating_breast_gray1172.png/250px-Dissected_lactating_breast_gray1172.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Dissection of a lactating breast.  
1 - Fat  
2 - Lactiferous duct/lobule  
3 - Lobule  
4 - Connective tissue  
5 - Sinus of lactiferous duct  
6 - Lactiferous duct

One distinguishes between a _simple mammary gland_, which consists of all the milk-secreting tissue leading to a single lactiferous duct, and a _complex mammary gland_, which consists of all the simple mammary glands serving one nipple.

Humans normally have two complex mammary glands, one in each breast, and each complex mammary gland consists of 10-20 simple glands. (The presence of more than two nipples is known as polythelia and the presence of more than two complex mammary glands as polymastia.)

Also, **"click"** this;["Breast tissue"](http://health.howstuffworks.com/adam-200040.htm%7C), to this a movie visual of the breast.

#### Development and hormonal control

The development of mammary glands is controlled by hormones. The mammary glands exist in both sexes, but they are rudimentary until puberty when in response to ovarian hormones, they begin to develop in the female. Click this [[1]](http://health.howstuffworks.com/adam-200042.htm)to see what breast tissue does in a female during menustration. Estrogen promotes formation, while testosterone inhibits it.

At the time of birth, the baby has lactiferous ducts but no alveoli. Little branching occurs before puberty when ovarian estrogens stimulate branching differentiation of the ducts into spherical masses of cells that will become alveoli. True secretory alveoli only develop in pregnancy, where rising levels of estrogen and progesterone cause further branching and differentiation of the duct cells, together with an increase in adipose tissue and a richer blood flow.

Colostrum is secreted in late pregnancy and for the first few days after giving birth. True milk secretion (lactation) begins a few days later due to a reduction in circulating progesterone and the presence of the hormone prolactin. The suckling of the baby causes the release of the hormone oxytocin which stimulates contraction of the myoepithelial cells.

#### Breast cancer

As described above, the cells of mammary glands can easily be induced to grow and multiply by hormones. If this growth runs out of control, cancer results. Almost all instances of breast cancer originate in the lobules or ducts of the mammary glands.

**Types of breast cancer**

  * [DCIS](http://en.wikibooks.org/w/index.php?title=DCIS&action=edit&redlink=1): Ductal Carcinoma in Situ
  * [LCIS](http://en.wikibooks.org/w/index.php?title=LCIS&action=edit&redlink=1): Lobular Carcinoma in Situ
  * [Invasive ductal carcinoma](http://en.wikibooks.org/w/index.php?title=Invasive_ductal_carcinoma&action=edit&redlink=1)
  * [Invasive lobular carcinoma](http://en.wikibooks.org/w/index.php?title=Invasive_lobular_carcinoma&action=edit&redlink=1)
  * [Inflammatory breast cancer](http://en.wikibooks.org/w/index.php?title=Inflammatory_breast_cancer&action=edit&redlink=1)
  * [Paget's disease](http://en.wikibooks.org/w/index.php?title=Paget%27s_disease&action=edit&redlink=1)

![](../../../upload.wikimedia.org/wikipedia/commons/a/a5/En_Breast_cancer_illustrations.gif)

Early Signs of Breast Cancer

## Homeostasis

As a whole, the integumentary system plays a big part in maintaining homeostasis. The integumentary system is the outermost organ system of the body and many of its functions are related to this location. The skin protects the body against pathogens and chemicals, minimizes loss or entry of water, and blocks the harmful effects of sunlight. Sensory receptors in the skin provide information about the external environment, helping the skin regulate body temperature in response to environmental changes and helping the body react to pain and other tactile stimuli. The large surface area of the skin makes it ideal for temperature regulation. The rate of heat loss can be regulated by the amount of blood flowing through the blood vessels in the dermis close to the surface of the skin. When the body temperature rises, as for example during exercise, sympathetic tone is reduced and this brings about dilation of the blood vessels supplying the skin. The increase in skin blood flow allows heat to be lost more rapidly so that body temperature does not rise above the normal homeostatic range. The rate of heat loss can also be boosted by the production of sweat, which takes up additional heat as it evaporates. Conversely, if heat production is less than required, the dermal vessels constrict, sweating stops, and heat is conserved by the body.

## Glossary

Areolar
    Areolar connective tissue is a pliable, mesh-like tissue with a fluid matrix and functions to cushion and protect body organs. It acts as a packaging tissue holding the internal organs together and in correct placement.

[Basal lamina](http://en.wikipedia.org/wiki/Basement_membrane)  
    Basal lamina (often erroneously called basement membrane) is a layer on which epithelium sits. This layer is composed of an electron-dense layer (lamina densa) between two electron-lucid layers (lamina lucida), and is approximately 40-50 nm thick (with exceptions such as the 100-200 nm glomerular basement membrane).

[Dermis](http://en.wikipedia.org/wiki/Dermis)  
    The dermis is the layer of skin beneath the epidermis that consists of connective tissue and cushions the body from stress and strain. The dermis is tightly connected to the epidermis by a basement membrane.

[Epidermis](http://en.wikipedia.org/wiki/Epidermis_%28skin%29) 
    The epidermis is the outermost layer of the skin. It forms the waterproof, protective wrap over the body's surface and is made up of stratified squamous epithelium with an underlying basal lamina.

[Fibroblasts](http://en.wikipedia.org/wiki/Fibroblasts)  
    A fibroblast is a cell that makes the structural fibers and ground substance of connective tissue.

[Hair follicle](http://en.wikipedia.org/wiki/Hair_follicle)  
    A hair follicle is part of the skin that grows hair by packing old cells together.

[Hypodermis](http://en.wikipedia.org/wiki/Hypodermis)  
    The hypodermis (also called the hypoderm), is the lowermost layer of the integumentary system in vertebrates. It is derived from the mesoderm, but unlike the dermis, it is not derived from the dermatome region of the mesoderm.

Impetigo
    This is a superficial skin infection most common among children age 2–6 years. People who play close contact sports such as rugby, American football and wrestling are also susceptible, regardless of age. The name derives from the Latin impetere ("assail"). It is also known as school sores.

[Melanocytes](http://en.wikipedia.org/wiki/Melanocyte)  
    These are cells located in the bottom layer of the skin's epidermis and in the middle layer of the eye, the uvea. Through a process called melanogenesis, these cells produce melanin, a pigment in the skin, eyes, and hair.

[Melanoma](http://en.wikipedia.org/wiki/Melanoma)  
    A melanoma is a malignant tumor that originates in melanocytes. It is a highly malignant form of skin cancer, and, though rare, is responsible for the majority of skin cancer-related deaths.

[Onychosis](http://en.wikipedia.org/wiki/Onychosis)  
    Deformity or disease of the nails

[Papillary](http://en.wikipedia.org/wiki/Papillary) 
    The papillary layer is outermost and extends into the epidermis to supply it with vessels. It is composed of loosely arranged fibres. Papillary ridges make up the lines of the hands.

[Recticular Layer](http://en.wikipedia.org/wiki/Reticular_layer)  
    The reticular layer is more dense and is continuous with the hypodermis. It contains the bulk of the structures (such as sweat glands). The reticular layer is composed of irregularly arranged fibres and resists stretching.

For more fun pictures of other skin diseases and skin problems "click" to this cool website
    ["Dermatology Image Database"](http://tray.dermatology.uiowa.edu/Home.html%7C). Note: From this link then click "Clinical Skin Diseases Images".

## Review Questions

Answers for these questions can be found [here](http://en.wikibooks.org/wiki/Human_Physiology/Appendix_1:_answers_to_review_questions#Integumentary_System)

1\. Name all of the parts of the integumentary system.

2\. Name the cells that produce melanin and describe its function.

3\. Name and describe the importance of the cutaneous senses.

4\. Explain how sweating helps maintain normal body temperature.

5\. Explain where on the body hair has important functions and describe these functions.

6\. What is a melanoma?

A) The outermost layer of skin

B) A type of nail disease

C) A malignant tumor that originates in melanocytes

D) The lower most layer of skin

## References

    Brannon, Heather (2006). "Nail Anatomy" About, Inc., A part of The New York Times Company.

    [American Academy of Dermatology - Nail Health](http://www.aad.org/public/Publications/pamphlets/NailHealth.htm)

    Cobb, Judith. [Fingernails, Jewels or Tools](http://www.nspforum.com/faq/index.cgi?read=929)? _Nature's Field_ \- Nail diagnosis

    Graaff, Van De (2002). _Human Anatomy, Sixth Edition_. New York: McGraw-Hill.

    Mader, Sylvia S. (2004). _Human Biology_. New York: McGraw-Hill.

    Sorrentino, Sheila A. (2004). _Mosby's textbook for Nursing Assistants, 6th Edition_. St. Louis, Missouri: Mosby.

  


# The Nervous System

The **central nervous system** includes the **brain** and **spinal cord**. The brain and spinal cord are protected by bony structures, membranes, and fluid. The brain is held in the cranial cavity of the skull and it consists of the **cerebrum**, **cerebellum**, and the **brain stem**. The nerves involved are cranial nerves and spinal nerves.

  


![](../../../upload.wikimedia.org/wikipedia/commons/b/ba/Nervous_system_diagram.png)

Nervous system

## Overview of the entire nervous system

The nervous system has three main functions: sensory input, integration of data and motor output. Sensory input is when the body gathers information or data, by way of neurons, glia and synapses. The nervous system is composed of excitable nerve cells (neurons) and synapses that form between the neurons and connect them to centers throughout the body or to other neurons. These neurons operate on excitation or inhibition, and although nerve cells can vary in size and location, their communication with one another determines their function. These nerves conduct impulses from sensory receptors to the brain and spinal cord. The data is then processed by way of integration of data, which occurs only in the brain. After the brain has processed the information, impulses are then conducted from the brain and spinal cord to muscles and glands, which is called motor output. Glia cells are found within tissues and are not excitable but help with myelination, ionic regulation and extracellular fluid.

The nervous system is comprised of two major parts, or subdivisions, the central nervous system (CNS) and the peripheral nervous system (PNS). The CNS includes the brain and spinal cord. The brain is the body's "control center". The CNS has various centers located within it that carry out the sensory, motor and integration of data. These centers can be subdivided to Lower Centers (including the spinal cord and brain stem) and Higher centers communicating with the brain via effectors. The PNS is a vast network of spinal and cranial nerves that are linked to the brain and the spinal cord. It contains sensory receptors which help in processing changes in the internal and external environment. This information is sent to the CNS via afferent sensory nerves. The PNS is then subdivided into the autonomic nervous system and the somatic nervous system. The autonomic has involuntary control of internal organs, blood vessels, smooth and cardiac muscles. The somatic has voluntary control of skin, bones, joints, and skeletal muscle. The two systems function together, by way of nerves from the PNS entering and becoming part of the CNS, and vice versa.

## General functions of the CNS

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/0/0f/Central_nervous_system.svg/188px-Central_nervous_system.svg.png)

Brain, brain stem, and spinal cord.

When the central nervous system becomes damaged or peripheral nerves become trapped, it can increase or decrease your internal organs functionality, it can even affect your facial expressions, i.e. make you frown a lot, your smile becomes lop sided, your lungs can overwork, or underwork, the lung capacity is increased or decreased, your bladder can fill, but you are unable to urinate, your bowels become lapsed and you are unable to completely clear them upon each bowel movement, the muscles in your arms, legs and torso can become weaker and more fatty, not from lack of use, but from the nerves that run from your spine into them being restricted from working properly, you can suffer headaches, earaches, sore throats, blocked sinuses. Even your ability to orgasm can be affected.

**CNS:**  
The "Central Nervous System", comprised of brain, brain stem, and spinal cord.

  
The central nervous system (CNS) represents the largest part of the nervous system, including the brain and the spinal cord. Together with the peripheral nervous system (PNS), it has a fundamental role in the control of behavior.

The CNS is conceived as a system devoted to information processing, where an appropriate motor output is computed as a response to a sensory input. Many threads of research suggest that motor activity exists well before the maturation of the sensory systems, and senses only influence behavior without dictating it. This has brought the conception of the CNS as an autonomous system.

![Wikipedia-logo.png](../../../upload.wikimedia.org/wikipedia/commons/thumb/6/63/Wikipedia-logo.png/40px-Wikipedia-logo.png)

[Wikipedia](http://en.wikipedia.org/wiki/) has related information at _**[Central nervous system**_](http://en.wikipedia.org/wiki/Central_nervous_system)

## Structure and function of neurons

### Structure

Neurons are highly specialized for the processing and transmission of cellular signals. Given the diversity of functions performed by neurons in different parts of the nervous system, there is, as expected, a wide variety in the shape, size, and electrochemical properties of neurons. For instance, the soma of a neuron can vary in size from 4 to 100 micrometers in diameter.

The soma (cell body) is the central part of the neuron. It contains the nucleus of the cell, and therefore is where most protein synthesis occurs. The nucleus ranges from 3 to 18 micrometers in diameter. The dendrites of a neuron are cellular extensions with many branches, and metaphorically this overall shape and structure is referred to as a dendritic tree. This is where the majority of input to the neuron occurs. However, information outflow (i.e. from dendrites to other neurons) can also occur (except in chemical synapse in which backflow of impulse is inhibited by the fact that axon do not possess chemoreceptors and dendrites cannot secrete neurotransmitter chemical). This explains one way conduction of nerve impulse. The axon is a finer, cable-like projection which can extend tens, hundreds, or even tens of thousands of times the diameter of the soma in length. The axon carries nerve signals away from the soma (and also carry some types of information back to it). Many neurons have only one axon, but this axon may - and usually will - undergo extensive branching, enabling communication with many target cells. The part of the axon where it emerges from the soma is called the 'axon hillock'. Besides being an anatomical structure, the axon hillock is also the part of the neuron that has the greatest density of voltage-dependent sodium channels. This makes it the most easily-excited part of the neuron and the spike initiation zone for the axon: in neurological terms it has the greatest hyperpolarized action potential threshold. While the axon and axon hillock are generally involved in information outflow, this region can also receive input from other neurons as well. The axon terminal is a specialized structure at the end of the axon that is used to release neurotransmitter chemicals and communicate with target neurons. Although the canonical view of the neuron attributes dedicated functions to its various anatomical components, dendrites and axons often act in ways contrary to their so-called main function.

Axons and dendrites in the central nervous system are typically only about a micrometer thick, while some in the peripheral nervous system are much thicker. The soma is usually about 10–25 micrometers in diameter and often is not much larger than the cell nucleus it contains. The longest axon of a human motor neuron can be over a meter long, reaching from the base of the spine to the toes. Sensory neurons have axons that run from the toes to the dorsal columns, over 1.5 meters in adults. Giraffes have single axons several meters in length running along the entire length of their necks. Much of what is known about axonal function comes from studying the squids giant axon, an ideal experimental preparation because of its relatively immense size (0.5–1 millimeters thick, several centimeters long).

## Function

Sensory afferent neurons convey information from tissues and organs into the central nervous system. Efferent neurons transmit signals from the central nervous system to the effector cells and are sometimes called motor neurons. Interneurons connect neurons within specific regions of the central nervous system. Afferent and efferent can also refer generally to neurons which, respectively, bring information to or send information from brain region.

Classification by action on other neurons

Excitatory neurons excite their target postsynaptic neurons or target cells causing it to function. Motor neurons and somatic neurons are all excitatory neurons. Excitatory neurons in the brain are often glutamatergic. Spinal motor neurons, which synapse on muscle cells, use acetylcholine as their neurotransmitter. Inhibitory neurons inhibit their target neurons. Inhibitory neurons are also known as short axon neurons, interneurons or microneurons. The output of some brain structures (neostriatum, globus pallidus, cerebellum) are inhibitory. The primary inhibitory neurotransmitters are GABA and glycine. Modulatory neurons evoke more complex effects termed neuromodulation. These neurons use such neurotransmitters as dopamine, acetylcholine, serotonin and others. Each synapses can receive both excitatory and inhibitory signals and the outcome is determined by the adding up of summation.

## Excitatory and inhibitory process

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/3/33/Synapse_v_int.png/300px-Synapse_v_int.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Nerve Synapse

The release of an excitatory neurotransmitter (e.g. glutamate) at the synapses will cause an inflow of positively charged sodium ions (Na+) making a localized depolarization of the membrane. The current then flows to the resting (polarized) segment of the axon.

Inhibitory synapse causes an inflow of Cl- (chlorine) or outflow of K+ (potassium) making the synaptic membrane hyperpolarized. This increase prevents depolarization, causing a decrease in the possibility of an axon discharge. If they are both equal to their charges, then the operation will cancel itself out. This effect is referred to as summation.

There are two types of summation: spatial and temporal. Spatial summation requires several excitatory synapses (firing several times) to add up, thus causing an axon discharge. It also occurs within inhibitory synapses, where just the opposite will occur. In temporal summation, it causes an increase of the frequency at the same synapses until it is large enough to cause a discharge. Spatial and temporal summation can occur at the same time as well.

The neurons of the brain release inhibitory neurotransmitters far more than excitatory neurotransmitters, which helps explain why we are not aware of all memories and all sensory stimuli simultaneously. The majority of information stored in the brain is inhibited most of the time.

## Summation

When excitatory synapses exceed the amount of inhibitory synapses there are, then the excitatory synapses will prevail over the other. The same goes with inhibitory synapses, if there are more inhibitory synapses than excitatory, the synapses will be inhibited. To determine all of this is called summation.

Classification by discharge patterns:

Neurons can be classified according to their electrophysiological characteristics (note that a single action potential is not enough to move a large muscle, and instead will cause a twitch).

**Tonic or regular spiking:** Some neurons are typically constantly (or tonically) active. Example: interneurons in neurostriatum.

**Phasic or bursting:** Neurons that fire in bursts are called phasic.

**Fast spiking:** Some neurons are notable for their fast firing rates. For example, some types of cortical inhibitory interneurons, cells in globus pallidus.

**Thin-spike:** Action potentials of some neurons are more narrow compared to the others. For example, interneurons in prefrontal cortex are thin-spike neurons.

Classification by neurotransmitter released:

Some examples are cholinergic, GABAergic, glutamatergic and dopaminergic neurons.

### Central Nervous System

The central nervous system is the control center for the body. It regulates organ function, higher thought, and movement of the body. The central nervous system consists of the brain and spinal cord.

## Generation & propagation of an action potential

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/c/cc/Action_potential_vert.png/422px-Action_potential_vert.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Electrical characteristics of a neurochemical action potential.

  


### The Nerve Impulse

![](../../../upload.wikimedia.org/wikipedia/commons/1/12/Nerve_impulse.gif)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

animated action potential

When a nerve is stimulated the resting potential changes. Examples of such stimuli are pressure, electricity, chemicals, etc. Different neurons are sensitive to different stimuli(although most can register pain). The stimulus causes sodium ion channels to open. The rapid change in polarity that moves along the nerve fiber is called the "action potential." This moving change in polarity has several stages:

**Depolarization**
    The upswing is caused when positively charged sodium ions (Na+) suddenly rush through open sodium gates into a nerve cell. The membrane potential of the stimulated cell undergoes a localized change from -65 millivolts to 0 in a limited area. As additional sodium rushes in, the membrane potential actually reverses its polarity so that the outside of the membrane is negative relative to the inside. During this change of polarity the membrane actually develops a positive value for a moment(+40 millivolts). The change in voltage stimulates the opening of additional sodium channels (called a voltage-gated ion channel). This is an example of a positive feedback loop.
**Repolarization**
    The downswing is caused by the closing of sodium ion channels and the opening of potassium ion channels. Release of positively charged potassium ions (K+) from the nerve cell when potassium gates open. Again, these are opened in response to the positive voltage--they are voltage gated. This expulsion acts to restore the localized negative membrane potential of the cell (about -65 or -70 mV is typical for nerves). 

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/a/a5/Scheme_sodium-potassium_pump-en.svg/220px-Scheme_sodium-potassium_pump-en.svg.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

sodium potassium pump

**Refractory phase**
    is a short period of time after the depolarization stage. Shortly after the sodium gates open they close and go into an inactive conformation. The sodium gates cannot be opened again until the membrane is repolarized to its normal resting potential. The sodium-potassium pump returns sodium ions to the outside and potassium ions to the inside. During the refractory phase this particular area of the nerve cell membrane cannot be depolarized. This refractory area explains why action potentials can only move forward from the point of stimulation.

Increased permeability of the sodium channel occurs when there is a deficit of calcium ions. when there is a deficit of calcium ions (Ca+2) in the interstitial fluid the sodium channels are activated (opened) by very little increase of the membrane potential above the normal resting level. The nerve fiber can therefore fire off action potentials spontaneously, resulting in tetany. Could be caused by the lack of hormone from parathyroid glands. could be caused by hyperventilation, which leads to a higher pH, which causes calcium to bind and become unavailable. Speed of conduction. This area of depolarization/repolarization/recovery moves along a nerve fiber like a very fast wave. In myelinated fibers, conduction is hundreds of times faster because the action potential only occurs at the nodes of Ranvier (pictured below in 'types of neurons') by jumping from node to node. This is called "saltatory" conduction. Damage to the myelin sheath by the disease can cause severe impairment of nerve cell function. Some poisons and drugs interfere with nerve impulses by blocking sodium channels in nerves. See discussion on drug at the end of this outline.

## Brain

![](../../../upload.wikimedia.org/wikipedia/commons/5/5b/Brain-anatomy.jpg)

A color-coded image of the brain, showing the main sections.

The brain is found in the cranial cavity. Within it are found the higher nerve centers responsible for coordinating the sensory and motor systems of the body (forebrain). The brain stem houses the lower nerve centers (consisting of midbrain, pons, and medulla),

### Medulla

The medulla is the control center for respiratory, cardiovascular and digestive functions.

### Pons

The pons houses the control centers for respiration and inhibitory functions. Here it will interact with the cerebellum.

### Cerebrum

The cerebrum, or top portion of the brain, is divided by a deep crevice, called the longitudinal sulcus. The longitudinal sulcus separates the cerebrum in to the right and left hemispheres. In the hemispheres you will find the cerebral cortex, basal ganglia and the limbic system. The two hemispheres are connected by a bundle of nerve fibers called the corpus callosum. The right hemisphere is responsible for the left side of the body while the opposite is true of the left hemisphere. Each of the two hemispheres are divided into four separated lobes: the frontal in control of specialized motor control, learning, planning and speech; parietal in control of somatic sensory functions; occipital in control of vision; and temporal lobes which consists of hearing centers and some speech. Located deep to the temporal lobe of the cerebrum is the insula.

### Cerebellum

The cerebellum is the part of the brain that is located posterior to the medulla oblongata and pons. It coordinates skeletal muscles to produce smooth, graceful motions. The cerebellum receives information from our eyes, ears, muscles, and joints about what position our body is currently in (proprioception). It also receives output from the cerebral cortex about where these parts should be. After processing this information, the cerebellum sends motor impulses from the brain stem to the skeletal muscles. The main function of the cerebellum is coordination. The cerebellum is also responsible for balance and posture. It also assists us when we are learning a new motor skill, such as playing a sport or musical instrument. Recent research shows that apart from motor functions cerebellum also has some emotional role.

### **The Limbic System and Higher Mental Functions**

* * *

![](../../../upload.wikimedia.org/wikipedia/commons/5/5c/Brain_limbicsystem.jpg)

Image of the brain, showing the Limbic system.

#### **The Limbic System**

The Limbic System is a complex set of structures found just beneath the cerebrum and on both sides of the thalamus. It combines higher mental functions, and primitive emotion, into one system. It is often referred to as the emotional nervous system. It is not only responsible for our emotional lives, but also our higher mental functions, such as learning and formation of memories. The Limbic system explains why some things seem so pleasurable to us, such as eating and why some medical conditions are caused by mental stress, such as high blood pressure. There are two significant structures within the limbic system and several smaller structures that are important as well. They are:

  1. The Hippocampus
  2. The Amygdala
  3. The Thalamus
  4. The Hypothalamus
  5. The Fornix and Parahippocampus
  6. The Cingulate Gyrus

#### Structures of the Limbic System

##### Hippocampus

    The Hippocampus is found deep in the temporal lobe, shaped like a seahorse. It consists of two horns that curve back from the amygdala. It is situated in the brain so as to make the prefrontal area aware of our past experiences stored in that area. The prefrontal area of the brain consults this structure to use memories to modify our behavior. The hippocampus is responsible for memory.

##### Amygdala

    The Amygdala is a little almond shaped structure, deep inside the anteroinferior region of the temporal lobe, connects with the hippocampus, the septi nuclei, the prefrontal area and the medial dorsal nucleus of the thalamus. These connections make it possible for the amygdala to play its important role on the mediation and control of such activities and feelings as love, friendship, affection, and expression of mood. The amygdala is the center for identification of danger and is fundamental for self preservation. The amygdala is the nucleus responsible for fear..

##### Thalamus

    Lesions or stimulation of the medial, dorsal, and anterior nuclei of the thalamus are associated with changes in emotional reactivity. However, the importance of these nuclei on the regulation of emotional behavior is not due to the thalamus itself, but to the connections of these nuclei with other limbic system structures. The medial dorsal nucleus makes connections with cortical zones of the prefrontal area and with the hypothalamus. The anterior nuclei connect with the mamillary bodies and through them, via fornix, with the hippocampus and the cingulated gyrus, thus taking part in what is known as the Papez's circuit.

![](../../../upload.wikimedia.org/wikipedia/commons/b/b6/Hypothalamus.jpg)

Image of the brain showing the location of the hypothalamus.

##### Hypothalamus

    The Hypothalamus is a small part of the brain located just below the thalamus on both sides of the third ventricle. Lesions of the hypothalamus interfere with several vegetative functions and some so called motivated behaviors like sexuality, combativeness, and hunger. The hypothalamus also plays a role in emotion. Specifically, the lateral parts seem to be involved with pleasure and rage, while the medial part is linked to aversion, displeasure, and a tendency to uncontrollable and loud laughing. However, in general the hypothalamus has more to do with the expression of emotions. When the physical symptoms of emotion appear, the threat they pose returns, via the hypothalamus, to the limbic centers and then the prefrontal nuclei, increasing anxiety.

##### The Fornix and Parahippocampal

    These small structures are important connecting pathways for the limbic system.

##### The Cingulate Gyrus

    The Cingulate Gyrus is located in the medial side of the brain between the cingulated sulcus and the corpus callosum. There is still much to be learned about this gyrus, but it is already known that its frontal part coordinates smells and sights, with pleasant memories of previous emotions. The region participates in the emotional reaction to pain and in the regulation of aggressive behavior.

  


#### **Memory and Learning**

Memory is defined as : The mental faculty of retaining and recalling past experiences, the act or instance of remembering recollection. Learning takes place when we retain and utilize past memories.

Overall, the mechanisms of memory are not completely understood. Brain areas such as the hippocampus, the amygdala, the striatum, or the mammillary bodies are thought to be involved in specific types of memory. For example, the hippocampus is believed to be involved in spatial learning and declarative learning (learning information such as what you're reading now), while the amygdala is thought to be involved in emotional memory. Damage to certain areas in patients and animal models and subsequent memory deficits is a primary source of information. However, rather than implicating a specific area, it could be that damage to adjacent areas, or to a pathway traveling through the area is actually responsible for the observed deficit. Further, it is not sufficient to describe memory, and its counterpart, learning, as solely dependent on specific brain regions. Learning and memory are attributed to changes in neuronal synapses, thought to be mediated by long-term potentiation and long-term depression.

There are three basic types of memory:

  1. Sensory Memory
  2. Short Term Memory
  3. Long Term Memory

##### Sensory Memory

    The sensory memories act as a buffer for stimuli through senses. A sensory memory retains an exact copy of what is seen or heard: _iconic memory for visual, echoic memory for aural and haptic memory for touch._ Information is passed from sensory memory into short term memory. Some believe it lasts only 300 milliseconds, it has unlimited capacity. Selective attention determines what information moves from sensory memory to short term memory.

##### Short Term Memory

    Short Term Memory acts as a scratch pad for temporary recall of the information under process. For instance, in order to understand this sentence you need to hold in your mind the beginning of the sentence as you read the rest. Short term memory decays rapidly and also has a limited capacity. Chunking of information can lead to an increase in the short term memory capacity, this is the reason why a hyphenated phone number is easier to remember than a single long number. The successful formation of a chunk is known as _closure._ Interference often causes disturbance in short term memory retention. This accounts for the desire to complete a task held in short term memory as soon as possible.

Within short term memory there are three basic operations:

  1. Iconic memory - the ability to hold visual images
  2. Acoustic memory - the ability to hold sounds. Can be held longer than iconic.
  3. Working memory - an active process to keep it until it is put to use. Note that the goal is not really to move the information from short term memory to long term memory, but merely to put it to immediate use.

The process of transferring information from short term to long term memory involves the encoding or consolidation of information. This is not a function of time, that is, the longer the memory stays in the short term the more likely it is to be placed in the long term memory. On organizing complex information in short term before it can be encoded into the long term memory, in this process the meaningfulness or emotional content of an item may play a greater role in its retention in the long term memory. The limbic system sets up local reverberating circuits such as the Papez's Circuit.

##### Long Term Memory

    Long Term Memory is used for storage of information over a long time. Information from short to long term memory is transferred after a short period. Unlike short term memory, long term memory has little decay. Long term potential is an enhanced response at the synapse within the hippocampus. It is essential to memory storage. The limbic system isn't directly involved in long term memory necessarily but it selects them from short term memory, consolidates these memories by playing them like a continuous tape, and involves the hippocampus and amygdala.

There are two types of long term memory:

  1. Episodic Memory
  2. Semantic Memory

Episodic memory represents our memory of events and experiences in a serial form. It is from this memory that we can reconstruct the actual events that took place at a given point in our lives. Semantic memory, on the other hand, is a structured record of facts, concepts, and skills that we have acquired. The information in the semantic memory is derived from our own episode memory, such as that we can learn new facts or concepts from experiences.

There are three main activities that are related to long term memory:

  1. Storage
  2. Deletion
  3. Retrieval

Information for short term memory is stored in long term memory by rehearsal. The repeated exposure to a stimulus or the rehearsal of a piece of information transfers it into long term memory. Experiments also suggest that learning is most effective if it is distributed over time. Deletion is mainly caused by decay and interference. Emotional factors also affect long term memory. However, it is debatable whether we actually ever forget anything or whether it just sometimes becomes increasingly difficult to retrieve it. Information may not be recalled sometimes but may be recognized, or may be recalled only with prompting. This leads us to the third operation of memory, information retrieval.

There are two types of information retrieval:

  1. Recall
  2. Recognition

In recall, the information is reproduced from memory. In recognition the presentation of the information provides the knowledge that the information has been seen before. Recognition is of lesser complexity, as the information is provided as a cue. However, the recall may be assisted by the provision of retrieval cues which enable the subject to quickly access the information in memory.

##### Long-term Potentiation

![warning](../../../upload.wikimedia.org/wikipedia/commons/thumb/d/dc/Nuvola_apps_important_yellow.svg/40px-Nuvola_apps_important_yellow.svg.png)

Long term potentiation is largely theoretical. Many of the concepts have experimental backing, but there is still a substantial amount of controversy over other parts. There are also processes involved in long-term potentiation that go beyond the scope of this introductory book, and have been simplified. Consider this a starting place.

.

Long-term potentiation (LTP) is the lasting enhancement of connections between two neurons that results from stimulating them simultaneously. Since neurons communicate via chemical synapses, and because memories are believed to be stored within these synapses, LTP and its opposing process, long-term depression, are widely considered the major cellular mechanisms that underlie learning and memory. This has been proven by lab experiments. When one of the chemicals involved (PKMzeta, it will be discussed later) is inhibited in rats, it causes retrograde amnesia with short term memory left intact (meaning they can't recall events from before the inhibitor was given).

By enhancing synaptic transmission, LTP improves the ability of two neuron, one presynaptic and the other postsynaptic, to communicate with one another across a synapse. The precise mechanism for this enhancement isn't known, but it varies based on things like brain region, age and species. This will focus on LTP in the CA1 section of the hippocampus, because that's what is well known.

The end result of LTP is a well established neural circuit that can be called upon later for memory.

LTP in the CA1 hippocampus is called NMDA receptor-dependent LTP. It has four main properties.

  * Rapid induction

    LTP can be rapidly induced by applying one or more brief, high-frequency, stimulus to a presynaptic cell.

  * Input specificity

    Once induced, LTP at one synapse does not spread to other synapses; rather LTP is input specific. LTP is only propagated to those synapses according to the rules of associativity and cooperativity.

  * Associativity

    Associativity refers to the observation that when weak stimulation of a single pathway is insufficient for the induction of LTP, simultaneous strong stimulation of another pathway will induce LTP at both pathways.

  * Cooperativity

    LTP can be induced either by strong tetanic stimulation of a single pathway to a synapse, or cooperatively via the weaker stimulation of many. When one pathway into a synapse is stimulated weakly, it produces insufficient postsynaptic depolarization to induce LTP. In contrast, when weak stimuli are applied to many pathways that converge on a single patch of postsynaptic membrane, the individual postsynaptic depolarizations generated may collectively depolarize the postsynaptic cell enough to induce LTP cooperatively. Synaptic tagging, discussed later, may be a common mechanism underlying associativity and cooperativity.

LTP is generally divided into three parts that occur sequentially: Short-term potentiation, early LTP (E-LTP) and late LTP (L-LTP). Short-term potentiation isn't well understood and will not be discussed.

E-LTP and L-LTP phases of LTP are each characterized by a series of three events: induction, maintenance and expression. Induction happens when a short-lived signal triggers that phase to begin. Maintenance corresponds to the persistent biochemical changes that occur in response to the induction of that phase. Expression entails the long-lasting cellular changes that result from activation of the maintenance signal.

Each phase of LTP has a set of mediator molecules that dictate the events of that phase. These molecules include protein receptors, enzymes, and signaling molecules that allow progression from one phase to the next. In addition to mediators, there are modulator molecules that interact with mediators to fine tune the LTP. Modulators are a bit beyond the scope of this introductory book, and won't be discussed here.

###### Early Phase

Induction

E-LTP induction begins when the calcium inside the postsynaptic cell exceeds a threshold. In many types of LTP, the flow of calcium into the cell requires the NMDA receptor, which is why these types of LTP are considered NMDA receptor-dependent.

When a stimulus is applied to the presynaptic neuron, it releases a neurotransmitter, typically glutamate, onto the postsynaptic cell membrane where it binds to AMPA receptors, or AMPARs. This causes an influx of sodium ions into the postsynaptic cell, this short lived depolarization is called the excitatory postsynaptic potential (EPSP) and makes it easier for the neuron to fire an action potential.

A single stimulus doesn't cause a big enough depolarization to trigger an E-LTP, instead it relies on EPSP summation. If EPSPs are reaching the cell before the others decay, they will add up. When the depolarization reaches a critical level, NMDA receptors lose the magnesium molecule they were originally plugged with and let calcium in. The rapid rise in calcium within the postsynaptic neuron trigger the short lasting activation of several enzymes that mediate E-LTP induction. Of particular importance are some protein kinase enzymes, including CaMKII and PKC. To a lesser extent, PKA and MAPK activation also contribute.

Maintenance

During the maintenance stage of E-LTP, CaMKII and PKC lose their dependence on calcium and become autonomously active. They then carry out phosphorylation that underlies E-LTP expression.

Expression

CaMKII and PKC phosphorylate existing AMPA receptors to increase their activity, and mediate the insertion of additional AMPA receptors onto the postsynaptic cell membrane. This is achieved by having a pool of nonsynaptic AMPA receptors adjacent to the postsynaptic membrane. When the appropriate stimulus arrives, the nonsynaptic AMPA receptors are brought into the postsynaptic membrane under the influence of protein kinases.

AMPA receptors are one of the most common type of receptors in the brain. Their effect is excitatory. By adding more AMPA receptors, and increasing their activity, future stimuli will generate larger postsynaptic responses.

###### Late Phase

Late LTP is the natural extension of E-LTP. L-LTP requires gene transcription and protein synthesis in the postsynaptic cell, unlike E-LTP. Late LTP is also associated with the presynaptic synthesis of synaptotagmin and an increase in synaptic vesicle number, suggesting that L-LTP induces protein synthesis not only in postsynaptic cells, but in presynaptic cells as well. This is discussed under "retrograde messenger" below.

Induction

Late LTP is induced by changes in gene expression and protein synthesis brought about by persistent activation of protein kinases activated during E-LTP, such as MAPK. In fact, MAPK--Specifically the ERK subfamily of MAPKs--may be the molecular link between E-LTP and L-LTP, since many signaling cascades involved in E-LTP, including CaMKII and PKC, can converge on ERK.

Maintenance

Upon activation, ERK may phosphorylate a number of cytoplasmic and nuclear molecules that ultimately result in the protein synthesis and morphological changes associated with L-LTP. These chemicals may include transcription factors such as CREB. ERK-mediated changes in transcription factor activity may trigger the synthesis of proteins that underlie the maintenance of L-LTP. PKMzeta is one such molecule. When this molecule is inhibited in rats, they experience retrograde amnesia (where you can't recall previous events but short term memory works fine).

Expression

Aside from PKMzeta, many of the proteins synthesized during L-LTP are unknown. They are though to increase postsynaptic dendritic spine number, surface area and sensitivity to the neurotransmitter associated with L-LTP expression.

###### Retrograde Signaling

Retrograde signaling is a hypothesis that attempts to explain that, while LTP is induced and expressed postsynaptically, some evidence suggests that it is expressed presynaptically as well. The hypothesis gets its name because normal synaptic transmission is directional and proceeds from the presynaptic to the postsynaptic cell. For induction to occur postsynaptically and be partially expressed presynaptically, a message must travel from the postsynaptic cell to the presynaptic cell in a retrograde (reverse) direction. Once there, the message presumably initiates a cascade of events that leads to a presynaptic component of expression, such as the increased probability of neurotransmitter vesicle release.

Retrograde signaling is currently a contentious subject as some investigators do not believe the presynaptic cell contributes at all to the expression of LTP. Even among proponents of the hypothesis there is controversy over the identity of the messenger.

#### Language and Speech

Language depends on semantic memory so some of the same areas in the brain are involved in both memory and language. Articulation, the forming of speech, is represented bilaterally in the motor areas. However, for most individuals, language analysis and speech formation take place in regions of the left hemisphere only. The two regions involved are:

  1. Broca's Area
  2. Wernicke's Area

Broca's area is located just in front of the voice control area of the left motor cortex. This region assembles the motor of speech and writing. For example, patients with lesions in this area:

  1. Are unable to understand language perfectly
  2. May not be able to write clearly
  3. Usually speak spontaneously

Wernicke's area is part of the auditory and visual associations cortex. This region is responsible for the analysis and formation of language content. For example, patients with lesions in this area:

  1. Are unable to name objects
  2. Are unable to understand the meaning of words
  3. Articulate speech readily but usually nonsensically

#### Diseases of the Limbic System

There are several well known diseases that are disorders of the limbic system. Several are discussed here.

##### Schizophrenia

An increased dopamine (DA) response in the limbic system results in schizophrenia. DA may be synthesized or secreted in excess, DA receptors may be supersensitive, and DA regulatory mechanism may be defective. Symptoms are decreased by drugs which block DA receptors. Symptoms of schizophrenia are:

  1. Loss of touch with reality
  2. Decreased ability to think and reason
  3. Decreased ability to concentrate
  4. Decreased memory
  5. Regress in child-like behavior
  6. Altered mood and impulsive behavior
  7. Auditory hallucinations

Symptoms may be so severe that the individual cannot function.

##### Depression

Depression is the most common major mental illness and is characterized by both emotional and physical symptoms. Symptoms of depression are:

  1. Intense sadness and despair
  2. Anxiety
  3. Loss of ability to concentrate
  4. Pessimism
  5. Feelings of low self esteem
  6. Insomnia or hypersomnia
  7. Increased or decreased appetite
  8. Changes in body temperature and endocrine gland function

10 to 15% of depressed individuals display suicidal behavior during their lifetime.

The cause of depression and its symptoms are a mystery but we do understand that it is an illness associated with biochemical changes in the brain. A lot of research goes on to explain that it is associated with a lack of amines serotonin and norephinephrine. Therefore pharmacological treatment strategies often try to increase amine concentrations in the brain.

One class of antidepressants is monoamine oxidase inhibitors. Mono amine oxidase is a enzyme that breaks down your amines like norephinephrine and serotonin. Because the antidepressants inhibit their degradation they will remain in the synaptic cleft for a longer period of time making the effect just as if you had increased theses types of neurotransmitters.

A newer class of antidepressants is selective serotonin reuptake inhibitors (SSRI's). With SSRI's decreasing the uptake of serotonin back into the cell that will increase the amount of serotonin present in the synaptic cleft. SSRI's are more specific than the monoamine oxidase inhibitors because they only affect serotonergic synapses. You might recognize these SSRI's by name as Prozac and Paxil.

##### Bipolar Disorder

Another common form of depression is manic depression. Manic is an acute state characterized by:

  1. Excessive elation and impaired judgment
  2. Insomnia and irritability
  3. Hyperactivity
  4. Uncontrolled speech

Manic depression, also known as bipolar disorder, displays mood swings between manic and depression. The limbic system receptors are unregulated. Drugs used are unique mood stabilizers.

The hippocampus is particularly vulnerable to several disease processes, including ischemia, which is any obstruction of blood flow or oxygen deprivation, Alzheimer’s disease, and epilepsy. These diseases selectively attack CA1, which effectively cuts through the hippocampal circuit.

##### An Autism Link

A connection between autism and the limbic system has also been noted as well. URL: <http://www.autism.org/limbic.html>

##### Case Study

_Central Pain Syndrome_

I was 42 years old when my life changed forever. I had a stroke. As an avid viewer of medical programs on television I assumed that I would have physical therapy for my paralyzed left side and get on with my life. No one ever mentioned pain or the possibility of pain, as a result of the stroke. I did experience unusual sensitivity to touch while still in the hospital, but nothing to prepare me for what was to come.

The part of my brain that is damaged is the Thalamus. This turns out to be the pain center and what I have now is an out of control Thalamus, resulting in Thalamic Pain syndrome, also called Central Pain Syndrome. This means that 24 hours a day, seven days a week, my brain sends messages of pain and it never goes away. I am under the care of physicians, who not only understand chronic pain, but are also willing to treat it with whatever medications offer some help. None of the medications, not even narcotic medications, take the pain away. They just allow me to manage it so I can function.

## The Peripheral Nervous System

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/c/c9/Gray686.png/250px-Gray686.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

The Cranial Nerves

The **peripheral nervous system** includes 12 cranial nerves 31 pairs of spinal nerves. It can be subdivided into the **somatic** and **autonomic** systems. It is a way of communication from the central nervous system to the rest of the body by nerve impulses that regulate the functions of the human body.

The twelve cranial nerves are

    I **Olfactory Nerve** for smell
    II **Optic Nerve** for vision
    III **Oculomotor** for looking around
    IV **Trochlear** for moving eye
    V **Trigeminal** for feeling touch on face
    VI **Abducens** to move eye muscles
    VII **Facial** to smile, wink, and help us taste
    VIII **Vestibulocochlear** to help with balance, equilibrium, and hearing
    IX **Glossopharengeal** for swallowing and gagging
    X **Vagus** for swallowing, talking, and parasympathetic actions of digestion
    XI **Spinal accessory** for shrugging shoulders
    XII **Hypoglossal** for tongue more divided into different regions as muscles

The 10 out of the 12 cranial nerves originate from the brain stem, and mainly control the functions of the anatomic structures of the head with some exceptions. CN X receives visceral sensory information from the thorax and abdomen, and CN XI is responsible for innervating the sternocleidomastoid and trapezius muscles, neither of which is exclusively in the head.

Spinal nerves take their origins from the spinal cord. They control the functions of the rest of the body. In humans, there are 31 pairs of spinal nerves: 8 cervical, 12 thoracic, 5 lumbar, 5 sacral and 1 coccygeal. The naming convention for spinal nerves is to name it after the vertebra immediately above it. Thus the fourth thoracic nerve originates just below the fourth thoracic vertebra. This convention breaks down in the cervical spine. The first spinal nerve originates above the first cervical vertebra and is called C1. This continues down to the last cervical spinal nerve, C8. There are only 7 cervical vertebrae and 8 cervical spinal nerves.

### Lateral cord

The **lateral cord** gives rise to the following nerves:

  * The lateral pectoral nerve, C5, C6 and C7 to the pectoralis major muscle, or musculus pectoralis major.
  * The musculocutaneous nerve which innervates the biceps muscle
  * The median nerve, partly. The other part comes from the medial cord. See below for details.

### Posterior cord

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/4/47/Dermatoms.svg/300px-Dermatoms.svg.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

diagram showing human dermatoms, i.e., skin regions with respect to the routing of their nerve connection of their afferent nerves through the spinal cord.

The **posterior cord** gives rise to the following nerves:

  * The upper subscapular nerve, C7 and C8, to the subscapularis muscle, or musculus supca of the rotator cuff.
  * The lower subscapular nerve, C5 and C6, to the teres major muscle, or the musculus teres major, also of the rotator cuff.
  * The thoracodorsal nerve, C6, C7 and C8, to the latissimus dorsi muscle, or musculus latissimus dorsi.
  * The axillary nerve, which supplies sensation to the shoulder and motor to the deltoid muscle or musculus deltoideus, and the teres minor muscle, or musculus teres minor.
  * The radial nerve, or nervus radialis, which innervates the triceps brachii muscle, the brachioradialis muscle, or musculus brachioradialis,, the extensor muscles of the fingers and wrist (extensor carpi radialis muscle), and the extensor and abductor muscles of the thumb. See radial nerve injuries.

### Medial cord

The **medial cord** gives rise to the following nerves:

  * The median pectoral nerve, C8 and T1, to the pectoralis muscle
  * The medial brachial cutaneous nerve, T1
  * The medial antebrachial cutaneous nerve, C8 and T1
  * The median nerve, partly. The other part comes from the lateral cord. C7, C8 and T1 nerve roots. The first branch of the median nerve is to the pronator teres muscle, then the flexor carpi radialis, the palmaris longus and the flexor digitorum superficialis. The median nerve provides sensation to the anterior palm, the anterior thumb, index finger and middle finger. It is the nerve compressed in carpal tunnel syndrome.
  * The ulnar nerve originates in nerve roots C7, C8 and T1. It provides sensation to the ring and pinky fingers. It innervates the flexor carpi ulnaris muscle, the flexor digitorum profundus muscle to the ring and pinky fingers, and the intrinsic muscles of the hand (the interosseous muscle, the lumbrical muscles and the flexor pollicus brevis muscle). This nerve traverses a groove on the elbow called the cubital tunnel, also known as the funny bone. Striking the nerve at this point produces an unpleasant sensation in the ring and little fingers.

### Other thoracic spinal nerves (T3-T12)

The remainder of the thoracic spinal nerves, T3 through T12, do little recombining. They form the **intercostal nerves**, so named because they run between the ribs. For points of reference, the 7th intercostal nerve terminates at the lower end of the sternum, also known as the xyphoid process. The 10th intercostal nerve terminates at the umbilicus, or the belly button.

The **somatic nervous system** is that part of the peripheral nervous system associated with the voluntary control of body movements through the action of skeletal muscles, and also reception of external stimuli. The somatic nervous system consists of afferent fibers that receive information from external sources, and efferent fibers that are responsible for muscle contraction. The somatic system includes the pathways from the skin and skeletal muscles to the Central Nervous System. It is also described as involved with activities that involve consciousness.

The basic route of the efferent somatic nervous system includes a two neuron sequence. The first is the upper motor neuron, whose cell body is located in the precentral gyrus (Brodman Area 4) of the brain. It receives stimuli from this area to control skeletal (voluntary) muscle. The upper motor neuron carries this stimulus down the corticospinal tract and synapses in the ventral horn of the spinal cord with the alpha motor neuron, a lower motor neuron. The upper motor neuron releases acetylcholine from its axon terminal knobs and these are received by nicotinic receptors on the alpha motor neuron. The alpha motor neurons cell body sends the stimulus down its axon via the ventral root of the spinal cord and proceeds to its neuromuscular junction of its skeletal muscle. There, it releases acetylcholine from its axon terminal knobs to the muscles nicotinic receptors, resulting in stimulus to contract the muscle.

The somatic system includes all the neurons connected with the muscles, sense organs and skin. It deals with sensory information and controls the movement of the body.

## The Autonomic System

The **Autonomic system** deals with the visceral organs, like the heart, stomach, gland, and the intestines. It regulates systems that are unconsciously carried out to keep our body alive and well, such as breathing, digestion (peristalsis), and regulation of the heartbeat. The Autonomic system consists of the **sympathetic** and the **parasympathetic** divisions. Both divisions work without conscious effort, and they have similar nerve pathways, but the sympathetic and parasympathetic systems generally have opposite effects on target tissues (they are antagonistic). By controlling the relative input from each division, the autonomic system regulates many aspects of homeostasis. One of the main nerves for the parasympathetic autonomic system is Cranial Nerve X, the Vagus nerve.

![](../../../upload.wikimedia.org/wikipedia/commons/7/77/Gray838.png)

**Figure 1:** The right sympathetic chain and its connections with the thoracic, abdominal, and pelvic plexuses. (After Schwalbe.)

### The Sympathetic and Parasympathetic Systems

The sympathetic nervous system activates what is often termed the fight or flight response, as it is most active under sudden stressful circumstances (such as being attacked). This response is also known as sympathetico-adrenal response of the body, as the pre-ganglionic sympathetic fibers that end in the adrenal medulla (but also all other sympathetic fibers) secrete acetylcholine, which activates the secretion of adrenaline (epinephrine) and to a lesser extent noradrenaline (norepinephrine) from it. Therefore, this response that acts primarily on the cardiovascular system is mediated directly via impulses transmitted through the sympathetic nervous system and indirectly via catecholamines secreted from the adrenal medulla.

Western science typically looks at the SNS as an automatic regulation system, that is, one that operates without the intervention of conscious thought. Some evolutionary theorists suggest that the sympathetic nervous system operated in early organisms to maintain survival (Origins of Consciousness, Robert Ornstein; et al.), as the sympathetic nervous system is responsible for priming the body for action. One example of this priming is in the moments before waking, in which sympathetic outflow spontaneously increases in preparation for action.

The parasympathetic nervous system is part of the autonomic nervous system. Sometimes called the rest and digest system or feed and breed. The parasympathetic system conserves energy as it slows the heart rate, increases intestinal and gland activity, and relaxes sphincter muscles in the gastrointestinal tract.

After high stress situations (ie: fighting for your life) the parasympathetic nervous system has a backlash reaction that balances out the reaction of the sympathetic nervous system. For example, the increase in heart rate that comes along with a sympathetic reaction will result in an abnormally slow heart rate during a parasympathetic reaction.

### Organization

Sympathetic nerves originate inside the vertebral column, toward the middle of the spinal cord in the intermediolateral cell column (or lateral horn), beginning at the first thoracic segment of the spinal cord and extending into the second or third lumbar segments. Because its cells begin in the thoracic and lumbar regions of the spinal cord, the SNS is said to have a thoracolumbar outflow. Axons of these nerves leave the spinal cord in the ventral branches (rami) of the spinal nerves, and then separate out as 'white rami' (so called from the shiny white sheaths of myelin around each axon) which connect to two chain ganglia extending alongside the vertebral column on the left and right. These elongated ganglia are also known as paravertebral ganglia or sympathetic trunks. In these hubs, connections (synapses) are made which then distribute the nerves to major organs, glands, and other parts of the body. [1]

In order to reach the target organs and glands, the axons must travel long distances in the body, and, to accomplish this, many axons link up with the axon of a second cell. The ends of the axons do not make direct contact, but rather link across a space, the synapse.

In the SNS and other components of the peripheral nervous system, these synapses are made at sites called ganglia. The cell that sends its fiber is called a preganglionic cell, while the cell whose fiber leaves the ganglion is called a postganglionic cell. As mentioned previously, the preganglionic cells of the SNS are located between the first thoracic segment and the second or third lumbar segments of the spinal cord. Postganglionic cells have their cell bodies in the ganglia and send their axons to target organs or glands.

The ganglia include not just the sympathetic trunks but also the superior cervical ganglion (which sends sympathetic nerve fibers to the head), and the celiac and mesenteric ganglia (which send sympathetic fibers to the gut).

### Information transmission

Messages travel through the SNS in a bidirectional flow. Efferent messages can trigger changes in different parts of the body simultaneously. For example, the sympathetic nervous system can accelerate heart rate; widen bronchial passages; decrease motility (movement) of the large intestine; constrict blood vessels; increase peristalsis in the esophagus; cause pupil dilation, piloerection (goose bumps) and perspiration (sweating); and raise blood pressure. Afferent messages carry sensations such as heat, cold, or pain.

The first synapse (in the sympathetic chain) is mediated by nicotinic receptors physiologically activated by acetylcholine, and the target synapse is mediated by adrenergic receptors physiologically activated by either noradrenaline or adrenaline. An exception is with sweat glands which receive sympathetic innervation but have muscarinic acetylcholine receptors which are normally characteristic of PNS. Another exception is with certain deep muscle blood vessels, which have acetylcholine receptors and which dilate (rather than constrict) with an increase in sympathetic tone. The sympathetic system cell bodies are located on the spinal cord excluding the cranial and sacral regions, specifically the thoracolumbar region (T1-L3). The preganglonic neurons exit from the vertebral column and synapse with the postganglonic nerouns in the sympathetic trunk.

The parasympathetic nervous system is one of three divisions of the autonomic nervous system. Sometimes called the rest and digest system, the parasympathetic system conserves energy as it slows the heart rate, increases intestinal and gland activity, and relaxes sphincter muscles in the gastrointestinal tract.

### Relationship to sympathetic

While an oversimplification, it is said that the parasympathetic system acts in a reciprocal manner to the effects of the sympathetic nervous system; in fact, in some tissues innervated by both systems, the effects are synergistic.

  


### Receptors

The parasympathetic nervous system uses only acetylcholine (ACh) as its neurotransmitter. The ACh acts on two types of receptors, the muscarinic and nicotinic cholinergic receptors. Most transmissions occur in two stages: When stimulated, the preganglionic nerve releases ACh at the ganglion, which acts on nicotinic receptors of the postganglionic nerve. The postganglionic nerve then releases ACh to stimulate the muscarinic receptors of the target organ.

  
The three main types of muscarinic receptors that are well characterised are:

  


  * The M1 muscarinic receptors are located in the neural system.
  * The M2 muscarinic receptors are located in the heart, and act to bring the heart back to normal after the actions of the sympathetic nervous system: slowing down the heart rate, reducing contractile forces of the atrial cardiac muscle, and reducing conduction velocity of the atrioventricular node (AV node). Note, they have no effect on the contractile forces of the ventricular muscle.
  * The M3 muscarinic receptors are located at many places in the body, such as the smooth muscles of the blood vessels, as well as the lungs, which means that they cause vasoconstriction and bronchoconstriction. They are also in the smooth muscles of the gastrointestinal tract (GIT), which help in increasing intestinal motility and dilating sphincters. The M3 receptors are also located in many glands that help to stimulate secretion in salivary glands and other glands of the body.

## Nervous Tissue

The nervous system coordinates the activity of the muscles, monitors the organs, constructs and also stops input from the senses, and initiates actions. Prominent participants in a nervous system include neurons and nerves, which play roles in such coordination.Our nervous tissue only consists of two types of cells. These cells are neurons and neuroglia cells. The neurons are responsible for transmitting nerve impulses. Neuroglia cells are responsible for supporting and nourishing the neuron cells.

### Types of Neurons

![Neuron.svg](../../../upload.wikimedia.org/wikipedia/commons/thumb/b/b5/Neuron.svg/400px-Neuron.svg.png)

There are three types of neurons in the body. We have sensory neurons, interneurons, and motor neurons. Neurons are a major class of cells in the nervous system. Neurons are sometimes called nerve cells, though this term is technically imprecise, as many neurons do not form nerves. In vertebrates, neurons are found in the brain, the spinal cord and in the nerves and ganglia of the peripheral nervous system. Their main role is to process and transmit information. Neurons have excitable membranes, which allow them to generate and propagate electrical impulses. Sensory neuron takes nerve impulses or messages right from the sensory receptor and delivers it to the central nervous system. A sensory receptor is a structure that can find any kind of change in it's surroundings or environment.

### Structure of a neuron

Neurons have three different parts to them. They all have an axon, a cell body and dendrites. The axon is the part of the neuron that conducts nerve impulses. Axons can get to be quite long. When an axon is present in nerves, it is called a nerve fiber. A cell body has a nucleous and it also has other organelles. The dendrites are the short pieces that come off of the cell body that receive the signals from sensory receptors and other neurons.

### Myelin Sheath

Schwann cells contain a lipid substance called myelin in their plasma membranes. When schwann cells wrap around axons, a myelin sheath forms. There are gaps that have no myelin sheath around them; these gaps are called nodes of Ranvier. Myelin sheathes make excellent insulators. Axons that are longer have a myelin sheath, while shorter axons do not. The disease multiple sclerosis is an autoimmune disease where the body attacks the myelin sheath of the central nervous system.

## Case Study

A 35-year-old male in 1986 had been admitted to a hospital in Florida three weeks previous to being diagnosed, with complaints of weakness and spasticity in the right leg, difficulties with balance, and fatigue and malaise. Tests performed at the Florida hospital had revealed abnormalities in spinal fluid and MRI brain scan. The patient complained of being severely depressed and anxious. He had anger at his circumstances and frequent crying spells. One month previously he had noticed aching and loss of vision in the left eye that had since improved.

This man was diagnosed with Multiple Sclerosis (MS). MS is a chronic, degenerative, and progressive disorder that affects the nerve fibers in the brain and spinal cord. Myelin is a fatty substance that surrounds and insulates the nerve fibers and facilitates the conduction of the nerve impulse transmissions. MS is characterized by intermittent damage to myelin (called demyelination) caused by the destruction of specialized cells (oligodendrocytes) that form the substance. Demyelination causes scarring and hardening (sclerosis) of nerve fibers usually in the spinal cord, brain stem, and optic nerves, which slows nerve impulses and results in weakness, numbness, pain, and vision loss. Because different nerves are affected at different times, MS symptoms often worsen (exacerbate), improve, and develop in different areas of the body. Early symptoms of the disorder may include vision changes (blurred vision, blind spots) and muscle weakness. MS can progress steadily or cause acute attacks (exacerbations) followed by partial or complete reduction in symptoms (remission). Most patients with the disease have a normal lifespan.

There are different types of MS
    Multiple sclerosis is classified according to frequency and severity of neurological symptoms, the ability of the CNS to recover, and the accumulation of damage.

![Types of MS.jpg](../../../upload.wikimedia.org/wikipedia/commons/b/b7/Types_of_MS.jpg)

### Treating Depression

Every now and then we all feel a little blue, these feelings can be caused by losing a loved one. Clinical depression goes much further than just feeling down. Depression has many symptoms, including lack of energy, abnormal eating habits (either too much or too little) and sleeping problems (also too much or too little). Often a person can feel worthless and have thoughts of committing suicide. The cause of depression and its symptoms are a mystery but we do understand that it is an illness associated with biochemical changes in the brain. A lot of research goes on to explain that it is associated with a lack of amines serotonin and norephinephrine. Therefore pharmacological treatment strategies often try to increase amine concentrations in the brain.

One class of antidepressants is monoamine oxidase inhibitors. Mono amine oxidase is a enzyme that breaks down your amines like norephinephrine and serotonin. Because the antidepressants inhibit their degradation they will remain in the synaptic cleft for a longer period of time making the effect just as if you had increased theses types of neurotransmitters.

A newer class of antidepressants is selective serotonin reuptake inhibitors (SSRI's). With SSRI's decreasing the uptake of serotonin back into the cell that will increase the amount of serotonin present in the synaptic cleft. SSRI's are more specific than the monoamine oxidase inhibitors because they only affect serotonergic synapses. You might recognize these SSRI's by name as Prozac and Paxil.

## Drugs

A drug is, generally speaking, any substance that changes the way your body works. Some drugs have a medicinal effect, and some are used recreationally. They have diverse effects, depending on the drug. Drugs can do anything from diminish pain, to preventing blood clots, to helping a depressed person.

Different drugs work in different ways, called the mechanism of action, the drugs covered here will all act on the nervous system via receptors on different neurons. There are also drugs that change how enzymes work, but that's not part of the nervous system (at least directly) and will not be discussed here.

You've probably heard the terms stimulant (excitatory) and depressant (inhibitory). This is a broad way of classifying drugs that work on the CNS. Depressants slow down neural function, and stimulants speed it up.

Most of the common depressants (including alcohol, benzodiazepines, barbiturates and GHB) work on GABA receptors, although there are others. Opiates, for example, work on mu opioid receptors and also produce inhibitory effects, and some antipsychotics block serotonin. See the alcohol section below to see one way this can work.

Stimulants work mostly with epinephrine, dopamine or serotonin (or a combination of them). Many of them either mimic one, or stop them from leaving the synapse, causing more action potentials to be fired. Methamphetamine, discussed below, is a fairly typical stimulant drug.

### Drug Abuse

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/b/bf/PCP.jpg/60px-PCP.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

framed

Scientists have long accepted that there is a biological basis for drug addiction, though the exact mechanisms responsible are only now being identified. It is believed that addictive substances create dependence in the user by changing the brain's reward functions, located in the mesolimbic dopamine system—the part of the brain that reinforces certain behaviors such as eating, sexual intercourse, exercise, and social interaction. Addictive substances, through various means and to different degrees, cause the synapses of this system to flood with excessive amounts of dopamine, creating a brief rush of euphoria more commonly called a "high”. Some say that abuse begins when the user begins shirking responsibility in order to afford drugs or to have enough time to use them. Some say it begins when a person uses "excessive" amounts, while others draw the line at the point of legality, and others believe it amounts to chronic use despite degenerating mental and physical health in the user. Some think that any intoxicant consumption is an inappropriate activity. Here are some drugs that are abused frequently: Acid/LSD, Alcohol, various tryptamines and phenethylamines, Cocaine, Ecstasy/MDMA, Heroin, Inhalants, Marijuana, Methamphetamine, PCP/Phencyclidine, Prescription Medications, Smoking/Nicotine and Steroids.

### Alcohol

Alcohol is, and has been for thousands of years, one of the most commonly used drugs in the world. It is legal, with some restrictions and exceptions, nearly everywhere. It is a common misconception that somehow alcohol is 'better' or 'safer' than other recreational drugs. This is simply NOT the case. Alcohol is a depressant, and as such it has the potential to cause coma, respiratory depression/arrest and possibly death. Compared with some other (illegal in most places) drugs of recreational value (such as marijuana, serotonin based hallucinogens like LSD or psilocybin) alcohol is far more toxic and has more risk of overdose. That doesn't mean that moderate drinking will probably hurt you, though, either.

Short term effects from drinking (listed roughly as they appear, and as dosage goes up) are: decreased inhibitions and thusly judgment, flushing of the face, drowsiness, memory problems begin, severe motor impairment, blurry vision, dizziness, confusion, nausea, possible unconsciousness, coma, death (due to respiratory arrest or possibly aspiration on vomit).

Alcohol produces these effects mainly via the GABA receptors in the brain. When GABA (or in this case alcohol) binds to it's receptor, it lets either Cl- ions in, or K+ out. This is called hyperpolarization, or an inhibitory postsynaptic potential (IPSP). It makes it harder for the neuron to depolarize and hence harder for it to fire an action potential, slowing neural function. At higher doses alcohol will start to block NMDA. NMDA is involved in memory (see the long-term potentiation section) so this is thought to account for memory blackouts.

### Methamphetamine

In the US, medically prescribed **methamphetamine** is distributed in tablet form under the brand name Desoxyn®, generally for Attention Deficit Hyperactivity Disorder (ADHD) but also for narcolepsy or obesity.

Illicit methamphetamine comes in a variety of forms. Most commonly it is found as a colorless crystalline solid, sold on the street under a variety of names, such as: crystal meth or crystal. Methamphetamine may also be referred to as shards, rock, pony, crissie, crystal, glass, ice, Jib, critter, Tina, tweak or crank. Dope may refer to methamphetamine or other drugs, especially heroin or marijuana. The term "speed" can denote any stimulant including other amphetamines (e.g. adderall), cocaine and methylphenidate (Ritalin).

Methamphetamine can be injected (either subcutaneous, intramuscular or intravenous), smoked, snorted, swallowed, or used rectally or sublingually. The latter two being fairly uncommon. After administration, methamphetamine takes from a few seconds (smoked or injected IV) to around 30 minutes (oral) for effects to arise, lasting around eight hours depending on the route of administration. Effects/side effects include euphoria, anorexia, increased energy, clenching of the jaw/grinding of teeth (bruxism), weight loss, insomnia, tooth decay and psychosis among others.

Methamphetamine is neurotoxic to at least some areas of the brain, and owes most of it's effects to the neurotransmitters dopamine, norepinephrine and serotonin it releases. It also blocks the reuptake of those neurotransmitters, causing them to stay in the synaptic cleft longer than normal.

### Marijuana

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/e/e1/Cannabis_sativa.jpg/250px-Cannabis_sativa.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Cannabis sativa.

Marijuana contains a myriad of chemicals, called cannabinoids, that have psychoactive and medicinal effects when consumed, the major one being tetrahydrocannabinol (THC). THC serves to mimic the endogenous neurotransmitter anandamide (also found in chocolate) at the CB1 receptors in the brain. Other cannabinoids include Cannabidiol (CBD), cannabinol (CBN) and tetrahydrocannabivarin (THCV). Although THC is found in all parts of the plant, the flower of the female plant has the highest concentration, commonly around eight percent. The flowers can be used, or they can be refined. Trichomes contain most of the THC on the flowers and can be removed by a few different methods. These removed trichomes are called kief. Kief can, in turn, be pressed into hashish. By far the most common way to consume any of these products is by smoking, but it can be taken orally as well.

Cannabis has a very long, very good safety record. Nobody on record has ever died because of marijuana, directly at least. It is estimated that it would take 1-1.8 kilograms of average potency marijuana, taken orally, to have a fifty percent chance of killing a 68kg human. Despite this, the possession, use, or sale of psychoactive cannabis products became illegal in many parts of the world in the early 20th century. Since then, while some countries have intensified the enforcement of cannabis prohibition, others have reduced the priority of enforcement to the point of de facto legality. Cannabis remains illegal in the vast majority of the world's countries.

The nature and intensity of the immediate effects of cannabis consumption vary according to the dose, the species or hybridization of the source plant, the method of consumption, the user's mental and physical characteristics (such as possible tolerance), and the environment of consumption. This is sometimes referred to as set and setting. Smoking the same cannabis either in a different frame of mind (set) or in a different location (setting) can alter the effects or perception of the effects by the individual. Effects of cannabis consumption may be loosely classified as cognitive and physical. Anecdotal evidence suggests that the Cannabis sativa species tends to produce more of the cognitive or perceptual effects, while Cannabis indica tends to produce more of the physical effects.

## Review Questions

Answers for these questions can be found [here](http://en.wikibooks.org/wiki/Human_Physiology/Appendix_1:_answers_to_review_questions#The_Nervous_System)

1\. The junction between one neuron and the next, or between a neuron and an effector is called:

    A ) A synapse
    B ) A dendrite
    C ) A neuotransmitter
    D ) A ventricle
    E ) None of the above

2\. A fast excitatory synapses follows this order:

    A ) (1) neurotransmitter released (2) diffused across the synaptic cleft to a receptor protein (3) binding of the transmitter opens pores in the ion channels and positive ions move in.
    B ) (1) neurotransmitter released (2) diffused across the synaptic cleft to a receptor protein (3) binding of the transmitter opens pores in the ion channels and negative ions move in.
    C ) (1) neurotransmitter released (2) diffused across the synaptic cleft to a receptor amino acid (3) binding of the transmitter opens pores in the ion channels and positive ions move in.
    D ) (1) diffused across the synaptic cleft to a receptor protein (2) neurotransmitter released (3) binding of the transmitter opens pores in the ion channels and positive ions move in.
    E ) None of the above

3\. Resting potential is

    A ) excess positive ions accumulate inside the plasma membrane
    B ) excess negative ions accumulate inside the plasma membrane
    C ) excess positive ions accumulate outside the plasma membrane
    D ) both b & c
    E ) both a & c

4\. Sensory neurons have:

    A ) A short dendrite and a long axon
    B ) A short dendrite and a short axon
    C ) A long dendrite and a short axon
    D ) A long dendrite and a long axon
    E ) Their axons and dendrites may be either long or short

5\. ________blocks Acetylcholine receptor sites causing muscle relaxation.

    A ) Novocain
    B ) curare
    C ) Nicotine
    D ) Nerve gases

6\. Transmission across a synapse is dependent on the release of _______?

    A ) neurotransmitters
    B ) synaptic vesicle
    C ) neuromuscular tisssue
    D ) receptor proteins

7\. Motor neurons take messages

    A ) from the muscle fiber to the central nervous system
    B ) away from the central nervous system to the central nervous system
    C ) that are classified
    D ) away from the central nervous system to muscle fiber

8\. The medulla oblongata helps to regulate which of the following:

    A ) Breathing
    B ) Heartbeat
    C ) Sneezing
    D ) Vomiting
    E ) All of the above

9\. The nervous systems main components are what?

    A) The Synapses and Sprinal cord
    B) The neurons and the synapses
    C) The bain and the neurons
    D)The brain and the spinal cord

10\. Explain what LTP does to enhance communication between two neurons, on the postsynaptic end.

11\. Explain what LTP does to enhance communication between two neurons, on the presynaptic end.

## Glossary

**Afferent Messages:** carry sensations such as heat, cold, or pain

**Autonomic System:** deals with the visceral organs, like the heart, stomach, gland, and the intestines

**Axon:** the part of the neuron that conducts nerve impulses

**Cannabis:** a psychoactive drug produced from parts of the cannabis plant

**Central Nervous System (CNS):** the system that includes the brain and the spinal cord

**Cerebellum:** part of the brain that is located posterior to the medulla oblongata and pons, coordinates skeletal muscles to produce smooth, graceful motions

**Cerebrospinal Fluid (CSF):** acts a shock absorber for the central nervous system, protecting the brain and spinal cord from injury; it also has a high glucose content which serves as a nutritional factor

**Cerebrum** motor control, learning, speech, somatic sensory functions, vision,hearing and more.

**Dendrites:** short pieces that come off of the cell body that receive the signals from sensory receptors and other neurons

**Episodic Memory:** represents our memory of events and experiences in a serial form

**Excitatory Neurotransmitter:** a neurotransmitter that acts to elicit an action potential by opening sodium ion channels

**Longitudinal Sulcus:** separates the cerebrum in to the right and left hemispheres

**Long Term Memory:** used for storage of information over a long time

**Long-Term Potentiation (LTP)** long term communication enhancement between two neurons. Results in neural pathways that store memoris.

**Medulla** control center for respiratory, cardiovascular and digestive functions.

**Myelin:** a fatty substance that surrounds and insulates the nerve fibers and facilitates the conduction of the nerve impulse transmissions

**Multiple Sclerosis (MS):** disease that affects the CNS by causing hardening and scaring of the myelin

**Nodes of Ranvier:** unmyelinated gaps between sections of myelin

**Peripheral Nervous System (PNS):** a way of communication from the central nervous system to the rest of the body by nerve impulses that regulate the functions of the human body

**Pons** control centers for respiration and inhibitory functions.

**Postganglionic Cells:** have their cell bodies in the ganglia and send their axons to target organs or glands

**Postsynaptic Cells** the cell on the receiving (second) end of the synapse.

**Presynaptic Cell** The cell on the sending (first) end of the synapse.

**Proprioception** the sense that indicates whether the body is moving with required effort, as well as where various parts of the body are located in relation to each other.

**Sensory Receptor:** structure that can find any kind of change in it's surroundings or environment

**Somatic Nervous System (SNS):** the part of the peripheral nervous system associated with the voluntary control of body movements through the action of skeletal muscles, and also reception of external stimuli

**Synapses:** the gap between two neurons; new synapses lead to learning

## References

[http://action.painfoundation.org/site/News2?page=NewsArticle&id=5135&security=1&news_iv_ctrl=1061](http://action.painfoundation.org/site/News2?page=NewsArticle&id=5135&security=1&news_iv_ctrl=1061) Esther Wednesday, October 19, 2005

<http://www.neurologychannel.com/multiplesclerosis/>

<http://www.theraj.com/ms/casestudy.html>  


# The Senses

## What are Senses?

We experience reality through our senses. Senses are the physiological methods of perception, so a sense is a faculty by which outside stimuli are perceived. The senses and their operation, classification, and theory are overlapping topics studied by a variety of fields. Many neurologists disagree about how many senses there actually are due to a broad interpretation of the definition of a sense. Our senses are split into two different groups. Our **exteroceptors** detect stimulation from the outsides of our body. For example smell, taste, and equilibrium. The **interoceptors** receive stimulation from the inside of our bodies. For instance, blood pressure dropping, changes in the glucose and pH levels. Children are generally taught that there are five senses (sight, hearing, touch, smell, taste). However, it is generally agreed that there are at least seven different senses in humans, and a minimum of two more observed in other organisms. Sense can also differ from one person to the next. Take taste for an example: what may taste great to one person will taste awful to someone else. This has to do with how the brain interprets the stimuli that are received.

## Chemoreception

The senses of **gustation** (taste) and **olfaction** (smell) fall under the category of **chemoreception**. Specialized cells act as receptors for certain chemical compounds. As these compounds react with the receptors, an impulse is sent to the brain and is registered as a certain taste or smell. Gustation and olfaction are chemical senses because the receptors they contain are sensitive to the molecules in the food we eat, along with the air we breathe.

### Gustatory System

In humans, the sense of **taste** is transduced by **taste buds** and is conveyed via three of the twelve cranial nerves. Cranial nerve VII, the facial nerve, carries taste sensations from the anterior two thirds of the tongue (excluding the circumvallate papillae, see lingual papilla) and soft palate. Cranial nerve IX the glossopharyngeal nerve carries taste sensations from the posterior one third of the tongue (including the circumvallate papillae). Also a branch of the vagus nerve carries some taste sensations from the back of the oral cavity (i.e. pharynx and epiglottis). Information from these cranial nerves is processed by the gustatory system. Though there are small differences in sensation, which can be measured with highly specific instruments, all taste buds can respond to all types of taste. Sensitivity to all tastes is distributed across the whole tongue and indeed to other regions of the mouth where there are taste buds (epiglottis, soft palate).

#### Papilla

**Papilla** are specialized epithelial cells. There are four types of papillae: **filiform** (thread-shape), **fungiform** (mushroom-shape), **foliate** (leaf-shape), and **circumvallate** (ringed-circle). All papillae except the filiform have taste buds on their surface. Some act directly by ion channels, others act indirectly.

  * **Fungiform papillae** \- as the name suggests, are slightly mushroom shaped if looked at in section. These are present mostly at the apex (tip) of the tongue.
  * **Filiform papillae** \- these are thin, longer papillae that don't contain taste buds but are the most numerous. These papillae are mechanical and not involved in gustation.
  * **Foliate papillae** \- these are ridges and grooves towards the posterior part of the tongue.
  * **Circumvallate papillae** \- there are only about 3-14 of these papillae on most people and they are present at the back of the oral part of the tongue. They are arranged in a circular-shaped row just in front of the sulcus terminalis of the tongue.

#### Structure of Taste Buds

![](../../../upload.wikimedia.org/wikipedia/commons/6/6e/Gray1014.png)

The mouth cavity. The cheeks have been slit transversely and the tongue pulled forward.

![](../../../upload.wikimedia.org/wikipedia/commons/5/5a/Gray1018.png)

Semidiagrammatic view of a portion of the mucous membrane of the tongue. Two fungiform papillæ are shown. On some of the filiform papillæ the epithelial prolongations stand erect, in one they are spread out, and in three they are folded in.

Each taste bud is flask-like in shape, its broad base resting on the corium, and its neck opening by an orifice, the gustatory pore, between the cells of the epithelium.

The bud is formed by two kinds of cells: supporting cells and gustatory cells.

The supporting cells are mostly arranged like the staves of a cask, and form an outer envelope for the bud. Some, however, are found in the interior of the bud between the gustatory cells. The gustatory cells occupy the central portion of the bud; they are spindle-shaped, and each possesses a large spherical nucleus near the middle of the cell. The peripheral end of the cell terminates at the gustatory pore in a fine hair-like filament, the gustatory hair.

The central process passes toward the deep extremity of the bud, and there ends in single or bifurcated varicosities.

The nerve fibrils after losing their medullary sheaths enter the taste bud, and end in fine extremities between the gustatory cells; other nerve fibrils ramify between the supporting cells and terminate in fine extremities; these, however, are believed to be nerves of ordinary sensation and not gustatory.

  


#### Types of Taste

Salt
    Arguably the simplest receptor found in the mouth is the salt (NaCl) receptor. An ion channel in the taste cell wall allows Na+ ions to enter the cell. This on its own depolarizes the cell, and opens voltage-regulated Ca2+ gates, flooding the cell with ions and leading to neurotransmitter release. This sodium channel is known as EnAC and is composed of three subunits. EnAC can be blocked by the drug amiloride in many mammals, especially rats. The sensitivity of the salt taste to amiloride in humans, however, is much less pronounced, leading to conjecture that there may be additional receptor proteins besides EnAC that may not have been discovered yet.

Sour
    Sour taste signals the presence of acidic compounds (H+ ions in solution). There are three different receptor proteins at work in sour taste. The first is a simple ion channel which allows hydrogen ions to flow directly into the cell. The protein for this is EnAC, the same protein involved in the distinction of salt taste (this implies a relationship between salt and sour receptors and could explain why salty taste is reduced when a sour taste is present). There are also H+ gated channels present. The first is a K+ channel, which ordinarily allows K+ ions to escape from the cell. H+ ions block these, trapping the potassium ions inside the cell (this receptor is classified as MDEG1 of the EnAC/Deg Family). A third protein opens to Na+ ions when a hydrogen ion attaches to it, allowing the sodium ions to flow down the concentration gradient into the cell. The influx of ions leads to the opening of a voltage regulated Ca2+ gate. These receptors work together and lead to depolarization of the cell and neurotransmitter release.

Bitter
    There are many classes of bitter compounds which can be chemically very different. It is interesting that the human body has evolved a very sophisticated sense for bitter substances: we can distinguish between the many radically different compounds which produce a generally “bitter” response. This may be because the sense of bitter taste is so important to survival, as ingesting a bitter compound may lead to injury or death. Bitter compounds act through structures in the taste cell walls called G-protein coupled receptors (GPCR’s). Recently, a new group of GPCR’s was discovered, known as the T2R’s, which is thought to only respond to bitter stimuli. When the bitter compound activates the GPCR, it in turn releases gustducin, the G-protein it was coupled to. Gustducin is made of three subunits. When it is activated by the GPCR, its subunits break apart and activate phosphodiesterase, a nearby enzyme. It then converts a precursor within the cell into a secondary messenger, which closes potassium ion channels. This secondary messenger can stimulate the endoplasmic reticulum to release Ca2+, which contributes to depolarization. This leads to a build-up of potassium ions in the cell, depolarization, and neurotransmitter release. It is also possible for some bitter tastants to interact directly with the G-protein, because of a structural similarity to the relevant GPCR.

Sweet
    Like bitter tastes, sweet taste transduction involves GPCR’s. The specific mechanism depends on the specific molecule. “Natural” sweeteners such as saccharides activate the GPCR, which releases gustducin. The gustducin then activates the molecule adenylate cyclase, which is already inside the cell. This molecule increases concentration of the molecule cAMP, or adenosine 3', 5'-cyclic monophosphate. This protein will either directly or indirectly close potassium ion channels, leading to depolarization and neurotransmitter release. Synthetic sweeteners such as saccharin activate different GPCR’s, initiating a similar process of protein transitions, starting with the protein phospholipase A, which ultimately leads to the blocking of potassium ion channels.

Umami
    Umami is a Japanese word meaning "savory" or "meaty". It is thought that umami receptors act much the same way as bitter and sweet receptors (they involve GPCR’s), but not much is known about their specific function. We do know that umami detects glutamates that are common in meats, cheese and other protein-heavy foods. Umami receptors react to foods treated with monosodium glutamate (MSG). This explains why eating foods that have MSG in them often give a sense of fullness. It is thought that the amino acid L-glutamate bonds to a type of GPCR known as a metabotropic glutamate receptor (mGluR4). This causes the G-protein complex to activate a secondary receptor, which ultimately leads to neurotransmitter release. The intermediate steps are not known.

#### Disorders of the Tongue

    **Loss of taste**
    You may lose your sense of taste if the facial nerve is damaged. Then there is also Sjogren's Syndrome where the saliva production is reduced. In most cases the loss of taste is typically a symptom of **anosmia** \- a loss of the sense of smell.

    **Sore tongue**
    It is usually caused by some form of trauma, such as biting your tongue, or eating piping-hot or highly acidic food or drink.
    If your top and bottom teeth don’t fit neatly together, tongue trauma is more likely.
    Some people may experience a sore tongue from grinding their teeth **(bruxism).**
    Disorders such as diabetes, anemia, some types of vitamin deficiency and certain skin diseases can include a sore tongue among the range of symptoms.

    **Glossodynia**
    A condition characterized by a burning sensation on the tongue.

    **Benign migratory glossitis**
    This condition is characterized by irregular and inflamed patches on the tongue surface that often have white borders. The tongue may be generally swollen, red and sore. Another name for this condition is geographic tongue. The cause of benign migratory glossitis is unknown.
    **Risk factors are thought to include**: 

  * Mineral or vitamin deficiencies
  * Local irritants, such as strong mouthwashes, cigarettes or alcohol
  * Certain forms of anemia
  * Infection
  * Certain medications
  * Stress

### Olfactory System

**Olfaction** is the sense of smell. In humans the sence of **Smell** is received in **nasopharynx**. Airborne molecules go into solution on moist epithelial surface of nasal passage. An olfactory receptors neuron sends an impulse via Cranial nerve I the olfactory nerve. Although 80-90% of what we think is "taste" actually is due to smell. This is why when we have a head cold or stuffed up nose we have a harder time tasting our foods.

  


#### Receptors

Humans have 347 functional odor receptor genes; the other genes have nonsense mutations. This number was determined by analyzing the genome in the Human Genome Project; the number may vary among ethnic groups, and does vary among individuals. For example, not all people can smell androstenone, a component of male sweat.

Each olfactory receptor neuron in the nose expresses only one functional odor receptor. Odor receptor nerve cells may function like a key-lock system: if the odor molecules can fit into the lock the nerve cell will respond. According to shape theory, each receptor detects a feature of the odor molecule. Weak-shape theory, known as odotope theory, suggests that different receptors detect only small pieces of molecules, and these minimal inputs are combined to create a larger olfactory perception (similar to the way visual perception is built up of smaller, information-poor sensations, combined and refined to create a detailed overall perception). An alternative theory, the vibration theory proposed by Luca Turin (1996, 2002), posits that odor receptors detect the frequencies of vibrations of odor molecules in the infrared range by electron tunneling. However, the behavioral predictions of this theory have been found lacking (Keller and Vosshall, 2004).

An olfactory receptor neuron, also called an olfactory sensory neuron, is the primary transduction cell in the olfactory system. Humans have about 40 million olfactory receptor neurons. In vertebrates, olfactory receptor neurons reside on the olfactory epithelium in the nasal cavity. These cells are bipolar neurons with a dendrite facing the interior space of the nasal cavity and an axon that travels along the olfactory nerve to the olfactory bulb.

Many tiny hair-like cilia protrude from the olfactory receptor cell's dendrite and into the mucus covering the surface of the olfactory epithelium. These cilia contain olfactory receptors, a type of G protein-coupled receptor. Each olfactory receptor cell contains only one type of olfactory receptor, but many separate olfactory receptor cells contain the same type of olfactory receptor. The axons of olfactory receptor cells of the same type converge to form glomeruli in the olfactory bulb.

Olfactory receptors can bind to a variety of odor molecules. The activated olfactory receptor in turn activates the intracellular G-protein GOLF, and adenylate cyclase and production of Cyclic AMP opens ion channels in the cell membrane, resulting in an influx of sodium and calcium ions into the cell. This influx of positive ions causes the neuron to depolarize, generating an action potential.

Individual olfactory receptor neurons are replaced approximately every 40 days by neural stem cells residing in the olfactory epithelium. The regeneration of olfactory receptor cells, as one of the only few instances of adult neurogenesis in the central nervous system, has raised considerable interest in dissecting the pathways for neural development and differentiation in adult organisms.

#### In the brain

The axons from all the thousands of cells expressing the same odor receptor converge in the olfactory bulb. Mitral cells in the olfactory bulb send the information about the individual features to other parts of the olfactory system in the brain, which puts together the features into a representation of the odor. Since most odor molecules have many individual features, the combination of features gives the olfactory system a broad range of odors that it can detect.

Odor information is easily stored in long term memory and has strong connections to emotional memory. This is possibly due to the olfactory system's close anatomical ties to the limbic system and hippocampus, areas of the brain that have long been known to be involved in emotion and place memory, respectively.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/3/3a/Head_olfactory_nerve.jpg/250px-Head_olfactory_nerve.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

The Olfactory Nerve leading to the brain.

#### Pheromonal olfaction

Some pheromones are detected by the olfactory system, although in many vertebrates pheromones are also detected by the vomeronasal organ, located in the vomer, between the nose and the mouth. Snakes use it to smell prey, sticking their tongue out and touching it to the organ. Some mammals make a face called flehmen to direct air to this organ. In humans, it is unknown whether or not pheromones exist.

#### Olfaction and Gustation

Olfaction, taste and trigeminal receptors together contribute to flavor. It should be emphasized that there are no more than 5 distinctive tastes: salty, sour, sweet, bitter, and umami. The 10,000 different scents which humans usually recognize as 'tastes' are often lost or severely diminished with the loss of olfaction. This is the reason why food has little flavor when your nose is blocked, as from a cold.

The key nutrition players in our taste is the olfactory function, 80-90% of what we consider taste is dependent on our senses of smell. With aging our olfactory function declines. In the elderly careful monitoring of appetite is necessary due to the alterations in the olfactory function. Nutritionist suggest giving a dual approach of supplementation of the trace minerals zinc and iron to enhance the smell and taste senses.

#### Disorders of Olfaction

Anosmia
    Anosmia is the lack of olfaction, or a loss of the sense of smell. It can be either temporary or permanent. A related term, hyposmia refers to a decrease in the ability to smell. Some people may be anosmic for one particular odor. This is called "specific anosmia" and may be genetically based. Anosmia can have a number of detrimental effects. Patients with anosmia may find food less appetizing. Loss of smell can also be dangerous because it hinders the detection of gas leaks, fire, body odor, and spoiled food. The common view of anosmia as trivial can make it more difficult for a patient to receive the same types of medical aid as someone who has lost other senses, such as hearing or sight. A temporary loss of smell can be caused by a stuffy nose or infection. In contrast, a permanent loss of smell may be caused by death of olfactory receptor neurons in the nose, or by brain injury in which there is damage to the olfactory nerve or damage to brain areas that process smell. The lack of the sense of smell at birth, usually due to genetic factors, is referred as congenital anosmia. Anosmia may be an early sign of degenerative brain diseases such as Parkinson's disease and Alzheimer's disease. Another specific cause of permanent loss could be from damage to olfactory receptor neurons due to use of nasal sprays. To avoid loss of smell from nasal sprays, use them for only a short amount of time. Nasal sprays that are used to treat allergy related congestion are the only nasal sprays that are safe to use for extended periods of time.

Phantosmia
    Phantosmia is the phenomenon of smelling odors that aren't really present. (AKA Phantom odors) The most common odors are unpleasant smells such as rotting flesh, vomit, feces, smoke etc. Phantosmia often results from damage to the nervous tissue in the olfactory system. The damage can be caused by viral infection, trauma, surgery, and possibly exposure to toxins or drugs. It can also be induced by epilepsy affecting the olfactory cortex. It is also thought the condition can have psychiatric origins.

Dysosmia
    When things smell differently than they should.

## The Sense of Vision

**Vision** needs to have the work of both the eyes and the brain to process any information. The majority of the stimuli is done in the eyes and then the information is sent to the brain by the way of nerve impulses. At least one-third of the information of what the eye sees is processed in the cerebral cortex of the brain.

### Anatomy of the Eye

The human eye is a elongated ball about 1-inch (2.5 cm) in diameter and is protected by a bony socket in the skull. The eye has three layers or coats that make up the exterior wall of the eyeball, which are the sclera, choroid, and retina.

Sclera
    The outer layer of the eye is the sclera, which is a tough white fibrous layer that maintains, protects and supports the shape of the eye. The front of the sclera is transparent and is called the cornea. The cornea refracts light rays and acts like the outer window of the eye.

Choroid
    The middle thin layer of the eye is the choroid, also known as the choroidea or choroid coat, it is the vascular layer of the eye lying between the retina and the sclera. The choroid provides oxygen and nourishment to the outer layers of the retina. It also contains a nonreflective pigment that acts as a light shield and prevents light from scattering. Light enters the front of the eye through a hole in the choroid coat called the pupil. The iris contracts and dilates to compensate for the changes in light intensity. If the light is bright the iris then contracts making the pupil smaller, and if the light is dim, the iris dilates making the pupil bigger. Just posterior to the iris is the lens, which is composed mainly of proteins called crystallins. The lens is attached by the zonules to the ciliary body that contains the ciliary muscles that control the shape of the lens for accommodation. Along with the ciliary body and iris, the choroid forms the uveal tract. The uvea is the middle of the three concentric layers that make up an eye. The name is possibly a reference to its almost black color, wrinkled appearance and grape-like size and shape when stripped intact from a cadaveric eye.

Retina
    

![](../../../upload.wikimedia.org/wikipedia/commons/5/5b/Blindspot.jpg)

Illustration of the "blind spot." Situate your head about one foot from the monitor. Close your right eye and look at the dot on the right with your left eye. Move your head slowly closer. When you get to the correct spot, the dot on the left will disappear.

The third or the innermost layer of the eye is call the retina. In adult humans the entire retina is 72% of a sphere about 22 mm in diameter. The retina lays over the back two thirds of the choroid coat, which is located in the posterior compartment. The compartment is filled with vitreous humor which is a clear, gelatinous material. Within the retina there are cells called rod cells and cone cells also known as photoreceptors. The rod cells are very sensitive to light and do not see color, that is why when we are in a darkened room we see only shades of gray. The cone cells are sensitive to different wavelengths of light, and that is how we are able to tell different colors. It is a lack of cones sensitive to red, blue, or green light that causes individuals to have deficiencies in color vision or various kinds of color blindness. At the center of the retina is the optic disc, sometimes known as "the blind spot" because it lacks photoreceptors. It is where the optic nerve leaves the eye and takes the nerve impulses to the brain. The cornea and the lens of the eye focuses the light onto a small area of the retina called the **fovea centralis** where the cone cells are densely packed. The fovea is a pit that has the highest visual acuity and is responsible for our sharp central vision - there are no rods in the fovea.

![Fig retine.png](../../../upload.wikimedia.org/wikipedia/commons/2/21/Fig_retine.png)

_Retina's simplified axial organization. The retina is a stack of several neuronal layers. Light is concentrated from the eye and passes across these layers (from left to right) to hit the photoreceptors (right layer). This elicits chemical transformation mediating a propagation of signal to the bipolar and horizontal cells (middle yellow layer). The signal is then propagated to the amacrine and ganglion cells. These neurons ultimately may produce action potentials on their axons. This spatiotemporal pattern of spikes determines the raw input from the eyes to the brain._

Photoreceptors
    A photoreceptor, or photoreceptor cell, is a specialized type of neuron found in the eye's retina that is capable of phototransduction. More specifically, the photoreceptor sends signals to other neurons by a change in its membrane potential when it absorbs photons. Eventually, this information will be used by the visual system to form a complete representation of the visual world. There are 2 types of photoreceptors: **rods** are responsible for scotopic, or night vision, whereas **cones** are responsible for photopic, or daytime vision as well as color perception.

Extraocular muscles
    Each eye has six muscles that control its movements: the lateral rectus, the medial rectus, the inferior rectus, the superior rectus, the inferior oblique, and the superior oblique. When the muscles exert different tensions, a torque is exerted on the globe that causes it to turn. This is an almost pure rotation, with only about one millimeter of translation, thus, the eye can be considered as undergoing rotations about a single point in the center of the eye. Five of the extraocular muscles have their origin in the back of the orbit in a fibrous ring called the **annulus of Zinn**. Four of these then course forward through the orbit and insert onto the globe on its anterior half (i.e., in front of the eye's equator). These muscles are named after their straight paths, and are called the four rectus muscles, or four recti. They insert on the globe at 12, 3, 6, and 9 o'clock, and are called the superior, lateral, inferior and medial rectus muscles. (Note that lateral and medial are relative to the subject, with lateral toward the side and medial toward the midline, thus the medial rectus is the muscle closest to the nose).

### Eye Movement

The visual system in the brain is too slow to process that information if the images are slipping across the retina at more than a few degrees per second, thus, for humans to be able to see while moving, the brain must compensate for the motion of the head by turning the eyes. To get a clear view of the world, the brain must turn the eyes so that the image of the object of regard falls on the fovea. Eye movements are thus very important for visual perception, and any failure to make them correctly can lead to serious visual disabilities. Having two eyes is an added complication, because the brain must point both of them accurately enough that the object of regard falls on corresponding points of the two retinas; otherwise, double vision would occur. The movements of different body parts are controlled by striated muscles acting around joints. The movements of the eye are no exception, but they have special advantages not shared by skeletal muscles and joints, and so are considerably different.

Try This Experiment
    Hold your hand up, about one foot (30 cm) in front of your nose. Keep your head still, and shake your hand from side to side, slowly at first, and then faster and faster. At first you will be able to see your fingers quite clearly. But as the frequency of shaking passes about one hertz, the fingers will become a blur. Now, keep your hand still, and shake your head (up and down or left and right). No matter how fast you shake your head, the image of your fingers remains clear. This demonstrates that the brain can move the eyes opposite to head motion much better than it can follow, or pursue, a hand movement. When your pursuit system fails to keep up with the moving hand, images slip on the retina and you see a blurred hand.

### How we see an object

  * The light rays enter the eye through the cornea (transparent front portion of eye to focus the light rays)
  * Then, light rays move through the pupil, which is surrounded by Iris to keep out extra light
  * Then, light rays move through the crystalline lens (Clear lens to further focus the light rays )
  * Then, light rays move through the vitreous humor (clear jelly like substance)
  * Then, light rays fall on the retina, which processes and converts incident light to neuron signals using special pigments in rod and cone cells.
  * These neuron signals are transmitted through the optic nerve,
  * Then, the neuron signals move through the visual pathway - Optic nerve > Optic Chiasm > Optic Tract > Optic Radiations > Cortex
  * Then, the neuron signals reach the occipital (visual) cortex and its radiations for the brain's processing.
  * The visual cortex interprets the signals as images and along with other parts of the brain, interpret the images to extract form, meaning, memory and context of the images.

### Depth Perception

Depth perception is the visual ability to perceive the world in three dimensions. It is a trait common to many higher animals. Depth perception allows the beholder to accurately gauge the distance to an object.

Depth perception is often confused with binocular vision, also known as Stereopsis. Depth perception does rely on binocular vision, but it also uses many other monocular cues.

### Diseases, disorders, and age-related changes

There are many diseases, disorders, and age-related changes that may affect the eyes and surrounding structures. As the eye ages certain changes occur that can be attributed solely to the aging process. Most of these anatomic and physiologic processes follow a gradual decline. With aging, the quality of vision worsens due to reasons independent of aging eye diseases. While there are many changes of significance in the non-diseased eye, the most functionally important changes seem to be a reduction in pupil size and the loss of accommodation or focusing capability (presbyopia). The area of the pupil governs the amount of light that can reach the retina. The extent to which the pupil dilates also decreases with age. Because of the smaller pupil size, older eyes receive much less light at the retina. In comparison to younger people, it is as though older persons wear medium-density sunglasses in bright light and extremely dark glasses in dim light. Therefore, for any detailed visually guided tasks on which performance varies with illumination, older persons require extra lighting.

[File:Colorblind3.png](http://commons.wikimedia.org/wiki/Special:UploadWizard?wpDestFile=Colorblind3.png)

This image contains a two digit number similar to the sample above. Someone who is protanopic might not see this number.

Color Blindness
    Color Blindness or color vision deficiency, in humans is the inability to perceive differences between some or all colors that other people can distinguish. It is most often of genetic nature, but may also occur because of eye, nerve, or brain damage, or due to exposure to certain chemicals. There are many types of color blindness. The most common variety are hereditary (genetic) photoreceptor disorders, but it is also possible to acquire color blindness through damage to the retina, optic nerve, or higher brain areas. There is generally no treatment to cure color deficiencies, however, certain types of tinted filters and contact lenses may help an individual to distinguish different colors better.

Night Blindness
    Also known as Nyctalopia, is a condition making it difficult or impossible to see in the dark. It is a symptom of several eye diseases. Night blindness may exist from birth, or be caused by injury or malnutrition (for example, a lack of vitamin A). The most common cause of nyctalopia is retinitis pigmentosa, a disorder in which the rod cells in the retina gradually lose their ability to respond to the light. Patients suffering from this genetic condition have progressive nyctalopia and eventually their day-time vision may also be affected. In congenital stationary night blindness the rods do not work from birth, but as the name implies, sufferers do not get worse. Another cause of night blindness is a deficiency of retinol, or vitamin A, found in fish oils, liver and dairy products.

Day Blindness
    Also known as Hemeralopia is the inability to see clearly in bright light. The daytime vision gets worse and worse. Nighttime vision remains unchanged due to the use of rods as opposed to cones (during the day), which get affected by hemeralopia and in turn degrade the daytime optical response.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/2/2c/Floaters.jpg/110px-Floaters.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Impression of floaters, as seen against a blue sky.

Floater
    Also known as "Muscae Volitantes" are deposits of various size, shape, consistency, refractive index, and motility within the eye's normally transparent vitreous humour. Floaters are suspended in the vitreous humour, the thick fluid or gel that fills the eye. Thus, they generally follow the rapid motions of the eye, while drifting slowly within the fluid. Floaters are visible only because they do not remain perfectly fixed within the eye. The shapes are shadows projected onto the retina by tiny structures of protein or other cell debris discarded over the years and trapped in the vitreous humour. They are also common after cataract operations or after trauma. In some cases, floaters are congenital.

![](../../../upload.wikimedia.org/wikipedia/commons/5/50/Human_eyesight_two_children_and_ball_with_glaucoma.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Picture of children holding a ball as seen by someone with glaucoma.

Glaucoma
    A group of diseases of the optic nerve involving loss of retinal ganglion cells in a characteristic pattern of optic neuropathy. Although raised intraocular pressure is a significant risk factor for developing glaucoma, there is no set threshold for intraocular pressure that causes glaucoma. One person may develop nerve damage at a relatively low pressure, while another person may have high eye pressures for years and yet never develop damage. Untreated glaucoma leads to permanent damage of the optic nerve and resultant visual field loss, which can progress to blindness.

Visual Agnosia
    Visual agnosia is the inability of the brain to make sense of or make use of some part of otherwise normal visual stimulus, and is typified by the inability to recognize familiar objects or faces. This is distinct from blindness, which is a lack of sensory input to the brain due to damage to the eye or optic nerve. Visual agnosia is often due to damage, such as stroke, in posterior parietal lobe in the right hemisphere of the brain. Careful analysis of the nature of visual agnosia has led to improved understanding of the brain's role in normal vision.

Deadly Nightshade
    Deadly Nightshade is a plant oil that can potentially kill you. Atrophine taken from this plant causes your eyes to dilate. This was used in the middle ages by women who wanted to look more attractive for men. To this day, it is still used by opthamologists. How this works is that the atrophine is a competitor with acetylcholine. The Nightshadow goes into your receptors on the postsynaptic membrane of an action potential. This makes it so that the acetylcholine doesn’t have any receptor site so the Na ion is not able to be released.

### **Critical Thinking**

**The answers for these critical thinking questions is right [here](http://en.wikibooks.org/wiki/Human_Physiology/Appendix_1:_answers_to_review_questions#Critical_Thinking:_Vision)**

  1. Explain why you are normally unaware of your blind spot.
  2. Stare at a bright light for 10 seconds and then stare at a white sheet of paper. What do you observe and why?
  3. What is it that makes things "disappear" when you are staring at them at night, and how do you make them reappear?
  4. Name what rods are sensitive to and also what cones are sensitive to.
  5. Explain how Deadly Nightshade works.

## The Senses Of Hearing

The ear is the sense organ that collects and detects sound waves and plays a major role in the sense of balance and body position. The sensory receptors for both hearing and equilibrium are mechanoreceptors found in the inner ear; these receptors are hair cells that have stereocilia (long microvilli) that are extremely sensitive to mechanical stimulations.

### Anatomy of the Ear

The ear has three divisions: the outer ear, the middle ear, and the inner ear.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/7/7c/HumanEar.jpg/250px-HumanEar.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Anatomy of the human ear.

Outer Ear (Auricle, Ear Canal, Surface of Ear Drum)
    The outer ear is the most external portion of the ear. The outer ear includes the pinna (also called auricle), the ear canal, and the very most superficial layer of the ear drum (also called the tympanic membrane). Although the word "ear" may properly refer to the pinna (the flesh covered cartilage appendage on either side of the head), this portion of the ear is not vital for hearing. The complicated design of the human outer ear does help capture sound, but the most important functional aspect of the human outer ear is the ear canal itself. This outer ear canal skin is applied to cartilage; the thinner skin of the deep canal lies on the bone of the skull. If the ear canal is not open, hearing will be dampened. Ear wax (medical name - cerumen) is produced by glands in the skin of the outer portion of the ear canal. Only the thicker cerumen-producing ear canal skin has hairs. The outer ear ends at the most superficial layer of the tympanic membrane. The tympanic membrane is commonly called the ear drum.

Middle Ear (Air Filled Cavity behind the Ear Drum, includes most of the Ear Drum, and Ear Bones)
    The middle ear includes most of the ear drum (tympanic membrane) and the 3 ear bones ossicles: malleus (or hammer), incus (or anvil), and stapes (or stirrup). The opening of the Eustachian tube is also within the middle ear. The malleus has a long process (the handle) that is attached to the mobile portion of the ear drum. The incus is the bridge between the malleus and stapes. The stapes is the smallest named bone in the human body. The stapes transfers the vibrations of the incus to the **oval window**, a portion of the inner ear to which it is connected. It is the final bone in the chain to transfer vibrations from the eardrum to the inner ear. The arrangement of these 3 bones is a sort of Rube Goldberg device: movement of the tympanic membrane causes movement of the first bone, which causes movement of the second, which causes movement of the third. When this third bone pushes down, it causes movement of fluid within the cochlea (a portion of the inner ear). This particular fluid only moves when the stapes footplate is depressed into the inner ear. Unlike the open ear canal, however, the air of the middle ear is not in direct contact with the atmosphere outside the body. The Eustachian tube connects from the chamber of the middle ear to the back of the pharynx. The middle ear in humans is very much like a specialized paranasal sinus, called the tympanic cavity, it, like the paranasal sinuses, is a hollow mucosa lined cavity in the skull that is ventilated through the nose. The mastoid portion of the temporal bone, which can be felt as a bump in the skull behind the pinna, also contains air, which ventilates through the middle ear.

Inner Ear (Cochlea, Vestibule, and Semi-Circular Canals)
    The inner ear includes both the organ of hearing (the cochlea) and a sense organ (the labyrinth or vestibular apparatus) that is attuned to the effects of both gravity and motion. The balance portion of the inner ear consists of three semi-circular canals and the vestibule. The inner ear is encased in the hardest bone of the body. Within this ivory hard bone, there are fluid-filled hollows. Within the cochlea are three fluid filled spaces: the tympanic canal, the vestibular canal, and the middle canal. The eighth cranial nerve comes from the brain stem to enter the inner ear. When sound strikes the ear drum, the movement is transferred to the footplate of the stapes, which attaches to the oval window and presses into one of the fluid-filled ducts of the cochlea. The hair cells in the organ of Corti are stimulated by particular frequencies of sound, based on their location within the cochlea. High pitch sounds are at a higher frequency and, due to the shorter wavelength they "hit" the membrane "faster" (ie. close to the oval window). In contrast, low frequency sounds have large wavelengths, and will travel further through the scala vestibuli before "hitting" the tectorial membrane near the apex of the cochlea. The fluid inside the cochlea is moved, flowing against the receptor (hair) cells of the organ of Corti, which fire in a graded response based on the volume of the sound. The hair cells then stimulate the nerve cells in the Spiral Ganglion, which sends information through the auditory portion of the eighth cranial nerve to the brain. Humans are able to hear sounds between about 20 Hz and 20,000 Hz. Mammals that can hear lower frequency sounds, such as whales and elephants, have a longer cochlea. Humans tend to lose high-frequency hearing first, which has led some teenagers to using high-frequency ring tones (above 17,000 Hz) that may go undetected by their middle-aged teachers.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/0/0c/Cochlea-crosssection.png/250px-Cochlea-crosssection.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Cross section of the cochlea

Hair Cell
    Hair cells are columnar cells, each with a bundle of 100-200 specialized cilia at the top, for which they are named. These cilia are the mechanosensors for hearing. Lightly resting atop the longest cilia is the tectorial membrane, which moves back and forth with each cycle of sound, tilting the cilia and allowing electric current into the hair cell. Hair cells, like the photoreceptors of the eye, show a graded response, instead of the spikes typical of other neurons. Immediately over the hair cells of the organ of Corti is an overhanging “tectorial membrane.” When the Bones of the Middle Ear vibrate the oval window, these vibrations are transmitted to the fluid within the cochlea and eventually cause the round window on the cochlea to bulge outward. These vibrations deflect the membrane on which the Organ of Corti is located, causing the three rows of outer hair cells to “rub” against the overhanging tectorial membrane. By their muscle-like activity they ampify the weakest vibrations for the inner hair cells. The louder sounds are not amplified. The disturbed inner hair cells will then activate the cochlear nerve fibers. The current model is that cilia are attached to one another by “tip links”, structures which link the tips of one cilium to another. Stretching and compressing the tip links may open an ion channel and produce the receptor potential in the hair cell. These graded potentials are not bound by the “all or none” properties of an action potential. There are far fewer hair cells than afferent (leading to the brain) nerve fibers in the cochlea. The nerve that innervates the cochlea is the cochlear nerve, and forms cranial nerve number VIII with the vestibular nerve from the balance organ. Neuronal dendrites innervate cochlear hair cells. The neurotransmitter itself is thought to be **glutamate**. At the presynaptic juncture, there is a distinct “presynaptic dense body” or ribbon. This dense body is surrounded by synaptic vesicles and is thought to aid in the fast release of neurotransmitter. Efferent projections from the brain to the cochlea also play a role in the perception of sound. Efferent synapses occur on outer hair cells and on afferent dendrites under inner hair cells.

### Process of Hearing

Detection of sound motion is associated with the right posterior superior temporal gyrus. The superior temporal gyrus contains several important structures of the brain, including: (1)marking the location of the primary auditory cortex, the cortical region responsible for the sensation of sound. Sections 41 and 42 are called the primary auditory area of the cerebrum, and processes the basic characteristics of sound such as pitch and rhythm. The auditory association area is located within the temporal lobe of the brain, in an area called the Wernicke's area, or area 22. This area, near the lateral cerebral sulcus, is an important region for the processing of acoustic energy so that it can be distinguished as speech, music, or noise. It also interprets words that are heard into an associated thought pattern of understanding. The gnostic area of the cerebrum, (areas 5, 7, 39 and 40) helps to integrate all incoming sense patterns so that a common thought can be formed (correlated) using all arriving sensory information.

### Hearing Under Water

Hearing threshold and the ability to localize sound sources are reduced underwater. in which the speed of sound is faster than in air. Underwater, hearing is by bone conduction and localization of sound appears to depend on differences in amplitude detected by bone conduction.

### Localization of Sound by Humans

Humans are normally able to hear a variety of sound frequencies, from about 20Hz to 20kHz. Our ability to estimate just where the sound is coming from, sound localization, is dependent on both hearing ability of each of the two ears, and the exact quality of the sound. Since each ear lies on an opposite side of the head, a sound will reach the closest ear first, and its amplitide will be loudest in that ear. Much of the brain's ability to localize sound depends on interaural (between ears) intensity differences and interaural temporal or phase differences.

Two mechanisms are known to be used.

Bushy neurons can resolve time differences as small as the time it takes sound to pass one ear and reach the other (10 milliseconds). For high frequencies, frequencies with a wavelength shorter than the listener's head, more sound reaches the nearer ear. Human echolocation is a technique involving echolocation used by some blind humans to navigate within their environment.

### Process of Equilibrium

Equilibrioception or sense of balance is one of the physiological senses. It allows humans and animals to walk without falling. Some animals are better in this than humans, for example allowing a cat (as a quadruped using its inner ear and tail) to walk on a thin fence. All forms of equilibrioception can be described as the detection of acceleration.

It is determined by the level of fluid properly called endolymph in the labyrinth - a complex set of tubing in the inner ear.

When the sense of balance is interrupted it causes dizziness, disorientation and nausea.

You can temporarily disturb your sense of balance by closing your eyes and turning rapidly in circles five or six times. This starts the fluid swirling in circles inside your ear canal. When you stop turning it takes a few seconds for the fluid to lose momentum, and until then the sense from your inner ear conflicts with the information coming from your vision, causing dizziness and disorientation. Most astronauts find that their sense of balance is impaired when in orbit, because there is not enough gravity to keep the ear's fluid in balance. This causes a form of motion sickness called space sickness.

### Disorders with the Ear

**Case Study** A 45-year-old woman wakes up not feeling well. She believes that she may be coming down with the flu due to nausea that she is feeling, so she continues with her day. As the day progresses so does the feeling of nausea. While watching a movie with members of her family, the sick feeling seems to intensify and so they leave the movie. In the lobby of the movie theater she becomes very unbalanced and collapses. The fear is that she is experiencing a stroke. After being taken to the hospital via ambulance, the ER doctors also feel that it may be a stroke and do CAT scans to verify. Nothing shows up on the scans but the feeling of nausea and vertigo are intense. The woman is later diagnosed with an inner ear infection. The next 6-9 months of her life are filled with antibiotics, balance therapy and continued nausea and vertigo. Nothing seems to help so the doctors go into her inner ear surgically through her skull. They cut the vestibular nerve that is linked to the balance center on the left side. The right inner ear will eventually compensate for this loss of balance however it will take months of balance therapy. After a year from the onset on the inner ear infection, the woman has had three inner ear surgeries, loss of hearing in the left ear and problems with her balance. Doctors have told her they have done everything that they can and that she will now have to live with these conditions on a daily basis.

Deafness
    The word deaf can have at least two different meanings. The first term is used to indicate the presence of enough hearing loss such that an individual is not sensitive to sound. Someone with a partial loss of hearing is more likely to be referred to as hearing impaired or the qualified partially deaf by professionals. The second term is used to indicate someone who considers themselves 'culturally deaf', and they often use a capital D to distinguish this. Deaf people often are signers and consider that their Deafness is not something that needs to be medically fixed.

**Cochlear Implants** A cochlear implant is a device which has been used to restore hearing function to some deaf and hearing impaired people. It consists of an internal device; which extends electrodes into the cochlea and indirectly stimulates the auditory nerve, and an external device; which works much like a hearing aid, except it transmits information to the internal device rather than to the ear. The cochlear implant basically bypasses the middle ear and the cochlea hair cells, and allows some people with damage to these structures to hear 'electronically'.

![An inflamed Otitis Media.](../../../upload.wikimedia.org/wikipedia/commons/thumb/a/ae/Otitis_media_bullös.jpg/250px-Otitis_media_bullös.jpg)

Otitis Media
    An inflammation of the middle ear segment. It is usually associated with a buildup of fluid and frequently causes an earache. The fluid may or may not be infected. The typical progress of otitis media is: the tissues surrounding the Eustachian tube swell due to an infection and/or severe congestion. The Eustachian tube remains blocked most of the time. The air present in the middle ear is slowly absorbed into the surrounding tissues. A strong negative pressure creates a vacuum in the middle ear. The vacuum reaches a point where fluid from the surrounding tissues accumulates in the middle ear. Streptococcus pneumoniae and Haemophilus influenzae are the most common bacterial causes of otitis media. As well as being caused by Streptococcus pneumoniae and Haemophilus influenzae it can also be caused by the common cold.

Vertigo (dizziness)
    Vertigo, sometimes called a headrush, is a major symptom of a balance disorder. It is the sensation of spinning while the body is stationary with respect to the earth or surroundings. With the eyes shut, there will be a sensation that the body is in movement, called subjective vertigo; if the eyes are open, the surroundings will appear to move past the field of vision, called objective vertigo. The effects may be slight. It may cause nausea or, if severe, may give rise to difficulty with standing and walking. Vertigo is usually associated with a problem in the inner ear balance mechanisms (vestibular system), in the brain, or with the nerve connections between these two organs. The most common cause is benign paroxysmal positional vertigo, or BPPV. Vertigo can be a symptom of an underlying harmless cause, such as in BPPV or it can suggest more serious problems. These include drug toxicities, strokes or tumors (though these are much less common than BPPV).

Motion sickness
    Motion sickness is a condition in which the endolymph (the fluid found in the semicircular canals of the inner ears) becomes 'stirred up', causing confusion between the difference between apparent perceived movement (none or very little), and actual movement. Depending on the cause, it is also referred to as seasickness, carsickness, airsickness, or spacesickness. Nausea is the most common symptom of motion sickness. If the motion causing nausea is not resolved, the sufferer will frequently vomit within twenty minutes. Unlike ordinary sickness, vomiting in motion sickness tends not to relieve the nausea. If you don't want to consult a doctor, one common form of relief is to eat mints.

Dysacusis
    Dysacusis is a hearing impairment characterized by difficulty in processing details of sound, but not primarily a loss of the ability to perceive sound. May also refer to pain or discomfort due to sound.

### **Critical Thinking**

**The answers for these critical thinking questions can be found [here](http://en.wikibooks.org/wiki/Human_Physiology/Appendix_1:_answers_to_review_questions#Critical_Thinking:_Hearing).**

  1. Explain how the pitch of sound is coded. How is the loudness of sound coded?
  2. What do the three semicircular canals in the inner ear enable us to do? How do they accomplish this?
  3. What does the eustachian tube do? What does the eustachian tube have to do with a middle ear infection?
  4. What is the advantage of having a oval window?

## Touch

Touch is the first sense developed in the womb and the last sense used before death. With 50 touch receptors for every square centimeter and about 5 million sensory cells overall, the skin is very sensitive and is the largest and one of the most complex organs in our bodies. These touch receptors are grouped by type and include Mechanoreceptors (sensitive to pressure, vibration and slip), Thermoreceptors (sensitive to changes in temperature), and Nocioreceptors (responsible for pain).

### Pacinian Corpuscles

Pacinian corpuscles detect gross pressure changes and vibrations. They are the largest of the receptors. Any deformation in the corpuscle causes action potentials to be generated, by opening pressure-sensitive sodium ion channels in the axon membrane. This allows sodium ions to influx in, creating a receptor potential. Pacinian corpuscles cause action potentials when the skin is rapidly indented but not when the pressure is steady, due to the layers of connective tissue that cover the nerve ending (Kandel et al., 2000). It is thought that they respond to high velocity changes in joint position.

### Meissner's Corpuscle

Meissner's corpuscles are distributed throughout the skin, but concentrated in areas especially sensitive to light touch, such as the fingertips, palms, soles, lips, tongue, face, nipples and the external skin of the male and female genitals. They are primarily located just beneath the epidermis within the dermal papillae. Any physical deformation in the Meissner’s corpuscle will cause an action potential in the nerve. Since they are rapidly adapting or phasic, the action potentials generated quickly decrease and eventually cease. If the stimulus is removed, the corpuscle regains its shape and while doing so (ie: while physically reforming) causes another volley of action potentials to be generated. (This is the reason one stops "feeling" one's clothes.) This process is called **sensory adaption**. Because of their superficial location in the dermis, these corpuscles are particularly sensitive to touch and vibrations, but for the same reasons, they are limited in their detection because they can only signal that something is touching the skin. Meissner's corpuscles do not detect pain; this is signaled exclusively by free nerve endings.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/a/a4/Diagram_of_human_skin.jpg/500px-Diagram_of_human_skin.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Layers of the skin, showing the Merkel's Cell.

### Merkel’s Discs

Merkel’s Discs are Mechanoreceptors, making them sensitive to pressure and vibration. In humans, Merkel cells occur in the superficial skin layers, and are found clustered beneath the ridges of the fingertips that make up fingerprints. They’re somewhat rigid in structure, and the fact that they are not encapsulated, causes them to have a sustained response (in the form of action potentials or spikes) to mechanical deflection of the tissue. Merkel nerve endings are extremely sensitive to tissue displacement, and may respond to displacements of less than 1 um. Several studies indicate that they mediate high-resolution tactile discrimination, and are responsible for the ability of our fingertips to feel fine detailed surface patterns (e.g. for reading Braille).

### Ruffini corpuscles

Ruffini corpuscles are Thermoreceptors, aiding in the detection of temperature changes. Named after Angelo Ruffini, the Ruffini ending is a class of slowly adapting mechanoreceptor thought to exist only in the glabrous dermis and subcutaneous tissue of humans. This spindle-shaped receptor is sensitive to skin stretch, and contributes to the kinesthetic sense of and control of finger position and movement.

### Disorders of Touch

Sensory Processing Disorder
    In most people sensory integration occurs naturally without a thought process. But in some people the sensory integration does not develop properly and becomes distorted. In these people, the brain and central nervous system misinterprets everyday sensory information such as touch, sound and movement. Research is still being done on this disorder but they are finding direct links to SPD with other disorders like ADD/ADHD, premature birth, Autism, Down’s Syndrome and Fragile X.
Tactile defensiveness
    Considered a category of SPD, tactile defensiveness is an overreaction to the sense of touch. Identified by Dr. Jean Ayers in the 1960’s. A person with tactile defensiveness will react with a “flight or fight” reaction to touch stimuli that a normal person would interpret as harmless. Most cases are noticed in children or babies due to the fact that they do not want to be touched or cuddled as a normal child would. A child with this disorder will probably have these sign or symptoms:

  * Does not like to go barefoot or have feet touched
  * Does not enjoy baths, haircuts, nail clipping
  * Requires tags to be removed from all clothing
  * Does not want their face touched
  * Hard time eating because of textures, temperatures of the food
  * Does not want to touch anything that is messy or has a sticky texture

Congenital insensitivity to pain with anhidrosis or CIPA
    Exceedingly rare disease. There are only about 35 known cases in the United States. CIPA is a severe autosomal recessive condition in which the peripheral nerves demonstrate a loss of unmyelinated and small myelinated fibers. The actual physiopathological mechanism is still unknown and being studied- this is an extremely hard disease to study due to the rarity of cases. Most people with the disease will not live long due to injuries received that go untreated because they are unknown and severe

### Case Study

**Insensitivity to pain**

Wouldn't it wonderful if you could no longer feel pain. Is that not something we all would like to have? Or do we have pain for a good reason? Although it is rare there is a disease known as congenital insensitivity to pain. This genetic abnormality cause some people to lack certain components of the sensory system to receive pain. The exact reason for the problem is unknown and varies between people. Sadly people who have the disease often die in childhood. Injuries are very common with people who have congenital insensitivity to pain. They often will lose digits, may suffer from burns and their knees often have sores from kneeling to long. Clearly pain has a purpose, it is our warning signal when things are awry.

### The newborn's senses

Newborns can feel all different sensations, but respond most enthusiastically to soft stroking, cuddling and caressing. Gentle rocking back and forth will oftentimes calm a crying infant, as will massages and warm baths. Newborns may comfort themselves by sucking their thumbs, or a pacifier. The need to suckle is instinctive and allows newborns to feed.

Vision
    Newborn infants have unremarkable vision, being able to focus on objects only about 18 inches (45 cm) directly in front of their face. While this may not be much, it is all that is needed for the infant to look at the mother’s face when breastfeeding. When a newborn is not sleeping, or feeding, or crying, he or she may spend a lot of time staring at random objects. Usually anything that is shiny, has sharp contrasting colors, or has complex patterns will catch an infant's eye. However, the newborn has a preference for looking at other human faces above all else.

Hearing
    While still inside the mother, the infant can hear many internal noises, such as the mother's heartbeat, as well as many external noises including human voices, music and most other sounds. Therefore, although a newborn's ears may have some fluid present, he or she can hear sound from birth. Newborns usually respond to a female's voice over a male's. This may explain why people will unknowingly raise the pitch of their voice when talking to newborns. The sound of other human voices, especially the mother's, can have a calming or soothing effect on the newborn. Conversely, loud or sudden noises will startle and scare a newborn.

Taste
    Newborns can respond to different tastes, including sweet, sour, bitter, and salty substances, with preference toward sweets.

Smell
    A newborn has a developed sense of smell at birth, and within the first week of life can already distinguish the differences between the mother's own breast milk and the breast milk of another female.

Reflex Stimulation Response Age of disappearance Function

Eye blink
Bright light shinning in eyes or clap hands by eyes.
Closes eyelids quickly.
Permanent
This reflex protects the infant from an excessive amount of stimulation.

Withdrawal
Stick sole of foot with a stimulus like a pin.
This causes the foot to withdraw. Flexing of the knee to hip occurs.
Decreases after the 10th day of birth
This protects the infant from excessive unpleasant tactile stimulation.

Rooting
Touch cheek near the corner of the mouth.
The infant's head will turn towards the site of stimulation.
3 weeks (due to the voluntary response that is now capable for infant to do at this time)
This reflex helps baby to find the mothers' nipple.

Sucking
Place fingers in infant's mouth.
The infant will suck finger rhythmically.
4 months (voluntary sucking will come about)
This helps with feeding.

Swimming
Place the baby in pool of water face down.
The baby paddles and kicks in swimming movements.
4 to 6 month
This helps baby to survive if dropped into the water.

Moro
Hold infant in a cradling horizontal position and slightly lower the baby in a fast motion toward the ground while making a loud sound.
The baby will make a embracing motion and arch its back extending it's legs and throwing it's arms outward. Finally it will bring the arms in toward its body
6 months
In the evolutionary past this may have helped the baby cling to the mother.

Palmar grasp
Place the finger in baby's palm and press against the palm.
The baby will immediately grasp the finger.
3 to 4 months
This prepares infant for voluntary grasping.

Tonic neck
Turn the baby's head to one side while the baby is awake.
This will cause the baby to extend one arm in front of its eye or to the side to which the head has been turned.
4 months
This may prepare for voluntary reaching.

Stepping
Hold the baby under the arm and permit the bare feet of the baby to touch a flat surface.
The baby will lift one foot after the other in a stepping fashion.
2 months (this applies to a baby who has gained weight. For baby who is not as heavy, this reflex may be submissive.)
This prepares the baby for voluntary walking.

Babinski
Touch the foot in a stroking manner form the toe toward the heel.
The baby's toes will fan out and curl as the foot twists inward.
8 to 12 months
Unknown

## Review Questions

Answers for these questions can be found [here](http://en.wikibooks.org/wiki/Human_Physiology/Appendix_1:_answers_to_review_questions#Senses)

1\. Located under the hardest bone in the body, these control not only hearing but also a sense of gravity and motion:

A) The incus and the stapes

B) The pinna and the ear drum

C) the vestibular nerve and the semi circular canals

D) The eustachian tube and the stapes

  
2\. The retina does the following;

A) allows vision in light and dark, using cones and rods

B) Gives depth perception using binocular vision

C) Contains the ciliary muscles that control the shape of the lens

D) Protects and supports the shape of the eye

  
3\. This is the reason that we stop feeling the clothes that we are wearing

A) Merkel’s Discs are somewhat rigid in structure, and the fact that they are not encapsulated, causes them to have a sustained response

B) Meissner’s corpuscle are rapidly adapting or phasic, the action potentials generated quickly decrease and eventually cease

C) Ruffini corpuscles is a class of slowly adapting mechanoreceptor

D) Pacinian corpuscles allow sodium ions to influx in, creating a receptor potential

  
4\. When eating a piece of candy, I will use the following to sense that it is sweet

A) Fungiform papillae

B) Filiform papillae

C) Foliate papillae

D) Circumvallate papillae

E) All of the above

  
5\. If I have a cold, food may not taste as good to me because

A) The nerve fibrils are not functioning properly

B) My food will taste the same; taste and smell have nothing in common

C) Papilla become blocked by mucus and are unable to function

D) Olfaction, taste and trigeminal receptors together contribute to the flavor of my food

  
6\. Walking from a well lit room into a dark room would cause the following to occur

A) The sclera in the eye to open and eventually allow me to see in the dark

B) The extraocular muscles in the eye to open and eventually allow me to see in the dark

C) The cones in the eye to open and eventually allow me to see in the dark

D) the rods in the eye to open and eventually allow me to see in the dark

  
7\. Hair cells in the ear

A) Are the actual sensory receptors that will fire off action potentials when they are disturbed

B) Show a graded response, instead of the spikes typical of other neurons

C) “Rub” against the overhanging tectorial membrane

D) All of the above

  
8\. Eyesight decreases with age because

A) Older eyes receive much less light at the retina

B) There are numerous eye diseases that can affect an older eye

C) The extent to which the pupil dilates decreases with age

D) all of the above

  
9\. Teens walking off of a roller coaster in Magic Mountain seem to have vertigo because

A) The fluid in the auricle has not stopped moving causing conflicts with the information coming from your vision

B) the fluid in the cochlea has not stopped moving causing conflicts with the information coming from your vision

C) The fluid in the tympanic membrane has not stopped moving causing conflicts with the information coming from your vision

D) The fluid in the stirrup has not stopped moving causing conflicts with the information coming from your vision

  
10\. These receptors react to foods treated with monosodium glutamate

A) Salt

B) Sour

C) Bitter

D) Sweet

E) Umami

  
11\. What senses fall under the catagory of chemoreception?

A) Hearing and smell

B) Touch and hearing

C) Vision and taste

D) Taste and smell

## **Glossary**

**Anosmia**: Lack of olfaction, or a loss of the sense of smell

**Auditory Canal**: Tube from the auditory meatus or opening of the ear to the tympanic membrane

**Auditory Tube**: Either of the paired tubes connecting the middle ears to the nasopharynx; equalizes air pressure on the two sides of the eardrum

**Chemoreception**: Physiological response of a sense organ to a chemical stimulus

**Choroid**: Vascular layer of the eye lying between the retina and the sclera

**Circumvallate papillae**: Papillae that are present on the back of the oral part of the tongue

**Cochlea**: Is concerned with hearing, resembling a shell of a snail

**Dysosmia**: When things smell differently than they should

**Equilibrium**: Sense of balance

**Extraocular muscles**: Six muscles that control eye movements: lateral rectus, medial rectus, inferior rectus, superior rectus, inferior oblique and superior oblique

**Filiform papillae**: Thin, longer papillae that don't contain taste buds but are the most numerous

**Foliate papillae**: Ridges and grooves towards the posterior part of the tongue

**Fungiform papillae**: These are present mostly at the apex (tip) of the tongue- slightly mushroom shaped

**Gustation**: The sense of taste

**Hair Cell**: Mechanosensors for hearing, columnar cells each with a bundle of 100-200 specialized cilia at the top

**Haptic**: From the Greek Haphe, means pertaining to the sense of touch

**Hyposmia**: Decreased ability to smell

**Inner Ear**: Innermost part of the ear, contains the cochlea, westibule and semi-circular canals

**Mechanoreceptor**: Sensory receptor that responds to mechanical pressure or distortion

**Meissner's Corpuscle**: Encapsulated unmyelinated nerve endings, usually found in areas sensitive to light touch

**Middle Ear**: Air Filled Cavity behind the Ear Drum, includes most of the ear Drum and ear Bones

**Nasopharynx**: Nasal part of the pharynx that lies behind the nose and above the level of the soft palate

**Nociception**: The perception of pain

**Olfaction**: The sense of smell

**Otitis Media**: An inflammation of the middle ear

**Outer Ear**: External portion of the ear, includes the auricle, ear canal and surface of the ear drum

**Oval Window**: Fenestra that has the base of the stapes attached to it

**Pacinian Corpuscles**; Detect gross pressure changes and vibrations

**Papilla**: Specialized epithelial cells that are small projections on the top of the tongue

**Perception**: The brain’s interpretation of a sensation

**Phantosmia**: Phenomenon of smelling odors that aren't really present (AKA Phantom odors)

**Photoreceptors**: Specialized type of neuron found in the eye's retina that is capable of phototransduction

**Pinna**: Auricle of the ear

**Retina**: Thin layer of neural cells that lines the back of the eyeball of vertebrates and some cephalopods

**Round Window**: Fenestra leading into the cochlea

**Sclera**: White outer coating of the eye- gives the eye its shape and helps to protect the delicate inner parts

**Semicircular Canals**: Certain canals of the inner ear

**Sensation**: Occurs when nerve impulses arrive in the brain

**Sensory adaptation**: A decrease in response to stimuli

**Stapes**: One of the small bones in the tympanum of the ear; the stirrups bone

**Tactition**: The sense of pressure perception, generally in the skin

**Tympanic Membrane**: The membrane in the ear that vibrates to produce sound

**Umami**: Japanese word meaning savory or meaty- type of taste signal

## References

  * Hänig, D.P., 1901. Zur Psychophysik des Geschmackssinnes. Philosophische Studien, 17: 576-623.
  * Collings, V.B., 1974. Human Taste Response as a Function of Locus of Stimulation on the Tongue and Soft Palate. Perception & Psychophysics, 16: 169-174.
  * Buck, Linda and Richard Axel. (1991). A Novel Multigene Family May Encode Odorant Receptors: A Molecular Basis for Odor Recognition. Cell 65:175-183.

  


# The Muscular System

The **muscular system** is the biological system of humans that produces movement. The muscular system, in vertebrates, is controlled through the nervous system, although some muscles, like cardiac muscle, can be completely autonomous. **Muscle** is contractile tissue and is derived from the mesodermal layer of embryonic germ cells. Its function is to produce force and cause motion, either locomotion or movement within internal organs. Much of muscle contraction occurs without conscious thought and is necessary for survival, like the contraction of the heart or peristalsis, which pushes food through the digestive system. Voluntary muscle contraction is used to move the body and can be finely controlled, such as movements of the finger or gross movements that of the biceps and triceps.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/8/89/Illu_muscle_structure.jpg/300px-Illu_muscle_structure.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Muscle structure

Muscle is composed of muscle cells (sometimes known as "muscle fibers"). Within the cells are myofibrils; myofibrils contain sarcomeres which are composed of actin and myosin. Individual muscle cells are lined with endomysium. Muscle cells are bound together by perimysium into bundles called fascicles. These bundles are then grouped together to form muscle, and is lined by epimysium. Muscle spindles are distributed throughout the muscles, and provide sensory feedback information to the central nervous system. Skeletal muscle, which involves muscles from the skeletal tissue, is arranged in discrete groups. An example is the biceps brachii. It is connected by tendons to processes of the skeleton. In contrast, smooth muscle occurs at various scales in almost every organ, from the skin (in which it controls erection of body hair) to the blood vessels and digestive tract (in which it controls the caliber of a lumen and peristalsis, respectively).

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/c/c0/Skeletal_muscle.jpg/300px-Skeletal_muscle.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

A top-down view of skeletal muscle

There are approximately 640 skeletal muscles in the human body (see list of muscles of the human body). Contrary to popular belief, the number of muscle fibers cannot be increased through exercise; instead the muscle cells simply get bigger. It is however believed that myofibrils have a limited capacity for growth through hypertrophy and will split if subject to increased demand. There are three basic types of muscles in the body (smooth, cardiac, and skeletal). While they differ in many regards, they all use actin sliding against myosin to create muscle contraction and relaxation. In skeletal muscle, contraction is stimulated at each cell by nervous impulses that releases acetylcholine at the neuromuscular junction, createing action potentials along the cell membrane. All skeletal muscle and many smooth muscle contractions are stimulated by the binding of the neurotransmitter acetylcholine. Muscular activity accounts for most of the body's energy consumption. Muscles store energy for their own use in the form of glycogen, which represents about 1% of their mass. Glycogen can be rapidly converted to glucose when more energy is necessary.

## Types

There are three types of muscle:

![Illu muscle tissues.jpg](../../../upload.wikimedia.org/wikipedia/commons/1/1b/Illu_muscle_tissues.jpg)

  * **Smooth muscle** or "involuntary muscle" consists of spindle shaped muscle cells found within the walls of organs and structures such as the esophagus, stomach, intestines, bronchi, uterus, ureters, bladder, and blood vessels. Smooth muscle cells contain only one nucleus and no striations.
  * **Cardiac muscle** is also an "involuntary muscle" but it is striated in structure and appearance. Like smooth muscle, cardiac muscle cells contain only one nucleus. Cardiac muscle is found only within the heart.
  * **Skeletal muscle** or "voluntary muscle" is anchored by tendons to the bone and is used to effect skeletal movement such as locomotion. Skeletal muscle cells are multinucleated with the nuclei peripherally located. Skeletal muscle is called 'striated' because of the longitudinally striped appearance under light microscopy. Functions of the skeletal muscle include: 
    * Support of the body
    * Aids in bone movement
    * Helps maintain a constant temperature throughout the body
    * Assists with the movement of cardiovascular and lymphatic vessels through contractions
    * Protection of internal organs and contributing to joint stability

Cardiac and skeletal muscle are striated in that they contain sarcomere and are packed into highly-regular arrangements of bundles; smooth muscle has neither. Striated muscle is often used in short, intense bursts, whereas smooth muscle sustains longer or even near-permanent contractions.

Skeletal muscle is further divided into several subtypes:

  * Type I, slow oxidative, _slow twitch_, or "red" muscle is dense with capillaries and is rich in mitochondria and myoglobin, giving the muscle tissue its characteristic red color. It can carry more oxygen and sustain aerobic activity.
  * Type II, _fast twitch_, muscle has three major kinds that are, in order of increasing contractile speed: 
    * a) Type IIa, which, like slow muscle, is aerobic, rich in mitochondria and capillaries and appears red.
    * b) Type IIx (also known as type IId), which is less dense in mitochondria and myoglobin. This is the fastest muscle type in humans. It can contract more quickly and with a greater amount of force than oxidative muscle, but can sustain only short, anaerobic bursts of activity before muscle contraction becomes painful (often attributed to a build-up of lactic acid). N.B. in some books and articles this muscle in humans was, confusingly, called type IIB
    * c) Type IIb, which is anaerobic, glycolytic, "white" muscle that is even less dense in mitochondria and myoglobin. In small animals like rodents or rabbits this is the major fast muscle type, explaining the pale color of their meat.

For most muscles, contraction occurs as a result of conscious effort originating in the brain. The brain sends signals, in the form of action potentials, through the nervous system to the motor neuron that innervates the muscle fiber. However, some muscles (such as the heart) do not contract as a result of conscious effort. These are said to be autonomic. Also, it is not always necessary for the signals to originate from the brain. Reflexes are fast, unconscious muscular reactions that occur due to unexpected physical stimuli. The action potentials for reflexes originate in the spinal cord instead of the brain.

There are three general types of muscle contractions, skeletal muscle contractions, heart muscle contractions, and smooth muscle contractions.

## Muscular System Working With Other Body Systems

  * 1\. Homeostasis
  * 2\. Protection
  * 3\. Calcium Metabolism
  * 4\. Maintaining Body Temperature

## Skeletal Muscle Contractions

Steps of a skeletal muscle contraction:

  


  * An action potential reaches the axon of the motor neuron.
  * The action potential activates voltage gated calcium ion channels on the axon, and calcium rushes in.
  * The calcium causes acetylcholine vesicles in the axon to fuse with the membrane, releasing the acetylcholine into the cleft between the axon and the motor end plate of the muscle fiber.
  * The skeletal muscle fiber is excited by large mylenated nerve fibers which attach to the neuromuscular junction. There is one neuromuscular junction for each fiber.
  * The acetylcholine diffuses across the cleft and binds to nicotinic receptors on the motor end plate, opening channels in the membrane for sodium and potassium. Sodium rushes in, and potassium rushes out. However, because sodium is more permeable, the muscle fiber membrane becomes more positively charged, triggering an action potential.
  * The action potential on the muscle fiber causes the sarcoplasmic reticulum to release calcium ions(Ca++).
  * The calcium binds to the troponin present on the thin filaments of the myofibrils. The troponin then allosterically modulates the tropomyosin. Normally the tropomyosin physically obstructs binding sites for cross-bridge; once calcium binds to the troponin, the troponin forces the tropomyosin to move out of the way, unblocking the binding sites.
  * The cross-bridge (which is already in a ready-state) binds to the newly uncovered binding sites. It then delivers a power stroke.
  * ATP binds the cross-bridge, forcing it to conform in such a way as to break the actin-myosin bond. Another ATP is split to energize the cross bridge again.
  * Steps 7 and 8 repeat as long as calcium is present on thin filament.
  * Throughout this process, the calcium is actively pumped back into the sarcoplasmic reticulum. When no longer present on the thin filament, the tropomyosin changes back to its previous state, so as to block the binding sites again. The cross-bridge then ceases binding to the thin filament, and the contractions cease as well.
  * Muscle contraction remains as long as Ca++ is abundant in sarcoplasm.

Types of Contractions:

  * Isometric contraction--muscle does not shorten during contraction and does not require the sliding of myofibrils but muscles are stiff.
  * Isotonic contraction--inertia is used to move or work. More energy is used by the muscle and contraction lasts longer than isometric contraction. Isotonic muscle contraction is divided into two categories: concentric, where the muscle fibers shorten as the muscle contracts (ie. biceps brachialis on the up phase of a biceps curl); and eccentric, where the muscle fibers lengthen as they contract (ie. biceps brachialis on the down phase of a biceps curl).
  * Twitch--exciting the nerve to a muscle or by passing electrical stimulus through muscle itself. Some fibers contract quickly while others contract slowly.
  * Tonic -maintaining postural tone against the force of gravity.

The Efficiency of Muscle Contraction:

  * Only about 20% of input energy converts into muscular work. The rest of the energy is heat.
  * 50% of energy from food is used in ATP formation.
  * If a muscle contraction is slow or without movement, energy is lost as maintenance heat.
  * If muscle contraction is rapid, energy is used to overcome friction.

  
Summation of Muscle Contraction: It is the adding together of individual muscle twitches to make strong muscle movements.

  * Multiple motor unit summation--increasing number of motor units contracting simultaneously.
  * Wave summation--increasing rapidity of contraction of individual motor units.
  * Tetanization--higher frequency successive contractions fuse together and cannot be distinguished from one another.

## Sliding Filament theory

When a muscle contracts, the actin is pulled along myosin toward the center of the sarcomere until the actin and myosin filaments are completely overlapped. The H zone becomes smaller and smaller due to the increasing overlap of actin and myosin filaments, and the muscle shortens. Thus when the muscle is fully contracted, the H zone is no longer visible (as in the bottom diagram, left). Note that the actin and myosin filaments themselves do not change length, but instead slide past each other.

### Cellular Action of Skeletal Muscles

During cellular respiration the mitochondria, within skeletal muscle cells, convert glucose from the blood to carbon dioxide and water in the process of producing ATP (see [cell physiology](http://en.wikibooks.org/wiki/Human_Physiology/Cell_physiology)). ATP is needed for all muscular movement. When the need of ATP in the muscle is higher than the cells can produce with aerobic respiration, the cells will produce extra ATP in a process called anaerobic respiration. The first step of aerobic respiration(glycolysis) produces two ATP per glucose molecule. When the rest of the aerobic respiration pathway is occupied the pyruvate molecule can be converted to lactic acid. This method produces much less ATP than the aerobic method, but it does it faster and allows the muscles to do a bit more than if they relied solely on ATP production from aerobic respiration. The drawback to this method is that lactic acid accumulates and causes the muscles to fatigue. They will eventually stop contracting until the breakdown of lactic acid is sufficient to allow for movement once again. People experience this most noticeably when they repeatedly lift heavy things such as weights or sprint for a long distance. Muscle soreness sometimes occurs after vigorous activity, and is often misunderstood by the general public to be the result of lactic acid buildup. This is a misconception because the muscle does fatigue from lactic acid buildup, but it does not stay in the muscle tissue long enough to cause tissue breakdown or soreness. During heavy breathing, following exercise, the cells are converting the lactic acid either back into glucose or converting it to pyruvate and sending it through the additional steps of aerobic respiration. By the time a person is breathing normally again the lactic acid has been removed. The soreness is actually from small tears in the fibers themselves. After the fibers heal they will increase in size. The number of mitochondria will also increase if there is continued demand for additional ATP. Hence, through exercise the muscles can increase in both strength and endurance.

Another misconception is that as the muscle increases in size it also gains more fibers. This is not true. The fibers themselves increase in size rather than in quantity. The same holds true for adipose tissue--fat cells do not increase in number, but rather the amount of lipids (oil) in the cells increase.

Muscle fibers are also genetically programmed to reach a certain size and stop growing from there, so after awhile even the hardest working weightlifter will only reach a certain level of strength and endurance. Some people will get around this by taking steroids. The artificial steroids cause all sorts of trouble for the person. They can cause the adrenal glands to stop producing corticosteroids and glucosteroids. This leads to the atrophy of the gland's medulla and causes permanent loss of the production of these hormones. The testicles may also atrophy in response to steroids. Eventually the testes will stop making testosterone and sperm, rendering the male infertile.

One of the more serious problems associated with abnormal gain of muscle mass is heart failure. While for most people gaining muscle and losing fat is desirable, a body builder is at risk of producing more muscle mass than the heart can handle. One pound of fat contains about 3.5 miles of blood vessels, but one pound of muscle has about 6.5 miles. Hence, additional muscle causes the heart to pump more blood. Some people that have too much muscle will be very strong but will not have a healthy aerobic endurance, in part because of the difficulty of providing oxygenated blood to so much tissue.

[Sliding filament theory](http://3dotstudio.com/zz.html)
    This link shows the animation of the sliding filament theory.
[explanation and image of sliding filament theory](http://www.ucl.ac.uk/~sjjgsca/muscleSlidingFilament.html)
    this link gives a better demonstration of the theory with the explanation.

## Involuntary Muscle Movement

Spasms

When Smooth and skeletal muscles go through multiple spasms it is referred either as seizure or convulsion.

Cramps

Strenuous activities can cause painful spasms that are long, this is referred to as cramps.

## Injury

Sprain

An injury to a joint that involves a stretched or torn ligament.

Muscle Strain

A strain occurs when a muscle or the tendon that attaches it to the bone is overstretched or torn. Muscle strains are also called pulled muscles. Who gets it?

Anyone can strain a muscle. However, people involved in sports or other forms of strenuous exercise are more likely to strain a muscle. _What causes it?_

Muscles are bunches of fibers that can contract. Muscle strains usually occur during activities that require the muscle to tighten forcefully. The muscle is strained either because it is not properly stretched, or warmed up, before the activity; it is too weak; or because the muscle is already injured and not allowed time to recover. So, many muscle strains occur during exercise or sports activities. They can also occur when lifting heavy objects. What are the symptoms?

When a muscle is strained, it hurts and is difficult to move. You may also feel a burning sensation in the area of the injured muscle, or feel as though something has "popped." Sometimes the area of the strained muscle looks bruised or swells. A strained muscle might spasm, which means it contracts suddenly and involuntarily, causing severe pain. How is it diagnosed?

To diagnose a muscle strain, your doctor will examine the painful area, and ask how and when the injury happened. He or she may order other diagnostic tests, such as x-rays, to rule out any injury to the bone.

_What is the treatment?_

Muscle strains are treated with rest, ice, compression, and elevation, or RICE. You will be told to rest the injured area to reduce pain and swelling. If the strain is in the leg or foot area, you may need to use crutches. Ice packs are recommended at regular intervals (as recommended by your doctor) over the first few days after the injury. Ice causes the blood vessels to constrict, which reduces inflammation and pain. Anti-inflammatory medications might also be used to relieve pain. Compression and elevation help to reduce swelling. Your doctor may also recommend physical therapy to speed your recovery. You should avoid the type of activity that caused the injury until the muscle is completely healed. Self-care tips

You can prevent muscle strains by warming up for at least 10 minutes before participating in any strenuous exercise or heavy lifting. When you warm up, you increase the blood circulation to the muscle and prepare it for exercise. When starting any new exercise program or sport, it's important to begin gradually so your muscles are conditioned for the activity.

### Steroids

Anabolic steroids, which are synthetic versions of the primary male sex hormone testosterone, can be injected, taken orally, or used transdermally. These drugs are Controlled Substances that can be prescribed to treat conditions such as body wasting in patients with AIDS, and other diseases that occur when the body produces abnormally low amounts of testosterone. However, the doses prescribed to treat these medical conditions are 10 to 100 times lower than the doses that are used for performance enhancement.

Let me be clear:- while anabolic steroids can enhance certain types of performance or appearance,they are dangerous drugs, and when used inappropriately, they can cause a host of severe, long-lasting, and often irreversible negative health consequences. These drugs can stunt the height of growing adolescents, masculinize women, and alter sex characteristics of men. Anabolic steroids can lead to premature heart attacks, strokes, liver tumors, kidney failure and serious psychiatric problems. In addition, because steroids are often injected, users risk contracting or transmitting HIV or hepatitis.

Abuse of anabolic steroids differs from the abuse of other illicit substances because the initial use of anabolic steroids is not driven by the immediate euphoria that accompanies most drugs of abuse, such as cocaine, heroin, and marijuana, but by the desire of the user to change their appearance and performance, characteristics of great importance to adolescents. These effects of steroids can boost confidence and strength leading the user to overlook the potential serious long-term damage that these substances can cause.

Government agencies such as NIDA support research that increases our understanding of the impact of steroid use and improves our ability to prevent abuse of these drugs. For example, NIDA funding led to the development of two highly effective programs that not only prevent anabolic steroid abuse among male and female high school athletes, but also promote other healthy behaviors and attitudes. The ATLAS (targeting male athletes) and ATHENA (targeting female athletes) programs have been adopted by schools in 29 states and Puerto Rico. Both Congress and the Substance Abuse and Mental Health Services Administration have endorsed ATLAS and ATHENA as model prevention programs, which could and should be implemented in more communities throughout the country.

In addition to these prevention programs and other research efforts, also has invested in public education efforts to increase awareness about the dangers of steroid abuse. We have material on our website about steroid abuse at www.steroidabuse.gov and in April 2005 we again will distribute a "Game Plan" public service announcement designed to bring attention to abuse of anabolic steroids.

Research has shown that the inappropriate use of anabolic steroids can have catastrophic medical, psychiatric and behavioral consequences.

I hope that students, parents, teachers, coaches and others will take advantage of the information on our website about anabolic steroids abuse and join us in our prevention and education efforts. Participating in sports offers many benefits, but young people and adults shouldn't take unnecessary health risks in an effort to win.(Nora D. Volkow, M.D.)

-Human-made substances related to male sex hormones. Some athletes abuse anabolic steroids to enhance performance. Abuse of anabolic steroids can lead to serious health problems, some of which are irreversible.

Major side effects can include liver tumors and cancer, jaundice, high blood pressure, kidney tumors, severe acne, and trembling. In males, side effects may include shrinking of the testicles and breast development. In females, side effects may include growth of facial hair, menstrual changes, and deepened voice. In teenagers, growth may be halted prematurely and permanently.

The therapeutic use of steroids can be realized by patients and their doctors by using them in a manner that is beneficial to the person.

### MyoD and other muscular factors

MyoD is a protein and a transcription factor that activates muscle cell differentiation by turning on transcription of specific regulatory genes. It turns stem cells into myoblasts, a cell that can turn into many muscle cell, also called "muscle stem cell". MyoD belongs to a family of proteins knowns as myogenic regulatory factor(MRFs). MyoD can also turn on transcription of it's own regulatory genes (MyoD protein coding genes), and this means that it can produce more of itself. The positive feedback turns on transcription of other muscle proteins, cell cycle blockers, and microRNA-206. One of the main actions of MyoD is to remove cells from the cell cycle by enhancing the transcription of p21The function of MyoD is to commit mesoderm cells to a skeletal lineage. MyoD can also regulate muscle repair. One of the main actions of MyoD is to remove cells from the cell cycle by enhancing the transcription of p21. Bidirectional Signalling- muscle cells and nerves cells send signals back and forth to each other. Amyotrophic Lateral Sclerosis(ALS) is a loss of motor neuron and this blocks the formation of neuromuscular junctions. Therefore, no muscle growth which means a potential of leading to paralysis. Stephen Hawking suffers from this disease.

### Muscle Homeostasis

MicroRNA-206 indirectly forms neuromuscular junctions with motor neurons. Neuromuscular junction sends synaptic signals to MyoD and this blcoks MyoD and stops or limits muscle development. Myostatin is a protein that also blocks MyoD. Without myostatin, muscle development increases.

Myostatin Mutations In Sheep: they can have a mutant myostatin that causes microRNA-206 to block myostatin translation

Myostatin Mutations In Humans: humans with mutant myostatin will develop lots of muscle (like a body builder) is possible to create a drug that blocks myostatin production.

## Smooth Muscle Contraction

  * Contractions are initiated by an influx of calcium which binds to calmodulin.
  * The calcium-calmodulin complex binds to and activates myosin light-chain kinase.
  * Myosin light-chain kinase phosphorylates myosin light-chains using ATP, causing them to interact with actin filaments.
  * Powerstroke.
  * Calcium is actively pumped out of the cell by receptor regulated channels. A second messanger, IP3, causes the release.
  * As calcium is removed the calcium-calmodulin complex breaks away from the myosin light-chain kinase, stopping phosphorylation.
  * Myosin phophatase dephosphorylates the myosin. If the myosin was bound to an actin molecule, the release is slow, this is called a latch state. In this manner, smooth muscle is able to stay contracted for some time without the use of much ATP. If the myosin was not bound to an actin chain it loses its affinity for actin.

It should be noted that ATP is still needed for crossbridge cycling, and that there is no reserve, such as creatine phosphate, available. Most ATP is created from aerobic metabolism, however anaerobic production may take place in times of low oxygen concentrations.

## Cardiac Muscle

Cardiac muscle is found in the heart and lungs of humans

## ATP in the Human Body

Muscles cells, like all cells, use ATP as an energy source. The total quantity of ATP in the human body at any one time is about 0.1 Mole. The energy used by human cells requires the hydrolysis of 200 to 300 moles of ATP daily. This means that each ATP molecule is recycled 2000 to 3000 times during a single day. ATP cannot be stored, hence its consumption must closely follow its synthesis. On a per-hour basis, 1 kilogram of ATP is created, processed and then recycled in the body. Looking at it another way, a single cell uses about 10 million ATP molecules per second to meet its metabolic needs, and recycles all of its ATP molecules about every 20-30 seconds.

## Lactic Acid

Catabolized carbohydrates is known as glycolysis. The end product of glycolysis, pyruvate can go into different directions depending on aerobic or anaerobic conditions. In aerobic it goes through the Krebs cycle and in anaerobic it goes through the Cori cycle. In the Cori cycle pyruvate is converted to lactate, this forms lactic acid, lactic acid causes muscle fatigue. In the aerobic conditions pyruvate goes through the Krebs cycle. For more about Krebs cycle refer to chapter 2 Cell Physiology.

## Muscle Disorders

### Dermatomyositis and Polymyositis

Dermatomyositis and polymyositis cause inflammation of the muscles. They are rare disorders, affecting only about one in 100,000 people per year. More women than men are affected. Although the peak age of onset is in the 50s, the disorders can occur at any age.

Signs and symptoms — Patients complain of muscle weakness that usually worsens over several months, though in some cases symptoms come on suddenly. The affected muscles are close to the trunk (as opposed to in the wrists or feet), involving for example the hip, shoulder, or neck muscles. Muscles on both sides of the body are equally affected. In some cases, muscles are sore or tender. Some patients have involvement of the muscles of the pharynx (throat) or the esophagus (the tube leading from the throat to the stomach), causing problems with swallowing. In some cases, this leads to food being misdirected from the esophagus to the lungs, causing severe pneumonia.

In dermatomyositis, there is a rash, though sometimes the rash resolves before muscle problems occur. A number of different types of rash can occur, including rashes on the fingers, the chest and shoulders, or on the upper eyelids (show picture 1-3). In rare cases, the rash of dermatomyositis appears but myopathy never develops.

Other problems sometimes associated with these diseases include fever, weight loss, arthritis, cold-induced color changes in the fingers or toes (Raynaud phenomenon), and heart or lung problems.

### Muscle Atrophy

Alternative names : Atrophy of the muscles, Muscle wasting, Wasting

The majority of muscle atrophy in the general population results from disuse. People with sedentary jobs and senior citizens with decreased activity can lose muscle tone and develop significant atrophy. This type of atrophy is reversible with vigorous exercise. Bed-ridden people can undergo significant muscle wasting. Astronauts, free of the gravitational pull of Earth, can develop decreased muscle tone and loss of calcium from their bones following just a few days of weightlessness.

Muscle atrophy resulting from disease rather than disuse is generally one of two types, that resulting from damage to the nerves that supply the muscles, and disease of the muscle itself. Examples of diseases affecting the nerves that control muscles would be poliomyelitis, amyotrophic lateral sclerosis (ALS or Lou Gehrig's disease), and Guillain-Barre syndrome. Examples of diseases affecting primarily the muscles would include muscular dystrophy, myotonia congenita, and myotonic dystrophy as well as other congenital, inflammatory or metabolic myopathies.

Even minor muscle atrophy usually results in some loss of mobility or power.

Common Causes

  * some atrophy that occurs normally with ageing
  * cerebrovascular accident (stroke)
  * spinal cord injury
  * peripheral nerve injury (peripheral neuropathy)
  * other injury
  * prolonged immobilization
  * osteoarthritis
  * rheumatoid arthritis
  * prolonged corticosteroid therapy
  * diabetes (diabetic neuropathy)
  * burns
  * poliomyelitis
  * amyotrophic lateral sclerosis (ALS or Lou Gehrig's disease)
  * Guillain-Barre syndrome
  * muscular dystrophy
  * myotonia congenita
  * myotonic dystrophy
  * myopathy

### Muscular Dystrophy

Muscular dystrophy (MD) is a group of rare inherited muscle diseases in which muscle fibers are unusually susceptible to damage. Muscles, primarily voluntary muscles, become progressively weaker. In the late stages of muscular dystrophy, muscle fibers are often replaced by fat and connective tissue. In some types of muscular dystrophy, heart muscles, other involuntary muscles and other organs are affected.

The most common types of muscular dystrophy appear to be due to a genetic deficiency of the muscle protein dystrophin. There's no cure for muscular dystrophy, but medications and therapy can slow the course of the disease.

## Medical Mysteries

#### Sleep Twitches

The twitching phenomenon that happens in the early stage of sleep is called a hypnagogic massive jerk, or simply a hypnic jerk. It has also been referred to as a sleep start. There has been little research on this topic, but there have been some theories put forth. When the body drifts off into sleep, it undergoes physiological changes related to body temperature, breathing rate and muscular tone. Hypnic jerks may be the result of muscle changes. Another theory suggests that the transition from the waking to the sleeping state signals the body to relax. But the brain may interpret the relaxation as a sign of falling and then signal the arms and legs to wake up. Electroencephalogram studies have shown sleep starts affect almost 10 percent of the population regularly, 80 percent occasionally, and another 10 percent rarely.

Muscle movement or twitching also may take place during the Rapid Eye Movement, or REM, phase of sleep. This also is the time when dreams occur. During the REM phase, all voluntary muscular activity stops with a drop in muscle tone, but some individuals may experience slight eyelid or ear twitching or slight jerks. Some people with REM behavioral disorder, or RBD, may experience more violent muscular twitching and full-fledged activity during sleep. This is because they do not achieve muscle paralysis, and as a result, act out their dreams. Researchers think that people with RBD lack neurological barriers that define the different stages of sleep. New research done by the Mayo Clinic and published in the July 2003 issue of Sleep Medicine shows that melatonin can help lessen RBD symptoms.

Resources:

    Sleep twitches, or myoclonic jerks, as they are sometimes called, are explained in easily understood language on this website.
    Learn more about REM Behavior Disorder, or RBD, and treatment for sufferers.
    View information about various sleep disorders such as insomnia, apnea, and narcolepsy.

## Microbiology

_Clostridium tetani_
    Tetanus
    Normally a nerve impulse initiates contraction of a muscle. At the same time, an opposing muscle receives the signal to relax so as not to oppose the contraction. Tetanus toxin blocks the relaxation, so both sets of muscle contract. The usual cause of tetany is lack of calcium, but excess of phosphate (high phosphate-to-calcium ratio) can also trigger the spasms.

_Clostridium botulinum_
    Infant botulism (floppy baby syndrome) the most common form of botulism in the U.S. of the four forms of botulism.
    If ingested, the toxin is absorbed in the intestine, goes to the blood, and on to the nervous system. It acts on the peripheral nervous system by blocking the impulse that is normally passes along to the nervous system. By clocking the impulse that is normally passed along to motor end plates so the muscle contraction can be released, resulting in paralysis.

## Glossary

Actin
    A protien that forms a long polymer rods called microfilaments; Interacts with myosin to cause movement in muscles.

ATP
    "Adenosine Triphosphate" is a nucleotide that comes from adenosine that takes place in muscle tissue: This provides a large source of energy for cellular reactions.

Cardiac muscle
    is also an "involuntary muscle" but it's a specialized kind of muscle found only within the heart.

Clostridium botulinum
    A pathogen that causes botulism, gram stain positive, morphology is rod shaped, grows in anaerobic conditions, and produces spores.

Clostridium tetani
    A pathogen that causes lock jaw, gram stain positive, morphology is tennis racket shaped rod, grows in anaerobic conditions, and produces spores.

Cori cycle
    In anaerobic conditions produces lactic acid.

Cramp
    A localized muscle spasm that happens after strenuous activity.

Glycogen
    Glucose that has been converted for energy storage. Muscles store energy for their own use in this form.

Lactic acid
    Causes muscle fatigue.

Muscle
    Contractile tissue that is derived from the mesodermal layer of embryonic germ cells.

Muscular Dystrophy
    A hereditary disease characterized by progressive atrophy of muscle fibers

Myosin
    The fibrous motor protein that uses ATP to drive movements along actin filaments.

Sarcoplasmic Reticulum
    Smooth-surfaced tubules forming a plexus around each myofibril that function as a storage and release area for calcium ions (CA+2).

Skeletal muscle
    this "voluntary muscle" is anchored by tendons to the bone and is used to affect skeletal movement such as locomotion.

Smooth muscle
    this "involuntary muscle" is found within the walls of organs and structures such as the esophagus, stomach, intestines, bronchi, uterus, ureters, bladder, and blood vessels.

Sprain
    Injuries that involves a stretched or torn ligament.

Strain
    A injury to the muscle or tendon attachment

## References

  * Van De Graaff (2002) _Human Anatomy 6th ed._ McGraw-Hill Higher Education
  * Windmaier, P.W. Raff, H. Strang, T.S. (2004) _Vander, Sherman, & Luciano's Human Physiology, the Mechanisms of Body Function 9th ed._ Mcgraw-Hill

Neil A. Campbell , Jane B. Reece "Biology 8th edition"  


# Blood Physiology

## Overview of Blood

The primary function of blood is to supply oxygen and nutrients as well as constitutional elements to tissues and to remove waste products. Blood also enables hormones and other substances to be transported between tissues and organs. Problems with blood composition or circulation can lead to downstream tissue malfunction. Blood is also involved in maintaining homeostasis by acting as a medium for transferring heat to the skin and by acting as a buffer system for bodily pH.

The blood is circulated through the lungs and body by the pumping action of the heart. The right ventricle pressurizes the blood to send it through the capillaries of the lungs, while the left ventricle re-pressurizes the blood to send it throughout the body. Pressure is essentially lost in the capillaries, hence gravity and especially the actions of skeletal muscles are needed to return the blood to the heart.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/e/e6/Heart-and-lungs.jpg/220px-Heart-and-lungs.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Blood circulation from the heart to the lungs.

### Gas Exchange

**Oxygen (O2)** is the most immediate need of every cell and is carried throughout the body by the blood circulation. Oxygen is used at the cellular level as the final electron acceptor in the electron transport chain (the primary method of generating ATP for cellular reactions). Oxygen is carried in the blood bound to hemoglobin molecules within red blood cells. Hemoglobin binds oxygen when passing through the alveoli of the lungs and releases oxygen in the warmer, more acidic environment of bodily tissues, via simple diffusion.

**Carbon dioxide (CO2)** is removed from tissues by blood and released into the air via the lungs. Carbon dioxide is produced by cells as they undergo the processes of cellular respiration (particularly the Kreb's Cycle). The molecules are produced from carbons that were originally part of glucose. Most of the carbon dioxide combines with water and is carried in the plasma as bicarbonate ions. An excess of carbon dioxide (through exercise, or from holding ones breath) quickly shifts the blood pH to being more acidic (acidosis). Chemoreceptors in the brain and major blood vessels detect this shift and stimulate the breathing center of the brain (the medulla oblongata). Hence, as CO2 levels build up and the blood becomes more acidic, we involuntarily breathe faster, thus lowering CO2 levels and stabilizing blood pH. In contrast, a person who is hyperventilating (such as during a panic attack) will expire more CO2 than being produced in the body and the blood will become too alkaline (alkalosis).

## Blood Composition

**Blood** is a circulating tissue composed of fluid plasma and cells (red blood cells, white blood cells, platelets). Anatomically, blood is considered a connective tissue, due to its origin in the bones and its function. _Blood_ is the means and transport system of the body used in carrying elements (e.g. nutrition, waste, heat) from one location in the body to another, by way of blood vessels.

  
Blood is made of two parts:

  1. Plasma which makes up 55% of blood volume.
  2. Formed cellular elements (red and white blood cells, and platelets) which combine to make the remaining 45% of blood volume.

### Plasma makeup

**Plasma** is made up of 90% water, 7-8% soluble proteins (albumin maintains bloods osmotic integrity, others clot, etc), 1% carbon-dioxide, and 1% elements in transit. One percent of the plasma is salt, which helps with the pH of the blood. The largest group of solutes in plasma contains three important proteins to be discussed. There are: _albumins_, _globulins_, and _clotting proteins_.

_Albumins_ are the most common group of proteins in plasma and consist of nearly two-thirds of them (60-80%). They are produced in the liver. The main function of albumins is to maintain the osmotic balance between the blood and tissue fluids and is called _colloid osmotic pressure_. In addition, albumins assist in transport of different materials, such as vitamins and certain molecules and drugs (e.g. bilirubin, fatty acids, and penicillin).

_Globulins_ are a diverse group of proteins, designated into three groups: gamma, alpha, and beta. Their main function is to transport various substances in the blood. Gamma globulins assist the body's immune system in defense against infections and illness.

_Clotting proteins_ are mainly produced in the liver as well. There are at least 12 substances, known as "clotting factors" that participate in the clotting process. One important clotting protein that is part of this group is _fibrinogen_, one of the main components in the formation of blood clots. In response to tissue damage, fibrinogen makes fibrin threads, which serve as adhesive in binding platelets, red blood cells, and other molecules together, to stop the blood flow. (This will be discussed in more detail later on in the chapter.)

_Plasma_ also carries Respiratory gases; CO2 in large amounts(about 97%) and O2 in small amounts(about 3%), various nutrients(glucose, fats), wastes of metabolic exchange(urea, ammonia), hormones, and vitamins.

  


![](../../../upload.wikimedia.org/wikipedia/commons/1/13/Redbloodcells.jpg)

Picture of red blood cells.

### Red Blood Cells

#### Overview

**Red blood cell (erythrocyte)** also known as "RBCs". RBCs are formed in the **myeloid tissue** or most commonly known as red bone marrow, although when the body is under severe conditions the yellow bone marrow, which is also in the fatty places of the marrow in the body will also make RBCs. The formation of RBCs is called **erythropoiesis** ( _erythro / red; poiesis /_ _formation_). Red blood cells lose nuclei upon maturation, and take on a biconcave, dimpled, shape. They are about 7-8 micrometers in diameter. There are about 1000x more red blood cells than white blood cells. RBCs live about 120 days and do not self repair. RBCs contain hemoglobin which transports oxygen from the lungs to the rest of the body, such as to the muscles, where it releases the oxygen load.The hemoglobin gets its red color from their respiratory pigments.

  
**Shape**

RBCS have a shape of a disk that appears to be “caved in” or almost flattened in the middle; this is called _bi-concave_. This bi-concave shape allows the RBC to carry oxygen and pass through even the smallest capillaries in the lungs. This shape also allows RBCs to stack like dinner plates and bend as they flow smoothly through the narrow blood vessels in the body. RBCs lack a nucleus (no DNA) and no organelles, meaning that these cells cannot divide or replicate themselves like the cells in our skin and muscles. RBCs have a short life span of about 120 days, however, as long as our myeloid tissue is working correctly, we will produce about 2-3 million RBCs per second. That is about 200 billion a day! This allows us to have more to replace the ones we lose.

  
**Main Component**

The main component of the RBC is **hemoglobin** protein, of which there are about 250 million per cell. The word hemoglobin comes from "hemo" meaning blood and "globin" meaning protein. Hemoglobin is composed of four protein subunits: polypeptide globin chains that contain anywhere from 141 to 146 amino acids. Hemoglobin is responsible for the cell’s ability to transport oxygen and carbon dioxide. Hemoglobin, iron, and oxygen interact with each other, forming the RBCs' bright red color. You can call this interaction by product **oxyhemoglobin**. Carbon Monoxide binds with hemoglobin faster than oxygen, and stays bound for several hours, making hemoglobin temporarily unavailable for oxygen transport. One red blood cell contains about 200 million hemoglobin molecules. If all this hemoglobin was in the plasma rather than inside the cells, blood would be so "thick" that the heart would have a difficult time pumping it through. The thickness of blood is called viscosity. The greater the viscosity of blood, the more friction there is, and more pressure is needed to force blood through.

#### Functions

The main function is the transportation of oxygen throughout the body and the ability of the blood to carry out carbon dioxide which is called **carbamino – hemoglobin**. Maintaining the balance of blood is important. The balance can be measured by the acid and base levels in the blood. This is called **pH**. Normal pH of blood ranges between 7.35-7.45; this normal blood is called **Alkaline** (less acidic then water). A drop in pH is called **Acidic**. This condition is also called **Acidosis**. A jump in pH higher then 7.45 is called "**Alkalosis**". To maintain the homeostasis (or balance,) the blood has tiny molecules within the RBC that help prevent drops or increases from happening.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/2/24/Red_White_Blood_cells.jpg/220px-Red_White_Blood_cells.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

From left to right diagram of **Erythrocyte, Thrombocyte, and Leukocyte**

**Destruction**

Red blood cells are broken down and hemoglobin is released. The globin part of the hemoglobin is broken down into amino acid components, which in turn are recycled by the body. The iron is recovered and returned to the bone marrow to be reused. The heme portion of the molecule experiences a chemical change and then gets excreted as bile pigment (bilirubin) by the liver. Heme portion after being broken down contributes to the color of feces and your skin color changing after being bruised.

### White Blood Cells

**Shape**

White blood cells are different from red cells in the fact that they are usually larger in size 10-14 micrometers in diameter. White blood cells do not contain hemoglobin which in turn makes them translucent. Many times in diagrams or pictures white blood cells are represented in a blue color, mainly because blue is the color of the stain used to see the cells. White blood cells also have nucleii, that are some what segmented and are surrounded by electrons inside the membrane.

#### Functions

**White blood cells (leukocytes)** are also known as "WBC's". White blood cells are made in the bone marrow but they also divide in the blood and lymphatic systems. They are commonly amoeboid (cells that move or feed by means of temporary projections, called pseudopods (false feet), and escape the circulatory system through the capillary beds. The different types of WBC's are **Basophils**, **Eosinophils**, **Neutrophils**, **Monocytes**, **B-** and **T-cell lymphocytes**. Neutrophils, Eosinophils, and Basophils are all **granular** leukocytes. Lymphocytes and Monocytes are **agranular** leukocytes. Basophils store and synthesize histamine which is important in allergic reactions. They enter the tissues and become "mast cells" which help blood flow to injured tissues by the release of histamine. Eosinophils are chemotoxic and kill parasites. Neutrophils are the first to act when there is an infection and are also the most abundant white blood cells. Neutrophils fight bacteria and viruses by _phagocytosis_ which mean they engulf pathogens that may cause infection. The life span of a of Neutrophil is only about 12-48 hours. Monocytes are the biggest of the white blood cells and are responsible for rallying the cells to defend the body. Monocytes carry out phagocytosis and are also called macrophages. Lymphocytes help with our immune response. There are two Lymphocytes: the B- and T- cell. B-Lymphocytes produce antibodies that find and mark pathogens for destruction. T-Lymphocytes kill anything that they deem abnormal to the body.

WBCs are classified by phenotype which can be identified by looking at the WBCs under a microscope. The Granular phenotype are able to stain blue. The Agranular phenotype are able to stain red. Neutrophils make up 50-70% of Granular cells Eosinophils make up 2-4%, and Basophils 0-1%. Monocytes make up 2-8% of Agranular cells. B and T Lymphocytes make up 20-30%. As you can see, there is a great deal of differentiation between WBCs. These special cells help our bodies defend themselves against pathogens. Not only do they help our immune system but they remove toxins, wastes, and abnormal or damaged cells. Thus, we can say that WBCs' main function is being Phagocytic which means to engulf or swallow cells.

### Platelets

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/7/7f/Platelet_blood_bag.jpg/150px-Platelet_blood_bag.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

A 250 ml bag of newly collected platelets.

Platelets, also called thrombocytes, are membrane-bound cell fragments. Platelets have no nucleus,they are between one to two micrometers in diameter, and are about 1/10th to 1/20th as abundant as white blood cells. Less than 1% of whole blood consists of platelets. They result from fragmentation of large cells called Megakaryocytes - which are cells derived from stem cells in the bone marrow. Platelets are produced at a rate of 200 billion per day. Their production is regulated by the hormone called Thrombopoietin. The circulating life of a platelet is 8–10 days. The sticky surface of the platelets allow them to accumulate at the site of broken blood vessels to form a clot. This aids in the process of hemostasis ("blood stopping"). Platelets secrete factors that increase local platelet aggregation (e.g., Thromboxane A), enhance vasoconstriction (e.g., Serotonin), and promote blood coagulation (e.g., Thromboplastin).

## Hemostasis (Coagulation or Clotting)

Hemostasis is the natural process of stopping blood flow or loss of blood following an injury. (_hemo_ = blood; _stasis_ = standing). It has three stages: (1) vascular spasm, vasoconstriction, or intense contraction of blood vessels, (2) formation of a platelet plug and (3) blood clotting or coagulation. Once the flow of blood has been stopped, tissue repair can begin.

**Vascular spasm or Vasoconsriction:** In a normal individual, immediately after a blood vessel has been cut and endothelial cells are damaged, vasoconstriction occurs, thus slowing blood flow to the area. Smooth muscle in the vessel wall goes through spasms or intense contractions that constrict the vessel. If the vessels are small, spasms compress the inner walls together and may be able to stop the bleeding completely. If the vessels are medium to large-sized, the spasms slow down immediate outflow of blood, lessening the damage but still preparing the vessel for the later steps of hemostasis. These vascular spasms usually last for about 30 minutes, long enough for the next two stages of hemostasis to take place.

**Formation of a Platelet Plug:** Within 20 seconds of an injury, coagulation is initiated. Contrary to popular belief, clotting of a cut on the skin is not initiated by air or drying out, but by platelets adhering to and activated by collagen in the blood vessels endothelium. The activated platelets then release the contents of their granules, which contain a variety of substances that stimulate further platelet activation and enhance the hemostatic process.

When the lining of a blood vessel breaks and endothelial cells are damaged, revealing collagen proteins in the vessel wall, platelets swell, grow spiky extensions, and start clumping together. They start to stick to each other and the walls of the vessel. This continues as more platelets congregate and undergo these same transformations. This process results in a platelet plug that seals the injured area. If the injury is small, a platelet plug may be able to form and close it within several seconds. If the damage is more serious, the next step of blood clotting will take place. Platelets contain secretory granules. When they stick to the proteins in the vessel walls, they _degranulate_, thus releasing their products, which include ADP (adenosine diphosphate), serotonin, and thromboxane A2.

**A Blood Clot Forms:** If the platelet plug is not enough to stop the bleeding, the third stage of hemostasis begins: the formation of a blood clot. First, blood changes from a liquid to a gel. At least 12 substances called _clotting factors_ take part in a series of chemical reactions that eventually create a mesh of protein fibers within the blood. Each of the _clotting factors_ has a very specific function. We will discuss just three of the substances here: prothrombin, thrombin, and fibrin. Prothrombin and fibrinogen are proteins that are produced and deposited in the blood by the liver.

  * _Prothrombin_: When blood vessels are damaged, vessels and nearby platelets are stimulated to release a substance called _prothrombin activator_, which in turn activates the conversion of _prothrombin_, a plasma protein, into an enzyme called _thrombin_. This reaction requires calcium ions.
  * _Thrombin_: _Thrombin_ facilitates the conversion of a soluble plasma protein called _fibrinogen_ into long insoluble fibers or threads of the protein _fibrin_.
  * _Fibrin_: Fibrinogen is cleaved by thrombin to form its active form, "fibrin." Fibrin threads wind around the platelet plug at the damaged area of the blood vessel, forming an interlocking network of fibers and a framework for the clot. This net of fibers traps and helps hold platelets, blood cells and other molecules tight to the site of injury, functioning as the initial clot. This temporary fibrin clot can form in less than a minute, and usually does a good job of reducing the blood flow. Next, platelets in the clot begin to shrink, tightening the clot and drawing together the vessel walls. Usually, this whole process of clot formation and tightening takes less than a half hour.

The use of adsorbent chemicals, such as zeolites, and other hemostatic agents, are also being explored for use in sealing severe injuries quickly.

## ABO Group System

The **ABO blood group** is represented by substances on the surface of red blood cells (RBCs). These substances are important because they contain specific sequences of amino acid and carbohydrates which are antigenic. As well as being on the surface of RBCs, some of these antigens are also present on the cells of other tissues. A complete blood type describes the set of 29 substances on the surface of RBCs, and an individual's blood type is one of the many possible combinations of blood group antigens. Usually only the ABO blood group system and the presence or absence of the Rhesus D antigen (also known as the Rhesus factor or RH factor) are determined and used to describe the blood type. Over 400 different blood group antigens have been found, many of these being very rare. If an individual is exposed to a blood group antigen that is not recognized as self, the individual can become sensitized to that antigen; the immune system makes specific antibodies which binds specifically to a particular blood group antigen and an immunological memory against that particular antigen is formed. These antibodies can bind to antigens on the surface of transfused red blood cells (or other tissue cells) often leading to destruction of the cells by recruitment of other components of the immune system. Knowledge of a individual's blood type is important to identify appropriate blood for transfusion or tissue for organ transplantation.

### Surface Antigens

Several different RBC surface antigens stemming from one allele (or very closely linked genes) are collectively labeled as a blood group system (or blood group). The two most important blood group systems were discovered during early experiments with blood transfusion, the ABO group in 1901 and the Rhesus group in 1937 . These two blood groups are reflected in the common nomenclature A positive, O negative, etc. with letters referring to the ABO group and positive/negative to the presence/absence of the RhD antigen of the Rhesus group. Development of the Coombs test in 1945 and the advent of transfusion medicine led to discovery of more blood groups.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/5/51/Blood_Compatibility.svg/220px-Blood_Compatibility.svg.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Compatibility of blood types.

**Blood Group AB** individuals have both A and B antigens on the surface of their RBCs, and their blood serum does not contain any antibodies against either A or B antigen. Therefore, a individual with type AB blood can receive blood from any group (with AB being preferable), but can only donate blood to another group AB individual. AB blood is also known as "Universal receiver."

**Blood Group A** individuals have the A antigen on the surface of their RBCs, and blood serum containing IgM antibodies against the B antigen. Therefore, a group A individual can only receive blood from individuals of groups A or O (with A being preferable), and can donate blood to individuals of groups A or AB.

**Blood Group B** individuals have the B antigen on their surface of their RBCs, and blood serum containing IgM antibodies against the A antigen. Therefore, a group B individual can only receive blood from individuals of groups B or O (with B being preferabe), and can donate blood to individuals of groups B or AB.

**Blood group O** individuals do not have either A or B antigens on the surface of their RBCs, but their blood serum contains IgM antibodies against both A and B antigens. Therefore, a group O individual can only receive blood from a group O individual, but they can donate blood to individuals of any ABO blood group (ie A, B, O or AB). O blood is also know as "Universal donor."

### Inheritance

Blood types are inherited and represent contributions from both parents. The ABO blood type is controlled by a single gene with three alleles: i, IA, and IB. The gene encodes an enzyme that modifies the carbohydrate content of the red blood cell antigens.

IA gives type A, IB gives type B, i give types O

**Blood group inheritance**

Mother/Father **O** **A** **B** **AB**

**O**
O
O, A
O, B
A, B

**A**
O, A
O, A
O, A, B, AB
A, B, AB

**B**
O, B
O, A, B, AB
O, B
A, B, AB

**AB**
A, B
A, B, AB
A, B, AB
A, B, AB

IA and IB are dominant over i, so ii people have type O, IAIA or IAi have A, and IBIB or IBi have type B. IAIB people have both phenotypes because A and B are codominant, which means that type A and B parents can have an AB child. Thus, it is extremely unlikely for a type AB parent to have a type O child (it is not, however, direct proof of illegitimacy): the cis-AB phenotype has a single enzyme that creates both A and B antigens. The resulting red blood cells do not usually express A or B antigen at the same level that would be expected on common group A or B red blood cells, which can help solve the problem of an apparently genetically impossible blood group.

**Rh Factor**

Many people have the Rh Factor on the red blood cell. Rh carriers do not have the antibodies for the Rh Factor, but can make them if exposed to Rh. Most commonly Rh is seen when anti-Rh antibodies cross from the mothers placenta into the child before birth. The Rh Factor enters the child destroying the child's red blood cells. This is called Hemolytic Disease.

### Compatibility in Blood/Plasma Transfusions

Blood transfusions between donor and recipient of incompatible blood types can cause severe acute immunological reactions, hemolysis (RBCT destruction), renal failure, shock, and sometimes death. Antibodies can be highly active and can attack RBCs and bind components of the complement system to cause massive hemolysis of the transfused blood.

A patient should ideally receive their own blood or type-specific blood products to minimize the chance of a transfusion reaction. If time allows, the risk will further be reduced by cross-matching blood, in addition to blood typing both recipient and donor. Cross-matching involves mixing a sample of the recipient's blood with a sample of the donor's blood and checking to see if the mixture agglutinates, or forms clumps. Blood bank technicians usually check for agglutination with a microscope, and if it occurs, that particular donor's blood cannot be transfused to that particular recipient. Blood transfusion is a potentially risky medical procedure and it is vital that all blood specimens are correctly identified, so in cross-matching labeling is standardized using a barcode system known as ISBT 128.

**Plasma compatibility table**

Donor Recipient

O A B AB

O
OK
OK
OK
OK

A
OK
OK

B
OK
OK

AB
OK

When considering a plasma transfusion, keep in mind that plasma carries antibodies and no antigens. For example you can't give type O plasma to a type A, B or AB, because a person with type O blood has A and B antibodies and the recipient would have an immune response. On the other hand an AB donor could give plasma to anyone, since they have no antibodies.

The table to the right is for plasma transfusions, and it's just the opposite for RBC transfusions. It doesn't take the Rh factor into effect, though, because most people don't have antibodies for the Rhesus factor (it only happens upon exposure).

  


#### Hemolytic Disease of the Newborn

Often a pregnant woman carries a fetus with a different blood type to herself, and sometimes the mother forms antibodies against the red blood cells of the fetus, leading to low fetal blood counts, a condition known as hemolytic disease of the newborn.

Hemolytic disease of the newborn, (also known as HDN) is an alloimmune condition that develops in a fetus when the IgG antibodies produced by the mother and passing through the placenta include ones which attack the red blood cells in the fetal circulation. The red cells are broken down and the fetus can develop reticulocytosis and anemia. The fetal disease ranges from mild to very severe and fetal death from heart failure - hydrops fetalis - can occur. When the disease is moderate or severe many erythroblasts are present in the fetal blood and so these forms of the disease can be called erythroblastosis fetalis.

Before birth, options for treatment include intrauterine transfusion or early induction of labor when pulmonary maturity has been attained, fetal distress is present, or 35 to 37 weeks of gestation have passed. The mother may also undergo plasma exchange to reduce the circulating levels of antibody by as much as 75%.

After birth, treatment depends on the severity of the condition, but could include temperature stabilization and monitoring, phototherapy, transfusion with compatible packed red blood, exchange transfusion with a blood type compatible with both the infant and the mother, sodium bicarbonate for correction of acidosis and/or assisted ventilation.

Rh negative mothers who have had a pregnancy with or are pregnant with a Rh positive infant, are given Rh immune globulin (RhIG) also known as Rhogam, during pregnancy and after delivery to prevent sensitization to the D antigen. It works by binding any fetal red cells with the D antigen before the mother is able to produce an immune response and form anti-D IgG. A drawback to pre-partum administration of RhIG is that it causes a positive antibody screen when the mother is tested which is indistinguishable from immune reasons for antibody production.

## Diseases of the Blood

### Von Willebrand Disease

The most common inherited bleeding disorder, von Willebrand disease affects both men and women equally. Von Willebrand disease is similar to hemophilia in that it involves a definciency in the ability of blood to clot properly. Those affect by von Willebrand disease will have one or more of the following- low levels of von Willebrand factor (a protein that helps the blood to clot), and/or their von Willebrand factor doesn't work properly. While it is mostly an inherited disease (with factors contributed by both parents), von Willebrand disease may be an aquired syndrome in rare cases.

There are three types of von Willebrand disease: Type 1, which is the mildest and most common form of the disease; Type 2, which has four subtypes (2A, 2B, 2M, and 2N) and ranges from mild to moderate in severity; and finally, Type 3, which is very rare and is the most severe form.

Type 1

In type 1 von Willebrand disease, there is a low level of von Willebrand factor. The level of factor VIII may also be lower than normal. This is the mildest and most common form of the disease. About 3 out of 4 people diagnosed with von Willebrand disease have type 1.

Type 2

In type 2 von Willebrand disease, a defect in von Willebrand factor causes it to not work properly. Type 2 is divided into 2A, 2B, 2M, and 2N. Each is treated differently, so knowing the exact type is important.

People with type 1 and type 2 von Willebrand disease may have the following mild-to-moderate bleeding symptoms: easy bruising, nosebleeds, bleeding from the gums after a dental procedure, heavy menstrual bleeding in women, blood in their stools or urine (from bleeding in the intestines, stomach, kidneys or bladder), excessive bleeding after a cut or other accident or surgery.

Type 3

People with type 3 von Willebrand disease usually have no von Willebrand factor and very low factor VIII. Type 3 is severe and very rare.

Symptoms of type 3 von Willebrand disease might include any of the symptoms of types 1 and 2, and also include severe bleeding episodes for no reason, which can be life threatening if not treated immediately. Bleeding into soft tissues or joints (hemarthrosis), causing severe pain and swelling, is another symptom.

Treatment

Many people with von Willebrand disease do not require treatment to manage the disease. However, if treatment is ncessary, it may include a range of different interventions depending on the severity. These involve medicine to increase the level of von Willebrand factor in the blood (DDAVP), medicine to prevent the breakdown of clots (called antifibrinolytic drugs), medicine to control heavy menstrual bleeding in women (often birth control pills), or injection of clotting factor concentrates (containing von Willebrand factor and factor VIII).

### Disseminated Intravascular Coagulation

Disseminated intravascular coagulation (DIC), also called consumptive coagulopathy, is a pathological process in the body where the blood starts to coagulate throughout the whole body. This depletes the body of its platelets and coagulation factors, and there is a paradoxically increased risk of hemorrhage. It occurs in critically ill patients, especially those with Gram-negative sepsis (particularly meningococcal sepsis) and acute promyelocytic leukemia.

### Hemophilia

Hemophilia is a disease where there is low or no blood protein, causing an inability to produce blood clots. There are two types of Hemophilia: Type A, which is a deficiency in factor VIII and Type B, (Christmas disease) a deficiency on factor IX. Because people with hemophilia do not have the ability to make blood clots, even a little cut may kill them, or the smallest bump or jar to the body could cause severe bruising that doesn't get better for months.

Hemophilia is passed down from mothers to their sons. Hemophilia is sometimes known as the "Royal Disease". This is because Queen Victoria, Queen of England (1837-1901), was a carrier of hemophilia. The hemophilia disease was passed down to her son Leopold who ended up dying at age 31. Queen Victoria also had two daughters who were carriers. These daughters passed hemophilia into the Spanish, German, and Russian royal families. One of the most famous stories is that of the Russian royal family. Alexandra, granddaughter to Queen Victoria, married Nicholas (Tsar of Russia in the 1900s). Alexandra was a carrier of the disease and passed the disease to their first son, Tsarevich Alexi, who was heir to the throne of Russia. The family tried to keep their son's secret from the people, but Alexi suffered with serious bruises and extreme pain. The family found help from a monk named Rasputin. He kept their secret and gained a great deal of power over the family, making them think he was their only hope. During this time of great turmoil in Russia, Nicholas and Alexandra spent most of their attentions on their son, and not on the people. It wasn't long before the Bolshevik Revolution of 1917 began.

  


### Factor V Leiden

The opposite of Hemophilia, Factor V Leiden is the name given to a variant of human factor V that causes a hypercoagulability disorder. In this disorder the Leiden variant of factor V, cannot be inactivated by activated protein C. Factor V Leiden is the most common hereditary hypercoagulability disorder amongst Eurasians. It is named after the city Leiden (The Netherlands), where it was first identified in 1994 by Prof R. Bertina et al. Those that have it are at a slightly higher risk of developing blood clots than those without. Those that test positive for factor V should avoid (oral contraceptives, obesity, smoking, and high blood pressure.)

### Anemia

Anemia (AmE) or anaemia (BrE), from the Greek (Ἀναιμία) meaning "without blood", refers to a deficiency of red blood cells (RBCs) and/or hemoglobin. This results in a reduced ability of blood to transfer oxygen to the tissues, causing hypoxia. Since all human cells depend on oxygen for survival, varying degrees of anemia can have a wide range of clinical consequences. Hemoglobin (the oxygen-carrying protein in the red blood cells) has to be present to ensure adequate oxygenation of all body tissues and organs.

The three main classes of anemia include excessive blood loss (acutely such as a hemorrhage or chronically through low-volume loss), excessive blood cell destruction (hemolysis) or deficient red blood cell production (ineffective hematopoiesis). In menstruating women, dietary iron deficiency is a common cause of deficient red blood cell production.

### Sickle cell

![](../../../upload.wikimedia.org/wikipedia/commons/9/92/Sicklecells.jpg)

Image of RBC's with Sickle Cell mutations.

Sickle-cell disease is a general term for a group of genetic disorders caused by sickle hemoglobin (Hgb S or Hb S). In many forms of the disease, the red blood cells change shape upon deoxygenation because of polymerization of the abnormal sickle hemoglobin. This process damages the red blood cell membrane, and can cause the cells to become stuck in blood vessels. This deprives the downstream tissues of oxygen and causes ischemia and infarction. The disease is chronic and lifelong. Individuals are most often well, but their lives are punctuated by periodic painful attacks. In addition to periodic pain, there may be damage of internal organs, and/or stroke. Lifespan is often shortened with sufferers living to an average of 40 years. It is common in people from parts of the world where malaria is or was common, especially in sub-Saharan Africa or in descendants of those peoples.

Genetics: Sickle-cell disease is inherited in the autosomal recessive pattern, depicted above. The allele responsible for sickle cell anemia is autosomal recessive. A person who receives the defective gene from both father and mother develops the disease; a person who receives one defective and one healthy allele remains healthy, but can pass on the disease and is known as a carrier. If two parents who are carriers have a child, there is a 1-in-4 chance of their child developing the illness and a 1-in-2 chance of their child just being a carrier.

### Polycythemia

Polycythemia is a condition in which there is a net increase in the total circulating erythrocyte (red blood cell) mass of the body. There are several types of polycythemia.

Primary Polycythemia

In primary polycythemia, there may be 8 to 9 million and occasionally 11 million erythrocytes per cubic millimeter of blood (a normal range for adults is 4-5 million), and the hematocrit may be as high as 70 to 80%. In addition, the total blood volume can increase to as much as twice as normal. The entire vascular system can become markedly engorged with blood, and circulation times for blood throughout the body can increase up to twice the normal value. The increased numbers of erythrocytes can increase of the viscosity of the blood to as much as five times normal. Capillaries can become plugged by the very viscous blood, and the flow of blood through the vessels tends to be extremely sluggish.

As a consequence of the above, people with untreated Polycythemia are at a risk of various thrombotic events (deep venous thrombosis, pulmonary embolism), heart attack and stroke, and have a substantial risk of Budd-Chiari syndrome (hepatic vein thrombosis). The condition is considered chronic; no cure exists. Symptomatic treatment (see below) can normalize the blood count and most patients can live a normal life for years.

Secondary polycythemia

Secondary polycythemia is caused by either appropriate or inappropriate increases in the production of erythropoietin that result in an increased production of erythrocytes. In secondary polycythemia, there may be 6 to 8 million and occasionally 9 million erythrocytes per cubic millimeter of blood. A type of secondary polycythemia in which the production of erythropoietin increases appropriately is called physiologic polycythemia. Physiologic polycythemia occurs in individuals living at high altitudes (4275 to 5200 meters), where oxygen availability is less than at sea level. Many athletes train at higher altitudes to take advantage of this effect — a legal form of blood doping. Actual polychthemia sufferers have been known to use their condition as an athletic advantage for greater stamina.

Other causes of secondary polycythemia include smoking, renal or liver tumors, or heart or lung diseases that result in hypoxia. Endocrine abnormalities, prominently including pheochromocytoma and adrenal adenoma with Cushing's Syndrome, are also secondary causes. Athletes and bodybuilders who abuse anabolic steroids or erythropoietin may develop secondary polycythemia.

Relative polycythemia

Relative polycythemia is an apparent rise of the erythrocyte level in the blood; however, the underlying cause is reduced blood plasma. Relative polycythemia is often caused by fluid loss i.e. burns, dehydration and stress polycythemia.

### Leukemia

Leukemia is a cancer of the blood or bone marrow characterized by an abnormal proliferation of blood cells, usually white blood cells (leukocytes). It is part of the broad group of diseases called hematological neoplasms. Damage to the bone marrow, by way of displacing the normal marrow cells with increasing numbers of malignant cells, results in a lack of blood platelets, which are important in the blood clotting process. This means people with leukemia may become bruised, bleed excessively, or develop pin-prick bleeds (petechiae).

White blood cells, which are involved in fighting pathogens, may be suppressed or dysfunctional, putting the patient at the risk of developing infections. The red blood cell deficiency leads to anaemia, which may cause dyspnea. All symptoms may also be attributable to other diseases; for diagnosis, blood tests and a bone marrow biopsy are required.

## Glossary

**Albumin:** a major blood protein responsible for the maintenance of osmotic (water) pressure in the blood

**Anemia:** a deficiency of red blood cells or hemoglobin caused by lack of iron, folic acid or vitamin B12 in the diet, or by red blood cell destruction; associated with decreased ability of blood to carry oxygen

**B-Cell:** cell responsible for the distribution of antibodies

**Basophil:** this white blood cell enters damaged tissues and releases a histamine and other chemicals that promote inflammation in the body to fight pathogens

**Blood:** the means and transport system of the body used in carrying elements - nutrition, waste, heat - from one location in the body to another by way of blood vessels

**Eosinophil:** white blood cell that is involved in the immune response against parasitic worms (such as tapeworms and roundworms). Named because it stains with the red dye "eosin."

**Factor V Leiden** most common genetic hypercoagulability disorder.

**Formed Elements:** the red blood cells, white blood cells and platelets found in blood

**Hematocrit:** measurement of the % of red blood cells found in blood

**Hemoglobin (Hb):** iron-containing pigment in red blood cells that combines with and transports oxygen

**Hemophilia:** genetic disorder in which the affected individual may have uncontrollable bleeding; blood does not clot

**Hemostasis:** the process by which blood flow is stopped; also describes the clotting of blood

**Lymphocytes:** cells of the Lymphatic system, provide defense against specific pathogen or toxins

**Monocytes:** The largest white blood cell. Becomes a macrophage when activated. Engulfs pathogens and debris through phagocytosis, also involved in presenting antigens to B and T lymphocytes.

**Neutrophils:** the most common white blood cell; they are phagocytic and engulf pathogens or debris in the tissues; also release cytotoxic enzymes and chemicals to kill pathogens

**NK-Cells:** also known as "Natural Killer Cells", these T lymphocytes are responsible for surveillance and detection of abnormal tissue cells; important in preventing cancer

**Phagocytosis:** process by which amoeboid-like cells engulf and ingest, and thereby destroy, foreign matter or material

**T-Cell:** cells that mediate by coordinating the immune system and enter the peripheral tissues. They can attack foreign cells directly and control the activities of other lymphocytes ,

## Review Questions

Answers for these questions can be found [here](http://en.wikibooks.org/wiki/Human_Physiology/Appendix_1:_answers_to_review_questions#Blood_Physiology)

1.Taking aspirin every day can reduce the risk of heart disease because:

    A) it is a powerful vasodilator
    B) it blocks pain receptors in heart tissue
    C) it stops ventricular fibrillation
    D) it loosens plaque on arterial walls
    E) it prevents platelet clumping

2\. A hematocrit measures percentage of:

    A) White blood cells
    B) Plasma
    C) Platelets
    D) Red blood cells

3\. Fred's blood type is O- and Ginger's is B+. Fred and Ginger have a son who is AB+. What do you conclude?

    A) If they have a second child Ginger needs to have RhoGam shot
    B) There is no risk to a second child, unless it has a negative blood type
    C) If the child needs a blood transfusion Fred could provide it safely, but not Ginger
    D) Fred is not the boy’s father

4\. Which blood component plays the largest role in maintaining the osmotic pressure of blood?

    A) albumin
    B) carbon dioxide
    C) white blood cells
    D) fibrinogen
    E) globulins

5\. If you hold your breath for one minute

    A) The kidneys will increase sodium ion reabsorption
    B) Hydrogen-ion concentration in the blood will increase
    C) Your heart rate will greatly slow
    D) Hemoglobin will bind to oxygen more strongly

6\. Most of the carbon dioxide produced by tissues is transported to the lungs as:

    A) Small gas bubbles in the plasma
    B) Gas bound to hemoglobin in the red blood cells
    C) Bicarbonate ions in the plasma
    D) Gas bound to white blood cells and albumin
    E) Gas transported through the lymphatic system

7\. To prevent blood loss after a tissue injury, blood vessels first

    A) Form a platelet plug
    B) Form a clot
    C) Initiate the coagulation cascade
    D) Constrict and form barriers

8\. You take a blood sample from a male cyclist at the end of a long race. The hematocrit is 60%. The most likely conclusion is:

    A) This is within normal range for most adult males
    B) This cyclist is anemic
    C) This low of a hematocrit could indicate liver damage or leukemia
    D) The cyclist is dehydrated
    E) The cyclist has been taking pharmaceutical erythropoietin

9\. In a normal blood sample, which of the following cells will be the most abundant?

    A) Neutrophils
    B) Basophils
    C) Eosinophils
    D) Monocytes
    E) Lymphocytes

10\. A bag of donated blood does not clot because

    A) There is not enough oxygen
    B) It cannot dry out
    C) It is kept refrigerated
    D) There is no free calcium
    E) All of the above

11\. What is the primary function of bood?

    A) Supply nutrients to tissues
    B) Remove waste products
    C) To keep your body at one consistent temperature
    D) A and B
    E) B and C

12\. What is the main component of the Red blood cell?

    A) Albumin
    B) Globulins
    C) Hemoglobin
    D) Nucleus

  


# The Cardiovascular System

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/5/57/Heart_frontally_PDA.jpg/500px-Heart_frontally_PDA.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Model of a human heart

## Introduction

The heart is the life-giving, ever-beating muscle in your chest. From inside the womb until death, the thump goes on. The heart for the average human will contract about 3 billion times; never resting, never stopping to take a break except for a fraction of a second between beats. At 80 years of age, a person's heart will continue to beat an average of 100,000 times a day. Many believe that the heart is the first organ to become functional. Within weeks of conception the heart starts its mission of supplying the body with nutrients even though the embryo is no bigger than a capital letter on this page. The primary function of the heart is to pump blood through the arteries, capillaries, and veins. There are an estimated 60,000 miles of vessels throughout an adult body. Blood transports oxygen, nutrients, disease causing viruses, bacteria, hormones and has other important functions as well. The heart is the pump that keeps blood circulating properly. Americans today have many options to take care of their heart and circulatory system. Expanding medical technology has made it much easier to do so. This chapter is dedicated to the heart and its many parts.

## The Heart

The heart is a hollow, muscular organ about the size of a fist. It is responsible for pumping blood through the blood vessels by repeated, rhythmic contractions. The heart is composed of cardiac muscle, an involuntary muscle tissue that is found only within this organ. The term "cardiac" (as in cardiology) means "related to the heart” and comes from the Greek word kardia, for "heart." It has a four-chambered, double pump and is located in the thoracic cavity between the lungs. The cardiac muscle is self-exciting, meaning it has its own conduction system. This is in contrast with skeletal muscle, which requires either conscious or reflex nervous stimuli. The heart's rhythmic contractions occur spontaneously, although the frequency or heart rate can be changed by nervous or hormonal influence such as exercise or the perception of danger.

  


### Endocardium

The endocardium is the innermost lining of the heart which consists of the endothelial cells forming a smooth membrane in places, and a pocked and tribeculated surface in others (mainly the ventricles, or lower pumping chambers).

### Myocardium

The myocardium is the muscular tissue of the heart. The myocardium is composed of specialized cardiac muscle cells with an ability not possessed by muscle tissue elsewhere in the body. Cardiac muscle, like other muscles, can contract, but it can also conduct electricity, like nerves. The blood to the myocardium is supplied by the coronary arteries. If these arteries are occluded by atherosclerosis and/or thrombosis, this can lead to angina pectoris or myocardial infarction due to ischemia (lack of oxygen). Failure of the heart to contract properly (for various reasons) is termed heart failure, generally leading to fluid retention, edema, pulmonary edema, renal insufficiency, hepatomegaly, a shortened life expectancy and decreased quality of life.

### Epicardium

The outer most layer next to the myocardium is known as the Epicardium. This is the outer layer after endocardium and myocardium that consists of a thin layer of connective tissue and fat.

### Pericardium

The pericardium is the thick, membranous sac that surrounds the heart. It protects and lubricates the heart. There are two layers to the pericardium: the fibrous pericardium and the serous pericardium. The serous pericardium is divided into two layers; in between these two layers there is a space called the pericardial cavity.

### Heart Chambers

The heart has four chambers, two atria and two ventricles. The atria are smaller with thin walls, while the ventricles are larger and much stronger.

  


#### Atrium

There are two atria on either side of the heart. On the right side is the atrium that contains blood which is poor in oxygen. The left atrium contains blood which has been oxygenated and is ready to be sent to the body. The right atrium receives de-oxygenated blood from the superior vena cava and inferior vena cava. The left atrium receives oxygenated blood from the left and right pulmonary veins. Atria facilitate circulation primarily by allowing uninterrupted venous flow to the heart, preventing the inertia of interrupted venous flow that would otherwise occur at each ventricular systole.

  


#### Ventricles

The ventricle is a heart chamber which collects blood from an atrium and pumps it out of the heart. There are two ventricles: the right ventricle pumps blood into the pulmonary circulation for the lungs, and the left ventricle pumps blood into the aorta for systemic circulation to the rest of the body. Ventricles have thicker walls than the atria, and thus can create the higher blood pressure. Comparing the left and right ventricle, the left ventricle has thicker walls because it needs to pump blood to the whole body. This leads to the common misconception that the heart lies on the left side of the body.

### Septum

The inter ventricular septum (ventricular septum, or during development septum inferius) is the thick wall separating the lower chambers (the ventricles) of the heart from one another. The ventricular septum is directed backward and to the right, and is curved toward the right ventricle. The greater portion of it is thick and muscular and constitutes the muscular ventricular septum. Its upper and posterior part, which separates the aortic vestibule from the lower part of the right atrium and upper part of the right ventricle, is thin and fibrous, and is termed the membranous ventricular septum.hi

### Valves

The two atrioventricular (AV) valves are one-way valves that ensure that blood flows from the atria to the ventricles, and not the other way. The two semilunar (SL) valves are present in the arteries leaving the heart; they prevent blood from flowing back into the ventricles. The sound heard in a heart beat is the heart valves shutting. The right AV valve is also called the tricuspid valve because it has three flaps. It is located between the right atrium and the right ventricle. The tricuspid valve allows blood to flow from the right atrium into the right ventricle when the heart is relaxed during diastole. When the heart begins to contract, the heart enters a phase called systole, and the atrium pushes blood into the ventricle. Then, the ventricle begins to contract and blood pressure inside the heart rises. When the ventricular pressure exceeds the pressure in the atrium, the tricuspid valve snaps shut. The left AV valve is also called the bicuspid valve because it has two flaps. It is also known as the mitral valve due to the resemblance to a bishop's mitre (liturgical headdress). This valve prevents blood in the left ventricle from flowing into the left atrium. As it is on the left side of the heart, it must withstand a great deal of strain and pressure; this is why it is made of only two cusps, as a simpler mechanism entails a reduced risk of malfunction. There are two remaining valves called the Semilunar Valves. They have flaps that resemble half moons. The pulmonary semilunar valve lies between the right ventricle and the pulmonary trunk. The aortic semilunar valve is located between the ventricle and the aorta.

### Subvalvular Apparatus

The chordae tendinae are attached to papillary muscles that cause tension to better hold the valve. Together, the papillary muscles and the chordae tendinae are known as the subvalvular apparatus. The function of the subvalvular apparatus is to keep the valves from prolapsing into the atria when they close. The subvalvular apparatus have no effect on the opening and closing of the valves. This is caused entirely by the pressure gradient across the valve.

### Complications with the Heart

The most common congenital abnormality of the heart is the bicuspid aortic valve. In this condition, instead of three cusps, the aortic valve has two cusps. This condition is often undiagnosed until the person develops calcific aortic stenosis. Aortic stenosis occurs in this condition usually in patients in their 40s or 50s, an average of 10 years earlier than in people with normal aortic valves. Another common complication of rheumatic fever is thickening and stenosis (partial blocking) of the mitral valve. For patients who have had rheumatic fever dentists are advised to prophylactically administer antibiotics prior to dental work to prevent bacterial endocarditis that occurs when bacteria from the teeth enter the circulation and attach to damaged heart valves.

The aortic valve is a semilunar valve, but it´s called bicuspid because of it´s regular three "cusps" or "semilunar" valves, and is not to be confused with the left atrioventricular valve, which is more commonly called the mitral valve, and is one of the two cuspidal valves.

## Passage of Blood Through the Heart

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/e/e5/Diagram_of_the_human_heart_\(cropped\).svg/300px-Diagram_of_the_human_heart_\(cropped\).svg.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Diagram of the human heart

While it is convenient to describe the flow of the blood through the right side of the heart and then through the left side, it is important to realize that both atria contract at the same time and that both ventricles contract at the same time. The heart works as two pumps, one on the right and one on the left that works simultaneously. The right pump pumps the blood to the lungs or the pulmonary circulation at the same time that the left pump pumps blood to the rest of the body or the systemic circulation. Venous blood from systemic circulation (deoxygenated) enters the right atrium through the superior and inferior vena cava. The right atrium contracts and forces the blood through the tricuspid valve (right atrioventricular valve) and into the right ventricles. The right ventricles contract and force the blood through the pulmonary semilunar valve into the pulmonary trunk and out the pulmonary artery. This takes the blood to the lungs where the blood releases carbon dioxide and receives a new supply of oxygen. The new blood is carried in the pulmonary veins that take it to the left atrium. The left atrium then contracts and forces blood through the left atrioventricular, bicuspid, or mitral, valve into the left ventricle. The left ventricle contracts forcing blood through the aortic semilunar valve into the ascending aorta. It then branches to arteries carrying oxygen rich blood to all parts of the body.

  


### Blood Flow After the Heart

Aorta-Arteries-Arterioles-Capillaries-Venules-Veins-Vena Cava

### Blood Flow Through Capillaries

From the arterioles, the blood then enters one or more capillaries. The walls of capillaries are so thin and fragile that blood cells can only pass in single file. Inside the capillaries, exchange of oxygen and carbon dioxide takes place. Red blood cells inside the capillary releases their oxygen which passes through the wall and into the surrounding tissue. The tissue then releases waste, such as carbon dioxide, which then passes through the wall and into the red blood cells.

## The Circulatory System

The circulatory system is extremely important in sustaining life. It’s proper functioning is responsible for the delivery of oxygen and nutrients to all cells, as well as the removal of carbon dioxide, waste products, maintenance of optimum pH, and the mobility of the elements, proteins and cells, of the immune system. In developed countries, the two leading causes of death, myocardial infarction and stroke are each direct results of an arterial system that has been slowly and progressively compromised by years of deterioration.

### Arteries

Arteries are muscular blood vessels that carry blood away from the heart, oxygenated and deoxygenated blood . The pulmonary arteries will carry deoxygenated blood to the lungs and the sytemic arteries will carry oxygenated blood to the rest of the body. Arteries have a thick wall that consists of three layers. The inside layer is called the endothelium, the middle layer is mostly smooth muscle and the outside layer is connective tissue. The artery walls are thick so that when blood enters under pressure the walls can expand.

#### Arterioles

An arteriole is a small artery that extends and leads to capillaries. Arterioles have thick smooth muscular walls. These smooth muscles are able to contract (causing vessel constriction) and relax (causing vessel dilation). This contracting and relaxing affects blood pressure; the higher number of vessels dilated, the lower blood pressure will be. Arterioles are just visible to the naked eye.

### Capillaries

![Illu capillary.jpg](../../../upload.wikimedia.org/wikipedia/commons/d/da/Illu_capillary.jpg)

Capillaries are the smallest of a body’s vessels; they connect arteries and veins, and most closely interact with tissues. They are very prevalent in the body; total surface area is about 6,300 square meters. Because of this, no cell is very far from a capillary, no more than 50 micrometers away. The walls of capillaries are composed of a single layer of cells, the endothelium, which is the inner lining of all the vessels. This layer is so thin that molecules such as oxygen, water and lipids can pass through them by diffusion and enter the tissues. Waste products such as carbon dioxide and urea can diffuse back into the blood to be carried away for removal from the body.

The "capillary bed" is the network of capillaries present throughout the body. These beds are able to be “opened” and “closed” at any given time, according to need. This process is called autoregulation and capillary beds usually carry no more than 25% of the amount of blood it could hold at any time. The more metabolically active the cells, the more capillaries it will require to supply nutrients.

### Veins

Veins carry blood to the heart. The pulmonary veins will carry oxygenated blood to the heart awhile the systemic veins will carry deoxygenated to the heart. Most of the blood volume is found in the venous system; about 70% at any given time. The veins outer walls have the same three layers as the arteries, differing only because there is a lack of smooth muscle in the inner layer and less connective tissue on the outer layer. Veins have low blood pressure compared to arteries and need the help of skeletal muscles to bring blood back to the heart. Most veins have one-way valves called venous valves to prevent backflow caused by gravity. They also have a thick collagen outer layer, which helps maintain blood pressure and stop blood pooling. If a person is standing still for long periods or is bedridden, blood can accumulates in veins and can cause varicose veins. The hollow internal cavity in which the blood flows is called the lumen. A muscular layer allows veins to contract, which puts more blood into circulation. Veins are used medically as points of access to the blood stream, permitting the withdrawal of blood specimens (venipuncture) for testing purposes, and enabling the infusion of fluid, electrolytes, nutrition, and medications (intravenous delivery).

### Venules

A venule is a small vein that allows deoxygenated blood to return from the capillary beds to the larger blood veins, except in the pulmonary circuit were the blood is oxygenated. Venules have three layers; they have the same makeup as arteries with less smooth muscle, making them thinner.

## The Cardiovascular Pathways

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/7/74/Grafik_blutkreislauf.jpg/220px-Grafik_blutkreislauf.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Human circulatory system. Arteries are shown in red, veins blue.

The double circulatory system of blood flow refers to the separate systems of pulmonary circulation and the systemic circulation in amphibians, birds and mammals (including humans.) In contrast, fishes have a single circulation system. For instance, the adult human heart consists of two separated pumps, the right side with the right atrium and ventricle (which pumps deoxygenated blood into the pulmonary circulation), and the left side with the left atrium and ventricle (which pumps oxygenated blood into the systemic circulation). Blood in one circuit has to go through the heart to enter the other circuit. Blood circulates through the body two to three times every minute. In one day, the blood travels a total of 19,000 km (12,000 miles), or four times the distance across the U.S. from coast to coast.

### The Pulmonary Circuit

In the pulmonary circuit, blood is pumped to the lungs from the right ventricle of the heart. It is carried to the lungs via pulmonary arteries. At lungs, oxygen in the alveolae diffuses to the capillaries surrounding the alveolae and carbon dioxide inside the blood diffuses to the alveolae. As a result, blood is oxygenated which is then carried to the heart's left half -to the left atrium via pulmonary veins. Oxygen rich blood is prepared for the whole organs and tissues of the body. This is important because mitochondria inside the cells should use oxygen to produce energy from the organic compounds.

### The Systemic Circuit

The systemic circuit supplies oxygenated blood to the organ system. Oxygenated blood from the lungs is returned to the left atrium, then the ventricle contracts and pumps blood into the aorta. Systemic arteries split from the aorta and direct blood into the capillaries. Cells consume the oxygen and nutrients and add carbon dioxide, wastes, enzymes and hormones. The veins drain the deoxygenated blood from the capillaries and return the blood to the right atrium.

### Aorta

The aorta is the largest of the arteries in the systemic circuit. The blood is pumped from the left ventricle into the aorta and from there it branches to all parts of the body. The aorta is an elastic artery, and as such is able to distend. When the left ventricle contracts to force blood into the aorta, the aorta expands. This stretching gives the potential energy that will help maintain blood pressure during diastole, as during this time the aorta contracts passively.

### Superior Venae Cavae

The superior vena cava (SVC) is a large but short vein that carries de-oxygenated blood from the upper half of the body to the heart's right atrium. It is formed by the left and right brachiocephalic veins (also referred to as the innominate veins) which receive blood from the upper limbs and the head and neck. The azygous vein (which receives blood from the ribcage) joins it just before it enters the right atrium.

### Inferior Venae Cavae

The inferior vena cava (or IVC) is a large vein that carries de-oxygenated blood from the lower half of the body into the heart. It is formed by the left and right common iliac veins and transports blood to the right atrium of the heart. It is posterior to the abdominal cavity, and runs along side of the vertebral column on its right side.

### Coronary Arteries

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/4/46/Gray492.png/220px-Gray492.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Heart showing the Coronary Arteries

Heart showing the Coronary Arteries The coronary circulation consists of the blood vessels that supply blood to, and remove blood from, the heart muscle itself. Although blood fills the chambers of the heart, the muscle tissue of the heart, or myocardium, is so thick that it requires coronary blood vessels to deliver blood deep into the myocardium. The vessels that supply blood high in oxygen to the myocardium are known as coronary arteries. The vessels that remove the deoxygenated blood from the heart muscle are known as cardiac veins. The coronary arteries that run on the surface of the heart are called epicardial coronary arteries. These arteries, when healthy, are capable of auto regulation to maintain coronary blood flow at levels appropriate to the needs of the heart muscle. These relatively narrow vessels are commonly affected by atherosclerosis and can become blocked, causing angina or a heart attack. The coronary arteries are classified as "end circulation", since they represent the only source of blood supply to the myocardium: there is very little redundant blood supply, which is why blockage of these vessels can be so critical. In general there are two main coronary arteries, the left and right. • Right coronary artery • Left coronary artery Both of these arteries originate from the beginning (root) of the aorta, immediately above the aortic valve. As discussed below, the left coronary artery originates from the left aortic sinus, while the right coronary artery originates from the right aortic sinus. Four percent of people have a third, the posterior coronary artery. In rare cases, a patient will have one coronary artery that runs around the root of the aorta.

### Hepatic Veins

In human anatomy, the hepatic veins are the blood vessels that drain de-oxygenated blood from the liver and blood cleaned by the liver (from the stomach, pancreas, small intestine and colon) into the inferior vena cava. They arise from the substance of the liver, more specifically the central vein of the liver lobule. They can be differentiated into two groups, the upper group and lower group. The upper group of three typically arises from the posterior aspect of the liver and drain the quadrate lobe and left lobe. The lower group rise from the right lobe and caudate lobe, are variable in number, and are typically smaller than those in the upper group. None of the hepatic veins have valves.

## Cardiac Cycle

Cardiac cycle is the term used to describe the relaxation and contraction that occur, as a heart works to pump blood through the body. Heart rate is a term used to describe the frequency of the cardiac cycle. It is considered one of the four vital signs. Usually it is calculated as the number of contractions (heart beats) of the heart in one minute and expressed as "beats per minute" (bpm). When resting, the adult human heart beats at about 70 bpm (males) and 75 bpm (females), but this rate varies between people. However, the reference range is nominally between 60 bpm (if less termed bradycardia) and 100 bpm (if greater, termed tachycardia). Resting heart rates can be significantly lower in athletes, and significantly higher in the obese. The body can increase the heart rate in response to a wide variety of conditions in order to increase the cardiac output (the amount of blood ejected by the heart per unit time). Exercise, environmental stressors or psychological stress can cause the heart rate to increase above the resting rate. The pulse is the most straightforward way of measuring the heart rate, but it can be deceptive when some strokes do not lead to much cardiac output. In these cases (as happens in some arrhythmias), the heart rate may be considerably higher than the pulse. Every single 'beat' of the heart involves three major stages: atrial systole, ventricular systole and complete cardiac diastole. Throughout the cardiac cycle, the blood pressure increases and decreases. As ventricles contract the pressure rise, causing the AV valves to slam shut.

### Systole

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/3/34/Heart_systole.svg/200px-Heart_systole.svg.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

The heart in the systole phase.

The heart in the systole phase. Systole, or contraction, of the heart is initiated by the electrical cells of the sinoatrial node, which is the heart's natural pacemaker. These cells are activated spontaneously by depolarization of their membranes beyond a certain threshold for excitation. At this point, voltage-gated calcium channels on the cell membrane open and allow calcium ions to pass through, into the sarcoplasm, or interior, of the muscle cell. Some calcium ions bind to receptors on the sarcoplasmic reticulum causing an influx of calcium ions into the sarcoplasm. The calcium ions bind to the troponin, causing a conformation change, breaking the bond between the protein tropomyosin, to which the troponin is attached, and the myosin binding sites. This allows the myosin heads to bind to the myosin binding sites on the actin protein filament and contraction results as the myosin heads draw the actin filaments along, are bound by ATP, causing them to release the actin, and return to their original position, breaking down the ATP into ADP and a phosphate group. The action potential spreads via the passage of sodium ions through the gap junctions that connect the sarcoplasm of adjacent myocardial cells. Norepinephrine (noradrenaline) is released by the terminal boutons of depolarized sympathetic fibers, at the sinoatrial and atrioventricular nodes. Norepinephrine diffuses across the synaptic cleft binds to the β1-adrenoreceptors – G-protein linked receptors, consisting of seven transmembrane domains – shifting their equilibrium towards the active state. The receptor changes its conformation and mechanically activates the G-protein which is released. The G-protein is involved in the production of adenosine 3',5'-cyclic monophosphate (cAMP) from adenosine triphosphate (ATP) and this in turn activates the protein kinase (β-adrenoreceptor kinase). β-adrenoreceptor kinase phosphorylates the calcium ion channels in the sarcolemma, so that calcium ion influx is increased when they are activated by the appropriate transmembrane voltage. This will of course, cause more of the calcium receptors in the sarcoplasmic reticulum to be activated, creating a larger flow of calcium ions into the sarcoplasm. More troponin will be bound and more myosin binding sites cleared [of tropomyosin] so that more myosin heads can be recruited for the contraction and a greater force and speed of contraction results. [Phosphodiesterase catalyses the decomposition of cAMP to AMP so that it is no longer able to activate the protein kinase. AMP will of course, go on to be phosphorylated to ATP and may be recycled.] Noradrenaline also affects the atrioventricular node, reducing the delay before continuing conduction of the action potential via the bundle of HIS.

### Diastole

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/0/0f/Heart_diasystole.svg/200px-Heart_diasystole.svg.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

The heart in the diastole phase.

The heart in the diastole phase. Cardiac Diastole is the period of time when the heart relaxes after contraction in preparation for refilling with circulating blood. Ventricular diastole is when the ventricles are relaxing, while atrial diastole is when the atria are relaxing. Together they are known as complete cardiac diastole. It should be noted that even this relaxation is an active, energy-spending process. During ventricular diastole, the pressure in the (left and right) ventricles drops from the peak that it reaches in systole. When the pressure in the left ventricle drops to below the pressure in the left atrium, the mitral valve opens, and the left ventricle fills with blood that was accumulating in the left atrium. Likewise, when the pressure in the right ventricle drops below that in the right atrium, the tricuspid valve opens and the right ventricle fills with blood that was in the right atrium

### "Lub-Dub"

The first heart tone, or S1, "Lub" is caused by the closure of the atrioventricular valves, mitral and tricuspid, at the beginning of ventricular contraction, or systole. When the pressure in the ventricles rises above the pressure in the atria, these valves close to prevent regurgitation of blood from the ventricles into the atria. The second heart tone, or S2 (A2 and P2), "Dub" is caused by the closure of the aortic valve and pulmonic valve at the end of ventricular systole. As the left ventricle empties, its pressure falls below the pressure in the aorta, and the aortic valve closes. Similarly, as the pressure in the right ventricle falls below the pressure in the pulmonary artery, the pulmonic valve closes. During inspiration, negative intrathoracic pressure causes increased blood return into the right side of the heart. The increased blood volume in the right ventricle causes the pulmonic valve to stay open longer during ventricular systole. This causes an increased delay in the P2 component of S2. During expiration, the positive intrathoracic pressure causes decreased blood return to the right side of the heart. The reduced volume in the right ventricle allows the pulmonic valve to close earlier at the end of ventricular systole, causing P2 to occur earlier, and "closer" to A2. It is physiological to hear the splitting of the second heart tone by younger people and during inspiration. During expiration normally the interval between the two components shortens and the tone becomes merged.

## The Heart's Electrical Conduction System

The heart is primarily made up of muscle tissue. A network of nerve fibers coordinates the contraction and relaxation of the cardiac muscle tissue to obtain an efficient, wave-like pumping action of the heart

[How Stuff Works (The Heart)](http://health.howstuffworks.com/adam-200080.htm)

#### Control of Heartbeat

The heart contains two cardiac pacemakers that spontaneously cause the heart to beat. These can be controlled by the autonomic nervous system and circulating adrenaline. If the cardiac muscles just contracted and relaxed randomly at a natural rhythm the cycle would become disordered and the heart would become unable to carry on its function of being a pump. Sometimes when the heart undergoes great damage to one part of the cardiac muscle or the person incurs an electric shock, the cardiac cycle can become uncoordinated and chaotic. Some parts of the heart will contract whilst others will relax so that instead of contracting and relaxing as a whole, the heart will flutter abnormally. This is called fibrillation and can be fatal if not treated within 60 seconds.

  


  


![](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/9d/Bundleofhis.png/300px-Bundleofhis.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Schematic representation of the sinoatrial node and the atrioventricular bundle of His. The location of the SA node is shown in blue. The bundle, represented in red, originates near the orifice of the coronary sinus, undergoes slight enlargement to form the AV node. The AV node tapers down into the bundle of HIS, which passes into the ventricular septum and divides into two bundle branches, the left and right bundles. The ultimate distribution cannot be completely shown in this diagram.

**SA Node**  
The sinoatrial node (abbreviated SA node or SAN, also called the sinus node) is the impulse generating (pacemaker) tissue located in the right atrium of the heart. Although all of the heart's cells possess the ability to generate the electrical impulses (or action potentials) that trigger cardiac contraction, the sinoatrial node is what normally initiates it, simply because it generates impulses slightly faster than the other areas with pacemaker potential. Because cardiac myocytes, like all nerve cells, have refractory periods following contraction during which additional contractions cannot be triggered, their pacemaker potential is overridden by the sinoatrial node. The SA node emits a new impulse before either the AV or purkinje fibers reach threshold. The sinoatrial node (SA node) is a group of cells positioned on the wall of the right atrium, near the entrance of the superior vena cava. These cells are modified cardiac myocytes. They possess some contractile filaments, though they do not contract. Cells in the SA node will naturally discharge (create action potentials) at about 70-80 times/minute. Because the sinoatrial node is responsible for the rest of the heart's electrical activity, it is sometimes called the primary pacemaker. If the SA node doesn't function, or the impulse generated in the SA node is blocked before it travels down the electrical conduction system, a group of cells further down the heart will become the heart's pacemaker. These cells form the atrioventricular node (AV node), which is an area between the right atrium and ventricle, within the atrial septum. The impulses from the AV node will maintain a slower heart rate (about 40-60 beats per a minute). When there is a pathology in the AV node or purkinje fibers, an ectopic pacemaker can occur in different parts of the heart. The ectopic pacemaker typically discharges faster than the SA node and causes an abnormal sequence of contraction. The SA node is richly innervated by vagal and sympathetic fibers. This makes the SA node susceptible to autonomic influences. Stimulation of the vagus nerve causes decrease in the SA node rate (thereby causing decrease in the heart rate). Stimulation via sympathetic fibers causes increase in the SA node rate (thereby increasing the heart rate). The sympathetic nerves are distributed to all parts of the heart, especially in ventricular muscles. The parasympathetic nerves mainly control SA and AV nodes, some atrial muscle and ventricular muscle. Parasympathetic stimulation from the vagal nerves decreases the rate of the AV node by causing the release of acetylcholine at vagal endings which in turn increases the K+ permeability of the cardiac muscle fiber. Vagal stimulation can block transmission through AV junction or stop SA node contraction which is called "ventricular escape." When this happens, the purkinje fibers in the AV bundle develops a rhythm of their own. In the majority of patients, the SA node receives blood from the right coronary artery, meaning that a myocardial infarction occluding it will cause ischemia in the SA node unless there is a sufficiently good anastomosis from the left coronary artery. If not, death of the affected cells will stop the SA node from triggering the heartbeat

#### AV Node

The atrioventricular node (abbreviated AV node) is the tissue between the atria and the ventricles of the heart, which conducts the normal electrical impulse from the atria to the ventricles. The AV node receives two inputs from the atria: posteriorly via the crista terminalis, and anteriorly via the interatrial septum. [1] An important property that is unique to the AV node is decremental conduction. This is the property of the AV node that prevents rapid conduction to the ventricle in cases of rapid atrial rhythms, such as atrial fibrillation or atrial flutter. The atrioventricular node delays impulses for 0.1 second before spreading to the ventricle walls. The reason it is so important to delay the cardiac impulse is to ensure that the atria are empty completely before the ventricles contract (Campbell _et al._, 2002). The blood supply of the AV node is from a branch of the right coronary artery in 85% to 90% of individuals, and from a branch of the left circumflex artery in 10% to 15% of individuals. In certain types of supraventricular tachycardia, a person could have two AV Nodes; this will cause a loop in electrical current and uncontrollably-rapid heart beat. When this electricity catches up with itself, it will dissipate and return to normal heart-beat speed.

#### AV Bundle

The bundle of HIS is a collection of heart muscle cells specialized for electrical conduction that transmits the electrical impulses from the AV node (located between the atria and the ventricles) to the point of the apex of the fascicular branches. The fascicular branches then lead to the Purkinje fibers which innervate the ventricles, causing the cardiac muscle of the ventricles to contract at a paced interval. These specialized muscle fibers in the heart were named after the Swiss cardiologist Wilhelm His, Jr., who discovered them in 1893. Cardiac muscle is very specialized, as it is the only type of muscle that has an internal rhythm; i.e., it is myogenic which means that it can naturally contract and relax without receiving electrical impulses from nerves. When a cell of cardiac muscle is placed next to another, they will beat in unison. The fibers of the Bundle of HIS allow electrical conduction to occur more easily and quickly than typical cardiac muscle. They are an important part of the electrical conduction system of the heart as they transmit the impulse from the AV node (the ventricular pacemaker) to the rest of the heart. The bundle of HIS branches into the three bundle branches: the right left anterior and left posterior bundle branches that run along the intraventricular septum. The bundles give rise to thin filaments known as Purkinje fibers. These fibers distribute the impulse to the ventricular muscle. Together, the bundle branches and purkinje network comprise the ventricular conduction system. It takes about 0.03-0.04s for the impulse to travel from the bundle of HIS to the ventricular muscle. It is extremely important for these nodes to exist as they ensure the correct control and co-ordination of the heart and cardiac cycle and make sure all the contractions remain within the correct sequence and in sync.

#### Purkinje Fibers

Purkinje fibers (or Purkyne tissue) are located in the inner ventricular walls of the heart, just beneath the endocardium. These fibers are specialized myocardial fibers that conduct an electrical stimulus or impulse that enables the heart to contract in a coordinated fashion. Purkinje fibers work with the sinoatrial node (SA node) and the atrioventricular node (AV node) to control the heart rate. During the ventricular contraction portion of the cardiac cycle, the Purkinje fibers carry the contraction impulse from the left and right bundle branches to the myocardium of the ventricles. This causes the muscle tissue of the ventricles to contract and force blood out of the heart — either to the pulmonary circulation (from the right ventricle) or to the systemic circulation (from the left ventricle). They were discovered in 1839 by Jan Evangelista Purkinje, who gave them his name.

#### Pacemaker

The contractions of the heart are controlled by electrical impulses, these fire at a rate which controls the beat of the heart. The cells that create these rhythmical impulses are called pacemaker cells, and they directly control the heart rate. Artificial devices also called pacemakers can be used after damage to the body's intrinsic conduction system to produce these impulses synthetically.

#### Fibrillation

Fibrillation is when the heart flutters abnormally. This can be detected by an electrocardiogram which measures the waves of excitation passing through the heart and plotting a graph of potential difference (voltage) against time. If the heart and cardiac cycle is functioning properly the electrocardiogram shows a regular, repeating pattern. However if there is fibrillation there will be no apparent pattern, either in the much more common 'Atrial Fibrillation', or the less likely but much more dangerous 'Ventricular Fibrillation'. In a hospital during VF the monitor would make a sound and alert the doctors to treat the fibrillation by passing a huge current through the chest wall and shocking the heart out of its fibrillation. This causes the cardiac muscle to stop completely for 5 seconds and when it begins to beat again the cardiac cycle would have resumed to normal and the heart will be beating in a controlled manner again. Fibrillation is an example of "circus movement" of impulses through the heart muscle.

Circus movement occurs when an impulse begins in one part of the heart muscle and spreads in a circuitous pathway through the heart then returns to the originally excited muscle and "re-enters" it to stimulate it once more. The signal never stops. A cause of circus movement is long length pathway in which the muscle is no longer in a refractory state when the stimulus returns to it. A "flutter" is a circus movement in coordinated, low frequency waves that cause rapid heart rate. If the Bundle of HIS is blocked, it will result in dissociation between the activity of the atria and that of the ventricles, otherwise called a third degree heart block. The other cause of a third degree block would be a block of the right, left anterior, and left posterior bundle branches. A third degree block is very serious medical condition that will most likely require an artificial pacemaker.

## The ECG

E.C.G stands for Electrocardiogram and represents the electrophysiology of the heart. Cardiac electrophysiology is the science of the mechanisms, functions, and performance of the electrical activities of specific regions of the heart. The ECG is the recording of the heart's electrical activity as a graph. The graph can show the heart's rate and rhythm, it can detect enlargement of the heart, decreased blood flow, or the presence of current or past heart attacks. ECG's are inexpensive, Non-invasive, quick, and painless. Depending on the results, the patient’s medical history, and a physical exam; further tests or a combination of medications and lifestyle changes may be ordered.

#### How To Read An ECG

ECG Waveform

![Qrs.svg](../../../upload.wikimedia.org/wikipedia/commons/thumb/5/55/Qrs.svg/425px-Qrs.svg.png)

P
    P wave- indicates that the atria are electrically stimulated (depolarized) to pump blood into the ventricles.
QRS
    QRS complex- indicates that the ventricles are electrically stimulated (depolarized) to pump blood out.
ST
    ST segment- indicates the amount of time from the end of the contraction of the ventricles to the beginning of the T wave.
T
    T wave- indicates the recovery period (repolarization) of the ventricles.
U
    U wave- rarely seen, and thought to possibly be the repolarization of the papillary muscles

### Cardiac Muscle Contraction

After an action potential excites the plasma membrane of the cardiac muscle cell the contraction is due to an increase in the cytoplasmic concentration of Calcium ions. Similar to skeletal muscle, the release of Ca+ ions from the sarcoplasmic reticulum binds to troponin which allows actin to bind with myosin. The difference between skeletal muscle and cardiac muscle is that when the action potential opens voltage gated calcium ion channels in the T-tubules. The increase in cytosolic calcium causes calcium ions to bind to receptors on the surface of the sarcoplasmic reticulum. The binding of calcium ions to these receptors causes the opening of more calcium ion channels in the SR membrane. Calcium ions then rush out of the SR and bind to troponin and allow the myosin and actin to bind together which causes contraction. This sequence is called calcium-induced calcium release. Contraction ends when the level of cytosolic calcium returns to normal resting levels.

### Blood Pressure

Blood pressure is the pressure exerted by the blood on the walls of the blood vessels. Unless indicated otherwise, blood pressure refers to systemic arterial blood pressure, i.e., the pressure in the large arteries delivering blood to body parts other than the lungs, such as the brachial artery (in the arm). The pressure of the blood in other vessels is lower than the arterial pressure. Blood pressure values are universally stated in millimeters of mercury (mmHg). The systolic pressure is defined as the peak pressure in the arteries during the cardiac cycle; the diastolic pressure is the lowest pressure (at the resting phase of the cardiac cycle). The mean arterial pressure and pulse pressure are other important quantities. Typical values for a resting, healthy adult are approximately 120 mmHg systolic and 80mm Hg diastolic (written as 120/80 mmHg), with individual variations. These measures of blood pressure are not static, but undergo natural variations from one heartbeat to another, and throughout the day (in a circadian rhythm); they also change in response to stress, nutritional factors, drugs, or disease.

#### Systolic Pressure

Systolic Pressure is the highest when the blood is being pumped out of the left ventricle into the aorta during ventricular systole. The average high during systole is 120 mmHg.

#### Diastolic Pressure

Diastolic blood pressure lowers steadily to an average low of 80 mmHg during ventricular diastole.

## Cardiovascular Disease

Cardiovascular disease refers to the class of diseases that involve the heart and/or blood vessels (arteries and veins). While the term technically refers to any disease that affects the cardiovascular system, it is usually used to refer to those related to atherosclerosis (arterial disease). These conditions have similar causes, mechanisms, and treatments. Over 50 million Americans have cardiovascular problems, and most other Western countries face high and increasing rates of cardiovascular disease. It is the number 1 cause of death and disability in the United States and most European countries. By the time that heart problems are detected, the underlying cause (atherosclerosis) is usually quite advanced, having progressed for decades. There is therefore increased emphasis on preventing atherosclerosis by modifying risk factors, such as healthy eating, exercise and avoidance of smoking.

#### Hypertension

Hypertension or high blood pressure is a medical condition wherein the blood pressure is chronically elevated. Hypertension is defined by some authors as systolic pressure over 130 and diastolic over 85 mmHg. [[1]](Print_Version.html#cite_note-Totora-1) Hypertension often has an insidious or un-noticed onset and is sometimes called the _silent killer_ because stretching of the arteries causes microscopic tears in the arterial wall and accelerates degenerative changes. Persistent hypertension is one of the risk factors for strokes, heart attacks, heart failure and arterial aneurysm, and is a leading cause of chronic renal failure

#### Atherosclerosis

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/f/f1/Atherosclerosis,_aorta,_gross_pathology_PHIL_846_lores.jpg/230px-Atherosclerosis,_aorta,_gross_pathology_PHIL_846_lores.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Severe atherosclerosis of the [aorta](http://en.wikibooks.org/w/index.php?title=Aorta&action=edit&redlink=1). [Autopsy](http://en.wikibooks.org/w/index.php?title=Autopsy&action=edit&redlink=1) specimen.

Atherosclerosis is a disease affecting the arterial blood vessel. It is commonly referred to as a "hardening" or "furring" of the arteries. It is caused by the formation of multiple plaques within the arteries. Arteriosclerosis ("hardening of the artery") results from a deposition of tough, rigid collagen inside the vessel wall and around the atheroma. This increases the stiffness, decreases the elasticity of the artery wall. Atherosclerosis typically begins in early adolescence, is usually found in most major arteries, and yet is asymptomatic and not detected by most diagnostic methods during life. It most commonly becomes seriously symptomatic when interfering with the coronary circulation supplying the heart or cerebral circulation supplying the brain, and is considered the most important underlying cause of strokes, heart attacks, various heart diseases including congestive heart failure and most cardiovascular diseases in general.

#### Plaque

Plaque Atheroma or commonly known as plaque is an abnormal inflammatory accumulation of macrophage white blood cells within the walls of arteries.

#### Circulatory Shock

Circulatory Shock is a severe condition that results from reduced blood circulation.

#### Thrombus

A thrombus, or blood clot, is the final product of the blood coagulation step in hemostasis. It is achieved via the aggregation of platelets that form a platelet plug, and the activation of the humoral coagulation system (i.e. clotting factors). A thrombus is physiologic in cases of injury, but pathologic in case of thrombosis.

Preventing blood clots reduces the risk of stroke, heart attack and pulmonary embolism. Heparin and warfarin are often used to inhibit the formation and growth of existing blood clots, thereby allowing the body to shrink and dissolve the blood clots through normal methods.

#### Embolism

An embolism occurs when an object (the embolus) migrates from one part of the body (through circulation) and causes a blockage (occlusion) of a blood vessel in another part of the body. Blood clots form the most common embolic material by far: other possible embolic materials include fat globules (a fat embolism), air bubbles (an air embolism), septic emboli (containing pus and bacteria), or amniotic fluid.

#### Stroke

A stroke, also known as cerebrovascular accident (CVA), is an acute neurological injury whereby the blood supply to a part of the brain is interrupted. Strokes can be classified into two major categories: ischemic and hemorrhagic. ~80% of strokes are due to ischemia.

  * **Ischemic Stroke**: In ischemic stroke, which occurs in approximately 85-90% of strokes, a blood vessel becomes occluded and the blood supply to part of the brain is totally or partially blocked. Ischemic stroke is commonly divided into thrombotic stroke, embolic stroke, systemic hypoperfusion (Watershed or Border Zone stroke), or venous thrombosis
  * **Hemorrhagic Stroke**: A hemorrhagic stroke, or cerebral hemorrhage, is a form of stroke that occurs when a blood vessel in the brain ruptures or bleeds. Like ischemic strokes, hemorrhagic strokes interrupt the brain's blood supply because the bleeding vessel can no longer carry the blood to its target tissue. In addition, blood irritates brain tissue, disrupting the delicate chemical balance, and, if the bleeding continues, it can cause increased intracranial pressure which physically impinges on brain tissue and restricts blood flow into the brain. In this respect, hemorrhagic strokes are more dangerous than their more common counterpart, ischemic strokes. There are two types of hemorrhagic stroke: intracerebral hemorrhage, and subarachnoid hemorrhage.

The term "brain attack" is starting to come into use in the United States for stroke, just as the term "heart attack" is used for myocardial infarction, where a cutoff of blood causes necrosis to the tissue of the heart. Many hospitals have "brain attack" teams within their neurology departments specifically for swift treatment of stroke. If symptoms of stroke are detected at early on-set, special "clot busting" drugs may be administered. These clot busters will dissolve clots before they can cause tissue death and restore normal circulation. One of the initial drugs used to dissolve clots was **streptokinase**, although its use creates a possibility of clot destruction throughout the entire body, leading to serious hemorrhage. There are newer, third generation thrombolytics that are safer.

#### Heart Attack

![Wikipedia-logo.png](../../../upload.wikimedia.org/wikipedia/commons/thumb/6/63/Wikipedia-logo.png/40px-Wikipedia-logo.png)

[Wikipedia](http://en.wikipedia.org/wiki/) has related information at _**[Heart Attack**_](http://en.wikipedia.org/wiki/Heart_Attack)

Acute myocardial infarction (AMI or MI), commonly known as a heart attack, A heart attack occurs when the supply of blood and oxygen to an area of heart muscle is blocked, usually by a clot in a coronary artery. Often, this blockage leads to arrhythmias (irregular heartbeat or rhythm) that cause a severe decrease in the pumping function of the heart and may bring about sudden death. If the blockage is not treated within a few hours, the affected heart muscle will die and be replaced by scar tissue. It is the leading cause of death for both men and women all over the world

#### Angina Pectoris

Angina Pectoris is chest pain due to ischemia (a lack of blood and hence oxygen supply) of the heart muscle, generally due to obstruction or spasm of the coronary arteries (the heart's blood vessels).

#### Coronary Bypass

Coronary artery bypass surgery, coronary artery bypass graft surgery and heart bypass are surgical procedures performed on patients with coronary artery disease for the relief of angina and possible improved heart muscle function. Veins or arteries from elsewhere in the patient's body are grafted from the aorta to the coronary arteries, bypassing coronary artery narrowing caused by atherosclerosis and improves the blood supply to the myocardium (heart muscle).

#### Congestive Heart Failure

Congestive heart failure (CHF), also called congestive cardiac failure (CCF) or just heart failure, is a condition that can result from any structural or functional cardiac disorder that impairs the ability of the heart to fill with or pump a sufficient amount of blood throughout the body. It is not to be confused with "cessation of heartbeat", which is known as asystole, or with cardiac arrest, which is the cessation of normal cardiac function in the face of heart disease. Because not all patients have volume overload at the time of initial or subsequent evaluation, the term "heart failure" is preferred over the older term "congestive heart failure". Congestive heart failure is often undiagnosed due to a lack of a universally agreed definition and difficulties in diagnosis, particularly when the condition is considered "mild". Right sided heart failure commonly causes peripheral edema, or swelling of the extremities. Left sided heart failure commonly causes pulmonary edema, or fluid buildup in the lungs.

#### Aneurysm

An aneurysm (or aneurism) is a localized dilation or ballooning of a blood vessel by more than 50% of the diameter of the vessel and can lead to instant death at anytime. Aneurysms most commonly occur in arteries at the base of the brain (the circle of Willis) and in the aorta (the main artery coming out of the heart) - this is an aortic aneurysm. This bulge in a blood vessel, much like a bulge on an over-inflated inner tube, can lead to death at anytime. The larger an aneurysm becomes, the more likely it is to burst. Aneurysms are also described according to their shape: Saccular or fusiform. A saccular aneurysm resembles a small sack; a fusiform aneurysm is shaped like a spindle.

#### Dissolving Blood Clots

To dissolve blood clots you would use a drug that converts plasminogen (molecule found in blood), to plasmin, (enzyme that dissolves blood clots).

#### Clearing Clogged Arteries

One way to unblock a coronary artery (or other blood vessel) is percutaneous transluminal coronary angioplasty (PTCA), which was first performed in 1977. A wire is passed from the femoral artery in the leg or the radial artery in the arm up to the diseased coronary artery, to beyond the area of the coronary artery that is being worked upon. Over this wire, a balloon catheter is passed into the segment that is to be opened up. The end of the catheter contains a small folded balloon. When the balloon is hydraulically inflated, it compresses the atheromatous plaque and stretches the artery wall to expand. At the same time, if an expandable wire mesh tube (stent) was on the balloon, then the stent will be implanted (left behind) to support the new stretched open position of the artery from the inside.

### Dilated and Inflamed Veins

#### Varicose Veins

Varicose veins are veins on the leg which are large, twisted, and ropelike, and can cause pain, swelling, or itching. They are an extreme form of telangiectasia, or spider veins. Varicose veins result due to insufficiency of the valves in the communicating veins. These are veins which link the superficial and deep veins of the lower limb. Normally, blood flows from the superficial to the deep veins, facilitating return of blood to the heart. However, when the valve becomes defective, blood is forced into the superficial veins by the action of the muscle pump (which normally aids return of blood to the heart by compressing the deep veins). People who have varicose veins are more at risk of getting a Deep Vein Thrombosis (DVT) and pulmonary embolisms.

#### Phlebitis

Phlebitis is an inflammation of a vein, usually in the legs. This is usually the most serious if found in a deep vein. However, most people with the condition, perhaps 80 to 90 percent, are women. The disease may also have a genetic component, as it is known to run in families.

### Congenital Heart Defects

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/9f/Heart_vsd.svg/220px-Heart_vsd.svg.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Illustration of VSD

Heart defects present at birth are called congenital heart defects. Slightly less than 1% of all newborn infants have congenital heart disease. Eight defects are more common than all others and make up 80% of all congenital heart diseases, whereas the remaining 20% consist of many independently infrequent conditions or combinations of several defects.

#### Acyanotic Defects

Acyanotic heart defects are those in which there is a normal amount of oxygen in the bloodstream. The most common congenital heart defect is a ventral septal defect, which occurs in about 20% of all children with congenital heart disease. In VSD blood from the left ventricle is shunted to the right ventricle, resulting in oxygenated blood returning into pulmonic circulation. One of the potential problems of VSD is pulmonary hypertension.

#### Cyanotic Defects

Cyanotic heart defects refer to defects that result in decreased amounts of oxygen in the blood. In cyanotic heart defects deoxygenated blood from the right ventricle flows into the systemic circulation. Cyanotic defects include tetrogy of Fallot and transposition of the great arteries.

## Homeostasis

Homeostasis in the body is only possible if the cardiovascular system is working properly. This means that the system needs to deliver oxygen and nutrients to the tissue fluid that surrounds the cells and also take away the metabolic waste. The heart is composed of arteries that take blood from the heart, and vessels that return blood to the heart. Blood is pumped by the heart into two circuits: the pulmonary and systemic circuits. The pulmonary circuit carries blood through the lungs where gas exchange occurs and the systemic system transports blood to all parts of the body where exchange with tissue fluid takes place. The cardiovascular system works together with all other systems to maintain homeostasis.

### The Lymphatic System

The lymphatic system is closely related to the cardiovascular system. There are three main ways that they work together to maintain homeostasis: the lymphatic system receives the excess tissue fluid and returns it to the bloodstream, lacteal take fat molecules from the intestinal villi and transport them to the bloodstream and both systems work together to defend the body against disease.The lymphatic system can create white blood cells that fight off disease and infections.

## Interesting Facts

• Heart Disease is the number one killer in American women.  
• 16.7 million deaths are result forms of cardiovascular disease, heart disease and stroke.  
• Stress, eating high fat foods, obesity, tobacco and alcohol use are just some risk factors of developing heart disease.  
• Recent research suggests that taking a small dose of aspirin daily may help prevent a heart attack (because aspirin inhibits platelet clumping).  
• The length of all your blood vessels lined up is about 60,000 miles long! To put this in perspective, the Earth's circumference is 40,075.02 kilometres and 60,000 miles is around 96,000 km - so your blood vessels would go twice around the world and still have some to spare!

## Ways to a Healthy Heart

• Eating healthy, good nutrition.  


• Fitness and Exercise.  


• Having a healthy lifestyle; don't drink, smoke, or do drugs.  


• Lowering LDL cholesterol and high blood pressure.  


• Reduce the fat, sodium, and calories in your diet.  


## Aging

The heart muscle becomes less efficient with age, and there is a decrease in both maximum cardiac output and heart rate, although resting levels may be more than adequate. The health of the myocardium depends on its blood supply, and with age there is greater likelihood that atherosclerosis will narrow the coronary arteries. Atherosclerosis is the deposition of cholesterol on and in the walls of the arteries, which decreases blood flow and forms rough surfaces that may cause intravascular clot formation High blood pressure (hypertension) causes the left ventricle to work harder. It may enlarge and outgrow its blood supply, thus becoming weaker. A weak ventricle is not an efficient pump, and may progress to congestive heart failure. This process may be slow or rapid. The heart valves may become thickened by fibrosis, leading to heart murmurs and less efficient pumping. Arrhythmias are also more common with age, as the cells of the conduction pathway become less efficient.

## Shock

Physiological Stress

Physiological stress can be any kind of injury from burns, to broken bones; the body's response to stress is categorized in two phases the ebb phase (early phase) begins immediately after the injury. And the second phase is about 36 to 48 hours after injury is called the flow phase. In the ebb (shock) phase there is Inadequate circulation, decreased insulin level, decreased oxygen consumption, hypothermia (low body temperature), hypovolemia (low blood volume), and hypotension (low blood pressure). In the flow phase there is increased levels of catecholamine, glucocorticoids, and glucagons, normal or elevated insulin levels, catabolic (breakdown), hyperglycemic (high blood sugar), increased oxygen consumption/respiratory rate, hyperthermia (high body temperature) fever sets in, hypermetabolism, increased insulin resistance, increased cardiac output.

## Premature ventricular contractions (PVC's)

Excitation occurs through the SA node to the AV node if there are abnormalities or drug interference that malfunctions the AV node the ventricles will not receive the initiating stimuli and the autorhythmic cells in the bundle branches begin to initiate actions on their own rate becoming the pacemakers for the ventricles. This in turn will cause conduction disorder. With conduction that causes problems with the bundle branches there is the right and the left premature ventricular contractions. Right is most common and may go untreated. Left is always a serious problem and must be treated.

## Intrinsic Control of heartbeat

• SA node (located in the right atrium near the entrance of the superior vena cava)  
• AV node (located at the base of right atrium)  


• AV bundle (located in the intraventricular septum between the two ventricles that go in two directions right and left bundle branches that leave the septum to enter the walls of both ventricle)  
• Bundle Branches (the branching off the septum to the walls of the ventricles that run into the purkinje fibers that then make contact with ventricular myocardial cells to spread the impulse to the rest of the ventricles)  


![](../../../upload.wikimedia.org/wikipedia/commons/thumb/e/e5/ECG_principle_slow.gif/220px-ECG_principle_slow.gif)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Animation of a normal ECG wave.

## Electrocardiogram

• The P is the atrial depolarization  
• QRS is the ventricular depolarization, as well as atrial repolarization.  
• T is the ventricular repolarization  


  


![](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/9e/SinusRhythmLabels.svg/300px-SinusRhythmLabels.svg.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Schematic representation of normal ECG

## Extrinsic Control of Heartbeat

Autonomic system with two subdivisions: the sympathetic division and the parasympathetic division. Hormonal control of blood pressure

  * Epinephrine
  * Norepinephrine
  * ANP : Atrial natriuretic peptide
  * ADH: Antidiuretic hormone
  * Renin-Angiotension system

## Case Study

An example of the ever expanding technology for the heart is best described in this story: In 1955, when I was five years old, I first learned by my family physician that I had a heart murmur and that it would eventually need attention. By the time I was 15 in 1965, I had two cardiac catherizations at Rhode Island Hospital. The tests were inconclusive and I was told to go on with my life and wait and see if I had a problem. It wasn't until 1975 that I was told by my family physician that I should have my heart checked again. Dr. David Kitzes of Mariam Hospital performed another catherization. This time, unlike the others, I was told that because of new machine technology, Dr. Kitzes found that I had aortic stenosis, which is a narrowing of the valve passage by build-up of plaque due to the valve being malformed at birth. Dr. Kitzes informed me that I could lead a normal life until I was in my fifties or sixties before I would need corrective surgery. In 1996, I had an echocardiogram and it was determined that my heart was enlarged. My family physician said that I should see a cardiologist. I down played the visit as not being serious after hearing the same thing many times. This time I entered the office of Jon Lambrecht, I had never met him before. Within a few minutes my whole life was turned around. After asking me about my symptoms, which were fatigue, weakness, asthmatic symptoms, as well as ashen skin color and dizziness, he informed me of how serious my condition was and the only salvation was immediate open-heart surgery to replace the aortic valve. I began to cry as I thought my life was over. Dr. Lambrecht studied my reaction and told me that this condition is repairable and that I don't have a terminal illness. I didn't have a lot of time to think about it. Within 10 days from that visit, I was the recipient of a Meditronic Hall Prosthetic heart valve. The operation was performed by Dr. Robert Indeglia at Miriam Hospital in Providence, R.I. on March 20th, 1996. It has been almost 3 years since the surgery and I am doing better than I could have expected. In 1977 my son Kevin was born with Hypoplastic Left-heart Syndrome and only lived for 2 days because heart surgery wasn't performed like today. I am thankful that I lived at a time when medical technology paved the way for a second chance because of my new aortic heart valve. Our goal in this chapter is to take you by the hand and lead you through each part of the cardiovascular system, so that you too may learn and come to respect the greatness of this blood pumping machine we all call the heart.

  


#### Stroke

Cerebrovascular disease are those that affect blood vessels in the brain and happen to be the third cause of death in the United States only behind heart disease and cancer. Stroke (also called cerebrovascular accident or CVR) is a cerebrovascular disorder caused by a sudden decrease or stoppage of blood flow to a part of the brain. Decreased blood flow also known as ischemia is dangerous to any tissue but brain tissue is even more vulnerable, mainly due to the high rate of its metabolic reactions. In fact if you stopped blood flow for no more than three minutes it may be sufficient enough to cause death of most brain cells. For this reason a stroke can kill people within minutes or leave them with severe brain damage.

Strokes may be classified as either occlusive or hemorrhagic and may happen either in the interior of the brain or on its surface. In a occlusive stroke blood flow through a vessel is blocked. In a hemorrhagic stroke a blood vessel ruptures causing a hemorrhage.

## Summary

As with all of the body systems, the cardiovascular system plays a part in maintaining homeostasis. The nervous system regulates the functioning of the heart based on what the heart is supposed to do. The pumping of the heart maintains normal blood pressure and proper oxygenation of tissues. The vascular system forms passageways for the blood, but they aren't simply just a pipeline system. The vessels are not passive tubes, but rather active contributors to homeostasis. The arteries and veins help maintain blood pressure, and the capillaries provide sites for the necessary exchanges of materials between the blood and the tissues.

## Review Questions

Answers for these questions can be found [here](http://en.wikibooks.org/wiki/Human_Physiology/Appendix_1:_answers_to_review_questions#The_cardiovascular_system)

1\. This conducts electricity like nerves

    A) Epicardium
    B) Pericardium
    C) Myocardium
    D) Subvalaular Apparatus
    E) None of these, only nerves conduct electricity

2\. This carries the most blood at any given time in the body

    A) Veins
    B) Capillary Beds
    C) Veins
    D) Aorta
    E) Vena Cava

3\. The following contract together to pump blood

    A) Right atrium with the right ventricle and left atrium with the left ventricle
    B) Right atrium with left atrium and right ventricles with left ventricle
    C) Tricuspid valve and mitral valve
    D) Aorta and pulmonary artery
    E) Aorta, pulmonary artery and pulmonary vein

4\. This is the pacemaker of the heart

    A) AV node
    B) Purkinje fibers
    C) AV Bundle
    D) SA node
    E) None of these, a pacemaker is surgically inserted

5\. When reading an EKG, this letter shows the depolarization from the AV node down to the AV bundle

    A) S
    B) P
    C) U
    D) T
    E) Q

6\. The T wave in an EKG shows

    A) Resting potential
    B) Atrial depolarization
    C) SA node excitation
    D) Ventricle repolarization
    E) Purkinje Excitation

7\. Blood pressure is the measure of

    A) Pressure exerted by the blood on the walls of the blood vessels
    B) Pressure exerted by the blood on the arteries
    C) Pressure exerted by the blood on the veins
    D) Pressure exerted by the blood on the aorta
    E) Pressure exerted by the blood on the capillaries

8\. Systolic Pressure is

    A) An average of 120 mm Hg
    B) Lowers steadily during ventricle systole
    C) The highest when blood is being pumped out of the left ventricle into the aorta
    D) An average of 80 mm Hg
    E) Both A and C
    F) Both B and D

9\. The heart has how many chambers?

    A) One
    B) Two
    C) Three
    D) Four
    E) Five

## Glossary

  
**Acute myocardial infarction (AMI or MI)** commonly known as a heart attack, is a disease state that occurs when the blood supply to a part of the heart is interrupted. The resulting ischemia or oxygen shortage causes damage and potential death of heart tissue. **Aorta**: the largest of the arteries in the systemic circuit  
**Aortic Valve**: lies between the left ventricle and the aorta  
**Antidiuretic hormone:** Produced in the posterior pituitary ADH (vasopressin), major function is to regulate blood pressure by water retention by the kidneys.  
**Arteriole:** a small diameter blood vessel that extends and branches out from an artery and leads to capillaries  
**Atrial natriuretic peptide:** Produced in the atria of the heart, it increases urinary excretion of sodium which causes water loss which in turn the viscosity of the blood is lowered and in turn lowers the blood pressure.  
**Atrioventricular Node (abbreviated AV node):** the tissue between the atria and the ventricles of the heart, which conducts the normal electrical impulse from the atria to the ventricles  
**Atrioventricular valves:** large, multi-cusped valves that prevent backflow from the ventricles into the atria during systole  
**AV Bundle:** collection of heart muscle cells specialized for electrical conduction that transmits the electrical impulses from the AV node  
**Barbiturates:** CNS depressants, sedative-hypnotics  
**Blood Pressure:** the pressure exerted by the blood on the walls of the blood vessels  
**Capillaries:** the smallest of a body’s vessels, they connect arteries and veins  
**Cardiac Cycle:** term used to describe the sequence of events that occur as a heart works to pump blood through the body  
**Cerebral Vascular Accident (CVA):** Also known as a stroke, is a rapidly developing loss of a part of brain function or loss of consciousness due to an interruption in the blood supply to all or part of the brain. That is, a stroke involves the sudden loss of neuronal function due to a disturbance in cerebral perfusion. There are many different causes for the interruption of blood supply, and different parts of the brain can be affected. Because of this, a stroke can be quite heterogeneous. Patients with the same cause of stroke can have widely differing handicaps. Similarly, patients with the same clinical handicap can in fact have different causes of their stroke.  
**Chordae Tendinae:** cord-like tendons that connect the papillary muscles to the tricuspid valve and the mitral valve in the heart  
**Coronary Arteries:** blood vessels that supply blood to, and remove blood from, the heart muscle itself  
**Continuous Capillaries:** have a sealed epithelium and only allow small molecules, water and ions to diffuse  
**Deep-vein thrombosis (DVT):** is the formation of a blood clot ("thrombus") in a deep vein. It commonly affects the leg veins, such as the femoral vein or the popliteal vein or the deep veins of the pelvis. Occasionally the veins of the arm are affected  
**Diastole:** period of time when the heart relaxes after contraction in preparation for refilling with circulating blood  
**Diastolic Pressure:** lowest point in blood pressure where the heart relaxes  
**Edema:** The swelling that forms when too much tissue fluid forms or not enough taken away  
**Electrocardiogram:** the recording of the heart's electrical activity as a graph  
**Epinephrine:** Produced in the adrenal medulla of the adrenal glands, major function is vasoconstriction that will in turn increase respiratory rate and increase cardiac out put.  
**Fenestrated Capillaries:** have openings that allow larger molecules to diffuse  
**Fibrous Pericardium:** a dense connective tissue that protects the heart, anchoring it to the surrounding walls, and preventing it from overfilling with blood  
**Heart Rate:** term used to describe the frequency of the cardiac cycle  
**Hepatic Veins:** blood vessels that drain de-oxygenated blood from the liver and blood cleaned by the liver (from the stomach, pancreas, small intestine and colon) into the inferior vena cava  
**Hypertension or High Blood Pressure:** medical condition wherein the blood pressure is chronically elevated  
**Inferior Vena Cava (or IVC):** a large vein that carries de-oxygenated blood from the lower half of the body into the heart  
**Intraventricular Septum:** the stout wall separating the lower chambers (the ventricles) of the heart from one another  
**Left Atrium:**receives oxygenated blood from the left and right pulmonary veins  
**Lub:** first heart tone, or S1; caused by the closure of the atrioventricular valves, mitral and tricuspid, at the beginning of ventricular contraction, or systole  
**Lumen:** hollow internal cavity in which the blood flows  
**Lymph:** originates as blood plasma that leaks from the capillaries of the circulatory system, becoming interstitial fluid, filling the space between individual cells of tissue  
**Mitral valve:** also known as the bicuspid valve; prevents blood flowing from the left ventricle into the left atrium  
**Myocardium:** the muscular tissue of the heart.  
**Norepinephrine:** Produced in the adrenal medulla of the adrenal glands, major function is a strong vasoconstrictor that will in turn increase respiratory rate.  
**Pacemaker Cells:** cells that create these rhythmical impulses of the heart  
**Plaque: an abnormal inflammatory accumulation of macrophage white blood cells within the walls of arteries**  
**Pulmonary Valve:** lies between the right ventricle and the pulmonary artery; prevents back-flow of blood into the ventricle  
**Pulse: the number of heartbeats per minute**  
**Purkinje Fibers (or Purkinje tissue):** located in the inner ventricular walls of the heart, just beneath the endocardium; specialized myocardial fibers that conduct an electrical stimulus or impulse that enables the heart to contract in a coordinated fashion  
**Renin-Angiotension system:**  
**Right Atrium:** receives de-oxygenated blood from the superior vena cava and inferior vena cava  
**Serous Pericardium:** functions in lubricating the heart to prevent friction from occurring during heart activity  
**Semilunar Valves:** positioned on the pulmonary artery and the aorta  
**Sinoatrial Node:** (abbreviated SA node or SAN, also called the sinus node): the impulse generating (pacemaker) tissue located in the right atrium of the heart  
**Sinusoidal Capillaries:** special forms of fenestrated capillaries that have larger opening allowing RBCs and serum proteins to enter  
**Systole:** contraction of the heart  
_Systolic Pressure:'_ the highest point in blood pressure when the blood is being pumped out of the left ventricle into the aorta during ventricular systole  
**Superior Vena Cava (SVC):** a large but short vein that carries de-oxygenated blood from the upper half of the body to the heart's right atrium  
**Thrombus:** a blood clot in an intact blood vessel  
**Tricuspid Valve:** on the right side of the heart, between the right atrium and the right ventricle; allows blood to flow from the right atrium into the right ventricle when the heart is relaxed during diastole  
**Vasoconstriction:** the constriction of blood vessels  
**Vasodilation: the dilation of blood vessels**  
**Veins:**carry de-oxygenated blood from the capillary blood vessels to the right part of the heart  
**Ventricle:** a heart chamber which collects blood from an atrium  
**Venule:** a small blood vessel that allows deoxygenated blood to return from the capillary beds to the larger blood vessels called

## References

  1. Van De Graaff, Kent M. Human Anatomy. McGraw Hill Publishing, Burr Ridge, IL. 2002.
  2. Essentials of Anatomy and Physiology, Valerie C. Scanlon and Tina Sanders
  3. Tortora, G. & Grabowski, S. (2000) Principles of Anatomy & Physiology. Wiley & Sons. Brisbane, Singapore & Chichester.
  4. Anderson, RM. The Gross Physiology of the Cardiovascular System (1993) <<http://cardiac-output.info>>.

  1. [↑](Print_Version.html#cite_ref-Totora_1-0) Tortora, G. & Grabowski, S. (2000)Principles of anatomy and physiology. Ninth Edition. Wiley page 733.

  


# The Urinary System

![Illu urinary system.jpg](../../../upload.wikimedia.org/wikipedia/commons/1/18/Illu_urinary_system.jpg)

## Introduction

The **Urinary System** is a group of organs in the body concerned with filtering out excess fluid and other substances from the bloodstream. The substances are filtered out from the body in the form of **urine**. Urine is a liquid produced by the kidneys, collected in the bladder and excreted through the urethra. Urine is used to extract excess minerals or vitamins as well as blood corpuscles from the body. The Urinary organs include the kidneys, ureters, bladder, and urethra. The Urinary system works with the other systems of the body to help maintain homeostasis. The kidneys are the main organs of homeostasis because they maintain the acid base balance and the water salt balance of the blood.

## Functions of the Urinary System

One of the major functions of the Urinary system is the process of excretion. Excretion is the process of eliminating, from an organism, waste products of metabolism and other materials that are of no use. The urinary system maintains an appropriate fluid volume by regulating the amount of water that is excreted in the urine. Other aspects of its function include regulating the concentrations of various electrolytes in the body fluids and maintaining normal pH of the blood. Several body organs carry out excretion, but the kidneys are the most important excretory organ. The primary function of the kidneys is to maintain a stable internal environment (homeostasis) for optimal cell and tissue metabolism. They do this by separating urea, mineral salts, toxins, and other waste products from the blood. They also do the job of conserving water, salts, and electrolytes. At least one kidney must function properly for life to be maintained. **Six important roles of the kidneys are:**

**Regulation of plasma ionic composition.** Ions such as sodium, potassium, calcium, magnesium, chloride, bicarbonate, and phosphates are regulated by the amount that the kidney excretes.

**Regulation of plasma osmolarity.** The kidneys regulate osmolarity because they have direct control over how many ions and how much water a person excretes.

**Regulation of plasma volume.** Your kidneys are so important they even have an effect on your blood pressure. The kidneys control plasma volume by controlling how much water a person excretes. The plasma volume has a direct effect on the total blood volume, which has a direct effect on your blood pressure. Salt(NaCl)will cause osmosis to happen; the diffusion of water into the blood.

**Regulation of plasma hydrogen ion concentration (pH).** The kidneys partner up with the lungs and they together control the pH. The kidneys have a major role because they control the amount of bicarbonate excreted or held onto. The kidneys help maintain the blood Ph mainly by excreting hydrogen ions and reabsorbing bicarbonate ions as needed.

**Removal of metabolic waste products and foreign substances from the plasma.** One of the most important things the kidneys excrete is nitrogenous waste. As the liver breaks down amino acids it also releases ammonia. The liver then quickly combines that ammonia with carbon dioxide, creating **urea** which is the primary nitrogenous end product of metabolism in humans. The liver turns the ammonia into urea because it is much less toxic. We can also excrete some ammonia, creatinine and uric acid. The **creatinine** comes from the metabolic breakdown of creatine phospate (a high-energy phosphate in muscles). **Uric acid** comes from the break down of nucleotides. Uric acid is insoluble and too much uric acid in the blood will build up and form crystals that can collect in the joints and cause gout.

**Secretion of Hormones** The endocrine system has assistance from the kidney's when releasing hormones. Renin is released by the kidneys. Renin leads to the secretion of aldosterone which is released from the adrenal cortex. Aldosterone promotes the kidneys to reabsorb the sodium (Na+) ions. The kidneys also secrete erythropoietin when the blood doesn't have the capacity to carry oxygen. Erythropoietin stimulates red blood cell production. The Vitamin D from the skin is also activated with help from the kidneys. Calcium (Ca+) absorption from the digestive tract is promoted by vitamin D.

**CC: Chapter Check: Name the role of the kidneys and how they work?**

![Renin-angiotensin-aldosterone system.png](../../../upload.wikimedia.org/wikipedia/commons/thumb/a/a2/Renin-angiotensin-aldosterone_system.png/700px-Renin-angiotensin-aldosterone_system.png)

## Organs in the Urinary System

### Kidneys And Their Structure

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/7/7e/Kidney_PioM.png/250px-Kidney_PioM.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

1\. Renal pyramid 2. Interlobar artery 3. Renal artery 4. Renal vein 5. Renal hylum 6. Renal pelvis 7. Ureter 8. Minor calyx 9. Renal capsule 10. Inferior renal capsule 11. Superior renal capsule 12. Interlobar vein 13. Nephron 14. Minor calyx 15. Major calyx 16. Renal papilla 17. Renal column

The **kidneys** are a pair of bean shaped, brown organs about the size of your fist.It measures 10-12 cm long. They are covered by the renal capsule, which is a tough capsule of fibrous connective tissue. Adhering to the surface of each kidney is two layers of fat to help cushion them. There is a concaved side of the kidney that has a depression where a renal artery enters, and a renal vein and a ureter exit the kidney. The kidneys are located at the rear wall of the abdominal cavity just above the waistline, and are protected by the ribcage. They are considered retroperitoneal, which means they lie behind the peritoneum. There are three major regions of the kidney, **renal cortex**, **renal medulla** and the **renal pelvis**. The outer, granulated layer is the renal cortex. The cortex stretches down in between a radially striated inner layer. The inner radially striated layer is the renal medulla. This contains pyramid shaped tissue called the renal pyramids, separated by renal columns. The ureters are continuous with the renal pelvis and is the very center of the kidney.

#### Renal Vein

The **renal veins** are veins that drain the kidney. They connect the kidney to the inferior vena cava. Because the inferior vena cava is on the right half of the body, the left renal vein is generally the longer of the two. Unlike the right renal vein, the left renal vein often receives the left gonadal vein (left testicular vein in males, left ovarian vein in females). It frequently receives the left suprarenal vein as well.

#### Renal Artery

The **renal arteries** normally arise off the abdominal aorta and supply the kidneys with blood. The arterial supply of the kidneys are variable and there may be one or more renal arteries supplying each kidney. Due to the position of the aorta, the inferior vena cava and the kidneys in the body, the right renal artery is normally longer than the left renal artery. The right renal artery normally crosses posteriorly to the inferior vena cava. The renal arteries carry a large portion of the total blood flow to the kidneys. Up to a third of the total cardiac output can pass through the renal arteries to be filtered by the kidneys.

### Ureters

The **ureters** are two tubes that drain urine from the kidneys to the bladder. Each ureter is a muscular tube about 10 inches (25 cm) long. Muscles in the walls of the ureters send the urine in small spurts into the bladder, (a collapsible sac found on the forward part of the cavity of the bony pelvis that allows temporary storage of urine). After the urine enters the bladder from the ureters, small folds in the bladder mucosa act like valves preventing backward flow of the urine. The outlet of the bladder is controlled by a sphincter muscle. A full bladder stimulates sensory nerves in the bladder wall that relax the sphincter and allow release of the urine. However, relaxation of the sphincter is also in part a learned response under voluntary control. The released urine enters the urethra.

### Urinary Bladder

The **urinary bladder** is a hollow, muscular and distendible or elastic organ that sits on the pelvic floor (superior to the prostate in males). On its anterior border lies the pubic symphysis and, on its posterior border, the vagina (in females) and rectum (in males). The urinary bladder can hold approximately 17 to 18 ounces (500 to 530 ml) of urine, however the desire to micturate is usually experienced when it contains about 150 to 200 ml. When the bladder fills with urine (about half full), stretch receptors send nerve impulses to the spinal cord, which then sends a reflex nerve impulse back to the sphincter (muscular valve) at the neck of the bladder, causing it to relax and allow the flow of urine into the urethra. The Internal urethral sphincter is involuntary. The ureters enter the bladder diagonally from its dorsolateral floor in an area called the trigone. The trigone is a triangular shaped area on the postero-inferior wall of the bladder. The urethra exits at the lowest point of the triangle of the trigone. The urine in the bladder also helps regulate body temperature. A bladder when operating normally empties completely upon a complete discharge, otherwise it is a sign that its elasticity is compromised, when it becomes completely void of fluid, it may cause a chilling sensation due to the rapid change of body temperature.

### Urethra

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/0/06/Gray1139.png/250px-Gray1139.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Female urethra (labeled at bottom right.)

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/d/d7/Gray1142.png/200px-Gray1142.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Male Sphincter urethrae muscle - The male urethra laid open on its anterior (upper) surface. (Region visible, but muscle not labeled.)

The **urethra** is a muscular tube that connects the bladder with the outside of the body. The function of the urethra is to remove urine from the body. It measures about 1.5 inches (3.8 cm) in a woman but up to 8 inches (20 cm) in a man. Because the urethra is so much shorter in a woman it makes it much easier for a woman to get harmful bacteria in her bladder this is commonly called a bladder infection or a UTI. The most common bacteria of a UTI is E-coli from the large intestines that have been excreted in fecal matter. **Female urethra**

In the human female, the urethra is about 1-2 inches long and opens in the vulva between the clitoris and the vaginal opening.

Men have a longer urethra than women. This means that women tend to be more susceptible to infections of the bladder (cystitis) and the urinary tract.

**Male urethra**

In the human male, the urethra is about 8 inches long and opens at the end of the head of the penis.

The length of a male's urethra, and the fact it contains a number of bends, makes catheterisation more difficult.

  
The **urethral sphincter** is a collective name for the muscles used to control the flow of urine from the urinary bladder. These muscles surround the urethra, so that when they contract, the urethra is closed.

  * There are two distinct areas of muscle: the internal sphincter, at the bladder neck and
  * the external, or distal, sphincter.

Human males have much stronger sphincter muscles than females, meaning that they can retain a large amount of urine for twice as long, as much as 800mL, i.e. "hold it".

### Nephrons

A nephron is the basic structural and functional unit of the kidney. The name nephron comes from the Greek word (nephros) meaning kidney. Its chief function is to regulate water and soluble substances by filtering the blood, reabsorbing what is needed and excreting the rest as urine. Nephrons eliminate wastes from the body, regulate blood volume and pressure, control levels of electrolytes and metabolites, and regulate blood pH. Its functions are vital to life and are regulated by the endocrine system by hormones such as antidiuretic hormone, aldosterone, and parathyroid hormone.

Each nephron has its own supply of blood from two capillary regions from the renal artery. Each nephron is composed of an initial filtering component (the renal corpuscle) and a tubule specialized for reabsorption and secretion (the renal tubule). The renal corpuscle filters out large solutes from the blood, delivering water and small solutes to the renal tubule for modification.

#### Glomerulus

The glomerulus is a capillary tuft that receives its blood supply from an afferent arteriole of the renal circulation. The glomerular blood pressure provides the driving force for fluid and solutes to be filtered out of the blood and into the space made by Bowman's capsule. The remainder of the blood not filtered into the glomerulus passes into the narrower efferent arteriole. It then moves into the vasa recta, which are collecting capillaries intertwined with the convoluted tubules through the interstitial space, where the reabsorbed substances will also enter. This then combines with efferent venules from other nephrons into the renal vein, and rejoins with the main bloodstream.

  
**Afferent/Efferent Arterioles**

The afferent arteriole supplies blood to the glomerulus. A group of specialized cells known as **juxtaglomerular cells** are located around the afferent arteriole where it enters the renal corpuscle. The efferent arteriole drains the glomerulus. Between the two arterioles lies specialized cells called the **macula densa.** The juxtaglomerular cells and the macula densa collectively form the **juxtaglomerular apparatus.** It is in the juxtaglomerular apparatus cells that the enzyme **renin** is formed and stored. Renin is released in response to decreased blood pressure in the afferent arterioles, decreased sodium chloride in the distal convoluted tubule and sympathetic nerve stimulation of receptors (beta-adrenic) on the juxtaglomerular cells. Renin is needed to form Angiotensin I and Angiotensin II which stimulate the secretion of aldosterone by the adrenal cortex.

#### Glomerular Capsule or Bowman's Capsule

**Bowman's capsule** (also called the **glomerular capsule**) surrounds the glomerulus and is composed of visceral (simple squamous epithelial cells) (inner) and parietal (simple squamous epithelial cells) (outer) layers. The visceral layer lies just beneath the thickened glomerular basement membrane and is made of podocytes which send foot processes over the length of the glomerulus. Foot processes interdigitate with one another forming filtration slits that, in contrast to those in the glomeruluar endothelium, are spanned by diaphragms. The size of the filtration slits restricts the passage of large molecules (eg, albumin) and cells (eg, red blood cells and platelets). In addition, foot processes have a negatively-charged coat (glycocalyx) that limits the filtration of negatively-charged molecules, such as albumin. This action is called electrostatic repulsion.

The parietal layer of Bowman's capsule is lined by a single layer of squamous epithelium. Between the visceral and parietal layers is Bowman's space, into which the filtrate enters after passing through the podocytes' filtration slits. It is here that smooth muscle cells and macrophages lie between the capillaries and provide support for them. Unlike the visceral layer, the parietal layer does not function in filtration. Rather, the filtration barrier is formed by three components: the diaphragms of the filtration slits, the thick glomerular basement membrane, and the glycocalyx secreted by podocytes. 99% of glomerular filtrate will ultimately be reabsorbed.

The process of filtration of the blood in the Bowman's capsule is ultrafiltration (or glomerular filtration), and the normal rate of filtration is 125 ml/min, equivalent to ten times the blood volume daily. Measuring the glomerular filtration rate (GFR) is a diagnostic test of kidney function. A decreased GFR may be a sign of renal failure. Conditions that can affect GFR include: arterial pressure, afferent arteriole constriction, efferent arteriole constriction, plasma protein concentration and colloid osmotic pressure.

Any proteins that are roughly 30 kilodaltons or under can pass freely through the membrane. Although, there is some extra hindrance for negatively charged molecules due to the negative charge of the basement membrane and the podocytes. Any small molecules such as water, glucose, salt (NaCl), amino acids, and urea pass freely into Bowman's space, but cells, platelets and large proteins do not. As a result, the filtrate leaving the Bowman's capsule is very similar to blood plasma in composition as it passes into the proximal convoluted tubule. Together, the glomerulus and Bowman's capsule are called the renal corpuscle.

#### Proximal Convoluted Tubule (PCT)

The proximal tubule can be anatomically divided into two segments: the proximal convoluted tubule and the proximal straight tubule. The proximal convoluted tubule can be divided further into S1 and S2 segments based on the histological appearance of it's cells. Following this naming convention, the proximal straight tubule is commonly called the S3 segment. The proximal convoluted tubule has one layer of cuboidal cells in the lumen. This is the only place in the nephron that contains cuboidal cells. These cells are covered with millions of microvilli. The microvilli serve to increase surface area for reabsorption.

Fluid in the filtrate entering the proximal convoluted tubule is reabsorbed into the peritubular capillaries, including approximately two-thirds of the filtered salt and water and all filtered organic solutes (primarily glucose and amino acids). This is driven by sodium transport from the lumen into the blood by the Na+/K+ ATPase in the basolateral membrane of the epithelial cells. Much of the mass movement of water and solutes occurs in between the cells through the tight junctions, which in this case are not selective.

The solutes are absorbed isotonically, in that the osmotic potential of the fluid leaving the proximal tubule is the same as that of the initial glomerular filtrate. However, glucose, amino acids, inorganic phosphate, and some other solutes are reabsorbed via secondary active transport through cotransport channels driven by the sodium gradient out of the nephron.

#### Loop of the Nephron or Loop of Henle

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/0/02/Gray1128.png/450px-Gray1128.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

The Nephron Loop or Loop of Henle.

The loop of Henle (sometimes known as the nephron loop) is a U-shaped tube that consists of a descending limb and ascending limb. It begins in the cortex, receiving filtrate from the proximal convoluted tubule, extends into the medulla, and then returns to the cortex to empty into the distal convoluted tubule. Its primary role is to concentrate the salt in the interstitium, the tissue surrounding the loop.

Descending limb
    Its descending limb is permeable to water but completely impermeable to salt, and thus only indirectly contributes to the concentration of the interstitium. As the filtrate descends deeper into the hypertonic interstitium of the renal medulla, water flows freely out of the descending limb by osmosis until the tonicity of the filtrate and interstitium equilibrate. Longer descending limbs allow more time for water to flow out of the filtrate, so longer limbs make the filtrate more hypertonic than shorter limbs.

Ascending limb
    Unlike the descending limb, the ascending limb of Henle's loop is impermeable to water, a critical feature of the countercurrent exchange mechanism employed by the loop. The ascending limb actively pumps sodium out of the filtrate, generating the hypertonic interstitium that drives countercurrent exchange. In passing through the ascending limb, the filtrate grows hypotonic since it has lost much of its sodium content. This hypotonic filtrate is passed to the distal convoluted tubule in the renal cortex.

#### Distal Convoluted Tubule (DCT)

The distal convoluted tubule is similar to the proximal convoluted tubule in structure and function. Cells lining the tubule have numerous mitochondria, enabling active transport to take place by the energy supplied by ATP. Much of the ion transport taking place in the distal convoluted tubule is regulated by the endocrine system. In the presence of parathyroid hormone, the distal convoluted tubule reabsorbs more calcium and excretes more phosphate. When aldosterone is present, more sodium is reabsorbed and more potassium excreted. Atrial natriuretic peptide causes the distal convoluted tubule to excrete more sodium. In addition, the tubule also secretes hydrogen and ammonium to regulate pH. After traveling the length of the distal convoluted tubule, only 3% of water remains, and the remaining salt content is negligible. 97.9% of the water in the glomerular filtrate enters the convoluted tubules and collecting ducts by osmosis.

#### Collecting ducts

Each distal convoluted tubule delivers its filtrate to a system of collecting ducts, the first segment of which is the connecting tubule. The collecting duct system begins in the renal cortex and extends deep into the medulla. As the urine travels down the collecting duct system, it passes by the medullary interstitium which has a high sodium concentration as a result of the loop of Henle's countercurrent multiplier system. Though the collecting duct is normally impermeable to water, it becomes permeable in the presence of antidiuretic hormone (ADH). As much as three-fourths of the water from urine can be reabsorbed as it leaves the collecting duct by osmosis. Thus the levels of ADH determine whether urine will be concentrated or dilute. Dehydration results in an increase in ADH, while water sufficiency results in low ADH allowing for diluted urine. Lower portions of the collecting duct are also permeable to urea, allowing some of it to enter the medulla of the kidney, thus maintaining its high ion concentration (which is very important for the nephron).

Urine leaves the medullary collecting ducts through the renal papilla, emptying into the renal calyces, the renal pelvis, and finally into the bladder via the ureter. Because it has a different embryonic origin than the rest of the nephron (the collecting duct is from endoderm whereas the nephron is from mesoderm), the collecting duct is usually not considered a part of the nephron proper.

**Renal Hormones**

1\. Vitamin D- Becomes metabolically active in the kidney. Patients with renal disease have symptoms of disturbed calcium and phosphate balance.

2\. Erythropoietin- Released by the kidneys in response to decreased tissue oxygen levels (hypoxia).

3\. Natriuretic Hormone- Released from cardiocyte granules located in the right atria of the heart in response to increased atrial stretch. It inhibits ADH secretions which can contribute to the loss of sodium and water.

## Formation of Urine

Urine is formed in three steps: Filtration, Reabsorption, and Secretion.

### Filtration

Blood enters the afferent arteriole and flows into the glomerulus. Blood in the glomerulus has both filterable blood components and non-filterable blood components. Filterable blood components move toward the inside of the glomerulus while non-filterable blood components bypass the filtration process by exiting through the efferent arteriole. Filterable Blood components will then take a plasma like form called glomerular filtrate. A few of the filterable blood components are water, nitrogenous waste, nutrients and salts (ions). Nonfilterable blood components include formed elements such as blood cells and platelets along with plasma proteins. The glomerular filtrate is not the same consistency as urine, as much of it is reabsorbed into the blood as the filtrate passes through the tubules of the nephron.

### Reabsorption

Within the peritubular capillary network, molecules and ions are reabsorbed back into the blood. Sodium Chloride reabsorbed into the system increases the osmolarity of blood in comparison to the glomerular filtrate. This reabsorption process allows water (H2O) to pass from the glomerular filtrate back into the circulatory system.

Glucose and various amino acids also are reabsorbed into the circulatory system. These nutrients have carrier molecules that claim the glomerular molecule and release it back into the circulatory system. If all of the carrier molecules are used up, excess glucose or amino acids are set free into the urine. A complication of diabetes is the inability of the body to reabsorb glucose. If too much glucose appears in the glomerular filtrate it increases the osmolarity of the filtrate, causing water to be released into the urine rather than reabsorbed by the circulatory system. Frequent urination and unexplained thirst are warning signs of diabetes, due to water not being reabsorbed.

Glomerular filtrate has now been separated into two forms: Reabsorbed Filtrate and Non-reabsorbed Filtrate. Non-reabsorbed filtrate is now known as tubular fluid as it passes through the collecting duct to be processed into urine.

### Secretion

Some substances are removed from blood through the peritubular capillary network into the distal convoluted tubule or collecting duct. These substances are Hydrogen ions, creatinine, and drugs. Urine is a collection of substances that have not been reabsorbed during glomerular filtration or tubular reabsorbtion.

## Maintaining Water-Salt Balance

It is the job of the kidneys to maintain the water-salt balance of the blood. They also maintain blood volume as well as blood pressure. Simple examples of ways that this balance can be changed include ingestion of water, dehydration, blood loss and salt ingestion.

### Reabsorption of water

Direct control of water excretion in the kidneys is exercised by the anti-diuretic hormone (ADH), released by the posterior lobe of the pituitary gland. ADH causes the insertion of water channels into the membranes of cells lining the collecting ducts, allowing water reabsorption to occur. Without ADH, little water is reabsorbed in the collecting ducts and dilute urine is excreted. There are several factors that influence the secretion of ADH. The first of these happen when the blood plasma gets too concentrated. When this occurs, special receptors in the hypothalamus release ADH. When blood pressure falls, stretch receptors in the aorta and carotid arteries stimulate ADH secretion to increase volume of the blood.

### Reabsorption of Salt

The Kidneys also regulate the salt balance in the blood by controlling the excretion and the reabsorption of various ions. As noted above, ADH plays a role in increasing water reabsorption in the kidneys, thus helping to dilute bodily fluids. The kidneys also have a regulated mechanism for reabsorbing sodium in the distal nephron. This mechanism is controlled by aldosterone, a steroid hormone produced by the adrenal cortex. Aldosterone promotes the excretion of potassium ions and the reabsorption of sodium ions. The release of Aldosterone is initiated by the kidneys. The juxtaglomerular apparatus is a renal structure consisting of the macula densa, mesangial cells, and juxtaglomerular cells. Juxtaglomerular cells (JG cells, also known as granular cells) are the site of renin secretion. Renin is an enzyme that converts angiotensinogen (a large plasma protein produced by the liver) into Angiotensin I and eventually into Angiotensin II which stimulates the adrenal cortex to produce aldosterone. The reabsorption of sodium ions is followed by the reapsorption of water. This causes blood pressure as well as blood volume to increase.

**Atrial natriuretic hormone (ANH**) is released by the atria of the heart when cardiac cells are stretched due to increased blood volume. ANH inhibits the secretion of renin by the juxtaglomerular apparatus and the secretion of the aldosterone by the adrenal cortex. This promotes the excretion of sodium. When sodium is excreted so is water. This causes blood pressure and volume to decrease.

#### Hypernatremia

An increase in plasma sodium levels above normal is **hypernatremia**. Sodium is the primary solute in the extracellular fluid. Sodium levels have a major role in osmolarity regulation. For excitable cells the electrochemical gradient for sodium across the plasma membrane is critical for life. Water retention and an increased blood pressure usually are signs of hypernatremia. If the plasma sodium levels are below normal it is called **hyponatremia**. Signs of this are low plasma volume and hypotension.

#### Diuretics

A diuretic (colloquially called a water pill) is any drug that elevates the rate of bodily urine excretion (diuresis). Diuretics also decrease the extracellular fluid (ECF) volume, and are primarily used to produce a negative extracellular fluid balance. Caffeine, cranberry juice and alcohol are all weak diuretics. In medicine, diuretics are used to treat heart failure, liver cirrhosis, hypertension and certain kidney diseases. Diuretics alleviate the symptoms of these diseases by causing sodium and water loss through the urine. As urine is produced by the kidney, sodium and water – which cause edema related to the disease – move into the blood to replace the volume lost as urine, thereby reducing the pathological edema. Some diuretics, such as acetazolamide, help to make the urine more alkaline and are helpful in increasing excretion of substances such as aspirin in cases of overdose or poisoning. The antihypertensive actions of some diuretics (thiazides and loop diuretics in particular) are independent of their diuretic effect. That is, the reduction in blood pressure is not due to decreased blood volume resulting from increased urine production, but occurs through other mechanisms and at lower doses than that required to produce diuresis. Indapamide was specifically designed with this is mind, and has a larger therapeutic window for hypertension (without pronounced diuresis) than most other diuretics. Chemically, diuretics are a diverse group of compounds that either stimulate or inhibit various hormones that naturally occur in the body to regulate urine production by the kidneys. Alcohol produces diuresis through modulation of the vasopressin system.

## Diseases of the Kidney

**Diabetic nephropathy** (nephropatia diabetica), also known as Kimmelstiel-Wilson syndrome and intercapillary glomerulonephritis, is a progressive kidney disease caused by angiopathy of capillaries in the kidney glomeruli. It is characterized by nodular glomerulosclerosis. It is due to longstanding diabetes mellitus, and is a prime cause for dialysis in many Western countries.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/3/3a/Kidney_Stone1.jpg/220px-Kidney_Stone1.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

An image of a **kidney stone**.

In medicine, **hematuria** (or "haematuria") is the presence of blood in the urine. It is a sign of a large number of diseases of the kidneys and the urinary tract, ranging from trivial to lethal.

**Kidney stones**, also known as nephrolithiases, urolithiases or renal calculi, are solid accretions (crystals) of dissolved minerals in urine found inside the kidneys or ureters. They vary in size from as small as a grain of sand to as large as a golf ball. Kidney stones typically leave the body in the urine stream; if they grow relatively large before passing (on the order of millimeters), obstruction of a ureter and distention with urine can cause severe pain most commonly felt in the flank, lower abdomen and groin. Kidney stones are unrelated to gallstones.

**Case Study** I was 34 weeks pregnant when I noticed blood in my urine. I immediately went to my OBGYN where I was told that I had a bladder infection and given an antibiotic. The next morning I experienced the most intense pain. I was rushed to the ER where I was told that I had kidney stones. The doctors explained that there was nothing they could do as long as I was pregnant. The next 3 weeks of my life were filled with intense pain and multiple painkillers. After I delivered my baby, CAT scans were done and I was informed that I had 6 kidney stones. It took three more weeks for me to pass all of the stones the largest measuring 5 mm. The stones were tested and I was informed that my body had been building up calcium due to my pregnancy and this was the cause of the kidney stones. I continued to have kidney pain for 6 months after passing the stones. I now live my life on a low calcium diet and the hope that my body will not develop more kidney stones.

**Pyelonephritis** When an infection of the renal pelvis and calices, called pyelitis, spreads to involve the rest of the kidney as well, the result is pyelonephritis. It usually results from the spread of fecal bacterium Escherichia coli from the anal region superiorly through the urinary tract. In severe cases, the kidney swells and scars, abscesses form, and the renal pelvis fills with pus. Left untreated, the infected kidney may be severely damaged, but administration of antibiotics usually achieve a total cure.

**glomerulonephritis** Inflammation of the glomerular can be caused by immunologic abnormalities, drugs or toxins, vascular disorders, and systemic diseases. Glomerulonephritis can be acute, chronic or progressive. Two major changes in the urine are distinctive of glomerulonephritis: hematuria and proteinuria with albumin as the major protein. There is also a decrease in urine as there is a decrease in GFR (glomerular filtration rate). Renal failure is associated with oliguria (less than 400 ml of urine output per day).

**Renal Failure** Uremia is a syndrome of renal failure and includes elevated blood urea and creatinine levels. Acute renal failure can be reversed if diagnosed early. Acute renal failure can be caused by severe hypotension or severe glomerular disease. Diagnostic tests include BUN and plasma creatinine level tests. It is considered to be chronic renal failure if the decline of renal function to less than 25%.

## Diabetes Insipidus

This is caused by the deficiency of or decrease of ADH. The person with (DI) has the inability to concentrate their urine in water restriction, in turn they will void up 3 to 20 liters/day. There are two forms of (DI), neurogenic, and nephrogenic. In nephrogenic (DI) the kidneys do not respond to ADH. Usually the nephrogenic (DI) is characterized by the impairment of the urine concentrating capability of the kidney along with concentration of water. The cause may be a genetic trait, electrolyte disorder, or side effect of drugs such as lithium. In the neurogenic (DI), it is usually caused by head injury near the hypophysisal tract.

## Urinary tract infections (UTI's)

The second most common type of bacterial infections seen by health care providers is UTI's. Out of all the bacterias that colonize and cause urinary tract infections the big gun is _Escherichia coli_. In the hospital indwelling catheters and straight catheterizing predispose the opportunity for urinary tract infections. In females there are three stages in life that predispose urinary tract infections, that is menarche, manipulation between intercourse, and menopause. However, a small percentage of men and children will get urinary tract infections. In men it is usually due to the prostate gland growth which usually occurs in older age men. In children it can occur 3% to 5% in girls and 1% in boys, uncircumcised boys it is more common than circumcised ones to have a urinary tract infection, in girls it may be the result of onset of toilet training, some predispositions for getting urinary tract infection include family history and urinary tract anomalies. In neonates urinary tract infections is most common when bacteremia is present.

## Dialysis and Kidney Transplant

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/6/6e/Plugged_into_dialysis.jpg/200px-Plugged_into_dialysis.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Plugged into dialysis

Generally, humans can live normally with just one kidney. Only when the amount of functioning kidney tissue is greatly diminished will renal failure develop. If renal function is impaired, various forms of medications are used, while others are contraindicated. Provided that treatment is begun early, it may be possible to reverse chronic kidney failure due to diabetes or high blood pressure. If creatinine clearance (a measure of renal function) has fallen very low ("end-stage renal failure"), or if the renal dysfunction leads to severe symptoms, dialysis is commenced. Dialysis is a medical procedure, performed in various different forms, where the blood is filtered outside of the body.

Kidney transplantation is the only cure for end stage renal failure; dialysis, is a supportive treatment; a form of "buying time" to bridge the inevitable wait for a suitable organ.

The first successful kidney transplant was announced on March 4, 1954 at Peter Bent Brigham Hospital in Boston. The surgery was performed by Dr. Joseph E. Murray, who was awarded the Nobel Prize in Medicine in 1990 for this feat.

There are two types of kidney transplants: living donor transplant and a cadaveric (dead donor) transplant. When a kidney from a living donor, usually a blood relative, is transplanted into the patient's body, the donor's blood group and tissue type must be judged compatible with the patient's, and extensive medical tests are done to determine the health of the donor. Before a cadaveric donor's organs can be transplanted, a series of medical tests have to be done to determine if the organs are healthy. Also, in some countries, the family of the donor must give its consent for the organ donation. In both cases, the recipient of the new organ needs to take drugs to suppress their immune system to help prevent their body from rejecting the new kidney.

## Review Questions

Answers for these questions can be found [here](http://en.wikibooks.org/wiki/Human_Physiology/Appendix_1:_answers_to_review_questions#The_Urinary_System)

1.While reading a blood test I notice a high level of creatinine, I could assume from this that

    A) There is a possibility of a UTI
    B) There is a possibility of diabetes
    C) There is a possibility of kidney failure
    D) There is nothing wrong, this is normal

2.Direct control of water excretion in the kidneys is controlled by

    A) Anti-diuretic hormone
    B) The medulla oblongata
    C) Blood plasma
    D) Sodium amounts in the blood

3\. Nephrons

    A) Eliminate wastes from the body
    B) Regulate blood volume and pressure
    C) Control levels of electrolytes and metabolites
    D) Regulate blood pH
    E) All of the above

4\. If I am dehydrated, my body will increase

    A) ATP
    B) ADP
    C) Diluted urine
    D) ADH

5.Which part of the nephron removes water, ions and nutrients from the blood?

    A ) vasa recta
    B ) loop of henle
    C ) proximal convuluted tubule
    D ) peritubular capillaries
    E ) glomerulus

6.Kidneys have a direct effect on which of the following

    A ) Blood pressure
    B ) How much water a person excretes
    C ) Total blood volume
    D ) pH
    E ) all of the above

7\. Why do substances in the glomerulus enter the Bowman's capsule?

    A ) the magnetic charge of the Bowman's capsule attracts the substances
    B ) the substances are actively transported into the Bowman's capsule
    C ) blood pressure of the glomerulus is so great that most substances in blood move into capsule
    D ) little green men force it in with their ray guns

8\. What happens in tubular excretion?

    A ) urine bonds are formed between the wastes
    B ) wastes are diffused from the tubule
    C ) wastes move into the distal convoluted tubule from the blood
    D ) blood pressure forces wastes away from the kidney

9\. The countercurrent exchange system includes_________and_________.

    A ) glomerulus and macula densa
    B ) proximal convoluted tubule and distal convoluted tubule
    C ) loop of Henle and collecting tubule
    D ) afferent arteriole and efferent arteriole
    E ) ureters and bladder

10\. The function of the loop of the nephron in the process of urine formation is:

    A ) reabsorption of water
    B ) production of filtrate
    C ) reabsorption of solutes
    D ) secretion of solutes

11\. Name the six important roles of the kidneys.

## Glossary

**Antidiuretic:** lessening or decreasing of urine production or an agent that decreases the release of urine.

**Catheterisation:** a catheter is a tube that can be inserted into a body cavity, duct or vessel. Catheters thereby allow drainage or injection of fluids or access by surgical instruments. The process of inserting a catheter is catheterisation. In most uses a catheter is a thin, flexible tube: a "soft" catheter; in some uses, it is a larger, solid tube: a "hard" catheter.

**Dehydration:** condition resulting from excessive loss of body fluid.

**Diabetes:** a general term for a disease characterized by the begining stages and onset of renal failure. It is derived from the Greek word diabaínein, that literally means "passing through," or "siphon", a reference to one of diabetes' major symptoms—excessive urine production.

**Diuresis:** secretion and passage of large amounts of urine.

**Diuretic:** increasing of urine production, or an agent that increases the production of urine.

**Erythropoietin:** hormone that stimulates stem cells in the bone marrow to produce red blood cells

**Fibrous Capsule:** the kidney's loose connective tissue

**Glomerulus:** capillary tuft that receives its blood supply from an afferent arteriole of the renal circulation.

**Gluconeogenesis:** the cycle of producing a glucose form fat or protein; preformed by the kidney in times of long fasting, initially gluconeogenesis is preformed by the liver

**Juxtaglomerular (JG) cells:** Renin-secreting cells that are in contact with the macula densa and the afferent arterioles of the renal nephron.

**Juxtaglomerular apparatus (JGA):** A site of juxtaglomerular cells connecting with the macula densa where renin is secreted and sensor for control of secretion of golmerular filtration rate.

**Loop of Henle/ Nephron Loop:** u-shaped tube that consists of a descending limb and ascending limb; primary role is to concentrate the salt in the interstitium, the tissue surrounding the loop

**Medullary Pyramids or Renal Pyramids:** the cone shaped masses in the kidney

**Micturition:** another name for excretions

**Nephron:** basic structural and functional unit of the kidney; chief function is to regulate water and soluble substances by filtering the blood, reabsorbing what is needed and excreting the rest as urine

**Podocytes:** filtration membrane, in the visceral layer of the bowman's capsule

**Renal Calculi:** kidney stones, solid crystals of dissolved minerals in urine found inside the kidneys

**Renal Cortex:** outer portion of the kidney

**Renal Lobe:** each pyramid together with the associated overlying cortex

**Renal Pelvis:** a central space, or cavity that transmits urine to the urinary bladder via the ureter

**Renin:** hormone released by the Juxtaglomerular (JG) cells of the kidneys when blood pressure falls

**TURP:** transurethral resection of the prostate. During TURP, an instrument is inserted up the urethra to remove the section of the prostate that is blocking urine flow. This is most commonly caused by benign prostatic hyperplasia (BPH). A TURP usually requires hospitalization and is done using a general or spinal anesthetic. It is now the most common surgery used to remove part of an enlarged prostate.

**Urethra:** a muscular tube that connects the bladder with the outside of the body

**Ureters:** two tubes that drain urine from the kidneys to the bladder

**Urine:** liquid produced by the kidneys, collected in the bladder and excreted through the urethra

**Urinary Bladder:** a hollow, muscular and distensible or elastic organ that sits on the pelvic floor

**Urinary System:** a group of organs in the body concerned with filtering out excess fluid and other substances from the bloodstream

## References

  * Graaff, Van De (2002). "Human Anatomy, Sixth Edition". New York: McGraw-Hill.
  * Mader, Sylvia S. (2004). _Human Biology_. New York: McGraw-Hill.
  * Smith, Peter (1998). Internet reference, _[The Role of the Kidney_](http://www.liv.ac.uk/~petesmif/teaching/1bds_mb/notes/homeo/kidney.htm%7C). Department of Clinical Dental Sciences,The University of Liverpool.
  * McCance, Katherine L., Heuther, Sue E. (1994). "Pathophysiology: The Biological Basis for Disease In Adults and Children, Second Edition". Mosby-Year Book, Inc.

  


# The Respiritory System

The **Respiratory System** is crucial to every human being. Without it, we would cease to live outside of the womb. Let us begin by taking a look at the structure of the respiratory system and how vital it is to life. During inhalation or exhalation air is pulled towards or away from the lungs, by several cavities, tubes, and openings.

The organs of the respiratory system make sure that oxygen enters our bodies and carbon dioxide leaves our bodies.

The respiratory tract is the path of air from the nose to the lungs. It is divided into two sections: **Upper Respiratory Tract** and the **Lower Respiratory Tract**. Included in the upper respiratory tract are the **Nostrils**, **Nasal Cavities**, **Pharynx**, **Epiglottis**, and the **Larynx**. The lower respiratory tract consists of the **Trachea**, **Bronchi**, **Bronchioles**, and the **Lungs**.

As air moves along the respiratory tract it is warmed, moistened and filtered.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/e/e6/Heart-and-lungs.jpg/230px-Heart-and-lungs.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

The **lungs** flank the heart and great vessels in the chest cavity. (Source: _Gray's Anatomy of the Human Body_, 20th ed. 1918.)

## Functions

In this chapter we will discuss the four processes of respiration. They are:

  1. **BREATHING** or ventilation
  2. **EXTERNAL RESPIRATION**, which is the exchange of gases (oxygen and carbon dioxide) between inhaled air and the blood.
  3. **INTERNAL RESPIRATION**, which is the exchange of gases between the blood and tissue fluids.
  4. **CELLULAR RESPIRATION**

In addition to these main processes, the respiratory system serves for:

  * **REGULATION OF BLOOD pH**, which occurs in coordination with the kidneys, and as a
  * '_DEFENSE AGAINST MICROBES_
  * **Control of body temperature** due to loss of evaporate during expiration

## Breathing and Lung Mechanics

**Ventilation** is the exchange of air between the external environment and the alveoli. Air moves by bulk flow from an area of high pressure to low pressure. All pressures in the respiratory system are relative to atmospheric pressure (760mmHg at sea level). Air will move in or out of the lungs depending on the pressure in the alveoli. The body changes the pressure in the alveoli by changing the volume of the lungs. As volume increases pressure decreases and as volume decreases pressure increases. There are two phases of ventilation; inspiration and expiration. During each phase the body changes the lung dimensions to produce a flow of air either in or out of the lungs.

The body is able to stay at the dimensions of the lungs because of the relationship of the lungs to the thoracic wall. Each lung is completely enclosed in a sac called the pleural sac. Two structures contribute to the formation of this sac. The parietal pleura is attached to the thoracic wall where as the visceral pleura is attached to the lung itself. In-between these two membranes is a thin layer of intrapleural fluid. The intrapleural fluid completely surrounds the lungs and lubricates the two surfaces so that they can slide across each other. Changing the pressure of this fluid also allows the lungs and the thoracic wall to move together during normal breathing. Much the way two glass slides with water in-between them are difficult to pull apart, such is the relationship of the lungs to the thoracic wall.

The rhythm of ventilation is also controlled by the "Respiratory Center" which is located largely in the medulla oblongata of the brain stem. This is part of the autonomic system and as such is not controlled voluntarily (one can increase or decrease breathing rate voluntarily, but that involves a different part of the brain). While resting, the respiratory center sends out action potentials that travel along the phrenic nerves into the diaphragm and the external intercostal muscles of the rib cage, causing inhalation. Relaxed exhalation occurs between impulses when the muscles relax. Normal adults have a breathing rate of 12-20 respirations per minute.

### The Pathway of Air

When one breathes air in at sea level, the inhalation is composed of different gases. These gases and their quantities are Oxygen which makes up 21%, Nitrogen which is 78%, Carbon Dioxide with 0.04% and others with significantly smaller portions.

![](../../../upload.wikimedia.org/wikipedia/commons/4/4a/Illu_pharynx.jpg)

Diagram of the **Pharynx**.

In the process of breathing, air enters into the nasal cavity through the nostrils and is filtered by coarse hairs (_vibrissae_) and mucous that are found there. The vibrissae filter macroparticles, which are particles of large size. Dust, pollen, smoke, and fine particles are trapped in the mucous that lines the **nasal cavities** (hollow spaces within the bones of the skull that warm, moisten, and filter the air). There are three bony projections inside the nasal cavity. The **superior, middle, and inferior nasal conchae**. Air passes between these conchae via the nasal meatuses.

Air then travels past the nasopharynx, oropharynx, and laryngopharynx, which are the three portions that make up the pharynx. The **[pharynx**](http://en.wikipedia.org/wiki/pharynx) is a funnel-shaped tube that connects our nasal and oral cavities to the larynx. The **tonsils** which are part of the lymphatic system, form a ring at the connection of the oral cavity and the pharynx. Here, they protect against foreign invasion of antigens. Therefore the respiratory tract aids the immune system through this protection. Then the air travels through the **larynx**. The larynx closes at the epiglottis to prevent the passage of food or drink as a protection to our trachea and lungs. The larynx is also our voicebox; it contains vocal cords, in which it produces sound. Sound is produced from the vibration of the vocal cords when air passes through them.

The **trachea**, which is also known as our windpipe, has ciliated cells and mucous secreting cells lining it, and is held open by C-shaped cartilage rings. One of its functions is similar to the larynx and nasal cavity, by way of protection from dust and other particles. The dust will adhere to the sticky mucous and the cilia helps propel it back up the trachea, to where it is either swallowed or coughed up. The **mucociliary escalator** extends from the top of the trachea all the way down to the **bronchioles**, which we will discuss later. Through the trachea, the air is now able to pass into the bronchi.

### Inspiration

**Inspiration is initiated by contraction of the diaphragm** and in some cases the intercostals muscles when they receive nervous impulses. During normal quiet breathing, **the phrenic nerves stimulate the diaphragm to contract and move downward into the abdomen**. This downward movement of the diaphragm enlarges the thorax. When necessary, the intercostal muscles also increase the thorax by contacting and drawing the ribs upward and outward.

As the diaphragm contracts inferiorly and thoracic muscles pull the chest wall outwardly, the volume of the thoracic cavity increases. The lungs are held to the thoracic wall by negative pressure in the pleural cavity, a very thin space filled with a few milliliters of lubricating pleural fluid. The negative pressure in the pleural cavity is enough to hold the lungs open in spite of the inherent elasticity of the tissue. Hence, as the thoracic cavity increases in volume the lungs are pulled from all sides to expand, causing a drop in the pressure (a partial vacuum) within the lung itself (but note that this negative pressure is still not as great as the negative pressure within the pleural cavity--otherwise the lungs would pull away from the chest wall). Assuming the airway is open, air from the external environment then follows its pressure gradient down and expands the alveoli of the lungs, where gas exchange with the blood takes place. As long as pressure within the alveoli is lower than atmospheric pressure air will continue to move inwardly, but as soon as the pressure is stabilized air movement stops.

### Expiration

During quiet breathing, expiration is normally a passive process and does not require muscles to work (rather it is the result of the muscles relaxing). When the lungs are stretched and expanded, stretch receptors within the alveoli send inhibitory nerve impulses to the medulla oblongata, causing it to stop sending signals to the rib cage and diaphragm to contract. The muscles of respiration and the lungs themselves are elastic, so when the diaphragm and intercostal muscles relax there is an elastic recoil, which creates a positive pressure (pressure in the lungs becomes greater than atmospheric pressure), and air moves out of the lungs by flowing down its pressure gradient.

Although the respiratory system is primarily under involuntary control, and regulated by the medulla oblongata, we have some voluntary control over it also. This is due to the higher brain function of the cerebral cortex.

When under physical or emotional stress, more frequent and deep breathing is needed, and both inspiration and expiration will work as active processes. Additional muscles in the rib cage forcefully contract and push air quickly out of the lungs. In addition to deeper breathing, when coughing or sneezing we exhale forcibly. Our abdominal muscles will contract suddenly (when there is an urge to cough or sneeze), raising the abdominal pressure. The rapid increase in pressure pushes the relaxed diaphragm up against the pleural cavity. This causes air to be forced out of the lungs.

Another function of the respiratory system is to sing and to speak. By exerting conscious control over our breathing and regulating flow of air across the vocal cords we are able to create and modify sounds.

### Lung Compliance

**Lung Compliance** is the magnitude of the change in lung volume produced by a change in pulmonary pressure. Compliance can be considered the opposite of stiffness. A low lung compliance would mean that the lungs would need a greater than average change in intrapleural pressure to change the volume of the lungs. A high lung compliance would indicate that little pressure difference in intrapleural pressure is needed to change the volume of the lungs. More energy is required to breathe normally in a person with low lung compliance. Persons with low lung compliance due to disease therefore tend to take shallow breaths and breathe more frequently.

**Determination of Lung Compliance** Two major things determine lung compliance. The first is the elasticity of the lung tissue. Any thickening of lung tissues due to disease will decrease lung compliance. The second is surface tensions at air water interfaces in the alveoli. The surface of the alveoli cells is moist. The attractive force, between the water cells on the alveoli, is called surface tension. Thus, energy is required not only to expand the tissues of the lung but also to overcome the surface tension of the water that lines the alveoli.

To overcome the forces of surface tension, certain alveoli cells (Type II pneumocytes) secrete a protein and lipid complex called ""Surfactant””, which acts like a detergent by disrupting the hydrogen bonding of water that lines the alveoli, hence decreasing surface tension.

### Control of respiration

#### central control

#### Peripheral control

CO2 is converted to HCO3; most CO2 produced at the tissue cells is carried to lungs in the form of HCO3

  * CO2 & H2O form carbonic acid (H2CO3)
  * changes to H CO3 & H+ ions
  * result is H+ ions are buffered by plasma proteins

## Respiratory System: Upper and Lower Respiratory Tracts

For the sake of convenience, we will divide the respiratory system in to the upper and lower respiratory tracts:

### Upper Respiratory Tract

The upper respiratory tract consists of the nose and the pharynx. Its primary function is to receive the air from the external environment and filter, warm, and humidify it before it reaches the delicate lungs where gas exchange will occur.

Air enters through the nostrils of the nose and is partially filtered by the nose hairs, then flows into the nasal cavity. The nasal cavity is lined with epithelial tissue, containing blood vessels, which help warm the air; and secrete mucous, which further filters the air. The endothelial lining of the nasal cavity also contains tiny hairlike projections, called cilia. The _cilia_ serve to transport dust and other foreign particles, trapped in mucous, to the back of the nasal cavity and to the pharynx. There the mucus is either coughed out, or swallowed and digested by powerful stomach acids. After passing through the nasal cavity, the air flows down the pharynx to the larynx.

### Lower Respiratory Tract

The lower respiratory tract starts with the larynx, and includes the trachea, the two bronchi that branch from the trachea, and the lungs themselves. This is where gas exchange actually takes place.

  1. Larynx

The larynx (plural larynges), colloquially known as the voice box, is an organ in our neck involved in protection of the trachea and sound production. The larynx houses the vocal cords, and is situated just below where the tract of the pharynx splits into the trachea and the esophagus. The larynx contains two important structures: the epiglottis and the vocal cords.

The epiglottis is a flap of cartilage located at the opening to the larynx. During swallowing, the larynx (at the epiglottis and at the glottis) closes to prevent swallowed material from entering the lungs; the larynx is also pulled upwards to assist this process. Stimulation of the larynx by ingested matter produces a strong cough reflex to protect the lungs. Note: choking occurs when the epiglottis fails to cover the trachea, and food becomes lodged in our windpipe.

The vocal cords consist of two folds of connective tissue that stretch and vibrate when air passes through them, causing vocalization. The length the vocal cords are stretched determines what pitch the sound will have. The strength of expiration from the lungs also contributes to the loudness of the sound. Our ability to have some voluntary control over the respiratory system enables us to sing and to speak. In order for the larynx to function and produce sound, we need air. That is why we can't talk when we're swallowing.

  1. Trachea
  2. Bronchi
  3. Lungs

## Homeostasis and Gas Exchange

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/8/8b/Alveoli.svg/200px-Alveoli.svg.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Gas exchange

Homeostasis is maintained by the respiratory system in two ways: gas exchange and regulation of blood pH. Gas exchange is performed by the lungs by eliminating carbon dioxide, a waste product given off by cellular respiration. As carbon dioxide exits the body, oxygen needed for cellular respiration enters the body through the lungs. ATP, produced by cellular respiration, provides the energy for the body to perform many functions, including nerve conduction and muscle contraction. Lack of oxygen affects brain function, sense of judgment, and a host of other problems.

### Gas Exchange

Gas exchange in the lungs and in the alveoli is between the alveolar air and the blood in the pulmonary capillaries. This exchange is a result of increased concentration of oxygen, and a decrease of C02. This process of exchange is done through diffusion.

### External Respiration

External respiration is the exchange of gas between the air in the alveoli and the blood within the pulmonary capillaries. A normal rate of respiration is 12-25 breaths per minute. In external respiration, gases diffuse in either direction across the walls of the alveoli. Oxygen diffuses from the air into the blood and carbon dioxide diffuses out of the blood into the air. Most of the carbon dioxide is carried to the lungs **in plasma** as bicarbonate ions (HCO3-). When blood enters the pulmonary capillaries, the bicarbonate ions and hydrogen ions are converted to carbonic acid (H2CO3) and then back into carbon dioxide (CO2) and water. This chemical reaction also uses up hydrogen ions. The removal of these ions gives the blood a more neutral pH, allowing hemoglobin to bind up more oxygen. De-oxygenated blood "blue blood" coming from the pulmonary arteries, generally has an oxygen partial pressure (pp) of 40 mmHg and CO2 pp of 45 mmHg. Oxygenated blood leaving the lungs via the pulmonary veins has a O2 pp of 100 mmHg and CO2 pp of 40 mmHg. It should be noted that alveolar O2 pp is 105 mmHg, and not 100 mmHg. The reason why pulmonary venous return blood has a lower than expected O2 pp can be explained by "Ventilation Perfusion Mismatch".

### Internal Respiration

Internal respiration is the exchanging of gases at the cellular level.

#### The Passage Way From the Trachea to the Bronchioles

There is a point at the inferior portion of the trachea where it branches into two directions that form the right and left primary bronchus. This point is called the **Carina** which is the keel-like cartilage plate at the division point. We are now at the **Bronchial Tree**. It is named so because it has a series of respiratory tubes that branch off into smaller and smaller tubes as they run throughout the lungs.

#### Right and Left Lungs

![](../../../upload.wikimedia.org/wikipedia/commons/d/db/Illu_bronchi_lungs.jpg)

Diagram of the **lungs**

The **Right Primary Bronchus** is the first portion we come to, it then branches off into the **Lobar (secondary) Bronchi**, **Segmental (tertiary) Bronchi**, then to the **Bronchioles** which have little cartilage and are lined by simple cuboidal epithelium (See fig. 1). The bronchi are lined by pseudostratified columnar epithelium. Objects will likely lodge here at the junction of the Carina and the Right Primary Bronchus because of the vertical structure. Items have a tendency to fall in it, where as the Left Primary Bronchus has more of a curve to it which would make it hard to have things lodge there.

The **Left Primary Bronchus** has the same setup as the right with the lobar, segmental bronchi and the bronchioles.

The lungs are attached to the heart and trachea through structures that are called the **roots of the lungs**. The roots of the lungs are the bronchi, pulmonary vessels, bronchial vessels, lymphatic vessels, and nerves. These structures enter and leave at the **hilus** of the lung which is "the depression in the medial surface of a lung that forms the opening through which the bronchus, blood vessels, and nerves pass" (medlineplus.gov).

There are a number of **terminal bronchioles** connected to **respiratory bronchioles** which then advance into the **alveolar ducts** that then become **alveolar sacs**. Each bronchiole terminates in an elongated space enclosed by many air sacs called **alveoli** which are surrounded by blood capillaries. Present there as well, are **Alveolar Macrophages**, they ingest any microbes that reach the alveoli. The **Pulmonary Alveoli** are **microscopic**, which means they can only be seen through a microscope, membranous air sacs within the lungs. They are units of respiration and the site of gas exchange between the respiratory and circulatory systems.

### Cellular Respiration

First the oxygen must diffuse from the alveolus into the capillaries. It is able to do this because the capillaries are permeable to oxygen. After it is in the capillary, about 5% will be dissolved in the blood plasma. The other oxygen will bind to red blood cells. The red blood cells contain hemoglobin that carries oxygen. Blood with hemoglobin is able to transport 26 times more oxygen than plasma without hemoglobin. Our bodies would have to work much harder pumping more blood to supply our cells with oxygen without the help of hemoglobin. Once it diffuses by osmosis it combines with the hemoglobin to form oxyhemoglobin.

Now the blood carrying oxygen is pumped through the heart to the rest of the body. Oxygen will travel in the blood into arteries, arterioles, and eventually capillaries where it will be very close to body cells. Now with different conditions in temperature and pH (warmer and more acidic than in the lungs), and with pressure being exerted on the cells, the hemoglobin will give up the oxygen where it will diffuse to the cells to be used for cellular respiration, also called aerobic respiration. Cellular respiration is the process of moving energy from one chemical form (glucose) into another (ATP), since all cells use ATP for all metabolic reactions.

It is in the mitochondria of the cells where oxygen is actually consumed and carbon dioxide produced. Oxygen is produced as it combines with hydrogen ions to form water at the end of the electron transport chain (see chapter on cells). As cells take apart the carbon molecules from glucose, these get released as carbon dioxide. Each body cell releases carbon dioxide into nearby capillaries by diffusion, because the level of carbon dioxide is higher in the body cells than in the blood. In the capillaries, some of the carbon dioxide is dissolved in plasma and some is taken by the hemoglobin, but most enters the red blood cells where it binds with water to form carbonic acid. It travels to the capillaries surrounding the lung where a water molecule leaves, causing it to turn back into carbon dioxide. It then enters the lungs where it is exhaled into the atmosphere.

## Lung Capacity

![LungCapacity.jpg](../../../upload.wikimedia.org/wikipedia/commons/thumb/b/ba/LungCapacity.jpg/450px-LungCapacity.jpg)

The normal volume moved in or out of the lungs during quiet breathing is called **tidal volume**. When we are in a relaxed state, only a small amount of air is brought in and out, about 500 mL. You can increase both the amount you inhale, and the amount you exhale, by breathing deeply. Breathing in very deeply is **Inspiratory Reserve Volume** and can increase lung volume by 2900 mL, which is quite a bit more than the tidal volume of 500 mL. We can also increase expiration by contracting our thoracic and abdominal muscles. This is called **expiratory reserve volume** and is about 1400 ml of air. **Vital capacity** is the total of tidal, inspiratory reserve and expiratory reserve volumes; it is called vital capacity because it is vital for life, and the more air you can move, the better off you are. There are a number of illnesses that we will discuss later in the chapter that decrease vital capacity. Vital Capacity can vary a little depending on how much we can increase inspiration by expanding our chest and lungs. Some air that we breathe never even reaches the lungs! Instead it fills our nasal cavities, trachea, bronchi, and bronchioles. These passages aren't used in gas exchange so they are considered to be **dead air space**. To make sure that the inhaled air gets to the lungs, we need to breathe slowly and deeply. Even when we exhale deeply some air is still in the lungs,(about 1000 ml) and is called **residual volume**. This air isn't useful for gas exchange. There are certain types of diseases of the lung where residual volume builds up because the person cannot fully empty the lungs. This means that the vital capacity is also reduced because their lungs are filled with useless air.

## Stimulation of Breathing

There are two pathways of motor neuron stimulation of the respiratory muscles. The first is the control of voluntary breathing by the cerebral cortex. The second is involuntary breathing controlled by the medulla oblongata.

There are chemoreceptors in the aorta, the carotid body of carotid arteries, and in the medulla oblongata of the brainstem that are sensitive to pH. As carbon dioxide levels increase there is a buildup of carbonic acid, which releases hydrogen ions and lowers pH. Thus, the chemoreceptors do not respond to changes in oxygen levels (which actually change much more slowly), but to pH, which is dependent upon plasma carbon dioxide levels. **In other words, CO2 is the driving force for breathing**. The receptors in the aorta and the carotid sinus initiate a reflex that immediately stimulates breathing rate and the receptors in the medulla stimulate a sustained increase in breathing until blood pH returns to normal.

This response can be experienced by running a 100 meter dash. During this exertion (or any other sustained exercise) your muscle cells must metabolize ATP at a much faster rate than usual, and thus will produce much higher quantities of CO2. The blood pH drops as CO2 levels increase, and you will involuntarily increase breathing rate very soon after beginning the sprint. You will continue to breathe heavily after the race, thus expelling more carbon dioxide, until pH has returned to normal. Metabolic acidosis therefore is acutely corrected by respiratory compensation (hyperventilation).

## Regulation of Blood pH

Many of us are not aware of the importance of maintaining the acid/base balance of our blood. It is vital to our survival. Normal blood pH is set at 7.4, which is slightly alkaline or "basic". If the pH of our blood drops below 7.2 or rises above 7.6 then very soon our brains would cease functioning normally and we would be in big trouble. Blood pH levels below 6.9 or above 7.9 are usually fatal if they last for more than a short time. Another wonder of our amazing bodies is the ability to cope with every pH change – large or small. There are three factors in this process: the lungs, the kidneys and buffers.

So what exactly is pH? pH is the concentration of hydrogen ions (H+). Buffers are molecules which take in or release ions in order to maintain the H+ ion concentration at a certain level. When blood pH is too low and the blood becomes too acidic (acidosis), the presence of too many H+ ions is to blame. Buffers help to soak up those extra H+ ions. On the other hand, the lack of H+ ions causes the blood to be too basic (alkalosis). In this situation, buffers release H+ ions. Buffers function to maintain the pH of our blood by either donating or grabbing H+ ions as necessary to keep the number of H+ ions floating around the blood at just the right amount.

The most important buffer we have in our bodies is a mixture of carbon dioxide (CO2) and bicarbonate ion (HCO3). CO2 forms carbonic acid (H2CO3) when it dissolves in water and acts as an acid giving up hydrogen ions (H+) when needed. HCO3 is a base and soaks up hydrogen ions (H+) when there are too many of them. In a nutshell, blood pH is determined by a balance between bicarbonate and carbon dioxide.

**Bicarbonate Buffer System**. With this important system our bodies maintain homeostasis. (Note that H2CO3 is Carbonic Acid and HCO3 is Bicarbonate)

CO2 + H2O <\---> H2CO3 <\---> (H+) + HCO3

  * If pH is too high, carbonic acid will donate hydrogen ions (H+) and pH will drop.
  * If pH is too low, bicarbonate will bond with hydrogen ions (H+) and pH will rise.

Too much CO2 or too little HCO3 in the blood will cause acidosis. The CO2 level is increased when hypoventilation or slow breathing occurs, such as if you have emphysema or pneumonia. Bicarbonate will be lowered by ketoacidosis, a condition caused by excess fat metabolism (diabetes mellitus).

Too much HCO3 or too little CO2 in the blood will cause alkalosis. This condition is less common than acidosis. CO2 can be lowered by hyperventilation.

So, in summary, if you are going into respiratory acidosis the above equation will move to the right. The body's H+ and CO2 levels will rise and the pH will drop. To counteract this the body will breathe more and release H+. In contrast, if you are going into respiratory alkalosis the equation will move to the left. The body's H+ and CO2 levels will fall and the pH will rise. So the body will try to breathe less to release HCO3. You can think of it like a leak in a pipe: where ever there is a leak, the body will "fill the hole".

## Problems Associated With the Respiratory Tract and Breathing

The environment of the lung is very moist, which makes it a hospitable environment for bacteria. Many respiratory illnesses are the result of bacterial or viral infection of the lungs. Because we are constantly being exposed to harmful bacteria and viruses in our environment, our respiratory health can be adversely affected. There are a number of illnesses and diseases that can cause problems with breathing. Some are simple infections, and others are disorders that can be quite serious.

**Carbon Monoxide Poisoning**: caused when carbon monoxide binds to hemoglobin in place of oxygen. Carbon monoxide binds much tighter, without releasing, causing the hemoglobin to become unavailable to oxygen. The result can be fatal in a very short amount of time.

    Mild Symptoms: flu like symptoms, dizziness, fatigue, headaches, nausea, and irregular breathing
    Moderate Symptoms: chest pain, rapid heart beat, difficulty thinking, blurred vision, shortness of breath and unsteadiness
    Severe Symptoms: seizures, palpitations, disorientation, irregular heart beat, low blood pressure, coma and death.

**Pulmonary Embolism:** blockage of the pulmonary artery (or one of its branches) by a blood clot, fat, air or clumped tumor cells. By far the most common form of pulmonary embolism is a thromboembolism, which occurs when a blood clot, generally a venous thrombus, becomes dislodged from its site of formation and embolizes to the arterial blood supply of one of the lungs.

    Symptoms may include difficulty breathing, pain during breathing, and more rarely circulatory instability and death. Treatment, usually, is with anticoagulant medication.

## Upper Respiratory Tract Infections

The upper respiratory tract consists of our nasal cavities, pharynx, and larynx. Upper respiratory infections (URI) can spread from our nasal cavities to our sinuses, ears, and larynx. Sometimes a viral infection can lead to what is called a secondary bacterial infection. **"Strep throat"** is a primary bacterial infection and can lead to an upper respiratory infection that can be generalized or even systemic (affects the body as a whole). Antibiotics aren't used to treat viral infections, but are successful in treating most bacterial infections, including strep throat. The symptoms of strep throat can be a high fever, severe sore throat, white patches on a dark red throat, and stomach ache.

Sinusitis
    An infection of the cranial sinuses is called **sinusitis**. Only about 1-3% of URI's are accompanied by sinusitis. This "sinus infection" develops when nasal congestion blocks off the tiny openings that lead to the sinuses. Some symptoms include: post nasal discharge, facial pain that worsens when bending forward, and sometimes even tooth pain can be a symptom. Successful treatment depends on restoring the proper drainage of the sinuses. Taking a hot shower or sleeping upright can be very helpful. Otherwise, using a spray decongestant or sometimes a prescribed antibiotic will be necessary.

Otitis Media
    Otitis media in an infection of the middle ear. Even though the middle ear is not part of the respiratory tract, it is discussed here because it is often a complication seen in children who has a nasal infection. The infection can be spread by way of the '_auditory (Eustachian) tube_ that leads form the nasopharynx to the middle ear. The main symptom is usually pain. Sometimes though, vertigo, hearing loss, and dizziness may be present. Antibiotics can be prescribed and tubes are placed in the eardrum to prevent the buildup of pressure in the middle ear and the possibility of hearing loss.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/95/Tonsillitis.jpg/300px-Tonsillitis.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Photo of **Tonsillitis**.

Tonsillitis
    Tonsillitis occurs when the tonsils become swollen and inflamed. The tonsils located in the posterior wall of the nasopharynx are often referred to as adenoids. If you suffer from tonsillitis frequently and breathing becomes difficult, they can be removed surgically in a procedure called a tonsillectomy.

Laryngitis
    An infection of the larynx is called laryngitis. It is accompanied by hoarseness and being unable to speak in an audible voice. Usually, laryngitis disappears with treatment of the URI. Persistent hoarseness without a URI is a warning sign of cancer, and should be checked into by your physician.

  


## Lower Respiratory Tract Disorders

Lower respiratory tract disorders include infections, restrictive pulmonary disorders, obstructive pulmonary disorders, and lung cancer.

### Lower Respiratory Infections

Acute bronchitis
    An infection that is located in the primary and secondary bronchi is called bronchitis. Most of the time, it is preceded by a viral URI that led to a secondary bacterial infection. Usually, a nonproductive cough turns into a deep cough that will expectorate mucus and sometimes pus.
Pneumonia
    A bacterial or viral infection in the lungs where the bronchi and the alveoli fill with a thick fluid. Usually it is preceded by influenza. Symptoms of pneumonia include high fever & chills, with headache and chest pain. Pneumonia can be located in several lobules of the lung and obviously, the more lobules involved, the more serious the infection. It can be caused by a bacteria that is usually held in check, but due to stress or reduced immunity has gained the upper hand.

### Restrictive Pulmonary Disorders

Pulmonary Fibrosis
    Vital capacity is reduced in these types of disorders because the lungs have lost their elasticity. Inhaling particles such as sand, asbestos, coal dust, or fiberglass can lead to **pulmonary fibrosis**, a condition where fibrous tissue builds up in the lungs. This makes it so our lungs cannot inflate properly and are always tending toward deflation.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/e/e1/Asthma_before-after.png/400px-Asthma_before-after.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Diagram of the lungs during an **asthma attack**.

Asthma
    Asthma is a respiratory disease of the bronchi and bronchioles. The symptoms include wheezing, shortness of breath, and sometimes a cough that will expel mucus. The airways are very sensitive to irritants which can include pollen, dust, animal dander, and tobacco. Even being out in cold air can be an irritant. When exposed to an irritant, the smooth muscle in the bronchioles undergoes spasms. Most asthma patients have at least some degree of bronchial inflammation that reduces the diameter of the airways and contributes to the seriousness of the attack.

Emphysema
    Emphysema is a type of chronic obstructive pulmonary disease. Typically characterized by a loss of elasticity and surfactant in the alveoli, a loss of surface area decreases the gas exchange in the lungs. These patients have difficulty with too little expiratory pressure, not retaining inspired air long enough for sufficient gas exchange to happen.

Chronic Bronchitis
    Another type of chronic obstructive pulmonary disease, Chronic Bronchitis is caused by overproduction of mucus in the airways, causing an inadequate expiration of inspired air. Retention of air in the lungs reduces gas exchange at the alveoli, and can lead to a hypoxic drive. These patients are known as "blue bloaters", vulnerable to cyanosis and often have increased thoracic diameters.

## Respiratory Distress Syndrome

Pathophysiology

At birth the pressure needed to expand the lungs requires high inspiratory pressure. In the presence of normal surfactant levels the lungs retain as much as 40% of the residual volume after the first breath and thereafter will only require far lower inspiratory pressures. In the case of deficiency of surfactant the lungs will collapse between breaths, this makes the infant work hard and each breath is as hard as the first breath. If this goes on further the pulmonary capillary membranes become more permeable, letting in fibrin rich fluids between the alveolar spaces and in turn forms a hyaline membrane. The hyaline membrane is a barrier to gas exchange, this hyaline membrane then causes hypoxemia and carbon dioxide retention that in turn will further impair surfactant production.

Etiology

Type two alveolar cells produce surfactant and do not develop until the 25th to the 28th week of gestation, in this, respiratory distress syndrome is one of the most common respiratory disease in premature infants. Furthermore, surfactant deficiency and pulmonary immaturity together leads to alveolar collapse. Predisposing factors that contribute to poorly functioning type II alveolar cells in a premature baby are if the child is a preterm male, white infants, infants of mothers with diabetes, precipitous deliveries, cesarean section performed before the 38th week of gestation. Surfactant synthesis is influenced by hormones, this ranges form insulin and cortisol. Insulin inhibits surfactant production, explaining why infants of mothers with diabetes type 1 are at risk of development of respiratory distress syndrome. Cortisol can speed up maturation of type II cells and therefore production of surfactant. Finally, in the baby delivered by cesarean section are at greater risk of developing respiratory distress syndrome because the reduction of cortisol produced because the lack of stress that happens during vaginal delivery, hence cortisol increases in high stress and helps in the maturation of type II cells of the alveoli that cause surfactant.

Treatment

Today to prevent respiratory distress syndrome are animal sources and synthetic surfactants, and administrated through the airways by an endotracheal tube and the surfactant is suspended in a saline solution. Treatment is initiated post birth and in infants who are at high risk for respiratory distress syndrome.

## Sleep Apnea

![](../../../upload.wikimedia.org/wikibooks/en/thumb/1/18/Carlos_with_the_c-pap.jpg/200px-Carlos_with_the_c-pap.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

CPAP is the most common treatment for obstructive sleep apnea.

Sleep apnea or sleep apnoea is a sleep disorder characterized by pauses in breathing during sleep. These episodes, called apneas (literally, "without breath"), each last long enough so one or more breaths are missed, and occur repeatedly throughout sleep. The standard definition of any apneic event includes a minimum 10 second interval between breaths, with either a neurological arousal (3-second or greater shift in EEG frequency, measured at C3, C4, O1, or O2), or a blood oxygen desaturation of 3-4 percent or greater, or both arousal and desaturation. Sleep apnea is diagnosed with an overnight sleep test called polysomnogram. One method of treating central sleep apnea is with a special kind of CPAP, APAP, or VPAP machine with a Spontaneous Time (ST) feature. This machine forces the wearer to breathe a constant number of breaths per minute.

(CPAP), or continuous positive airway pressure, in which a controlled air compressor generates an airstream at a constant pressure. This pressure is prescribed by the patient's physician, based on an overnight test or titration.

## Nutrition for COPD (Chronic Obstructive Pulmonary Disease) Patients

Nutrition is particularly important for ventilator-dependent patient. When metabolizing macronutrients carbon dioxide and water are produced. The respiratory quotient (RQ) is a ratio of produced carbon dioxide to amount consumed. Carbohydrates metabolism produces the most amount of carbon dioxide so they have the highest (RQ). Fats produce the least amount of carbon dioxide along with proteins. Protein has a slightly higher RQ ratio. It is recommended that this kind of patient not exceed a 1.0 respiratory quotient (RQ). Lowering carbohydrates and supplementing fat or protein in the diet might not result in maintaining the desired outcome because, excess amounts fat or protein may also result in a respiratory quotient (RQ) higher than 1.0.

  * Please reference source and fact accuracy. It seems like by definition, it is impossible to exceed a respiratory quotient (RQ) of 1.0. *

## Case Study

**Cystic Fibrosis**

This disease is most common in Caucasians and will happen to 1 in every 2500 people. It is most known for its effects on the respiratory tract although it does effect other systems as well. The respiratory passages become clogged with a thick mucus that is difficult to expel even with vigorous coughing. Breathing becomes difficult and affected individuals run the risk of choking to death on their own secretions unless strenuous effort is made to clear the lungs multiple times every day. Victims frequently will die in the 20's of pneumonia. All of us secrete mucus by certain cells in the epithelium that line the respiratory passage ways. In normal cases the cells also secrete a watery fluid that will dilute the mucus making it easier to pass through the airways. In cystic fibrosis that secretion of watery fluid is impaired. This makes the mucus thicker and difficult to clear from the passageways. A recent discovery found that in cystic fibrosis is caused by a defect in a type of chloride protein found in apical membranes of epithelial calls in the respiratory system and elsewhere. This defect directly impedes the chlorine ions transport, which will then indirectly effect the transport of potassium ions. This causes the epithelium, to not create its osmotic gradient necessary for water secretion. It has been known for a long time that cystic fibrosis is caused by a recessive gene inheritance. This gene codes for a portion of the chloride channel protein, which can malfunction in a variety of ways, each with specific treatment required.

## Glossary

## References

  * Mader, Sylvia S. _Human Biology_. McGraw Hill Publishing, Burr Ridge, IL. 2004.
  * Van De Graaff, Kent M. _Human Anatomy_. McGraw Hill Publishing, Burr Ridge, IL. 2002.
  * Department of Environmental Biology, University of Adelaide, Adelaide, South Australia
  * [Wikipedia:Lung](http://en.wikipedia.org/wiki/Lung)
  * Medlineplus.gov. _Hilus_. [http://www2.merriam-webster.com/cgi-bin/mwmednlm?book=Medical&va=hilum+](http://www2.merriam-webster.com/cgi-bin/mwmednlm?book=Medical&va=hilum+)
  * [http://www.an-attorney-for-you.com/legal/carbon-monoxide.htm?gg&gclid=CIbc3P_r-YYCFQwpNAodgmLHJA](http://www.an-attorney-for-you.com/legal/carbon-monoxide.htm?gg&gclid=CIbc3P_r-YYCFQwpNAodgmLHJA)
  * www.ineedtoknow.com
  * _"The respiratory system"Authors Mary Kitteredge,intro. by C. Everett Koop, M.D.,SC.D., foreword by Sandra Thurman_

## External Resources

  * [Eastern Kentucky University Gary Ritchison Lecture Notes: Human Respiration (includes videos)](http://people.eku.edu/ritchisong/301notes6.htm)

  


# The Gastrointestinal System

## Introduction

Which organ is the most important organ in the body? Most people would say the heart or the brain, completely overlooking the gastrointestinal tract (_GI tract_). Though definitely not the most attractive organs in the body, they are certainly among the most important. The 30+ foot long tube that goes from the mouth to the anus is responsible for the many different body functions which will be reviewed in this chapter. The GI tract is imperative for our well being and our life-long health. A non-functioning or poorly functioning GI tract can be the source of many chronic health problems that can interfere with your quality of life. In many instances the death of a person begins in the intestines.

The old saying "you are what you eat" perhaps would be more accurate if worded "you are what you absorb and digest". Here we will be looking at the importance of these two functions of the digestive system: digestion and absorption.

The **Gastrointestinal System** is responsible for the breakdown and absorption of various foods and liquids needed to sustain life. Many different organs have essential roles in the digestion of food, from the mechanical disrupting by the teeth to the creation of bile (an [emulsifier](http://en.wikipedia.org/wiki/Emulsifier)) by the liver. Bile production of the liver plays a important role in digestion: from being stored and concentrated in the gallbladder during fasting stages to being discharged to the small intestine.

In order to understand the interactions of the different components we shall follow the food on its journey through the human body. During digestion, two main processes occur at the same time;

  * **Mechanical Digestion**: larger pieces of food get broken down into smaller pieces while being prepared for chemical digestion. Mechanical digestion starts in the mouth and continues into the stomach.
  * **Chemical Digestion**: starts in the mouth and continues into the intestines. Several different enzymes break down macromolecules into smaller molecules that can be absorbed.

The GI tract starts with the mouth and proceeds to the esophagus, stomach, small intestine (duodenum, jejunum, ileum), and then to the large intestine (colon), rectum, and terminates at the anus. You could probably say the human body is just like a big donut. The GI tract is the donut hole. We will also be discussing the pancreas and liver, and accessory organs of the gastrointestinal system that contribute materials to the small intestine.

## Layers of the GI Tract

The GI tract is composed of four layers or also know as Tunics. Each layer has different tissues and functions. From the inside out they are called: mucosa, submucosa, muscularis, and serosa.

**Mucosa**: The mucosa is the absorptive and secretory layer. It is composed of simple epithelium cells and a thin connective tissue. There are specialized goblet cells that secrete mucus throughout the GI tract located within the mucosa. On the mucosa layer there are Villi and Micro Villi.

**Submucosa**: The submucosa is relatively thick, highly vascular, and serves the mucosa. The absorbed elements that pass through the mucosa are picked up from the blood vessels of the submucosa. The submucosa also has glands and nerve plexuses.

**Muscularis**: The muscularis is responsible for segmental contractions and peristaltic movement in the GI tract. The muscularis is composed of two layers of muscle: an inner circular and outer longitudinal layer of smooth muscle. These muscles cause food to move and churn with digestive enzymes down the GI tract.

**Serosa**: The last layer is a protective layer. It is composed of avascular connective tissue and simple squamous epithelium. It secretes lubricating serous fluid. This is the visible layer on the outside of the organs.

## Accessory Organs

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/7/7d/Gray1024.png/220px-Gray1024.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Teeth, Tongue, and Salivary Glands

1\. Salivary glands

  * Parotid gland, submandibular gland, sublingual gland
  * Exocrine gland that produces saliva which begins the process of digestion with amylase

2\. Tongue

  * Manipulates food for chewing/swallowing
  * Main taste organ, covered in taste buds

3\. Teeth

  * For chewing food up

4\. Liver

  * Produces and excretes bile required for emulsifying fats. Some of the bile drains directly into the duodenum and some is stored in the gall bladder.
  * Helps metabolize proteins, lipids, and carbohydrates.
  * Urea, chief end product of mammalian metabolism, is formed in liver from amino acids and compounds of ammonia.
  * Breaks down insulin and other hormones.
  * Produces coagulation factors.

5\. Gallbladder

  * Bile storage.

6\. Pancreas

  * Exocrine functions: Digestive enzyme secretion. 
    * Stores zymogens (inactive enzymes) that will be activated by the brush boarder membrane in the small intestine when a person eats protein (amino acids).
    * Trypsinogen – Trypsin: digests protein.
    * Chymotypsinogen – Chymotrypsin: digests proteins.
    * Carboxypeptidases: digests proteins.
    * Lipase-lipid: digests fats.
    * Amylase: digests carbohydrates.
  * Endocrine functions: Hormone secretion. 
    * Somatostatin: inhibits the function of insulin. Produced if the body is getting too much glucose.
    * Glucagon: stimulates the stored glycogen in the liver to convert to glucose. Produced if the body does not have enough glucose.
    * Insulin: made in the beta cells of the Islets of Langerhans of the pancreas. Insulin is a hormone that regulates blood glucose.

7\. Vermiform appendix

  * There are a few theories on what the appendix does. 
    * Vestigal organ
    * Immune function
    * Helps maintain gut flora

## The Digestive System

![Digestive system diagram en.svg](../../../upload.wikimedia.org/wikipedia/commons/thumb/c/c5/Digestive_system_diagram_en.svg/300px-Digestive_system_diagram_en.svg.png)

The first step in the digestive system can actually begin before the food is even in your mouth. When you smell or see something that you just have to eat, you start to salivate in anticipation of eating, thus beginning the digestive process.

Food is the body's source of fuel. Nutrients in food give the body's cells the energy they need to operate. Before food can be used it has to be broken down into tiny little pieces so it can be absorbed and used by the body. In humans, proteins need to be broken down into amino acids, starches into sugars, and fats into fatty acids and glycerol.

During digestion two main processes occur at the same time:

  * **Mechanical Digestion**: larger pieces of food get broken down into smaller pieces while being prepared for chemical digestion. Mechanical digestion starts in the mouth and continues in to the stomach.
  * **Chemical Digestion**: several different enzymes break down macromolecules into smaller molecules that can be more efficiently absorbed. Chemical digestion starts with saliva and continues into the intestines.

The digestive system is made up by the alimentary canal, or the digestive tract, and other abdominal organs that play a part in digestion such as the liver and the pancreas. The alimentary canal is the long tube of organs that runs from the mouth (where the food enters) to the anus (where indigestible waste leaves). The organs in the alimentary canal include the mouth( for mastication),esophagus, stomach and the intestines. The average adult digestive tract is about thirty feet (30') long. While in the digestive tract the food is really passing _through_ the body rather than being _in_ the body. The smooth muscles of the tubular digestive organs move the food efficiently along as it is broken down into absorb-able atoms and molecules. During absorption, the nutrients that come from food (such as proteins, fats, carbohydrates, vitamins, and minerals) pass through the wall of the small intestine and into the bloodstream and lymph. In this way nutrients can be distributed throughout the rest of the body. In the large intestine there is re absorption of water and absorption of some minerals as feces are formed. The parts of the food that the body passes out through the anus is known as feces.

**Mastication**

Digestion begins in the mouth. A brain reflex triggers the flow of saliva when we see or even think about food. Saliva moistens the food while the teeth chew it up and make it easier to swallow. Amylase, which is the digestive enzyme found in saliva, starts to break down starch into simpler sugars before the food even leaves the mouth. The nervous pathway involved in salivary excretion requires stimulation of receptors in the mouth, sensory impulses to the brain stem, and parasympathetic impulses to salivary glands.

Swallowing your food happens when the muscles in your tongue and mouth move the food into your **pharynx**. The pharynx, which is the passageway for food and air, is about five inches (5") long. A small flap of skin called the epiglottis closes over the pharynx to prevent food from entering the trachea and thus choking. For swallowing to happen correctly a combination of 25 muscles must all work together at the same time. Salivary glands also produce an estimated three liters of saliva per day.

Enzyme Produced In Site of Release pH Level

Carbohydrate Digestion:

Salivary amylase
Salivary glands
Mouth
Neutral

Pancreatic amylase
Pancreas
Small intestine
Basic

Maltase
Small intestine
Small intestine
Basic

Protein Digestion:

Pepsin
Gastric glands
Stomach
Acidic

Trypsin
Pancreas
Small intestine
Basic

Peptidases
Small intestine
Small intestine
Basic

Nucleic Acid Digestion:

Nuclease
Pancreas
Small intestine
Basic

Nucleosidases
Pancreas
Small intestine
Basic

Fat Digestion:

Lipase
Pancreas
Small intestine
Basic

## Esophagus

![Gray1032.png](../../../upload.wikimedia.org/wikipedia/commons/thumb/a/a7/Gray1032.png/300px-Gray1032.png)

The **esophagus** (also spelled oesophagus/esophagus) or gullet is the muscular tube in vertebrates through which ingested food passes from the throat to the stomach. The esophagus is continuous with the laryngeal part of the pharynx at the level of the C6 vertebra. It connects the pharynx, which is the body cavity that is common to both the digestive and respiratory systems behind the mouth, with the stomach, where the second stage of digestion is initiated (the first stage is in the mouth with teeth and tongue masticating food and mixing it with saliva).

After passing through the throat, the food moves into the esophagus and is pushed down into the stomach by the process of _peristalsis_ (involuntary wavelike muscle contractions along the G.I. tract). At the end of the esophagus there is a sphincter that allows food into the stomach then closes back up so the food cannot travel back up into the esophagus.

**Histology**

The esophagus is lined with mucus membranes, and uses peristaltic action to move swallowed food down to the stomach.

The esophagus is lined by a _stratified squamous epithelium_, which is rapidly turned over, and serves a protective effect due to the high volume transit of food, saliva, and mucus into the stomach. The _lamina propria_ of the esophagus is sparse. The mucus secreting glands are located in the submucosa, and are connective structures called _papillae_.

The muscularis propria of the esophagus consists of _striated_ _muscle_ in the upper third (superior) part of the esophagus. The middle third consists of a combination of _smooth muscle_ and striated muscle, and the bottom (inferior) third is only smooth muscle. The distal end of the esophagus is slightly narrowed because of the thickened circular muscles. This part of the esophagus is called the lower esophageal sphincter. This aids in keeping food down and not being regurgitated.

The esophagus has a rich _lymphatic_ drainage as well.

## Stomach

The **stomach** is a thick walled organ that lies between the esophagus and the first part of the small intestine (the duodenum). It is on the left side of the abdominal cavity, the fundus of the stomach lying against the diaphragm. Lying beneath the stomach is the pancreas. The greater omentum hangs from the greater curvature.

A mucous membrane lines the stomach which contains glands (with _chief cells_) that secrete gastric juices, up to three quarts of this digestive fluid is produced daily. The gastric glands begin secreting before food enters the stomach due to the parasympathetic impulses of the vagus nerve, making the stomach also a storage vat for that acid.

The secretion of gastric juices occurs in three phases: cephalic, gastric, and intestinal. The cephalic phase is activated by the smell and taste of food and swallowing. The gastric phase is activated by the chemical effects of food and the distension of the stomach. The intestinal phase blocks the effect of the cephalic and gastric phases. Gastric juice also contains an enzyme named **pepsin**, which digests proteins, hydrochloric acid and mucus. Hydrochloric acid causes the stomach to maintain a pH of about 2, which helps kill off bacteria that comes into the digestive system via food.

The gastric juice is highly acidic with a pH of 1-3. It may cause or compound damage to the stomach wall or its layer of mucus, causing a peptic ulcer. On the inside of the stomach there are folds of skin call the gastric rugae. Gastric rugae make the stomach very extendable, especially after a very big meal.

![Gray1050-stomach.png](../../../upload.wikimedia.org/wikipedia/commons/thumb/b/ba/Gray1050-stomach.png/350px-Gray1050-stomach.png)

The stomach is divided into four sections, each of which has different cells and functions. The sections are: 1) Cardiac region, where the contents of the esophagus empty into the stomach, 2) Fundus, formed by the upper curvature of the organ, 3) Body, the main central region, and 4) Pylorus or atrium, the lower section of the organ that facilitates emptying the contents into the small intestine. Two smooth muscle valves, or sphincters, keep the contents of the stomach contained. They are the: 1) Cardiac or esophageal sphincter, dividing the tract above, and 2) Pyloric sphincter, dividing the stomach from the small intestine.

After receiving the **bolus** (chewed food) the process of peristalsis is started; mixed and churned with gastric juices the bolus is transformed into a semi-liquid substance called **chyme**. Stomach muscles mix up the food with enzymes and acids to make smaller digestible pieces. The pyloric sphincter, a walnut shaped muscular tube at the stomach outlet, keeps chyme in the stomach until it reaches the right consistency to pass into the small intestine. The food leaves the stomach in small squirts rather than all at once.

Water, alcohol, salt, and simple sugars can be absorbed directly through the stomach wall. However, most substances in our food need a little more digestion and must travel into the intestines before they can be absorbed. When the stomach is empty it is about the size of one fifth of a cup of fluid. When stretched and expanded, it can hold up to eight cups of food after a big meal.

**Gastric Glands**

There are many different gastric glands and they secret many different chemicals. Parietal cells secrete hydrochloric acid and intrinsic factor; chief cells secrete pepsinogen; goblet cells secrete mucus; argentaffin cells secrete serotonin and histamine; and G cells secrete the hormone gastrin.

  
**Vessels and nerves**

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/6/65/Gray532.png/220px-Gray532.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Nerves in the lower abdomen.

    **Arteries:** The arteries supplying the stomach are the left gastric, the right gastric and right gastroepiploic branches of the hepatic, and the left gastroepiploic and short gastric branches of the lineal. They supply the muscular coat, ramify in the submucous coat, and are finally distributed to the mucous membrane.

    **Capillaries:** The arteries break up at the base of the gastric tubules into a plexus of fine capillaries, which run upward between the tubules, anatomizing with each other, and ending in a plexus of larger capillaries, which surround the mouths of the tubes, and also form hexagonal meshes around the ducts.

    **Veins:** From these the veins arise, and pursue a straight course downward, between the tubules, to the submucous tissue; they end either in the lineal and superior mesenteric veins, or directly in the portal vein.

    **Lymphatics:** The lymphatics are numerous: They consist of a superficial and a deep set, and pass to the lymph glands found along the two curvatures of the organ.

    **Nerves:** The nerves are the terminal branches of the right and left urethra and other parts, the former being distributed upon the back, and the latter upon the front part of the organ. A great number of branches from the celiac plexus of the sympathetic are also distributed to it. Nerve plexuses are found in the submucous coat and between the layers of the muscular coat as in the intestine. From these plexuses fibrils are distributed to the muscular tissue and the mucous membrane.

  
**Disorders of the Stomach**

Disorders of the stomach are common. There can be a lot of different causes with a variety of symptoms. The strength of the inner lining of the stomach needs a careful balance of acid and mucus. If there is not enough mucus in the stomach, ulcers, abdominal pain, indigestion, heartburn, nausea and vomiting could all be caused by the extra acid.

Erosions, ulcers, and tumors can cause bleeding. When blood is in the stomach it starts the digestive process and turns black. When this happens, the person can have black stool or vomit. Some ulcers can bleed very slowly so the person won't recognize the loss of blood. Over time, the iron in your body will run out, which in turn, will cause anemia.

There isn't a known diet to prevent against getting ulcers. A balanced, healthy diet is always recommended. Smoking can also be a cause of problems in the stomach. Tobacco increases acid production and damages the lining of the stomach. It is not a proven fact that stress alone can cause an ulcer.

**Histology of the human stomach**

Like the other parts of the gastrointestinal tract, the stomach walls are made of a number of layers.

From the inside to the outside, the first main layer is the mucosa. This consists of an epithelium, the lamina propria underneath, and a thin bit of smooth muscle called the muscularis mucosa.

The submucosa lies under this and consists of fibrous connective tissue, separating the mucosa from the next layer, the muscularis externa. The muscularis in the stomach differs from that of other GI organs in that it has three layers of muscle instead of two. Under these muscle layers is the adventitia, layers of connective tissue continuous with the omenta.

The epithelium of the stomach forms deep pits, called fundic or oxyntic glands. Different types of cells are at different locations down the pits. The cells at the base of these pits are chief cells, responsible for production of pepsinogen, an inactive precursor of pepsin, which degrades proteins. The secretion of pepsinogen prevents self-digestion of the stomach cells.

Further up the pits, parietal cells produce gastric acid and a vital substance, intrinsic factor. The function of gastric acid is two fold 1) it kills most of the bacteria in food, stimulates hunger, and activates pepsinogen into pepsin, and 2) denatures the complex protein molecule as a precursor to protein digestion through enzyme action in the stomach and small intestines. Near the top of the pits, closest to the contents of the stomach, there are mucous-producing cells called goblet cells that help protect the stomach from self-digestion.

The muscularis externa is made up of three layers of smooth muscle. The innermost layer is obliquely-oriented: this is not seen in other parts of the digestive system: this layer is responsible for creating the motion that churns and physically breaks down the food. The next layers are the square and then the longitudinal, which are present as in other parts of the GI tract. The pyloric antrum which has thicker skin cells in its walls and performs more forceful contractions than the fundus. The pylorus is surrounded by a thick circular muscular wall which is normally tonically constricted forming a functional (if not anatomically discrete) pyloric sphincter, which controls the movement of chyme.

**Control of secretion and motility**

  
The movement and the flow of chemicals into the stomach are controlled by both the nervous system and by the various digestive system hormones.

The hormone gastrin causes an increase in the secretion of HCL, pepsinogen and intrinsic factor from parietal cells in the stomach. It also causes increased motility in the stomach. Gastrin is released by G-cells into the stomach. It is inhibited by pH normally less than 4 (high acid), as well as the hormone somatostatin.

Cholecystokinin (CCK) has most effect on the gall bladder, but it also decreases gastric emptying. In a different and rare manner, secretin, produced in the small intestine, has most effects on the pancreas, but will also diminish acid secretion in the stomach.

Gastric inhibitory peptide (GIP) and enteroglucagon decrease both gastric motility and secretion of pepsin. Other than gastrin, these hormones act to turn off the stomach action. This is in response to food products in the liver and gall bladder, which have not yet been absorbed. The stomach needs only to push food into the small intestine when the intestine is not busy. While the intestine is full and still digesting food, the stomach acts as a storage for food.

## **Small Intestine**

![](../../../upload.wikimedia.org/wikipedia/commons/0/04/Stomach_colon_rectum_diagram.gif)

Diagram showing the small intestine

The small intestine is the site where most of the chemical and mechanical digestion is carried out. Tiny projections called **villi** line the small intestine which absorbs digested food into the capillaries. Most of the food absorption takes place in the jejunum and the ileum.

The functions of a small intestine is, the digestion of proteins into peptides and amino acids principally occurs in the stomach but some also occurs in the small intestine. Peptides are degraded into amino acids; lipids (fats) are degraded into fatty acids and glycerol; and carbohydrates are degraded into simple sugars.

  


The three main sections of the small intestine is _The Duodenum, The Jejunum, The Ileum._

    **The Duodenum**

In anatomy of the digestive system, the **duodenum** is a hollow jointed tube connecting the stomach to the jejunum. It is the first and shortest part of the small intestine. It begins with the duodenal bulb and ends at the ligament of Treitz. The duodenum is almost entirely retro peritoneal. The duodenum is also where the bile and pancreatic juices enter the intestine.

    **The Jejunum**

The _Jejunum_ is a part of the small bowel, located between the distal end of duodenum and the proximal part of ileum. The jejunum and the ileum are suspended by an extensive mesentery giving the bowel great mobility within the abdomen. The inner surface of the jejunum, its mucous membrane, is covered in projections called villi, which increase the surface area of tissue available to absorb nutrients from the gut contents. It is different from the ileum due to fewer goblet cells and generally lacks Peyer's patches.

    **The Ileum**

Its function is to absorb vitamin B12 and bile salts. The wall itself is made up of folds, each of which has many tiny finger-like projections known as villi, on its surface. In turn, the epithelial cells which line these villi possess even larger numbers of micro villi. The cells that line the ileum contain the protease and carbohydrate enzymes responsible for the final stages of protein and carbohydrate digestion. These enzymes are present in the cytoplasm of the epithelial cells. The villi contain large numbers of capillaries which take the amino acids and glucose produced by digestion to the hepatic portal vein and the liver.

The terminal ileum continues to absorb bile salts, and is also crucial in the absorption of fat-soluble vitamins (Vitamin A, D, E and K). For fat-soluble vitamin absorption to occur, bile acids must be present.

## Large Intestine

![Large Intestinal Tract](../../../upload.wikimedia.org/wikipedia/commons/9/9b/Illu_intestine.jpg)

The large intestine (colon) extends from the end of the ileum to the anus. It is about 5 feet long, being one-fifth of the whole extent of the intestinal canal. It's caliber is largest at the commencement at the cecum, and gradually diminishes as far as the rectum, where there is a dilatation of considerable size just above the anal canal. It differs from the small intestine in by the greater caliber, more fixed position, sacculated form, and in possessing certain appendages to its external coat, the appendices epiploicæ. Further, its longitudinal muscular fibers do not form a continuous layer around the gut, but are arranged in three longitudinal bands or tæniæ.

The large intestine is divided into the cecum, colon, rectum, and anal canal. In its course, describes an arch which surrounds the convolutions of the small intestine. It commences in the right iliac region, in a dilated part, the cecum. It ascends through the right lumbar and hypochondriac regions to the under surface of the liver; here it takes a bend, the right colic flexure, to the left and passes transversely across the abdomen on the confines of the epigastric and umbilical regions, to the left hypochondriac region; it then bends again, the left colic flexure, and descends through the left lumbar and iliac regions to the pelvis, where it forms a bend called the sigmoid flexure; from this it is continued along the posterior wall of the pelvis to the anus.

There are trillions of bacteria, yeasts, and parasites living in our intestines, mostly in the colon. Over 400 species of organisms live in the colon. Most of these are very helpful to our health, while the minority are harmful. Helpful organisms _synthesize_ vitamins, like _B12_, _biotin_, and _vitamin K_. They breakdown toxins and stop proliferation of harmful organisms. They stimulate the immune system and produce short chain fatty acids (SCFAs) that are required for the health of colon cells and help prevent colon cancer. There are many beneficial bacteria but some of the most common and important are _Lactobacillus Acidophilus_ and various species of _Bifidobacterium._ These are available as "probiotics" from many sources.

## Pancreas, Liver, and Gallbladder

The pancreas, liver, and gallbladder are essential for digestion. The pancreas produces enzymes that help digest proteins, fats, and carbohydrates, the liver produces bile that helps the body absorb fat, and the gallbladder stores the bile until it is needed. The enzymes and bile travel through special channels called ducts and into the small intestine where they help break down the food.

**Pancreas**

The pancreas is located posterior to the stomach and in close association with the duodenum.

In humans, the pancreas is a 6-10 inch elongated organ in the abdomen located retro peritoneal. It is often described as having three regions: a head, body and tail. The pancreatic head abuts the second part of the duodenum while the tail extends towards the spleen. The pancreatic duct runs the length of the pancreas and empties into the second part of the duodenum at the ampulla of Vater. The common bile duct commonly joins the pancreatic duct at or near this point.

The pancreas is supplied arterially by the pancreaticoduodenal arteries, themselves branches of the superior mesenteric artery of the hepatic artery (branch of celiac trunk from the abdominal aorta). The superior mesenteric artery provides the inferior pancreaticoduodenal arteries while the gastroduodenal artery (one of the terminal branches of the hepatic artery) provides the superior pancreaticoduodenal artery. Venous drainage is via the pancreatic duodenal veins which end up in the portal vein. The splenic vein passed posterior to the pancreas but is said to not drain the pancreas itself. The portal vein is formed by the union of the superior mesenteric vein and splenic vein posterior to the body of the pancreas. In some people (as many as 40%) the inferior mesenteric vein also joins with the splenic vein behind the pancreas, in others it simply joins with the superior mesenteric vein instead.

The function of the pancreas is to produce enzymes that break down all categories of digestible foods (exocrine pancreas) and secrete hormones that affect carbohydrates metabolism (endocrine pancreas).

  * **Exocrine**

The pancreas is composed of pancreatic exocrine cells, whose ducts are arranged in clusters called acini (singular acinus). The cells are filled with secretory granules containing the precursor digestive enzymes (mainly trypsinogen, chymotrypsinogen, pancreatic lipase, and amylase) that are secreted into the lumen of the acinus. These granules are termed zymogen granules (zymogen referring to the inactive precursor enzymes.) It is important to synthesize inactive enzymes in the pancreas to avoid auto degradation, which can lead to pancreatitis.

The pancreas is near the liver, and is the main source of enzymes for digesting fats (lipids) and proteins - the intestinal walls have enzymes that will digest polysaccharides. Pancreatic secretions from ductal cells contain bicarbonate ions and are alkaline in order to neutralize the acidic chyme that the stomach churns out. Control of the exocrine function of the pancreas are via the hormone gastrin, cholecystokinin and secretin, which are hormones secreted by cells in the stomach and duodenum, in response to distension and/or food and which causes secretion of pancreatic juices.

The two major proteases which the pancreas are trypsinogen and chymotrypsinogen. These zymogens are inactivated forms of trypsin and chymotrypsin. Once released in the intestine, the enzyme enterokinase present in the intestinal mucosa activates trypsinogen by cleaving it to form trypsin. The free trypsin then cleaves the rest of the trypsinogen and chymotrypsinogen to their active forms.

Pancreatic secretions accumulate in intralobular ducts that drain the main pancreatic duct, which drains directly into the duodenum.

Due to the importance of its enzyme contents, injuring the pancreas is a very dangerous situation. A puncture of the pancreas tends to require careful medical intervention.

  * **Endocrine**

Scattered among the acini are the endocrine cells of the pancreas, in groups called the islets of Langerhans. They are:

Insulin-producing beta cells (50-80% of the islet cells) Glucagon-releasing alpha cells (15-20%) Somatostatin-producing delta cells (3-10%) Pancreatic polypeptide-containing PP cells (remaining %)

The islets are a compact collection of endocrine cells arranged in clusters and cords and are crisscrossed by a dense network of capillaries. The capillaries of the islets are lined by layers of endocrine cells in direct contact with vessels, and most endocrine cells are in direct contact with blood vessels, by either cytoplasmic processes or by direct apposition.

**Liver**

The liver is an organ in vertebrates, including human. It plays a major role in metabolism and has a number of functions in the body including glycogen storage, plasma protein synthesis, and drug detoxification. It also produces bile, which is important in digestion. It performs and regulates a wide variety of high-volume biochemical reaction requiring specialized tissues.

The liver normally weighs between 1.3 - 3.0 kilograms and is a soft, pinkish-brown "boomerang shaped" organ. It is the second largest organ (the largest being the skin) and the largest gland within the human body. its anatomical position in the body is immediately under the diaphragm on the right side of the upper abdomen, The liver lies on the right side of the stomach and makes a kind of bed for the gallbladder.

The liver is supplied by two main blood vessels on its right lobe: the hepatic artery and the portal vein. The hepatic artery normally comes off the celiac trunk. The portal vein brings venous blood from the spleen, pancreas, and small intestine, so that the liver can process the nutrients and byproducts of food digestion. The hepatic veins drain directly into the inferior vena cava.

The bile produced in the liver is collected in bile canaliculi, which merge from bile ducts. These eventually drain into the right and left hepatic ducts, which in turn merge to form the common hepatic duct. The cystic duct (from the gallbladder) joins with the common hepatic duct to form the common bile duct. Bile can either drain directly into the duodenum via the common bile duct or be temporarily stored in the gallbladder via the cystic duct. The common bile duct and the pancreatic duct enter the duodenum together at the ampulla of Vater. The branching's of the bile ducts resemble those of a tree, and indeed term "biliary tree" is commonly used in this setting.

The liver is among the few internal human organs capable of natural regeneration of lost tissue: as little as 25% of remaining liver can regenerate into a whole liver again. This is predominantly due to hepatocytes acting as unipotential stem cells. There is also some evidence of bio potential stem cells, called oval cell, which can differentiate into either hepatocytes or cholangiocytes (cells that line bile ducts).

The various functions of the liver are carried out by the liver cells or hepatocytes.

  * The liver produces and excretes bile requires for dissolving fats. Some of the bile drains directly into the duodenum, and some is stored in the gallbladder
  * The liver performs several roles in carbohydrate metabolism:
  * gluconeogenesis (the formation of glucose from certain amino acids, lactate or glycerol)
  * Glycogenolysis (the formation of glucose from glycogen)
  * Glycogenesis (the formation of glycogen from glucose)
  * The breakdown of insulin and other hormones
  * The liver is responsible for the mainstay of protein metabolism.
  * The liver also performs several roles in lipid metabolism:
  * cholesterol synthesis
  * The production of triglycerides (fats)
  * The liver produces coagulation factors I (fibrinogen), II (prothrombin), V, VII, IX, X and XI, as well as protein C, Protein S and antithrombin.
  * The liver breaks down hemoglobin, creating metabolites that are added to bile as pigment
  * The liver breaks down toxic substances and most medicinal products in a process called drug metabolism. This sometimes results in toxication, when the metabolite is more toxic than its precursor.
  * The liver converts ammonia to urea.
  * The liver stores a multitude of substances, including glucose in the form of glycogen, vitamin B12, iron, and copper
  * In the first trimester fetus, the liver is the main site of red blood cell production. By the 32nd weeks of gestation, the bone marrow has almost completely taken over that task.
  * The liver is responsible for immunological effects the reticuloendothelial system if the liver contains many immunologically active cells, acting as a 'sieve' for antigens carried to it via the portal system.

**Gallbladder**

The gallbladder is a pear shaped organ that stores about 50 ml of bile (or "gall") until the body needs it for digestion. The gallbladder is about 7-10cm long in humans and is dark green in appearance due to its contents (bile), not its tissue. It is connected to the liver and the duodenum by biliary tract.

The gallbladder is connected to the main bile duct through the gallbladder duct (cystic duct). The main biliary tract runs from the liver to the duodenum, and the cystic duct is effectively a "cul de sac", serving as entrance and exit to the gallbladder. The surface marking of the gallbladder is the intersection of the midclavicular line (MCL) and the trans pyloric plane, at the tip of the ninth rib. The blood supply is by the cystic artery and vein, which runs parallel to the cystic duct. The cystic artery is highly variable, and this is of clinical relevance since it must be clipped and cut during a cholecystectomy.

The gallbladder has a epithelial lining characterized by recesses called Aschoff's recesses, which are pouches inside the lining. Under epithelium there is a layer of connective tissue, followed by a muscular wall that contracts in response to cholecystokinin, a peptide hormone by the duodenum.

The gallbladder stores bile, which is released when food containing fat enters the digestive tract, stimulating the secretion of cholecystokinin (CCK). The bile emulsifies fats and neutralizes acids in partly digested food. After being stored in the gallbladder, the bile becomes more concentrated than when it left the liver, increasing its potency and intensifying its effect in fats.

## Anus

![Gray1078.png](../../../upload.wikimedia.org/wikipedia/commons/thumb/0/01/Gray1078.png/220px-Gray1078.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

The human anus is situated between the buttocks, posterior to the perineum. It has two anal sphincters, one internal, the other external. These hold the anus closed until defecation occurs. One sphincter consists of smooth muscle and its action is involuntary; the other consists of striated muscle and its action is voluntary. In many animals, the anus is surrounded by anal sacs. Role of the anus is when the rectum is full, the increase in intra-rectal pressure forces the walls of the anal canal apart allowing the fecal matter to enter the canal. The rectum shortens as material is forced into the anal canal and peristaltic waves propel the feces out of the rectum. The internal and external sphincters of the anus allow the feces to be passed by muscles pulling the anus up over the exiting feces.

## Conditions Affecting the Esophagus

There are two different types of conditions that may affect the esophagus. The first type is called congenital: meaning a person is born with it. The second type is called non-congenital: meaning the person develops it after birth. Some examples of these are:

**Tracheoesophageal fistula and esophageal atresia**

Both of these conditions are congenital. In _Tracheoesophageal fistula_ there is a connection between the esophagus and the wind pipe (trachea) where there shouldn't be one. In _Esophageal atresia_ the esophagus of a newborn does not connect to the stomach but comes to a dead end right before the stomach. Both conditions require corrective surgery and are usually detected right after the baby is born. In some cases, it can be detected before the baby is born.

**Esophagitis**

Esophagitis is inflammation of the esophagus and is a non-congenital condition. Esophagitis can be caused by certain medications or by infections. It can also be caused by gastroesophageal reflux disease (gerd), a condition where the esophageal sphincter allows the acidic contents of the stomach to move back up into the esophagus. Gastroesophageal reflux disease can be treated with medications, but it can also be corrected by changing what you eat.

## Conditions Affecting the Stomach and Intestines

Everybody has experienced constipation or diarrhea in their lifetime. With constipation, the contents of the large intestines don't move along fast enough and waste material stays in the large intestines so long. All water is extracted out of the waste and it becomes hard. With diarrhea you get the exact opposite reaction. Waste moves along too fast and the large intestines can't absorb the water before the waste is pushed through. Common flora bacteria assists in the prevention of many serious problems. Here are some more examples of common stomach and intestinal disorders:

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/6/6c/Acute_Appendicitis.jpg/220px-Acute_Appendicitis.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Acute Appendicitis: An exemplary case of acute appendicitis in a 10-year-old boy. The organ is enlarged and sausage-like (botuliform). This longitudinal section shows the angry red inflamed mucosa with its irregular luminal surface. Diagnosed and removed early in the course of the disease, this appendix does not show late complications, like transmural necrosis, perforation, and abscess formation.

**Appendicitis**

Appendicitis is the inflammation of the appendix, the finger-like pouch that extends from the cecum. The most common symptoms are abdominal pain, loss of appetite, fever, and vomiting. Kids and teenagers are the most common victims of appendicitis and must be corrected by surgery. While mild cases may resolve without treatment, most require removal of the inflamed appendix, either by laparotomy or laparoscopy. Untreated, mortality is high, mainly due to peritonitis and shock.

**Celiac Disease**

Celiac disease is a disorder in which a person's digestive system is damaged by the response of the immune system to a protein called gluten, which is found in rye, wheat, and barley, and also in foods like breakfast cereal and pizza crust. People that have celiac disease experience abdominal pain, diarrhea, bloating, exhaustion, and depression when they eat foods with gluten in them. They also have difficulty digesting their food. Celiac disease runs in families and becomes active after some sort of stress, like viral infections or surgery. The symptoms can be managed by following a gluten free diet. Doctors can diagnose this condition by taking a full medical history or with a blood test.

**Diverticulitis**

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/f/f1/Benign_gastric_ulcer_1.jpg/220px-Benign_gastric_ulcer_1.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Benign gastric ulcer

Diverticulitis is a common disease of the bowel, in particular the large intestine. Diverticulitis develops from diverticulosis, which involves the formation of pouches (diverticula) on the outside of the colon. Diverticulitis results if one of these diverticula becomes inflamed. In complicated diverticulitis, bacteria may subsequently infect the outside of the colon if an inflamed diverticula bursts open. If the infection spreads to the lining of the abdominal cavity (peritoneum), this can cause a potentially fatal peritonitis. Sometimes inflamed diverticula can cause narrowing of the bowel, leading to an obstruction. Also, the affected part of the colon could adhere to the bladder or other organ in the pelvic cavity, causing a fistula, or abnormal communication between the colon and an adjacent organ.

**Gastritis and Peptic ulcers**

Usually the stomach and the duodenum are resistant to irritation because of the strong acids produced by the stomach. But sometimes a bacteria called Helicobacter pylori or the chronic use of drugs or certain medications, weakens the mucous layer that coats the stomach and the duodenum, allowing acid to get through the sensitive lining beneath. This can cause irritation and inflammation of the lining of the stomach, which is called gastritis, or cause peptic ulcers, which are holes or sores that form in the lining of the stomach and duodenum and cause pain and bleeding. Medications are the best way to treat this condition.

**Gastrointestinal Infections**

Gastrointestinal infections can be caused by bacteria such as Campylobacter, Salmonella, E. coli, or Shigella. They can also be caused by viruses or by intestinal parasites like amebiasis and Giardiasis. The most common symptoms of gastrointestinal infections Abdominal pain and cramps, Diarrhea, and vomiting. These conditions usually go away on there own and don't need medical attention.

**Inflammatory Bowel Disease**

Inflammatory bowel disease is the chronic inflammation of the intestines, which usually affect older kids, teens and adults. There are two major types, _ulcerative colitis_ and _Crohn's_ _disease_ and indeterminate colitis, which occurs in 10-15% of patients. Ulcerative colitis usually affects just the rectum and large intestine, while Crohn's disease can affect the whole gastrointestinal tract from mouth to anus along with some other parts of the body. Patients with these diseases also suffer from extraintestinal symptoms including joint pain and red eye, which can signal a flare of the disease. These diseases are treated with medications and if necessary, Intravenous or IV feeding, or in the more serious cases, surgery to remove the damaged areas of the intestines.

**Polyp**

A polyp is an abnormal growth of tissue (tumor) projecting from a mucous membrane. If it is attached to the surface by a narrow elongated stalk it is said to be pedunculated. If no stalk is present it is said to be sessile. Polyps are commonly found in the colon, stomach, nose, urinary bladder and uterus. They may also occur elsewhere in the body where mucous membranes exist like the cervix and small intestine.

## Disorders of the Pancreas, Liver, and Gallbladder

Disorders of the pancreas, liver, and gallbladder affect the ability to produce enzymes and acids that aid in digestion. examples of these disorders are.

**Cystic Fibrosis**

Cystic fibrosis is a chronic, inherited illness where the production of abnormally thick mucous blocks the duct or passageways in the pancreas and prevents the digestive fluids from entering the intestines, making it difficult for the person with the disorder to digest protein and fats, which cause important nutrients to pass through without being digested. People with this disorder take supplements and digestive enzymes to help manage their digestive problems.

**Hepatitis**

Hepatitis is a viral condition that inflames a person's liver which can cause it to lose its ability to function. Viral hepatitis, like hepatitis A, B, and C, is extremely contagious. Hepatitis A, which is a mild form of hepatitis, can be treated at home, but more serious cases that involve liver damage, might require hospitalization.

**Cholecystitis**

Acute or chronic inflammation if the gallbladder causes abdominal pain. 90% of cases of acute cholecystitis are caused by the presence of gallstones. The actual inflammation is due to secondary infection with bacteria of an obstructed gallbladder, with the obstruction caused by the gallstones. Gallbladder conditions are very rare in kids and teenagers but can occur when the kid or teenager has sickle cell anemia or in kids being treated with long term medications.

**Cholestasis**

Cholestasis is the blockage in the supply of bile into the digestive tract. It can be "intrahepatic" (the obstruction is in the liver) or "extrahepatic" (outside the liver). It can lead to jaundice, and is identified by the presence of elevated bilirubin level that is mainly conjugated.

**Biliary colic**

This is when a gallstone blocks either the common bile duct or the duct leading into it from the gallbladder. This condition causes severe pain in the right upper abdomen and sometimes through to the upper back. It is described by many doctors as the most severe pain in existence, between childbirth and a heart attack. Other symptoms are nausea and vomiting and diarrhea, bleeding caused by continual vomiting, and dehydration caused by the nausea and diarrhea. Another more serious complication is total blockage of the bile duct which leads to jaundice, which if it is not corrected naturally or by surgical procedure can be fatal as it causes liver damage. The only long term solution is the removal of the gallbladder.

## Gastrointestinal Dysfunctions

As we age, the amount of digestive enzymes produced by the body drops way down. This leads to decreased and slower digestion, slower absorption of nutrients and increased accumulation of fecal mater in the intestinal tract. Undigested food material and metabolic waste can also build up due to slow elimination, starting of a series of health problems.

When digestion slows, it turns the intestines into a toxic environment. Helpful organisms cannot live in toxic environments. When the beneficial organisms die they are replaced by harmful organisms, such as yeasts and parasites, the most common being _Candida albicans_. This leads to changes in the intestinal wall which produces _leaky gut syndrome_ which allows many toxic chemicals to be introduced into the blood stream. As a result the entire toxic load of the body is increased, which causes a bigger burden on the liver, kidneys and other body organs. When this happens the organs that are normally used for eliminating waste and supplying nutrients the GI tract becomes into a large dump for waste. This problem is made worse by the use of junk food, prescriptions, over the counter medications, antibiotics and a diet that is too low in fiber.

Most people never even think about their GI tract. We are all concerned about what the outside of our body look like, but we completely ignore the inside. Because our bodies a very resilient. deterioration of the digestive system can go on for years with no symptoms or side-effect. When symptoms finally do appear they are usually very non-specific, they include: decreased energy, headaches, diarrhea, constipation, heartburn, and acid reflux. Over the years these symptoms become more serious, they include: asthma, food allergies, arthritis, and cancer.

Poor digestion, poor absorption, and bacterial imbalance can be traced to a lot of chronic conditions. Every organ in the body receives nutrients for the GI tract. I if the GI tract is malfunctioning then the whole body suffers.

It is possible to return good health to your GI tract by improving digestion, consuming the right amount of fiber, cutting out junk food and refined sugars.

You can improve the function of the intestines by taking fiber supplements and vitamins (especially B12 and vitamin K). Some doctors suggest herbal or vitamin enema's to cleanse and relieve constipation and to help stimulate _peristaltic movement_ which will help to move the bowels.

**Irritable Bowel Syndrome**

Irritable Bowel Syndrome (IBS) is a disorder with symptoms that are most commonly bloating, abdominal pain, cramping, constipation, and diarrhea. IBS causes a lot of pain and discomfort. It does not cause permanent damage to the intestines and does not lead to serious diseases such as cancer. Most of the people affected with IBS can control their symptoms with stress management, diet, and prescription medication. For others IBS can be debilitating, they may be unable to go to work, travel, attend social events or leave home for even short periods of time.

About 20 percent of the adult population has some symptoms of IBS, making it one of the most common intestinal disorders diagnosed by physicians. It is more common in men than women and in about 50 percent of people affected it starts at about age 35.

Researchers have not found out what exactly causes IBS. One idea is that people with IBS have a large intestine (colon) that is sensitive to certain foods and stress. The immune system may also be involved. It has also been reported that _serotonin_ is linked with normal GI functioning. 95 percent of the body's serotonin is located in the GI tract (the other 5 percent is in the brain). People with IBS have diminished receptor activity, causing abnormal levels of serotonin in the GI tract. Because of this IBS patients experience problems with bowel movement, mobility, and sensation having more sensitive pain receptors in their GI tract. Many IBS patients suffer from depression and anxiety which can make symptoms worse.

There is no cure for IBS, but medications are an important part of relieving symptoms. Fiber supplements or laxatives are helpful for constipation. Anti diarrhoeals such as Imodium can help with diarrhea. An antispasmodic is commonly prescribed for colon muscle spasms. Antidepressants and pain medication are also commonly prescribed. [12]

**Gastrointestinal Stromal Tumor**

Gastrointestinal Stromal Tumors or GIST is an uncommon type of cancer in the GI tract (esophagus, stomach, small intestine, and colon). These types of cancers begin in the connective tissue like fat, muscles, nerves , cartilage, etc.

GIST originates in the stroma cells. Stroma cells are strung along the GI tract and are part of the system that helps the body to know when to move food through the digestive system. Over half of Gist's occur in the stomach. Most cases occur in people between the ages of forty and eighty, but can also show up in a person of any age.

All GIST's of any size or location have the ability to spread. Even if a GIST is removed, it can reappear in the same area, or may even spread outside of the GI tract.

In the early stages, GIST is hard to diagnose because in the early stages symptoms cannot be recognized. In the later stages a person can have vague abdominal pain, vomiting, abdominal bleeding that shows up in stool or vomit, low blood counts causing anemia, and having an early feeling of being full causing a decrease in appetite.

GIST is now recognized as an aggressive cancer that is able to spread to other parts of the body. People who have been diagnosed with GIST should get treatment as soon as possible.

**Food Allergies**

Food allergies occur when the immune system thinks that a certain protein in any kind of food is a foreign object and will try to fight against it.

Only about eight percent of children and two percent of adults actually have a food allergy. A person can be allergic to any kind of food, but the most common food allergies are from nuts, cow's milk, eggs, soy, fish, and shellfish. Most people who have a food allergy are allergic to less than four different foods.

The most common signs of food allergies are hives, swelling, itchy skin, itchiness, tingling or swelling in the mouth, coughing, trouble breathing, diarrhea, and vomiting. The two most common chronic illness that are associated with food allergies are eczema and asthma.

Food allergies can be fatal if it causes the reaction called anaphylaxis. This reaction makes it hard for the person to breathe. This can be treated by an epinephrine injection.

**GERD, Heartburn, Acid Reflux**

GERD, or Gastroesophageal Reflux Disease occurs when the lower esophageal sphincter is not able to close properly. When this happens, contents from the stomach called reflux leak back into the esophagus and the stomach.

When the stomach refluxes, stomach acid touches the lining of the esophagus and causes it to have a burning feeling in the throat or the chest. This is what heartburn is. When you taste the fluid in the back of your throat, it is called acid indigestion. It is common for a person to get occasional heartburn, but when it occurs more than twice a week it can be considered as GERD. GERD can occur in people of all ages including infants.

Some symptoms of GERD include having a pain in your chest, hoarseness, having trouble swallowing, or having the feeling of food being stuck in your throat. The main symptoms are having persistent heartburn and acid regurgitation. GERD can also cause bad breath and a dry cough.

No one knows why people get GERD. Some things that could contribute to GERD are alcohol use, pregnancy, being overweight and smoking. Certain foods might also contribute like citrus fruits, caffeine, spicy, fatty, and dried foods, and also mint flavorings.

Over-the-counter antacids or medications that help stop acid production and help the muscles empty the stomach are commonly used to treat GERD.

**Constipation**

Not everyone is on the same schedule for having a bowel movement. Depending on the person, a "normal" schedule can range anywhere from three times a day to three times a week. If you start having bowel movements less than your own personal schedule, then you might be getting the signs of constipation.

Constipation is when you have trouble having bowel movements. The stool is very hard making it hard to pass and causing a person to strain. You may even feel like you have to have a bowel movement even after you have already had one.

When you digest food, the waste products go through your intestines by the muscles contracting. When in the large intestine, most of the water and salt from the waste products are reabsorbed because they are needed by the body for our everyday functions. You can become constipated if too much water is absorbed, or if waste products move too slowly.

Not getting enough fluids, a low fiber diet, age, not being physically active, depression, stress and pregnancy can all be causes of constipation. Medications and narcotics can also cause a person to get constipated. Chronic constipation may be a symptom of a liver problem such as a urea cycle disorder.

The best way for a person to treat constipation is to make sure that you are getting enough fluids as well as fiber in your diet. By doing this, the bulk of your stool is increased and also makes the stool softer so that it can move through your intestines more easily. Being more active and increasing your daily exercise also helps keep you regulated.

**Hemorrhoids**

Hemorrhoids (also known as haemorrhoids, emerods, or piles) are varicosities or swelling and inflammation of veins in the rectum and anus.

Two of the most common types of hemorrhoids are external and internal hemorrhoids.

  * **External hemorrhoids** are those that occur outside of the anal verge (the distal end of the anal canal). They are sometimes painful, and can be accompanied by swelling and irritation. Itching, although often thought to be a symptom from external hemorrhoids, is more commonly due to skin irritation. 
    * If the vein ruptures and a blood clot develops, the hemorrhoid becomes a **thrombosed hemorrhoid**.
  * **Internal hemorrhoids** are those that occur inside the rectum. As this area lacks pain sensory receptor|receptors, internal hemorrhoids are usually not painful and most people are not aware that they have them. Internal hemorrhoids, however, may bleed when irritated.
  * Untreated internal hemorrhoids can lead to two severe forms of hemorrhoids: prolapsed and strangulated hemorrhoids. 
    * **Prolapsed hemorrhoids** are internal hemorrhoids that are so distended that they are pushed outside of the anus.
    * If the anal sphincter muscle goes into spasm and traps a prolapsed hemorrhoid outside of the anal opening, the supply of blood is cut off, and the hemorrhoid becomes a **strangulated hemorrhoid**.

## Bleeding in the Gastrointestinal tract

Bleeding in the gastrointestinal tract doesn't always mean you have a disease, it's usually a symptom of a digestive problem. The cause of the bleeding may not be that serious, it could be something that can be cured or controlled such as hemorrhoids. However, locating the source of the bleeding is very important. The gastrointestinal tract contains many important organs like the esophagus, stomach, small intestine, large intestine or colon, rectum, and anus. Bleeding can come from one or more of these area from a small ulcer in the stomach, or a large surface like the inflammation of the colon. Sometimes a person doesn't even know they are bleeding. When this happens, it is called hidden, or occult bleeding. Simple tests can detect hidden blood in the stool.

**What Causes Bleeding in the Digestive Tract**

Esophageal bleeding may be caused by Mallory-Weiss syndrome which is a tear in the esophagus. Mallory-Weiss syndrome is usually caused by excessive vomiting or may be caused by childbirth, a hiatal hernia, or increased pressure in the abdomen caused by coughing. Various medications can cause stomach ulcers or inflammations. Medications containing aspirin or alcohol, and various other medications(mainly those used for arthritis) are some examples of these.

Benign tumors or cancer of the stomach may also cause bleeding. These disorders don't usually produce massive bleeding. The most common source of bleeding usually occurs from ulcers in the duodenum. Researchers believe that these ulcers are caused by excessive stomach acid and a bacteria called Helicobacter Pylori.

In the lower digestive tract, the most common source of bleeding is in the large intestine, and the rectum. Hemorrhoids are the most common cause of bleeding in the digestive tract. Hemorrhoids are enlarged veins in the anal area which produces bright red blood that you see in the toilet or on the toilet paper.

**How do you Recognize Bleeding in the Digestive Tract**

The signs of bleeding in the digestive tract vary depending on the site and severity of the bleeding. If the blood is coming from the rectum, it would be bright red blood. If it was coming from higher up in the colon or from the small intestine, the blood would be darker. When the blood is coming from the stomach, esophagus, or the duodenum, the stool would be black and tarry.

If the bleeding is hidden, or occult, a person may not notice changes in the stool color. If extensive bleeding occurs, a person may feel dizzy, faint, weak, short of breath, have diarrhea or cramp abdominal pain. Shock can also occur along with rapid pulse, drop in blood pressure, and difficulty urinating. Fatigue, lethargy, and pallor from anemia will settle in if the bleeding is slow. Anemia is when the bloods iron-rich substance, hemoglobin, is diminished.

**Common Causes of Bleeding in the Digestive Tract**

  * Hemorrhoids
  * Gastritis (inflammation)
  * Inflammation (ulcerative colitis)
  * Colo rectal Polyps
  * Colo rectal Cancer
  * Duodenal Ulcer
  * Enlarged Veins
  * Esophagitis (inflammation of the esophagus)
  * Mallory-Weiss Syndrome
  * Ulcers

Iron and beets can also turn the blood red or black giving a false indication of blood in the stool.

  
**How Bleeding in the Digestive Tract is Diagnosed**

To diagnose bleeding in the digestive tract the bleeding must be located and a complete history and physical are very important. Here are some of the procedures that diagnose the cause of bleeding.

**Endoscopy**

An endoscopy is a common diagnostic technique that allows direct viewing of the bleeding site. Since the endoscope can detect lesions and confirm the absence or presence of bleeding, doctors often use this method to diagnose acute bleeding, the endoscope can also be used to treat the cause of bleeding as well.

The endoscope is a flexible instrument that can be inserted through the mouth or rectum. The instrument allows the doctors to see inside the esophagus, stomach, duodenum(esophagoduodenoscopy), sigmoid colon(sigmoidoscopy), and rectum(rectoscopy, to collect small samples of tissues, take pictures, and stop the bleeding. There is a new procedure out using a long endoscope that can be inserted during surgery to locate a source of bleeding in the small intestine.

**Capsule Endoscopy**

Capsule endoscopy helps doctors to see and examine the lining of the middle part of the gastrointestinal tract, which includes the three parts of the small intestine (duodenum, jejunum, ileum). The capsule is a small pill sized video camera called an endoscope. It has its own lens and light that transfers the images to a monitor so the doctor can view them outside of the body. This process is also referred to as small bowel endoscopy, capsule endoscopy, or wireless endoscopy.

The most common reason for doing a capsule endoscopy is to look for the causes of bleeding that is coming from the small intestine. It is also able to help detect ulcers, tumors, and Crohn's disease.

**Angiography**

Angiography is a technique that uses dye to highlight blood vessels. This procedure is used when the patient is bleeding badly enough that it allows the dye to leak out of the blood vessels and identifies the bleeding site. In some situations, Angiography allows the patient to have medication injections that may stop the bleeding.

**Radionuclide Scanning**

Radionuclide scanning is a non-invasive screening technique used for locating sites of acute bleeding, especially in the lower GI tract. This procedure injects small amounts of radioactive material that either attach to the persons red blood cells or are suspended in the blood. Special pictures are taken that allows doctors to see the blood escaping. Barium x-rays, angiography, and radionuclide scans can be used to locate sites of chronic occult bleeding.

**How to Recognize Blood in the Stool and Vomit**

  * Bright red blood coating the stool
  * Dark blood mixed with the stool
  * Black or tarry stool
  * Bright red blood in the vomit
  * Grainy appearance in vomit

**Symptoms of Acute Bleeding**

  * Weakness
  * Shortness of breath
  * Dizziness
  * Cramp abdominal pain
  * Feeling light headed
  * Diarrhea

**Symptoms of Chronic Bleeding**

  * Fatigue
  * Shortness of breath
  * Lethargy
  * Pallor

  


## Colonoscopy

A colonoscopy is a test to look at the inside of your colon. Everyone should have a colonoscopy by the time they are 50 to check for diseases of the colon. Colonoscopy is best known for its use in early detection of colorectal cancer, the second leading cause of cancer deaths in the United States. Colon cancer develops from growths like polyps within the intestinal wall. These growths often take 5-10 years to develop usually without symptoms. You are at a higher risk to have this disease if you have a close relative who has had it. If you are going to develop a polyp, you will probably do so after age 50. So the American College of Gastroenterology (the digestive specialists) recommends screening examinations every 5 years for early detection and removal of these cancer-causing growths after that age. Don't make excuses! It's not so bad and it may save your life!

## Case Study

Bob had a history of chronic pain in his intestinal area. He wasn't so sure what it was. The doctor suspected what it was and gave Bob antibiotics. It helped. It so happened that whenever Bob ate popcorn or nuts he would get this pain. Sometimes it would just go away... other times he had to go on antibiotics. The doctor ordered some tests. Bob would have to stay away from nuts, popcorn, tomatoes, strawberries, and anything else with seeds or hard parts. Seems something in his bowels couldn't tolerate those foods. Bob ate a pretty healthy diet so he couldn't understand what was happening. A few years later, Bob had another series of painful episodes. The pain was so great Bob could hardly stand let alone go to work. This time the doctor did more tests and found out that his lower intestine was almost blocked. Surgery was ordered. What did Bob have?

## Glossary

Amebiasis 
    An inflammation if the intestines caused by infestation with Entameba histolytica (a type of ameba) and characterized by frequent loose stools flecked with blood and mucus

Amylase 
    An enzyme produces in the pancreas and salivary glands that help in the digestions of starches.

Bile 
    A bitter, alkaline, brownish-yellow or greenish-yellow fluid that is secreted by the liver, stored in the gallbladder, and discharged into the duodenum and aids in the emulsification, digestion, and absorption of fats. Also called gall.

Biotin 
    Biotin is used in cell growth, the production of fatty acids, metabolism of fats, and amino acids. It plays a role in the Krebs Cycle. Biotin is also helpful in maintaining a steady blood sugar level. It is often recommended for strengthening hair and nails.

B12 
    A vitamin important for the normal formation of red blood cells and the health of the nerve tissues. Undetected and untreated B12 deficiency can lead to anemia and permanent nerve and brain damage

Candida Albicans 
    Found in animals and in man. Has been isolated from the skin and mucosa of man, but has also been recovered from leaves, flowers, water, and soil. Reported to be allergenic. A common cause of superficial infection, oral and vaginal infection, sepsis, and disseminated disease. Cells from the organism are usually not airborne and are considered to be normal component of the flora of the mouth and other mucous membranes on the body.

Chemical digestion
    Is a chemical breakdown of food when being in the mouth (oral cavity). Is the digestive secretions of saliva that moistens food and introduces gastric juices and enzymes that are produced in the stimulation to certain macronutrients, such as, carbohydrates. In this, the mouth saliva carries an enzyme called amylase for breaking down carbohydrates.

Cholecystokinin (CCK) 
    Cholecystokinin (also called pancreozymin), this is a hormone in the small intestinal cells (intestinal mucosa) that is produced in response to food. This hormone regulates the release of secretions of many organs that aid digestion, such as, bicarbonate from the pancreas to reduce the acidity of digestive juices like the chyme that enters the small intestine form the stomach that contains hydrochloric acid (HCL).

Chylomicrons 
    The lipoproteins first formed after absorption of lipids form food.

Chyme 
    The thick semi fluid mass of partly digested food that is passed from the stomach to the duodenum.

Crohn's Disease 
    Described as skip lessions in the large and small bowel it is a malabsorption disorder that can affect the gastrointestinal tract for the mouth to the anus.

Deamination
    When an amino acid group breaks off an amino acid that makes a molecule of ammonia and keto acid.

Emulsifier 
    A mixture of two immiscible (unblendable) substances.

Gastrin  
    The stomach mucosa secretes a hormone gastrin that increases the release of gastric juices.

GI tract 
    Gastrointestinal Tract, The tube that extends from the mouth to the anus in which the movement of muscles and release of hormones and enzymes digest food.

Hydrochloric 
    The chemical substance hydrochloric acid is the water-based solution of hydrogen chloride (HCI) gas. It is a strong acid, the major component of stomach acid and of wide industrial use.

Lactobacillus Acidophilus 
    Important resident inhabitant of the human small and large intestines, mouth, and vagina. Secretes natural antibiotic substances which strengthen the body against various disease-causing microbes

Leaky gut syndrome 
    Abnormal level of intestinal permeability

Lingual lipase 
    An enzyme produced only in infancy to aid digestion of long-chain fatty acids.

Lipase 
    An enzyme produced by microorganisms that split the fat molecules into fatty acids which create flavor

Mechanical digestion
    The crushing of the teeth and rhythms made by the movement of the tongue, the teeth aid in tearing and pulverizing food, while the tongue helps with peristalsis (movement), of food down the esophagus.

Micelles 
    A product of lipids and bile assist in lipid absorption.

Microvilli 
    On the villi in the small intestine is mivrovilli, these projections called brush border microvilli secrete specific enzymes for disaccharide hydrolysis, these further aid the absorption of the carbohydrate by yielding a monosaccharide that then can go through portal circulation to liver circulation to be further processed into immediate use for energy or glycogen storage.

Peristalsis 
    The wavelike muscular contractions of the intestine or other tubular structure that propel the contents onward by alternate contraction and relaxation.

Pharynx 

Proliferation 
    The process of reproduction or division of cells

Proteases 
    Protein enzyme

Rennin 
    Only produced during infancy and is a gastric protease and functions with calcium to clot with milk proteins casein, to slow the movement of milk so that digestion is prolonged.

Serotonin 
    chemical messenger in the brain that affects emotions, behavior, and thought

Synthesize 
    To create something, such as chemicals in the body, from simpler, raw materials

Ulcerative Colitis 

Villi 
    A minute projection arising from a mucous membrane, especially one of the vascular projections of the small intestine.

Vitamin K 
    A substance that promotes the clotting of blood

**Case Study Answer** Bob has diverticulitis. The doctor was afraid that if he had another bad infection that scar tissue would eventually block his colon completely and burst, which would necessitate a colostomy. Bob ended up having to have surgery to remove the damaged part of his colon. The doctor removed almost 18 inches of Bob's large intestine. Bob is doing fine now and most importantly, he can now eat his favorite food - nuts! Note: Sometimes a diet rich in fiber can help you avoid this dreaded problem. Sometimes, like in Bob's case, the predisposition to have this problem runs in the family. All of his siblings and his father suffered from this same ailment. Stress is another factor that can exacerbate this disease. So.. don't worry, be happy and eat fiber!

## External links

  * [Appendicitis Update Review](http://www.appendicitisreview.com/), An updated Issue on Appendicitis

## References

1: Chen Ts, Chen PS. Intestinal autointoxication: A gastrointestinal leitmotive: Journal Clinical Gastroenterology

2: Ernst E. Colonic irrigation and the theory of autointoxication: A triumph of ignorance over science. Journal of Gastroenterology

3: Alvarez WC. Origin of the so-called auto-intoxication symptoms.

4: Donaldson AN. Relation of constipation to intestinal intoxication.

5: Kenney JJ. Fit for Life: Some notes on the book and its roots. Nutrition Forum

6: Use of enemas is limited. FDA consumer

7: Amebiasis associated with colonic irrigation - Colorado. Morbidity and Mortality Weekly Report

8: Istre GR and others. An outbreak of amebiasis spread by colonic irrigation at a chiropractic clinic

9: Benjamin R and others. The case against colonic irrigation

10: Eisele JW, Reay DT. Deaths related to coffee enemas

11: Jarvis WT. Colonic Irrigation. National Council Against Health Fraud.

12: National Digestive Disease Information Clearinghouse (NDDIC)  


# Nutrition

## The Community and Nutrition Programs

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/1/12/Harvard_food_pyramid.png/400px-Harvard_food_pyramid.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Harvard's Food Pyramid

Connections between nutrition and health have probably been understood, at least to some degree, among all people of all places and times. For example, around 400 BC Hippocrates said, "Let food be your medicine and medicine be your food." Understanding the physiological needs of our cells helps us understand why food has such an impact on overall health. In this chapter we introduce nutrition by examining how cells use different nutrients and then discuss disease conditions that are tied to nutritional problems.

## Nutrition and Health in the Community

The nutritional status of people in our communities is a concern not only for quality of life, but also for economics (treating illness costs far more than preventing it). Various public health agencies are striving to prevent nutritional deficiencies and improve overall health. In the U.S., the government supplies a variety of resources such as state assistance, WIC (Women Infant and Child), and so forth. In addition, there have been many government agencies and voluntary health and scientific associations, such as the American Heart Association, that focus on life style and dietary factors that prevent chronic and life-threatening diseases. The U.S. Department of Agriculture (USDA) and the U.S. Department of Health and Human Services (USDHHS) developed dietary guidelines in 1977 that were compiled and displayed as the food guide pyramid. The food guide pyramid was revised as "My Pyramid," but this new chart is confusing to most people. Harvard School of Public Health developed an alternative healthy eating pyramid (shown at left) based on long-term nutritional studies. This pyramid differs from the old USDA pyramid in several key aspects: for example, exercise is at the bottom to remind us of its important role in our health. Also, not all carbohydrates are at the bottom (white bread, white rice, and potatoes are now at the top with sugars), and not all oils are at the top (plant oils are at the bottom). Other resource, such as the Recommended Daily Allowance (RDA) have helped people become more aware of nutritional needs, yet obesity and chronic health problems continue to rise.

## Nutritional Requirements

Our bodies have both caloric and nutritional needs. Living tissue is kept alive by the expenditure of energy in ATP molecules, which energy came from the break down of food molecules. Caloric need refers to the energy needed each day to carry out the varied chemical reactions in each cell. When looking at a nutritional label, we can easily see how many Calories are in a serving. These Calories (big "C") are actually kilocalories (1000 calories). Technically, a calorie (little "c") is the amount of energy needed to raise the temperature of 1 mL of water by 1 °C. How many Calories a person needs daily varies greatly by age, sex, height, and physical activity levels. If the amount of energy taken in exceeds the amount of energy used, then the excess energy is stored as adipose tissue (fat), regardless of the source of the energy.

In addition to daily energy needs, there are nutritional needs to prevent the body from losing its own fats, carbohydrates, and proteins. Such molecules are continuously broken down, and must be replaced regularly. Essential amino acids and essential fatty acids are particularly important building blocks in replacing these molecules. Vitamins and minerals are not used as energy, but are essential in tissue and enzyme structure or reactions.

## Carbohydrates

Macronutrient
    An energy-yielding nutrient. Macronutrients are those nutrients that together provide the vast majority of metabolic energy to an organism. The three main macronutrients are carbohydrates, proteins, and fat.

Micronutrients
    Microminerals or trace elements, are dietary minerals needed by the human body in very small quantities (generally less than 100mg/day) as opposed to macrominerals which are required in larger quantities.

**Functions**

    **Glucose** it is the most easily used by the body. It is a simple carbohydrate that circulates in the blood and is the main source of energy for the muscles, central nervous system, and brain (the brain can also use ketone bodies).

**Carbohydrates** are made of organic compounds carbon, hydrogen, and oxygen.

There are three sizes of carbohydrate and they are distinguished by a classification of two that is, _simple carbohydrates_ (mono saccharides and disaccharides) and _complex carbohydrates_ (polysaccharides). Polysaccharides are the most abundant carbohydrate in the body along with glycogen.

The break down of polysaccharides goes as follows: Polysaccharides are digested into monosaccharides including glucose which goes into the intestinal epithelium and into the bloodstream. The molecules of glucose are taken by glucose transporters and delivered into the cells of the body. While glucose is in the cells it can be oxidized for energy or provide substrates to other metabolic reactions or of course into glycogen for storage.

    **A.** _Monosaccharides_ = Single carbohydrate unit such as, Glucose, Fructose, and Galactose.

    **B.** _Disaccharides_ = Two single carbohydrates bound together such as, Sucrose, Maltose, and Lactose.

    **C.** _Polysaccharides_ = Have many units of monosaccharides joined together such as, Starch and Fiber.

**Fiber**

Fiber is carbohydrates that cannot be digested. It is in all eatable plants such as fruits vegetables, grains and Legumes. There are many ways of categorizing fiber types. First, from the foods they come from such as grains, which is called cereal fiber. Second, if they are soluble fiber or insoluble fiber. Soluble fiber partially dissolves in water and insoluble fiber does not.

Adults need about 21-38 grams of fiber a day. Children ages 1 and up need 19 grams a day. On average Americans eat only 15 grams a day.

Fiber helps reduce the chances of having the following conditions: colon cancer, heart disease, type 2 diabetes, diverticular disease, and constipation.

**Glycemic Index**

Glycemic Index is a new way of classifying carbohydrates. It measures how fast and how far blood sugar will rise after consuming carbohydrates. Foods that are considered to have a high glycemic index are converted almost immediately to blood sugar which causes it to rise rapidly. Foods that are considered to have a low glycemic index are digested slower causing a slower rise in blood sugar. Examples of high glycemic index foods are potatoes, white rice, white flour, anything refined, anything with a lot of sugar which includes high fructose corn syrup. Examples of low glycemic index foods are whole grains (brown rice, 100% whole wheat bread, whole grain pasta, high fiber cereals), high fiber fruits and vegetables, and many legumes. According to the Harvard School of Public Health, "The most comprehensive list of the glycemic index of foods was published in the July, 2002, issue of the American Journal of Clinical Nutrition. A searchable database maintained by the University of Sydney is available online."

## Proteins

**Functions**

    **Protein** forms hormones, enzymes, and antibodies. It is part of fluid and electrolyte regulation, the buffering effect for pH, and transporter of nutrients. A good example of a protein is the oxygen carrying hemoglobin found in red blood cells.

**Proteins** are made of carbon, hydrogen, oxygen, and nitrogen, an inorganic molecule, the thing that clearly distinguishes them from the other macronutrients.

**A.** _Amino acids_ are the building blocks of proteins.

**B.** _Polypeptide_ are a group of amino acids bonded together 10-100 or more.

The body requires amino acids to produce new body protein (protein retention) and to replace damaged proteins (maintenance) that are lost in the urine.

Proteins are relatively large molecules made of amino acids joined together in chains by peptide bonds. Amino acids are the basic structural building units of proteins. They form short polymer chains called peptides or longer poly-peptides which in turn form structures called proteins. The process of protein synthesis is controlled by an mRNA template. In this process tRNA transfers amino acids to the mRNA to form protein chains.

There are twenty standard amino acids used by cells in making proteins. Vertebrates, including humans, are able to synthesize 11 of these amino acids from other molecules. The remaining nine amino acids cannot be synthesized by our cells, and are termed "'essential amino acids'". These essential amino acids must be obtained from foods.

  
The **9 Essential Amino Acids** have the following names: **H**istidine, **I**soleucine, **L**eucine, **L**ysine, **M**ethionine, **P**henylalanine, **T**hreonine, **T**ryptophan, **V**aline

You can remember these with this saying “Hey It's Like Lovely Material; Please Touch The Velvet”.

The **11 Non-essential Amino Acids** are as follows:

**A**lanine, **A**rginine, **A**spartic acid, **C**ysteine, **C**ystine, **G**lutamic acid, **G**lutamine, **G**lycine, **P**roline, **S**erine, T**yrosine**

How about this memory device, "Almost Always Aunt Cindy Can Get Great Gum Popping Sounds Together" (This section needs to be corrected. Cystine is not one of the 20 common amino acids. It should be replaced by asparagine which is missing from the list. Also histidine is not essential for adults while cysteine, tyrosine, histidine, and arginine are required for infants and growing children. Some amino acids are also essential for specific subpopulations, e.g., tyrosine for individuals with PKU.)

### The 20 Amino Acids and What They Do!

Amino Acid Abbrev. Remarks

Alanine
A
Ala
Very abundant, very versatile. More stiff than glycine, but small enough to pose only small steric limits for the protein conformation. It behaves fairly neutrally, can be located in both hydrophilic regions on the protein outside and the hydrophobic areas inside.

Cysteine
C
Cys
The sulfur atom binds readily to heavy metal ions. Under oxidizing conditions, two cysteines can join together in a disulfide bond to form the amino acid cystine. When cystines are part of a protein, insulin for example, this stabilizes tertiary structure and makes the protein more resistant to denaturation; disulphide bridges are therefore common in proteins that have to function in harsh environments including digestive enzymes (e.g., pepsin and chymotrypsin) and structural proteins (e.g., keratin). Disulphides are also found in peptides too small to hold a stable shape on their own (e.g., insulin).

Aspartic acid
D
Asp
Behaves similarly to glutamic acid. Carries a hydrophilic acidic group with strong negative charge. Usually is located on the outer surface of the protein, making it water-soluble. Binds to positively-charged molecules and ions, often used in enzymes to fix the metal ion. When located inside of the protein, aspartate and glutamate are usually paired with arginine and lysine.

Glutamate
E
Glu
Behaves similar to aspartic acid. Has longer, slightly more flexible side chain. Also serves as an excitatory neurotransmitter in the CNS.

Phenylalanine
F
Phe
Essential for humans. Phenylalanine, tyrosine, and tryptophan contain large rigid aromatic group on the side chain. These are the biggest amino acids. Like isoleucine, leucine and valine, these are hydrophobic and tend to orient towards the interior of the folded protein molecule.

Glycine
G
Gly
Because of the two hydrogen atoms at the α carbon, glycine is not optically active. It is the smallest amino acid, rotates easily, adds flexibility to the protein chain. It is able to fit into the tightest spaces, e.g., the triple helix of collagen. As too much flexibility is usually not desired, as a structural component it is less common than alanine.

Histidine
H
His
In even slightly acidic conditions protonation of the nitrogen occurs, changing the properties of histidine and the polypeptide as a whole. It is used by many proteins as a regulatory mechanism, changing the conformation and behavior of the polypeptide in acidic regions such as the late endosome or lysosome, enforcing conformation change in enzymes. However only a few histidines are needed for this, so it is comparatively scarce.

Isoleucine
I
Ile
Essential for humans. Isoleucine, leucine and valine have large aliphatic hydrophobic side chains. Their molecules are rigid, and their mutual hydrophobic interactions are important for the correct folding of proteins, as these chains tend to be located inside of the protein molecule.

Lysine
K
Lys
Essential for humans. Behaves similarly to arginine. Contains a long flexible side-chain with a positively-charged end. The flexibility of the chain makes lysine and arginine suitable for binding to molecules with many negative charges on their surfaces. E.g., DNA-binding proteins have their active regions rich with arginine and lysine. The strong charge makes these two amino acids prone to be located on the outer hydrophilic surfaces of the proteins; when they are found inside, they are usually paired with a corresponding negatively-charged amino acid, e.g., aspartate or glutamate.

Leucine
L
Leu
Essential for humans. Behaves similar to isoleucine and valine. See isoleucine.

Methionine
M
Met
Essential for humans. Always the first amino acid to be incorporated into a protein; sometimes removed after translation. Like cysteine, contains sulfur, but with a methyl group instead of hydrogen. This methyl group can be activated, and is used in many reactions where a new carbon atom is being added to another molecule.

Asparagine
N
Asn
Similar to aspartic acid. Asn contains an amide group where Asp has a carboxyl.

Proline
P
Pro
Contains an unusual ring to the N-end amine group, which forces the CO-NH amide sequence into a fixed conformation. Can disrupt protein folding structures like α helix or β sheet, forcing the desired kink in the protein chain. Common in collagen, where it often undergoes a post-translational modification to hydroxyproline. Uncommon elsewhere.

Glutamine
Q
Gln
Similar to glutamic acid. Gln contains an amide group where Glu has a carboxyl. Used in proteins and as a storage for ammonia.

Arginine
R
Arg
Functionally similar to lysine.

Serine
S
Ser
Serine and threonine have a short group ended with a hydroxyl group. Its hydrogen is easy to remove, so serine and threonine often act as hydrogen donors in enzymes. Both are very hydrophilic, therefore the outer regions of soluble proteins tend to be rich with them.

Threonine
T
Thr
Essential for humans. Behaves similarly to serine.

Valine
V
Val
Essential for humans. Behaves similarly to isoleucine and leucine. See isoleucine.

Tryptophan
W
Trp
Essential for humans. Behaves similarly to phenylalanine and tyrosine (see phenylalanine). Precursor of serotonin.

Tyrosine
Y
Tyr
Behaves similarly to phenylalanine and tryptophan (see phenylalanine). Precursor of melanin, epinephrine, and thyroid hormones.

  
Dietary proteins fall into two categories: complete proteins and incomplete proteins. Complete proteins include ample amounts of all essential amino acids. Examples of foods that will include these great complete proteins are meat, fish, poultry, cheese, eggs, and milk. Incomplete proteins contain some but not all of the essential amino acids required by the human body. Examples of incomplete proteins include legumes, rice, and leafy green vegetables. Someone who chooses a vegan lifestyle must be careful to combine various plant proteins to obtain all the essential amino acids on a daily basis, but it can be accomplished.

Ingested proteins are broken down into amino acids during digestion. They are then absorbed by the villi of the small intestine and enter the blood stream. Our cells use these amino acids to assemble new proteins that are used as enzymes, cell receptors, hormones, and structural features. Each protein has its own unique amino acid sequence that is specified by the nucleotide sequence of the gene encoding that protein (see [Genetics and Inheritance](http://en.wikibooks.org/wiki/Human_Physiology/Genetics_and_inheritance)). If we are deficient in even a single amino acid, then our cells cannot make the proteins they require.

## Lipids

Macronutrient
    Provides 9 Kcalories per gram; it is an energy-yielding nutrient.

**Functions** are stored energy (adipose tissue), organ protection, temperature regulator, insulation such as myelin that covers nerve cells, lipid membrane around cells, and emulsifiers to keep fats dispersed in body fluids.

**Lipids** are made of organic molecules carbon, hydrogen, and oxygen. Fats consist of glycerol fatty acids joined by an ester bond.

  * **A.** _Triglycerides_ \- composed of three fatty acids and one glycerol molecule.
  * **B.** _Saturated fatty acid_ \- fatty acid with carbon chains fully saturated with hydrogen.
  * **C.** _Monounsaturated fatty acid_ \- fatty acid that has a carbon chain with one unsaturated double bond.
  * **D.** _Polyunsaturated fatty acid_ \- fatty acid that has two or more double bonds on the carbon chain.

**Essential fatty acids** part of the polyunsaturated fatty acids

  * **E.** _Linoleic acid_ an essential polyunsaturated fatty acid, its first double bond is at the 6th carbon and this is why it can be called Omega 6.
  * **F.** _Linolenic acid_ an essential polyunsaturated fatty acid, its first double bond is at the 3rd carbon and this is why it can be called Omega 3, and is the main member of the omega-3 family.
  * **G.** _Eicosapentaenoic acid (EPA)_ , may be derived inefficiently from linolenic acid and is the main fatty acid found in fish, also called omega 3.
  * **H.** _Docosahexaenoic acid (DHA)_, is an omega 3 fatty acid, is synthesized in body from alph-linolenic acid, and is present in fish. DHA is present in retina and brain.

Nonessential

  * **I.** _Sterols_ serve a vital function in the body, are produced by the body, and are not essential nutrients. This structure of a lipid is cholesterol which is a waxy substance that doesn't look like a triglyceride. It doesn't have a glycerol backbone or fatty acids, but because it is impermeable in water, it is a lipid.
  * **J.** _CIS- Trans Fatty acids_ hydrogenation makes monounsaturated and polyunsaturated fatty acids go from a state of their original form that is _cis_ to a _trans_ form. Addition of hydrogen ions will cause vegetable oil to harden. Additionally, they may stimulate cholesterol synthesis, and are potentially carcinogenic.

**Absorption process of triglycerides.** This is the fat that your body deals with most of the time. They are absorbed with the transport of chylomicrons into the lymphatic system which in turn will pour into the blood stream at the thoracic duct. Once it enters the blood stream, the chylomicrons take the triglycerides into the cells. The triglycerides that are on the outer part of the chylomicrons are broken down by lipoprotein lipase. Lipoprotein lipase can be found on the walls of capillaries. It is this enzyme that will break it into fatty acids and monoglycerides. The fatty acids are taken by the body's cells while the monoglycerides are taken to the liver to be processed. Medium chain triglycerides (MCTs) bypass chylomicron lipoprotein function allowing the body to quickly utilize them for energy. Due to their shorter chain length, MCTs possess 8.3kcal as opposed to the 9kcal content of long chain triglycerides. MCT consumption due to the immediate utilization by the body also results in the rapid formation of ketones and less ability to be stored as adipose tissue.

**More Info on Lipids:**

  * 1\. Lipids are structural components found in every cell of the human body. That is, they form the lipid bilayer found in individual cells. They also serve as the myelin sheath found in neurons.
  * 2\. Lipids provide us with energy. Most of that energy is in the form of triacylglycerols.
  * 3\. Both lipids and lipid derivatives serve as vitamins and hormones.
  * 4\. Lipophilic bile acids aid in lipid solubility.

**Recommendations for Fat Intake:** Although there are different types of fat the effect on health and disease, the basic message is simple: leave out the bad fats and replace them with good fats. Try to limit saturated fats in your diet, and try to eliminate trans fats from partially hydrogenated oils.Replace saturated and trans fats with polyunsaturated and monounsaturated fats. As of January 1, 2006, trans fat must be listed on food labels. More and more "trans-fat" free products are becoming available. Keep in mind, though, that according to the FDA, a product claiming to have zero trans fat can actually contain up to a half gram. You may still want to scan the ingredient list for "partially hydrogenated vegetable oil" and "vegetable shortening," and look for an alternative product without those words.

## Vitamins and Minerals

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/b/be/La_Boqueria.JPG/300px-La_Boqueria.JPG)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Fruits and vegetables are often a good source of vitamins.

We all need micronutrients in small quantities to sustain health. Micronutrients include dietary minerals and vitamins. While all minerals and vitamins can be obtained through food, many people do not consume enough to meet their micronutrient needs and instead may take a supplement.

Microminerals or trace elements include at least iron, cobalt, chromium, copper, iodine, manganese, selenium, zinc, and molybdenum. They are dietary minerals needed by the human body in very small quantities (generally less than 100mg/day) as opposed to macrominerals which are required in larger quantities. (Note that the use of the term "mineral" here is distinct from the usage in the geological sciences.)

  


### Vitamins

Vitamins are organic compounds that are essential for our body to function properly. Most vitamins are obtained from what you consume, because the body is unable to manufacture most of the essential vitamins that you need to survive. Here are types of vitamins and their roles:

Vitamin Food Sources Functions Problems When Deficient Problems With Taking Too Much

A (retinol)
Ingested in a precursor form. Found in animal sources such as milk and eggs. Also found in carrots and spinach (contain pro vitamin A carotenoids).
Vitamin A is a fat-soluble vitamin. It helps cells differentiate, also lowering your risk of getting cancer. Vitamin A helps to keep vision healthy. It is required during pregnancy. Vitamin A also influences the function and development of sperm, ovaries and placenta and is a vital component of the reproductive process.
Night blindness, impaired growth of bones and teeth
Headache, dizziness, nausea, hair loss, abnormal development of fetus

B1 (thiamine)
Found in wheat germ, whole wheat, peas, beans, enriched flour, fish, peanuts and meats.
Vitamin B1 is a water-soluble vitamin that the body requires to break down carbohydrates, fat and protein. The body needs vitamin b in order to make adenine triphosphate (ATP). Vitamin B1 is also essential for the proper functioning of nerve cells.
Beriberi, muscular weakness, enlarged heart
Can interfere with the absorption of other vitamins

B2 (riboflavin)
Found in milk cheese, leafy green vegetables, liver, soybeans yeast and almonds. Exposure to light destroys riboflavin.
Vitamin B2 is a water-soluble vitamin that helps the body process amino acids and fats. Activated vitamin B6 and folic acid helps convert carbohydrates to adenosine triphosphate (ATP). Sometimes vitamin B2 can act as an antioxidant.
Dermatitis, blurred vision, growth failure
Unknown

B3 (niacin)
Found in beets, brewer's yeast, beef liver, beef kidney, pork, turkey, chicken, veal, fish, salmon, swordfish, tuna, sunflower seeds, and peanuts.
Vitamin B3 is required for cell respiration and helps release the energy in carbohydrates, fats, and proteins. It helps with proper circulation and healthy skin, functioning of the nervous system, and normal secretion of bile and stomach fluids. It is used in the synthesis of sex hormones, treating schizophrenia and other mental illnesses, and as a memory-enhancer.
Pellagra, diarrhea, mental disorders
High blood sugar and uric acid, vasodilation

C (ascorbic acid)
Found in citrus fruits such as oranges, grapefruit and lemon.
Vitamin C is an essential water-soluble vitamin. It is needed to make collagen. Vitamin C also aids in the formation of liver bile which helps to detoxify alcohol and other substances. Evidence indicates that vitamin C levels in the eye decrease with age and this may be a cause of cataracts. Vitamin C has been reported to reduce activity of the enzyme, aldose reductase, which helps protect people with diabetes. It may also protect the body against accumulation or retention of the toxic mineral, lead.
Scurvy, delayed wound healing, infections
Gout, kidney stones, diarrhea, decreased copper

D
Produced by the human body during exposure to the ultraviolet rays of the sun.
Vitamin D is a fat-soluble vitamin that helps maintain blood levels of calcium. Vitamin D is necessary for healthy bones and teeth. Vitamin D plays a role in immunity and blood cell formation and also helps cells differentiate this lowers your chance of getting cancer.
Lack of Vitamin D results in rickets for children and osteomalacia for adults.
Calcification of soft tissue, diarrhea, possible renal damage

E
Found in vegetable oils, nuts, and green leafy vegetables. Fortified cereals are also an important source of vitamin E in the United States.
Vitamin E is an antioxidant that protects cell membranes and other fat-soluble parts of the body, such as LDL cholesterol (the “bad” cholesterol), from damage.
Unknown
Diarrhea, nausea, headaches, fatigue, muscle weakness

K
Found in kale, collard greens, spinach, mustard greens, turnip greens and Brussels sprouts. Also found vegetable oils such as soybean, canola, cottonseed, and olive. Additionally, the normal flora of the large intestine produce vitamin K, which our body is able to absorb and use
Vitamin K by helping transport Ca, vitamin K is necessary for proper bone growth and blood coagulation.
Easy bruising and bleeding
Can interfere with anticoagulant medication

Folic acid
Found in many vegetables including, broccoli, peas, asparagus, spinach, green leafy types. Also found in fresh fruit, liver and yeast.
Coenzyme needed for production of hemoglobin and formation of DNA.
Megaloblastic anemia, spina bifida
May mask B12 deficiency

B12
Found in meat, fish, eggs and milk but not in vegetables.
Vitamin B12 is needed to make red blood cells. Supplements can help some types of anemia.
Pernicious anemia
Unknown

B6 (pyridoxine)
Found in cereals, yeast, liver, and fish.
Vitamin B6 is a coenzyme in amino acid synthesis.
Rare to be deficient, convulsions, vomiting, seborrhea, muscular weakness
Insomnia, neuropathy

_Folic acid and cancer prevention_

Women of childbearing age are often encouraged to take a folic acid supplement to help reduce the risk of certain birth defects. Research cited by the Harvard School of Public Health shows that folic acid may have even more benefits, and not just for the developing fetus. Their study shows that people who get more than the recommended amount of folic acid due to diet or supplements can actually lower the risk of developing colon or breast cancer. Since alcohol blocks the absorption of folic acid and inactivates circulating folate, this can be especially important to those who drink alcohol frequently (more than one drink per day). The current recommended intake for folic acid is 400 micrograms per day. There are many excellent sources of folic acid, including prepared breakfast cereals, beans, and fortified grains. So if you would like to reduce your risk of colon or breast cancer, be sure to get more than 400 micrograms per day!

_**Fat soluble vitamins A, D, E, K**_

With fat soluble vitamins you need the presence of fat in your diet to absorb them, this is because the bile will not be secreted to help with emulsification and therefore the fat vitamins will not be broken down for absorption. Fat soluble vitamins are stored in organs such as the liver, spleen, and other fatty tissues in the body. Because of this, excessive amounts of fat-soluble vitamins can accumulate in the body resulting in toxicity, but this rarely comes from excessive dietary intake but rather from improper use of vitamin supplements. The other, water-soluble vitamins, do not build up to toxic levels because they are regularly excreted in the urine.

### Minerals

Minerals are atoms of certain chemical elements that are essential for body processes. Minerals are _inorganic_, meaning that they do not contain the element carbon. They are either produced by our body, or we obtain them by eating certain foods that contain them. They are ions found in blood plasma and cell cytoplasm, such as sodium, potassium, and chloride. In addition, minerals represent much of the chemical composition of bones (calcium, phosphorus, oxygen). They also contribute to nerve and muscle activity (sodium, potassium, calcium). Minerals serve several many other functions as well. There are 21 minerals considered essential for our bodies. Nine of the essential minerals in the body account for less than .01% of your body weight. Because of the small amount of these minerals that our body needs, we call them _trace minerals_. The 12 most important minerals and their functions are listed below:

Mineral Source Use in the body

Calcium (Ca)
Calcium can be found in dairy products, dark green vegetables and legumes.
It contributes to bone and teeth formation. In addition, calcium also contributes to nerve and muscle action, and blood clotting.

Chloride (Cl)
Chloride is mainly found in table salt.
It plays a role in the acid-base balance, stomach acid formation, and body water balance.

Copper (Cu)
Copper can be found in seafood, nuts, and legumes.
It participates in the synthesis of hemoglobin and melanin.

Fluorine (F)
Fluorine is evident in fluoridated water, tea, and seafood.
It accounts for the maintenance of teeth, and perhaps the maintenance of bone as well.

Iodine (I)
Iodine is a component in iodized salt, marine fish and shellfish.
Although a very small amount is needed for our body, according to some, iodine still plays a role in our body's function. It can also be found in seaweed. It is needed for the thyroid hormone.

Iron (Fe)
Iron can be found in green leafy vegetables, whole grains foods, legumes, meats, and eggs.
It is needed for composition of hemoglobin, myoglobin, and certain enzymes.

Magnesium (Mg)
Magnesium is found in whole grains foods, and in green leafy vegetables.
It is the coenzyme found in several enzymes.

Phosphorus (P)
Phosphorus can be found in meat, poultry, and whole grain foods.
It serves as components of bones, teeth, phospholipids, ATP, and nucleic acids.

Potassium (K)
Potassium is widespread in the diet, especially in meats and grains.
It deals with muscle and nerve function, and also is a major component of intracellular fluid.

Sodium (Na)
Sodium is found in table salt, is a major component of water and also widespread in the diet.
It participates in the functioning of muscles and nerves.

Sulfur (S)
Sulfur is found in meat and diary products.
It is a component of many proteins.

Zinc (Zn)
Zinc is found in whole grain foods, meats, and seafood.
It is a component of many enzymes.

## Nutritional Disorders

Body Mass Index became popular during the early 1980s as obesity started to become a discernible issue in prosperous Western society. BMI provided a simple numeric measure of a person's "fatness" or "thinness", allowing health professionals to discuss the problems of over- and under-weight more objectively with their patients. However, BMI has become controversial because many people, including physicians, have come to rely on its apparent numerical "authority" for medical diagnosis – but that has never been the BMI's purpose. It is meant to be used as a simple means of classifying sedentary (physically inactive) individuals with an average body composition.[1] For these individuals, the current value settings are as follows: a BMI of 18.5 to 25 may indicate optimal weight; a BMI lower than 18.5 suggests the person is underweight while a number above 25 may indicate the person is overweight; a BMI below 15 may indicate the person has an eating disorder; a number above 30 suggests the person is obese (over 40, morbidly obese).

In physiology, the term “weight” is used interchangeably with “mass”. For a given body shape and given density, the BMI will be proportional to weight e.g. if all body weight increase by 50%, the BMI increases by 50%.

BMI is defined as the individual's body weight divided by the square of their height. The formulas universally used in medicine produce a unit of measure that is not dimensionless; it has units of kg/m2. Body mass index may be accurately calculated using any of the formulas below.

SI units
US units
UK mixed units

![\\mathrm{BMI} = \\frac{\\mathit{weight} \\ \\mathrm{\(kg\)}}{\\mathit{height}^2 \(\\mathrm{m^2}\)}](../../../upload.wikimedia.org/math/f/0/0/f008d4097fdb77b5f55e053a2477184c.png)
![\\mathrm{BMI} = 703\\cdot \\frac{\\mathit{weight} \\ \(\\mathrm{lb}\)}{\\mathit{height}^2 \\ \(\\mathrm{in^2}\)}](../../../upload.wikimedia.org/math/6/2/c/62cc7e0b1080ab5c7edec6cc1325a630.png)
![\\mathrm{BMI} = 6.35\\cdot \\frac{\\mathit{weight} \\ \(\\mathrm{stone}\)}{\\mathit{height}^2 \\ \(\\mathrm{m^2}\)}](../../../upload.wikimedia.org/math/1/1/1/1119936b9f4c7d967d58ef02375d85a0.png)

BMI Weight Status

Below 18.5
Underweight

18.5 - 24.9
Normal

25.0 - 29.9
Overweight

30.0 and Above
Obese

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/d/da/LA2-vx06-konsthallen-skulptur.jpg/220px-LA2-vx06-konsthallen-skulptur.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Either way can be a disorder.

The U.S. National Health and Nutrition Examination Survey of 1994 indicates that 59% of American men and 49% of women have BMIs over 25. Extreme obesity — a BMI of 40 or more — was found in 2% of the men and 4% of the women. There are differing opinions on the threshold for being underweight in females, doctors quote anything from 18.5 to 20 as being the lowest weight, the most frequently stated being 19. A BMI nearing 15 is usually used as an indicator for starvation and the health risks involved, with a BMI <17.5 being one of the criteria for the diagnosis of anorexia nervosa.

**Anorexia nervosa:** is a psychiatric diagnosis that describes an eating disorder characterized by low body weight and body image distortion with an obsessive fear of gaining weight. Individuals with anorexia often control body weight by voluntary starvation, purging, vomiting, excessive exercise, or other weight control measures, such as diet pills or diuretic drugs. It primarily affects young adolescent girls in the Western world and has one of the highest mortality rates of any psychiatric condition, with approximately 10% of people diagnosed with the condition eventually dying due to related factors.[1] Anorexia nervosa is a complex condition, involving psychological, neurobiological, and sociological components.[2]

**Bulimia nervosa:** commonly known as bulimia, is generally considered a psychological condition in which the subject engages in recurrent binge eating followed by an intentional purging. This purging is done in order to compensate for the excessive intake of the food and to prevent weight gain. Purging typically takes the form of vomiting; inappropriate use of laxatives, enemas, diuretics or other medication, and excessive physical exercise.

## Metabolism

**Absorptive and post absorptive stage of metabolism**

The body has two phases to its metabolic cycle. The first is known as the absorptive stage. This stage happens 3-4 hours after a typical meal. During this phase nutrients are absorbed by the body. In other words this is the stage where energy is stored into macromolecules. During the post-absorptive stage the nutrients are not being absorbed instead this is the stage where it is being mobilized.

**Insulin**

The changes in the body that occur between the absorptive and post-absorptive stages are triggered by the changes in the plasma concentration of insulin. Insulin encourages the synthesis of energy storage molecules. When plasma glucose levels in the bloodstream increase during the absorptive stage, insulin is secreted from the pancreas. When the plasma glucose levels decrease, the post-absorptive phase begins. Insulin acts on several different tissues in the body and influences almost every major aspect of energy metabolism. Insulin supports and promotes all aspects of the absorptive phase by helping store energy in all tissues. It also inhibits the reactions of the post-absorptive phase. Insulin also affects the transport of nutrients across the membrane of ALL body cells except for those located in the liver and CNS. Insulin also has a part in growth where it needs to be present in the blood stream in order for the hormones to effect normally.

**Epinephrine and sympathetic nervous activity on metabolism**

The sympathetic system and epinephrine suppress insulin and stimulate glucagon secretion. This effects the post absorptive phase by making metabolic adjustments. During the post absorptive phase, plasma glucose levels decrease and cause an increase of glycogen secretion. It also acts directly on glucose receptors in the CNS. This causes a rise in epinephrine secretion by the adrenal medulla. The rise in epinephrine creates a cascade event where the body sends signals to all the tissues (except skeletal muscles) to switch to the post absorptive phase.

## Diabetes

**Diabetes Mellitus**

Diabetes is essentially any condition which is characterized by an increase in urine production and secretion. The Random House Webster's Unabridged Dictionary defines it as the following... "A disorder of carbohydrate metabolism, usually occurring in genetically predisposed individuals, characterized by inadequate production or utilization of insulin and resulting in excessive amounts of glucose in the blood and urine, excessive thirst, weight loss, and in some cases progressive destruction of small blood vessels leading to such complications as infections and gangrene of the limbs or blindness." In other words, when food is put into the body you get high levels of glucose in your blood stream thus resulting in the release of Insulin to take up and metabolize this glucose. It also stimulates the liver to store the glucose as glycogen, thus resulting in the storage of nutrients and the lowering of glucose levels in the blood. On the flip side you have Glucagon which helps in the breakdown of the stored nutrients when you need them, thus having the opposite effect of Insulin. People who are unable to produce insulin on their own, or are lacking/have damaged their insulin receptors develop what is known as "Diabetes Mellitus." There are two types of Diabetes Mellitus: Type I, aka, Insulin Dependent Diabetes Mellitus (IDDM), and Type II, aka, Non-Insulin Diabetes Mellitus (NIDDM).

Type I diabetes is believed to be an autoimmune disease which has been present since birth or has been brought on by exposure to a virus which causes insulin production by the pancreas to be impaired. This usually results in a person having to receive insulin from an external source. Without this external administration, the body would turn to the metabolism of fat, which leads to the build up of Ketones in the blood, which leads to blood acidosis and could result in a coma or possible death. The onset of Type I diabetes is most commonly seen under the age of 25.

Although Type II Diabetes is like Type I in many ways, it's onset is usually the result of poor lifestyle choices, particularly eating a diet high in sugars and fats while getting little or no physical exercise. Following this routine will quickly lead to damaging or the shutting down of your insulin receptors completely, thus resulting in the lack of glucose storage and the expulsion of essential nutrients from the body via urination. Just like with Type I Diabetes, Type II can have detrimental effects on the body including blindness, kidney disease, atherosclerosis, and again, even lead to the loss of extremities due to gangrene.

Doctors have projected that upwards of seven million Americans may have diabetes, yet many may not know it. If you or someone you know has been suffering from such things as: frequent urination, especially at night; unusual hunger and/or thirst; unexplained weight gain or loss; blurred vision; sores that don't heal; or excessive fatigue then it is highly recommended that you have your fasting blood glucose level checked by a physician. Maintaining an active lifestyle and making sound nutritional choices may greatly extend your life by protecting you from the ills of diabetes.

## Calories, Exercise, and Weight

**Energy Balance and Body Weight**

Energy is measured in units called calories. A calorie is the amount of energy that is needed to raise the temperature of 1 gram of water by one degree Celsius. Because a calorie is such a small amount, scientists use a larger unit to measure intake, called a _kilocalorie_. A kilocalorie is also referred to often as a capital "C" _Calorie_, and is equal to 1000 calories. When we "count" calories, we are actually counting the big Calories.

The old saying, "you are what you eat" is very much true. According to scientists, the average adult consumes 900,000 calories per year. Most people tend to take in more calories than their body needs. An intake of 120 extra calories a day, or around 5% excess in calories, yields an annual increase of 12 extra pounds of body weight. The more developed countries tend to consume more calories than others because of the increasing availability and dieting habits of eating refined foods with little nutrition in them and lots of saturated fat. In our society, there is a huge emphasis put on a person's image and how thin they are, and less emphasis put on what's most important--the nutrition our body receives. While our body do need calories every day to keep us going, we need to watch the amount of calories we consume in order to maintain good health and proper body weight.

Our Caloric intake is linked directly to our health status. Being _overweight_ is generally defined as being 15-20% above ideal body weight, while _obesity_ is defined as being more than 20% above it. People who weigh 10% less than ideal are considered _underweight_. This is less common in the more developed countries. In less developed countries such as South Africa, being underweight is quite common because they lack the nutrition to maintain good health.

**How do we gain weight?** When we consume more calories than our body can burn in a day, the excess energy is stored in specialized cells as fat. It is also important to know that the three classes of nutrients have different Caloric contents. Carbohydrates and proteins contain only four Calories per gram, while fat contains about nine. Because of this, it is essential that we watch our amount of fat intake. If we continuously feed our body more calories than is needed, our body will produce more fat cells, to store the excess energy. This contributes to gaining weight.

It is more difficult for chronically overweight persons to lose weight than normal-weight persons. This is because they are constantly fighting the body's own weight-control system, which responds as if the excess weight were normal. Our body is capable of measuring how much we intake, and maintaining our weight. When an overweight person goes on a diet, and consumes less calories, their body will respond as if they are starving, and try to save energy where it can to make up for the decrease in received calories.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/b/be/Marines_do_pushups.jpg/300px-Marines_do_pushups.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Exercise is a great way to maintain healthy body weight.

  
**Maintaining a healthy body weight**

To maintain a stable body weight, our consumption of calories needs to be equal to the amount of calories we use in a day. You can determine your daily energy needs by determining your _basal metabolic rate (BMR)_. Your BMR is the energy your body needs to perform essential activities. Some examples of essential activities are breathing, and maintaining organ function. Your metabolic rate can be influenced by your age, gender, muscular activity, body surface area and environmental temperature.

**Physical Activity: An efficient way to use calories**

Although the BMR stays about the same, we can dramatically change the amount of calories we burn in a day by participating in physical activity. It is important to note that heavier people do more work per hour than normal-weight people, for the same level of activity. We must spend about 3,500 Calories to lose one pound of fat. The best approach to weight loss, recommended by nutritionists, is to reduce the Caloric intake by a small amount each day while gradually increasing your amount of physical activity.

  


**BMR: Determining how many calories we need**

There are several factors that influence the BMR. Each person's body has different needs. BMR needs vary with gender and body composition. Muscle tissue consumes more energy than fat tissue. Typically, males need more calories than females, because they generally have more muscle tissue. Males use up calories faster than women. BMR also varies with your age as well. As we age, our body needs less and less calories. In addition, some health conditions can contribute to our needed calories. Health conditions such as fever, infections, and hyperthyroidism are examples of health conditions that lower your BMR. Our stress level effects our needed calorie intake as well. So does our increase or decrease in consumption, and our rate of metabolism, which varies with individual genetics.

**Calculating Your BMR**

Here are the steps to determining your BMR, or, the amount of energy your body needs to perform essential activities:

1\. First calculate your weight into kilograms. This is obtained by dividing the number of pounds by 2.2.

2\. For Males: multiply your weight in kilograms by 1.0. For Females: multiply your weight in kilograms by 0.9.

3\. This number approximates the number of Calories you consumer per hour. Now multiply this number by 24 to estimate how many Calories you need per day to support basic metabolic functions.

4\. The end result is your personal basal metabolic rate!

**Exercise**

Living a healthy, well-balanced life involves good nutrition and adequate exercise. They work hand in hand.

  * There are many benefits to exercising. 
    * Your chances of living longer increases.
    * You decrease your chances of getting diseases such as: 
      * Heart disease or problems with circulation
      * Many types of cancer
      * Type 2 diabetes
      * Arthritis
      * Osteoporosis
      * Depression
      * Anxiety
      * Controls weight
  * The costs of being physically active far outweigh the medical costs for those who are not physically active.
  * Cardiovascular Exercise 
    * Thirty minutes a day of moderate intensity exercise or physical activity has been shown to make noticeable increases in breathing and heart rate.
    * METs (metabolic equivalents) are the amount of energy it takes while at rest (1 calorie per every 2.2 pounds of body weight per hour). Moderate intensity activities can get you to burn energy 3-6 times more depending upon the activity.
    * Walking is ideal for everyone.
    * The MET scale chart on the Harvard School of Public Health web site is interesting.
  * Feeling what’s right 
    * A study suggests that those with disabilities, who are older, or who are out of shape get the same benefit from 30 minutes of lower intensity exercise as those who are younger and more fit do from more intense activity.
  * Beyond the heart 
    * There are other areas that benefit from different types of exercise such as strength training. These types of exercises help balance, muscle strength, and overall function. 
      * Resistance or strength training can possibly decrease the loss of lean muscle tissue and even replace some already lost. 
        * It can also decrease fat mass and increase resting metabolic rate.
        * It is effective in fighting osteoporosis.
        * It also helps maintain functional tasks in older populations.
      * Flexibility training or stretching exercises increases range of motion, decrease soreness, and injury.

## Glossary

Amino acids
    The building blocks of protein in the body. There are nine essential amino acids that are not manufactured by the body and must come from the diet.

Anabolism
    Refers the cumulative metabolic intracellular, molecular processes by which every cell repairs itself and grows.(synthesizing).

Anorexia
    A common eating disorder characterized by an abnormal loss of the appetite for food

antioxidants
    Compounds that protect against cell damage inflicted by molecules called oxygen-free radicals, which are a major cause of disease and aging.

Bulimia Nervosa
    Eating disorder characterized by binge eating followed by an intentional purging.

Catabolism
    The opposite of Anabolism. The metabolic process that breaks down molecules into smaller units. It is made up of degradative chemical reactions in the living cell.

Cirrhosis of the liver
    An irreversible advanced scarring of the liver as a result of chronic inflammation of the liver. Can be caused by alcoholism or obesity.

Complete Proteins
    Proteins that contain ample amounts of all of the essential amino acids

Deamination
    When an amino acid group breaks off an amino acid that makes a molecule of ammonia and ketoacid.

Diverticulosis
    A diet low in dietary fiber increases the risk, this is the pouches called diverticula formation on the outer portion of the large intestine.

Gastric Bypass Surgery
    An operation where a small gastric pouch is created and the remainder of the stomach bypassed

Incomplete Proteins
    Proteins that contain some but not all of all of the essential amino acids required by the body

Ipecac
    A drug used to induce vomiting

Kwashiorkor
    A childhood form of malnutrition caused by general lack of protein or deficiency in one or more amino acids. Appearance of a person with this is a swollen belly due to inadequate production of albumin, which causes the blood to have a lower osmotic pressure, resulting in more fluids escaping from the plasma.

Marasmus
    malnutrition cause by a lack of kilocalorie intake. Appearance of a person with this is a skeletal one.

Malnutrition
    An imbalanced nutrient and or energy intake.

Obesity
    A condition in which the natural energy reserve in fatty tissue increased to a point where it is thought to be a risk factor for certain health conditions or increased mortality

Peptide
    Two or more amino acids linked together by a bond called a peptide bond.

Polypeptide
    A string of amino acids linked together by peptide bonds. A protein is an example of a polypeptide.

Starvation
    A severe reduction in vitamin, nutrient, and energy intake, and is the most extreme form of malnutrition

## Health Information Online

## Review Questions

Answers for these questions can be found [here](http://en.wikibooks.org/wiki/Human_Physiology/Appendix_1:_answers_to_review_questions#Nutrition)

1\. Nonessential amino acids

    A) are stored in the body
    B) are only needed occasionally
    C) can be produced in the body
    D) can be taken in supplements

2\. Micronutrients include

    A) minerals and vitamins
    B) lipids and fatty acids
    C) amino acids and proteins
    D) vitamins and minerals

3\. The body requires amino acids to

    A) produce new red blood cells
    B) produce new protein
    C) replace damaged red blood cells
    D) replace damaged protein
    E) A and C
    F) B and D

4\. The function of lipids

    A) store energy
    B) organ protection
    C) temperature regulator
    D) emulsifiers
    E) all of the above

5\. This vitamin is a vital component of the reproductive process and lowers the risk of getting cancer

    A) B12
    B) Folic Acid
    C) Niacin
    D) Thiamine
    E) Retinol

6\. This vitamin is needed to make red blood cells

    A) B1
    B) B2
    C) B6
    D) B12

7\. This participates in the synthesis of hemoglobin and melanin

    A) Copper
    B) Chloride
    C) Calcium
    D) Iron
    E) Iodine

8\. I go to visit my grandmother and see that she has multiple bruises- from this I may assume that

    A) she has a vitamin A deficiency
    B) she is old and just clumsy
    C) she has a vitamin K deficiency
    D) she has scurvy
    E) she has rickets

9\. As a pirate I may get scurvy because

    A) I am not getting enough vegetables on the ship
    B) I am not getting enough fruit on the ship
    C) I am eating too much fish on the ship
    D) I am getting too much sun on the ship
    E) I am drinking too much rum on the ship

10\. I am taking anticoagulant medication and it doesn’t seem to be working, this could be because

    A) I have too much vitamin A
    B) I have too much B12
    C) I have too much sodium
    D) I have too much vitamin E
    E) I have too much vitamin K

11\. Which of these are fat soluble?

    A) Vitamin K
    B) Vitamin E
    C) Vitamin D
    D) Vitamin A
    E) All of the above

## References

  * Van De Graaff (2002) _Human Anatomy 6th ed._ McGraw-Hill Higher Education
  * Windmaier, P.W. Raff, H. Strang, T.S. (2004) _Vander, Sherman, & Luciano's Human Physiology, the Mechanisms of Body Function 9th ed._ Mcgraw-Hill
  * Starr & McMillan (2001) _Human Biology 6th ed._ Thomson-Brooks/cole.
  * Spurlock, Morgan (2004) _Super Size Me_ Hart Sharp Video
  * Sylvia S. Mader (2006) _Human Biology 9th ed._ McGraw-Hill Higher Education
  * Anatomy & Physiology Revealed (2007) _McGraw-Hill Higher Education_
  * Random House Webster's Unabridged Dictionary 2nd ed. (2001) Random House

  


# The Endocrine System

## Introduction To The Endocrine System

The endocrine system is a control system of ductless glands that secrete hormones within specific organs. Hormones act as "messengers," and are carried by the bloodstream to different cells in the body, which interpret these messages and act on them.

It seems like a far fetched idea that a small chemical can enter the bloodstream and cause an action at a distant location in the body. Yet this occurs in our bodies everyday of our lives. The ability to maintain homeostasis and respond to stimuli is largely due to hormones secreted within the body. Without hormones, you could not grow, maintain a constant temperature, produce offspring, or perform the basic actions and functions that are essential for life.

The endocrine system provides an electrochemical connection from the hypothalamus of the brain to all the organs that control the body metabolism, growth and development, and reproduction.

There are two types of hormones secreted in the endocrine system: Steroidal (or lipid based) and non-steroidal, (or protein based) hormones.

The endocrine system regulates its hormones through negative feedback, except in very specific cases like childbirth. Increases in hormone activity decrease the production of that hormone. The immune system and other factors contribute as control factors also, altogether maintaining constant levels of hormones.

## Types of Glands

![](../../../upload.wikimedia.org/wikipedia/commons/d/da/Illu_endocrine_system.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Major endocrine glands. (Male left, female on the right.) **1.** Pineal gland **2.** Pituitary gland **3.** Thyroid gland **4.** Thymus **5.** Adrenal gland **6.** Pancreas **7.** Ovary **8.** Testis

**Exocrine Glands** are those which release their cellular secretions through a duct which empties to the outside or into the lumen (empty internal space) of an organ. These include certain sweat glands, salivary and pancreatic glands, and mammary glands. They are not considered a part of the endocrine system.

**Endocrine Glands** are those glands which have no duct and release their secretions directly into the intercellular fluid or into the blood. The collection of endocrine glands makes up the endocrine system.

    The main endocrine glands are the pituitary (anterior and posterior lobes), thyroid, parathyroid, adrenal (cortex and medulla), pancreas and gonads.

    The pituitary gland is attached to the hypothalamus of the lower forebrain.

    The thyroid gland consists of two lateral masses, connected by a cross bridge, that are attached to the trachea. They are slightly inferior to the larynx.

    The parathyroid glands are four masses of tissue, two embedded posterior in each lateral mass of the thyroid gland.

    One adrenal gland is located on top of each kidney. The cortex is the outer layer of the adrenal gland. The medulla is the inner core.

    The pancreas is along the lower curvature of the stomach, close to where it meets the first region of the small intestine, the duodenum.

    The gonads (ovaries and testes) are found in the pelvic cavity.

## Hormones and Types

A **hormone** is a type of chemical signal. They are a means of communication between cells.

The endocrine system produces hormones that are instrumental in maintaining homeostasis and regulating reproduction and development. A hormone is a chemical messenger produced by a cell that effects specific change in the cellular activity of other cells (target cells). Unlike exocrine glands (which produce substances such as saliva, milk, stomach acid and digestive enzymes), endocrine glands do not secrete substances into ducts (tubes). Instead, endocrine glands secrete their hormones directly into the surrounding extra cellular space. The hormones then diffuse into nearby capillaries and are transported throughout the body in the blood.

The endocrine and nervous systems often work toward the same goal. Both influence other cells with chemicals (hormones and neurotransmitters). However, they attain their goals differently. Neurotransmitters act immediately (within milliseconds) on adjacent muscle, gland, or other nervous cells, and their effect is short-lived. In contrast, hormones take longer to produce their intended effect (seconds to days), may affect any cell, nearby or distant, and produce effects that last as long as they remain in the blood, which could be up to several hours.

In the following table there are the major hormones, their target and their function once in the target cell.

Endocrine Gland Hormone Released Chemical Class Target Tissue/Organ Major Function of Hormone

**Hypothalamus**
Hypothalamic releasing and inhibiting hormones
Peptide
Anterior pituitary
Regulate anterior pituitary hormone

**Posterior Pituitary**
Antidiuretic (ADH)
Peptide
Kidneys
Stimulates water reabsorption by kidneys

Oxytocin
Peptide
Uterus, mammary glands
Stimulates uterine muscle contractions and release of milk by mammary glands

**Anterior Pituitary**
Thyroid stimulating (TSH)
Glycoprotein
Thyroid
Stimulates thyroid

Adrenocorticotropic (ACTH)
Peptide
Adrenal cortex
Stimulates adrenal cortex

Gonadotropic (FSH, LH)
Glycoprotein
Gonads
Egg and sperm production, sex hormone production

Prolactin (PRL)
Protein
Mammary glands
Milk production

Growth (GH)
Protein
Soft tissue, bones
Cell division, protein synthesis and bone growth

**Thyroid**
Thyroxine (T4) and Triiodothyronie (T3)
Iodinated amino acid
All tissue
Increase metabolic rate, regulates growth and development

Calcitonin
Peptide
Bones, kidneys and intestine
Lowers blood calcium level

**Parathyroids**
Parathyroid (PTH)
Peptide
Bones, kidneys and intestine
Raises blood calcium level

**Adrenal Cortex**
Glucocorticoids (cortisol)
Steroid
All tissue
Raise blood glucose level, stimulates breakdown of protein

Mineralocorticoids (aldosterone)
Steroid
Kidneys
Reabsorb sodium and excrete potassium

Sex Hormones
Steroid
Gonads, skin, muscles and bones
Stimulates reproductive organs and brings on sex characteristics

**Adrenal Medulla**
Epinephrine and norepinephrine
Modified amino acid
Cardiac and other muscles
Released in emergency situations, raises blood glucose level, “fight or flight” response

**Pancreas**
Insulin
Protein
Liver, muscles, adipose tissue
Lowers blood glucose levels, promotes formation of glycogen

Glucagon
Protein
Liver, muscles, adipose tissue
Raises blood glucose levels

**Testes**
Androgens (testosterone)
Steroid
Gonads, skin, muscles and bone
Stimulates male sex characteristics

**Ovaries**
Estrogen and progesterone
Steroid
Gonads, skin, muscles and bones
Stimulates female sex characteristics

**Thymus**
Thymosins
Peptide
T lymphocytes
Stimulates production and maturation of T lymphocytes

**Pineal Gland**
Melatonin
Modified amino acid
Brain
Controls circadian and circannual rhythms, possibly involved in maturation of sexual organs

Hormones can be chemically classified into four groups:

  1. **Amino acid-derived**: Hormones that are modified amino acids.
  2. **Polypeptide and proteins**: Hormones that are chains of amino acids of less than or more than about 100 amino acids, respectively. Some protein hormones are actually glycoproteins, containing glucose or other carbohydrate groups.
  3. **Steroids**: Hormones that are lipids synthesized from cholesterol. Steroids are characterized by four interlocking carbohydrate rings.
  4. **Eicosanoids**: Are lipids synthesized from the fatty acid chains of phospholipids found in plasma membrane.

Hormones circulating in the blood diffuse into the interstitial fluids surrounding the cell. Cells with specific receptors for a hormone respond with an action that is appropriate for the cell. Because of the specificity of hormone and target cell, the effects produced by a single hormone may vary among different kinds of target cells.

Hormones activate target cells by one of two methods, depending upon the chemical nature of the hormone.

  * **Lipid-soluble** hormones (steroid hormones and hormones of the thyroid gland) diffuse through the cell membranes of target cells. The lipid-soluble hormone then binds to a receptor protein that, in turn, activates a DNA segment that turns on specific genes. The proteins produced as result of the transcription of the genes and subsequent translation of mRNA act as enzymes that regulate specific physiological cell activity.
  * **Water-soluble** hormones (polypeptide, protein, and most amino acid hormones) bind to a receptor protein on the plasma membrane of the cell. The receptor protein, in turn, stimulates the production of one of the following second messengers:

Cyclic AMP (cAMP) is produced when the receptor protein activates another membrane-bound protein called a G protein. The G protein activates adenylate cyclase, the enzyme that catalyzes the production of cAMP from ATP. Cyclic AMP then triggers an enzyme that generates specific cellular changes.

Inositol triphosphate (IP3) is produced from membrane phospholipids. IP3, in turn, triggers the release of CA2+ from the endoplasmic reticulum, which then activates enzymes that generate cellular changes.

Endocrine glands release hormones in response to one or more of the following stimuli:

  1. Hormones from other endocrine glands.
  2. Chemical characteristics of the blood (other than hormones).
  3. Neural stimulation.

Most hormone production is managed by a negative feedback system. The nervous system and certain endocrine tissues monitor various internal conditions of the body. If action is required to maintain homeostasis, hormones are released, either directly by an endocrine gland or indirectly through the action of the hypothalamus of the brain, which stimulates other endocrine glands to release hormones. The hormones activate target cells, which initiate physiological changes that adjust the body conditions. When normal conditions have been recovered, the corrective action - the production of hormones - is discontinued. Thus, in negative feedback, when the original (abnormal) condition has been repaired, or negated, corrective actions decrease or discontinue. For example, the amount of glucose in the blood controls the secretion of insulin and glucagons via negative feedback.

The production of some hormones is controlled by positive feedback. In such a system, hormones cause a condition to intensify, rather than decrease. As the condition intensifies, hormone production increases. Such positive feedback is uncommon, but does occur during childbirth, where hormone levels build with increasingly intense labor contractions. Also in lactation, hormone levels increase in response to nursing, which causes an increase in milk production. The hormone produced by the hypothalamus causing the milk let down and uterine contraction is **oxytocin**.

## Endocrine Glands

### Pituitary gland

The hypothalamus makes up the lower region of the diencephalons and lies just above the brain stem. The pituitary gland (hypophysis) is attached to the bottom of the hypothalamus by a slender stalk called the infundibulum. The pituitary gland consists of two major regions, the anterior pituitary gland (anterior lobe or adenohypophysis) and the posterior pituitary gland (posterior lobe or neurohypophysis). The hypothalamus also controls the glandular secretion of the pituitary gland.

The hypothalamus oversees many internal body conditions. It receives nervous stimuli from receptors throughout the body and monitors chemical and physical characteristics of the blood, including temperature, blood pressure, and nutrient, hormone, and water content. When deviations from homeostasis occur or when certain developmental changes are required, the hypothalamus stimulates cellular activity in various parts of the body by directing the release of hormones from the anterior and posterior pituitary glands. The hypothalamus communicates directives to these glands by one of the following two pathways: The pituitary gland is found in the inferior part of the brain and is connected by the pituitary stalk. It can be referred to as the master gland because it is the main place for everything that happens within the endocrine system. It is divided into two sections: the **anterior** lobe (adenohypophysis) and the **posterior** lobe (neurohypophysis). The Anterior pituitary is involved in sending hormones that control all other hormones of the body.

#### Posterior pituitary

Communication between the hypothalamus and the posterior pituitary occurs through neurosecretory cells that span the short distance between the hypothalamus and the posterior pituitary. Hormones produced by the cell bodies of the neurosecretory cells are packaged in vesicles and transported through the axon and stored in the axon terminals that lie in the posterior pituitary. When the neurosecretory cells are stimulated, the action potential generated triggers the release of the stored hormones from the axon terminals to a capillary network within the posterior pituitary. Two hormones, oxytocin and antidiuretic hormone (ADH), are produced and released this way. Decreased ADH release or decreased renal sensitivity to ADH produces a condition known as diabetes insipidus. Diabetes insipidus is characterized by polyuria (excess urine production), hypernatremia (increased blood sodium content) and polydipsia (thirst). Oxytosin is secreted by paraventricular nucleus and a small quantity is secreted by supraoptic nucleus in hypothalamus.oxytocin is secreted in both males and females.in female acts on mammary glands and uterus.in males facilitates release of sperm in to urethra by causing contraction of vas deferens.

The posterior lobe is composed of neural tissue [neural ectoderm] and is derived from hypothalamus. Its function is to store oxytocin and antidiuretic hormone. When the hypothalamic neurons fire these hormones are release into the capillaries of the posterior lobe.

The posterior pituitary is, in effect, a projection of the hypothalamus. It does not produce its own hormones, but only stores and releases the hormones oxytocin and antidiuretic hormone. ADH is also known as arginine vasopressin (AVP) or simply vasopressin.

#### Anterior pituitary

The anterior lobe is derived from oral ectoderm and is composed of glandular epithelium. Communication between the hypothalamus and the anterior pituitary (adenohypophysis) occurs through hormones (releasing hormones and inhibiting hormones) produced by the hypothalamus and delivered to the anterior pituitary via a portal network of capillaries. It consists of three divisions: 1. pars distalis, 2. pars tuberalis, 3. pars intermedia. The releasing and inhibiting hormones are produced by specialized neurons of the hypothalamus called neurosecretory cells. The hormones are released into a capillary network or primary plexus, and transported through veins or hypophyseal portal veins, to a second capillary network or secondary plexus that supplies the anterior pituitary. The hormones then diffuse from the secondary plexus aunshine into the anterior pituitary, where they initiate the production of specific hormones by the anterior pituitary. Many of the hormones produced by the anterior pituitary are tropic hormones or tropins, which are hormones that stimulate other endocrine glands to secrete their hormones.

The anterior pituitary lobe receives releasing hormones from the hypothalamus via a portal vein system known as the hypothalamic-hypophyseal portal system.

The anterior pituitary secretes:

  * thyroid-stimulating hormone (TSH)
  * adrenocorticotropic hormone (ACH)
  * prolactin
  * follicle-stimulating hormone (FSH)
  * luteinizing hormone (LH)
  * growth hormone (GH)
  * endorphins
  * and other hormones

It does this in response to a variety of chemical signals from the hypothalamus, which travels to the anterior lobe by way of a special capillary system from the hypothalamus, down the median eminence, to the anterior lobe. These include:

  * thyrotropin-releasing hormone (TRH)
  * corticotropin-releasing hormone (CRH)
  * dopamine (DA), also called 'prolactin inhibiting factor' (PIF)
  * gonadotropin-releasing hormone (GnRH)
  * growth hormone releasing hormone (GHRH)

These hormones from the hypothalamus cause release of the respective hormone from the pituitary. The control of release of hormones from the pituitary is via negative feedback from the target gland. For example homeostasis of thyroid hormones is achieved by the following mechanism; TRH from the hypothalamus stimulates the release of TSH from the anterior pituitary. The TSH, in turn, stimulates the release of thyroid hormones form the thyroid gland. The thyroid hormones then cause negative feedback, suppressing the release of TRH and TSH.

The heart, gastrointestinal tract, the placenta, the kidneys and the skin, whose major function is not the secretion of hormones, also contain some specialized cells that produce hormones.

In addition, all cells, except red blood cells secrete a class of hormones called eicosanoids. These hormones are paracrines, or local hormones, that primarily affect neighboring cells. Two groups of eicosanoids, the prostaglandins (PGs) and the leukotrienes (LTs), have a wide range of varying effects that depend upon the nature of the target cell. Eicosanoid activity, for example, may impact blood pressure, blood clotting, immune and inflammatory responses, reproductive processes, and the contraction of smooth muscles.

## Antagonistic Hormones

Maintaining homeostasis often requires conditions to be limited to a narrow range. When conditions exceed the upper limit of homeostasis, specific action, usually the production of a hormone is triggered. When conditions return to normal, hormone production is discontinued. If conditions exceed the lower limits of homeostasis, a different action, usually the production of a second hormone is triggered. Hormones that act to return body conditions to within acceptable limits from opposite extremes are called **antagonistic hormones**. The two glands that are the most responsible for homeostasis is the thyroid and the parathyroid.

The regulation of blood glucose concentration (through negative feedback) illustrates how the endocrine system maintains homeostasis by the action of antagonistic hormones. Bundles of cells in the pancreas called the islets of Langerhans contain two kinds of cells, **alpha cells** and **beta cells**. These cells control blood glucose concentration by producing the antagonistic hormones insulin and glucagon.

Beta cells secrete **insulin**. When the concentration of blood glucose raises such in after eating, beta cells secret insulin into the blood. Insulin stimulates the liver and most other body cells to absorb glucose. Liver and muscle cells convert glucose to glycogen, for short term storage, and adipose cells convert glucose to fat. In response, glucose concentration decreases in the blood, and insulin secretion discontinues through negative feedback from declining levels of glucose.

Alpha cells secrete **glucagon**. When the concentration of blood glucose drops such as during exercise, alpha cells secrete glucagon into the blood. Glucagon stimulates the liver to release glucose. The glucose in the liver originates from the breakdown of glycogen. Glucagon also stimulates the production of ketone bodies from amino acids and fatty acids. Ketone bodies are an alternative energy source to glucose for some tissues. When blood glucose levels return to normal, glucagon secretion discontinues through negative feedback.

Another example of antagonistic hormones occurs in the maintenance of Ca2+ ion concentration in the blood. Parathyroid hormone (PTH) from the parathyroid glands increases Ca2+ in the blood by increasing Ca2+ absorption in the intestines and reabsorption in the kidneys and stimulating Ca2+ release from bones. Calcitonin (CT) produces the opposite effect by inhibiting the breakdown of bone matrix and decreasing the release of calcium in the blood.

### Thyroid gland

The **Thyroid gland** is one of the largest endocrine glands in the body. It is positioned on the neck just below the Larynx and has two lobes with one on either side of the trachea. It is involved in the production of the hormones T3 (triiodothyronine) and T4 (thyroxine). These hormones increase the metabolic activity of the body‘s cells. The thyroid also produces and releases the hormone calcitonin (thyrocalcitonin) which contributes to the regulation of blood calcium levels. Thyrocalcitonin or calcitonin decreases the concentration of calcium in the blood. Most of the calcium removed from the blood is stored in the bones.

The thyroid hormone consists of two components, thyroxine and iodine. This hormone increases the metabolism of most body cells. A deficiency of iodine in the diet leads to the enlargement of the thyroid gland, known as a simple goiter. Hypothyroidism during early development leads to cretinism. In adults, it produces myxedema, characterized by obesity and lethargy. Hyperthyroidism leads to a condition known as exophthalmic goiter, characterized by weight loss as well as hyperactive and irritable behavior.

The thyroid gland is a two-lobed gland that manifests a remarkably powerful active transport mechanism for up-taking iodide ions from the blood. As blood flows through the gland, iodide is converted to an active form of iodine. This iodine combines with an amino acid called tyrosine. Two molecules of iodinated tyrosine then combine to form thryroxine. Following its formation, the thyroxine becomes bound to a polysaccharide-protein material called thyroglobulin. The normal thyroid gland may store several weeks supply of thyroxine in this bound form. An enzymatic splitting of the thyroxine from the thyroglobulin occurs when a specific hormone is released into the blood. This hormone, produced by the pituitary gland, is known as thyroid-stimulating hormone (TSH). TSH stimulates certain major rate-limiting steps in thyroxine secretion, and thereby alters its rate of release. A variety of bodily defects, either dietary, hereditary, or disease induced, may decrease the amount of thyroxine released into the blood. The most popular of these defects is one that results from dietary iodine deficiency. The thyroid gland enlarges, in the continued presence of TSH from the pituitary, to form a goiter. This is a futile attempt to synthesize thyroid hormones, for iodine levels that are too low. Normally, thyroid hormones act via a negative feedback loop on the pituitary to decrease stimulation of the thyroid. In goiter, the feedback loop cannot be in operation - hence continual stimulation of the thyroid and the inevitable protuberance on the neck. Formerly, the principal source of iodine came from seafood. As a result, goiter was prevalent amongst inland areas far removed from the sea. Today, the incidence of goiter has been drastically reduced by adding iodine to table salt.

Thyroxine serves to stimulate oxidative metabolism in cells; it increases the oxygen consumption and heat production of most body tissues, a notable exception being the brain. Thyroxine is also necessary for normal growth. The most likely explanation being that thyroxine promotes the effects of growth hormone on protein synthesis. The absence of thyroxine significantly reduces the ability of growth hormone to stimulate amino acid uptake and RNA synthesis. Thyroxine also plays a crucial role in the closely related area of organ development, particularly that of the central nervous system.

If there is an insufficient amount of thyroxine, a condition referred to as hypothyroidism results. Symptoms of hypothyroidism stem from the fact that there is a reduction in the rate of oxidative energy-releasing reactions within the body cells. Usually the patient shows puffy skin, sluggishness, and lowered vitality. Other symptoms of hypothyroidism include weight gain, decreased libido, inability to tolerate cold, muscle pain and spasm, and brittle nails. Hypothyroidism in children, a condition known as cretinism, can result in mental retardation, dwarfism, and permanent sexual immaturity. Sometimes the thyroid gland produces too much thyroxine, a condition known as hyperthyroidism. This condition produces symptoms such as an abnormally high body temperature, profuse sweating, high blood pressure, loss of weight, irritability, insomnia and muscular pain and weakness. It also causes the characteristic symptom of the eyeballs protruding from the skull called exopthalmia. This is surprising because it is not a symptom usually related to a fast metabolism. Hyperthyroidism has been treated by partial removal or by partial radiation destruction of the gland. More recently, several drugs that inhibit thyroid activity have been discovered, and their use is replacing the former surgical procedures. Unfortunately thyroid conditions require lifetime treatment and because of the body's need for a sensitive balance of thyroid hormone both supplementing and suppressing thyroid function can take months or even years to regulate.

  


#### T3 and T4 Function within the body

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/f/fe/Amphibian_Metamorphosis.PNG/220px-Amphibian_Metamorphosis.PNG)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Iodine and T4 stimulate the spectacular apoptosis (programmed cell death) of the cells of the larval gills, tail and fins Transforming the aquatic, vegetarian tadpole into the terrestrial, carnivorous frog with better neurological, visuospatial, olfactory and cognitive abilities for hunting. Contrary to amphibian metamorphosis, thyroidectomy and hypothyroidism in mammals may be considered a sort of phylogenetic and metabolic regression to a former stage of reptilian life. Indeed, many disorders that seem to afflict hypothyroid humans have reptilian-like features, such as dry, hairless, scaly, cold skin and a general slowdown of metabolism, digestion, heart rate and nervous reflexes, with lethargic cerebration, hyperuricemia and hypothermia ( Venturi, 2000).

The Production of T3 and T4 are regulated by thyroid stimulating hormone (TSH), released by the pituitary gland. TSH Production is increased when T3 and T4 levels are too low. The thyroid hormones are released throughout the body to direct the body's metabolism. They stimulate all cells within the body to work at a better metabolic rate. Without these hormones the body's cells would not be able to regulate the speed at which they performed chemical actions. Their release will be increased under certain situations such as cold temperatures when a higher metabolism is needed to generate heat. When children are born with thyroid hormone deficiency they have problems with physical growth and developmental problems. Brain development can also be severely impaired

#### The significance of iodine

Thyroid hormone cannot be produced without an abundant source of iodine. The iodine concentration within the body, although significant, can be as little as 1/25th the concentration within the thyroid itself. When the thyroid is low on iodine the body will try harder to produce T3 and T4 which will often result in a swelling of the thyroid gland, resulting in a goiter.

## Extrathyroidal iodine

![](../../../upload.wikimedia.org/wikipedia/commons/8/83/Sequence_of_123-iodide_total_body_human_scintiscans.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Sequence of 123-iodide human scintiscans after an intravenous injection, (from left) after 30 minutes, 20 hours, and 48 hours. A high and rapid concentration of radio-iodide is evident in the periencephalic and cerebrospinal fluid (left), salivary glands, oral mucosa and the stomach. In the thyroid gland, I-concentration is more progressive, also in the reservoir (from 1% after 30 minutes, to 5.8 % after 48 hours, of the total injected dose. Highest iodide-concentration by the mammary gland is evident only in pregnancy and lactation. High excretion of radio-iodide is observed in the urine.[[1]](Print_Version.html#cite_note-2)

Iodine accounts for 65% of the molecular weight of T4 and 59% of the T3. 15–20 mg of iodine is concentrated in thyroid tissue and hormones, but 70% of the body's iodine is distributed in other tissues, including mammary glands, eyes, gastric mucosa, the cervix, and salivary glands. In the cells of these tissues iodide enters directly by [sodium-iodide symporter](http://en.wikibooks.org/w/index.php?title=Sodium-iodide_symporter&action=edit&redlink=1) (NIS). Its role in mammary tissue is related to fetal and neonatal development, but its role in the other tissues is unknown. It has been shown to act as an antioxidant in these tissues.

The US Food and Nutrition Board and Institute of Medicine recommended daily allowance of iodine ranges from 150 micrograms /day for adult humans to 290 micrograms /day for lactating mothers. However, the thyroid gland needs no more than 70 micrograms /day to synthesize the requisite daily amounts of T4 and T3. These higher recommended daily allowance levels of iodine seem necessary for optimal function of a number of body systems, including lactating breast, gastric mucosa, salivary glands, oral mucosa, thymus, epidermis, choroid plexus, etc.[[2]](Print_Version.html#cite_note-3)[[3]](Print_Version.html#cite_note-4)[[4]](Print_Version.html#cite_note-5) Moreover, iodine can add to double bonds of docosahexaenoic acid and arachidonic acid of cellular membranes, making them less reactive to free oxygen radicals.[[5]](Print_Version.html#cite_note-6)

#### Calcitonin

Calcitonin is a 32 amino acid polypeptide hormone. It is an additional hormone produced by the thyroid, and contributes to the regulation of blood calcium levels. Thyroid cells produce calcitonin in response to high calcium levels in the blood. This hormone will stimulate movement of calcium into the bone structure. It can also be used therapeutically for the treatment of hypercalcemia or osteoporosis. Without this hormone calcium will stay within the blood instead of moving into bones to keep them strong and growing. Its importance in humans has not been as well established as its importance in other animals.

### Parathyroid gland

There are four parathyroid glands. They are small, light-colored lumps that stick out from the surface of the thyroid gland. All four glands are located on the thyroid gland. They are butterfly-shaped and located inside the neck, more specifically on both sides of the windpipe. One of the parathyroid glands most important functions is to regulate the body's calcium and phosphorus levels. Another function of the parathyroid glands is to secrete parathyroid hormone, which causes the release of the calcium present in bone to extracellular fluid. PTH does this by depressing the production of osteoblasts, special cells of the body involved in the production of bone and activating osteoclasts, other specialized cells involved in the removal of bone.

There are two major types of cells that make up parathyroid tissue:

  * One of the major cells is called **oxyphil cells**. Their function is basically unknown.
  * The second type are called **chief cells**. Chief cells produce parathyroid hormone.

The structure of a parathyroid gland is very different from that of a thyroid gland. The chief cells that produce parathyroid hormone are arranged in tightly-packed nests around small blood vessels, quite unlike the thyroid cells that produce thyroid hormones, which are arranged in spheres called the thyroid follicles.

PTH or **Parathyroid Hormone** is secreted from these four glands. It is released directly into the bloodstream and travels to its target cells which are often quite far away. It then binds to a structure called a receptor, that is found either inside or on the surface of the target cells.

Receptors bind a specific hormone and the result is a specific physiologic response, meaning a normal response of the body.

PTH finds its major target cells in bone, kidneys, and the gastrointestinal system.

Calcitonin, a hormone produced by the thyroid gland that also regulates ECF calcium levels and serves to counteract the calcium-producing effects of PTH.

The adult body contains as much as 1 kg of calcium. Most of this calcium is found in bone and teeth.

The four parathyroid glands secrete the parathyroid hormone (PTH). It opposes the effect of thyrocalcitonin. It does this by removing calcium from its storage sites in bones, releasing it into the bloodstream. It also signals the kidneys to reabsorb more of this mineral, transporting it into the blood. It also signals the small intestine to absorb more of this mineral, transporting it from the diet into the blood.

Calcium is important for steps of body metabolism. Blood cannot clot without sufficient calcium. Skeletal muscles require this mineral in order to contract. A deficiency of PTH can lead to tetany, muscle weakness due to lack of available calcium in the blood.

The parathyroid glands were long thought to be part of the thyroid or to be functionally associated with it. We now know that their close proximity to the thyroid is misleading: both developmentally and functionally, they are totally distinct from the thyroid.

The parathyroid hormone, called parathormone, regulates the calcium-phosphate balance between the blood and other tissues. Production of this hormone is directly controlled by the calcium concentration of the extracellular fluid bathing the cells of these glands. Parathormone exerts at least the following five effects: (1) it increases gastrointestinal absorption of calcium by stimulating the active transport system and moves calcium from the gut lumen into the blood; (2) it increases the movement of calcium and phosphate from bone into extracellular fluid. This is accomplished by stimulating osteoclasts to break down bone structure, thus liberating calcium phosphate into the blood. In this way, the store of calcium contained in bone is tapped; (3) it increases re-absorption of calcium by the renal tubules, thereby decreasing urinary calcium excretion; (4) it reduces the re-absorption of phosphate by the renal tubules (5)it stimulates the synthesis of 1,25-dihydrixycholecalciferol by the kidney.

The first three effects result in a higher extracellular calcium concentration. The adaptive value of the fourth is to prevent the formation of kidney stones.

If parathyroid glands are removed accidentally during surgery on the thyroid, there would be a rise in the phosphate concentration in the blood. There would also be a drop in the calcium concentration as more calcium is excreted by the kidneys and intestines, and more incorporated into the bone. This can produce serious disturbances, particularly in the muscles and nerves, which use calcium ions for normal functioning. Over activity of the parathyroid glands, which can result from a tumor on the glands, produces a weakening of the bones. This is a condition that makes them much more vulnerable to fracturing because of excessive withdrawal of calcium from the bones.

### Adrenal glands

**Adrenal glands** are a pair of ductless glands located above the kidneys. Through hormonal secretions, the adrenal glands regulate many essential functions in the body, including biochemical balances that influence athletic training and general stress response. The glucocorticoids include corticosterone, cortisone, and hydrocortisone or cortisol. These hormones serve to stimulate the conversion of amino acids into carbohydrates which is a process known as gluconeogenesis, and the formation of glycogen by the liver. They also stimulate the formation of reserve glycogen in the tissues, such as in the muscles. The glucocorticoids also participate in lipid and protein metabolism. The cortex of the adrenal gland is known to produce over 20 hormones, but their study can be simplified by classifying them into three categories: glucocorticoids, mineralocorticoids, and sex hormones.

They are triangular-shaped glands located on top of the kidneys. They produce hormones such as estrogen, progesterone, steroids, cortisol, and cortisone, and chemicals such as adrenalin (epinephrine), norepinephrine, and dopamine. When the glands produce more or less hormones than required by the body, disease conditions may occur.

The adrenal cortex secretes at least two families of hormones, the **glucocorticoids** and **mineral corticoids**. The **adrenal medulla** secretes the hormones **epinephrine** (adrenalin) and **norepinephrine** (noradrenalin).

**Adrenal Cortex:** The hormones made by the Adrenal Cortex supply long-term responses to stress. The two major hormones produced are the **Mineral Corticoids** and the **Glucocorticoids.** The Mineral Corticoids regulate the salt and water balance, leading to the increase of blood volume and blood pressure. The **Glucocorticoids** are monitoring the ACTH, in turn regulating carbohydrates, proteins, and fat metabolism. This causes an increase in blood glucose. Glucocorticoids also reduce the body's inflammatory response.

**Cortisol** is one of the most active glucocorticoids. It usually reduces the effects of inflammation or swelling throughout the body. It also stimulates the production of glucose from fats and proteins, which is a process referred to as **gluconeogenesis**.

**Aldosterone** is one example of a mineralocorticoid. It signals the tubules in the kidney nephrons to reabsorb sodium while secreting or eliminating potassium. If sodium levels are low in the blood, the kidney secretes more **renin**, which is an enzyme that stimulates the formation of **angiotensin** from a molecule made from the liver. Angiotensin stimulates aldosterone secretion. As a result, more sodium is reabsorbed as it enters the blood.

Aldosterone, the major mineralocorticoid, stimulates the cells of the distal convoluted tubules of the kidneys to decrease re-absorption of potassium and increase re-absorption of sodium. This in turn leads to an increased re-absorption of chloride and water. These hormones, together with such hormones as insulin and glucagon, are important regulators of the ionic environment of the internal fluid.

The renin-angiotensin-aldosterone mechanism can raise blood pressure if it tends to drop. It does this in two ways. Angiotensin is a vasoconstrictor, decreasing the diameter of blood vessels. As vessels constrict, blood pressure increases. In addition, as sodium is reabsorbed, the blood passing through the kidney becomes more hypertonic. Water follows the sodium into the hypertonic blood by osmosis. This increases the amount of volume in the blood and also increases the blood pressure.

**Adrenal Medulla** The hypothalamus starts nerve impulses that travel the path from the bloodstream, spinal cord, and sympathetic nerve fibers to the Adrenal Medulla, which then releases hormones. The effects of these hormones provide a short-term response to stress

Excessive secretion of the glucocorticoids causes **Cushing's syndrome**, characterized by muscle atrophy or degeneration and hypertension or high blood pressure. Under secretion of these substances produces **Addison's disease**, characterized by low blood pressure and stress.

Epinephrine and norepinephrine produce the "fight or flight" response, similar to the effect from the sympathetic nervous system. Therefore, they increase heart rate, breathing rate, blood flow to most skeletal muscles, and the concentration of glucose in the blood. They decrease blood flow to the digestive organs and diminish most digestive processes.

  


![](../../../upload.wikimedia.org/wikipedia/commons/3/3f/Gray1183.png)

Suprarenal glands viewed from the front.

  


![](../../../upload.wikimedia.org/wikipedia/commons/f/f5/Gray1184.png)

Suprarenal glands viewed from behind.

The adrenal sex hormones consist mainly of male sex hormones (androgens) and lesser amounts of female sex hormones (estrogens and progesterone). Normally, the sex hormones released from the adrenal cortex are insignificant due to the low concentration of secretion. However, in cases of excess secretion, masculine or feminine effects appear. The most common syndrome of this sort is "virilism" of the female.

Should there be an insufficient supply of cortical hormones, a condition known as Addison's disease would result. This disease is characterized by an excessive excretion of sodium ions, and hence water, due to lack of mineralocorticoids. Accompanying this is a decreased blood glucose level due to a deficient supply of glucocorticoids. The effect of a decreased androgen supply cannot be observed immediately. Injections of adrenal cortical hormones promptly relieve these symptoms.

Hormonal production in the adrenal cortex is directly controlled by the anterior pituitary hormone called adrenocorticotropic hormone (ACTH).

The two adrenal glands lie very close to the kidneys. Each adrenal gland is actually a double gland, composed of an inner core like medulla and an outer cortex. Each of these is functionally unrelated.

The adrenal medulla secretes two hormone, adrenalin or epinephrine and noradrenalin or norepinephrine, whose functions are very similar but not identical. The adrenal medulla is derived embryologically from neural tissue. It has been likened to an overgrown sympathetic ganglion whose cell bodies do not send out nerve fibers, but release their active substances directly into the blood, thereby fulfilling the criteria for an endocrine gland. In controlling epinephrine secretion, the adrenal medulla behaves just like any sympathetic ganglion, and is dependent upon stimulation by sympathetic preganglionic fibers.

Epinephrine promotes several responses, all of which are helpful in coping with emergencies: the blood pressure rises, the heart rate increases, the glucose content of the blood rises because of glycogen breakdown, the spleen contracts and squeezes out a reserve supply of blood, the clotting time decreases, the pupils dilate, the blood flow to skeletal muscles increase, the blood supply to intestinal smooth muscle decreases and hairs become erect. These adrenal functions, which mobilize the resources of the body in emergencies, have been called the fight-or-flight response. Norepinephrine stimulates reactions similar to those produced by epinephrine, but is less effective in conversion of glycogen to glucose.

The significance of the adrenal medulla may seem questionable since the complete removal of the gland causes few noticeable changes; humans can still exhibit the flight-or-fight response. This occurs because the sympathetic nervous system complements the adrenal medulla in stimulating the fight-or-flight response, and the absence of the hormonal control will be compensated for by the nervous system.

### Pancreas

The **pancreas** is very important organ in the digestion system and the circulatory system because it helps to maintain our blood sugar levels. The pancreas is considered to be part of the gastrointestinal system. It produces digestive enzymes to be released into the small intestine to aid in reducing food particles to basic elements that can be absorbed by the intestine and used by the body. It has another very different function in that it forms insulin, glucagon and other hormones to be sent into the bloodstream to regulate blood sugar levels and other activities throughout the body.

  


It has a pear-shape to it and is approximately 6 inches long. It is located in the middle and back portion of the abdomen. The pancreas is connected to the first part of the small intestine, the duodenum, and lies behind the stomach. The pancreas is made up of glandular tissue: any substance secreted by the cells of the pancreas will be secreted outside of the organ.

The digestive juices produced by the pancreas are secreted into the duodenum via a Y-shaped duct, at the point where the common bile duct from the liver and the pancreatic duct join just before entering the duodenum. The digestive enzymes carried into the duodenum are representative of the exocrine function of the pancreas, in which specific substances are made to be passed directly into another organ.

**Note:**  
The pancreas is both an exocrine and an endocrine organ.

The pancreas is unusual among the body's glands in that it also has a very important endocrine function. Small groups of special cells called **islet cells** throughout the organ make the hormones of insulin and glucagon. These, of course, are hormones that are critical in regulating blood sugar levels. These hormones are secreted directly into the bloodstream to affect organs all over the body.

No organ except the pancreas makes significant amounts of insulin or glucagon.

**Insulin** acts to lower blood sugar levels by allowing the sugar to flow into cells. Glucagon acts to raise blood sugar levels by causing glucose to be released into the circulation from its storage sites. Insulin and glucagon act in an opposite but balanced fashion to keep blood sugar levels stable.

A healthy working pancreas in the human body is important for maintaining good health by preventing malnutrition, and maintaining normal levels of blood sugar. The digestive tract needs the help of the enzymes produced by the pancreas to reduce food particles to their simplest elements, or the nutrients cannot be absorbed. Carbohydrates must be broken down into individual sugar molecules. Proteins must be reduced to simple amino acids. Fats must be broken down into fatty acids. The pancreatic enzymes are important in all these transformations. The basic particles can then easily be transported into the cells that line the intestine, and from there they can be further altered and transported to different tissues in the body as fuel sources and construction materials. Similarly, the body cannot maintain normal blood sugar levels without the balanced action of insulin and glucagon.

The pancreas contains exocrine and endocrine cells. Groups of endocrine cells, the **islets of Langerhans**, secrete two hormones. The beta cells secrete **insulin**; the alpha cells secrete **glucagon**. The level of sugar in the blood depends on the opposing action of these two hormones.

Insulin decreases the concentration of glucose in the blood. Most of the glucose enters the cells of the liver and skeletal muscles. In these cells, this **monosaccharide** is converted to the polysaccharide glycogen. Therefore, insulin promotes **glycogenesis** or glycogen synthesis, in which glucose molecules are added to chains of glycogen. Excess glucose is also stored as fat in adipose tissue cells in response to insulin.

Insulin deficiency leads to the development of **diabetes mellitus**, specifically **type I**, juvenile diabetes. As the pancreas does not produce sufficient insulin, it is treated by insulin injections. In **type II** or maturity onset diabetes, the pancreas does produce enough insulin, but the target cells do not respond to it.

As already stated, the pancreas is a mixed gland having both endocrine and exocrine functions. The exocrine portion secretes digestive enzymes into the duodenum via the pancreatic duct. The endocrine portion secretes two hormones, insulin and glucagon, into the blood.

Insulin is a hormone that acts directly or indirectly on most tissues of the body, with the exception of the brain. The most important action of insulin is the stimulation of the uptake of glucose by many tissues, particularly the liver, muscle and fat. The uptake of glucose by the cells decreases blood glucose and increases the availability of glucose for the cellular reactions in which glucose participates. Thus, glucose oxidation, fat synthesis, and glycogen synthesis are all accentuated by an uptake of glucose. It is important to note that insulin does not alter glucose uptake by the brain, nor does it influence the active transport of glucose across the renal tubules and gastrointestinal epithelium.

As stated, insulin stimulates glycogen synthesis. In addition, it also increases the activity of the enzyme that catalyzes the rate-limiting step in glycogen synthesis. Insulin also increases triglyceride levels by inhibiting triglyceride breakdown, and by stimulating production of triglyceride through fatty acid and glycerophosphate synthesis. The net protein synthesis is also increased by insulin, which stimulates the active membrane transport of amino acids, particularly into muscle cells. Insulin also has effects on other liver enzymes, but the precise mechanisms by which insulin induces these changes are not well understood.

Insulin is secreted by beta cells, which are located in the part of the pancreas known as the islets of Langerhans. These groups of cells, which are located randomly throughout the pancreas, also consist of other secretory cells called alpha cells. It is these alpha cells that secrete glucagon. Glucagon is a hormone that has the following major effects: it increases hepatic synthesis of glucose from pyruvate, lactate, glycerol, and amino acids (a process called gluconeogenesis, which also raises the plasma glucose level); and it increases the breakdown of adipose tissue triglyceride, thereby raising the plasma levels of fatty acids and glycerol. The glucagon secreting alpha cells in the pancreas, like the beta cells, respond to changes in the concentration of glucose in the blood flowing through the pancreas; no other nerves or hormones are involved.

It should be noted that glucagon has the opposite effects of insulin. Glucagon elevates the plasma glucose, whereas insulin stimulates its uptake and thereby reduces plasma glucose levels; glucagon elevates fatty acid concentrations, whereas insulin converts fatty acids and glycerol into triglycerides, thereby inhibiting triglyceride breakdown.

The alpha and beta cells of the pancreas make up a push-pull system for regulating the plasma glucose level.

### Sex organs

The Sex organs (Gonads) are the testes in the male, and the ovaries in the female. Both of these organs produce and secrete hormones that are balanced by the hypothalamus and pituitary glands.

The main hormones from the reproductive organs are:

**Testosterone** is more prominent in males. It belongs to the family of androgens, which are steroid hormones producing masculine effects. Testosterone stimulates the development and functioning of the primary sex organs. It also stimulates the development and maintenance of secondary male characteristics, such as hair growth on the face and the deep pitch of the voice.

**Estrogen** In females, this hormone stimulates the development of the uterus and vagina. It is also responsible for the development and maintenance of secondary female characteristics, such as fat distribution throughout the body and the width of the pelvis.

#### Male

![Prostate.gif](../../../upload.wikimedia.org/wikipedia/commons/f/f1/Prostate.gif)

The **testes** produce **androgens** (i.e., "testosterone"). **Testosterone** is classified as a steroid and is responsible for many of the physical characteristics in males like.

  * Broad shoulders
  * Muscular body
  * Hair

Testosterone increases protein production. Hormones that build up protein are called **anabolic steroids**. Anabolic steroids are available commercially and are being used by athletes because they help improve their physical ability, however, they do have major side effects such as:

  * Liver and kidney disorders
  * Hypertension (high blood pressure)
  * Decreased sperm count and impotency
  * Aggressive behavior ("roid rage")
  * Balding
  * Acne

  


#### Female

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/6/68/Scheme_female_reproductive_system-en.svg/240px-Scheme_female_reproductive_system-en.svg.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Schematic frontal view of female anatomy.

The **ovaries** produce **estrogen** and **progesterone**. Estrogen increases at the time of puberty and causes the growth of the uterus and vagina. Without estrogen egg maturation would not occur. Estrogen is also responsible for secondary sex characteristics such as female body hair and fat distribution. Estrogen and Progesterone are responsible for the development of the breast and for the uterine cycle. Progesterone is a female hormone secreted by the corpus luteum after ovulation during the second half of the menstrual cycle. It prepares the lining of the uterus for implantation of a fertilized egg and allows for complete shedding of the endometrium at the time of menstruation. In the event of pregnancy, the progesterone level remains stable beginning a week or so after conception.

  


### Pineal gland

The pineal gland (also called the pineal body or epiphysis) is a small endocrine gland in the brain. It is located near the center of the brain, between the two hemispheres, tucked in a groove where the two rounded thalamic bodies join.it consists of two types of cells 1.parenchymal cells 2.neuroglial cells.

  
The pineal gland is a reddish-gray body about the size of a pea (8 mm in humans) located just rostro-dorsal to the superior colliculus and behind and beneath the stria medullaris, between the laterally positioned thalamic bodies. It is part of the epithalamus.

The pineal gland is a midline structure, and is often seen in plain skull X-rays, as it is often calcified. The main hormone produced and secreted by the pineal gland is melatonin. Secretion is highest at night and between the ages of 0-5.melatonin acts mainly on gonads.

  


## Glossary

**Adrenal Gland:** endocrine gland that is located on top of each kidney

**Amino Acid-derived:** hormones that are modified amino acids

**Antagonistic Hormones:** hormones that act to return body conditions to within acceptable limits from opposite extremes

**Calcitonin:** hormone produced by the thyroid; contributes to the regulation of blood calcium levels

**Eicosanoids:** lipids that are synthesized from the fatty acid chains of phospholipids found in plasma membrane

**Endocrine Glands:** glands that have no duct and release their secretions directly into the intercellular fluid or into the blood

**Endocrine System:** a control system of ductless glands that secrete chemical messengers called hormones

**Estrogen:** hormone in females; stimulates the development of the uterus and vagina

**Exocrine Glands:** glands that release their cellular secretions through a duct which empties to the outside or into the lumen (empty internal space) of an organ

**Hormone:** a specific chemical substance produced by certain cells that control, or help to control, cellular processes elsewhere in an organism

**Insulin:** hormone that acts to lower blood sugar levels by allowing the sugar to flow into cells

**Iodine:** chemical in the body; Thyroid hormone can not be produced with out it

**Lipid-soluble Hormones:** diffuse through the cell membranes of target cells

**Parathyroid:** four masses of tissue, two embedded posterior in each lateral mass of the thyroid gland

**Pancreas:** organ involved with the digestion system and the circulatory system; helps to maintain blood sugar levels

**Pineal Gland:** small endocrine gland in the brain located near the center of the brain, between the two hemispheres, tucked in a groove where the two rounded thalamic bodies join

**Pituitary Gland:** endocrine gland that is attached to the hypothalamus of the lower forebrain

**Polypeptide and Proteins:** hormones that are chains of amino acids of less than or more than about 100 amino acids

**Steroids:** hormones that are lipids that are synthesized from cholesterol; characterized by four interlocking carbohydrate rings

**Testosterone:** hormone more prominent in males; belongs to the family of androgens, which are steroid hormones producing masculinizing effects

**Thyroid Gland:** endocrine gland that consists of two lateral masses that are attached to the trachea

**Thyroxine:** serves to stimulate oxidative metabolism in cells; increases the oxygen consumption and heat production of most body tissues

**Water-soluble Hormones:** bind to a receptor protein on the plasma membrane of the cell

## Chapter Review Questions

Answers for these questions can be found [here](http://en.wikibooks.org/wiki/Human_Physiology/Appendix_1:_answers_to_review_questions#The_Endocrine_System)

1\. My child just fell and was hurt, the anxious feeling that I feel is caused by:

    A) glucagon
    B) insulin
    C) epinephrine
    D) adrenocorticotropic
    E) None of these

2\. All of Bob’s life he has had to take insulin shots, this is caused because

    A) his beta cells don’t function correctly
    B) his alpha cells don’t function correctly
    C) his DA hormone isn’t functioning correctly
    D) his GHRH hormone isn’t functioning correctly

3\. The reason iodine is in salt is

    A) to prevent diabetes
    B) to prevent simple goiters
    C) to prevent Addison’s disease
    D) to prevent Cushing's syndrome

4\. All hormones react to a negative feedback except

    A) progesterone
    B) estrogen
    C) prolactin
    D) oxytocin
    E) none of these

5\. If I have a high blood calcium level it may be due to

    A) calcitonin
    B) parathyroid
    C) glucocorticoids
    D) glucagon

6\. Hormones that are lipids that are synthesized from cholesterol

    A) protein
    B) amino acid-derived
    C) polypeptide
    D) steroids
    E) eicosanoids

7\. This type of hormone must bind to a receptor protein on the plasma membrane of the cell

    A) water soluble
    B) lipid soluble
    C) steroid
    D) polypeptide
    E) a and d
    F) b and c

8\. Endocrine glands release hormones in response to

    A) Hormones from other endocrine glands
    B) Chemical characteristics of the blood
    C) Neural stimulation
    D) All of the above

9\. The anterior pituitary secretes

    A) oxytocin
    B) endorphins
    C) ADH
    D) TRH

10\. Chief cells produce

    A) epinephrine
    B) glucagon
    C) insulin
    D) mineralocorticoids
    E) parathyroid hormone

11\. Name the eight major endocrine glands.

12\. Name the four major groups hormones can be chemically classified into.

## References

  1. [↑](Print_Version.html#cite_ref-Totora_1-0) Tortora, G. & Grabowski, S. (2000)Principles of anatomy and physiology. Ninth Edition. Wiley page 733.

  


# The Male Reproductive System

## Introduction

In simple terms, reproduction is the process by which organisms create descendants. This miracle is a characteristic that all living things have in common and sets them apart from nonliving things. But even though the reproductive system is essential to keeping a species alive, it is not essential to keeping an individual alive.

In human reproduction, two kinds of sex cells or gametes are involved. Sperm, the male gamete, and a secondary oocyte (along with first polar body and corona radiata), the female gamete must meet in the female reproductive system to create a new individual. For reproduction to occur, both the female and male reproductive systems are essential. It is a common misnomer to refer to a woman's gametic cell as an egg or ovum, but this is impossible. A secondary oocyte must be fertilized by the male gamete before it becomes an "ovum" or "egg".

While both the female and male reproductive systems are involved with producing, nourishing and transporting either the oocyte or sperm, they are different in shape and structure. The male has reproductive organs, or genitals, that are both inside and outside the pelvis, while the female has reproductive organs entirely within the pelvis.

The male reproductive system consists of the testes and a series of ducts and glands. Sperm are produced in the testes and are transported through the reproductive ducts. These ducts include the epididymis, ductus deferens, ejaculatory duct and urethra. The reproductive glands produce secretions that become part of semen, the fluid that is ejaculated from the urethra. These glands include the seminal vesicles, prostate gland, and bulbourethral glands.

## Structure

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/3/35/Male_anatomy.png/440px-Male_anatomy.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

The human male reproductive system

### Testes

The testes (singular, testis) are located in the scrotum (a sac of skin between the upper thighs). In the male fetus, the testes develop near the kidneys, then descend into the scrotum just before birth. Each testis is about 1 1/2 inches long by 1 inch wide. Testosterone is produced in the testes which stimulates the production of sperm as well as give secondary sex characteristics beginning at puberty.

**Scrotum**

The two testicles are each held in a fleshy sac called the scrotum. The major function of the scrotal sac is to keep the testes cooler than thirty-seven degrees Celsius (ninety-eight point six degrees Fahrenheit). The external appearance of the scrotum varies at different times in the same individual depending upon temperature and the subsequent contraction or relaxation of two muscles. These two muscles contract involuntarily when it is cold to move the testes closer to the heat of the body in the pelvic region. This causes the scrotum to appear tightly wrinkled. On the contrary, they relax in warm temperatures causing the testes to lower and the scrotum to become flaccid. The temperature of the testes is maintained at about thirty-five degrees Celsius (ninety-five degrees Fahrenheit), which is below normal body temperature. Temperature has to be lower than normal in order for _spermatogenesis_ (sperm production) to take place.

  
.

The two muscles that regulate the temperature of the testes are the dartos and cremaster muscles:

  * _Dartos Muscle_

The dartos muscle is a layer of smooth muscle fibers in the subcutaneous tissue of the scrotum (surrounding the scrotum). This muscle is responsible for wrinkling up the scrotum, in conditions of cold weather, in order to maintain the correct temperature for spermatogenesis.

  * _Cremaster Muscle_

The cremaster muscle is a thin strand of skeletal muscle associated with the testes and spermatic cord. This muscle is a continuation of the internal oblique muscle of the abdominal wall, from which it is derived. **Seminiferous Tubules**

Each testis contains over 100 yards of tightly packed seminiferous tubules. Around 90% of the weight of each testes consists of seminiferous tubules. The seminiferous tubules are the functional units of the testis, where spermatogenesis takes place. Once the sperm are produced, they moved from the seminiferous tubules into the rete testis for further maturation.

**Interstitial Cells (Cells of Leydig)**

In between the seminiferous tubules within the testes, are instititial cells, or, _Cells of Leydig_. They are responsible for secreting the male sex hormones (i.e., testosterone).

**Sertoli Cells**

A Sertoli cell (a kind of sustentacular cell) is a 'nurse' cell of the testes which is part of a seminiferous tubule.

It is activated by follicle-stimulating hormone, and has FSH-receptor on its membranes.

Its main function is to nurture the developing sperm cells through the stages of spermatogenesis. Because of this, it has also been called the "mother cell." It provides both secretory and structural support.

  


Other functions During the Maturation phase of spermiogenesis, the Sertoli cells consume the unneeded portions of the spermatazoa.

**Efferent ductules**

The sperm are transported out of the testis and into the epididymis through a series of efferent ductules.

**Blood Supply**

The testes receive blood through the testicular arteries (gonadal artery). Venous blood is drained by the testicular veins. The right testicular vein drains directly into the inferior vena cava. The left testicular vein drains into the left renal vein.

### Epididymis

The seminiferous tubules join together to become the epididymis. The epididymis is a tube that is about 2 inches that is coiled on the posterior surface of each testis. Within the epididymis the sperm complete their maturation and their flagella become functional. This is also a site to store sperm, nourishing them until the next ejaculation. Smooth muscle in the wall of the epididymis propels the sperm into the ductus deferens. Vasa efferentia from the rete testis open into the epididymis which is a highly coiled tubule. The epididymis has three parts- 1)head or caput epididymis- it is the proximal part of the epididymis. It caries the sperms from the testis. 2)body or corpus epididymis- it the highly convoluted middle part of the epididymis 3)tail or cauda epididymis- it is the last part that takes part in carrying the sperms to the vas deferens. The cauda epididymis continues to form less convoluted vas deferens.

### Ductus Deferens

The ductus (vas) deferens, also called sperm duct, or, spermatic deferens, extends from the epididymis in the scrotum on its own side into the abdominal cavity through the inguinal canal. The inguinal canal is an opening in the abdominal wall for the spermatic cord (a connective tissue sheath that contains the ductus deferens, testicular blood vessels, and nerves. The smooth muscle layer of the ductus deferens contracts in waves of peristalsis during ejaculation.

### Seminal Vesicles

The pair of seminal vesicles are posterior to the urinary bladder. They secrete fructose to provide an energy source for sperm and alkalinity to enhance sperm mobility. The duct of each seminal vesicle joins the ductus deferens on that side to form the ejaculatory duct.

### Ejaculatory Ducts

There are two ejaculatory ducts. Each receives sperm from the ductus deferens and the secretions of the seminal vesicle on its own side. Both ejaculatory ducts empty into the single urethra.

### Prostate Gland

The prostate gland is a muscular gland that surrounds the first inch of the urethra as it emerges from the bladder. The smooth muscle of the prostate gland contracts during ejaculation to contribute to the expulsion of semen from the urethra.

### Bulbourethral Glands

The bulbourethral glands also called Cowper's glands are located below the prostate gland and empty into the urethra. The alkalinity of seminal fluid helps neutralize the acidic vaginal pH and permits sperm mobility in what might otherwise be an unfavorable environment.

### Penis

The penis is an external genital organ. The distal end of the penis is called the glans penis and is covered with a fold of skin called the prepuce or foreskin. Within the penis are masses of erectile tissue. Each consists of a framework of smooth muscle and connective tissue that contains blood sinuses, which are large, irregular vascular channels.

### Urethra

The urethra, which is the last part of the urinary tract, traverses the corpus spongiosum and its opening, known as the meatus, lies on the tip of the glans penis. It is both a passage for urine and for the ejaculation of semen.

### Overview of Male Reproductive System Structure and Function

STRUCTURE LOCATION & DESCRIPTION FUNCTION

Bulbourethral glands (2)
Pea sized organs posterior to the prostate on either side of the urethra.
Secretion of gelatinous seminal fluid called pre-ejaculate. This fluid helps to lubricate the urethra for spermatozoa to pass through, and to help flush out any residual urine or foreign matter. (< 1% of semen)

Cells of Leydig (Interstitial cells of Leydig)
Adjacent to the seminiferous tubules in the testicle.
Responsible for production of testosterone. Closely related to nerves.

Cremaster muscle
Covers the testes.
Raises and lowers scrotum to help regulate temperature and promote spermatogenesis. Voluntary and involuntary contraction.

Dartos muscle
Layer of smooth muscular fiber outside the external spermatic fascia but below the skin
Contraction by wrinkling to decrease surface area available for heat loss to testicles, or expansion to increase surface area available to promote heat loss; also helps raise and lower scrotum to help regulate temperature

Efferent ductules
Part of the testes and connect the rete testis with the epididymis
Ducts for sperm to get to epididymis

Ejaculatory ducts (2)
Begins at the vas deferens, passes through the prostate, and empties into the urethra at the Colliculus seminalis.
Causes reflex for ejaculation. During ejaculation, semen passes through the ducts and exits the body via the penis.

Epididymis
Tightly coiled duct lying just outside each testis connecting efferent ducts to vas deferens.
Storage and maturation of sperm.

Penis
Three columns of erectile tissue: two corpora cavernosa and one corpus spongiosum. Urethra passes through penis.
Male reproductive organ and also male organ of urination.

Prostate gland
Surrounds the urethra just below the urinary bladder and can be felt during a rectal exam.
Stores and secretes a clear, slightly alkaline fluid constituting up to one-third of the volume of semen. Raise vaginal pH.(25-30% of semen)

Scrotum
Pouch of skin and muscle that holds testicles.
Regulates temperature at slightly below body temperature.

Semen
Usually white but can be yellow, gray or pink (blood stained). After ejaculation, semen first goes through a clotting process and then becomes more liquid.
Components are sperm, and "seminal plasma". Seminal plasma is produced by contributions from the seminal vesicle, prostate, and bulbourethral glands.

Seminal vesicles (2)
Convoluted structure attached to vas deferens near the base of the urinary bladder.
About 65-75% of the seminal fluid in humans originates from the seminal vesicles. Contain proteins, enzymes, fructose, mucus, vitamin C, flavins, phosphorylcholine and prostaglandins. High fructose concentrations provide nutrient energy for the spermatozoa as they travel through the female reproductive system.

Seminiferous tubules (2)
Long coiled structure contained in the chambers of the testis; joins with vas deferens.
Meiosis takes place here, creation of gametes (sperm).

Sertoli cells
Junctions of the Sertoli cells form the blood-testis barrier, a structure that partitions the interstitial blood compartment of the testis from the adluminal compartment of the seminiferous tubules.
Cells responsible for nurturing and development of sperm cells , provides both secretory and structural support; activated by FSH. Also called "mother cells" or "nurse cells".

Testes
Inside scrotum, outside of body.
Gonads that produce sperm and male sex hormones.Production of testosterone by cells of Leydig in the testicles.

Testicular arteries (Gonadal arteries)
Branch of the abdominal aorta. It is a paired artery. Each passes obliquely downward and laterally behind the peritoneum.
Supplies blood to the testes.

Urethra
Connects bladder to outside body, about 8 inches long.
Tubular structure that receives urine from bladder and carries it to outside of the body. Also passage for sperm.

Vas deferens
Muscular tubes connecting the left and right epididymis to the ejaculatory ducts to move sperm. Each tube is about 30 cm long.
During ejaculation the smooth muscle in the vas deferens wall contracts, propelling sperm forward. Sperm are transferred from the vas deferens into the urethra, collecting fluids from accessory sex glands en route

### Composition of human semen

The components of semen come from two sources: sperm, and "seminal plasma". Seminal plasma, in turn, is produced by contributions from the seminal vesicle, prostate, and bulbourethral glands.

Seminal plasma of humans contains a complex range of organic and inorganic constituents.

The seminal plasma provides a nutritive and protective medium for the spermatozoa during their journey through the female reproductive tract. The normal environment of the vagina is a hostile one for sperm cells, as it is very acidic (from the native microflora producing lactic acid), viscous, and patrolled by immune cells. The components in the seminal plasma attempt to compensate for this hostile environment. Basic amines such as putrescine, spermine, spermidine and cadaverine are responsible for the smell and flavor of semen. These alkaline bases counteract the acidic environment of the vaginal canal, and protect DNA inside the sperm from acidic denaturation.

**The components and contributions of semen are as follows:**

**GLAND** **APPROXIMATE %** **DESCRIPTION**

testes
2-5%
Approximately 200- to 500-million spermatozoa (also called sperm or spermatozoans), produced in the testes, are released per ejaculation

seminal vesicle
65-75%
amino acids, citrate, enzymes, flavins, fructose (the main energy source of sperm cells, which rely entirely on sugars from the seminal plasma for energy), phosphorylcholine, prostaglandins (involved in suppressing an immune response by the female against the foreign semen), proteins, vitamin C

prostate
25-30%
acid phosphatase, citric acid, fibrinolysin, prostate specific antigen, proteolytic enzymes, zinc (serves to help to stabilize the DNA-containing chromatin in the sperm cells. A zinc deficiency may result in lowered fertility because of increased sperm fragility. Zinc deficiency can also adversely affect spermatogenesis.)

bulbourethral glands
< 1%
galactose, mucus (serve to increase the mobility of sperm cells in the vagina and cervix by creating a less viscous channel for the sperm cells to swim through, and preventing their diffusion out of the semen. Contributes to the cohesive jelly-like texture of semen.), pre-ejaculate, sialic acid

A 1992 World Health Organization report described normal human semen as having a volume of 2 ml or greater, pH of 7.2 to 8.0, sperm concentration of 20x106 spermatozoa/ml or more, sperm count of 40x106 spermatozoa per ejaculate or more and motility of 50% or more with forward progression (categories a and b) of 25% or more with rapid progression (category a) within 60 minutes of ejaculation.[2]

## Functions

![Male reproductive hormone chart.gif](../../../upload.wikimedia.org/wikipedia/commons/thumb/5/5f/Male_reproductive_hormone_chart.gif/550px-Male_reproductive_hormone_chart.gif)

### Hormone Regulation

Hormones which control reproduction in males are:

  
Gonadotropin-Releasing Hormone (GnRH):

  * The hypothalamus secretes this hormone into the pituitary gland in the brain.
  * There are two gonadotropic hormones, FSH and LH.

Luteinizing Hormone (LH):

  * The pituitary gland secretes this hormone after receiving a GnRH signal from the hypothalamus.
  * LH stimulates Leydig cells, in the testes, telling them to produce testosterone.

Follicle-Stimulating Hormone (FSH):

  * The pituitary gland also secretes this hormone.
  * Testosterone helps FSH run through the bloodstream to make Sertoli cells, located in the seminiferous tubules of the testes, to make immature sperm to mature sperm.

Testosterone:

  * Also know as "the male hormone" and "androgen".
  * Testosterone is vital for the production of sperm.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/e/ef/Gray1154.png/150px-Gray1154.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

The constituent cavernous cylinders of the penis.

### Erection

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/f/f6/Gray1155.png/250px-Gray1155.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Transverse section of the penis.

The erection of the penis is its enlarged and firm state. It depends on a complex interaction of psychological, neural, vascular and endocrine factors. The term is also applied to the process that leads to this state.

A penile erection occurs when two tubular structures that run the length of the penis, the corpora cavernosa, become engorged with venous blood. This is a result of parasympathetic nerve induced vasodilation. This may result from any of various physiological stimuli. The corpus spongiosum is a single tubular structure located just below the corpora cavernosa, which contains the urethra, through which urine and semen pass during urination and ejaculation, respectively. This may also become slightly engorged with blood, but less so than the corpora cavernosa.

Penile erection usually results from sexual stimulation and/or arousal, but can also occur by such causes as a full urinary bladder or spontaneously during the course of a day or at night, often during erotic or wet dreams. An erection results in swelling and enlargement of the penis. Erection enables sexual intercourse and other sexual activities (sexual functions), though it is not essential for all sexual activities.

### Ejaculation

Emission is the term used when sperm moves into the urethra. Ejaculation is the term used when sperm is forced out of the urethra and the penis. These are both stimulated by sympathetic nerves.

### Sperm Production

A spermatozoon or spermatozoan (pl. spermatozoa), from the ancient Greek σπέρμα (seed) and ζῷον (living being) and more commonly known as a sperm cell, is the haploid cell that is the male gamete.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/9f/Complete_diagram_of_a_human_spermatozoa_en.svg/220px-Complete_diagram_of_a_human_spermatozoa_en.svg.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

A mature human [Spermatozoon](http://en.wikibooks.org/w/index.php?title=Spermatozoon&action=edit&redlink=1)

Spermatagonia divides several times during the process of sperm development. The entire process of sperm formation and maturation takes about 9-10 weeks. The separate divisions that take place and what happens in each are as follows:

  * **First division**: The first division is done by mitosis, and ensures a constant supply of _spermatocytes_, each with the diploid number of chromosomes.
  * **Second division**: Spermatocytes then undergo a series of two cell divisions during meiosis to become _secondary spermatocytes_.
  * **Third division**: Secondary Spermatocytes finally become _spermatids_. Spermatids, which are haploid cells, mature slowly to become the male gametes, or _sperm_.

The sperm is the main reproductive cell in males. The sperms differ in that each carry a set of chromosomes dividing each into either a male, or female sperm. The females differ in that they carry a X gene, while the male sperm carry a Y gene. The female sperm also differ phenotypically in that they have a larger head in comparison to the male sperms. This contributes to the male sperm being lighter, and therefore faster and stronger swimmers than their female counterparts (although statistically there is still a 50% chance of an either XY or XX embryo forming.

Spermatozoan stream lines are straight and parallel. The tail flagellates, which we now know propels the sperm cell (at about 1-3 mm/minute in humans) by rotating like a propeller, in a circular motion, not side to side like a whip. The cell is characterized by a minimum of cytoplasm. During fertilization, the sperm's mitochondria gets destroyed by the egg cell, and this means only the mother is able to provide the baby's mitochondria and mitochondrial DNA, which has an important application in tracing maternal ancestry. However it has been recently discovered that mitochondrial DNA can be recombinant.

Spermatozoa are produced in the seminiferous tubules of the testes in a process called spermatogenesis. Round cells called spermatogonia divide and differentiate eventually to become spermatozoa. During copulation the vagina is inseminated, the spermatozoa move through _chemotaxis_ (see glossary) to the ovum inside a Fallopian tube or the uterus.

### Sperm Pathway

Spermatogenesis takes place inside a male’s testes, specifically in the walls of the seminiferous tubules. The epididymis is a tortuously coiled structure topping the testis, it receives immature sperm from the testis and stores it for several days. When ejaculation occurs, sperm is forcefully expelled from the tail of the epididymis into the ductus deferens. Sperm travels through the ductus deferens and up the spermatic cord into the pelvic cavity, over the ureter to the prostate behind the bladder. Here, the vas deferens joins with the seminal vesicle to form the ejaculatory duct, which passes through the prostate and empties into the urethra. Upon the sperm's exit from the testes, into the vas deferens, muscular movements take over. When ejaculation occurs, rhythmic muscle movements of _peristalsis_ propel the sperm forward. This continues throughout the remainder of the sperm's journey through the male reproductive system.

Sperm cells become even more active when they begin to interact with the _fertilizing layer_ of an egg cell. They swim faster and their tail movements become more forceful and erratic. This behavior is called "hyper activation."

A recent discovery links hyper activation to a sudden influx of calcium ions into the tails. The whip-like tail (flagellum) of the sperm is studded with ion channels formed by proteins called CatSper. These channels are selective, allowing only calcium ion to pass. The opening of CatSper channels is responsible for the influx of calcium. The sudden rise in calcium levels causes the flagellum to form deeper bends, propelling the sperm more forcefully through the viscous environment.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/8/83/Acrosome_reaction_diagram_en.svg/440px-Acrosome_reaction_diagram_en.svg.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Acrosome reaction on a Sea Urchin cell

The sperm use their tails to push themselves into the epididymis, where they complete their development. It takes sperm about 4 to 6 weeks to travel through the epididymis. The sperm then move to the vas deferens, or sperm duct. The seminal vesicles and prostate gland produce a whitish fluid called seminal fluid, which mixes with sperm to form semen when a male is sexually stimulated.

The penis, which usually hangs limp, becomes hard when a male is sexually excited. Tissues in the penis fill with blood and it becomes stiff and erect (an erection). The rigidity of the erect penis makes it easier to insert into the female's vagina during sexual intercourse, and the extended length allows it to reach deeper into the female's _oviduct_, the passage from the ovaries to the outside of the body (allowing a shorter travel distance for the spermatozoa).

When the erect penis is stimulated to orgasm, muscles around the reproductive organs contract and force the semen through the duct system and urethra. Semen is pushed out of the male's body through his urethra - ejaculation. The speed of the semen is about 70 mph when ejaculation comes and it can contain 100 to 600 million sperm cells. When the male ejaculates during intercourse, semen is deposited into the fornix at the base of the female's vagina, near the cervix. From the fornix, the sperm make their way up through the cervix and move through the uterus with help from uterine contractions.

Sperm hyperactivity is necessary for breaking through two physical barriers that protect the egg from fertilization. The first barrier to sperm is made up of so-called cumulus cells embedded in a gel-like substance made primarily of hyaluronic acid. The cumulus cells develop in the ovary with the egg and support it as it grows.

The second barrier coating the oocyte is a thick shell formed by glycoproteins called the zona pellucida. One of the proteins that make up the zona pellucida binds to a partner molecule on the sperm. This lock-and-key type mechanism is species-specific and prevents the sperm and egg of different species from fusing. There is some evidence that this binding is what triggers the acrosome to release the enzymes that allow the sperm to fuse with the egg.

When a sperm cell reaches the egg the acrosome releases its enzymes. These enzymes weaken the shell, allowing the sperm cell to penetrate it and reach the plasma membrane of the egg. Part of the sperm's cell membrane then fuses with the egg cell's membrane, and the sperm cell sinks into the egg (at which point the sperm tail falls off).

Upon penetration, the egg cell membrane undergoes a change and becomes impenetrable, preventing further fertilization.

The binding of the sperm to an ovum is called a zygote. A zygote is a single cell, with a complete set of chromosomes, that normally develops into an embryo.

## Puberty

In addition to producing sperm, the male reproductive system also produces sex hormones, which help a boy develop into a sexually mature man during puberty. When a baby boy is born, he has all the parts of his reproductive system in place, but it isn't until puberty that his reproductive organs mature and become fully functional. As an newborn FSH and LH levels are high and after a few weeks levels drop to extremely low. When puberty begins, usually between the ages of 10 and 14, the pituitary gland - which is located in the brain - secretes hormones that stimulate the testicles to produce testosterone. The production of testosterone brings about many physical changes. Although the timing of these changes is different for each individual male, the stages of puberty generally follow a set sequence.

  * First stage: the scrotum and testes grow larger, the _apocrine glands_ develop (see explanation of apocrine glands in glossary).
  * Second stage: the penis becomes longer, and the seminal vesicles and prostate gland grow. Hair begins to grow in the pubic region. Reproductive capacity has usually developed by this stage.
  * Third stage: hair begins to appear on the face and underarms. During this time, a male's voice also deepens. Fertility continues to increase.

### Testicular size, function, and fertility

In boys, testicular enlargement is the first physical manifestation of puberty (and is termed gonadarche). Testes in prepubertal boys change little in size from about 1 year of age to the onset of puberty, averaging about 2–3 cc in volume and about 1.5-2 cm in length. Testicular size continues to increase throughout puberty, reaching maximal adult size about 6 years later. While 18-20 cc is reportedly an average adult size, there is wide variation in the normal population.

The testes have two primary functions: to produce hormones and to produce sperm. The Leydig cells produce testosterone (as described below), which in turn produces most of the changes of male puberty. However, most of the increasing bulk of testicular tissue is spermatogenic tissue (primarily Sertoli and interstitial cells). The development of sperm production and fertility in males is not as well researched. Sperm can be detected in the morning urine of most boys after the first year of pubertal changes (and occasionally earlier).

### Genitalia

A boy's penis grows little from the fourth year of life until puberty. Average prepubertal penile length is 4 cm. The prepubertal genitalia are described as stage 1. Within months after growth of the testes begins, rising levels of testosterone promote growth of the penis and scrotum. This earliest discernible beginning of pubertal growth of the genitalia is referred to as stage 2. The penis continues to grow until about 18 years of age, reaching an average adult size of about 10-16 cm.

Although erections and orgasm can occur in prepubertal boys, they become much more common during puberty, accompanied by development of _libido_ (sexual desire). Ejaculation becomes possible early in puberty; prior to this boys may experience dry orgasms. Emission of seminal fluid may occur due to masturbation or spontaneously during sleep (commonly termed a _wet dream_, and more clinically called a _nocturnal emission_). The ability to ejaculate is a fairly early event in puberty compared to the other characteristics, and can occur even before reproductive capacity itself. In parallel to the irregularity of the first few periods of a girl, for the first one or two years after a boy's first ejaculation, his seminal fluid may contain few active sperm.

If the foreskin of a boy does not become retractable during childhood, it normally begins to retract during puberty. This occurs as a result of the increased production of testosterone and other hormones in the body.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/8/8c/Androgenic_hair.JPG/150px-Androgenic_hair.JPG)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Distribution of androgenic hair on female and male body

### Genital Erection

The penis contains two chambers called the corpora cavernosa, which run the length of the organ. A spongy tissue, full of muscle, veins, arteries, etc. fills these chambers. The corpora cavernosa are surrounded by a membrane, called the tunica albuginea.

Erection begins with sensory or mental stimulation, or both. Impulses from the brain and local nerves cause the muscles of the corpora cavernosa to relax, allowing blood to flow in and fill the spaces. The blood creates pressure in the corpora cavernosa, making the penis expand. The tunica albuginea helps trap the blood in the corpora cavernosa, thereby sustaining erection. When muscles in the penis contract to stop the inflow of blood and open outflow channels, erection is reversed.

### Pubic hair in boys

Pubic hair often appears on a boy shortly after the genitalia begin to grow. As in girls, the first appearance of pubic hair is termed pubarche and the pubic hairs are usually first visible at the dorsal (abdominal) base of the penis. The first few hairs are described as stage 2. Stage 3 is usually reached within another 6 to 12 months, when the hairs are too numerous to count. By stage 4, the pubic hairs densely fill the "pubic triangle." Stage 5 refers to spread of pubic hair to the inner thighs and upward towards the umbilicus as part of the developing abdominal hair.

## Sexual Homology

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/3/35/Male_anatomy.png/440px-Male_anatomy.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

The human male reproductive system

![](../../../upload.wikimedia.org/wikipedia/commons/7/7a/Female_reproductive_system_lateral.png)

Cross-sectional diagram of the female reproductive organs.

In short, this is a known list of sex organs that evolve from the same tissue in a human life.

Indifferent Male Female

Gonad
Testis
Ovary

Mullerian duct
Appendix testis
Fallopian tubes

Mullerian duct
Prostatic utricle
Uterus, proximal vagina

Wolffian duct
Rete testis
Rete ovarii

Mesonephric tubules
Efferent ducts
Epoophoron

Wolffian duct
Epididymis
Gartner's duct

Wolffian duct
Vas deferens

Wolffian duct
Seminal vesicle

Wolffian duct
Prostate
Skene's glands

Urogenital sinus
Bladder, urethra
Bladder, urethra, distal vagina

Urogenital sinus
Bulbourethral gland
Bartholin's gland

Genital swelling
Scrotum
Labia majora

Urogenital folds
Distal urethra
Labia minora

Genital tubercle
Penis
Clitoris

Prepuce
Foreskin
Clitoral hood

Bulb of penis
Vestibular bulbs

Glans penis
Clitoral glans

Crus of penis
Clitoral crura

## Aging

For most men, testosterone secretion continues throughout life, as does sperm production, though both diminish with advancing age. Probably the most common reproductive problem for older men is prostatic hypertrophy, enlargement of the prostate gland. This causes the urethra to compress and urination becomes difficult. Residual urine in the bladder increases the chance of urinary tract infections. Prostate hypertrophy is usually benign, but cancer of the prostate is one of the more common cancers in elderly men. A TURP is commonly used to correct this problem if the symptoms do not improve in response to home treatment and medication.

Erectile dysfunction (ED) is another common problem seen in aging males. In older men, ED usually has a physical cause, such as disease, injury, or side effects of drugs. Any disorder that impairs blood flow in the penis or causes injury to the nerves has the potential to cause ED. Although it is not an inevitable part of aging, incidences increases with age: About 5 percent of 40-year-old men and between 15 and 25 percent of 65-year-old men experience ED. As discouraging as Erectile dysfunction may be, it is treatable at any age, and awareness of this fact has been growing. More men have been seeking help and returning to normal sexual activity because of improved, successful treatments for ED.

## Things that can go wrong with the male reproductive system

Boys may sometimes experience reproductive system problems. Below are some examples of disorders that affect the male reproductive system (Disorders of the Scrotum, Testicles, or Epididymis). Conditions affecting the scrotal contents may involve the testicles, epididymis, or the scrotum itself.

  * **Testicular trauma.** Even a mild injury to the testicles can cause severe pain, bruising, or swelling. Most testicular injuries occur when the testicles are struck, hit, kicked, or crushed, usually during sports or due to other trauma. Testicular torsion, when 1 of the testicles twists around, cutting off the blood supply, is also a problem that some teen males experience - although it's not common. Surgery is needed to untwist the cord and save the testicle.
  * **[Varicocele](http://en.wikipedia.org/wiki/Varicocele).** This is a varicose vein (an abnormally swollen vein) in the network of veins that run from the testicles. Varicoceles commonly develop while a boy is going through puberty. A varicocele is usually not harmful, although in some people it may damage the testicle or decrease sperm production, so it helps for you to take your child to see his doctor if he is concerned about changes in his testicles.
  * **[Testicular cancer](http://en.wikipedia.org/wiki/Testicular_cancer).** This is one of the most common cancers in men younger than 40. It occurs when cells in the testicle divide abnormally and form a tumor. Testicular cancer can spread to other parts of the body, but if it's detected early, the cure rate is excellent. Teen boys should be encouraged to learn to perform testicular self-examinations.
  * **[Epididymitis](http://en.wikipedia.org/wiki/Epididymitis)** is inflammation of the epididymis, the coiled tubes that connect the testes with the vas deferens. It is usually caused by an infection, such as the sexually transmitted disease chlamydia, and results in pain and swelling next to 1 of the testicles.
  * **[Hydrocele](http://en.wikipedia.org/wiki/Hydrocele).** A hydrocele occurs when fluid collects in the membranes surrounding the testes. Hydroceles may cause swelling of the testicle but are generally painless. In some cases, surgery may be needed to correct the condition.
  * **[Inguinal hernia](http://en.wikipedia.org/wiki/Inguinal_hernia).** When a portion of the intestines pushes through an abnormal opening or weakening of the abdominal wall and into the groin or scrotum, it is known as an inguinal hernia. The hernia may look like a bulge or swelling in the groin area. It can be corrected with surgery.

### Disorders of Penis

Disorders of the Penis Disorders affecting the penis include the following:

  * **Inflammation of the penis.** Symptoms of penile inflammation include redness, itching, swelling, and pain. Balanitis occurs when the glans (the head of the penis) becomes inflamed. Posthitis is foreskin inflammation, which is usually due to a yeast or bacterial infection.
  * **Hypospadias.** This is a disorder in which the urethra opens on the underside of the penis, not at the tip.
  * **Phimosis.** This is a tightness of the foreskin of the penis and is common in newborns and young children. It usually resolves itself without treatment. If it interferes with urination, circumcision (removal of the foreskin) may be recommended.
  * **Paraphimosis.** This may develop when a boy's uncircumcised penis is retracted but doesn't return to the unretracted position. As a result, blood flow to the penis may be impaired, and your child may experience pain and swelling. A doctor may try to use lubricant to make a small incision so the foreskin can be pulled forward. If that doesn't work, circumcision may be recommended.
  * **Ambiguous genitalia.** This occurs when a child is born with genitals that aren't clearly male or female. In most boys born with this disorder, the penis may be very small or nonexistent, but testicular tissue is present. In a small number of cases, the child may have both testicular and ovarian tissue.
  * **Micro penis.** This is a disorder in which the penis, although normally formed, is well below the average size, as determined by standard measurements.
  * **Sexually transmitted diseases.** Sexually transmitted diseases (STDs) that can affect boys include human immunodeficiency virus/acquired immunodeficiency syndrome (HIV/AIDS), human papillomavirus (HPV, or genital warts), syphilis, chlamydia, gonorrhea, genital herpes, and hepatitis B. They are spread from one person to another mainly through sexual intercourse.
  * **Erectile dysfunction.** E.D. is the inability to get or keep an erection firm enough for sexual intercourse. This can also called impotence. The word "impotence" may also be used to describe other problems that can interfere with sexual intercourse and reproduction, such as problems with ejaculation or orgasm and lack of sexual desire. Using the term erectile dysfunction clarifies that those other problems are not involved.

### Contraceptive for Men

Vasectomy: In the procedure the vas deferens of each testes is cut and tied off to prevent the passage of sperm. Sperm is still produced and stored in crypt sites causing inflammation. Because of this inflammatory response the immune system acts on them destroying them and then having antisperm antibodies. This causes a lower possibility if the vasectomy is reversed to becoming fertile again.

Condoms: A device, usually made of latex, or more recently polyurethane, that is used during sexual intercourse. It is put on a man's penis and physically blocks ejaculated semen from entering the body of a sexual partner. Condoms are used to prevent pregnancy, transmission of sexually transmitted diseases (STDs - such as gonorrhea, syphilis, and HIV), or both.

## Review Questions

Answers for these questions can be found [here](http://en.wikibooks.org/wiki/Human_Physiology/Appendix_1:_answers_to_review_questions#The_male_reproductive_system)

1\. This is needed to make immature sperm mature

    A) FHS
    B) LH
    C) FSH
    D) HL

2\. These become engorged with blood in an erection

    A) corpora cavernosa
    B) fibrous envelope
    C) septum pectiniforme
    D) integument
    E) dorsal veins

3\. The difference between male and female sperm

    A) female sperm have a larger head
    B) male sperm are lighter
    C) female sperm are faster
    D) male sperm are weaker
    E) A and B
    F) C and D

4\. The entire process of sperm formation takes about

    A) 5-6 weeks
    B) 7-8 weeks
    C) 3-4 weeks
    D) 9-10 weeks

5\. Hyper Activation occurs when

    A) the sperm are introduced into the urethra
    B) the sperm are ejaculated into the vaginal canal
    C) the sperm begin to interact with the fertilizing layer of an egg cell
    D) the sperm reach the cervix

  
6\. It takes sperm ___________ weeks to travel through the epididymis

    A) 6-8
    B) 1-3
    C) 2-4
    D) 4-6

7\. While singing in the choir, Ben suddenly notices his voice is constantly cracking. This is caused by

    A) androgens
    B) LH
    C) FSH
    D) Ben’s inability to sing

8\. In sexual homology, the glans penis in the male is equal to _____________ in the female

    A) clitoral hood
    B) clitoris
    C) clitoral glans
    D) clitoral crura

9\. In sexual homology, the ___________ in the male is equal to the fallopian tubes in the female

    A) testis
    B) appendix testis
    C) vas deferens
    D) seminal vesicle
    E) efferent ducts

10\. Joe has a bulge in the groin area that seems to get worse when he lifts things. This most likely is

    A) epididymitis
    B) testicular cancer
    C) varicocele
    D) hydrocele
    E) inguinal hernia

## Glossary

**Androgen**: The generic term for any natural or synthetic compound, usually a steroid hormone, that stimulates or controls the development and maintenance of masculine characteristics in vertebrates by binding to androgen receptors. This includes the activity of the accessory male sex organs and development of male secondary sex characteristics. They are also the precursor of all estrogens, the female sex hormones. The primary and most well-known androgen is testosterone.

**Apocrine Glands**: Apocrine sweat glands develop during the early to mid puberty ages approximately around the age of 15 and release more than normal amounts of sweat for approximately a month and subsequently regulate and release normal amounts of sweat after a certain period of time. They are located wherever there is body hair. These glands produce sweat that contains fatty materials. Mainly present in the armpits and around the genital area, their activity is the main cause of sweat odor, due to the bacteria that break down the organic compounds in the sweat.

**Bulbourethral Glands**: male accessory sex glands that secrete mucus for lubrication

**Chemotaxis**: Chemotaxis is a kind of taxis, in which bodily cells, bacteria, and other single-cell or multicellular organisms direct their movements according to certain chemicals in their environment. This is important for bacteria to find food (for example, glucose) by swimming towards the highest concentration of food molecules, or to flee from poisons (for example, phenol). In multicellular organisms, chemotaxis is critical to development as well as normal function. In addition, it has been recognized that mechanisms that allow chemotaxis in animals can be subverted during cancer metastasis.

**Corpora Cavernosa**: one of a pair of a sponge-like regions of erectile tissue which contain most of the blood in the male penis during erection

**Ductus Deferens**: epididymal ducts from each testis converge to form a large, thick walled, muscular duct

**Ejaculatory Ducts**: two ducts, receive sperm from the ductus deferens and secretions from the seminal vesicle; the ducts then empty into the urethra

**Epididymis**: comma shaped and loosely attached to the rear surface of each testis

**Erectile Tissue**: smooth muscle and connective tissue inside the penis that contain blood sinuses; large, irregular vascular channels

**Erection**: the penis at its enlarged and firm state; occurs when the corpora cavernosa become engorged with venous blood

**Flagellum**: the whip-like tail of a sperm, propels the sperm towards the egg in hopes of achieving fertilization

**Follicle-Stimulating Hormone (FSH)**: hormone that stimulates production of sertoli cells, to make immature sperm to mature sperm

**Glans Penis**: distal end of the penis, covered with the foreskin

**Gonadotropin-Releasing Hormone (GnRH)**: hormone secreted by the hypothalamus into the pituitary gland; two types, FSH and LH

**Libido**: In its common usage, it means sexual desire; however, more technical definitions, such as those found in the work of Carl Jung, are more general, referring to libido as the free creative—or psychic—energy an individual has to put toward personal development, or individuation.

**Luteinizing Hormone (LH)**: hormone that stimulates Leydig cells in the testes to produce testosterone

**Oviduct**: the passage in females from the ovaries to the outside of the body.

**Penis**: external genital organ of the male

**Prostate Gland**: male accessory sex gland that secretes an alkaline fluid, which neutralizes acidic vaginal secretions

**Puberty**: the period of maturation and arousal of the dormant and nonfunctional reproductive system; usually occurs in males between the ages of 10 and 15

**Scrotum**: skin covered sac that houses the male testicles; keeps the testicles away form the body so that they can stay a few degrees cooler than the body, for better sperm production

**Seminal Vesicle**: male accessory sex glands that supply fructose to ejaculated sperm and secrete prostaglandins

**Seminiferous Tubules**: highly coiled tubules within the testes that produce spermatozoa

**Sertoli Cell**: A Sertoli cell (a kind of sustentacular cell) is a 'nurse' cell of the testes which is part of a seminiferous tubule.

It is activated by follicle-stimulating hormone, and has FSH-receptor on its membranes.

Its main function is to nurture the developing sperm cells through the stages of spermatogenesis. Because of this, it has also been called the "mother cell." It provides both secretory and structural support.

**Sexual Homology**: sex organs that evolve from the same tissues in both male and females

**Sperm**: main reproductive cell in males

**Spermatogenesis**: sperm production

**Testes**: located in the scrotum, produces testosterone which stimulates production of sperm

**Testosterone**: male sex hormone secreted by the leydig cells of the testes, vital for the production of sperm

**TURP**: transurethral resection of the prostate. During TURP, an instrument is inserted up the urethra to remove the section of the prostate that is blocking urine flow. This is most commonly caused by benign prostatic hyperplasia (BPH). A TURP usually requires hospitalization and is done using a general or spinal anesthetic. It is now the most common surgery used to remove part of an enlarged prostate.

**Urethra**: the last part of the urinary tract; in males, it is the passage for both urine and sperm

**Varicocele**: varicose vein of the testicles, sometimes a cause of male infertility

**Vasectomy**: most common sterilization procedure in males; small segment of each ductus deferens is surgically removed after it passes from the testis

## Summary

Both male and female reproductive systems may seem somewhat isolated from other body systems in that their purpose is to create new life and not just to maintain existing life. There are however significant relationships between the reproductive system and other body systems. All systems relate in one way or another to help our bodies maintain homeostasis.

## References

  * "Essentials of Anatomy and Physiology" by Valerie C. Scanlon and Tina Sanders
  * "Web MD": <http://www.webmd.com>
  * Wikibook: [Sexual Health](http://en.wikibooks.org/wiki/Sexual_Health)

  


# The Female Reproductive System

## Introduction

All living things reproduce. This is something that sets the living apart from non-living. Even though the reproductive system is essential to keeping a species alive, it is not essential to keeping an individual alive. This chapter describes the different parts of the female reproductive system: the organs involved in the process of reproduction, hormones that regulate a woman's body, the menstrual cycle, ovulation and pregnancy, the female's role in genetic division, birth control, sexually transmitted diseases and other diseases and disorders.

### Reproduction

Reproduction can be defined as the process by which an organism continues its species. In the human reproductive process, two kinds of sex cells ( gametes), are involved: the male gamete (sperm), and the female gamete (egg or ovum). These two gametes meet within the female's uterine tubes located one on each side of the upper pelvic cavity, and begin to create a new individual. The female needs a male to fertilize her egg; she then carries offspring through pregnancy and childbirth.

#### Similarities between male and female reproductive systems

The reproductive systems of the male and female have some basic similarities and some specialized differences. They are the same in that most of the reproductive organs of both sexes develop from similar embryonic tissue, meaning they are homologous. Both systems have gonads that produce (sperm and egg or ovum) and sex organs. And both systems experience maturation of their reproductive organs, which become functional during puberty as a result of the gonads secreting sex hormones.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/3/35/Male_anatomy.png/440px-Male_anatomy.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

The human male reproductive system

![](../../../upload.wikimedia.org/wikipedia/commons/7/7a/Female_reproductive_system_lateral.png)

Cross-sectional diagram of the female reproductive organs.

In short, this is a known list of sex organs that evolve from the same tissues in a human life.

Undifferentiated Male Female

Gonad
Testis
Ovary

Mullerian duct
Appendix testis
Fallopian tubes

Mullerian duct
Prostatic utricle
Uterus, proximal

Wolffian duct
Rete testis
Rete ovarii

Mesonephric tubules
Efferent ducts
Epoophoron

Wolffian duct
Epididymis
Gartner's duct

Wolffian duct
Vas deferens

Wolffian duct
Seminal vesicle

Wolffian duct
Prostate
Skene's glands

Urogenital sinus
Bladder, urethra
Bladder, urethra, distal

Urogenital sinus
Bulbourethral gland
Bartholin's gland

Genital swelling
Scrotum
Labia majora

Urogenital folds
Distal urethra
Labia minora

Genital tubercle
Penis
Clitoris

Prepuce
Clitoral hood

Bulb of penis
Vestibular bulbs

Glans penis
Clitoral glans

Crus of penis
Clitoral crura

#### Differences between male and female reproductive systems

The differences between the female and male reproductive systems are based on the functions of each individual's role in the reproduction cycle. A male who is healthy, and sexually mature, continuously produces sperm. The development of women's "eggs" are arrested during fetal development. This means she is born with a predetermined number of oocytes and cannot produce new ones.

At about 5 months gestation, the ovaries contain approximately six to seven million oogonia, which initiate meiosis. The oogonia produce primary oocytes that are arrested in prophase I of meiosis from the time of birth until puberty. After puberty, during each menstrual cycle, one or several oocytes resume meiosis and undergo their first meiotic division during ovulation. This results in the production of a secondary oocyte and one polar body. The meiotic division is arrested in metaphase II. Fertilization triggers completion of the second meiotic division and the result is one ovum and an additional polar body.

The ovaries of a newborn baby girl contain about one million oocytes. This number declines to 400,000 to 500,000 by the time puberty is reached. On average, 500-1000 oocytes are ovulated during a woman's reproductive lifetime.

When a young woman reaches puberty around age 10 to 13, a promary oocyte is discharged from one of the ovaries every 28 days. This continues until the woman reaches menopause, usually around the age of 50 years. Occytes are present at birth, and age as a woman ages.

Female Reproductive System

  * Produces eggs (ova)
  * Secretes sex hormones
  * Receives the male spermatazoa during
  * Protects and nourishes the fertilized egg until it is fully developed
  * Delivers fetus through birth canal
  * Provides nourishment to the baby through milk secreted by mammary glands in the breast

## External Genitals

![Gray1229.png](../../../upload.wikimedia.org/wikipedia/commons/thumb/2/26/Gray1229.png/250px-Gray1229.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

**Vulva**

The external female genitalia is referred to as vulva. It consists of the labia majora and labia minora (while these names translate as "large" and "small" lips, often the "minora" can protrude outside the "majora"), mons pubis, clitoris, opening of the urethra (meatus), vaginal vestibule, vestibular bulbs, vestibular glands.

The term "vagina" is often improperly used as a generic term to refer to the vulva or female genitals, even though - strictly speaking - the vagina is a specific internal structure and the vulva is the exterior genitalia only. Calling the vulva the vagina is akin to calling the mouth the throat.

**Mons Veneris**

The **mons veneris**, Latin for "mound of Venus" (Roman Goddess of love) is the soft mound at the front of the vulva (fatty tissue covering the pubic bone). It is also referred to as the mons pubis. The mons veneris protects the pubic bone and vulva from the impact of sexual intercourse. After puberty, it is covered with pubic hair, usually in a triangular shape. Heredity can play a role in the amount of pubic hair an individual grows.

**Labia Majora**

The **labia majora** are the outer "lips" of the vulva. They are pads of loose connective and adipose tissue, as well as some smooth muscle. The labia majora wrap around the vulva from the mons pubis to the perineum. The labia majora generally hides, partially or entirely, the other parts of the vulva. There is also a longitudinal separation called the pudendal cleft. These labia are usually covered with pubic hair. The color of the outside skin of the labia majora is usually close to the overall color of the individual, although there may be some variation. The inside skin is usually pink to light brown. They contain numerous sweat and oil glands. It has been suggested that the scent from these oils are sexually arousing.

**Labia Minora**

Medial to the labia majora are the labia minora. The **labia minora** are the inner lips of the vulva. They are thin stretches of tissue within the labia majora that fold and protect the vagina, urethra, and clitoris. The appearance of labia minora can vary widely, from tiny lips that hide between the labia majora to large lips that protrude. There is no pubic hair on the labia minora, but there are sebaceous glands. The two smaller lips of the labia minora come together longitudinally to form the prepuce, a fold that covers part of the clitoris. The labia minora protect the vaginal and urethral openings. Both the inner and outer labia are quite sensitive to touch and pressure.

**Clitoris**

![Clitoris inner anatomy.gif](../../../upload.wikimedia.org/wikipedia/commons/thumb/8/8c/Clitoris_inner_anatomy.gif/133px-Clitoris_inner_anatomy.gif)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

The **clitoris**, visible as the small white oval between the top of the labia minora and the clitoral hood, is a small body of spongy tissue that functions solely for sexual pleasure. Only the tip or glans of the clitoris shows externally, but the organ itself is elongated and branched into two forks, the crura, which extend downward along the rim of the vaginal opening toward the perineum. Thus the clitoris is much larger than most people think it is, about 4" long on average.

The clitoral glans or external tip of the clitoris is protected by the prepuce, or clitoral hood, a covering of tissue similar to the foreskin of the male penis. However, unlike the penis, the clitoris does not contain any part of the urethra.

During sexual excitement, the clitoris erects and extends, the hood retracts, making the clitoral glans more accessible. The size of the clitoris is variable between women. On some, the clitoral glans is very small; on others, it is large and the hood does not completely cover it.

**Urethra**

The opening to the urethra is just below the clitoris. Although it is not related to sex or reproduction, it is included in the vulva. The **urethra** is actually used for the passage of urine. The urethra is connected to the bladder. In females the urethra is 1.5 inches long, compared to males whose urethra is 8 inches long. Because the urethra is so close to the anus, women should always wipe themselves from front to back to avoid infecting the vagina and urethra with bacteria. This location issue is the reason for bladder infections being more common among females.

**Hymen**

  
The hymen is a thin fold of mucous membrane that separates the lumen of the vagina from the urethral sinus. Sometimes it may partially cover the vaginal orifice. The hymen is usually perforated during later fetal development.

Because of the belief that first vaginal penetration would usually tear this membrane and cause bleeding, its "intactness" has been considered a guarantor of virginity. However, the hymen is a poor indicator of whether a woman has actually engaged in sexual intercourse because a normal hymen does not completely block the vaginal opening. The normal hymen is never actually "intact" since there is always an opening in it. Furthermore, there is not always bleeding at first vaginal penetration. The blood that is sometimes, but not always, observed after first penetration can be due to tearing of the hymen, but it can also be from injury to nearby tissues.

A tear to the hymen, medically referred to as a "transection," can be seen in a small percentage of women or girls after first penetration. A transection is caused by penetrating trauma. Masturbation and tampon insertion can, but generally are not forceful enough to cause penetrating trauma to the hymen. Therefore, the appearance of the hymen is not a reliable indicator of virginity or chastity.

**Perineum**

The perineum is the short stretch of skin starting at the bottom of the vulva and extending to the anus. It is a diamond shaped area between the symphysis pubis and the coccyx. This area forms the floor of the pelvis and contains the external sex organs and the anal opening. It can be further divided into the urogenital triangle in front and the anal triangle in back.

The perineum in some women may tear during the birth of an infant and this is apparently natural. Some physicians however, may cut the perineum preemptively on the grounds that the "tearing" may be more harmful than a precise cut by a scalpel. If a physician decides the cut is necessary, they will perform it. The cut is called an episiotomy.

## Internal Genitals

### Vagina

The **vagina** is a muscular, hollow tube that extends from the vaginal opening to the cervix of the uterus. It is situated between the urinary bladder and the rectum. It is about three to five inches long in a grown woman. The muscular wall allows the vagina to expand and contract. The muscular walls are lined with mucous membranes, which keep it protected and moist. A thin sheet of tissue with one or more holes in it, called the hymen, partially covers the opening of the vagina. The vagina receives sperm during sexual intercourse from the penis. The sperm that survive the acidic condition of the vagina continue on through to the fallopian tubes where fertilization may occur.

The vagina is made up of three layers, an inner mucosal layer, a middle muscularis layer, and an outer fibrous layer. The inner layer is made of vaginal rugae that stretch and allow penetration to occur. These also help with stimulation of the penis. microscopically the vaginal rugae has glands that secrete an acidic mucus (pH of around 4.0.) that keeps bacterial growth down. The outer muscular layer is especially important with delivery of a fetus and placenta.

  


Purposes of the Vagina

  * Receives a male's erect penis and semen during sexual intercourse.
  * Pathway through a woman's body for the baby to take during childbirth.
  * Provides the route for the menstrual blood (menses) from the uterus, to leave the body.
  * May hold forms of birth control, such as a diaphragm, FemCap, Nuva Ring, or female condom.

**Clinical Application:**  
Pelvic inflammatory disease (PID) is a widespread infection that originates in the vagina and uterus and spreads to the uterine tubes, ovaries, and ultimately the pelvic peritoneum. This condition, which occurs in about 10% of women is usually caused by chlamydial or gonorrheal infection, other bacteria infecting the vagina may be involved as well. Signs and symptoms include tenderness of the lower abdomen, fever, and a vaginal discharge. Even a single episode of PID can cause infertility, due to scarring that blocks the uterine tubes. Therefore, patients are immediately given broad-spectrum antibiotics whenever PID is suspected.

### Cervix

The **cervix** (from Latin "neck") is the lower, narrow portion of the uterus where it joins with the top end of the vagina. Where they join together forms an almost 90 degree curve. It is cylindrical or conical in shape and protrudes through the upper anterior vaginal wall. Approximately half its length is visible with appropriate medical equipment; the remainder lies above the vagina beyond view. It is occasionally called "cervix uteri", or "neck of the uterus".

During menstruation, the cervix stretches open slightly to allow the endometrium to be shed. This stretching is believed to be part of the cramping pain that many women experience. Evidence for this is given by the fact that some women's cramps subside or disappear after their first vaginal birth because the cervical opening has widened.

The portion projecting into the vagina is referred to as the portio vaginalis or **ectocervix**. On average, the ectocervix is three cm long and two and a half cm wide. It has a convex, elliptical surface and is divided into anterior and posterior lips. The ectocervix's opening is called the external os. The size and shape of the external os and the ectocervix varies widely with age, hormonal state, and whether the woman has had a vaginal birth. In women who have not had a vaginal birth the external os appears as a small, circular opening. In women who have had a vaginal birth, the ectocervix appears bulkier and the external os appears wider, more slit-like and gaping.

The passageway between the external os and the uterine cavity is referred to as the **endocervical canal**. It varies widely in length and width, along with the cervix overall. Flattened anterior to posterior, the endocervical canal measures seven to eight mm at its widest in reproductive-aged women. The endocervical canal terminates at the internal os which is the opening of the cervix inside the uterine cavity.

During childbirth, contractions of the uterus will dilate the cervix up to 10 cm in diameter to allow the child to pass through. During orgasm, the cervix convulses and the external os dilates.

### Uterus

The **uterus** is shaped like an upside-down pear, with a thick lining and muscular walls. Located near the floor of the pelvic cavity, it is hollow to allow a blastocyte, or fertilized egg, to implant and grow. It also allows for the inner lining of the uterus to build up until a fertilized egg is implanted, or it is sloughed off during menses.

The uterus contains some of the strongest muscles in the female body. These muscles are able to expand and contract to accommodate a growing fetus and then help push the baby out during labor. These muscles also contract rhythmically during an orgasm in a wave like action. It is thought that this is to help push or guide the sperm up the uterus to the fallopian tubes where fertilization may be possible.

The uterus is only about three inches long and two inches wide, but during pregnancy it changes rapidly and dramatically. The top rim of the uterus is called the fundus and is a landmark for many doctors to track the progress of a pregnancy. The uterine cavity refers to the fundus of the uterus and the body of the uterus.

Helping support the uterus are ligaments that attach from the body of the uterus to the pelvic wall and abdominal wall. During pregnancy the ligaments prolapse due to the growing uterus, but retract after childbirth. In some cases after menopause, they may lose elasticity and uterine prolapse may occur. This can be fixed with surgery.

Some problems of the uterus include uterine fibroids, pelvic pain (including endometriosis, adenomyosis), pelvic relaxation (or prolapse), heavy or abnormal menstrual bleeding, and cancer. It is only after all alternative options have been considered that surgery is recommended in these cases. This surgery is called hysterectomy. Hysterectomy is the removal of the uterus, and may include the removal of one or both of the ovaries. Once performed it is irreversible. After a hysterectomy, many women begin a form of alternate hormone therapy due to the lack of ovaries and hormone production.

![Gray1161.png](../../../upload.wikimedia.org/wikipedia/commons/5/5c/Gray1161.png)

### Fallopian Tubes

At the upper corners of the uterus are the **fallopian tubes**. There are two fallopian tubes, also called the uterine tubes or the oviducts. Each fallopian tube attaches to a side of the uterus and connects to an ovary. They are positioned between the ligaments that support the uterus. The fallopian tubes are about four inches long and about as wide as a piece of spaghetti. Within each tube is a tiny passageway no wider than a sewing needle. At the other end of each fallopian tube is a fringed area that looks like a funnel. This fringed area, called the infundibulum, lies close to the ovary, but is not attached. The ovaries alternately release an egg. When an ovary does ovulate, or release an egg, it is swept into the lumen of the fallopian tube by the fimbriae.

Once the egg is in the fallopian tube, tiny hairs in the tube's lining help push it down the narrow passageway toward the uterus. The oocyte, or developing egg cell, takes four to five days to travel down the length of the fallopian tube. If enough sperm are ejaculated during sexual intercourse and there is an oocyte in the fallopian tube, fertilization will occur. After fertilization occurs, the zygote, or fertilized egg, will continue down to the uterus and implant itself in the uterine wall where it will grow and develop.

If a zygote doesn't move down to the uterus and implants itself in the fallopian tube, it is called a ectopic or tubal pregnancy. If this occurs, the pregnancy will need to be terminated to prevent permanent damage to the fallopian tube, possible hemorrhage and possible death of the mother.

### Mammary glands

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/6/64/Illu_breast_anatomy.jpg/200px-Illu_breast_anatomy.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Cross section of the breast of a human female.

**Mammary glands** are the organs that produce milk for the sustenance of a baby. These exocrine glands are enlarged and modified sweat glands.

#### Structure

The basic components of the mammary gland are the **alveoli** (hollow cavities, a few millimetres large) lined with milk-secreting epithelial cells and surrounded by myoepithelial cells. These alveoli join up to form groups known as **lobules**, and each lobule has a **lactiferous duct** that drains into openings in the nipple. The **myoepithelial** cells can contract, similar to muscle cells, and thereby push the milk from the alveoli through the lactiferous ducts towards the nipple, where it collects in widenings (sinuses) of the ducts. A suckling baby essentially squeezes the milk out of these sinuses.

The development of mammary glands is controlled by hormones. The mammary glands exist in both sexes, but they are rudimentary until puberty when - in response to ovarian hormones - they begin to develop in the female. Estrogen promotes formation, while testosterone inhibits it.

At the time of birth, the baby has lactiferous ducts but no alveoli. Little branching occurs before puberty when ovarian estrogens stimulate branching differentiation of the ducts into spherical masses of cells that will become alveoli. True secretory alveoli only develop in pregnancy, where rising levels of estrogen and progesterone cause further branching and differentiation of the duct cells, together with an increase in adipose tissue and a richer blood flow.

Colostrum is secreted in late pregnancy and for the first few days after giving birth. True milk secretion (lactation) begins a few days later due to a reduction in circulating progesterone and the presence of the hormone prolactin. The suckling of the baby causes the release of the hormone oxytocin which stimulates contraction of the myoepithelial cells.

The cells of mammary glands can easily be induced to grow and multiply by hormones. If this growth runs out of control, cancer results. Almost all instances of breast cancer originate in the lobules or ducts of the mammary glands.

STRUCTURE LOCATION & DESCRIPTION FUNCTION

Breasts
Upper chest one on each side containing alveolar cells (milk production), myoepithelial cells (contract to expel milk), and duct walls (help with extraction of milk).
Lactation milk/nutrition for newborn.

Cervix
The lower narrower portion of the uterus.
During childbirth, contractions of the uterus will dilate the cervix up to 10 cm in diameter to allow the child to pass through. During orgasm, the cervix convulses and the external os dilates

Clitoris
Small erectile organ directly in front of the vestibule.
Sexual excitation, engorged with blood.

Fallopian tubes
Extending upper part of the uterus on either side.
Egg transportation from ovary to uterus (fertilization usually takes place here).

Hymen
Thin membrane that partially covers the vagina in young females.

Labia majora
Outer skin folds that surround the entrance to the vagina.
Lubrication during mating.

Labia minora
Inner skin folds that surround the entrance to the vagina.
Lubrication during mating.

Mons
Mound of skin and underlying fatty tissue, central in lower pelvic region

Ovaries (female gonads)
Pelvic region on either side of the uterus.
Provides an environment for maturation of oocyte. Synthesizes and secretes sex hormones (estrogen and progesterone).

Perineum
Short stretch of skin starting at the bottom of the vulva and extending to the anus.

Urethra
Pelvic cavity above bladder, tilted.
Passage of urine.

Uterus
Center of pelvic cavity.
To house and nourish developing human.

Vagina
Canal about 10-8 cm long going from the cervix to the outside of the body.
Receives penis during mating. Pathway through a womans body for the baby to take during childbirth. Provides the route for the menstrual blood (menses) from the uterus, to leave the body. May hold forms of birth control, such as an IUD, diaphragm, neva ring, or female condom

Vulva
Surround entrance to the reproductive tract.(encompasses all external genitalia)

Endometrium
The innermost layer of uterine wall.
Contains glands that secrete fluids that bathe the utrine lining.

Myometrium
Smooth muscle in uterine wall.
Contracts to help expel the baby.

## The Female Reproductive Cycle

Towards the end of puberty, girls begin to release eggs as part of a monthly period called the female reproductive cycle, or **menstrual cycle** (menstrual referring to "monthly"). Approximately every 28 days, during ovulation, an ovary sends a tiny egg into one of the fallopian tubes. Unless the egg is fertilized by a sperm while in the fallopian in the two to three days following ovulation, the egg dries up and leaves the body about two weeks later through the vagina. This process is called menstruation. Blood and tissues from the inner lining of the uterus (the endometrium) combine to form the menstrual flow, which generally lasts from four to seven days. The first period is called **menarche**. During menstruation arteries that supply the lining of the uterus constrict and capillaries weaken. Blood spilling from the damaged vessels detaches layers of the lining, not all at once but in random patches. Endometrium mucus and blood descending from the uterus, through the liquid creates the menstruation flow.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/f/f0/MenstrualCycle.png/300px-MenstrualCycle.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Menstrual cycle

The reproductive cycle can be divided into an ovarian cycle and a uterine cycle (compare ovarian histology and uterine histology in the diagram on the right). During the **uterine cycle**, the endometrial lining of the uterus builds up under the influence of increasing levels of estrogen (labeled as estradiol in the image). Follicles develop, and within a few days one matures into an **ovum**, or egg. The ovary then releases this egg, at the time of **ovulation**. After ovulation the uterine lining enters a secretory phase, or the **ovarian cycle**, in preparation for implantation, under the influence of progesterone. Progesterone is produced by the corpus luteum (the follicle after ovulation) and enriches the uterus with a thick lining of blood vessels and capillaries so that it can sustain the growing fetus. If fertilization and implantation occur, the embryo produces Human Chorionic Gonadotropin (HCG), which maintains the corpus luteum and causes it to continue producing progesterone until the placenta can take over production of progesterone. Hence, progesterone is "pro gestational" and maintains the uterine lining during all of pregnancy. If fertilization and implantation do not occur the corpus luteum degenerates into a corpus albicans, and progesterone levels fall. This fall in progesterone levels cause the endometrium lining to break down and sluff off through the vagina. This is called menstruation, which marks the low point for estrogen activity and is the starting point of a new cycle.

Common usage refers to menstruation and menses as a period. This bleeding serves as a sign that a woman has not become pregnant. However, this cannot be taken as certainty, as sometimes there is some bleeding in early pregnancy. During the reproductive years, failure to menstruate may provide the first indication to a woman that she may have become pregnant.

Menstruation forms a normal part of a natural cyclic process occurring in healthy women between puberty and the end of the reproductive years. The onset of menstruation, known as **menarche**, occurs at an average age of 12, but is normal anywhere between 8 and 16. Factors such as heredity, diet, and overall health can accelerate or delay the onset of menarche.

**Signs of ovulation**

The female body produces outward signs that can be easily recognized at the time of ovulation. The two main signs are thinning of the cervical mucus and a slight change in body temperature.

**Thinning of the Cervical Mucus**

After menstruation and right before ovulation, a woman will experience an increase of cervical mucus. At first, it will be thick and yellowish in color and will not be very plentiful. Leading up to ovulation, it will become thinner and clearer. On or around the day of ovulation, the cervical mucus will be very thin, clear and stretchy. It can be compared to the consistency of egg whites. This appearance is known as 'spinnbarkeit'.

**Temperature Change**

A woman can also tell the time of ovulation by taking her basal body temperature daily. This is a temperature taken with a very sensitive thermometer first thing in the morning before the woman gets out of bed. The temperature is then tracked to show changes. In the uterine cycle, a normal temperature will be around 97.0 – 98.0. The day of ovulation the temperature spikes down, usually into the 96.0 – 97.0 range and then the next morning it will spike up to normal of around 98.6 and stay in that range until menstruation begins.

Both of these methods are used for conception and contraception. They are more efficient in conception due to the fact that sperm can live for two to three days inside of the fallopian tubes. A woman could be off by a couple of days in her calculations and still become pregnant.

**Menopause** is the physiological cessation of menstrual cycles associated with advancing age. Menopause is sometimes referred to as "the change of life" or climacteric. Menopause occurs as the ovaries stop producing estrogen, causing the reproductive system to gradually shut down. As the body adapts to the changing levels of natural hormones, vasomotor symptoms such as hot flashes and palpitations, psychological symptoms such as increased depression, anxiety, irritability, mood swings and lack of concentration, and atrophic symptoms such as vaginal dryness and urgency of urination appear. Together with these symptoms, the woman may also have increasingly scanty and erratic menstrual periods.

Technically, menopause refers to the cessation of menses; the gradual process through which this occurs, which typically takes a year but may last as little as six months or more than five years, is known as climacteric. A natural or physiological menopause is that which occurs as a part of a woman's normal aging process. However, menopause can be surgically induced by such procedures as hysterectomy.

The average onset of menopause is 50.5 years, but some women enter menopause at a younger age, especially if they have suffered from cancer or another serious illness and undergone chemotherapy. Premature menopause is defined as menopause occurring before the age of 40, and occurs in 1% of women. Other causes of premature menopause include autoimmune disorders, thyroid disease, and diabetes mellitus.

Premature menopause is diagnosed by measuring the levels of follicle stimulating hormone (FSH) and luteinizing hormone (LH). The levels of these hormones will be higher if menopause has occurred. Rates of premature menopause have been found to be significantly higher in both fraternal and identical twins; approximately 5% of twins reach menopause before the age of 40. The reasons for this are not completely understood. Post-menopausal women are at increased risk of osteoporosis.

**Perimenopause** refers to the time preceding menopause, during which the production of hormones such as estrogen and progesterone diminish and become more irregular. During this period fertility diminishes. Menopause is arbitrarily defined as a minimum of twelve months without menstruation. Perimenopause can begin as early as age 35, although it usually begins much later. It can last for a few months or for several years. The duration of perimenopause cannot be predicted in advance.

**Premenstrual Syndrome** (PMS) It is common for women to experience some discomfort in the days leading up to their periods. PMS usually is at its worst the seven days before a period starts and can continue through the end of the period. PMS includes both physical and emotional symptoms: acne, bloating, fatigue, backaches, sore breasts, headaches, constipation, diarrhea, food cravings, depression, irritability, difficulty concentrating or handling stress.

## Ovarian and Uterine Cycles in the Nonpregnant Woman

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/3/39/Ovulation.jpg/220px-Ovulation.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

An ovary about to release an egg.

Ovarian Cycle Events Uterine Cycle Events

Follicular phase - Days 1-13
FSH secretion begins.
Menstruation - Days 2-5
Endometrium breaks down.

Follicle maturation occurs.
Proliferative phase - Days 6-13
Endometrium rebuilds.

Estrogen secretion is prominent.

Ovulation - Day 14*
LH spike occurs.

Luteal phase - Days 15-28
LH secretion continues.
Secretory phase - Days 15-28
Endometrial thickens, and glands are secretory.

Corpus luteum forms.

Progesterone secretion is prominent.

(*)Assuming a 28 day cycle.

There are two phases of the ovarian cycle the follicular phase and the luteal phase. In the follicular phase about 10-25 follicles are taken from preantral or early antrial follicles to develop further. Seven days later the dominant follicle is selected to develop to full maturity. This is the pre-cursor for ovulation. Follicles themselves secrete FSH and estrogen, and these two hormones stimulate follicular growth and development. Ovulation marks the beginning of the luteal phase. This is started by the wall of the Graffian follicle to rupture and cause a flow of antral fluid that will carry the oocyte to the ovary's surface. The ruptured follicle is then turned into a gland (corpus luteum). Which secretes estrogens and progesterone. This is all triggered by and abrupt change in plasma LH levels. After ovulation the released oocyte enters the uterine tube, where it will be either fertilized or discarded.

The uterine cycle operates in sync with the ovarian cycle and is divided into three phases. The first phase in the menstrual phase. It is named the menstrual phase because in corresponds with the shedding the the uterine lining or more commonly called menstruation. The corpus luteum degenerates causing plasma estrogen and progesterone levels to decrease and in turn causes menstruation. Blood vessels in the outer most layer of the endometrium constrict and decrease blood flow to the tissues killing these tissues. After the tissues die they start to separate from the underlying endometrail tissues. Eventually the dead tissue is shed. This shedding of the tissues ruptures blood vessels and causes bleeding. Now we have the proliferative phase. During this phase the uterus renews itself and prepares for pregnancy. The endomitrial tissue that is left after menstruation begins to grow. The endometrial glands grow and enlarge causing more blood vessels. The cervical canal has glands that secrete a thin mucous that helps deposited sperm. Estrogen promotes uterine changes in this phase. The last phase is the secretory phase. This is where the endometrium is transformed to make it the best environment for implantation and subsequent housing and nourishment of the developing embryo. By doing this the endometrium will do things like have an enriched blood supply, begin to secrete fluids rich in glycogen, and even form a plug at the end of the cervical canal so that microorganisms can not enter. These changes in the uterus are caused by progesterone, due to the corpus luteum. At the end of the secretory phase the corpus luteum degenerates, and progesterone levels fall. This will trigger menstruation.

## Sexual Reproduction

**Sexual reproduction** is a type of reproduction that results in increasing genetic diversity of the offspring. In sexual reproduction, genes from two individuals are combined in random ways with each new generation. Sex hormones released into the body by the endocrine system signal the body when it is time to start puberty. The female and male reproductive systems are the only systems so vastly different that each sex has their own different organs. All other systems have "unisex" organs.

Reproduction is characterized by two processes. The first, meiosis, involves the halving of the 46 of chromosomes. The second process, fertilization, leads the fusion of two gametes and the restoration of the original number of chromosomes: 23 chromosomes from the paternal side and 23 from the maternal side. During meiosis, the chromosomes of each pair usually cross over to achieve genetic recombination.

Sexual reproduction cannot happen without the sexual organs called gonads. Both sexes have gonads: in females, the gonads are the ovaries. The female gonads produce female gametes (eggs); the male gonads produce male gametes (sperm). After an egg is fertilized by the sperm, the fertilized egg is called the zygote.

The fertilization usually occurs in the oviducts, but can happen in the uterus itself. The zygote then implants itself in the wall of the uterus, where it begins the processes of embryogenesis and morphogenesis. The womens body carries out this process of reproduction for 40 weeks, until delivery of the fetus from the uterus through the vagina (birth canal). Even after birth, the female continues with the reproduction process by supplying the milk to nourish the infant.

## Infertility

**Infertility** is the inability to naturally conceive a child or the inability to carry a pregnancy to term. There are many reasons why a couple may not be able to conceive without medical assistance. Infertility affects approximately 15% of couples. Roughly 40% of cases involve a male contribution or factor, 40% involve a female factor, and the remaining 20% involve both sexes. Healthy couples in their mid-20s having regular sex have a one-in-four chance of getting pregnant in any given month. This is called "Fecundity".

### Primary vs. secondary

According to the American Society for Reproductive Medicine, infertility affects about 6.1 million people in the United States, equivalent to 10% of the reproductive age population. Female infertility accounts for one third of infertility cases, male infertility for another third, combined male and female infertility for another 15%, and the remainder of cases are "unexplained_._

"Secondary infertility" is difficulty conceiving after already having conceived and carried a normal pregnancy. Apart from various medical conditions (e.g. hormonal), this may come as a result of age and stress felt to provide a sibling for their first child. Technically, secondary infertility is not present if there has been a change of partners.

### Factors of Infertility

Factors relating to female infertility are:

  * General factors 
    * Diabetes mellitus,thyroid disorders,adrenal disease
    * Significant liver,kidney disease
    * Psychological factors
  * Hypothalamic-pituitary factors: 
    * Kallmann syndrome
    * Hypothalamic dysfunction
    * Hyperprolactinemia
    * Hypopituitarism
  * Ovarian factors 
    * Polycystic ovary syndrome
    * Anovulation
    * Diminished ovarian reserve
    * Luteal dysfunction
    * Premature menopause
    * Gonadal dysgenesis (Turner syndrome)
    * Ovarian neoplasm
  * Tubal/peritoneal factors 
    * Endometriosis
    * Pelvic adhesions
    * Pelvic inflammatory disease(PID, usually due to chlamydia)
    * Tubal occlusion
  * Uterine factors 
    * Uterine malformations
    * Uterine fibroids (leiomyoma)
    * Asherman's Syndrome
  * Cervical factors 
    * Cervical stenosis
    * Antisperm antibodies
    * Insufficient cervical mucus (for the travel and survival of sperm)
  * Vaginal factors 
    * Vaginismus
    * Vaginal obstruction
  * Genetic factors 
    * Various intersexuality|intersexed conditions, such as androgen insensitivity syndrome

### Combined Infertility

In some cases, both the man and woman may be infertile or sub-fertile, and the couple's infertility arises from the combination of these factors. In other cases, the cause is suspected to be immunological or genetic; it may be that each partner is independently fertile but the couple cannot conceive together without assistance.

### Unexplained Infertility

In about 15% of cases of infertility, investigation will show no abnormalities. In these cases abnormalities are likely to be present but not detected by current methods. Possible problems could be that the egg is not released at the optimum time for fertilization, that it may not enter the fallopian tube, sperm may not be able to reach the egg, fertilization may fail to occur, transport of the zygote may be disturbed, or implantation fails. It is increasingly recognized that egg quality is of critical importance.

### Diagnosis of Infertility

Diagnosis of infertility begins with a medical history and physical exam. The healthcare provider may order tests, including the following:

  * an endometrial biopsy, which tests the lining of the uterus
  * hormone testing, to measure levels of female hormones
  * laparoscopy, which allows the provider to see the pelvic organs
  * ovulation testing, which detects the release of an egg from the ovary
  * Pap smear, to check for signs of infection
  * pelvic exam, to look for abnormalities or infection
  * a postcoital test, which is done after sex to check for problems with secretions
  * special X-ray tests

### Treatment

  * Fertility medication which stimulates the ovaries to "ripen" and release eggs (e.g. Clomifene|clomifene citrate, which stimulates ovulation)
  * Surgery to restore potency of obstructed fallopian tubes (tuboplasty)
  * Donor insemination which involves the woman being artificially inseminated or artificially inseminated with donor sperm.
  * In vitro fertilization (IVF) in which eggs are removed from the woman, fertilized and then placed in the woman's uterus, bypassing the fallopian tubes. Variations on IVF include: 
    * Use of donor eggs and/or sperm in IVF. This happens when a couple's eggs and/or sperm are unusable, or to avoid passing on a genetic disease.
    * Intracytoplasmic sperm injection (ICSI) in which a single sperm is injected directly into an egg; the fertilized egg is then placed in the woman's uterus as in IVF.
    * Zygote intrafallopian transfer(ZIFT) in which eggs are removed from the woman, fertilized and then placed in the woman's fallopian tubes rather than the uterus.
    * Gamete intrafallopian transfer(GIFT) in which eggs are removed from the woman, and placed in one of the fallopian tubes, along with the man's sperm. This allows fertilization to take place inside the woman's body.
  * Other assisted reproductive technology (ART): 
    * Assisted hatching
    * Fertility preservation
    * Freezing (cryopreservation) of sperm, eggs, & reproductive tissue
    * Frozen embryo transfer (FET)
  * Alternative and complimentary treatments 
    * Acupuncture Recent controlled trials published in Fertility and Sterility have shown acupuncture to increase the success rate of IVF by as much as 60%. Acupuncture was also reported to be effective in the treatment of female anovular infertility, World Health Organization, Acupuncture: Review and Analysis of Reports on Controlled Trials (2002).
    * Diet and supplements
    * Healthy lifestyle

## Types of Birth Control

**Birth control** is a regimen of one or more actions, devices, or medications followed in order to deliberately prevent or reduce the likelihood of a woman becoming pregnant. Methods and intentions typically termed birth control may be considered a pivotal ingredient to family planning. Mechanisms which are intended to reduce the likelihood of the fertilization of an ovum by a sperm may more specifically be referred to as contraception. Contraception differs from abortion in that the former prevents fertilization, while the latter terminates an already established pregnancy. Methods of birth control (e.g. the pill, IUDs, implants, patches, injections, vaginal ring and some others) which may prevent the implantation of an embryo if fertilization occurs are medically considered to be contraception. It is advised to talk with a doctor before choosing a contraceptive. If you have genetics problems or blood conditions, such as factor V leiden, certain contraceptives can be deadly.

Type Procedure Method Effectiveness Risks

Abstinence
Refrain from sexual intercourse
No sperm in vagina
100%
None

Rhythm Method
Intercourse is avoided for about an 8-day span every month in middle of her cycle, from about five days before ovulation to three days after ovulation.
fertilization is only possible during 8-day span in middle of menstrual cycle
70-80%
None

Withdrawal
The man withdraws his penis from the vagina at just the right moment before ejaculation.
sperm are unable to enter vagina if male penis is removed at the right time
70-80%
None

Tubal Ligation (Vasectomy)
Oviducts are cut and tied
No eggs in oviduct
Almost 99%
About 75% Irreversible

Hormonal IUD (intrauterine device)
Flexible, plastic coil inserted by physician
Releases small amounts of estrogen. In most cases, stops egg from developing and being released, but can also operate by killing a fertilized egg by preventing its implantation
About 99%
May cause infections, uterine perforation

Oral Contraceptive
Hormone medication taken daily
Stops release of FSH and LH, but can also operate by killing a fertilized egg by preventing its implantation
More than 90%
Blood clots, especially in smokers

Contraceptive Implants
Tubes of progesterone implanted under the skin
Stops release of FSH and LH, but can also operate by killing a fertilized egg by preventing its implantation
More than 90%
None known

Contraceptive Injections
Injections of hormones
Stops release of FSH and LH, but can also operate by killing a fertilized egg by preventing its implantation
About 99%
Possible osteoporosis

Diaphragm
Latex cup inserted into vagina to cover cervix before intercourse
Blocks entrance of sperm into uterus
With spermicide, about 90%
Latex or spermicide allergy

Cervical Cap
Latex cup held by suction over cervix
Delivers spermicide near cervix
Almost 85%
UTI, latex or spermicide allergy

Female Condom
Polyurethane liner fitted inside vagina
Blocks entrance of sperm into uterus and prevents STD’s
Almost 85%
None

Male Condom
soft sheath, made of latex or animal membrane, encloses penis, trapping ejaculated sperm
Blocks entrance of sperm into vagina and prevents STD's
90%
None

Jellies, Cream, Foams
Spermicidal products inserted before intercourse
Kills large number of sperm
About 75%
UTI, allergy to spermicides

Natural Family Planning
Keep record of ovulation using various methods
Avoid sexual intercourse near ovulation
About 70%
None known

Douche
Vagina cleansed after intercourse
Washes out sperm
Less than 70%
None known

Plan B Pill
Pill taken after intercourse
Prevents release of egg, fertilization of egg, but can also operate by killing a fertilized egg by preventing its implantation
About 89%
Same as oral contraceptive

## Sexually Transmitted Diseases

**Sexually transmitted diseases** (STDs) are diseases or infections that have a significant probability of transmission between humans by means of sexual contact: vaginal intercourse, oral sex, and/or anal sex. Many STDs are (more easily) transmitted through the mucous membranes of the penis, vulva, and (less often) the mouth. The visible membrane covering the head of the penis is a mucous membrane, though, for those who are circumcised it is usually dry and produces no mucus (similar to the lips of the mouth). Mucous membranes differ from skin in that they allow certain pathogens (viruses or bacteria) into the body (more easily).

The probability of transmitting infections through sex is far greater than by more casual means of transmission, such as non-sexual contact—touching, sharing cutlery, and shaking hands. Although mucous membranes exist in the mouth as well as in the genitals, many STDs are more likely to be transmitted through oral sex than through deep kissing. Many infections that are easily transmitted from the mouth to the genitals or from the genitals to the mouth, are much harder to transmit from one mouth to another. With HIV, genital fluids happen to contain a great deal more of the pathogen than saliva. Some infections labeled as STDs can be transmitted by direct skin contact. Herpes simplex and HPV are both examples. Depending on the STD, a person who has has the disease but has no symptoms may or may not be able to spread the infection. For example, a person is much more likely to spread herpes infection when blisters are present than when they are absent. However, a person can spread HIV infection at any time, even if he/she has not developed symptoms of AIDS.

All sexual behaviors that involve contact with the bodily fluids of another person should be considered to hold some risk of transmission of sexually transmitted diseases. Most attention has focused on controlling HIV, which causes AIDS, but each STD presents a different situation.

As may be noted from the name, sexually transmitted diseases are transmitted from one person to another by certain sexual activities rather than being actually caused by those sexual activities. Bacteria, fungi, protozoa or viruses are still the causative agents. It is not possible to catch any sexually transmitted disease from a sexual activity with a person who is not carrying a disease; conversely, a person who has an STD received it from contact (sexual or otherwise) with someone who is infected.

Although the likelihood of transmitting diseases by sexual activities varies a great deal, in general, all sexual activities between two (or more) people should be considered as being a two-way route for the transmission of STDs (i.e. "giving" or "receiving" are both risky).

### Prevention of Sexually Transmitted Diseases

Although healthcare professionals suggest that safer sex, such as the use of condoms, as the most reliable way of decreasing the risk of contracting sexually transmitted diseases during sexual activity, safer sex should by no means be considered an absolute safeguard. The transfer of and exposure to bodily fluids, such as blood transfusions and other blood products, sharing injection needles, needle-stick injuries (when medical staff are inadvertently jabbed or pricked with needles during medical procedures), sharing tattoo needles, and childbirth are all avenues of transmission. These means put certain groups, such as doctors, haemophiliacs and drug users, particularly at risk.

### **Human Papillomavirus (HPV)**

There are over 100 types of this virus which is often asymptomatic. Nearly 3 out of 4 Americans between ages 15 and 49 have been infected. It can be contracted through one partner and remain dormant allowing it to be transmitted to another. Some types can cause cervical cancer.

Genital HPV infection is a sexually transmitted disease that is caused by human papillomavirus. Human papillomavirus is the name of a group of viruses that includes more than 100 different strains. More than 30 of these are sexually transmitted and they can infect the genital area of men and women. Approximately 20 million people are currently infected with HPV and at least 50% of sexually active men and women will acquire HPV at some point in their lives. By age 50 at least 80% of women will have acquired HPV and about 6.2 million Americans get a new HPV infection each year. Most people who have HPV don't know that they are infected. The virus lives in the skin or mucous membranes and usually causes no symptoms. Commonly some people get visible genital warts or have pre-cancerous changes in the cervix, vulva, anus, or penis. Very rarely, HPV results in anal or genital cancers. Genital warts usually appear soft, moist, pink, or flesh colored swellings. They can be raised, flat, single, or multiple, small or large and sometimes cauliflower shaped. Warts may not appear for weeks or months or not at all and the only way to diagnose them is by visible inspection. Most women are diagnosed with HPV on the basis of abnormal pap tests and there are no tests available for men. There is no cure for HPV. The surest way to eliminate risk for HPV is to refrain from any genital contact with another individual. For those who choose to be sexually active, a long term monogamous relationship with an uninfected partner is the strategy most likely to prevent future HPV infections.The next best way to help reduce risk is using a condom but the effectiveness is unknown.

**What is the connection between HPV and cervical cancer?** All types of HPV cause mild pap test abnormalities which do not have serious consequences. Approximately 10 of the 30 identified HPV types can lead to development of cervical cancer. Research as shown that for most women, 90% cervical HPV infection becomes undetectable within two years. Although only a small proportion of women have persistent infection, persistent infection with the high risk types of HPV is the main risk factor for cervical cancer.

A pap test can detect pre-cancerous and cancerous cells on the cervix. Regular pap testing and careful medical follow up, with treatment if necessary, can help ensure that pre-cancerous changes in the cervix caused by HPV infection do not develop into life threatening cervical cancer. The pap test used in the U.S. cervical cancer screening programs is responsible for greatly reducing deaths from cervical cancer.

## **Diseases and Disorders of the Female Reproductive System**

Women are commonly dealing with many different diseases and disorders that pertain to the reproductive system. Here are some of the most common:

  1. **Vulvovaginitis** (pronounced:vul-vo-vah-juh-ni-tus) is an inflammation of the vulva and vagina. It may be caused by irritating substances such as laundry soap, bubble baths or poor hygiene such as wiping from back to front. Symptoms include redness and itching in these areas and sometimes vaginal discharge. It can also be caused by an overgrowth of candida, a fungus normally present in the vagina.
  2. **Nonmenstrual vaginal bleeding** is most commonly due to the presence of a foreign body in the vagina. It may also be due to urethral prolapse, a condition in which the mucous membranes of the urethra protrude into the vagina and forms a tiny, donut shaped mass of tissue that bleeds easily. It can also be due to a straddle injury or vaginal trauma from sexual abuse.
  3. **Ectopic Pregnancy** occurs when a fertilized egg or zygote doesn't travel into the uterus, but instead grows rapidly in the fallopian tube. Women with this condition can develop severe abdominal pain and should see a doctor because surgery may be necessary.
  4. **Ovarian tumors**,although rare, can occur. Women with ovarian tumors may have abdominal pain and masses that can be felt in the abdomen. Surgery may be needed to remove the tumor.
  5. **Ovarian cysts** are noncancerous sacs filled with fluid or semi-solid material. Although they are common and generally harmless, they can become a problem if they grow very large. Large cysts may push on surrounding organs, causing abdominal pain. In most cases, cysts will pass or disappear on their own and treatment is not necessary. If the cysts are painful and occur frequently, a doctor may prescribe birth control pills to alter their growth and occurrences. Surgery is also an option if they need to be removed.
  6. **Polycystic ovary syndrome** is a hormone disorder in which too many hormones are produced by the ovaries. This condition causes the ovaries to become enlarged and develop many fluid filled sacs or cysts. It often first appears during the teen years. Depending on the type and the severity of the condition, it may be treated with drugs to regulate hormone balance and menstruation.
  7. **Trichomonas vaginalis** inflammatory condition of the vagina usually a bacterial infection also called vaginosis.
  8. **Dysmenorrhea** is painful periods.
  9. **Menorrhagia** is when a woman has very heavy periods with excess bleeding.
  10. **Oligomenorrhea** is when a woman misses or has infrequent periods, even though she has been menstruating for a while and is not pregnant.
  11. **Amenorrhea** is when a girl has not started her period by the time she is 16 years old or 3 years after puberty has started, has not developed signs of puberty by 14, or has had normal periods but has stopped menstruating for some reasons other than pregnancy.
  12. **Toxic shock syndrome** is caused by toxins released into the body during a type of bacterial infection that is more likely to develop if a tampon is left in too long. It can produce high fever, diarrhea, vomiting, and shock.
  13. **Candidasis** symptoms of yeast infections include itching, burning and discharge. Yeast organisms are always present in all people, but are usually prevented from "overgrowth" (uncontrolled multiplication resulting in symptoms) by naturally occurring microorganisms.

At least three quarters of all women will experience candidiasis at some point in their lives. The Candida albicans organism is found in the vaginas of almost all women and normally causes no problems. However, when it gets out of balance with the other "normal flora," such as lactobacilli (which can also be harmed by using douches), an overgrowth of yeast can result in noticeable symptoms. Pregnancy, the use of oral contraceptives, engaging in vaginal sex after anal sex in an unhygienic manner, and using lubricants containing glycerin have been found to be causally related to yeast infections. Diabetes mellitus and the use of antibiotics are also linked to an increased incidence of yeast infections. Candidiasis can be sexually transmitted between partners. Diet has been found to be the cause in some animals. Hormone Replacement Therapy and Infertility Treatment may be factors.

There are also cancer's of the female reproductive system, such as:

  1. Cervical cancer
  2. Ovarian cancer
  3. Uterine cancer
  4. Breast cancer

**Endometriosis**

Endometriosis is the most common gynecological diseases, affecting more than 5.5 million women in North America alone! The two most common symptoms are pain and infertility. In this disease a specialized type of tissue that normally lines the inside of the uterus,(the endometrium) becomes implanted outside the uterus, most commonly on the fallopian tubes, ovaries, or the tissue lining the pelvis. During the menstrual cycle, hormones signal the lining of the uterus to thicken to prepare for possible pregnancy. If a pregnancy doesn't occur, the hormone levels decrease, causing the thickened lining to shed.

When endometrial tissue is located in other parts it continues to act in it's normal way: It thickens, breaks down and bleeds each month as the hormone levels rise and fall. However, because there's nowhere for the blood from this mislocated tissue to exit the body, it becomes trapped and surrounding tissue becomes irritated. Trapped blood may lead to growth of cysts. Cysts in turn may form scar tissue and adhesions. This causes pain in the area of the misplaced tissue, usually the pelvis. Endometriosis can cause fertility problems. In fact, scars and adhesions on the ovaries or fallopian tubes can prevent pregnancy. Endometriosis can be mild, moderate or severe and tends to get worse over time without treatment. The most common symptoms are:

  1. **Painful periods** Pelvic pain and severe cramping, intense back pain and abdominal pain.
  2. **Pain at other times** Women may experience pelvic pain during ovulation, sharp deep pain in pelvis during intercourse, or pain during bowel movements or urination.
  3. **Excessive bleeding** Heavy periods or bleeding between periods.
  4. **Infertility** Approximately 30-40% of women

The cause of endometriosis remains mysterious. Scientists are studying the roles that hormones and the immune system play in this condition. One theory holds that menstrual blood containing endometrial cells flows back through the fallopian tubes, takes root and grows. Another hypothesis proposes that the bloodstream carries endometrial cells to other sites in the body. Still another theory speculates that a predisposition toward endometriosis may be carried in the genes of certain families.

Other researchers believe that certain cells present within the abdomen in some women retain their ability to specialize into endometrial cells. These same cells were responsible for the growth of the woman's reproductive organs when she was an embryo. It is believed that genetic or environmental influences in later life allow these cells to give rise to endometrial tissue outside the uterus.

Experts estimate that up to one in ten American women of childbearing age have endometriosis. There is some thinking that previous damage to cells that line the pelvis can lead to endometriosis. There are several ways to diagnose endometriosis:

  1. **Pelvic exam**
  2. **Ultrasound**
  3. **Laparoscopy** Usually used, most correct diagnosis
  4. **Blood test**

Endometriosis can be treated with:

  1. **Pain medication**
  2. **Hormone therapy**
    1. **Oral contraceptives**
    2. **Gonadotropin-releasing hormone(Gn-Rh)agonists and antagonists**
    3. **Danazol(Danocrine)**
    4. **Medroxyprogesterone(Depo-Provera)**
  3. **Conservative surgery** which removes endometrial growths.
  4. **Hysterectomy**

## Check Your Understanding

Answers for these questions can be found [here](http://en.wikibooks.org/wiki/Human_Physiology/Appendix_1:_answers_to_review_questions#The_female_reproductive_system)

1\. In homology, the ___________ in the female is equal to the penis in the male

    A) labia majora
    B) clitoral hood
    C) clitoris
    D) labia minora
    E) none of the above

2\. This contains some of the strongest muscles in the human body

    A) uterus
    B) clitoris
    C) cervix
    D) labia majora

3\. This protects the vaginal and urethral openings

    A) labia majora
    B) labia minora
    C) clitoris
    D) urethra

4\. Sally has noticed that her cervical mucus has changed and now resembles egg whites- from this Sally could assume

    A) her period will begin soon
    B) nothing, this is a normal occurrence
    C) she has a yeast infection
    D) she is ovulating

5\. Debbie recently went to the OBGYN and was diagnosed with PCOD (polycystic ovary syndrome) because of this she has

    A) nothing, its normal in women
    B) antisperm antibodies
    C) an overproduction of LH
    D) leaking of milk from her mammary glands
    E) problems becoming pregnant

6\. Angie went to the doctor because she has had pain in her leg recently- this could be caused by

    A) ovulation pain
    B) her period that will be starting tomorrow
    C) premenstrual syndrome
    D) a blood clot resulting from her birth control pill

7\. Sue recently started her period and has noticed that they are very heavy and painful, and that they are inconsistent in their timing. One explanation could be

    A) endometriosis
    B) ovarian cancer
    C) candidasis
    D) toxic shock syndrome
    E) amenorrhea

8\. Mary is getting married and is not ready to become a mother- she chooses this birth control because of its high effectiveness

    A) natural family planning
    B) a diaphragm
    C) contraceptive injections
    D) a spermicide foam

9\. The release of LH in woman causes

    A) menstration
    B) ovulation
    C) increase of endometrial lining
    D) decrease of endometrial lining
    E) nothing LH only does something in the male reproductive system

10\. When the ovaries stop producing estrogen, this occurs

    A) ovulation
    B) implantation
    C) premenstrual syndrome
    D) menopause

11\. Infertility affects what percentage of couples?

    A) 5%
    B) 10%
    C) 15%
    D) 20%

12\. What is the only 100% effective form of birth control?

    A) Tubal ligation
    B) IUD
    C) Natural family planning
    D) Abstinence

## Glossary

**Adhesions**: Abnormal tissue that binds organs together

**Alveoli**: Basic components of the mammary glands; lined with milk-secreting epithelial cells

**Birth Control**: regimen of one or more actions, devices, or medications followed in order to deliberately prevent or reduce the likelihood of a woman becoming pregnant

**Cervical Mucus**: Mucus secreted by the cervix, near ovulation it helps to lower the acidity of the vagina

**Cervix**: Lower, narrow portion of the uterus where it joins with the top of the vagina

**Clitoris**: Small body of spongy tissue that functions solely for sexual pleasure

**Chromosomes**: Structures in the nucleus that contain the genes for genetic expression

**Ectocervix**: Portion of the cervix projecting into vagina

**Endocervical Canal**: Passageway between the external os and the uterine cavity

**Endometrium**: The inner lining of the uterus

**Fallopian Tubes**: Located at the upper end of the vagina, passage way for the egg from the ovary

**Factor V Leiden**: This is the name given to a variant of human factor V that causes a hypercoagulability disorder. In this disorder the Leiden variant of factor V, cannot be inactivated by activated protein C. Factor V Leiden is the most common hereditary hypercoagulability disorder amongst Eurasians. It is named after the city Leiden (The Netherlands), where it was first identified in 1994 by Prof R. Bertina et al.

**Gamete**: A haploid sex cell; either an egg cell or a sperm cell

**Gene**: That portion of the DNA of a chromosome containing the information needed to synthesize a particular protein molecule

**Gonad**: A reproductive organ, testis or ovary that produces gametes and sex hormones

**Hormone**: A chemical substance produced in an endocrine gland and secreted into the bloodstream that acts on target cells to produce a specific effect

**Hymen**: Thin fold of mucous membrane that separates the lumen of the vagina from the urethral sinus

**Infertility**: Inability to naturally conceive a child or the inability to carry a pregnancy to term

**Labia Majora**: Outer "lips" of the vulva, made of loose connective tissue and adipose tissue with some smooth muscle

**Labia Minora**: Inner lips of the vulva, folds and protects the vagina, urethra and clitoris

**Mammary Glands**: Organs that produce milk for the sustenance of a baby

**Meiosis**: A specialized type of cell division by which gametes, or haploid sex cells, are formed

**Menarche**: The first menstrual discharge; occurs normally between the ages of 9 and 17

**Menopause**: The period marked by the cessation of menstrual periods in the human female

**Menstrual Cycle**: The rhythmic female reproductive cycle characterized by physical changes in the uterine lining

**Menstruation**: The discharge of blood and tissue from the uterus at the end of menstrual cycle

**Mittelschmerz**: Pain near the lower abdomen site at the time of ovulation; German for ovulation pain

**Mons Veneris**: soft mound at the front of the vulva (fatty tissue covering the pubic bone)

**Ovarian Cycle**: Last phase of the reproductive cycle; if no implantation occurs, causes the breakdown of the endometrial lining and causes menstruation

**Ovulation**: The rupture of an ovarian follicle with the release of an ovum

**Perineum**: External region between the scrotum and the anus in a male or between the vulva and anus in a female

**Premenstrual Syndrome (PMS)**: Time leading up to menstruation; includes both physical and emotional symptoms: acne, bloating, fatigue, backaches, sore breasts, headaches, constipation, diarrhea, food cravings, depression, irritability, difficulty concentrating or handling stress

**Puberty**: The period of development in which the reproductive organs become functional and the secondary sex characteristics are expressed

**Reproduction**: Process by which an organism continues its species

**Sexually transmitted diseases (STDs)**: diseases or infections that have a significant probability of transmission between humans by means of sexual contact

  
**Urethra**: Located below the clitoris, used for the passage of urine

**Uterine Cycle**: First part of the reproductive cycle; the time when the endrometrial lining builds up and follicles develop

**Uterus**: Major reproductive organ, receives fertilized eggs which become implanted in the lining, the lining (endometrium) provides nourishment to developing fetus; contains some of the strongest muscles in the female body and is able to stretch during fetus development

**Vagina**: Muscular, hollow tube that extends from the vaginal opening to the cervix

**Vulva**: External female genitals, includes labia majora, labia minora, mons pubis, clitoris, meatus, vaginal vestibule, vestibule bulbs and vestibular glands

## References

  * Essentials of Anatomy and Physiology. Fourth Edition. Valerie C. Scanlon and Tina Sanders.
  * Human Anatomy. Sixth Edition. Van De Graaff.
  * Wikibook: [Sexual Health](http://en.wikibooks.org/wiki/Sexual_Health)
  * <http://www.fda.gov/cder/drug/infopage/planBQandAhtm>
  * <http://www.goplanb.com/Forconsumers>
  * American Social Health Association;ashastd.org
  * <http://www.cdc.gov>
  * <http://www.mayoclinic.com>

  


# Pregnancy and Birth

## Introduction

In this chapter we will discuss the topics covering pregnancy, from conception to birth. The chapter will cover fertilization, implantation of the zygote, to becoming a fetus, the three trimesters, and the progressive development of the fetus through the weeks of pregnancy. It will cover the topic of birth and different birthing methods.

## Fertilization

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/8/86/Sperm-egg.jpg/300px-Sperm-egg.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

A sperm fertilizing an ovum

Fertilization is the joining of a sperm and an egg. A sperm is a male gamete that is released into the vagina of a female during intercourse. In order for fertilization to occur there must be a mature ovum present. Every month one of the ovaries releases an egg which will meet one of the A 4 million sperm the male ejaculates into the vagina. The sperm swim through the cervix and into the uterus which lead to the fallopian tubes. This is where fertilization is most likely to take place. The high amount of sperm in the ejaculate is needed because only around 100 survive to enter reach the fertilization site. In order to penetrate the egg the sperm must first break through 2 barriers surrounding the ovum. The acrosome of sperm comes in contact with the corona radiata and releases digestive enzymes that break down a gelatinous layer around the egg called, the zona pellucida. Once a sperm reaches the plasma membrane of the egg it sets off a reaction that spreads across the membrane of the egg preventing other sperm from breaking through the egg membrane. Once the sperm reaches the inside of the egg it sheds its tail and the two nuclei fuse and now the 23 chromosomes from the egg and the 23 chromosomes of the sperm join and they become a _zygote_. Chromosomes contain all the information needed to determine the genetic structure of the new baby. Normally all human beings have two chromosomes that determine sex: A combination of X and Y makes a male or a combination of X and X makes a female. All ovum have X sex chromosomes where as sperm have both X or Y sex chromosomes. Therefore, the male gametes determine the sex of the baby.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/6/6b/Embryo,_8_cells.jpg/300px-Embryo,_8_cells.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

An 8-cell embryo in the process of cleavage.

## Pre-embryonic Period

After fertilization, the zygote begins a process of dividing by _mitosis_ in a process called _cleavage_. It divides until it reaches 16 cells. It is now referred to as a _morula_. As the morula floats freely within the uterus, it starts to bring nutrients into the cells. The morula fills with fluid and the cells inside start to form two separate groups. At this stage it is now a _blastocyst_. The inner layer of cells is called the embryoblast, and will become the fetus. The outer layer is called a trophoblast which will develop into part of the placenta. At this point the zona pellucida is disintegrating. The trophoblast contains specialized cells that become extensions, like fingers, that grow into the endometrium once in contact with the well thickened endometrium.

### Implantation

The blastocyst preserves itself by secreting a hormone that indirectly stops menstruation. The trophoblast cells secrete hCG hormones that help maintain the corpus luteum that would normally regress. In turn, the corpus luteum continues to secrete progesterone, which maintains the endometrium of the uterus in the secretory phase. This helps the blastocyst to continue to grow and stay embedded within the endometrium. The fetal life support system and the placenta begin to form, and eventually the placenta will take over the job of producing progesterone.

  * Gastrulation and Formation

The embryoblast within the blastocyst forms 3 primary germs layers: ectoderm, mesoderm, and endoderm.

#### Ectoderm

This forms the nervous tissue and the epithelium covering the outer body surface. Epidermis of skin, including hair and nails, glands of skin, linings of oral cavity, nasal cavity, anal canal, vagina, brain, spinal cord, sensory organs, lens of eye and epithelium of conjunctiva (a membrane that covers the sclera and lines the inside of the eyelids), pituitary gland, adrenal medulla, and enamel of teeth.

#### Mesoderm

This forms all of the muscle tissue and the connective tissue of the body, as well as the kidneys and the epithelium of the serous membranes and blood vessels. All muscle tissue (skeletal, smooth, cardiac), all connective tissue (fibrous connective tissue, bone, blood, cartilage), dentin of teeth, adrenal cortex, kidneys and ureters, internal reproductive viscera, epithelium lining vessels, joint cavities, and the serous body cavities.

#### Endoderm

Forms the lining epithelium and glands of the visceral body systems. Lining epithelium and glands of digestive, respiratory, and parts of urogenital systems, thyroid and parathyroid glands, and thymus.

## Formation of Placenta

As changes to the endometrium occur, cellular growth and the accumulation of glycogen cause fetal and maternal tissue to come together. This formation makes the functional unit called the placenta. The placenta does not mix blood between mother and fetus, but allows nutrients and waste products to diffuse between the two blood systems. The placenta provides protection by filtering out many harmful substances that the mother comes in contact with. Unfortunately the placenta cannot protect against some teratogens including but not limited to:

  * Thalidomide
  * Heroin
  * Cocaine
  * Aspirin
  * Alcohol
  * Chemicals in cigarette smoke
  * Propecia, also known as Finasteride, which can cause birth defects simply by a woman handling a broken pill during pregnancy.

## Amniotic Fluid

Attached to placenta is the membranous sac which surrounds and protects the embryo. This sac is called the amnion. It grows and begins to fill, mainly with water, around two weeks after fertilization. This liquid is called Amniotic fluid, it allows the fetus to move freely, without the walls of the uterus being too tight against its body. Buoyancy is also provided here for comfort. After a further 10 weeks the liquid contains proteins, carbohydrates, lipids and phospholipids, urea and electrolytes, all which aid in the growth of the fetus. In the late stages of gestation much of the amniotic fluid consists of fetal urine. The fetus swallows the fluid and then voids it to prepare its digestive organs for use after birth. The fetus also "breathes" the fluid to aid in lung growth and development.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/e/e4/Fetus.jpg/220px-Fetus.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

A small part of the placenta is shown at the bottom, while the fluid-filled amnion surrounds it

Not enough amniontic fluid, or oligohydramnios, can be a concern during pregnancy. Oligohydramnios can be caused by infection, kidney dysfunction or malformation (since much of the late amniotic fluid volume is urine), procedures such as chorionic villus sampling (CVS), and preterm, premature rupture of membranes (PPROM). One possible outcome of oligohydramnios can cause is underdeveloped, or hypoplastic, lungs. This condition is potentially fatal and the baby can die shortly after birth. Babies with too little amniotic fluid can also develop contractures of the limbs, including clubbing of the feet and hands.

As with too little fluid, too much fluid or polyhydramnios, can be a cause or an indicator of problems for the mother and baby. Polyhydramnios is a predisposing risk factor for cord prolapse and is sometimes a side effect of a macrosomic pregnancy. In both cases, however, the majority of pregnancies proceed normally and the baby is born healthy.

Preterm, premature rupture of membranes (PPROM) is a condition where the amniotic sac leaks fluid before 38 weeks of gestation. This can be caused by a bacterial infection or by a defect in the structure of the amniotic sac, uterus, or cervix. In some cases the leak can spontaneously heal, but in most cases of PPROM, labor begins within 48 hours of membrane rupture. When this occurs, it is necessary that the mother receive treatment immediately to postpone labor if the fetus is not viable, for as long as is safe, and for antibiotic treatments to avoid possible infection in the mother and baby. If rupture occurs too early in pregnancy little can be done to save the fetus.

A very rare and most often fatal obstetric complication is an amniotic fluid embolism, or leakage of amniotic fluid into the mothers vascular systems causing an allergic reation. This allergic reaction results in cardiorespiratory (heart and lung) collapse, developing into a condition known as disseminated intravascular coagulation in which the mothers blood looses it's ability to clot.

Amniotic band syndrome, or ABS, occurs when the inner fetal membrane (amnion) ruptures without injury to the outer membrane (chorion). Fibrous bands from the ruptured amnion float in the amniotic fluid and can entangle the fetus, reducing blood supply and causing congenital limb abnormalities dysmelia. In some cases a complete "natural" amputation of a digit(s) or limb may occur before birth or the digit(s) or limbs may be necrotic (dead) requiring surgical removal.

## Endocrine Function of the Placenta

There are pituitary like hormones and steroid hormones secreted from the placenta. The pituitary like hormones are hCG and hCS. HCG is similar to LH and helps maintain the mothers corpus luteum. HCS is like prolactin and growth hormone and help aid in increasing fat breakdown that spares the use of glucose from the mothers tissues. This effect leaves more glucose available to the placenta and the fetus for necessary growth. The steroid hormones are progesterone and estrogen. Progesterone helps maintain the endrometrium and supports the growth of mammary glands. Estrogen also helps maintain the endrometrium and growth of mammary glands as well as inhibits prolactin secretion.

## Developing Baby

The womb is expanding, the baby is growing and taking all the nourishment from the mother. What once started as a microscopic two-celled egg, will be formed into a baby in just 12 weeks. The baby develops from conception to term, in a month-to-month progress.

### Overview of Developmental Milestones

WEEK CHANGES IN MOTHER DEVELOPMENT OF BABY

**Pre-embryonic Development**

1 week
Ovulation Occurs
Fertilization occurs, cell division begins and continues, chorion appears

**Embryonic Development**

2 weeks
Symptoms of early pregnancy (nausea, breast swelling and tenderness, fatigue); blood pregnancy tests may show positive
Implantation occurs; amnion and yolk sac appear; embryo has tissue; placenta begins to form

3 weeks
First period missed; urine pregnancy test may show positive; early pregnancy symptoms continue
Nervous system begins to develop; allantois and blood vessels are present and placenta is well formed

4 weeks
Limb buds form; heart is beating; nervous system further develops; embryo has tail; other systems are forming

5 weeks
Uterus is the size of a hen's egg; mother may need to urinate frequently
Embryo is curved, head is large, limb buds are showing division, nose, ears and eyes are noticeable

6 weeks
Uterus is the size of an orange
Fingers and toes are present and skeleton is cartilaginous

8 weeks
Uterus can be felt above the pubic bone
Fetus begins to look human; limbs are developing and major organs forming; facial features are becoming refined

**Fetal Development**

12 weeks
Uterus is the size of a grapefruit
Head grows faster than the rest of the body; facial features are apparent, but there is no layer of fat yet and the skin is translucent; gender can be distinguished via ultrasound; fingernails appear

16 weeks
Fetal movement can be felt
Fine hair (lanugo) grows over the body; fetus resembles a tiny human being; skeleton is visible

20-22 weeks
Uterus reaches up to the level of umbilicus and pregnancy is obvious
Vernix caseosa, the protective fatty coating, begins to be deposited; heartbeat can be heard

24 weeks
Doctor can tell where baby's head, back and limbs are; breasts have enlarged and nipples and areola are darker, colostrum is produced
Fully formed but still thin; much larger and very active, all major organs are working, the lungs and digestive system need more time to develop; body is covered in fine hair called lanugo

32 weeks
Uterus reaches halfway between umbilicus and rib cage
Most babies are in a head down position in the womb; head is more in proportion to the body; eyes are open; babies born at this stage have a good chance of living

36 weeks
Weight gain is averaging about a pound a week; standing and walking are becoming very difficult because the center of gravity is thrown forward
Body hair begins to disappear, fat is being deposited

40 weeks
Uterus is up to the rib cage, causing shortness of breath and heartburn; sleeping is very difficult
Not much room to move in the womb; fully mature, baby moves less, and the surrounding fluid reduces and the womb expands its limits

## Embryonic Development at Specific Stages

### First trimester

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/2/21/Tubal_Pregnancy_with_embryo.jpg/300px-Tubal_Pregnancy_with_embryo.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

An embryo this tiny shows very distinct anatomic features, including tail, limb buds, heart (which actually protrudes from the chest), eye cups, cornea/lens, brain, and prominent segmentation into somites. The gestational sac is surrounded by a myriad of chorionic villi resembling elongate party balloons. This embryo is about five weeks old (or seven weeks in the biologically misleading but eminently practical dating system used in obstetrics).

_**4 Weeks**_

  * There are only the beginnings of facial features. All the major organs are starting to form. Gill-like folds that develop into facial features, beginnings of the spinal cord, skin is translucent, and rudimentary (basic; minimal) heart develops.

_**6 Weeks**_

  * The length from crown to rump is about the size of a finger tip, ¾ ". The beginnings of all the major organs will have formed.
  * The embryo floats in a fluid filled bubble that will develop into the amniotic sac. The sac is covered by a protective layer of cells, called chorion. The yolk sac supplies the embryo with all its nutrients until the placenta is fully developed and takes over at around the twelfth week. During the first 12 weeks, the embryo will develop features and major organs of a human being. The embryo is susceptible to harmful environmental influences. This is a vital time for the embryo to develop healthily; taking supplements of folic acid, avoiding certain foods, and eliminating alcohol, cigarettes, and any unnecessary drugs or medicines.

_**9 Weeks**_

  * The length from crown to rump approximately 1 1/4". The facial features are becoming more distinct, and the “tail” has disappeared. The muscles are also developing. Eyes are formed but eyelids are still closed over them. Arms now bend at the elbow and rudimentary hands and fingers develop. Knees will have formed and developing feet with distinct toes.
  * Heart- is now a four-chambered and fully formed organ; it beats about 180 times per minute.
  * Brain and nervous system- is four times the size it was at 6 weeks. Special glial cells are being formed within the neural tube; they allow nerve cells to be joined so that messages can be transmitted from the brain to the body.
  * Digestive system- the mouth, intestine, and stomach are developing very rapidly, but do not function yet.
  * The fetal life-support system- the placental tissue that initially surrounds the fetus and the amniotic sac is becoming concentrated in one circular area on the womb wall to form the placenta.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/f/f3/Embryo_at_14_weeks_profile.JPG/220px-Embryo_at_14_weeks_profile.JPG)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Sonogram of a fetus at 14 weeks (Profile)

  
_**12 Weeks**_

  * At twelve weeks the fetus looks like a tiny human. It is about 2 1/2" long and weighs 1/2 oz. Arms and legs are now beginning to move. Skin is red and translucent. Fingers and toes are more defined, and nails are starting to grow.
  * Heart is complete and working, pumping blood to all parts of the body. Digestive system has formed and is linked to the mouth and intestines. Sexual organs have formed inside the body, but cannot yet establish the sex of the baby.

### Second Trimester

_**20 Weeks**_

  * By 20 weeks the fetus will be about 6 1/3" long and weighs 12 oz. Movements are for more coordinated. The sexual organs are well developed and are usually visible on ultra sound.
  * The fetus is growing very quickly. At this stage, the mother should feel the movements of the fetus. Movements are more noticeable as the fetus's leg bones achieve their final relative proportions in a process called _quickening_. Quickening is the process of muscles contracting that cause movement at the fetus's synovial joints. The joint movement enhances the nutrition of the articular cartilage and prevents the fusion of connective tissues within the joint. It also promotes bone hardening.
  * From now on, the fully developed placenta will provide all the fetus' needs until birth; oxygen, nutrients and protective antibodies.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/3/39/3dultrasound.png/220px-3dultrasound.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Fetus at 29 weeks gestation in 3D

### Third Trimester

_**29 Weeks**_

  * By 29 weeks the baby is about 10" long and weighs about 2 lbs. 7 oz.
  * The brain grows much larger, and fatty protective sheath covers the nerve fibers; this important development allows brain impulses to travel faster, enhancing the ability to learn. The lungs have developed most of their airways and air sacs. The placenta is quite selective in what it allows to pass from the mother to the baby's blood, stopping some harmful substances, such as certain drugs, from crossing over.

_**40 Weeks**_

  * The baby is now ready to be born. When the head of the baby moves down from high in the mother's abdomen and settles deeper into her pelvis in preparation for birth, it is called engagement. This can happen any time between 36 weeks and labor.
  * In the last four weeks of pregnancy the baby puts on a lot of weight and develops a thick layer of fat. All organs are completely formed and functioning.

### Umbilical Cord

This is the life support for a growing embryo. The umbilical cord stretches between the placenta and the fetus. This cord contains the umbilical arteries and vein. The umbilical cord forms by week 5 of conception. The average cord is close to 22 inches long and may have the appearance of a coil. The umbilical cord is very rich in stem cells and is often used for parents who choose to store their stem cells in a blood bank or donate it to a blood bank. These stem cells can be used to treat over 45 disorders and is an alternative from extracting the stem cells from a donor.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/a/a8/Human_placenta_baby_side.jpg/350px-Human_placenta_baby_side.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Human placenta shown a few minutes after birth. The side shown faces the baby with the umbilical cord top right. The unseen side connects to the uterine wall. The white fringe surrounding the bottom is the remnants of the amniotic sac. You can see the differences in the umbilical vein and arteries.

  * _Umbilical Arteries_

The exchange of gases, nutrients and oxygen takes place between the maternal blood and fetal blood. There are 2 main arteries.

  * _Umbilical Vein_

Vein that carries nutrients and oxygen away from the placenta to the growing fetus. It also carries oxygen and nutrient rich blood. There is only 1 main vein.

  * Fetus doesn't use its lungs for gas exchange, only a small amount of blood is pumped to fetal lungs in order to support their development.

#### Umbilical Abnormalities

  * _Single Umbilical Artery_

One artery instead of two will result in chromosomal abnormalities. Some of these defects include poor fetal growth, preterm delivery, and still births. This can be detected by a routine ultrasound. If an ultrasound is done and no other complications or abnormalities are detected, the baby will usually be born healthy.

  * _Umbilical Prolapse_

This condition usually happens when a cord is too long. The baby may be born prematurely or will be breech.

  * _Umbilical Nuchal Loops_

This condition happens when the umbilical cord is wrapped around the baby's head at least one or more times. This can be detected when a baby is in stress or by a simple ultrasound. In most cases the mother will have a cesarean delivery. In other cases the cord may be wrapped around the hands or feet.

  * _Vasa Previa_

This occurs in one in every 3,000 births, which can become life threatening for the unborn baby. This complication happens when the umbilical cord inserts abnormally in the fetal membranes of the placenta, which appears abnormally shaped or positioned. Major risks include unprotected fetal blood vessels cross the cervix, oftentimes rupturing the membranes. Also, lack of blood pressure due from pressure, causes the loss of oxygen to the baby. Women who will be at risk for this would be those who already have experienced placenta previa or have used in vitro fertilization.

  * _Umbilical Cord Knots_

About 1% of babies are born with one or more knots in their umbilical cord. Some knots happen during labor; others happen from moving around in the womb. Most knots occur when the umbilical cord is too long. In some cases the knots can become tight, cutting off the oxygen supply to the baby. Cord knots result in miscarriages and stillbirth in 5% and 10% of most cases. Most will require a cesarean delivery.

  * _Umbilical Clotting_

This is more common with genetic defects, such as Factor V Leiden. This complication will prevent blood flow to and from the baby and many times will cause the placenta to also clot and die. If this is not caught early enough, the baby will die of starvation in the womb. A simple ultrasound can determine if there are problems with the blood flow.

## Pregnancy from the mother's perspective

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/8/87/Grosse_derGebarmutter_in_den_einzelneMonaten_derSchwangerschaft.gif/220px-Grosse_derGebarmutter_in_den_einzelneMonaten_derSchwangerschaft.gif)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Growth of the uterus in a pregnant female.

An initial sign of pregnancy is amenorrhea, or the absence of menstruation. Menses cease because the blastocyte begins the release of hCG or human chorionic gonadotropin. Most pregnancy tests are specifically designed to recognize the presence of hCG, and hCG levels can be tested through the mothers blood to learn whether or not a pregnancy is progressing normally.

Human pregnancy lasts approximately 40 weeks from the time of the last menstrual cycle to childbirth (38 weeks from fertilization). The medical term for a pregnant woman is genetalian, just as the medical term for the potential baby is embryo (early weeks) and then fetus (until birth). A woman who is pregnant for the first time is known as a primigravida or gravida 1: a woman who has never been pregnant is known as a gravida 0; similarly, the terms para 0, para 1 and so on are used for the number of times a woman has given birth.

In many societies' medical and legal definitions, human pregnancy is somewhat arbitrarily divided into three trimester periods, as a means to simplify reference to the different stages of fetal development. The first trimester period carries the highest risk of miscarriage (spontaneous death of embryo or fetus). During the second trimester the development of the fetus can start to be monitored and diagnosed. The third trimester marks the beginning of viability, which means the fetus might survive if an early birth occurs.

### Changing Body

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/f/f1/Placenta.svg/250px-Placenta.svg.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

(38 weeks) A fully developed fetus in the mothers abdomen

As soon as a woman becomes pregnant, her body begins to change so that it can support both herself and the unborn baby. All of the body functions start to work much harder. The heart has to pump more blood around the body, in particular to the womb, placenta, and the fetus. As well as physical demands, pregnancy also causes a range of emotional reactions.

  * The first trimester, the first twelve weeks, little is visible.
  * The second trimester, 13-27 Weeks, the waistline is rapidly growing, the abdomen becomes noticeably pregnant.
  * The third trimester, 28-40 weeks, the body expands rapidly and the womb enlarges and presses against the diaphragm.

#### First Trimester

In the early weeks the mother is likely to be more tired. As the uterus begins to grow, the "bump" becomes noticeable. This is a good time to start looking into options on birthing and doctors.

  * Physical feelings: tiredness, nausea, constipation, frequent urination, food cravings, change in size of breasts, fainting or dizziness, bloated stomach, and high emotions.

#### Second Trimester

The mother will probably be feeling full of energy and excitement.

    

  * Physical feelings: More energy, constipation, heartburn, and indigestion. The breasts continue to grow, as does an increase in appetite. There is mild swelling in the feet, ankles, hands, and face. There is also more baby movement. There may be emotional ups and downs in the feeling of pregnancy, and short-term memory may be poor.
  * The hormones estrogen, progesterone, human placental lactogen, oxytocin, and prolactin prepare the body for feeding the baby, and cause the breasts to enlarge, becoming painful and tender.
  * The fetus, placenta, and amniotic fluid account for just over a third of the weight gain during pregnancy. The remaining weight comes from increased blood volume, fluid retention, and extra body fat. The suggested weight gain in most pregnancies is between 25-40 lbs.

#### Third Trimester

Physical feelings
    Shortness of breath, tiredness, difficulty in moving and sleeping, and frequent urination. The emotional mood swings ease off, but the mother begins to feel less enthusiastic about being pregnant. She may become impatient and restless and just wants for the birth to be over. 

  * The body is changing to cope with the ever increasing size of the womb. The baby grows and pushes out the lower back of the mother. The breathing rate of the baby is growing very quickly. At this stage, the mother should feel the movements of the fetus. Other signs may be the nipples secreting colostrum, Braxton-Hicks' contractions may begin, and blood flow to the womb has increased tenfold since conception.

### Prenatal Care

Once the female confirms her pregnancy, she will need to find out her physical condition and what to expect in the coming months. Women typically begin pre-natal care at approximately 8-10 weeks gestation, and pregnancy care should continue until approximately 6 weeks postpartum. The main purpose of the prenatal visits is to perform preventative medicine. Most complications in pregnancy are best treated if they are caught early on. A series of tests will be performed throughout the pregnancy to judge the mother and fetus' well-being including:

  * Mother's history
  * Urine tests for glucose, protein, and infection
  * The mother's weight
  * Blood tests such as a complete blood count, HIV test, or the triple screen which is test used most commonly to look for neural tube defects and Downs Syndrome.
  * Physical examination
  * Blood pressure
  * Fetal heart monitoring
  * Ultrasound scans
  * Non-stress tests

Continuous care is the best way to ensure a healthy mother and baby.

## Labor and Birth

Labor is defined as contractions _and_ cervical change, contractions alone are not labor.

  * Pre-Labor Signs: as your body is preparing for labor, there are a few things that should be expected to happen within four to six weeks of labor.
  1. Pressure on the pelvic area
  2. Occasional brownish discharge
  3. Energy level is noticeably increasing or decreasing
  4. Loss of the mucus plug (does not always exist)/increasing discharge
  5. Braxton Hicks contractions (painless contraction of the uterus)
  6. Movement of the baby into the pelvis
  * False Labor Signs: there are a few signs that indicate false labor.
  1. Timing of the contractions are irregular and do not become more frequent or more intense
  2. Contractions stop during rest, stopping what the mother is doing, walking, or changing position
  3. Inconsistent in strength (strong one minute then weak the next)
  4. Location of pain is in the front only
  * True Labor
  1. Pain in the lower back, radiating towards the front abdomen, possibly also the legs
  2. Contractions increase in strength and are closer together; coming now on a regular basis, 30 to 70 seconds apart
  3. The mucous plug is detached, showing bloody discharge
  4. The water breaks (usually this does not break until the doctor does it), when this happens, contractions become much stronger
  5. Some women have the sudden need to go to the bathroom, diarrhea is common
  6. Contractions continue despite movement
  7. The cervix is thinning and dilating

When the contractions of labor begin, the walls of the uterus start to contract. They are stimulated by the release of the pituitary hormone _oxytocin_. The contractions cause the cervix to widen and begin to open. As labor progresses the amniotic sac can rupture causing a slow or a fast gush of fluids. Labor usually begins within a 24 hour period after the amniotic sac has ruptured. As contractions become closer and stronger the cervix will gradually start to dilate. The first stage of labor is broken into three parts:

  * **Early Phase** First is the early phase of labor, when the cervix dilates from 1-4 centimeters, this can be the longest and most exhausting part for the mother.
  * **Active Phase** The cervix dilates on average 1 cm per hour in the active phase of labor dilating from 4-7 centimeters. If an epidural is requested it is usually given in this phase.
  * **Transition** This is often considered the most intense part of labor with contractions lasting longer and having shorter rest periods in between them. Dilation from 8-10 centimeters occurs during transition. Some women experience nausea and vomiting during this phase as well as rectal pressure and an urge to push.

At this point the labor enters the second stage, or the birth of the baby. The mother begins pushing to aid in the birth of the baby, this part of labor can last minutes, or even hours. A fetus usually delivered head first. 'Crowning' is the term used when the fetus' head can be seen between the mothers labia as it emerges. At this point if necessary the birth attendant may perform an episiotomy, which is a small surgical incision on the perineum. This procedure is usually done to deliver the baby more quickly in response to fetal distress.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/0/05/Medio-lateral-episiotomy-blank.png/220px-Medio-lateral-episiotomy-blank.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Diagram showing an episiotomie

The third stage of labor is the delivery of the afterbirth (placenta).

Oxytocin continues to be released to shrink the size of the uterus and aid in the limiting of blood loss from the site of the placenta. As the uterus shrinks the attachment site blood vessels, some of which can be as large as an adult finger, shrink also. The average blood loss in a routine vaginal delivery is 400-500 cc.

There are times when a mother may need outside aid in the delivery of the baby, some of these methods include:

  * Forceps, an instrument used to cradle the fetus' head and manipulate the head under the pubic bone to more easily pass through the birth canal.
  * Vacuum Extraction, a suction cup is applied to the baby's head, and a plunger is used to suck any air from between the suction cup and the head to create a good seal. The babies head is then manipulated through the birth canal. This usually leaves a baby's head bruised, but the mark fades within weeks after birth.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/1/1a/Caesarian_shown.jpg/220px-Caesarian_shown.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

C-section Birth

  * Cesarean section, or C-section, is the delivery of a baby through a surgical abdominal incision (Abdominal delivery - Abdominal birth - Cesarean section). A C-section delivery is performed when a vaginal birth is not possible or is not safe for the mother or child. Surgery is usually done while the woman is awake but anesthetized from the chest to the legs by epidural or spinal anesthesia. An incision is made across the abdomen just above the pubic area. The uterus is opened, and often brought through the incision after delivery for better visualization. The amniotic fluid is drained, and the baby is delivered. The baby's mouth and nose are cleared of fluids, and the umbilical cord is clamped and cut. After delivery a nursery nurse or pediatrician check the make sure that the baby is breathing and responding. Due to a variety of medical and social factors, C-sections have become fairly common; around 25% of births are performed by C-section. C-sections carry some risks to mother and baby. Compared to a vaginal birth, the risks to mother include increased risk of death, surgical injury, infection, postpartum depression, and hemorrhage, although these are rare. Babies born by c-section are more likely to be admitted to the ICU for breathing problems. Mothers are advised to carefully weigh the risks of C-section versus vaginal birth.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/4/47/Caesarian_shown_cropped.jpg/220px-Caesarian_shown_cropped.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Newborn baby

### Delivery Options

Hospital Births
    The chances of having natural, uncomplicated birth are optimized by carefully selecting your obstetrician and hospital. Doctors who work with midwives have lower cesarean section rates because midwives handle less complicated pregnancies. Delivering babies by abdominal surgery has been steadily rising in America over the past two decades, so that now 22-30% of births in American hospitals are cesarean section. The U.S., despite having the most advanced technology and highly trained medical personnel, ranks 23rd in infant mortality and 18th in perinatal mortality.

    Medical interventions such as epidural anesthesia, pitocin augmentation of labor, vacuum extraction of fetus, episiotomy and separation of newborn and mother are common in American hospitals. There are circumstances where medical procedures such as these are necessary, but many parents and professionals now question the routine use of such interventions. In some cases, the routine use of these procedures have lead to further complications. For example, the epidural anesthetic, while providing pain relief, has shown to increase the operative vaginal delivery rate (i.e. forceps and vacuum extraction rates slightly) especially in first time mothers. Epidurals have not been shown to increase the cesarean section rate in recent well documented studies.

Freestanding Birth Centers & Water Birth
    "Freestanding" Birth Centers are not inside of or affiliated with a hospital. They are run by collaboration of midwives or physicians. This is an alternative choice for the woman who does not wish to birth in a hospital environment yet is not comfortable giving birth at home. Birth centers do not provide any additional measure of safety than most planned home births with qualified midwives; they may provide the expectant couple with the physiological comfort necessary to enable the mother to relax.

    Out of hospital birth centers are designed for women having low-risk pregnancies who want drug-free birth with minimal intervention in a home-like environment. Family members may participate in the birth. C-sections rates are lower than most hospitals because the pregnancies are low risk. Freestanding Birth Centers are an alternative choice for a woman who has had a previous cesarean and wishes to maximize her chances of a vaginal delivery. However, vaginal birth attempts after a prior cesarean section have a 1-2% risk of uterine rupture. Heath insurance may cover costs. Many birth centers offer birthing tubs where one can give birth in water.

Homebirth
    Birth at home provides parents with intimacy, privacy, comfort and family-centered experience. Childbirth at home may be a safe option for healthy women having normal pregnancies. It is for those who have a very strong desire for natural childbirth and who are willing to take high degree of responsibility for their health care and baby's birth. At home, the parents and midwife are in control of the birthing environment, and strict time perimeters for length of labor are not imposed, or routine medical interventions such as IVs done. However, the World Health Organization (WHO) states that "giving birth in a health facility (not necessarily a hospital) with professional staff is safer by far than doing so at home." (The World Health Report 2005). Also, the American College of Obstetricians and Gynecologists (ACOG) opposes out of hospital births. In choosing the comfort of home parents are also choosing to be further away from lifesaving measures should complications arise.

    Homebirth midwives provide complete prenatal care including monthly visits, laboratory tests, screening for infections. They provide nutritional counseling and support for psycho-social issues. There is a chance that a rare, but critical emergency might occur during the birth where hospital services may not be able to be obtained quick enough. Again, the WHO states that "it is just before, during, and in the very first hours and days after birth that life is most at risk," (The World Health Report 2005) and that "many of the complications that result in maternal deaths and many that contribute to perinatal deaths are unpredictable, and their onset can be both sudden and severe." (WHO Birth and Emergency Preparedness in Antenatal Care, 2006) Home birth midwives are trained to know when an emergency requires medical interface and can provide stabilizing measures until critical care can be obtained. While homebirth midwives generally have the training, equipment, and medicine to handle many complications, there is great variation in training and skill level among midwives. In choosing a homebirth midwife one should careful examine credentials and training.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/f/f3/Umbilicalcord.jpg/200px-Umbilicalcord.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

A newborn with umbilical cord still attached (3 minutes.)

### Postpartum care

After the baby is born the umbilical cord is clamped and cut and the baby is looked over by a doctor or nurse. The baby is given an APGAR score at one and five minutes after birth. This is an analysis of how well the baby is performing its vital functions.

The five criteria of the Apgar score:

  Score of 0 Score of 1 Score of 2 Acronym

Skin color
blue all over
blue at extremities
normal
**A**ppearance

Heart rate
absent
<100
>100
**P**ulse

Reflex irritability
no response  
to stimulation
grimace/feeble cry  
when stimulated
sneeze/cough/pulls away  
when stimulated
**G**rimace

Muscle tone
none
some flexion
active movement
**A**ctivity

Respiration
absent
weak or irregular
strong
**R**espiration

If tearing, or an episiotomy occurs the wound is closed with absorbable suture. The mother is closely watched for blood loss, infection, or any other possible complications. Breastfeeding should be initiated as soon as possible after delivery as the stimulation of oxytocin in the mother aids in hemostasis.

## Risks in Pregnancy

Pregnancies that warrant close attention usually come from an existing medical condition such as asthma, diabetes, epilepsy, or a condition developed because of pregnancy. Conditions that arise during pregnancy will require special treatment. The purpose of prenatal care is to detect these conditions, and to monitor and deal with them before they become serious.

  


  * **Preeclampsia** is the medical term for high blood pressure during pregnancy. It is also characterized by edema, blurry vision, liver pain, and can progress into Eclampsia in which the mother can experience seizures, coma or even death.
  * **Gestational Diabetes** is diabetes mellitus that develops during pregnancy. All women should be tested for the condition at about 28 weeks gestation. Gestational and pre-existing diabetes can cause large for gestational age babies, a sudden drop in a neonates blood sugar after birth, and has a high risk for stillbirth

Other serious risks include:

  * **Teratogens** (substances that cause birth defects including alcohol and certain prescription and recreational drugs)
  * **Infection** (such as rubella or cytomegalovirus) An infection in the eleventh week is less likely to damage the heart, but the baby may be born deaf.
  * **Genetics** (such as Factor V Leiden) Diabetes, blood conditions, etc.
  * **Radiation** (ionizing radiation such as X-rays, radiation therapy, or accidental exposure to radiation)
  * **Nutritional deficiencies**
  * **Fetal Alcohol Syndrome** or **FAS** exposure is the leading known cause of mental retardation in the Western world. It is a disorder of permanent birth defects that occurs in the offspring of women who drink alcohol during pregnancy, depending on the amount, frequency, and timing of alcohol consumption. Alcohol crosses the placental barrier and can stunt fetal growth or weight, create distinctive facial stigmata, damage neurons and brain structures, and cause other physical, mental, or behavioral problems. Drinking during pregnancy should be avoided. Women who drink more than 4 or 5 drinks per day may cause permanent damage to their fetus, including, behavioral problems, sight and hearing loss, deformed organs and central nervous system dysfunction.
  * **Smoking** can cause low birth weight, still birth, birth defects, preterm births and immature lung development. It can also contribute to addiction in the child's later teen years.
  * **Illegal Drugs** can be the most devastating. Risks include SIDS (Sudden Infants Death Syndrome), learning disorders, birth defects, uncontrollable trembling, hyperactive, and drug dependency. Most drugs can be tested by a simple urine or blood test.
  * **Medications**. All medication use should be discussed with your doctor. Many over the counter and prescription drugs have warning labels. Follow these precautions to help avoid birth defects or other related problems.

### Miscarriage

Miscarriage or spontaneous abortion is the natural or spontaneous end of a pregnancy at a stage where the embryo or the fetus is incapable of surviving, generally defined in humans at a gestation of prior to 20 weeks. Miscarriages are the most common complication of pregnancy. Basic Facts: 15-20% of pregnancies end in miscarriage, 70% of the time there is a chromosomal abnormality with the fetus, and one miscarriage does not increase your risk in the next pregnancy. Miscarriage is almost never the mother's fault.

If the products of conception are not completely expelled after fetal death this is known as a missed abortion and is usually treated surgically by a procedure known as a D&C or dilation and curettage.

### Bleeding During Pregnancy

Vaginal bleeding at any stage should be taken seriously. Severe bleeding in the early weeks may be a sign of miscarriage. However, 25% of pregnant patient bleed in the first trimester. After 24 weeks the mother should seek medical advice immediately. Third trimester bleeding in pregnancy is often one of the first signs of placenta previa; placenta is across the opening of the cervix. An ultrasound should be performed to establish the location. Other causes of late term bleeding include:

  * **Preterm Labor** or labor that occurs before 38 weeks gestation that can have multiple causes
  * **Placental Abruption** is a condition in which the placenta is torn away from the uterine wall causing loss of oxygen and nutrients to the baby, and hemorrhage of mother and baby from the large blood vessels in the placenta. Most women, but not all experience heavy bleeding and abdominal pain. This is a life threatening emergency as a fetus can only survive as long as 50% of the placenta is still attached.

### Blood Conditions

Individuals either have, or do not have, the Rhesus factor (or Rh D antigen) on the surface of their red blood cells. This is usually indicated by 'RhD positive' (does have the RhD antigen) or 'RhD negative' (does not have the antigen) suffix to the ABO blood type i.e. A+ B- blood typing. This is a problem only when an Rh-negative woman has a partner who is Rh-positive resulting in an Rh-positive baby. If the mother's and the baby's blood come into contact during the birth, her body produces antibodies against the baby's blood. This problem usually does not affect the current pregnancy but can be dangerous for future pregnancies as the antibodies stay in the blood causing an immune response against future Rh+ fetus. In essence the mother's body "rejects" the fetus as it would a foreign body. A drug called Rhogam is now given by injection given at 28-30 weeks gestation and given again if there is confirmation that the baby is Rh positive within 24 hours after birth to protect the future pregnancies. Rh isoimmunization is rare in our day. Rh- mothers should also be given the injection after miscarriage or abortion.

If a mother is untreated they are at risk to subsequently deliver babies who suffer from hemolytic disease of the newborn. Hemolytic disease of the newborn, also known as HDN, is an alloimmune condition that develops in a fetus, when the IgG antibodies that have been produced by the mother and have passed through the placenta include ones which attack the red blood cells in the fetal circulation. The red cells are broken down and the fetus can develop reticulocytosis and anemia. This fetal disease ranges from mild to very severe, and fetal death from heart failure (hydrops fetalis) can occur. When the disease is moderate or severe, many erythroblasts are present in the fetal blood and so these forms of the disease can be called erythroblastosis fetalis (or erythroblastosis foetalis). Hemolysis leads to elevated bilirubin levels. After delivery bilirubin is no longer cleared (via the placenta) from the neonate's blood and the symptoms of jaundice (yellowish skin and yellow discoloration of the whites of the eyes) increase within 24 hours after birth. Like any other severe neonatal jaundice, there is the possibility of acute or chronic kernicterus. Profound anemia can cause high-output heart failure, with pallor, enlarged liver and/or spleen, generalized swelling, and respiratory distress. The prenatal manifestations are known as hydrops fetalis; in severe forms this can include petechiae and purpura. The infant may be stillborn or die shortly after birth.

### Other Abnormalities

Physical and Genetic Defects: Physical anomalies are present at birth. Examples are; cardiac, facial (such as cleft palate), club foot, etc. These do not always endanger the baby's life. 1-2% of babies are born with a significant congenital abnormality. 4-6% with something relatively minor.

  * Chromosomal Abnormalities: Occur when there is a problem in the baby's genetic makeup; these include conditions such as Down syndrome. Other genetic defects, such as cystic fibrosis, can be inherited from the parents.

## Staying Healthy

Pregnancy and childbirth place great demands, it is important to keep healthy. The more healthy and relaxed the mother is, the better it will be to cope with the demands of pregnancy. A healthy lifestyle combines many factors:

Balanced Diet
    A poor diet can cause a low birth weight. Excessive weight gain during pregnancy can cause back problems, varicose veins, or indicate preclampsia. Advice on diet often includes to eat foods that are high in nutritional content. Sufficient protein, vitamins, carbohydrates, fats, and minerals, as well as fiber. Limit intake of saturated fats and sugar, and salt. Drink plenty of fluids.
Regular Exercise
    Mild exercise, such as walking or swimming, is beneficial and will help cope with the workload of pregnancy and the demands of labor. Mother's should listen to her body and stop exercising when it tells her to. Exercise should never be painful.
Baby's Health
    Smoking reduces the oxygen and nutrients passing via the placenta to the baby. Avoid alcohol to avoid serious birth defects.

## In vitro Fertilization and Artificial Implantation

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/7/79/Icsi.JPG/300px-Icsi.JPG)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Oocyte is injected with sperm outside of the womb.

**An alternative when other methods of achieving contraception have failed.**

In vitro fertilization (IVF) is a technique in which egg cells are fertilized by sperm outside the woman's womb. IVF is a major treatment in infertility when other methods of achieving conception have failed. The process involves hormonally controlling the ovulatory process, removing ova (eggs) from the woman's ovaries and letting sperm fertilize them in a fluid medium. The fertilized egg (zygote) is then transferred to the patient's uterus with the intent to establish a successful pregnancy.

The term in vitro, from the Latin root, is used, because early biological experiments involving cultivation of tissues outside the living organism from which they came, were carried out in glass containers such as beakers, test tubes, or petri dishes.

While the overall live birth rate via IVF in the U.S. is about 27% per cycle (33% pregnancy rate), the chances of a successful pregnancy via IVF vary widely based on the age of the woman (or, more precisely, on the age of the eggs involved). Where the woman's own eggs are used as opposed to those of a donor, for women under 35, the pregnancy rate is commonly approximately 43% per cycle (37% live birth), while for women over 40, the rate falls drastically - to only 4% for women over 42. Other factors that determine success rates include the quality of the eggs and sperm, the duration of the infertility, the health of the uterus, and the medical expertise. It is a common practice for IVF programmes to boost the pregnancy rate by placing multiple embryos during embryo transfer. A flip side of this practice is a higher risk of multiple pregnancy, itself associated with obstetric complications.

**Embryo cryopreservation** If multiple embryos are generated, patients may choose to freeze embryos that are not transferred. Those embryos are placed in liquid nitrogen and can be preserved for a long time. There are currently 500,000 frozen embryos in the United States. The advantage is that patients who fail to conceive may become pregnant using such embryos without having to go through a full IVF cycle. Or, if pregnancy occurred, they could return later for another pregnancy.

## Embryonic stem cells

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/3/3c/Stem_cells_diagram.png/500px-Stem_cells_diagram.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Pluripotent, embryonic stem cells originate as inner mass cells with in a blastocyst. The stem cells can become any tissue in the body, excluding a placenta. Only the morula's cells are totipotent, able to become all tissues and a placenta.

Embryonic celtic cell lines (ES cell lines) are cultures of cells derived from the epiblast tissue of the inner cell mass (ICM) of a blastocyst. A blastocyst is an early stage embryo - approximately 4 to 5 days old in humans and consisting of 50-150 cells. ES cells are _pluripotent_, and give rise during development to all derivatives of the three primary germ layers: ectoderm, endoderm and mesoderm. In other words, they can develop into each of the more than 200 cell types of the adult body when given sufficient and necessary stimulation for a specific cell type. They do not contribute to the extra-embryonic membranes or the placenta. This means they can become any kind of human tissue (ie. heart tissue, nerve tissue, etc.).

When given no stimuli for differentiation, ES cells will continue to divide in vitro and each daughter cell will remain pluripotent. The pluripotency of ES cells has been rigorously demonstrated in vitro and in vivo, thus they can be indeed classified as stem cells.

Because of their unique combined abilities of unlimited expansion and pluripotency, embryonic stem cells are a potential source for regenerative medicine and tissue replacement after injury or disease. To date, no approved medical treatments have been derived from embryonic stem cell research. This is not surprising considering that many nations currently have moratoria (suspension of practices) on either ES cell research or the production of new ES cell lines.

There exists a widespread controversy over stem cell research that emanates from the techniques used in the creation and usage of stem cells. Embryonic stem cell research is particularly controversial because, with the present state of technology, starting a stem cell line requires the destruction of a human embryo and/or therapeutic cloning. Opponents of the research argue that this practice is a slippery slope to reproductive cloning and tantamount to the instrumentalization of a human being. Contrarily, some medical researchers in the field argue that it is necessary to pursue embryonic stem cell research because the resultant technologies are expected to have significant medical potential, and that the embryos used for research are only those meant for destruction anyway (as a product of in vitro fertilization). This in turn, conflicts with opponents in the pro-life movement, who argue that an embryo is a human being and therefore entitled to dignity even if legally slated for destruction. The ensuing debate has prompted authorities around the world to seek regulatory frameworks and highlighted the fact that stem cell research represents a social and ethical challenge.

  * **Reproductive Cloning**

Reproductive Cloning is a technology used to generate an animal that contains the same nuclear DNA as another currently or previously existing animal. Scientists transfer the genetic material from the nucleus of a donor adult cell to an egg whose nucleus, and thus its genetic material has been removed. The egg containing the DNA, now reconstructed, has to be treated with chemicals or electric current in order to stimulate cell division. Once the cloned embryo reaches a suitable stage, it is transferred to the uterus of a female host to continue development until birth. Currently this is illegal to practice in the United States.

  * **Therapeutic Cloning**

Recent research by researchers led by Anthony Atala of Wake Forest University and a team from Harvard University has found that amniotic fluid, in addition to its main functions of cushioning a growing fetus and providing buoyancy, is also a plentiful source of non-embryonic stem cells. These cells have demonstrated the ability to differentiate into a number of different cell-types, including brain, liver and bone.

Therapeutic Cloning refers to a procedure that allows the cloning of specific body parts and organs to be used for medical purposes. Although this has not been realized, much research is being done on the subject.

## Pregnancy and Lactation

A mother's milk is ideal because it meets the baby's specific needs. Lactation is a neuroendocrine response in _milk production_. Sucking stimulates the sensory nerve endings in the nipples and sends stimulus to the hypothalamus. The hypothalamus stimulates anterior pituitary and prolactin is released. In _milk let-down_ the sucking stimulates sensory nerves in the nipples. This stimulates the hypothalamus which then stimulates the posterior pituitary and releases oxytocin. Sucking also stimulates contraction of the cells around the alveoli in the mammary cells. Milk then flows into the milk ducts causing milk let-down.

Breast milk provides all the nutrients required for the first 4-6 months. It contains carbohydrates (such as lactose), fats (such as linoleic acid), and easily digestible proteins (such as alpha-lactalbumin). Breast milk also contains an adequate supply of vitamins and minerals, digestive enzymes, hormones and immunological factors.

The first milk produced after birth is called _colostrum_. This is synthesized during the end of pregnancy and 3-5 days of postpartum. It is very high in protein and low in fat and carbohydrates, and contains immunoglobulins. This help the baby have a first bowel movement and prevent jaundice. The bowel movement that results from the colostrum is a different color and consistency than future bowel movements once the mother's milk comes in. In some cultures the colostrum is discarded because of the difference, but what they do not know is that it is the best thing for the baby.

The composition varies in breast milk during feeding, and over time with development of the baby. When breastfeeding there are three names for the composition of the milk: the fore milk, present during the beginning of breastfeeding; mid is the middle of feeding; and hind which is toward the end of the feeding and contains a composition high in fat.

When breastfeeding the female should consider the types of food that will be consumed. If the mother is on a low fat diet or if foods like garlic, broccoli, and onions are eaten, it may affect the baby's preference for breast feeding. Also, the mother should consider not breastfeeding after the consumption of alcohol, caffeine, smoking, and certain medications.

Barriers of breastfeeding are lack of professional and social support, misinformation, embarrassment, early discharge form the hospital without instruction, and returning to work or school without adequate lactation rooms and if the mother refuses to tend breastfed infant.

When breastfeeding initiate as soon after delivery as possible, position the baby correctly, feed on demand from both breasts at each feeding and at least 10 minutes on each breast. Additionally there should be a good educator in the case the infant is not latching on.

A common problem that may happen when breastfeeding is _mastitis_, which is an inflammation of one or both breasts and is usually associated with the infection of a blocked milk duct during lactation. The symptoms include flu-like symptoms, red streaks on the breast, and hot skin. Antibiotics may be necessary to clear the infection. _Thrush_ may also happen and could be passed between mom and baby. A symptom of thrush includes white flecks on tongue, and the baby and mother should be treated by a doctor.

Breast milk is recommended through the first 12 months. Supplementation of cow's milk is not recommended due to the high protein that would cause liver damage to the baby.

Why breastfeed?

  * It is easily digested
  * Composition changes with infant needs
  * Changes during a feeding, high in fat at the end of feeding
  * Antibodies in milk
  * Breastfeeding moms miss less work because babies are sick less
  * Fewer allergies
  * Less spit-up
  * Less constipation and diarrhea
  * Better jaw development
  * Decreased risk of SIDS (Sudden Infant Death Syndrome)
  * Higher IQ
  * Decreased risk of diabetes, Crohn's Disease, Celiac Sprue
  * Bonding
  * Convenient, always at the correct temperature and ready to go
  * Less expensive
  * Helps the uterus return to normal size more quickly
  * Less incidence of postpartum “blues”
  * Lower risk of breast cancer
  * Lower risk of osteoporosis

## Postpartum Depression

Having a baby is usually one of the happiest times in a woman's life, but for some women, it can include times of sadness and depression. More women actually suffer from postpartum depression than we really know. Women usually ignore the emotional and physical signs, dealing with their feelings on their own.

Postpartum depression affects approximately 10 to 15 percent of new mothers. It often causes anxiety and obsession about caring for the baby or the cleanliness of the home. It may cause changes in sleep patterns and affect relationships including the ability to form a bond with the baby and other family members. Some mothers with postpartum depression have thoughts of wanting to die or of hurting the baby. If the symptoms are so severe that they keep the mother from being able to function, medical treatment is necessary.

Baby blues are common due to rapid hormonal changes but resolve after 1-2 weeks. Postpartum depression is characterized by persisting symptoms, and the mother should notify her provider immediately.

## Testing Your Knowledge

Answers for these questions can be found [here](http://en.wikibooks.org/wiki/Human_Physiology/Appendix_1:_answers_to_review_questions#Pregnancy_and_birth)

1\. Is at this stage that an egg implants in the uterine lining

    A) morula
    B) zygote
    C) blastocyst
    D) embryoblast

2\. Which part of the embryoblast will become the central nervous system in development

    A) ectogerm
    B) mesoderm
    C) endoderm

3\. This hormone is only produced in the human body when a woman is pregnant

    A) estrogen
    B) HCG
    C) progesterone
    D) FSH
    E) LH

4\. By this week of pregnancy, the beginnings of all major organs have formed

    A) 4
    B) 7
    C) 5
    D) 6
    E) 8

5\. Stem cells are found in the embryoblast and use of them is very controversial, another place to find stem cells that are usable to treat leukemia and other disorders is the

    A) morula
    B) chorion
    C) amnion
    D) amniotic fluid
    E) umbilical cord

6\. The cervix dilates on an average of ______ per hour in the active phase of labor

    A) 2 mm
    B) 2 cm
    C) 1mm
    D) 1 cm

7\. The contractions of the uterus are stimulated by the release of

    A) oxytocin
    B) FSH
    C) LH
    D) prolactin
    E) estrogen

8\. A sign of pre-labor is

    A) irregular contractions
    B) pain in the front only
    C) loss of the mucus plug
    D) contractions stop during rest

9\. This is the most common complication of pregnancy

    A) preclampcia
    B) miscarriage
    C) smoking
    D) Rh factor
    E) teratogens

10\. Sue decides to breastfeed because she has been told that colostrum contains

    A) high protein
    B) low fat
    C) immunoglobulins
    D) all of the above
    E) none of the above

11\. What is the first milk after birth called?

    A) thrush
    B) mastitis
    C) colostrum
    D) milk let down

## Glossary

**Abruption**: Premature separation of the placenta from the wall of the womb

**Amnion**: An embryonic membrane that encircles a developing fetus and contains amniotic fluid.

**Amniocentesis**: A procedure in which a small sample of amniotic fluid is removed from around the fetus

**Amniotic fluid**: The fluid surrounding the fetus

**Amniotomy**: (artificial rupture of membranes, ARM) Breaking the membranes using a special plastic hook

**Anemia**: Lack of hemoglobin in red blood cells, due to iron deficiency or disease

**Antepartum Hemorrhage**: (APH) Vaginal bleeding that happens after 24 weeks of pregnancy and before delivery

**Breech**: The baby is lying bottom down in the womb

**Celiac sprue**: Nutrient absorption impairment which is improved when gluten is removed form the diet. Characteristic mucosal lesion of the small intestine.

**Cephalic**: The baby is lying head down in the womb

**Chorion**: The embryonic membrane that forms the outermost covering around the developing fetus.

**Chorion Villus Sampling**: (CVS) A method for sampling placental tissue for genetic or chromosome studies.

**Colostrum** the fluid that is made late in pregnancy and the first few days postpartum in the breast that contains immunologic substances and essential nutrients.

**Cleavage**: The early successive divisions of embryonic cells into smaller and smaller cells.

**Cilia**: The fine hairs that line the fallopian tubes'

**Cordocentesis**: The procedure for taking blood from the fetal umbilical cord via a needle through the mother’s abdomen

**Copulation**: (Coitus, sexual intercourse) is the procreative act of a man's erect penis is inserted into a woman's vagina. At climax, semen is ejaculated from the penis at the cervix of the uterus. Sperm then propel themselves into the uterine tubes where fertilization may occur if an egg is present.

**Crohn's disease**: Skip lesions in the colon and is a malabsorptive disease.

**Cystitis**: Infection of the bladder

**Dizygous**: Not identical (fraternal) twins

**Doppler**: A form of ultrasound used specially to investigate blood flow in the placenta or in the fetus

**Down Syndrome**: (Trisomy 21) A disorder caused by the presence of an extra chromosome 21 in the cells

**Ectopic Pregnancy**: A pregnancy that develops outside of the womb

**Edema**: Swelling of the fingers, legs, toes, and face.

**Embryo**: The medical term for the baby from conception to about six weeks

**Engagement**: The process in which the head of the baby moves down from high in the mother's abdomen and settles deeper into her pelvis in preparation for birth. This can happen any time between 36 weeks and labor.

**Epidural Anesthesia**: A method of numbing the nerves of the lower spinal cord to ensure a pain-free labor

**Episiotomy**: A cut of the perineum and vagina performed to make the delivery easier

**External Fetal Monitor**: An electronic monitor used to record the fetal heartbeat and mother’s contractions

**Fallopian Tubes**: (uterine tubes) Two tubular structures (one on each side of the womb) leading from the ovaries to the uterus

**Fertilization**: The union of an egg cell and a sperm cell is present wherein 23 chromosomes from each parent come together to form a zygote. After sperm penetrates, the ovum undergoes a chemical change to prevent other sperm from entering. Multiple births can occur from complete division of the conceptus during early cleavage or from fertilization of multiple ova. Birth control techniques are designed to prevent ovulation or to prevent fertilization by barriers, that keep sperm and ova separated.

**Fetus**: Medical term for the baby from six weeks after conception until birth

**Forceps**: Metal instruments that fit on either side of the baby's head and are used to help deliver the baby

**Fundus**: The top of the womb

**Germ layer**: Layers of cells within an embryo that form the body organs during development.

**Glial Cells** (neuroglia; glia): Non-neuronal cells that provide support and nutrition, maintain homeostasis, form myelin, and participate in signal transmission in the nervous system. In the human brain, glia are estimated to outnumber neurons by about 10 to 1.

Glial cells provide support and protection for neurons, the other main type of cell in the central nervous system. They are thus known as the "glue" of the nervous system. The four main functions of glial cells are to surround neurons and hold them in place, to supply nutrients and oxygen to neurons, to insulate one neuron from another, and to destroy pathogens and remove dead neurons.

**Hemoglobin**: (Hb)The oxygen carrying constituent of red blood cells

**Induction of labor**: (IOL) the procedure for initiating labor artificially

**In utero death**: (IUD)the death of the unborn fetus after 24 weeks

**In vitro fertilization**: (IVF) a method of assisted conception in which fertilization occurs outside the mother's and the embryo is replaced in the womb

**Lanugo**: fine hair that covers the fetus in the womb

**Lochia**: blood loss after birth

**Mastitis** inflammation of the breast most frequently in lactation.

**Neonatal**: baby less than 28 days old

**Nuchal scan**: special ultrasound scan that gives an estimate of the risk of Down syndrome

**Oocyte**: one egg that is released from the ovary at each ovulation

**Placenta**: The structure by which an unborn child is attached to it's mother's uterine wall and through which it is nourished.

**Postnatal**: After birth

**Prenatal**: Before birth

**Quickening**: The process that occurs between the seventeenth and twentieth weeks of fetal development, the fetus's leg bones achieve their final relative proportions. In this process the muscles contract, causing movement at the fetus's sinovial joints. The joint movement enhances the nutrition of the articular cartilage and prevents the fusion of connective tissues within the joint. It also promotes bone hardening. It is this stage, where the fetus's bones become more developed and harder, that the mother begins to notice fetal movement.

**Rudimentary**: Basic; minimal; with less than, or only the minimum, necessary

**Thrush**: Creamy white flakes on a red papillae on tongue and tongue may be enlarged.

**Umbilical cord**: The cord like structures that connects the fetus to the placenta.

**Zygote**: A cell produced by the fusion of an egg and a sperm; a fertilized egg cell.

## Reference

  * _"as your baby grows From Conception to Birth"published by American Baby_
  * <http://www.babybluesconnection.com>
  * _"Pregnancy and Birth"_ authors: Dr. Karina Reynolds, Dr. Christoph Lees, Grainne McCartan
  * _"Fundamental Concepts of Human Anatomy"_ authors: M.J. Shively D.V.M., M.S., Ph.D. and D.P. Homan B.S., M.S.
  * _"Essentials of Anatomy and Physiology" authors:Valerie C. Scanlon and Tina Sanders, fourth edition_
  * <http://www.MERLOT.com> Stanford Site
  * _"The New Parent" author DR. Miriam Stoppard_
  * _www.marchofdimes.com_
  * _<http://health.allrefer.com/health/fetal-development-info.html>_
  * _American Pregnancy Association_

Internet groups: International Awareness Network: www.ican-online.org  


# Genetics and Inheritance

## Introduction

![Wikipedia-logo.png](../../../upload.wikimedia.org/wikipedia/commons/thumb/6/63/Wikipedia-logo.png/40px-Wikipedia-logo.png)

[Wikipedia](http://en.wikipedia.org/wiki/) has related information at _**[Natural Selection**_](http://en.wikipedia.org/wiki/Natural_Selection)

Genetics is the science of the way traits are passed from parent to offspring. For all forms of life, continuity of the species depends upon the genetic code being passed from parent to offspring. Evolution by natural selection is dependent on traits being heritable. Genetics is very important in human physiology because all attributes of the human body are affected by a person’s genetic code. It can be as simple as eye color, height, or hair color. Or it can be as complex as how well your liver processes toxins, whether you will be prone to heart disease or breast cancer, and whether you will be color blind. Defects in the genetic code can be tragic. For example: Down Syndrome, Turner Syndrome, and Klinefelter's Syndrome are diseases caused by chromosomal abnormalities. Cystic fibrosis is caused by a single change in the genetic sequence.

Genetic inheritance begins at the time of conception. You inherited 23 chromosomes from your mother and 23 from your father. Together they form 22 pairs of autosomal chromosomes and a pair of sex chromosomes (either XX if you are female, or XY if you are male). Homologous chromosomes have the same genes in the same positions, but may have different alleles (varieties) of those genes. There can be many alleles of a gene within a population, but an individual within that population only has two copies, and can be homozygous (both copies the same) or heterozygous (the two copies are different) for any given gene.

![Wikipedia-logo.png](../../../upload.wikimedia.org/wikipedia/commons/thumb/6/63/Wikipedia-logo.png/40px-Wikipedia-logo.png)

[Wikipedia](http://en.wikipedia.org/wiki/) has related information at _**[Human Genome Project**_](http://en.wikipedia.org/wiki/Human_Genome_Project)

Genetics is important to medicine. As more is understood about how genetics affects certain defects and diseases, cures and treatments can be more readily developed for these disorders. The sequence of the human genome (approximately 3 billion base pairs in a human haploid genome with an estimated 20,000-25,000 protein-coding genes) was completed in 2003, but we are far from understanding the functions and regulations of all the genes. In some ways medicine is moving from diagnosis based on symptoms towards diagnosis based on genetics, and we are moving into what many are calling the age of personalized medicine.

### DNA

**Deoxyribonucleic acid** (DNA) is the macromolecule that stores the information necessary to build structual and functional cellular components. It also provides the basis for inheritance when DNA is passed from parent to offspring. The union of these concepts about DNA allows us to devise a working definition of a gene. A **gene** is a segment of DNA that codes for the synthesis of a protein and acts as a unit of inheritance that can be transmitted from generation to generation. The external appearance (_phenotype_) of an organism is determined to a large extent by the genes it inherits (_genotype_). Thus, one can begin to see how variation at the DNA level can cause variation at the level of the entire organism. These concepts form the basis of **genetics** and **evolutionary theory**.

### Gene

![](../../../upload.wikimedia.org/wikipedia/commons/8/81/ADN_animation.gif)

rotating animation of a DNA molecule.

A gene is made up of short sections of DNA which are contained on a chromosome within the nucleus of a cell. Genes control the development and function of all organs and all working systems in the body. A gene has a certain influence on how the cell works; the same gene in many different cells determines a certain physical or biochemical feature of the whole body (e.g. eye color or reproductive functions). All human cells hold approximately 30,000 different genes. Even though each cell has identical copies of all of the same genes, different cells express or repress different genes. This is what accounts for the differences between, let's say, a liver cell and a brain cell . Genotype is the actual pair of genes that a person has for a trait of interest. For example, a woman could be a carrier for hemophilia by having one normal copy of the gene for a particular clotting protein and one defective copy. A Phenotype is the organism’s physical appearance as it relates to a certain trait. In the case of the woman carrier, her phenotype is normal (because the normal copy of the gene is dominant to the defective copy). The phenotype can be for any measurable trait, such as eye color, finger length, height, physiological traits like the ability to pump calcium ions from mucosal cells, behavioral traits like smiles, and biochemical traits like blood types and cholesterol levels. Genotype cannot always be predicted by phenotype (we would not know the woman was a carrier of hemophilia just based on her appearance), but can be determined through pedigree charts or direct genetic testing. Even though genotype is a strong predictor of phenotype, environmental factors can also play a strong role in determining phenotype. Identical twins, for example, are genetic clones resulting from the early splitting of an embryo, but they can be quite different in personality, body mass, and even fingerprints.

### Genetics

**Genetics** (from the Greek _genno_ = give birth) is the science of genes, heredity, and the variation of organisms. The word "genetics" was first suggested to describe the study of inheritance and the science of variation by prominent British scientist William Bateson in a personal letter to Adam Sedgwick, dated April 18, 1905. Bateson first used the term "genetics" publicly at the Third International Conference on Genetics (London, England) in 1906.

Heredity and variations form the basis of genetics. Humans apply knowledge of genetics in prehistory with the domestication and breeding of plants and animals. In modern research, genetics provide important tools for the investigation of the function of a particular gene, e.g., analysis of genetic interactions. Within organisms, genetic information is generally carried in _chromosomes_, where it is represented in the chemical structure of particular DNA molecules.

  


![](../../../upload.wikimedia.org/wikipedia/commons/thumb/e/ea/Mendel_seven_characters.svg/220px-Mendel_seven_characters.svg.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

diagram showing the seven characters observed by Mendel

Genes encode the information necessary for synthesizing the amino-acid sequences in proteins, which in turn play a large role in determining the final phenotype, or physical appearance of the organism. In diploid organisms, a dominant allele on one chromosome will mask the expression of a recessive allele on the other. While most genes are dominant/recessive, others may be codominant or show different patterns of expression. The phrase "to code for" is often used to mean a gene contains the instructions about a particular protein, (as in the gene codes for the protein). The "one gene, one protein" concept is now known to be the simplistic. For example, a single gene may produce multiple products, depending on how its transcription is regulated. Genes code for the nucleotide sequence in mRNA and rRNA, required for protein synthesis.

Gregor Mendel researched principals of heredity in plants. He soon realized that these principals also apply to people and animals and are the same for all living animals.

Gregor Mendel experimented with common pea plants. Over generations of the pea plants, he noticed that certain traits can show up in offspring with out blending any of the parent's characteristics. This is a very important observation because at this point the theory was that inherited traits blend from one generation to another.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/8/82/Mendelian_inheritance_1_2_1.png/220px-Mendelian_inheritance_1_2_1.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Mendelian inheritance 1 2 1

Pea plant reproduction is easily manipulated. They have both male and female parts and can easily be grown in large numbers. For this reason, pea plants can either self-pollinate or cross-pollinate with other pea plants.

In cross pollinating two true-breeding plants, for example one that came from a long line of yellow peas and the other that came from a long line of green peas, the first generation of offspring always came out with all yellow peas. The following generations had a ratio of 3:1 yellow to green. In this and in all of the other pea plant traits Mendel observed, one form was dominant over another so it masked the presence of the other allele. Even if the phenotype (presence) is covered up, the genotype (allele) can be passed on to other generations.

  
**Time line of notable discoveries**

1859 [Charles Darwin](http://en.wikipedia.org/wiki/Charles_Darwin) publishes ["The Origin of Species"](http://en.wikipedia.org/wiki/The_Origin_of_Species)

1865 [Gregor Mendel's](http://en.wikipedia.org/wiki/Gregor_Mendel) paper, _[Experiments on Plant Hybridization_](http://en.wikipedia.org/wiki/Experiments_on_Plant_Hybridization)

1903 Chromosomes are discovered to be hereditary units

1906 The term "genetics" is first introduced publicly by the British biologist [William Bateson](http://en.wikipedia.org/wiki/William_Bateson) at the Third International Conference on Genetics in London, England

1910 [Thomas Hunt Morgan](http://en.wikipedia.org/wiki/Thomas_Hunt_Morgan) shows that genes reside on chromosomes, and discovered linked genes on chromosomes that do NOT follow Mendel's law of independent allele segregation

1913 [Alfred Sturtevant](http://en.wikipedia.org/wiki/Alfred_Sturtevant) makes the first genetic map of a chromosome

1913 Gene maps show chromosomes contain linear arranged genes

1918 [Ronald Fisher](http://en.wikipedia.org/wiki/Ronald_Fisher) publishes _On the correlation between relatives on the supposition of Mendelian inheritance_ \- the modern synthesis starts.

1927 Physical changes in genes are called mutations

1928 Fredrick Griffith discovers a hereditary molecule that is transmissible between bacteria

1931 Crossing over is the cause of recombination

1941 [Edward Lawrie Tatum](http://en.wikipedia.org/wiki/Edward_Lawrie_Tatum) and [George Wells Beadle](http://en.wikipedia.org/wiki/George_Wells_Beadle) show that genes code for proteins

1944 [Oswald Theodore Avery](http://en.wikipedia.org/wiki/Oswald_Theodore_Avery), [Colin McLeod](http://en.wikipedia.org/wiki/Colin_McLeod) and [Maclyn McCarty](http://en.wikipedia.org/wiki/Maclyn_McCarty) isolate [DNA](http://en.wikipedia.org/wiki/DNA) as the genetic material (at that time called transforming principle)

1950 [Erwin Chargaff](http://en.wikipedia.org/wiki/Erwin_Chargaff) shows that the four nucleotides are not present in nucleic acid in stable proportions, but that some general rules appear to hold. (e.g., the nucleotide bases Adenine-Thymine and Cytosine-guanine always remain in equal proportions)

1950 Barbra McClintock discovers transposons in maize

1952 The [Hershey-Chase experiment](http://en.wikipedia.org/wiki/Hershey-Chase_experiment) proves the genetic information of phages (and all other organisms) to be DNA

1953 DNA structure is resolved to be a double helix by [James D. Watson](http://en.wikipedia.org/wiki/James_D._Watson) and [Francis Crick](http://en.wikipedia.org/wiki/Francis_Crick), with help from [Rosalind Franklin](http://en.wikipedia.org/wiki/Rosalind_Franklin)

1956 Jo Hin Tjio and [Albert Levan](http://en.wikipedia.org/wiki/Albert_Levan) established the correct chromosome number in humans to be 46

1958 The [Meselson-Stahl experiment](http://en.wikipedia.org/wiki/Meselson-Stahl_experiment) demonstrates that DNA is semi-conservatively replicated

1961 The genetic code is arranged in triplets

1964 [Howard Temin](http://en.wikipedia.org/wiki/Howard_Temin) showed using RNA viruses that Watson's central dogma is not always true

1970 Restriction enzymes were discovered in studies of a bacterium _Haemophilus influenzae_, enabling scientists to cut and paste DNA

1977 DNA is sequenced for the first time by Fred Sangr, [Walter Gilbert](http://en.wikipedia.org/wiki/Walter_Gilbert), and Allan Maxam working independently. Sanger's lab complete the entire genome of sequence of Bacteriophage

1983 [Kary Banks Mullis](http://en.wikipedia.org/wiki/Kary_Banks_Mullis) discovers the polymerase chain reaction (PCR) enabling the easy amplification of DNA

1985 [Alec Jeffreys](http://en.wikipedia.org/wiki/Alec_Jeffreys) discovers genetic finger printing

1989 The first human gene is sequenced by Francis Collin and [Lap-Chee Tsui](http://en.wikipedia.org/wiki/Lap-Chee_Tsui). It encodes the CFTR protein. Defect in this gene causes [Cystic Fibrosis](http://en.wikipedia.org/wiki/Cystic_Fibrosis)

1995 The genome of [Haemophilus](http://en.wikipedia.org/wiki/Haemophilus) influenza is the first genome of a free living organism to be sequenced.

1996 [Saccharomyces](http://en.wikipedia.org/wiki/Saccharomyces) cerevisiae is the first eukaryote genome sequence to be released.

1998 The first genome sequence for a multicellular eukaryote, [C. elegans](http://en.wikipedia.org/wiki/C._elegans) is released.

2001 First draft sequences of the human genome are released simultaneously by the [Human Genome Project](http://en.wikipedia.org/wiki/Human_Genome_Project) and Celera Genomic

2003 (14 April) Successful completion of Human Genome Project with 99% of the genome sequenced to a 99.99% accuracy

2006 Marcus Pembrey and Olov Bygren publish _Sex-specifics_, _male line trans-generational responses in humans_, a proof of epigenetics

## Transcription and Translation

![Transcription.png](../../../upload.wikimedia.org/wikipedia/commons/thumb/3/32/Transcription.png/300px-Transcription.png)

**Transcription** is the process of making RNA. In response to an enzyme RNA polymerase breaks the hydrogen bonds of the gene. A gene is a segment of DNA which contains the information for making a protein. As it breaks the hydrogen bonds it begins to move down the gene. Next the RNA polymerase will line up the nucleotides so they are complementary. Some types of RNA will leave the nucleus and perform a specific function.

**Translation** is the synthesis of the protein on the ribosome as the mRNA moves across the ribosome. There are eleven basic steps to translation.

1\. The mRNA base sequence determines the order of assembling of the amino acids to form specific proteins.

2\. Transcription occurs in the nucleus, and once you have completed transcription the mRNA will leave the nuecleus, and go into the cytoplasm where the mRNA will bind to a free floating ribosome, where it will attach to a small ribosomal subunit.

3\. Methionine-tRNA binds to the nucleotides AUG. AUG is known as the start codon and is found at the beginning of each mRNA.

4\. The complex then binds to a large ribosomal subunit. Methionine-tRNA is bound to the P site of the ribosome.

5\. Another tRNA containing a second amino acid (lysine) binds to the second amino acid. Binding to the second condon of mRNA (on the A-site of the ribosome).

6\. Peptidyl transferase, forms a peptide3 bond between the two amino acids (methionine and lysine)

7\. The first amino tRNA is released and mRNA is translocated one codon carrying the second tRNA (still carrying the two amino acids) to the P site.

8\. Another tRNA with attached amino acid (glutamine) moves into the A site and binds to that codon.

9\. It will now form a peptide bond with lysine and glutamine

10\. Now the tRNA in the P site will be let go, and mRNA is translocated one codon, (the tRNA with three amino acids) to the P site.

11\. This will continue going until it reaches the stop codon (UAG) on the mRNA. Then this codon will tell it to release the polypeptide chain.

  
These are some good sites to visit

    **A** <http://www.studiodaily.com/main/technique/tprojects/6850.html>
    **B** <http://multimedia.mcb.harvard.edu/media.html>

Select **A** the video of the Inner Life of a Cell. If you want to hear the descriptions in this process go to **B** web site and select the Inner Life: view the animation.

## Inheritance

Children inherit traits, disorders, and characteristics from their parents. Children tend to resemble their parents especially in physical appearance. However they may also have the same mannerisms, personality, and a lot of the time the same mental abilities or disabilities. Many negatives and positives tend to "run in the family". A lot of the time people will use the excuse "It runs in the family" for things that have alternative reasons, such as a whole family may be overweight, yes it may "run in the family" but it could also be because of all the hamburgers and extra mayo that they all eat. Or the fact that after they eat the hamburgers they all sit on the couch and don't move for the rest of the evening. Children may have the same habits (good or bad) as their parents, like biting their nails or enjoying reading books. These things aren't inherited they are happening because children imitate their parents, they want to be like mom or dad. Good examples are just as important as good genes.

Inheritance pattern Description Examples

Autosomal dominant
Only one mutated copy of the gene is needed for a person to be affected by an autosomal dominant disorder. Each affected person usually has one affected parent. There is a 50% chance that a child will inherit the mutated gene. Many disease conditions that are autosomal dominant have low penetrance, which means that although only one mutated copy is needed, a relatively small proportion of those who inherit that mutation go on to develop the disease, often later in life.
Huntingtons disease, Neurofibromatosis 1, HBOC syndrome, Hereditary nonpolyposis colorectal cancer

Autosomal recessive
Two copies of the gene must be mutated for a person to be affected by an autosomal recessive disorder. An affected person usually has unaffected parents who each carry a single copy of the mutated gene (and are referred to as carriers). Two unaffected people who each carry one copy of the mutated gene have a 25% chance with each pregnancy of having a child affected by the disorder.
Cystic fibrosis, Sickle cell anemia, Tay-Sachs disease, Spinal muscular atrophy, Muscular dystrophy

X-linked dominant
X-linked dominant disorders are caused by mutations in genes on the X chromosome. Only a few disorders have this inheritance pattern. Females are more frequently affected than males, and the chance of passing on an X-linked dominant disorder differs between men and women. The sons of a man with an X-linked dominant disorder will not be affected, and his daughters will all inherit the condition. A woman with an X-linked dominant disorder has a 50% chance of having an affected daughter or son with each pregnancy. Some X-linked dominant conditions, such as Aicardi Syndrome, are fatal to boys, therefore only girls have them (and boys with Klinefelter Syndrome).
Hypophosphatemia, Aicardi Syndrome

X-linked recessive
X-linked recessive disorders are also caused by mutations in genes on the X chromosome. Males are more frequently affected than females, and the chance of passing on the disorder differs between men and women. The sons of a man with an X-linked recessive disorder will not be affected, and his daughters will carry one copy of the mutated gene. With each pregnancy, a woman who carries an X-linked recessive disorder has a 50% chance of having sons who are affected and a 50% chance of having daughters who carry one copy of the mutated gene.
Hemophilia A, Duchenne muscular dystrophy, Color blindness, Turner Syndrome

Y-linked
Y-linked disorders are caused by mutations on the Y chromosome. Only males can get them, and all of the sons of an affected father are affected. Since the Y chromosome is very small, Y-linked disorders only cause infertility, and may be circumvented with the help of some fertility treatments.
Male Infertility

Mitochondrial
This type of inheritance, also known as maternal inheritance, applies to genes in mitochondrial DNA. Because only egg cells contribute mitochondria to the developing embryo, only females can pass on mitochondrial conditions to their children.
Leber's Hereditary Optic Neuropathy (LHON)

### Mechanisms of inheritance

A person's cells hold the exact genes that originated from the sperm and egg of his parents at the time of conception. The genes of a cell are formed into long strands of DNA. Most of the genes that control characteristic are in pairs, one gene from mom and one gene from dad. Everybody has 22 pairs of chromosomes (_autosomes_) and two more genes called sex-linked chromosomes. Females have two X (XX) chromosomes and males have an X and a Y (XY) chromosome. Inherited traits and disorders can be divided into three categories: unifactorial inheritance, sex-linked inheritance, and multifactor inheritance.

  


### Unifactorial Inheritance

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/3/3e/Autorecessive.svg/400px-Autorecessive.svg.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Chart showing the possibilities of contracting a recessive defect, from two carrier parents.

Traits such as blood type, eye color, hair color, and taste are each thought to be controlled by a single pair of genes. The Austrian monk Gregor Mendel was the first to discover this phenomenon, and it is now referred to as _the laws of Mendelian inheritance_. The genes deciding a single trait may have several forms (_alleles_). For example, the gene responsible for hair color has two main alleles: red and brown. The four possibilities are thus

Brown/red, which would result in brown hair, Red/red, resulting in red hair, Brown/brown, resulting in brown hair, or Red/brown, resulting in red hair.

The genetic codes for red and brown can be either dominant or recessive. In any case, the dominant gene overrides the recessive.

When two people create a child, they each supply their own set of genes. In simplistic cases, such as the red/brown hair, each parent supplies one "code", contributing to the child's hair color. For example, if dad has brown/red he has a 50% chance of passing brown hair to his child and a 50% of passing red hair. When combined with a mom who has brown/brown (who would supply 100% brown), the child has a 75% chance of having brown hair and a 25% chance of having red hair. Similar rules apply to different traits and characteristics, though they are usually far more complex.

### Multifactorial inheritance

Some traits are found to be determined by genes and environmental effects. Height for example seems to be controlled by multiple genes, some are "tall" genes and some are "short" genes. A child may inherit all the "tall" genes from both parents and will end up taller than both parents. Or the child my inherit all the "short" genes and be the shortest in the family. More often than not the child inherits both "tall" and "short" genes and ends up about the same height as the rest of the family. Good diet and exercise can help a person with "short" genes end up attaining an average height. Babies born with drug addiction or alcohol addiction are a sad example of environmental inheritance. When mom is doing drugs or drinking, everything that she takes the baby takes. These babies often have developmental problems and learning disabilities. A baby born with _Fetal alcohol syndrome_ is usually abnormally short, has small eyes and a small jaw, may have heart defects, a cleft lip and palate, may suck poorly, sleep poorly, and be irritable. About one fifth of the babies born with fetal alcohol syndrome die within the first weeks of life, those that live are often mentally and physically handicapped.

  


### Sex-linked Inheritance

![X-linked recessive inheritance](../../../upload.wikimedia.org/wikipedia/commons/a/a3/XlinkRecessive.jpg)

Sex-linked inheritance is quite obvious, it determines your gender. Male gender is caused by the Y chromosome which is only found in males and is inherited from their fathers. The genes on the Y chromosomes direct the development of the male sex organs. The x chromosome is not as closely related to the female sex because it is contained in both males and females. Males have a single X and females have double XX. The X chromosome is to regulate regular development and it seems that the Y is added just for the male genitalia. When there is a default with the X chromosomes in males it is almost always persistent because there is not the extra X chromosome that females have to counteract the problem. Certain traits like colorblindness and hemophilia are on alleles carried on the X chromosome. For example if a woman is colorblind all of her sons will be colorblind. Whereas all of her daughters will be carriers for colorblindness.

### Exceptions to simple inheritance

Our knowledge of the mechanisms of genetic inheritance has grown a lot since Mendel's time. It is now understood, that if you inherit one allele, it can sometimes increase the chance of inheriting another and can affect when or how a trait is expressed in an individuals phenotype. There are levels of dominance and recessiveness with some traits. Mendel's simple rules of inheritance does not always apply in these exceptions.

  


### Polygenic Traits

Polygenic traits are traits determined by the combined effect of more than one pair of genes. Human stature is an example of this trait. The size of all body parts from head to foot combined determines height. The size of each individual body part are determined by numerous genes. Human skin, eyes, and hair are also polygenic genes because they are determined by more than one allele at a different location.

### Intermediate Expressions

When there is incomplete dominance, blending can occur resulting in heterozygous individuals. An example of intermediate expression is the pitch of a human male voice. Homozygous men have the lowest and highest voice for this trait (AA and aa). The child killer Tay- Sachs is also characterized by incomplete dominance.

### Co-dominance

For some traits, two alleles can be co-dominant. Were both alleles are expressed in heterozygous individuals. An example of that would be a person with AB blood. These people have the characteristics of both A and B blood types when tested.

### Multiple-Allele Series

There are some traits that are controlled by far more alleles. For example, the human HLA system, which is responsible for accepting or rejecting foreign tissue in our bodies, can have as many as 30,000,000 different genotypes! The HLA system is what causes the rejection of organ transplants. The multiple allele series is very common, as geneticists learn more about genetics, they realize that it is more common than the simple two allele ones.

### Modifying and Regulator Genes

Modifying and regulator genes are the two classes of genes that may have an effect on how the other genes function. _Modifying Genes_ alter how other genes are expressed in the phenotype. For example, a dominant cataracts gene may impair vision at various degrees, depending on the presence of a specific allele for a companion modifying gene. However, cataracts can also come from excessive exposure to ultraviolet rays and diabetes. _Regulator Genes_ also known as homoerotic genes, can either initiate or block the expression of other genes. They also control a variety of chemicals in plants and animals. For example, Regulator genes control the time of production of certain proteins that will be new structural parts of our bodies. Regulator genes also work as a master switch starting the development of our body parts right after conception and are also responsible for the changes in our bodies as we get older. They control the aging processes and maturation.

### Incomplete penetrates

Some genes are incomplete penetrate. Which means, unless some environmental factors are present, the effect does not occur. For example, you can inherit the gene for diabetes, but never get the disease, unless you were greatly stressed, extremely overweight, or didn't get enough sleep at night.

  


## Inherited Genetic Disease

Some of the most common inherited diseases are _hemochromatosis_, _cystic fibrosis_, _sickle cell_ _anemia_ and _hemophilia_. They are all passed along from the parents and even if the parents don't show signs of the disease they may be carriers which mean that all of the children they have may be born with the disease. There is genetic testing that may be done prenatally to determine if the baby is conflicted with one of these diseases.

  


### Hemochromatosis

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/a/a4/Autorecessive.jpg/220px-Autorecessive.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Haemochromatosis types 1-3 are inherited in an [autosomal recessive](http://en.wikibooks.org/w/index.php?title=Recessive_gene&action=edit&redlink=1) fashion.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/e/e1/Autodominant.jpg/220px-Autodominant.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Haemochromatosis type 4 is inherited in an [autosomal dominant](http://en.wikibooks.org/w/index.php?title=Autosomal_dominant&action=edit&redlink=1) fashion.

Even though most people have never heard of hemochromatosis it is the most common inherited disease. About 1 in 300 are born with hemochromatis and 1 in 9 are carriers. The main characteristic is the intake of too much iron into the inflicted body. Iron is crucial to the workings of _hemoglobin_ but too much iron is just as bad as too little iron. With hemochromatosis deposits of iron form on almost every major organ especially the liver, heart and pancreas, which causes complete organ failure. Hemochromatosis patients usually absorb two or three times the iron that is needed for normal people. Hemochromatosis was first discovered in 1865 and most patients have Celtic ancestry dating back 60 or 70 generations.

  


#### Treatments for hemochromatosis

The most common treatment for hemochromatosis is to induce anemia and maintain it until the iron storage is reduced. This is done by therapeutic phlebotomy. Phlebotomy is the removal of a unit of blood (about 500 mls.) This must be done one to two times a week and can take weeks, months, or years to complete. After this treatment some patients will never have to do it again and others will have to do it many times over the course of their life. Patients who undergo their recommended treatments usually go on to live a long and healthy life. Patients who decide against treatment increase their chances of problems such as organ failure -- or even death. Along with phlebotomy treatment, patients should stick to a low iron diet and should not cook with iron cookware.

### Cystic Fibrosis (CF)

Cystic fibrosis is a disease that causes thick, sticky mucus to build up in the lungs and digestive tract. It is the most common lung disease in children and young adults and may cause early death. The mucus builds up in the breathing passages of the lungs and in the pancreas. The build up of the mucus results in terrible lung infections and digestion problems. Cystic fibrosis may also cause problem with the sweat gland and a man's reproductive system. There are more than 1,000 mutations of the CF gene, symptoms vary from person to person. The most common symptoms are: No bowel movements for the first 24 to 48 hours of life, stools that are pale or clay colored, foul smelling or that float, infants that have salty-tasting skin, recurrent respiratory infections like pneumonia, coughing or wheezing, weight loss or low weight gain in childhood, diarrhea, delayed growth, and excessive fatigue. Most patients are diagnosed by their first birthday but less severe cases sometimes aren't caught until after 18 years of age. 40% of patients are over 18 years old and the average life span of CF patients is about 35 years old, which is a huge increase over the last 30 years. Patients usually die of lung complications.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/0/09/CFTR.jpg/350px-CFTR.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

**CFTR protein -** Molecular structure of the CFTR protein

#### Treatment for cystic fibrosis

In 2005 the U.S food and drug administration approved the first DNA based blood test to help detect CF. Other tests to help detect CF include: Sweat chloride test, which is the standard test for CF. High salt levels in the patients sweat is an indication of CF, Fecal fat test, upper GI and small bowel series, and measurements of pancreatic function. After a diagnosis has been made there are a number of treatments available, these include: Antibiotics for respiratory infections, pancreatic enzyme replacement, vitamin supplements (mostly A, D, E, and K), inhalers to open the airways, enzyme replacement therapy which makes it easier to cough up the mucus, pain relievers, and in very severe cases, lung transplants.

### Sickle cell anemia

Sickle cell anemia is an inherited disease of the red blood cells which causes abnormally shaped red cells. A typical red blood cell has about 270 million hemoglobin molecules, which bind with oxygen. In a person with sickle cell disease, one amino acid is changed in the hemoglobin molecule, and the end result is misshapen red blood cells. In a patient with sickle cell disease the red blood cells change from the normal round shape to the shape of a sickle or "C" shaped. The abnormal shape causes the cells to get stuck in some blood vessels which causes blockage in the vessel. This causes pain and can destroy organs because of the lack of oxygen. Sickle cells live only 10 to 20 days and a normal cell lives about 120 days.

![](../../../upload.wikimedia.org/wikipedia/commons/9/92/Sicklecells.jpg)

Red blood cells with sickle-cell mutations.

This rapid death of blood cells leads to chronic anemia. Complications can include severe pain, terrible infection, swelling of the feet and hands, stroke, damage to the eyes, and damaged body organs. These effects can vary from person to person depending on the type of sickle cell disease they have. Some patients are mostly healthy and others are in the hospital more than they are out. Thanks to diagnosis and treatment advancements, most children born with sickle cell grow up to have a normal and relatively healthy life. The form of sickle cell is determined by which genes they inherit from the parents. When a child inherits a sickle cell gene (hemoglobin gene) from each parent it is called hemoglobin SS disease ( which is the formal name for sickle cell). When a child inherits a sickle cell gene from one parent and a different abnormal gene from the other parent, it is a form of disease called hemoglobin SC disease or hemoglobin S-thalassemia. If a child inherits a normal gene from one parent and a sickle cell gene from the other, the child will not have sickle cell but will be a carrier and may pass it to their children. Sickle cell affects mostly African Americans and some Latino Americans. A person who is a carrier (has one copy of the gene) is resistant to malaria. This heterozygote advantage explains why the gene is more common in people in equatorial regions, or who are descendants of such people (such as African Americans).

#### Treatment for Sickle cell anemia

Sickle cell is diagnosed at birth with a simple blood test. If the first blood test is positive then a second test is done just for confirmation. Because of the high risk of infections that occur with sickle cell, early diagnosis is very important. Other than a bone marrow transplant there is no known cure for sickle cell. Bone marrow transplants have a high risk of rejection and aren't an available option for every patient. The patient would need a bone marrow donor match with a low risk of rejection. Even without a cure, with the use of pain medications and antibiotic treatments, children with sickle cell can live a long and happy life. Blood transfusions are sometimes used to treat episodes of severe pain. For adults who have recurrent pain episodes (at least 3 yearly), a cancer drug, hydroxyurea (marketed as Droxia), has been approved to relieve symptoms. It appears to work by increasing the flexibility of sickle cells.

### Hemophilia

About two thirds of people who have Hemophilia have inherited it. For the other third, there is no known cause for possessing the disorder. There are two types of hemophilia, Type A and Type B. Both are caused by a low level or a complete absence of protein in the blood. Without this protein, blood is not able to clot.

Some of the symptoms of Hemophilia are bleeding in the joints, knees, and ankles. Stiffness without pain in the joints, stiffness with a lot of warmth,(most ability for movement is lost due to swelling) blood in the urine or stool, excessive bleeding after surgery or loosing a tooth, excessive bruising, abnormal menstrual bleeding, and nose bleeds that last for long periods of time.

Hemophiliacs blood does not coagulate like a normal persons. Coagulation controls bleeding, it changes blood from a liquid to a solid. Within seconds of a cut or scrape, platelets, calcium and other tissue factors start working together to form a clot. Over a short time the clot strengthens and then dissolves as the injury heals. Hemophiliacs are missing the clotting factor, or it isn't working correctly which causes them to bleed for a longer time. The most common myth is that a person with a bleeding disorder will bleed to death from a minor wound or that their blood flows faster than somebody without a bleeding disorder. Some of the risks hemophilia are: Scarring of the joints or joint disease, vision loss from bleeding of the eyes, chronic anemia from blood loss, a neurological or psychiatric problem, death which may occur from large amounts of blood loss or bleeding in the brain or other vital organs. Most cases of hemophilia are caused from inherited disorders but sometimes people can get it from vitamin K deficiency, liver disease, or treatments like prolonged use of antibiotics or anti coagulation drugs. Hemophilia is the best known bleeding disorder and it has had the most research done on it, so hemophiliacs have a slight advantage over people with other bleeding disorders.

#### Treatment for hemophilia

To treat Hemophilia, a Clotting Factor is needed. It is in the shape of powder kept in a small, sterile glass bottle. It has to be kept in the fridge. When needed, The Clotting Factor is mixed with sterile water, then one minute later it can be injected into a vein. It may also be mixed with a large amount of water and injected through an IV.

There are over 140 centers that specialize in hemophilia. Most of these centers are "Comprehensive Care Facilities". Comprehensive care facilities provide all the services needed by a hemophiliac and their family. Services provided include: Primary physician, nurse coordinator, physiotherapist, and dentist. Hemophiliacs require a special dentist because of the higher risk of bleeding. It is recommended that hemophiliacs go to the treatment centers twice a year for a complete check-up.

The basic and most common treatment for patients with hemophilia A and B is factor replacement therapy. Factor replacement therapy is the IV injection of Factor VIII and IX concentrates which help control bleeding. This concentrate comes from two sources: human plasma and genetically engineered cells made by DNA technology. This concentrate is what the hemophiliac is lacking in their own genes. After the injection is given the patients blood becomes "normal" for a couple of hours which gives time for a clot to from at the site of a damaged blood vessel. This treatment is not a permanent cure, within about 3 days there is no trace left in the system. Today's Factor treatments are much more concentrated than they were in the past so very little is required even if the patient is going in for major surgery or has a major injury. Treatments are also very convenient, they can be stored at home in the fridge for up to 6 months. So if the patient is injured they don't need to go to the hospital they can give themself an injection at home. After the injection it only takes about 15-20 minutes for the clotting process to begin. There is a risk of contracting other disease such as AIDS from Factor VIII that is made from human plasma, but as technology gets better the cases of AIDS has dropped. There is no possibility of contracting diseases from genetic engineering Factor VIII.

Hemophiliacs can live a long life. The most common reason for early death among patients has been from AIDS related complications.

## Non-heritable Genetic Disorders

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/a/ab/21_trisomy_-_Down_syndrome.png/350px-21_trisomy_-_Down_syndrome.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Karyotype of 21 trisomy-Down syndrome

Any disorder caused totally or in part by a fault (or faults) of the genetic material passed from parent to child is considered a genetic disorder. The genes for many of these disorders are passed from one generation to the next, and children born with a heritable genetic disorder often have one or more extended family members with the same disorder. There are also genetic disorders that appear due to spontaneous faults in the genetic material, in which case a child is born with a disorder with no apparent family history.

**Down Syndrome,** also known as Trisomy 21, is a chromosome abnormality that effects one out of every 800-1000 newborn babies. During anaphase II of meiosis the sister chromatids of chromosome 21 fail to separate, resulting in an egg with an extra chromosome, and a fetus with three copies (trisomy) of this chromosome. At birth this defect is recognizable because of the physical features such as almond shaped eyes, a flattened face, and less muscle tone than a normal newborn baby. During pregnancy, it is possible to detect the Down Syndrome defect by doing amniocentesis testing. There is a risk to the unborn baby and it is not recommended unless the pregnant mother is over the age of thirty-five. Other non-lethal chromosomal abnormalities include additional osex chromosome abnormalities which is when a baby girl (about 1 in 2,500)is born with one x instead of two (xx) this can cause physical abnormalities and defective reproduction systems. Boys can also be born with extra X's (XXY or XXXY) which will cause reproductive problems and sometimes mental retardation.

**Chromosomal Abnormalities** In most cases with a chromosomal abnormality all the cells are affected. Defects can have anywhere from little effect to a lethal effect depending on the type of abnormality. Of the 1 in 200 babies born having some sort of chromosomal abnormality, about 1/3 of these results in spontaneous abortion. Abnormalities usually form shortly after fertilization and mom or dad usually has the same abnormality. There is no cure for these abnormalities. Tests are possible early in pregnancy and if a problem is detected the parents can choose to abort the fetus.

## Mutant Genes

Mutation is a permanent change in a segment of DNA.

Mutations are changes in the genetic material of the cell. Substances that can cause genetic mutations are called mutagen agents. Mutagen agents can be anything from radiation from x-rays, the sun, toxins in the earth, air, and water viruses. Many gene mutations are completely harmless since they do not change the amino acid sequence of the protein the gene codes for.

Mutations can be good, bad, or indifferent. They can be good for you because their mutation can be better and stronger than the original. They can be bad because it might take away the survival of the organism. However, most of the time, they are indifferent because the mutation is no different than the original.

The not so harmless ones can lead to cancer, birth defects, and inherited diseases. Mutations usually happen at the time of cell division. When the cell divides, one cell contracts a defect, which is then passed down to each cell as they continue to divide.

Teratogens refers to any environmental agent that causes damage during the prenatal period. Examples of Common Teratogens:

  * drugs: prescription, non-prescription, and illegal drugs
  * tobacco, alcohol,
  * radiation,
  * environmental pollution,
  * infectious disease,
  * STD's,
  * Aids,
  * Parasites,

Sensitive period to teratogen exposure, in the embryonic period is most vital. Fetal damage is minor.

## Genetic Engineering

![Wikipedia-logo.png](../../../upload.wikimedia.org/wikipedia/commons/thumb/6/63/Wikipedia-logo.png/40px-Wikipedia-logo.png)

[Wikipedia](http://en.wikipedia.org/wiki/) has related information at _**[Genetic Engineering**_](http://en.wikipedia.org/wiki/Genetic_Engineering)

Genetic Engineering is where the DNA or gene gets changed by a scientist to make a gene with the characteristics that they want it to have, and to get rid of the characteristics that they don't want the gene to have. This process can be applied towards any plant, animal, or person.

The main reason for genetic engineering is to "mass produce" a certain protein. Each cell is responsible for producing a certain protein and these proteins can be used for medical treatment and diagnosis. The job of each gene is to control the production of a particular protein in a living cell. If the gene responsible for _synthesizing_ a important or useful protein can be found, and if that gene can be inserted into another cell that can be made to reproduce, then a colony of cells containing that gene can be grown and the protein will be manufactured in large quantities. This process is responsible for insulin and growth hormones and it is also used in vaccines to help prevent hepatitis and a treatment to help prevent viral infections. It also helps in genetically engineering Factor VIII which is a treatment for hemophilia.

The first step is to find the gene in the DNA of a cell that is responsible for the manufacturing of the desired protein. Then that gene is either extracted or the exact chemical structure is figures out to be synthesized. The last step is to insert the DNA into the recipient which is done by using special enzymes to split a molecule of the recipients cell and inserting the new gene.

There have been many steps taken to bring technology closer to being able to fix genetically inherited diseases. Hopefully someday there will be a lot less babies born with genetic diseases and disorders.

## Gene Therapy

Gene therapy is a way to correct the defective genes that are the cause of disease development. When the genes are altered proteins are not able to function normally and as a result of this, defects can occur. Current gene therapy is still being experimented with, but in some cases it is very effective.

Genes are carried on chromosomes and are the basic physical and functional parts of hereditary. When there is a genetic disorder, gene therapy can help fix the problem either permanently or at least temporarily. The most common form of gene therapy is to insert a gene into a nonspecific place to replace a malfunctioning gene. Another method is gene swapping, where an abnormal gene is replaced by a normal gene. Genes could also be repaired through "selective reverse mutation" which returns the gene to it's original function. The degree to which a gene is turned on or off can also be altered.

Gene therapy works on the principle belief that a virus genome can be manipulated to remove disease causing genes and new therapeutic genes can be inserted in their place. These new genes are called gene therapy vectors. (The virus container is the vector and the new gene is the payload.)

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/3/3d/Gene_therapy.jpg/340px-Gene_therapy.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Gene therapy using an [Adenovirus](http://en.wikibooks.org/w/index.php?title=Adenovirus&action=edit&redlink=1) vector. A new gene is inserted into an adenovirus vector, which is used to introduce the modified [DNA](http://en.wikibooks.org/w/index.php?title=DNA&action=edit&redlink=1) into a human cell. If the treatment is successful, the new gene will make a functional [protein](http://en.wikibooks.org/w/index.php?title=Protein&action=edit&redlink=1).

A few of the different viruses used as gene therapy vectors are: **Retroviruses** \- A class of viruses that can create double-stranded DNA copies of their original RNA genomes. Theses copies of its genomes can be mixed into the chromosomes of "host" cells. HIV is a type of retrovirus. **Adenoviruses** \- A class of viruses with double-stranded DNA genome that cause respiratory, intestinal, and eye infections in humans. The common cold is an adenovirus. Adeno-associated viruses - A class of small, single-stranded DNA viruses that can insert their genetic material at a specific site on chromosome 19. (chromosome 19 represents about 2% of the human genome and contains about 1,500 genes. Some of the genes included are genes that code for insulin-dependent diabetes, myotonic dystrophy, migraines, and inherited high blood cholesterol). A class of double-stranded DNA viruses that infect a particular cell type, neurons, called **Herpes simplex viruses** is another common virus used in gene therapy. It is the virus that causes cold sores.

Major advancements have been made in gene therapy. There are many new discoveries in helping cure and treat diseases that claim millions of lives. Some of the disease that have cures or treatments because of gene therapy include: Parkinson's, Huntington's, Cystic Fibrosis, Some cancers, "Bubble Boy" syndrome and sickle cell. With technology jumping ahead, maybe someday there will be a cure for every life threatening disease.

## Genetic Regulation of Development and Homeostasis

It is very easy to think of Genetics as why I have blue eyes while both of my parents have brown eyes. Or how hemophilia is passed down from mother to son, and not mother to daughter. But Genetics is more in depth than that. At conception you started as a single cell. That cell started to divide. You didn’t increase in mass just in the number of cells. Once the bundle of cells reached a certain number, things changed. You started gaining mass by acquiring new resources (from your mother) and increasing in cell number. Your cells specialized. Some cells became the liver. Others became heart, lungs, brain, and so forth. Why is this? How did that little bundle of cells "know" when it was time to specialize? It is because your DNA has regulatory control over your entire system. If it didn’t, that bundle of cells would just keep dividing as undifferentiated cells and never specialize, never gain form or function. Thanks to the genetic regulatory control over your system, your anatomy forms correctly with everything in its proper place. Even after fetal development gene regulation still controls what each cell produces and how it functions. Puberty just doesn’t happen at the age of twelve. Puberty happens because genes in your genetic code are triggered by your growth and development, causing your endocrine system to start producing the proper hormones, thus causing you to mature sexually.

![Wikipedia-logo.png](../../../upload.wikimedia.org/wikipedia/commons/thumb/6/63/Wikipedia-logo.png/40px-Wikipedia-logo.png)

[Wikipedia](http://en.wikipedia.org/wiki/) has related information at _**[Regulation of gene expression**_](http://en.wikipedia.org/wiki/Regulation_of_gene_expression)

Even aging is genetically controlled. The mechanisms of genetic regulation are not discussed here, but it is worth noting that any step of gene expression may be modulated, from the DNA-RNA transcription step to post-translational modification of a protein. Gene regulation gives the cell control over structure and function, and is the basis for cellular differentiation. A cell can also respond to changes in its environment by altering gene expression. For example, a pancreatic cell exposed to high glucose levels releases pre-formed insulin that it was storing. Yet, if the high levels of glucose continue, the cell will transcribe additional copies of the gene for making insulin and thus increase insulin production to meet demand. This is homeostasis in action.

## Glossary

**Allele:** one member of a pair of genes that occupy a specific position on a specific chromosome

**Autosome:** chromosome that is not a sex chromosome

**Chromosome:** threadlike strand of DNA and associated proteins in the nucleus of cells that carries the genes and functions in the transmission of heredity information

**Cystic Fibrosis:** recessive genetic disorder affecting the mucus lining of the lungs, leading to breathing problems and other difficulties

**Fetal Alcohol Syndrome:** combination of birth defects resulting form high (sometimes low) alcohol consumption by the mother during pregnancy

**Gene:** is a segment of nucleic acid that contains the information necessary to produce a functional product, usually a protein.

**Genetics:** is the science of genes, heredity, and the variation of organisms.

**Genome:** complete set of genetic information of an organism including DNA and RNA

**Genotype:** actual set of genes an organism has. It is the blue print of gentic material.

**Hemochromatosis:** metabolic disorder that causes increased absorption of iron, which is deposited in the body tissues and organs; the iron accumulates in the body where it may become toxic and causes damage

**Hemoglobin:** component of red blood cells that carries oxygen

**Hemophilia:** group of heredity disorders in which affected individuals fail to make enough of certain proteins needed to form blood clots

**Inheritance:** characteristics given to a child by a parent

**Modifying Gene:** alters how other genes are expressed in the phenotype

**Multifactorial Inheritance:** trait or disorder determined by multiple genes and/or environmental effects

**Phenotype:** organisms physical appearance

**Polygenic:** trait whose expression is influenced by more than one gene

**Regulator Genes:** initiate or block the expression of other genes.

**Sex-linked:** pertaining to a trait of a disorder determined by the sex chromosome in a persons cells or by the genes carried on those chromosomes

**Sickle Cell Anemia:** recessive disorder in which red blood cells take on an unusual shape, leading to other problems with the blood

**Synthesize:** to make using biochemical processes

**Unifactorial Inheritance:** trait or disorder determined by a single pair of genes

**Zygote:** cell formed by the union of male and female gametes. A Zygote is a cell that is the result of fertilization.

## Review questions

Answers for these questions can be found [here](http://en.wikibooks.org/wiki/Human_Physiology/Appendix_1:_answers_to_review_questions#Genetics_and_inheritance)

1\. DNA is found on

    A) endoplasmic reticulum
    B) ribosomes
    C) chromosomes
    D) cytoplasm

2\. Even though each cell has identical copies of all of the same genes, different cells __________ or ___________ different genes

    A) express, repress
    B) genotype, phenotype
    C) dominant, recessive

3\. In diploid organisms, a dominant allele on one chromosome will

    A) show the expression of a recessive allele
    B) mask the expression of a recessive allele
    C) show that there are dominant alleles on both chromosomes
    D) none of the above

4\. Transcription occurs in the

    A) cytoplasm
    B) golgi apparatus
    C) mitochondria
    D) nucleus

5\. This is the start codon and is found at the beginning of each mRNA

    A) AGU
    B) GAU
    C) UAG
    D) GUA
    E) AUG

6\. Sara was born with cystic fibrosis, from this we could assume that

    A) all of her siblings also have cystic fibrosis
    B) only her dad is a carrier
    C) only her mom is a carrier
    D) both of her parents are carriers

7\. Jesse was born with a flattened face, almond eyes and less muscle tone; it could be assumed that he has

    A) a chromosome abnormality on chromosome 21
    B) a chromosome abnormality on chromosome 19
    C) a chromosome abnormality on chromosome 20
    D) a chromosome abnormality on chromosome 22
    E) No chromosome abnormality, these are his inherited traits

8\. The most common inherited disease is

    A) hemochromatosis
    B) cystic fibrosis
    C) sickle cell anemia
    D) hemophilia
    E) all of the above

9\. Being a carrier of sickle cell anemia means that the person will

    A) also be a carrier of hemophilia
    B) be resistant to malaria
    C) have children that all have sickle cell anemia
    D) have children that all have malaria
    E) none of the above

10\. Hemophilia is

    A) a Y linked disease
    B) an XY linked disease
    C) an X linked disease

  


# Human Development

## Overview

We are born, we grow up, we age, and then we die. Unless disease or trauma occurs, most humans go through the various stages of life described above. Human Development is the process of growing to maturity. Traditionally, theories that explain senescence have generally been divided between the programmed and stochastic theories of aging. Programmed theories imply that aging is regulated by biological clocks operating throughout the life span. This regulation would depend on changes in gene expression that affect the systems responsible for maintenance, repair and defense responses. Stochastic theories blame environmental impacts on living organisms that induce cumulative damage at various levels as the cause of aging. Examples of environmental impacts range from damage to deoxyribonucleic acid (DNA), damage to tissues and cells by oxygen radicals (widely known as free radicals countered by the even more well known antioxidants), and cross-linking. However, aging is now seen as a combination of genetic and environmental processes; a progressive failure of homeostatic mechanisms involving maintenance and repair genes, stochastic events leading to molecular damage and molecular heterogeneity, and chance events determining the probability of death. Homeostasis, as we have seen throughout this book, is maintained through complex and interacting systems, and aging is considered to be a progressive shrinkage of homeostatic capabilities, mainly due to increased molecular heterogeneity. In this chapter we explore the physiology of all stages of human development, with a particular emphasis on the aging process.

## Apoptosis

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/1/1f/Apoptosis-blank.png/200px-Apoptosis-blank.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

A cell undergoing apoptosis. In just one of many scenarios of apoptosis, the process is triggered by another neighboring cell; the dying cell eventually transmits signals that tell the phagocytes, which are a part of the immune system, to engulf it.

Apoptosis is the process of regulated cell death and removal. In some cases cell damage can trigger apoptosis, but it is usually a normal function of the cell. Apoptosis results in controlled auto digestion of the cells content. The cell membrane stays in place and the cells contents are not dispersed. When this process is near completion, "eat me" signals, like phosphatidylserine, appear on the surface of the cell membrane. This in turn attracts phagocytic scavengers that complete the process of removing the dead cell without eliciting an inflammatory response. Unlike necrosis, which is a form of cell death that results from acute cellular injury, apoptosis is carried out in an ordered process that generally confers advantages during an organism's life cycle.

Apoptosis Rates
    The rate at which cells of the body die varies widely between different cell types. Some cells, such as white blood cells, live for only a matter of hours while other cells can live throughout the duration of the lifespan of the individual.

Homeostasis
    Apoptosis is a regulated function that results in a relatively consistent number of cells in the body. This balancing act is part of homeostasis (see chapter 1) that is required by living organisms. An example of this is that blood cells are constantly being produced and apoptosis takes place to eliminate a similar number of older cells. Homeostasis was discovered by Claude Bernard around the year 1851.

Development
    Apoptosis also plays a key role in growth and development. An example of how apoptosis enables development is the differentiation of human fingers in a developing embryo. Apoptosis is the function that enables the embryos fingers to separate.

Disorders
    Too much apoptosis causes cell-loss disorders such as osteoporosis, whereas too little apoptosis results in uncontrolled cell proliferation, namely cancer.

## Growth and development

Growth and development is an ongoing process that begins at conception and continues through the remainder of our lives. There is a broad spectrum of physical and psychological changes that are part of the maturation and life of the individual.

Growth is a physical change that can be weighed and measured. Development is psychological and social changes to the individual such as behaviors and thinking patterns. Growth and development are two complimentary processes that together make up the individual.

Examples of this difference are all around us. One notable example involves infants. Infants understand speech much earlier than their bodies have matured enough to physically perform it. Thus it is evident that their speech patterns develop before the physical growth of their vocal chords is adequate to facilitate speech.

The rate of development and growth varies dependent on many factors such as age and genetic disposition. Babies grow at roughly the same pace and benchmarks for their physical and social development are roughly standard. However, as any parent can tell you, no two children develop exactly within the same time frame. Thus an appropriate time span should be used. For example: one brother may gain weight faster than another. Growth spurts can vary. Some children can speak entire sentences before their first year while others may not speak at all until two or three. This creates a greater diversity among human beings.

  
The following chart focuses on reflexes of the developing infant.

Reflex Stimulation Response Age of disappearance Function

Eye blink
Bright light shinning in eyes or clap hands by eyes
closes eyelids quickly
Permanent
This reflex protects the infant from a lot of stimulation

Withdrawal
Stick sole of foot with stimulus like a pin
This cause the foot to withdraw, this occurs with the use of flexing of the knee to hip
Decreases after the 10th day of birth
This is a protection for the infant in the instance of unpleasant tactile stimulation

Rooting
Touch cheek near the corner of the mouth
The infants head will turn towards the site of stimulation
3 weeks (due to the voluntary response that is now capable for infant to do at this time)
This reflex helps baby to find the mothers nipple

Sucking
Place fingers in infant's mouth
The infant will suck finger rhythmically
4 months (voluntary sucking will come about)
This helps with feeding

Swimming
Place the baby in pool of water face down
The baby paddles and kicks in swimming movements
4 to 6 month
This helps baby to survive if dropped into the water

Moro
Hold infant in a cradling horizontal position and slightly lower the baby in a fast motion toward the ground while making a loud sound supporting the baby
The baby will make a embracing motion and arch its back extending its legs throwing its arms outward, and finally it will bring arms in toward its body
6 months
In the evolutionary past this may have helped the baby cling to the mother

Palmar grasp
Place the finger in baby's palm and press against the palm
The baby will immediately grasp the finger
3 to 4 months
This prepares infant for when voluntary grasping comes about

Tonic neck
Turn the baby's head to one side while the baby is awake
This will cause the baby to extend one arm in front of its eye on one arm to the side to which the head has been turned
4 months
This may prepare for voluntary reaching

Stepping/marching
When you hold the baby under the arm and permit the bare feet of the baby to touch a flat surface
The baby will lift one foot after the other in a stepping fashion
2 months (this applies to a baby who has gained weight a baby who is not as heavy this reflex may be submissive)
This prepares the baby for voluntary walking

Babinski
Touch the foot in a stroking manner form the toe toward the heel
The baby's toes will fan out and curl as the foot twists in
8 to 12 months
Unknown

## Neonatal

The neonatal period extends from birth to somewhere between 2 weeks and 1 month.

Immediately after the baby is born, uterine contractions force blood, fluid, and the placenta from the mother's body. The umbilical cord—the baby's lifeline to it's mother—is now severed. Without the placenta to remove waste, carbon dioxide builds up in the baby's blood. This fact, along with the actions of medical personnel, stimulates the control center in the brain, which in turn responds by triggering inhalation. Thus the newborn takes its first breath. As the newborn's lungs begin to function, the bypass vessels of fetal circulation begin to close. The bypass connecting the atria of the heart, known as the foramen ovale, normally closes slowly during the first year.

During this period the body goes through drastic physiological changes. The most critical need is for the body to get enough oxygen as well as an adequate supply of blood. (The respiratory and heart rate of a newborn is much faster than that of an adult.)

  


  


![](../../../upload.wikimedia.org/wikipedia/commons/thumb/3/38/HumanNewborn.JPG/260px-HumanNewborn.JPG)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Newborn infant, just seconds after vaginal delivery.

### The newborn's appearance

A newborn's skin is oftentimes grayish to dusky blue in color. As soon as the newborn begins to breathe, usually within a minute or two of birth, the skin's color returns to normal tones. Newborns are wet, covered in streaks of blood, and coated with a white substance known as vernix caseosa, which is believed to act as an antibacterial barrier. The newborn may also have Mongolian spots, various other birthmarks, or peeling skin, particularly at the wrists, hands, ankles, and feet.

A newborn's shoulders and hips are narrow, the abdomen protrudes slightly, and the arms and legs are relatively short. The average weight of a full-term newborn is approximately 7 ½ pounds (3.2kg), but can be anywhere from 5.5–10 pounds (2.7–4.6kg). The average total body length is 14–20 inches (35.6–50.8cm), although premature newborns may be much smaller. The Apgar score is a measure of a newborn's transition from the womb during the first ten minutes of life.

A newborn's head is very large in proportion to the rest of the body, and the cranium is enormous relative to his or her face. While the adult human skull is about 1/8 of the total body length, the newborn's is twice that. At birth, many regions of the newborn's skull have not yet been converted to bone. These "soft spots" are known as fontanels; the two largest are the diamond-shaped anterior fontanel, located at the top front portion of the head, and the smaller triangular-shaped posterior fontanel, which lies at the back of the head.

During labor and birth, the infant's skull changes shape to fit through the birth canal, sometimes causing the child to be born with a misshapen or elongated head. This will usually return to normal on its own within a few days or weeks. Special exercises sometimes advised by physicians may assist the process.

Some newborns have a fine, downy body hair called lanugo. It may be particularly noticeable on the back, shoulders, forehead, ears and face of premature infants. Lanugo disappears within a few weeks. Likewise, not all infants are born with lush heads of hair. Some may be nearly bald while others may have very fine, almost invisible hair. Some babies are even born with a full head of hair. Amongst fair-skinned parents, this fine hair may be blond, even if the parents are not. The scalp may also be temporarily bruised or swollen, especially in hairless newborns, and the area around the eyes may be puffy.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/e/e5/Newborn_minutes_after_delivery.jpg/260px-Newborn_minutes_after_delivery.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Traces of vernix caseosa on a full term newborn

A newborn's genitals are enlarged and reddened, with male infants having an unusually large scrotum. The breasts may also be enlarged, even in male infants. This is caused by naturally-occurring maternal hormones and is a temporary condition. Females (and even males) may actually discharge milk from their nipples, and/or a bloody or milky-like substance from the vagina. In either case, this is considered normal and will disappear in time.

The umbilical cord of a newborn is bluish-white in color. After birth, the umbilical cord is normally cut, leaving a 1–2 inch stub. The umbilical stub will dry out, shrivel, darken, and spontaneously fall off within about 3 weeks. Occasionally, hospitals may apply triple dye to the umbilical stub to prevent infection, which may temporarily color the stub and surrounding skin purple.

Newborns lose many of the above physical characteristics quickly. Thus prototypical older babies look very different. While older babies are considered "cute", newborns can be "unattractive" by the same criteria and first time parents may need to be educated in this regard.

Neonatal jaundice
    Neonatal jaundice is usually harmless: this condition is often seen in infants around the second day after birth, lasting until day 8 in normal births, or to around day 14 in premature births. Serum Bilirubin initially increase because a newborn does not need as many red blood cells as it did as a fetus (since there is a higher concentration of oxygen in the air than what was available through the umbilical vein). The newborn's liver processes the breakdown of the extra red blood cells, but some bilirubin does build up in the blood. Normally bilirubin levels drop to a low level without any intervention required. In babies where the bilirubin levels are a concern (particularly in pre-term infants), a common treatment is to use UV lights ("bili lights") on the newborn baby.

## Changes in body Size and Muscle fat makeup

By the end of the first year an infant's height is increased by 50% and by the age of 2 the baby will have grown 75% greater.

By 5 months a baby will have doubled its weight, and tripled its weight by the first year. By the age of 2, a baby's weight will have quadrupled.

Infants and toddlers grow in little spurts over the first 21 months of life. A baby can go through periods of 7 to 63 days with no growth but they can add as much as an inch in one 24 hour period. During the day before a growth spurt, parents describe their babies as irritable and very hungry.

The best way to estimate a child's physical maturity is to use _skeletal age_, a measure of bone development. This is done by having a x-ray of the long bones of the body to see the extent to which soft, pliable cartilage has hardened into bone.

  


  * Changes in body Proportions

**Cephalocaudal trend** means that growth occurs from head to tail. The head develops more rapidly than the lower part of the body. At birth the head takes up to one fourth of the total body length and legs only one third. The lower body catches up by age 2 and the head accounts for only one fifth and legs for nearly one half of the body length.

**Proximodistal trend** means that head growth proceeds literally from near to far or from center of the body outward.

At birth the brain is nearer its adult shape and size than any other physical structure. The brain continues to develop at an astounding pace throughout infancy and toddlerhood.

  


  * The Brain Development

The neurons of infants and adults differ in 2 significant ways: Growth of neural fibers and synapses increases connective structures. When synapses are formed, many surrounding neurons die. This occurs in 20 to 80 percent of the brain region.

**Dendrites synapses**: Synapses are tiny gaps between neurons where fiber from different neurons come close together but do not touch. Neurons release chemicals that cross the synapses sending messages to one another. During the prenatal period the neural tube produces far more neurons than the brain will ever need. **Myelinization**: The coating of neural fibers with a fatty sheath called myelin that improves the efficiency of message transfer. Multi-layered lipid cholesterol and protein covering produced by neuralgia cause a rapid gain in overall size of brain due to neural fibers and myelination.

**Synaptic pruning**: Neurons seldom stimulated soon loose their synapses. Neurons not needed at the moment return to an uncommitted state so they can support future development. However, if synaptic pruning occurs in old age neurons will lose their synapses. If neurons are stimulated at a young age, even though neurons were pruned, they will be stimulated again.

**Cerebral Cortex**: Surrounding the brain, it is the largest most complex brain structure. The cortex is divided into four major lobes: occipital lobe, parietal lobe, temporal lobe, and frontal lobe which is the last to develop.

**Brain plasticity**: The brain is highly plastic. Many areas are not yet committed to specific functions. If a part of the brain is damaged, other parts can take over tasks that they would not normally have handled.

  * Changing states of Arousal

How children develop more regular “sleep patterns” around 4 to 6 months of age: Sleep patterns are more developed as the brain develops. It is not until the first year of life that the secretion of _melatonin_, a hormone produced in the brain, affects more drowsiness in the night than in the day. In addition, REM is decreased.

## Infancy

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/8/8e/Black_&_White_004.jpg/220px-Black_&_White_004.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Infant

Infancy is the period that follows the neonatal period and includes the first two years of life. During this time tremendous growth, coordination and mental development occur. Most infants learn to walk, manipulate objects and can form basic words by the end of infancy. Another characteristic of infancy is the development of deciduous teeth.

**Deciduous Teeth**

_Deciduous teeth_, otherwise known as milk teeth, baby teeth, or primary teeth, are the first set of teeth in the growth development of humans and many other animals. They develop during the embryonic stage of development and erupt - become visible in the mouth - during infancy. They are usually lost and replaced by permanent teeth, but in the absence of permanent replacements, they can remain functional for many years. (Concise)

Deciduous teeth start to form during the embryo phase of pregnancy. The development of deciduous teeth starts at the sixth week of development as the dental lamina. This process starts at the midline and then spreads back into the posterior region. By the time the embryo is eight weeks old, there are ten areas on the upper and lower arches that will eventually become the deciduous dentition. These teeth will continue to form until they erupt in the mouth. In the deciduous dentition there are a total of twenty teeth: five per quadrant and ten per arch. In most babies the eruption of these teeth begins at the age of six months and continues until twenty-five to thirty-three months of age. The first teeth seen in the mouth are the mandibular centrals and the last are the maxillary second molars. However it is not unheard of for a baby to be born with teeth.

![](../../../upload.wikimedia.org/wikipedia/commons/d/de/Gray1004.png)

Deciduous teeth.

The deciduous dentition is made up of centrals, laterals, canines, first molars, and second molars; there is one in each quadrant, making a total of four of each tooth. All of these are replaced with a permanent counterpart except for the first and second molars; they are replaced by premolars. These teeth will remain until the age of six. At that time, the permanent teeth start to appear in the mouth resulting in mixed dentition. The erupting permanent teeth causes root resorption, where the permanent teeth push down on the roots of the deciduous teeth causing the roots to be dissolved and become absorbed by the forming permanent teeth. The process of shedding deciduous teeth and the replacement by permanent teeth is called exfoliation. This will last from age six until age twelve. By age twelve there are only permanent teeth remaining.

Deciduous teeth are considered essential in the development of the oral cavity by dental researchers and dentists. The permanent teeth replacements develop from the same tooth bud as the deciduous teeth; this provides a guide for permanent teeth eruption. Also the muscles of the jaw and the formation of the jaw bones depend on the primary teeth in order to maintain the proper space for permanent teeth. The roots of deciduous teeth provide an opening for the permanent teeth to erupt through. These teeth are also needed in the development of a child’s ability to speak and chew their food correctly.

## Adolescence

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/8/8f/High_school_students.jpg/250px-High_school_students.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

American high school students

  
Adolescence is the period of psychological and social transition between childhood and adulthood. Adolescence is the transitional stage of human development in which a juvenile matures into an adult. This transition involves biological, social, and psychological changes, though the biological ones are the easiest to measure objectively. The time is identified with dramatic changes in the body, along with developments in a person's psychology and academic career. In the onset of adolescence, children usually complete elementary school and enter secondary education, such as middle school or high school. A person between early childhood and the teenage years is sometimes referred to as a pre-teen or 'tween.

Physical maturation resulting from puberty leads to an interest in sexual activities, sometimes leading to teenage pregnancy. Since teens may not be emotionally or mentally mature enough or financially able to support children, sexual activity among adolescents is considered problematic.

At this age there is also a greater probability of drug and alcohol use, mental health disorders such as schizophrenia, eating disorders such as anorexia, and clinical depression. The unstable emotions or lack of emotional intelligence among some adolescents may also lead to youth crime.

Searching for a unique identity is one of the problems that adolescents often face. Some, but not all, teenagers often challenge the authority or the rules as a way to establish their individuality. They may crave adulthood and be eager to find their place in society. While adolescents are eager to grow up and be treated like adults, they also idolize athletes, movie stars and celebrities. They want to be like these role models - whether or not these role models actually have qualities that should be aspired to.

### Female

In females, puberty is caused by alterations in brain functions that result in increased secretion by the hypothalamus of gonadotropin-releasing hormone (GnRH). Increased levels of GnRH stimulate secretion of pituitary gonadatrophins FSH and LH which cause follicle development and estrogen secretion. Estrogen is responsible for accessory sex organs and secondary sex characteristics. Menarche, the first menstrual cycle, occurs at about 12.5 years of age as a result of the release of FSH.

**Breast development**

The first physical sign of puberty in girls is usually a firm, tender lump under the center of the areola(e) of one or both breasts, occurring on average at about 10.5 years. This is referred to as thelarche. By the widely used Tanner staging of puberty, this is stage 2 of breast development (stage 1 is a flat, prepubertal breast). Within 6-12 months, the swelling has clearly begun in both sides, softened, and can be felt and seen extending beyond the edges of the areolae. This is stage 3 of breast development. By another 12 months (stage 4), the breasts are approaching mature size and shape, with areolae and papillae forming a secondary mound. In most young women, this mound disappears into the contour of the mature breast (stage 5), although there is so much variation in sizes and shapes of adult breasts that distinguishing advanced stages is of little clinical value.

**Pubic hair in girls**

Pubic hair is often the second unequivocal change of puberty. It is referred to as pubarche and the pubic hairs are usually visible first along the labia. The first few hairs are described as Tanner stage 2. Stage 3 is usually reached within another 6–12 months, when the hairs are too numerous to count and appear on the mons as well. By stage 4, the pubic hairs densely fill the "pubic triangle." Stage 5 refers to spread of pubic hair to the thighs and sometimes as abdominal hair upward towards the umbilicus. In about 15% of girls, the earliest pubic hair appears before breast development begins.

  
**Vagina, uterus, ovaries**

The mucosal surface of the vagina also changes in response to increasing levels of estrogen, becoming thicker and a duller pink in color (in contrast to the brighter red of the prepubertal vaginal mucosa). Whitish secretions (physiologic leukorrhea) are a normal effect of estrogen as well. In the next 2 years following thelarche, the uterus and ovaries increase in size. The ovaries usually contain small cysts visible by ultrasound.

  
**Menstruation and fertility**

![Wikipedia-logo.png](../../../upload.wikimedia.org/wikipedia/commons/thumb/6/63/Wikipedia-logo.png/40px-Wikipedia-logo.png)

[Wikipedia](http://en.wikipedia.org/wiki/) has related information at _**[Menarche**_](http://en.wikipedia.org/wiki/Menarche)

The first menstrual bleeding is referred to as **menarche**. The average age of menarche in American girls is about 12.7 years, usually about 2 years after thelarche. Menses (menstrual periods) are not always regular and monthly in the first 2 years after menarche. Ovulation is necessary for fertility, and may or may not accompany the earliest menses. By 2 years after menarche, most girls are ovulating at least several times a year. Over 90% of girls who experience menarche before age 13 years are experiencing very regular, predictable menses accompanied by ovulation within 2 years, and a higher proportion of those with later menarche may not establish regular ovulation for 4 years or more. However, initiation of ovulation after menarche is not inevitable, and a high proportion of girls with continued irregularity several years from menarche will continue to have prolonged irregularity and anovulation, and are at higher risk for reduced fertility.

**Pelvic shape, fat distribution, and body composition**

During this period, also in response to rising levels of estrogen, the lower half of the pelvis widens. This prepares the body for the time when she will give birth by enlarging the birth canal. Fat tissue increases to a greater percentage of the body composition than in males, especially in the typical female distribution of breasts, hips, and thighs. This produces the typical female body shape. Also, the fat goes to the buttocks of a girl, giving their buttocks more shape and curve.

**Body and facial hair in girls**

In the months and years following the appearance of pubic hair, other areas of skin which respond to androgens develop heavier hair (androgenic hair) in roughly the following sequence: underarm (axillary) hair, perianal hair, upper lip hair, sideburn (preauricular) hair, and periareolar hair. Arm and leg hair becomes heavier gradually over a period of 10 years or more. While the appearance of hair in some of these areas is not always wanted, particularly in Western culture, it rarely indicates a hormone imbalance unless it occurs elsewhere as well, such as under the chin and in the midline of the chest.

**Height growth in girls**

The estrogen-induced pubertal growth spurt in girls begins at the same time the earliest breast changes begin, or even a few months before, making it one of the earliest manifestations of puberty in girls. Growth of the legs and feet accelerates first, so that many girls have longer legs in proportion to their torso in the first year of puberty. The rate of growth tends to reach a peak velocity (as much as 7.5-10 cm or 3-4 inches per year) midway between thelarche and menarche and is already declining by the time menarche occurs. In the 2 years following menarche most girls grow about 5 cm (2 inches) before growth ceases at maximal adult height. This last growth primarily involves the spine rather than the limbs.

**Body odor, skin changes, and acne**

![](../../../upload.wikimedia.org/wikipedia/commons/d/da/AcneVulgarisUSMIL.jpg)

**Acne** on the face and body.

Rising levels of androgens can change the fatty acid composition of perspiration, resulting in a more "adult" body odor. This often precedes thelarche and pubarche by 1 or more years. Another androgen effect is increased secretion of oil (sebum) from the skin. This change increases the susceptibility to acne vulgaris, a characteristic affliction of puberty greatly variable in its severity.

### Male

The onset of puberty for males is similar to that of females. GnRH secretion from the hypothalamus results in an increase in pituitary gonadatropins secretion of LH / ICSH and FSH. The pituitary gonadatropins stimulate the seminiferous tubules and testosterone secretion. Testosterone causes changes in the accessory reproductive organs, secondary sex characteristics and male sex drive.

**Testicular size, function, and fertility**

In boys, testicular enlargement is the first physical manifestation of puberty. It is termed gonadarche. The testes in prepubertal boys change little in size from about 1 year of age to the onset of puberty, averaging about 2–3 cc in volume and about 1.5-2 cm in length. Testicular size continues to increase throughout puberty, reaching maximal adult size about 6 years later. While 18-20 cc is reportedly an average adult size, there is wide variation in the normal population.

The testes have two primary functions: to produce hormones and to produce sperm. The Leydig cells produce testosterone (as described below), which in turn produces most of the changes of male puberty. However, most of the increasing bulk of testicular tissue is spermatogenic tissue (primarily Sertoli and interstitial cells). The development of sperm production and fertility in males is not as well documented. Sperm can be detected in the morning urine of most boys after the first year of pubertal changes (and occasionally earlier).

  
**Genitalia**

A boy's penis grows little from the fourth year of life until puberty. Average prepubertal penile length is 4 cm. The prepubertal genitalia are described as stage 1. Within months after growth of the testes begins, rising levels of testosterone promote growth of the penis and scrotum. This earliest discernible beginning of pubertal growth of the genitalia is referred to as stage 2. The penis continues to grow until about 21 years of age, reaching an average adult size of about 7-15.5 cm.

Although erections and orgasms occur in prepubertal boys, they become much more common during puberty, accompanied by a markedly increased libido. Ejaculation becomes possible early in puberty; prior to this boys may experience dry orgasms. Emission of seminal fluid may occur due to masturbation or spontaneously during sleep (commonly termed a wet dream, and more clinically called a nocturnal emission). The ability to ejaculate is a fairly early event in puberty compared to the other characteristics. However, in parallel to the irregularity of the first few periods of a girl, for the first one or two years after a boy's first ejaculation, his seminal fluid may contain few active sperm.

  
**Pubic hair in boys**

Pubic hair often appears on a boy shortly after the genitalia begin to grow. As in girls, the first appearance of pubic hair is termed pubarche and the pubic hairs are usually first visible at the dorsal (abdominal) base of the penis. The first few hairs are described as stage 2. Stage 3 is usually reached within another 6–12 months, when the hairs are too numerous to count. By stage 4, the pubic hairs densely fill the "pubic triangle." Stage 5 refers to spread of pubic hair to the thighs and upward towards the umbilicus as part of the developing abdominal hair.

**Body and facial hair in boys**

In the months and years following the appearance of pubic hair, other areas of skin which respond to androgens develop heavier hair (androgenic hair) in roughly the following sequence: underarm (axillary) hair, perianal hair, upper lip hair, sideburn (preauricular) hair, periareolar hair, and the rest of the beard area. Arm, leg, chest, abdominal, and back hair become heavier more gradually. There is a large range in amount of body hair among adult men, and significant differences in timing and quantity of hair growth among different ethnic groups.

  
**Voice change**

Under the influence of androgens, the voice box, or larynx, grows in both genders. This growth is far more prominent in boys, causing the male voice to drop, rather abruptly, about one octave, because the larger vocal folds have a lower fundamental frequency. Occasionally, this is accompanied by cracking and breaking sounds in the early stages. Most of the voice change happens during stage 4 of male puberty around the time of peak growth. However, it usually precedes the development of significant facial hair by several months to years.

**Height growth in boys**

Compared to girls' early growth spurt, growth accelerates more slowly in boys and lasts longer, resulting in a taller adult stature among males than females (on average about 10 cm or 4 inches). The difference is attributed to the much greater potency of estradiol compared to testosterone in promoting bone growth, maturation, and epiphyseal closure. In boys, growth begins to accelerate about 9 months after the first signs of testicular enlargement and the peak year of the growth spurt occurs about 2 years after the onset of puberty, reaching a peak velocity of about 8.5–12 cm or 3.5–5 inches per year. The feet and hands experience their growth spurt first, followed by the limbs, and finally ending in the trunk. Epiphyseal closure and adult height are reached more slowly, at an average age of about 17.5 years. As in girls, this last growth primarily involves the spine rather than the limbs.

**Male musculature and body shape**

By the end of puberty, adult men have heavier bones and nearly twice as much skeletal muscle. Some of the bone growth (e.g., shoulder width and jaw) is disproportionately greater, resulting in noticeably different male and female skeletal shapes. The average adult male has about 150% of the lean body mass of an average female, and about 50% of the body fat.

This muscle develops mainly during the later stages of puberty, and muscle growth can continue even after a male is biologically adult. The peak of the so-called "strength spurt," the rate of muscle growth, is attained about one year after a male experiences his peak growth rate.

**Breast development in boys: pubertal gynecomastia**

Estradiol is produced from testosterone in male puberty as well as female, and male breasts often respond to the rising estradiol levels. This is termed gynecomastia. In most boys, the breast development is minimal, similar to what would be termed a "breast bud" in a girl, but in many boys, breast growth is substantial. It usually occurs after puberty is underway, may increase for a year or two, and usually diminishes by the end of puberty. It is increased by extra adipose tissue if the boy is overweight.

Although this is a normal part of male puberty, breast development for some boys is as unwelcome as upper lip hair in girls. If the boy's distress becomes too substantial during development, breast tissue can be removed and corrected surgically.

## Adulthood

![Wikipedia-logo.png](../../../upload.wikimedia.org/wikipedia/commons/thumb/6/63/Wikipedia-logo.png/40px-Wikipedia-logo.png)

[Wikipedia](http://en.wikipedia.org/wiki/) has related information at _**[adult**_](http://en.wikipedia.org/wiki/Adult)

The term "adult" generally refers to a fully developed person from maturity (the end of puberty) onward. The age at which a person is physiologically an adult is age 17 for females and age 18 for males. Adulthood can also refer to a person's ability to care for them self independently, and raise a family of their own; or it can simply mean reaching a specified age. Graduating high school, residing in one's own residence and attaining financial independence are all synonymous with adulthood in the United States.

### Adult characteristics

There are some qualities that symbolize adultness in most cultures. Not always is there a concordance between the qualities and the physical age of the person.

The adult character is comprised of:

  * **Self-control** \- restraint, emotional control.
  * **Stability** \- stable personality, strength.
  * **Independence** \- ability to self-regulate.
  * **Seriousness** \- ability to deal with life in a serious manner.
  * **Responsibility** \- accountability, commitment and reliability.
  * **Method/Tact** \- ability to think ahead and plan for the future, patience.
  * **Endurance** \- ability and willingness to cope with difficulties that present themselves.
  * **Experience** \- breadth of mind, understanding.
  * **Objectivity** \- perspective and realism.

  


![](../../../upload.wikimedia.org/wikipedia/commons/thumb/c/c3/Maslow's_hierarchy_of_needs.png/400px-Maslow's_hierarchy_of_needs.png)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

This diagram shows Maslow's hierarchy of needs, represented as a pyramid with the more primitive needs at the bottom.

Abraham Maslow, a psychologist, developed Maslow's Hierarchy of Needs. It is a chart outlining basic needs that a person must meet to function and survive in life. It also attempts to explain what motivates people in life. The needs on the lower level must be met before moving up the ladder, as the higher needs only come into focus once all the needs that are lower down in the pyramid are satisfied. People can get stuck on levels and some people may never reach certain levels because of circumstances in their life. When one stage is fulfilled you naturally move to the next.

**Physical or Physiological:** These include shelter, oxygen, food, water, rest and elimination, all of which are vital to a person's life and essential to survival.

**Security or Safety:** This involves not only actually being secure and safe, but also the feeling of safety and security. This is something that people typically learn from their childhood and something that helps lay the groundwork for developing other skills and moving up to the next step in the ladder.

**Social (Love/Belonging):** This involves developing friendships and eventually relationships. This involves emotionally-based relationships in general, such as friendship, sexual intimacy, and having a supportive and communicative family.

**Esteem:** This is where people learn to develop self-esteem and confidence. According to Maslow, all humans have a need to be respected, to have self-respect, and to respect others. People need to engage themselves in order to gain recognition and have an activity or activities that give the person a sense of contribution, be it in a profession or hobby,

**Self-Actualization:** The highest level you can reach according to Maslow. Maslow writes the following of self-actualizing people:

  * They embrace the facts and realities of the world (including themselves) rather than denying or avoiding them.
  * They are spontaneous in their ideas and actions.
  * They are creative.
  * They are interested in solving problems; this often includes the problems of others. Solving these problems is often a key focus in their lives.
  * They feel a closeness to other people, and generally appreciate life.
  * They have a system of morality that is fully internalized and independent of external authority.
  * They have discernment and are able to view all things in an objective manner. Prejudices are absent.

In short, self-actualization is reaching one's fullest potential.

Most people accomplish the two lower levels in their lifetime, but may get stuck on upper levels. While self-actualization is a useful concept to many, others insist there is no proof that every individual has this capacity or even the goal to achieve it.

## Menopause

![Wikipedia-logo.png](../../../upload.wikimedia.org/wikipedia/commons/thumb/6/63/Wikipedia-logo.png/40px-Wikipedia-logo.png)

[Wikipedia](http://en.wikipedia.org/wiki/) has related information at _**[Menopause**_](http://en.wikipedia.org/wiki/Menopause)

Menopause occurs as the ovaries stop producing estrogen, causing the reproductive system to gradually shut down. As the body adapts to the changing levels of natural hormones, vasomotor symptoms such as hot flashes and palpitations, psychological symptoms such as increased depression, anxiety, irritability, mood swings and lack of concentration, and atrophic symptoms such as vaginal dryness and urgency of urination appear. Together with these symptoms, the woman may also have increasingly scanty and erratic menstrual periods.

Technically, menopause refers to the cessation of menses; whereas the gradual process through which this occurs, which typically takes a year but may last as little as six months or more than five years, is known as climacteric. Popular use, however, replaces climacteric with menopause. A natural or physiological menopause is that which occurs as a part of a woman's normal aging process. However, menopause can be surgically induced by such procedures as hysterectomy (when this procedure includes oophorectomy, removal of the ovaries).

The average onset of menopause is 50.5 years, but some women enter menopause at a younger age, especially if they have suffered from cancer or another serious illness and undergone chemotherapy. Premature menopause (or premature ovarian failure) is defined as menopause occurring before the age of 40, and occurs in one percent of women. Other causes of premature menopause include autoimmune disorders, thyroid disease, and diabetes mellitus. Premature menopause is diagnosed by measuring the levels of follicle stimulating hormone (FSH) and luteinizing hormone (LH); the levels of these hormones will be higher if menopause has occurred. Rates of premature menopause have been found to be significantly higher in both fraternal and identical twins; approximately five percent of twins reach menopause before the age of 40. The reasons for this are not completely understood. Transplants of ovarian tissue between identical twins have been successful in restoring fertility.

Post-menopausal women, especially Caucasian women of European descent, are at increased risk of osteoporosis.

Animals other than human beings rarely experience menopause, possibly because they simply do not live long enough to reach it. However, recent studies have shown menopause in gorillas, with an average age of 44 at onset.

Perimenopause refers to the time preceding menopause, during which the production of hormones such as estrogen and progesterone diminishes and becomes more irregular. During this period fertility diminishes. Menopause is arbitrarily defined as a minimum of twelve months without menstruation. Perimenopause can begin as early as age 35, although it usually begins much later. It can last for a few months or for several years. The duration of perimenopause cannot be predicted in advance.

### Grandmother Hypothesis

Human females have the unique distinction of being one of the only species to stop reproduction well before the end of their life span. This evolutionary distinction is odd because most other species continue to reproduce until death, thus maximizing the number of offspring they produce. The grandmother hypothesis essentially states that the presence of a grandmother has beneficial effect on the survival of an infant. Humans are one of the slowest developing species in the animal kingdom, and unlike many species, infants, toddlers and children must be continuously cared for to ensure their survival. (Compare that to the salmon that swims up stream, spawns and dies.)

  


### Etiology

The cessation of menses is the result of the eventual atresia (degeneration and reabsorption) of almost all oocytes in the ovaries. This causes an increase in circulating FSH and LH levels as there are a decreased number of oocytes responding to these hormones and producing estrogen. This decrease in the production of estrogen leads to the post-menopausal symptoms of hot flashes, insomnia, osteoporosis, atherosclerosis, vaginal atrophy and depression.

Cigarette smoking has been found to decrease the age of menopause by as much as one year however, premature menopause (before the age of 40) is generally idiopathic.

### Symptoms

The clinical features of menopause are caused by the estrogen deficiency.

  * vasomotor instability
  * hot flashes, hot flushes
  * sleep disturbances
  * Urogenital atrophy
  * dyspareunia
  * itching
  * dryness
  * bleeding
  * urinary frequency
  * urinary urgency
  * urinary incontinence
  * skeletal

**Breast Atrophy**

  * skin thinning
  * decreased elasticity
  * Psychological

**Mood Disturbance**

  * irritability
  * fatigue
  * decreased libido
  * memory loss
  * depression

**Treatments:** Medical treatments for menopausal symptoms have been developed. Most notably, Hormone Replacement Therapy (HRT), has been used to reduce the weakening of bones (known as osteoporosis). However, some women have resisted the implication that menopause is a disorder, seeing it as a natural stage of life. There has also been scientific controversy over whether the benefits of HRT outweigh the risks. For many years, women were advised to take hormone therapy after menopause to reduce their risk of heart disease and various aspects of aging. However, a large, randomized, controlled trial (the Women's Health Initiative) found that women undergoing HRT had an increased risk of Alzheimer's disease, breast cancer, heart disease and stroke.

### Osteoporosis

Osteoporosis is a skeletal disease resulting in bone loss and changes in the bone quality that leads to diminished bone strength and an increased risk to sustain fractures. The main cause of osteoporosis is a loss estrogen following menopause. Osteoporosis can be prevented and treated using a number of different drugs and lifestyle modifications including proper diet, exercise and hormone replacement therapy. The link to Wikipedia Osteoporosis is a great source of additional information.

**Preventing Osteoporosis** The old saying that an ounce of prevention is worth a pound of cure holds true for osteoporosis. In researching osteoporosis I found that while there are some treatments for osteoporosis, a healthy lifestyle throughout your life is a much more effective way of combating the effects of this disease. It is generally acknowledged that a regular weight bearing exercise plan is helpful in maintaining bone mass. Additionally, adequate dietary calcium and vitamin D intake throughout ones life are important factors in building up and maintaining bone mass.

Estrogen and progesterone treatments in postmenopausal women have proven to be effective in treating bone loss. There are also two groups of drugs that interfere with the re-absorption of bone by osteoclasts called bisphosphonates and lective estrogen receptor modulators (SERMS).

An estimated 52 million men and woman will be afflicted with crumbling, weakened bone's by the year 2010. Osteoporosis is three to four times more common in woman than men. While some men do get osteoporosis, they are less likely to do so because men have frames that are 25 percent larger and heavier than women. Women are also more susceptible to the disease because they are more likely than a man to go on crash diets. This kind of diet may interfere with the three main factors associated with osteoporosis and having healthy bones: having enough vitamin D, having enough calcium, and having enough estrogen. There are approximately 1 million to 1.3 million hip fractures every year that are related to osteoporosis. Men on steroids, people with arthritis, people undergoing chemotherapy, along with those suffering from anorexia all have an increased chance of having osteoporosis.

#### Osteoporosis related links

[Wikipedia Osteoporosis Page](http://en.wikipedia.org/wiki/Osteoporosis) This is a wikipedia link with a complete discussion of osteoporosis.

[National Osteoporosis Foundation](http://www.nof.org/osteoporosis/) This page links to the National Osteoporosis Foundation

## Old Age

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/2/23/Black_hmong_women_sapa_vietnam_1999.jpg/220px-Black_hmong_women_sapa_vietnam_1999.jpg)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Hmong women

### Why do people age?

Some researchers believe we are programmed by an internal biological clock to age. The idea is that each type of cell, tissue and organ is like a clock that ticks at its own pace. In the body our cells divide 80 to 90 times at the most. At the end of each chromosome there are repeated stretches of DNA called telomeres. A bit of each telomere is lost during every cell division. When only a nub remains the cells stop dividing and die.

A different hypothesis is that aging is a result of accumulated damage to DNA from environmental attacks and a decline in DNA's mechanism of self repair. Things such as free radicals attack DNA and other molecules causing structural changes. These changes in DNA endanger the synthesis of enzymes and other proteins that are required for life. This damage interferes with cell division.

Most researchers believe that aging is a combination of an internal clock that ticks out the life span of cells and the accumulation damage to DNA.

## Old Age Diseases

### Diabetes

Diabetes mellitus is a disease characterized by persistent hyperglycemia (high blood sugar levels), resulting either from inadequate secretion of the hormone insulin, an inadequate response of target cells to insulin, or a combination of these factors. Diabetes is a metabolic disease requiring medical diagnosis, treatment and lifestyle changes

Type 1 diabetes mellitus is characterized by loss of the insulin-producing beta cells of the islets of Langerhans of the pancreas. Sensitivity and responsiveness to insulin are usually normal, especially in the early stages. This type comprises up to 10% of total cases in North America and Europe, though this varies by geographical location. This type of diabetes can affect children or adults, but has traditionally been termed "juvenile diabetes" because it represents a majority of cases of diabetes affecting children. The most common cause of beta cell loss leading to type 1 diabetes is autoimmune destruction, accompanied by antibodies directed against insulin and islet cell proteins. The principal treatment of type 1 diabetes, even from the earliest stages, is replacement of insulin. Without insulin, ketosis and diabetic ketoacidosis can develop.

Type 2 diabetes mellitus is due to a combination of defective insulin secretion and defective responsiveness to insulin (often termed reduced insulin sensitivity). In early stages the predominant abnormality is reduced insulin sensitivity, characterized by elevated levels of insulin in the blood. The initial defect of insulin secretion is subtle and initially involves only the earliest phase of insulin secretion. In the early stages, hyperglycemia can be reversed by a variety of measures and medications that improve insulin sensitivity or reduce glucose production by the liver, but as the disease progresses the impairment of insulin secretion worsens, and therapeutic replacement of insulin often becomes necessary. Type 2 diabetes is quite common, comprising 90% or more of cases of diabetes in many populations. There is a strong association with obesity and with aging, although in the last decade it has increasingly begun to affect older children and adolescents. In the past, this type of diabetes was often termed adult-onset diabetes or maturity-onset diabetes.

Gestational diabetes, Type III, also involve a combination of inadequate insulin secretion and responsiveness, resembling type 2 diabetes in several respects. It develops during pregnancy and may improve or disappear after delivery. Even though it may be transient, gestational diabetes may damage the health of the fetus or mother, and about 40% of women with gestational diabetes develop type 2 diabetes later in life.

### Congestive Heart Failure

Congestive heart failure (CHF), also called congestive cardiac failure (CCF) or just heart failure, is a condition that can result from any structural or functional cardiac disorder that impairs the ability of the heart to fill with or pump a sufficient amount of blood throughout the body. It is not to be confused with "cessation of heartbeat", which is known as asystole, or with cardiac arrest, which is the cessation of normal cardiac function in the face of heart disease. Because not all patients have volume overload at the time of initial or subsequent evaluation, the term "heart failure" is preferred over the older term "congestive heart failure". Congestive heart failure is often undiagnosed due to a lack of a universally agreed definition and difficulties in diagnosis, particularly when the condition is considered "mild".

### Stroke

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/6/67/MCA-Stroke-Brain-Human-1.JPG/150px-MCA-Stroke-Brain-Human-1.JPG)

![](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/magnify-clip.png)

Stroke

A stroke, also known as cerebrovascular accident (CVA), is an acute neurologic injury whereby the blood supply to a part of the brain is interrupted. Stroke can also be said to be a syndrome of sudden loss of neuronal function due to disturbance in cerebral perfusion. This disturbance in perfusion is commonly on the arterial side of the circulation, but can be on the venous side.

The part of the brain with disturbed perfusion can no longer receive adequate oxygen carried by the blood; brain cells are therefore damaged or die, impairing function from that part of the brain. Stroke is a medical emergency and can cause permanent neurologic damage or even death if not promptly diagnosed and treated. It is the third leading cause of death and adult disability in the US and industrialized European nations. On average, a stroke occurs every 45 seconds and someone dies every 3 minutes. Of every 5 deaths from stroke, 2 occur in men and 3 in women.

### Progeria

The term Progeria narrowly refers to Hutchinson-Gilford Progeria syndrome, but the term is also used more generally to describe any of the so-called "accelerated aging diseases". The word progeria is derived from the Greek for "prematurely old". Because the "accelerated aging" diseases display different aspects of aging, but never every aspect, they are often called "segmental progerias" by biogerontologists. Hutchinson-Gilford Progeria syndrome is an extremely rare genetic condition which causes physical changes that resemble greatly accelerated aging in sufferers. The disease affects between 1 in 4 million (estimated actual) and 1 in 8 million (reported) newborns. Currently, there are approximately 40-45 known cases in the world. There is no known cure. Most people with progeria die around 13 years of age. Progeria is of interest to scientists because the disease may reveal clues about the process of aging. Unlike most other "accelerated aging diseases" (such as Werner's syndrome, Cockayne's syndrome or xeroderma pigmentosum), progeria is not caused by defective DNA repair. It is caused by mutations in a LMNA (Lamin A protein) gene on chromosome 1. Nuclear lamina is a protein scaffold around the edge of the nucleus that helps organize nuclear processes such as RNA and DNA synthesis.

## The effects of Aging on the Body

### **Cardiovascular System**

The heart loses about 1% of its reserve plumbing capacity every year after we turn 30. Change in blood vessels that serve brain tissue reduce nourishment to the brain, resulting in the malfunction and death of brain cells. By the time we turn 80, cerebral blood flow is 20% less, and renal blood flow is 50% less than when we were age 30. As we age our heart goes through certain structural changes: the walls of the heart thicken and the heart becomes heavier, heart valves stiffen and are more likely to calcify, and the aorta, the major vessel carrying blood out of the heart, becomes larger.

![Wikipedia-logo.png](../../../upload.wikimedia.org/wikipedia/commons/thumb/6/63/Wikipedia-logo.png/40px-Wikipedia-logo.png)

[Wikipedia](http://en.wikipedia.org/wiki/) has related information at _**[Heart Attack**_](http://en.wikipedia.org/wiki/Heart_Attack)

Heart Attack / Myocardial infarction
    Acute myocardial infarction (AMI or MI), commonly known as a heart attack, is a disease that occurs when the blood supply to a part of the heart is interrupted, causing death of heart tissue. It is the leading cause of death for both men and women all over the world. The term myocardial infarction is derived from myocardium (the heart muscle) and infarction (tissue death). The phrase "heart attack" sometimes refers to heart problems other than MI, such as unstable angina pectoris and sudden cardiac death.

Congestive Heart Failure
    In the elderly, ventricular diastolic stiffness can lead to pulmonary circulatory congestion. Aortic stenosis and aortic insufficiency, elevate left ventricular preload to the point where the left ventricle becomes stiff and noncompliant, and is common in people 75 years of age or older. Elevated pressures are transmitted to the pulmonary vasculature and lead to pulmonary edema.

### **Musculoskeletal System**

Bones
    Aging is accompanied by the loss of bone tissue. The haversian systems in compact bone undergo slow erosion, lacunae are enlarged, canals become widened, and the endosteal cortex converts to spongy bone. The endosteal surface gradually erodes until the rate of loss exceeds the rate of deposition. Bone remodeling cycle takes longer to complete because bone cells slow in the rate of resorption and deposition of bone tissue. The rate of mineralization also slows down. The number of bone cells also decreases because the bone marrow becomes fatty and unable to provide an adequate supply of precursor cells. Because bones become less dense, they become more prone to fractures. Although bone degeneration is inevitable, it is variable if steps are taken before the mid-twenties -around this time our bones break down faster than they rebuild. Bone density increases when our bones are stressed, so physical activity is important. Vitamins and good diet also help build up bone mass.

Joints
    Cartilage becomes more rigid, fragile, and susceptible to fibrillation. Loss of elasticity and resiliency is attributed to more cross-linking of collagen to elastin, decrease in water content, and decreasing concentrations of glycosaminoglycans. Joints are also more prone to fracture due to the loss of bone mass.

Muscles
    Decrease in the range of motion of the joint is related to the change of ligaments and muscles. As the body ages, muscle bulk and strength declines especially after the age of 70. As much as 30% of skeletal muscle are lost by age 80. Muscle fibers, RNA synthesis and mitochondrial volume loss may all be contributors to muscle decline. Other factors that could contribute to muscle loss of the aged are: change in activity level, reduced nerve supply to muscle, cardiovascular disease, and nutritional deficiencies. In women, menopause will cause muscle mass to decrease significantly, especially in the first three post-menopausal years.

### **Nervous System**

One of the effects of aging on the nervous system is the loss of neurons. By the age of 30, the brain begins to lose thousands of neurons each day. The cerebral cortex can lose as much as 45% of its cells and the brain can weigh 7% less than in the prime of our lives. Associated with the loss of neurons comes a decreased capacity to send nerve impulses to and from the brain. Because of this the processing of information slows down. In addition the voluntary motor movement's slow down, reflex time increases, and conduction velocity decreases. Parkinson's disease is the most common movement disorder of the nervous system. As we age there are some degenerative changes along with some disease's involving the sense organ's that can alter vision, touch, smell, and taste. Loss of hearing is also associated with aging. It is usually the result of changes in important structures of the inner ear.

#### Dementia

Dementia (from Latin de- "apart, away" + mens (genitive mentis) "mind") is the progressive decline in cognitive function due to damage or disease in the brain beyond what might be expected from normal aging. Particularly affected areas may be memory, attention, language and problem solving, although particularly in the later stages of the condition, affected persons may be disoriented in time (not knowing what day, week, month or year it is), place (not knowing where they are) and person (not knowing who they are). Symptoms of dementia can be classified as either reversible or irreversible depending upon the etiology of the disease. Less than 10% of all dementias are reversible. Dementia is a non-specific term that encompasses many disease processes, just as fever is attributable to many etiologies.

#### Alzheimers disease

Alzheimer's disease (AD) is a neurodegenerative disease characterized by progressive cognitive deterioration together with declining activities of daily living and neuropsychiatric symptoms or behavioral changes. It is the most common cause of dementia. The most striking early symptom is short term memory loss (amnesia), which usually manifests as minor forgetfulness that becomes steadily more pronounced with illness progression, with relative preservation of older memories. As the disorder progresses, cognitive (intellectual) impairment extends to the domains of language (aphasia), skilled movements (apraxia), recognition (agnosia), and those functions (such as decision-making and planning) closely related to the frontal and temporal lobes of the brain as they become disconnected from the limbic system, reflecting extension of the underlying pathological process. This consists principally of neuronal loss or atrophy, together with an inflammatory response to the deposition of amyloid plaques and neurofibrillary tangles. Genetic factors are known to be important, and autosomal dominant mutations in three different genes (presenilin 1, presenilin 2, and amyloid precursor protein) have been identified that account for a small number of cases of familial, early-onset AD. For late onset AD (LOAD), only one susceptibility gene has so far been identified: the epsilon 4 allele of the apolipoprotein E gene. Age of onset itself has a heritability of around 50%.

### **Digestive System**

The changes associated with aging of the digestive system include loss of strength and tone of muscular tissue and supporting muscular tissue, decreased secretory mechanisms, decreased motility of the digestive organs, along with changes in neurosensory feedback regarding enzyme and hormone release, and diminished response to internal sensations and pain. In the upper GI tract common changes include periodontal disease, difficulty in swallowing, reduced sensitivity to mouth irritations and sores, loss of taste, gastritis, and peptic ulcer disease. Changes that may appear in the small intestine include, appendicitis, duodenal ulcers, malabsorption, and maldigestion. Other pathologies that increase in occurrence with age are acute pancreatitis, jaundice, and gallbladder problems. Large intestinal changes such as hemorrhoids and constipation may also occur. Cancer of the rectum are quite common.

### **Urinary System**

As we get older kidney function diminishes. By the age of 70, the filtering mechanism is only about half as effective as it was at age 40. Because water balance is altered and the sensation of thirst diminishes with age, older people are more susceptible to dehydration. This causes more urinary tract infections in the elderly. other problems may include nocturia (excessive urination at night), increased frequency of urination, polyuria (excessive urine production), dysuria (painful urination), incontinence, and hematuria (blood in the urine). Some kidney diseases that are common as we age include acute and chronic kidney inflammations and renal calculi (kidney stones). The prostate gland is often implicated in various disorders of the urinary tract. Prostate cancer is the most common cancer in elderly males. Because the prostate gland encircles part of the urethra, an enlarged prostate gland may cause difficulty in urination

### **Respiratory Systems**

With the advancing of age, the airways and tissue of the respiratory tract become less elastic and more rigid. The walls of the alveoli break down, so there is less total respiratory surface available for gas exchange. This decreases the lung capacity by as much as 30% by the age of 70. Therefore, elderly people are more susceptible to pneumonia, bronchitis, emphysema, and other pulmonary disorders. For a more complete discussion of the respiratory system please visit [Chapter 12 The Respiratory System.](http://en.wikibooks.org/wiki/Human_Physiology/The_respiratory_system)

#### Lung cancer

![](../../../upload.wikimedia.org/wikipedia/commons/8/81/Cancer_smoking_lung_cancer_correlation_from_NIH.png)

Correlation between lung cancer and smoking tobacco.

Lung cancer is a cancer of the lungs characterized by the presence of malignant tumors. Most commonly it is bronchogenic carcinoma (about 90%). Lung cancer is one of the most lethal forms of cancer worldwide, causing up to 3 million deaths annually. Only one in ten patients diagnosed with this disease will survive the next five years. Although lung cancer was previously an illness that affected predominately men, lung cancer rate for women has been increasing in the last few decades. This has been attributed to the rising ratio of female to male smokers. More women die of lung cancer than any other cancer, including breast cancer, ovarian cancer and uterine cancers combined. Current research indicates that the factor with the greatest impact on risk of lung cancer is long-term exposure to inhaled carcinogens. The most common means of such exposure is tobacco smoke.

#### Vision

Changes in vision begin at an early age. The cornea becomes thicker and less curved. The anterior chamber decreases in size and volume. The lens becomes thicker and more opaque, and also increases rigidity and loses elasticity. The ciliary muscles atrophy and the pupil constricts. There is also a reduction of rods and nerve cells of the retina.

#### Hearing

Approximately one third of people over the age of 65 have hearing loss. The ability to distinguish between high and low frequency diminishes with age. Loss of hearing for sounds of high-frequency (presbycusis) is the most common, although the ability to distinguish sound localization also decreases. It is believed that the hearing loss isn't so much an age change as it is due to the accumulation of noise damage.

#### Taste and Smell

Sensitivity to odors and taste decline with age. The sense of smell begins to degenerate with the loss of olfactory sensory neurons and loss of cells from the olfactory bulb. The decline in taste sensation is more gradual than that of smell. The elderly have trouble differentiating between flavors. The number of fungiform papillae of the tongue decline by 50% by the age of 50. Taste could also be affected by the loss of salivary gland secretions, notably amylase. This loss of taste and smell can have a significant effect on an elder's health. With the reduced ability to taste and smell, it is difficult to adjust food intake as they can no longer rely on their taste receptors to tell them if something is too salty, or too sweet. This can also cause the problem in that they might not be able to detect if something is spoiled, making them at a higher risk for food poisoning.

### **Cellular Aging**

As people age, oxygen intake decreases as well as the basal metabolic rate. The decrease in the metabolic rate, delayed shivering response, sedentary lifestyle, decreased vasoconstrictor response, diminished sweating, and poor nutrition are reasons why the elderly cannot maintain body temperature. There is also a decrease in total body water (TBW). In newborns, TBW is 75% to 80%. TBW continues to decline in childhood to 60% to 65%, to less than 60% in adults.

### **Organism Aging**

Aging is generally characterized by the declining ability to respond to stress, increasing homeostatic imbalance and increased risk of disease. Because of this, death is the ultimate consequence of aging. Differences in maximum life span between species correspond to different "rates of aging". For example, inherited differences in the rate of aging make a mouse elderly at 3 years and a human elderly at 90 years. These genetic differences affect a variety of physiological processes, probably including the efficiency of DNA repair, antioxidant enzymes, and rates of free radical production.

## Life Expectancy

<http://www.senescence.info/definitions.html>

## Stages of Grief- Death and Dying

We go through several stages of grief as we near death, receive catastrophic news, or go through some type of life-altering experience. There are five defined stages according to Elisabeth Kübler-Ross. She states, however, that these steps don't always come in order, and are not always experienced all together by everyone. She does claim that a person will always experience at least two of the stages.

The stages are:

**Denial**\- This isn't happening, there must have been some mistake.

**Anger**\- Why me? It's not fair, how could you do this to me?!? (aimed toward some other "responsible" party)

**Bargaining**\- Just give me 2 more years...let me live to see________.

**Depression**\- extreme sadness, lack of motivation or desire to fight anymore

**Acceptance**\- I'm ok with this.

## Sidenotes: Aubrey de Grey

Aubrey David Nicholas Jasper de Grey, Ph.D., (born 20 April 1963 in London, England) is a controversial biomedical gerontologist who lives in the city of Cambridge, UK. He is working to expedite the development of a cure for human aging, a medical goal he refers to as engineered negligible senescence. To this end, he has identified what he concludes are the seven areas of the aging process that need to be addressed medically before this can be done. He has been interviewed in recent years in many news sources, including CBS 60 Minutes, BBC, the New York Times, Fortune Magazine, and Popular Science. His main activities at present are as chairman and chief science officer of the Methuselah Foundation and editor-in-chief of the academic journal Rejuvenation Research. Here are the seven biological causes of senescence and possible solutions:

  1. Cell loss or atrophy. Cell depletion can be partly corrected by therapies involving exercise and growth factors. But stem cell therapy is almost certainly required for any more than just partial replacement of lost cells. This research would involve a large number of details, but is occurring on many fronts.
  2. Nuclear mutations and epimutations. A mutation in a functional gene of a cell can cause that cell to malfunction or to produce a malfunctioning product. Because of the sheer number of cells Dr. de Grey believes that redundancy takes care of this problem, although cells that have mutated to produce toxic products might have to be disabled. In Dr. de Grey's opinion, the effect of mutations and epimutations that really matters is cancer, since if even one cell turns into a cancer cell it might spread and become deadly. This is to be corrected by whole-body interdiction of lengthening telomeres, or any other cure for cancer, if any is ever found.
  3. Mutant mitochondria. Because of the highly oxidative environment in mitochondria and their lack of the sophisticated repair systems found in the cell nucleus, mitochondrial mutations are believed to a be a major cause of progressive cellular degeneration. This is to be corrected by moving the DNA for mitochondria completely within the cellular nucleus, where it is better protected. In humans all but 13 proteins are already protected in this way. It has been experimentally shown the operation is feasible.
  4. Cellular senescence. Cellular senescence might be corrected by forcing senescent cells to destroy themselves, a process called apoptosis. Cell killing with suicide genes or vaccines was suggested for making the cells do apoptosis. Healthy cells would multiply to replace them.
  5. Extracellular cross-links. These are chemical bonds between structures that are part of the body, but not within a cell. In senescent people many of these become brittle and weak. The proposal is to further develop small-molecular drugs and enzymes to break links caused by sugar-bonding (glycation), and other common forms of chemical linking.
  6. Junk outside cells. Junk outside cells might be removed by enhanced phagocytosis (the normal process used by the immune system), and small drugs able to break chemical beta-bonds. The large junk in this class can be removed surgically. Junk here means useless things accumulated by a body, but which cannot be digested or removed by its processes, such as the amyloid plaques characteristic of Alzheimer's disease. The oft-mentioned 'toxins' that people claim cause many diseases would probably also fit under this class.
  7. Junk inside cells. Junk inside cells might be removed by adding new enzymes to the cell's natural digestion organ, the lysosome. These enzymes would be taken from bacteria, molds and other organisms that are known to completely digest animal bodies.

Dr. de Grey's research proposals are highly controversial, with many critics arguing the highly complicated biomedical phenomena involved contain too many unknowns for intervention to be considered remotely foreseeable.

## Discoveries In Growth And Development

  * **Medieval times**

In this time the thought was once children emerged form infancy, they were regarded as miniature already formed adults.

  * **Religious influence of parenting 16th Century**

Puritan belief harsh restrictive parenting practices were recommended as the most efficient means of taming the depraved child.

  * **John Locke's 17th Century**

Tabula Rosa = Blank slate in this the thought was that children are to begin with nothing at all and all kinds of experiences can shape their characters. This is seen as a negative vision of the development of children because children do contribute to his or her own development.

  * **Jean Jacques Rousseau 18th Century**

Noble savages = endowed with a sense of right or wrong. Children have built in moral sense 1st concept of stage, 2nd maturation of growth refers to genetically determined naturally unfolding course. He saw development as a discontinuous stagewise process mapped cut by nature.

  * **Charles Darwin the forefather of Scientific Child Study 1859-1936, 19th century**

The famous theory of evolution, _the survival of the fittest_, and _natural selection_.

  * **G. Stanley Hall regarded as the founder of the child study movement 1846-1924**

One of the most influential American psychologists of the early twentieth century. The Normative Approach = normative period measures of large numbers of individuals and age related averages are computed to represent typical development.

  * **The mental testing movement early 20th Century**

French psychologist Alfred Binet and Colleague Theodore Simon were the first to come up with a successful intelligence test IQ at Stanford University.

  * **Sigmund Freud 1856-1939**

Theory '_psychosexual theory_, ID, Ego, and Superego.

  * **Erik Erikson 1902-1994**

Theory _psychosocial theory_

  * **John Watson 1978-1958**

Behaviorism and Social Learning Theory

  * **Ivan Pavlov**

Classical conditioning

  * **B.F. Skinner**

Operant Conditioning

  * **Albert Bandura**

Social learning theory

  * **Jean Piaget's**

Cognitive-developmental theory

## Review Questions

Answers for these questions can be found [here](http://en.wikibooks.org/wiki/Human_Physiology/Appendix_1:_answers_to_review_questions#Development:_birth_through_death)

1\. Growth is the most rapid in

    A) puberty
    B) childhood
    C) infancy
    D) adulthood
    E) Growth is always the same

2\. This hormone stimulates puberty

    A) GnRH
    B) LH
    C) FSH
    D) TSH

3\. Compared to girls' early growth spurt, growth __________in boys and __________

    A) is quicker, lasts longer
    B) accelerates more slowly, lasts longer
    C) is slower, shorter
    D) None of the above

4\. This quality symbolizes adulthood in most cultures

    A) stability
    B) method/tact
    C) endurance
    D) objectivity
    E) all of the above

5\. Susie has a very hard time keeping friends, according to Maslow, this could be because

    A) as a child she had a supportive family
    B) she likes to help solve the problems of others
    C) as a teenager her self-esteem was low
    D) as a baby she wasn’t breastfed
    E) as a child she lived in an environment that never made her feel safe

6\. According to Maslow, in order for me to reach my full potential of self-actualization I must first

    A) feel safe
    B) gain self-esteem
    C) have friendship
    D) have food
    E) all of the above

7\. Humans are one of the _________ developing species in the animal kingdom

    A) slowest
    B) quickest
    C) average
    D) none of the above

8\. Jenny thinks that she might be going through menopause, a symptom of this is

    A) bleeding
    B) frequent urination
    C) itchiness
    D) none of the above
    E) all of the above

9\. It is estimated that 52 million people will be afflicted with this by 2010

    A) Progeria
    B) osteoporosis
    C) Alzheimer’s
    D) dementia

10\. This is the leading cause of death for both men and women

    A) progeria
    B) cancer
    C) congestive heart failure
    D) osteoporosis
    E) heart attack

## Glossary

Alzheimer's disease
    The most common form of dementia. It is a progressive condition that destroys brain cells, resulting in the loss of intellectual abilities

Apoptosis
    The process of regulated cell death

Appositional bone growth
    The growth in diameter of bones around the diaphysis occurs by deposition of bone beneath the periosteum.

Bilirubin
    A chemical breakdown product of hemoglobin.

canaliculi
    small channels or canals in bone.

Deciduous teeth
    The first set of teeth in the growth development of humans and many other animals. (milk teeth, baby teeth, or primary teeth)

Dementia
    The progressive decline in cognitive function due to damage or disease in the brain beyond what might be expected from normal aging.

Epiphyseal Plate
    The cartilage in growing long bones that allows lengthwise growth. The plate ossifies at the end of puberty.

Haversian system
    The basic structural unit of compact bone which includes a central canal, lamellae, lacunae, osteocytes, and canaliculi.

Intramembranous ossification
    The type of bone formation responsible for the development of flat bones, especially those found in the skull. In intramembranous ossification mesenchymal cells develop into bone without first going through a cartilage stage.

lacunae
    spaces between bone lamellae.

lamellae
    concentric layers of bone matrix.

  


Menopause
    The permanent cessation of menstrual cycles.

Menarche
    The first menstrual bleeding, usually occurs at about 12.7 years of age.

Mongolian spots
    are common among darker-skinned races, such as Asian, East Indian, and African. They are flat, pigmented lesions with unclear borders and irregular shape. They appear commonly at the base of the spine, on the buttocks and back. They may also can appear as high as the shoulders and elsewhere. Mongolian spots are benign skin markings and are not associated with any conditions or illnesses.

  


Necrosis
    A form of cell death that results from acute cellular injury.

Osteoporosis
    A condition that is characterized by a decrease in bone mass and density, causing bones to become fragile.

Puberty
    The process of physical changes by which a child's body becomes an adult body capable of reproduction

Pyloric Stenosis
    Narrowing of the pyloric sphincter that reduces or eliminates the passage of food from the stomach to the small intestine, often causing projectile vomiting in infants.

Trabeculae
    spongy bones that make plates or bars instead of concentric layers.

## References

  * [Methuselahmouse.org](http://www.methuselahmouse.org)
  * Van De Graaff (2002) _Human Anatomy 6th ed._ McGraw-Hill Higher Education
  * Windmaier, P.W. Raff, H. Strang, T.S. (2004) _Vander, Sherman, & Luciano's Human Physiology, the Mechanisms of Body Function 9th ed._ Mcgraw-Hill
  * Starr & McMillan (2001) _Human Biology 6th ed._ Thomson-Brooks/cole.
  * McCance, Kathryn L., Heuther, Sue E. (1994) _Pathophysiology: the biological basis for diseases in adults and children._ Mosby-Year Book, Inc.

  


# Notes and Resources

## Authors

  1. [Kevin Young](http://en.wikibooks.org/wiki/User:Provophys), U.S. Citizen and resident thereof. I was teaching pre-nursing students of [Provo College](http://www.trade-schools.net/provo-college/default.asp) through distance education technology, while working at the [Utah State University](http://en.wikipedia.org/wiki/Utah_state_university) Brigham City campus (as of Jan 2011 I teach at [Arizona Western College](http://azwestern.edu)). This textbook was created as a class project. I created the chapter pages, but the students are the primary authors (hopefully some of them will list themselves here). The fall 2006 class created the initial structure and content, while the spring 2007 students went through each chapter making improvements.
  2. Nursing student, Provo College [Stephanie Greenwood](http://en.wikibooks.org/w/index.php?title=User:Stephanie_greenwood&action=edit&redlink=1) 18:13, 30 April 2007 (UTC)
  3. Nursing Student, Provo College Danette Seyffert 16:07, 1 May 2007 (UTC)

Hopefully this project expands far beyond our classroom. If you wish to be listed specifically as a contributing author, then **please include your name above this paragraph**. We also thank all those who wrote Wikipedia articles that we borrowed from extensively, and all anonymous contributors.

## Content and Contributions

To the best of our knowlege, this is the world's first and only [open content](http://en.wikipedia.org/wiki/Open_content) **human physiology textbook**. Not only will this be the **most affordable** introductory physiology textbook available, but we anticipate that it will become the _**best**_ one available. We will focus all our efforts for two months on providing valuable content and organizing and explaining information clearly. We anticipate the book becoming increasingly better as worldwide awareness and participation spread.

The original text of this work, and the home of the project, can be found at:

    Wikibooks: [Human Physiology](http://en.wikibooks.org/wiki/Human_Physiology)
    <http://en.wikibooks.org/wiki/Human_Physiology>

Notes on the preferred style and formatting of each page can be at:

    Wikibooks: [Human Physiology/Formatting](http://en.wikibooks.org/wiki/Human_Physiology/Formatting)
    <http://en.wikibooks.org/wiki/Human_Physiology/Formatting>

## License

All content contained herein is available under the [GNU free documentation license](http://en.wikibooks.org/wiki/Wikibooks:GNU_Free_Documentation_License). You can copy it, print it, sell it, and create derivative works from it, in accordance with that license.

This book links to several outside sources, and these links are designed to be illustrative learning tools. Links to outside content do not represent an endorsement of that content. Any of the content not found under the Wikimedia site and subsites is not guaranteed to be under the [GDFL](http://en.wikibooks.org/wiki/Wikibooks:GNU_Free_Documentation_License); they are most likely not.  


![](http://en.wikibooks.org/wiki/Special:CentralAutoLogin/start?type=1x1)

Retrieved from "[http://en.wikibooks.org/w/index.php?title=Human_Physiology/Print_Version&oldid=2604257](http://en.wikibooks.org/w/index.php?title=Human_Physiology/Print_Version&oldid=2604257)" 

Hidden categories: 

  * [Pages with broken file links](http://en.wikibooks.org/wiki/Category:Pages_with_broken_file_links)
  * [No references for citations](http://en.wikibooks.org/wiki/Category:No_references_for_citations)

## Navigation menu

### Personal tools

  * [Create account](http://en.wikibooks.org/w/index.php?title=Special:UserLogin&returnto=Human+Physiology%2FPrint+Version&type=signup)
  * [Log in](http://en.wikibooks.org/w/index.php?title=Special:UserLogin&returnto=Human+Physiology%2FPrint+Version)

### Namespaces

  * [Book](Print_Version.html)
  * [Discussion](http://en.wikibooks.org/w/index.php?title=Talk:Human_Physiology/Print_Version&action=edit&redlink=1)

### 

### Variants

### Views

  * [Read](Print_Version.html)
  * [Edit](http://en.wikibooks.org/w/index.php?title=Human_Physiology/Print_Version&action=edit)
  * [View history](http://en.wikibooks.org/w/index.php?title=Human_Physiology/Print_Version&action=history)

### Actions

### Search

### Navigation

  * [Main Page](/wiki/Main_Page)
  * [Help](https://www.mediawiki.org/wiki/Special:MyLanguage/Help:Contents)
  * ### Navigation

    * [Main Page](http://en.wikibooks.org/wiki/Main_Page)
    * [Help](https://www.mediawiki.org/wiki/Special:MyLanguage/Help:Contents)
    * [Browse](http://en.wikibooks.org/wiki/Wikibooks:Card_Catalog_Office)
    * [Cookbook](http://en.wikibooks.org/wiki/Cookbook:Table_of_Contents)
    * [Wikijunior](http://en.wikibooks.org/wiki/Wikijunior)
    * [Featured books](http://en.wikibooks.org/wiki/Wikibooks:Featured_books)
    * [Recent changes](http://en.wikibooks.org/wiki/Special:RecentChanges)
    * [Donations](http://donate.wikimedia.org/wiki/Special:FundraiserRedirector?utm_source=donate&utm_medium=sidebar&utm_campaign=C13_en.wikibooks.org&uselang=en)
    * [Random book](http://en.wikibooks.org/wiki/Special:RandomRootpage)
    * [Using Wikibooks](http://en.wikibooks.org/wiki/Using_Wikibooks)

### Community

    * [Reading room](http://en.wikibooks.org/wiki/Wikibooks:Reading_room)
    * [Community portal](http://en.wikibooks.org/wiki/Wikibooks:Community_Portal)
    * [Bulletin Board](http://en.wikibooks.org/wiki/Wikibooks:Reading_room/Bulletin_Board)
    * [Help out!](http://en.wikibooks.org/wiki/Wikibooks:Maintenance)
    * [Policies and guidelines](http://en.wikibooks.org/wiki/Wikibooks:Policies_and_guidelines)
    * [Contact us](http://en.wikibooks.org/wiki/Wikibooks:Contact_us)

### Tools

    * [What links here](http://en.wikibooks.org/wiki/Special:WhatLinksHere/Human_Physiology/Print_Version)
    * [Related changes](http://en.wikibooks.org/wiki/Special:RecentChangesLinked/Human_Physiology/Print_Version)
    * [Upload file](http://commons.wikimedia.org/wiki/Special:UploadWizard)
    * [Special pages](http://en.wikibooks.org/wiki/Special:SpecialPages)
    * [Permanent link](http://en.wikibooks.org/w/index.php?title=Human_Physiology/Print_Version&oldid=2604257)
    * [Page information](http://en.wikibooks.org/w/index.php?title=Human_Physiology/Print_Version&action=info)
    * [Cite this page](http://en.wikibooks.org/w/index.php?title=Special:Cite&page=Human_Physiology%2FPrint_Version&id=2604257)

### In other languages

    * [Français](http://fr.wikibooks.org/wiki/Cours_de_physiologie/Contr%C3%B4le,_r%C3%A9gulation_et_hom%C3%A9ostasie)
    * [Português](http://pt.wikibooks.org/wiki/Fisiologia/Sistema_respirat%C3%B3rio)
    * ### Sister projects

    * [Wikipedia](http://en.wikipedia.org/wiki/Main_Page)
    * [Wikiversity](http://en.wikiversity.org/wiki/Wikiversity:Main_Page)
    * [Wiktionary](http://en.wiktionary.org/wiki/Wiktionary:Main_Page)
    * [Wikiquote](http://en.wikiquote.org/wiki/Main_Page)
    * [Wikisource](http://en.wikisource.org/wiki/Main_Page)
    * [Wikinews](http://en.wikinews.org/wiki/Main_Page)
    * [Wikivoyage](http://en.wikivoyage.org/wiki/Main_Page)
    * [Commons](http://commons.wikimedia.org/wiki/Main_Page)
    * [Wikidata](http://www.wikidata.org/wiki/Wikidata:Main_Page)

### Print/export

    * [Create a collection](http://en.wikibooks.org/w/index.php?title=Special:Book&bookcmd=book_creator&referer=Human+Physiology%2FPrint+Version)
    * [Download as PDF](http://en.wikibooks.org/w/index.php?title=Special:Book&bookcmd=render_article&arttitle=Human+Physiology%2FPrint+Version&oldid=2604257&writer=rl)
    * [Printable version](http://en.wikibooks.org/w/index.php?title=Human_Physiology/Print_Version&printable=yes)

    * This page was last modified on 27 January 2014, at 10:18.
    * Text is available under the [Creative Commons Attribution-ShareAlike License.](http://creativecommons.org/licenses/by-sa/3.0/); additional terms may apply. By using this site, you agree to the [Terms of Use](http://wikimediafoundation.org/wiki/Terms_of_Use) and [Privacy Policy.](http://wikimediafoundation.org/wiki/Privacy_policy)
    * [Privacy policy](http://wikimediafoundation.org/wiki/Privacy_policy)
    * [About Wikibooks](http://en.wikibooks.org/wiki/Wikibooks:Welcome)
    * [Disclaimers](http://en.wikibooks.org/wiki/Wikibooks:General_disclaimer)
    * [Developers](https://www.mediawiki.org/wiki/Special:MyLanguage/How_to_contribute)
    * [Mobile view](http://en.m.wikibooks.org/wiki/Human_Physiology/Print_Version)
    * ![Wikimedia Foundation](http://bits.wikimedia.org/images/wikimedia-button.png)
    * ![Powered by MediaWiki](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/poweredby_mediawiki_88x31.png)
