# Introduction to Chemical Engineering Processes/Print Version

From Wikibooks, open books for an open world

< [Introduction to Chemical Engineering Processes](http://en.wikibooks.org/wiki/Introduction_to_Chemical_Engineering_Processes)

![Unreviewed changes are displayed on this page](http://bits.wikimedia.org/static-1.24wmf1/extensions/FlaggedRevs/frontend/modules/img/1.png)This page may need to be [reviewed](http://en.wikibooks.org/wiki/Wikibooks:REVIEW) for quality.

Jump to: [navigation](Print_Version.html#mw-navigation), [search](Print_Version.html#p-search)

![Printer.svg](../../../upload.wikimedia.org/wikipedia/commons/thumb/2/23/Printer.svg/40px-Printer.svg.png)

**This is the [print version](http://en.wikibooks.org/wiki/Help:Print_versions) of [Introduction to Chemical Engineering Processes](http://en.wikibooks.org/wiki/Introduction_to_Chemical_Engineering_Processes)**  
You won't see this message or any elements not part of the book's content when you print or [preview](http://en.wikibooks.org/w/index.php?title=Introduction_to_Chemical_Engineering_Processes/Print_Version&action=purge&printable=yes) this page.

# **Chapter 1: Prerequisites**

## Consistency of units

Most values that you'll run across as an engineer will consist of a number and a unit. Some do not have a unit because they are a pure number (like pi, π) or a ratio. In order to solve a problem effectively, all the types of units should be consistent with each other, or should be in the same _system_. A system of units defines each of the basic unit types with respect to some measurement that can be easily duplicated, so that, for example, 5 ft. is the same length in Australia as it is in the United States. There are five commonly-used base unit types or _dimensions_ that one might encounter (shown with their abbreviated forms for the purpose of dimensional analysis):

    **Length** (L), or the physical distance between two positions with respect to some standard distance
    **Time** (t), or how long something takes with respect to how long some natural phenomenon takes to occur
    **Mass** (M), a measure of the inertia of a material relative to that of a standard
    **Temperature** (T), a measure of the average kinetic energy of the molecules in a material relative to a standard
    **Electric Current** (E), a measure of the total charge that moves in a certain amount of time

**Note:** It would make more commonsense to have **Electric Charge** as a base unit, since current is charge per time, and you may find it convenient to think of charge as the fundamental unit. However, current proved easier to measure very accurately and reproducibly, so the physicists decided it would be their reference.

There are several different consistent systems of units. In most of the world (apart from the US and to some extent the UK) the SI system is standard. It is also used in refereed scientific and engineering journals in these two countries. In practice, it is essential for a chemical engineer to be proficient in the SI system, but to be able to use data in units of other systems and to be able to specify designs in the preferred unit system for the job.

### Units of Common Physical Properties

Every system of units has a large number of _derived_ units which are, as the name implies, derived from the base units. These new units are based on the physical definitions of other quantities and involve the combination of different variables. Below is a list of several common derived system properties and the corresponding dimensions (![\\dot=](../../../upload.wikimedia.org/math/e/0/1/e011dac23072797e8fc73cf575709782.png) denotes unit equivalence). If you don't know what one of these properties is, you will learn it eventually:

Property Dimensions Property Dimensions

[Mass](http://en.wikipedia.org/wiki/Mass)
M
[Length](http://en.wikipedia.org/wiki/Length)
L

[Time](http://en.wikipedia.org/wiki/Time)
t
[Temperature](http://en.wikipedia.org/wiki/Temperature)
T

[Area](http://en.wikipedia.org/wiki/Area)
![L^2](../../../upload.wikimedia.org/math/d/e/1/de149b273e767d38e3dce00b38b78b81.png)
[Volume](http://en.wikipedia.org/wiki/Volume)
![ L^3 ](../../../upload.wikimedia.org/math/4/4/5/4453f587a0be574b3a04327cd0c0c4f0.png)

[Velocity](http://en.wikipedia.org/wiki/Velocity)
![ \\frac{L}{t} ](../../../upload.wikimedia.org/math/5/4/0/540c7bffa2dbccb52a25619d4bb1d385.png)
[Acceleration](http://en.wikipedia.org/wiki/Acceleration)
![ \\frac{L}{t^2} ](../../../upload.wikimedia.org/math/d/6/1/d61c56d25bdbdafb5e2aab59c40dd6e8.png)

[Force](http://en.wikipedia.org/wiki/Force)
![ \\frac{ M*L}{t^2} ](../../../upload.wikimedia.org/math/5/f/9/5f98c78f0dffd7a941b2a9f493a53399.png)
[Energy](http://en.wikipedia.org/wiki/Energy)/[Work](http://en.wikipedia.org/wiki/Mechanical_work)/[Heat](http://en.wikipedia.org/wiki/Heat)
![ \\frac{M*L^2}{t^2} ](../../../upload.wikimedia.org/math/9/5/f/95fc65a9e621aa7c19c808fdf404c905.png)

[Power](http://en.wikipedia.org/wiki/Power_\(physics\))
![ \\frac{M*L^2}{t^3} ](../../../upload.wikimedia.org/math/0/4/5/045474287e34407516a244f332e27c5e.png)
[Pressure](http://en.wikipedia.org/wiki/Pressure)
![ \\frac{M}{L*t^2} ](../../../upload.wikimedia.org/math/7/f/3/7f3d1385828252e2bf8e8f05021a9e08.png)

[Density](http://en.wikipedia.org/wiki/Density)
![ \\frac{M}{L^3} ](../../../upload.wikimedia.org/math/e/0/8/e0856719efecd068926abcbc08946186.png)
[Viscosity](http://en.wikipedia.org/wiki/Viscosity)
![ \\frac{M}{L*t} ](../../../upload.wikimedia.org/math/7/8/a/78a8178be822807c6897c531763edf92.png)

[Diffusivity](http://en.wikipedia.org/wiki/Thermal_diffusivity)
![ \\frac{L^2}{s} ](../../../upload.wikimedia.org/math/a/1/c/a1c40a16765fcb20f5b8d9da18ac2280.png)
[Thermal Conductivity](http://en.wikipedia.org/wiki/Thermal_conductivity)
![ \\frac{M*L}{t^3*T} ](../../../upload.wikimedia.org/math/9/7/3/97313929eae7bdbf84a36c142c3a5d8e.png)

[Specific Heat Capacity](http://en.wikipedia.org/wiki/Specific_heat_capacity)
![ \\frac{L^2}{t^2*T} ](../../../upload.wikimedia.org/math/f/9/2/f9224b626d674a1eb51b2ec3021ff97a.png)
[Specific Enthalpy](http://en.wikipedia.org/wiki/Enthalpy#Specific_enthalpy)
![ \\frac{L^2}{t^2} ](../../../upload.wikimedia.org/math/f/a/c/facd9efd90f94709c14d6f50378b94a2.png)

[Specific Gibbs Energy](http://en.wikipedia.org/wiki/Gibbs_free_energy)
![ \\frac{L^2}{t^2} ](../../../upload.wikimedia.org/math/f/a/c/facd9efd90f94709c14d6f50378b94a2.png)
[Specific Energy](http://en.wikipedia.org/wiki/Entropy)
![ \\frac{L^2}{t^2*T} ](../../../upload.wikimedia.org/math/f/9/2/f9224b626d674a1eb51b2ec3021ff97a.png)

### SI (kg-m-s) System

This is the most commonly-used system of units in the world, and is based heavily on factors of 10. It was originally based on the properties of water, though currently there are more precise standards in place. The major dimensions are:

Dimension name SI unit SI abbreviation

Length
meter
m

Time
second
s

Mass
kilogram
kg

Temperature
kelvin
K

Electric Current
ampere
A

Amount of substance
mole
mol

Note that the kilogram, not the gram, is a base unit.

The close relationship to water is that one m3 of water weighs (approximately) 1000 kg.

A base unit that can be difficult to understand is the _mole_. A mole represents 6.022*1023 particles of _any substance_. (The number is known as Avogadro's Number, or the [Avogadro constant](http://en.wikipedia.org/wiki/Avogadro_constant).) This usually means the number of atoms or molecules of an element or compound. Chemical engineers commonly use kilomoles. The relative molecular mass (= molecular weight) of water H2O is about 18, being made up of 2 H atoms (atomic mass = 1) and one O atom (atomic mass = 16). Thus 18 kilograms of water constitute 1 kilomole of H2O and contain 2 kilomoles of H atoms and 1 kilomole of O atoms.

Each of these base units can be made smaller or larger in units of ten by adding the appropriate **metric prefixes**. The specific meanings are (from the [SI](http://en.wikipedia.org/wiki/SI) page on Wikipedia):

SI Prefixes

Name
yotta
zetta
exa
peta
tera
giga
mega
kilo
hecto
deca

Symbol
Y
Z
E
P
T
G
M
k
h
da

Factor
1024
1021
1018
1015
1012
109
106
103
102
101

Name
deci
centi
milli
micro
nano
pico
femto
atto
zepto
yocto

Symbol
d
c
m
µ
n
p
f
a
z
y

Factor
10-1
10-2
10-3
10-6
10-9
10-12
10-15
10-18
10-21
10-24

If you see a length of 1 km, according to the chart, the prefix "k" means there are 103 of something, and the following "m" means that it is meters. So 1 km = 103 meters. There should always be a space between the number and the unit and between different units which are multiplied together. There must not be a space between the multiplier and the unit. Thus 13 mA means 13 milliamps, but 13 m A means 13 meter-amps.

As noted above, the kilogram is a base unit, but the multipliers are added to the gram. 1000 kg = 1 Mg; 0.001 kg = 1 g.

In chemical engineering practice, we tend not to use the very large or small ends of the table, but you should know at least as large as mega (M), and as small as nano (n). The relationship between different sizes of metric units was deliberately made simple because you will have to do it all of the time. You may feel uncomfortable with it at first if you're from the U.S. but trust me, after working with the English system you'll learn to appreciate the simplicity of the Metric system.

#### Derived units in the SI system

Imagine if every time you calculated a pressure, you would have to write the units in kg/(m s2). This would become cumbersome quickly, so the SI people set up **derived units** to use as shorthand for such combinations as these. Note that units named after a person do not start with a capital letter, but the abbreviation does! For example "a force of one newton" and " a force of 1.0 N". The most common ones used by chemical engineers are as follows:

Property name Long SI Units SI Name SI Abbreviation Equivalencies

Force
![ \\frac{kg*m}{s^2} ](../../../upload.wikimedia.org/math/8/9/f/89f73b4ffe52136567883dfb8c3ebf23.png)
newton
N
Mass * acceleration

Energy
![ \\frac{kg*m^2}{s^2} ](../../../upload.wikimedia.org/math/b/e/5/be58dde0af11d5c3d29f40f7cedcd050.png)
joule
J
![ N*m ](../../../upload.wikimedia.org/math/9/1/2/9125e804af759fa313980bd23cc14022.png), ![ Pa*m^3 ](../../../upload.wikimedia.org/math/d/f/e/dfe6bf11f3836c3b8e7e067c69ccaf72.png)

Power
![ \\frac{kg*m^2}{s^3} ](../../../upload.wikimedia.org/math/f/d/5/fd58a79eaa2c129769fc37dc8fcdb6b3.png)
watt
W
![ \\frac{N*m}{s} ](../../../upload.wikimedia.org/math/9/0/0/900e2aba9f87d8b76114372a1b8d8adb.png) or ![ \\frac{J}{s} ](../../../upload.wikimedia.org/math/3/b/d/3bd06e52930d859cbfaf39663cd8cbfd.png)

Pressure
![ \\frac{kg}{m*s^2} ](../../../upload.wikimedia.org/math/5/6/f/56f1cd9eebc87fd32c40816299fc42bf.png)
pascal
Pa
![ \\frac{N}{m^2} ](../../../upload.wikimedia.org/math/9/9/7/997d5a6efabf290ff5e14163bfb3ebe8.png)

  


#### Allowed units in the SI system

Some units are not simply derived from the base units or regular multiples, but are in common use and are therefore permitted. Thus, though periods of time can be expressed in kiloseconds or megaseconds, we are allowed to use minutes, hours and days. The term 'liter' (US) or 'litre' (European) is understood to be the same as 1 x 10-3 m3, and the term tonne (not ton) is understood to be the same as 1000 kg. The bar is a unit of pressure meaning 100 kPa, which is very close to the chemists' standard atmosphere (which is 101.325 kPa). The Celsius scale of temperature is understood to be the number of kelvin above 273.15 K. Thus we are allowed to write "the chemical reactor has a throughput of 4.3 tonnes per day at 5 bar and 200 °C" and we will be understood. However, it may be necessary to change to base or derived units in order to carry out calculations.

  


### cgs (cm-g-s) system

This was the first metric system and may be found old publications (before 1960). There is no reason why a chemical engineer should work in it today, but you may have to convert data from old books. The base units of length and mass were the centimeter and gram. The unit of force was a _dyne_; the unit of energy was an _erg_. The value of _g_, the standard acceleration due to gravity was 981 cm/s/s. The viscosity units poise (especially centipoise, cP) and stokes (especially centistokes cSt) are a hangover from this system and may be found in relatively recent publications. You should convert them to SI.

Note that chemists often work with grams and cubic centimeters, but these are part of SI. Just because you work with cm, g, and s, does not mean you are using the cgs system. See [w:cgs](http://en.wikipedia.org/wiki/cgs) if you really want to know.

### British, Imperial or American (gravitational) system

This system was established with the authority of the British Empire. It is known in Britain as the Imperial system, in America as the British (sometimes English) system, and in much of the world as the American system, since the USA is the only major market for chemical engineering which uses it. The engineering version uses a subset of this traditional or customary measure plus the _pound force_ and the ampere.

Its peculiarity lies in the relationship between force and mass. According to Isaac Newton for a fixed mass accelerating under the influence of a force:

force = mass x acceleration or _f = m a_

In the SI system a force of 1 newton acting on a mass of 1 kilogram produces an acceleration of 1 meter per second. Simple!

In the Imperial system a force of 1 pound-force acting on a mass of 1 pound produces an acceleration of 32 feet per second per second. This is because this is the natural acceleration under gravity. Older American books often include a _g_ in the formula which do not appear in European versions of the same equation. The _g_ represents the relationship between force and mass in the unit system (which is 1 in SI): here it is 32. For a while, American (mainly) engineers used a version of the metric system including the kilogram-force and thus _g_, which had the value 9.81. Physicists call these both _gravitational_ systems.

The common units are based on traditional measures which were practical in agriculture and shipping, and do not go in steps of 10, 100, 1000 etc. Instead of using prefixes you use names for larger units, and can use combinations of units for the same dimension, e.g. 6 yards 2 feet and 8 and a quarter inches ( 6 yd 2 ft 8¼ in) However, engineers tend to use one unit and a decimal, e.g. 20.7 ft, e.g. 13.47 in. The foot can also be denoted by a single mark and the inch by a double mark, e.g. 4 feet 7 inches was 4' 7". Note that the US gallon is smaller than the Imperial gallon (5/6 in fact), when you are doing conversions to SI.

The temperature scale is that of Fahrenheit, in which the melting point of ice is 32 °F. Absolute zero is -459.67 °F. For thermodynamic temperatures, the number of degrees Fahrenheit above absolute zero is the Rankine scale. Thus the melting point of ice is 459.67 °R.

The following are common units in this system.

Dimension name Imperial unit Imperial abbreviation

Length
foot, inch
ft, in

Time
second, minute, or hour
sec, min, and hr, respectively

Force
pound-force
![ lb_f ](../../../upload.wikimedia.org/math/e/0/c/e0c7af41382392273bc78090514c254b.png)

Temperature
degree Fahrenheit
°F

Electric current
ampere
A

A common derived unit is the pound(-force) per square inch, or psi. Note that psig or psi(g) means psi above atmospheric pressure. Energy is measured in British Thermal Units, generally BTU, sometimes B.Th U. Power is horsepower, hp.

### "Parts-per" notation

The "parts-per" notation is a unit that deals with very small traces of species within a mixture of gases or liquids. Parts-per million (ppm) and parts-per billion (ppb), as well as parts-per trillion (ppt) (American definition of trillion 1012), refer to mass or mole ratios and communicate how many parts of the species are present-per million, billion, or trillion parts of the mixture. Generally mass ratios are used when dealing with liquids and mole ratios are used when dealing with gases, though either kind of ratio can be used for whichever phase a chemical is in (ratios are discussed in a later chapter).

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  

    
    
    Let's say the air around us contains 20 ppm He (Helium). 
    

This means that, if one assumes that a molar basis is being used, for every **million** moles of air there are 20 moles of Helium. If the example was in terms of _ppb_, this would mean that for every **billion** moles of air there are 20 moles of Helium.

### A Word About Conversions

It is generally safe to convert all data into SI then work your calculations out in that system, converting back if necessary at the end. If you are skilled enough in the American system, you may be able to carry some calculations within that system. It is best to consult a conversion table or program for the necessary changes and especially important to keep good track of the units.

However, do not make the mistake of just writing down the numbers you get from the calculator or program.

For example, if you have a pressure drop in a pipe of 16 psi, and the conversion factor 1 psi = 6.895 kPa, your calculator will give 16 x 6.895 = 110.32. However, your answer should be 110 kPa because your starting value was only given to a precision of two figures. The conversion factor cannot add accuracy!

_If every value is written in terms of the same base units, and the equation that is used is correct, then the units of the answer will be consistent and in terms of the same base units._

## How to convert between units

### Finding equivalences

The first thing you need in order to convert between units is the equivalence between the units you want and the units you have. To do this use a **conversion table**. See [w:Conversion of units](http://en.wikipedia.org/wiki/Conversion_of_units) for a fairly extensive (but not exhaustive) list of common units and their equivalences.

Conversions within the metric system usually are not listed, because it is assumed that one can use the prefixes and the fact that ![ 1 \\mbox{ mL} = 1 \\mbox{ cm}^3 ](../../../upload.wikimedia.org/math/c/7/2/c720da676d13dae6cd0562b19eb62036.png) to convert anything that is desired.

Conversions within the English system and especially between the English and metric system are sometimes (but not on Wikipedia) written in the form:

![1 \(unit 1\) = \(number\) \(unit 2\) = \(number\) \(unit 3\) = .... ](../../../upload.wikimedia.org/math/a/6/b/a6b286ed938a1787454c5bdd523af47a.png)

For example, you might recall the following conversion from chemistry class:

![ 1 \\mbox{ atm} = 760 \\mbox{ mmHg} = 1.013*10^5 \\mbox{ Pa} = 1.013 \\mbox{ bar} = ....](../../../upload.wikimedia.org/math/b/a/e/bae7e54bdc25be1fda33ff89478c3270.png)

The table on Wikipedia takes a slightly different approach: the column on the far left side is the unit we have 1 of, the middle is the definition of the unit on the left, and on the far right-hand column we have the metric equivalent. One listing is the conversion from feet to meters:

![ \\mbox{ foot \(International\)  ft}  = 1/3 \\mbox{ yd}  = 0.3048\\mbox{ m} ](../../../upload.wikimedia.org/math/c/a/6/ca69891fb0aa87d0346a3b6e433677a2.png)

Both methods are common and one should be able to use either to look up conversions.

### Using the equivalences

Once the equivalences are determined, use the general form:

  


![ \\mbox{What you want} = \\mbox{What you have} * \\frac{\\mbox{What you want}}{\\mbox{What you have}} ](../../../upload.wikimedia.org/math/c/c/7/cc7748587c2198383202b90997b97720.png)

The fraction on the right comes directly from the conversion tables.

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Convert 800 mmHg into bars

**Solution** If you wanted to convert 800 mmHg to bars, using the horizontal list, you could do it directly:

![ bars = 800\\mbox{ mmHg}*\\frac{1.013\\mbox{ bar}}{760\\mbox{ mmHg}} = 1.066\\mbox{ bar} ](../../../upload.wikimedia.org/math/f/2/6/f26531c186d8f5fd67700f51cc88ee5b.png)

Using the tables from Wikipedia, you need to convert to an _intermediate_ (the metric unit) and then convert from the intermediate to the desired unit. We would find that

![ 1 \\mbox{ mmHg}=133.322\\mbox{ Pa} ](../../../upload.wikimedia.org/math/2/7/4/27461c3159846b17d0f83b340b02fab0.png) and ![1\\mbox{ bar}=10^5\\mbox{ Pa}](../../../upload.wikimedia.org/math/b/4/1/b41837f539ab3a89e36d94d438a7c922.png)

Again, we have to set it up using the same general form, just we have to do it twice:

![ bars = 800\\mbox{ mmHg}*\\frac{133.322\\mbox{ Pa}}{1\\mbox{ mmHg}}*\\frac{1\\mbox{ bar}}{10^5\\mbox{ Pa}} = 1.066\\mbox{ bar}](../../../upload.wikimedia.org/math/9/6/5/96596b9b73e2e4f5ab9449c3adc3e20c.png)

Setting these up takes practice, there will be some examples at the end of the section on this. It's a **very important** skill for any engineer.

One way to keep from avoiding "doing it backwards" is to write everything out and make sure your units cancel out as they should! If you try to do it backwards you'll end up with something like this:

![ bars = 800\\mbox{ mmHg}*\\frac{760\\mbox{ mmHg}}{1.013\\mbox{ bar}} =  6.0*10^5 \\frac{{mmHg}^2}{bar}](../../../upload.wikimedia.org/math/7/3/4/734c005522e8661562d627302705e75d.png)

If you write everything (even conversions within the metric system!) out, and make sure that everything cancels, you'll help mitigate unit-changing errors. About 30-40% of all mistakes I've seen have been unit-related, which is why there is such a long section in here about it. Remember them well.

## Dimensional analysis as a check on equations

Since we know what the units of velocity, pressure, energy, force, and so on should be in terms of the base units L, M, t, T, and E, we can use this knowledge to check the feasibility of equations that involve these quantities.

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Analyze the following equation for dimensional consistency: ![P = g*h ](../../../upload.wikimedia.org/math/0/e/b/0eb4999c79b276297159ee6d8b2b1c10.png) where g is the gravitational acceleration and h is the height of the fluid

**Solution** We could check this equation by plugging in our units:

![P \\dot= M/\(L*t^2\) \\mbox{ , } h \\dot= L \\mbox{ , } g \\dot= L/t^2 ](../../../upload.wikimedia.org/math/0/b/e/0bedeaf224b20130308d3ca5c4a94774.png)  
![g * h \\dot= L^2/t^2 \\neq M/\(L*t^2\) ](../../../upload.wikimedia.org/math/1/5/7/157a97c2bf83dee54bef9df5a79d3fc3.png)  


Since g*h doesn't have the same units as P, the equation must be wrong _regardless of the system of units we are using!_ The correct equation, in fact, is:

![ P = \\rho*g*h ](../../../upload.wikimedia.org/math/c/1/d/c1d19e0f3f626749792317cdbd358df5.png)  
where ![ \\rho ](../../../upload.wikimedia.org/math/f/7/f/f7f177957cf064a93e9811df8fe65ed1.png) is the density of the fluid. Density has base units of ![ M/L^3](../../../upload.wikimedia.org/math/7/c/b/7cb5b5ebd21ae9e73bf785776852e76b.png) so

![ \\rho*g*h \\dot= M/L^3 * L^2/t^2 \\dot= M/\(L*t^2\)](../../../upload.wikimedia.org/math/3/2/1/321fe112da7ee1e410923fcd639f4645.png) which are the units of pressure.  


This does not tell us the equation is correct but it _does_ tell us that the units are consistent, which is necessary though not sufficient to obtain a correct equation. _This is a useful way to detect algebraic mistakes that would otherwise be hard to find._ The ability to do this with an algebraic equation is a good argument against plugging in numbers too soon!

You may well be forced to do dimensional analysis in chemical engineering classes or if you do research. For much of the rest of the time, you will probably find it easier to check the units, particularly if you are using the SI system. In the above example, you think:

  * Pressure = force / area
  * Force = mass x acceleration
  * Pressure = mass x acceleration / area

So 1 pascal (unit of pressure) = 1 kg x (m s-2) / (m2) = 1 kg m-1 s-2

  * Now _g_ is 9.81 m s-2 and _h_ is in meters
  * So _gh_ is in units m2 s-2

To make _gh_ match pressure we need to multiply by something having the units kg m-3, which we recognise as density.

Note dimensional analysis (or unit checking) does not tell you about numerical values that you might have to insert, such as 9.81 or π. Nor does it tell you if you should use the radius or the diameter of a pipe in fluid mechanics!

## Importance of Significant Figures

Significant figures (also called significant digits) are an important part of scientific and mathematical calculations, and deals with the accuracy and precision of numbers. It is important to estimate uncertainty in the final result, and this is where significant figures become very important.

### Precision and Accuracy

Before discussing how to deal with significant figures one should discuss what precision and accuracy in relation to chemical experiments and engineering. **Precision** refers to the reproducibility of results and measurements in an experiment, while **accuracy** refers to how close the value is to the actual or true value. Results can be both precise and accurate, neither precise nor accurate, precise and not accurate, or vice versa. The validity of the results increases as they are more accurate and precise.

An useful analogy that helps distinguish the difference between accuracy and precision is the use of a target. The bullseye of the target represents the true value, while the holes made by each shot (each trial) represents the validity.

  * ![](../../../upload.wikimedia.org/wikipedia/commons/thumb/5/51/Accuracy_and_precision-highprecisionlowaccuracy.gif/120px-Accuracy_and_precision-highprecisionlowaccuracy.gif)

High precision, but unfortunately low accuracy

  * ![](../../../upload.wikimedia.org/wikipedia/commons/thumb/6/67/Accuracy_and_precision-highaccuracylowprecision.gif/120px-Accuracy_and_precision-highaccuracylowprecision.gif)

High accuracy, but not very good precision

As the above images show, the first has a lot of holes (black spots) covering a small area. The small area represents a precise experiment, yet it seems that there is a faultiness within the experiment, most likely due to systematic error, rather than random error. The second image represents an accurate though imprecise experiment. The holes are near the bullseye, even "touching" or within, though the problem is that they are spread out. This could be due to random error, systematic error, or not being careful in measuring.

### Counting Significant Figures

There are three preliminary rules to counting significant. They deal with non-zero numbers, zeros, and exact numbers.

1) _Non-zero numbers_ \- all non-zero numbers are considered significant figures

2) _Zeros_ \- there are three different types of zeros

  * _leading zeros_ \- zeros that precede digits - do not count as significant figures (example: .0002 has one significant figure)
  * _captive zeros_ \- zeros that are "caught" between two digits - do count as significant figures (example: 101.205 has six significant figures)
  * _trailing zeros_ \- zeros that are at the end of a string of numbers and zeros - only count if there is a decimal place (example: 100 has one significant figure, while 1.00 has three, as well as 100.)

3) _Exact numbers_ \- these are numbers not obtained by measurements, and are determined by counting. An example of this is if one counted the number of millimetres in a centimetre (**10** \- it is the definition of a millimetre), but another example would would be if you have **3** apples.

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  

    
    
    How many significant figures do the following numbers have? Assume none of them are exact numbers.
    

a) 4.2362 - all numbers, so five

b) 2.0 - zeros after a decimal point count, so two

c) 9900 - only two in this case, because there is no decimal point

d) .44205 - there is a "captive zero," which means it counts, so five

e) .05 - only the five counts, so one

f) 3.9400E9 - tricky one, but scientific notation helps make the zeros at the end noticeable; there are five

### The Parable of the Cement Block

People new to the field often question the importance of significant figures, but they have great practical importance, for they are a quick way to tell how accurate a number is. Including too many can not only make your numbers harder to read, it can also have serious negative consequences.

As an anecdote, consider two engineers who work for a construction company. They need to order cement bricks for a certain project. They have to build a wall that is 10 feet wide, and plan to lay the base with 30 bricks. The first engineer does not consider the importance of significant figures and calculates that the bricks need to be 0.3333 feet wide and the second does and reports the number as 0.33, figuring that a precision of ![ \\pm 0.01ft ](../../../upload.wikimedia.org/math/e/2/1/e217fd5006d442b4d179b762686a98ec.png) (0.1 inches) would be precise enough for the work she was doing.

Now, when the cement company received the orders from the first engineer, they had a great deal of trouble. Their machines were precise but not so precise that they could consistently cut to within 0.0001 feet. However, after a good deal of trial and error and testing, and some waste from products that did not meet the specification, they finally machined all of the bricks that were needed. The other engineer's orders were much easier, and generated minimal waste.

When the engineers received the bills, they compared the bill for the services, and the first one was shocked at how expensive hers was. When they consulted with the company, the company explained the situation: they needed such a high precision for the first order that they required significant extra labor to meet the specification, as well as some extra material. Therefore it was much more costly to produce.

What is the point of this story? Significant figures matter. It is important to have a reasonable gauge of how accurate a number is so that you know not only what the number is but how much you can trust it and how limited it is. The engineer will have to make decisions about how precisely he or she needs to specify design specifications, and how precise measurement instruments (and control systems!) have to be. If you do not need 99.9999% purity then you probably don't need an expensive assay to detect generic impurities at a 0.0001% level (though the lab technicians will probably have to still test for heavy metals and such), and likewise you will not have to design nearly as large of a distillation column to achieve the separations necessary for such a high purity.

### Mathematical Operations and Significant Figures

Most likely at one point, the numbers obtained in one's measurements will be used within mathematical operations. What does one do if each number has a different amount of significant figures? If one adds 2.0 litres of liquid with 1.000252 litres, how much does one have afterwards? What would 2.45 times 223.5 get?

For addition and subtraction, the result has the same number of **decimal places** as the least precise measurement use in the calculation. This means that 112.420020 + 5.2105231 + 1.4 would have have a single decimal place but there can be any amount of numbers to the left of the decimal point (in this case the answer is 119.0).

For multiplication and division, the number that is the least precise measurement, or the **number of digits**. This means that 2.499 is more precise than 2.7, since the former has four digits while the latter has two. This means that 5.000 divided by 2.5 (both being measurements of some kind) would lead to an answer of 2.0.

### Rounding

So now you know how to pick which numbers to drop if there is a question about significant figures, but one also has to take into account rounding. Once one has decided which digit should be the last digit kept, one must decide whether to round up or down.

  * If the number is greater than five (6 to 9), one rounds up - 1.36 becomes 1.4
  * If the number is less than five (1 to 4), one rounds down - 1.34 becomes 1.3

What does one do when there is a five? There is a special case that deals with the number five, since, if you have not noticed, it is in the middle (between 1 and 9). Often in primary school one learns to just round up, but engineers tend to do something different, called unbiased rounding.

  * If the number before the five is even, then one rounds down - 1.45 becomes 1.4
  * If the number before the five is odd, then one rounds up - 1.55 becomes 1.6
  * Another case is this: 1.4501, where the numbers after five are greater than zero, so one would round to 1.5

Note: **Remember that rounding is generally done at the end of calculations, not before the calculations are made.**

Why is this done? Engineers make many calculations that often matter, since time, money, etc. are being taken into account, it is best to make sure that the final results are not synthetic or untrue to what the actual value should be. This relates back to accuracy and precision.

![](../../../upload.wikimedia.org/wikipedia/commons/thumb/1/1d/Information_icon4.svg/40px-Information_icon4.svg.png)
This is just a proposed layout and content. To discuss, go to the [discussion page](http://en.wikibooks.org/w/index.php?title=Talk:Introduction_to_Chemical_Engineering_Processes/General_chemistry_review&action=edit&redlink=1) or edit it yourself.

  


## Stoichiometry

[Le Système International d'Unités (SI Units)](http://en.wikibooks.org/wiki/Introduction_to_Chemical_Engineering_Processes)

### Mole

## Periodic Table

### Key Elements and Molecules

## Acid-Base

## Bonding

## Structure and Formula

## Ideal Gas Law

![PV = nRT](../../../upload.wikimedia.org/math/e/0/4/e0412dbb6e50de76a383b8f6e7d81dce.png)

P = Pressure; V = Volume; n = moles; R = Ideal gas constant; T = Temperature

## Enthalpy

## Entropy

## Branches of Chemistry

  * **Inorganic Chemistry** \- The study of the synthesis and behavior of inorganic and organometallic compounds. This field covers all chemical compounds except the myriad organic compounds.
  * **Organic Chemistry** \- The study of the structure, properties, and reactions of organic compounds and organic materials, i.e., matter in its various forms that contain carbon atoms.
  * **Physical Chemistry** \- The study of macroscopic, atomic, subatomic, and particulate phenomena in chemical systems in terms of laws and concepts of physics. It applies the principles, practices and concepts of physics such as motion, energy, force, time, thermodynamics, quantum chemistry, statistical mechanics and dynamics, equilibrium.
  * **Analytical Chemistry** -
  * **Biochemistry** -
  * **Organometallic chemistry** -

## Chapter 1 Practice Problems

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Problem:**  


**1.** Perform the following conversions, using the appropriate number of significant figures in your answer:

_a)_ ![ 1.5 \\frac{g}{s} \\rightarrow \\frac{lb}{hr} ](../../../upload.wikimedia.org/math/3/6/c/36c18edf7b0bd8b7e6c2b5db19dd4f99.png)

_b)_ ![ 4.5 * 10^2 \\mbox{ W} \\rightarrow \\frac{btu}{min} ](../../../upload.wikimedia.org/math/d/f/c/dfca4cbaaf45ce0e4683767f61297ddb.png)

_c)_ ![ 34 \\frac{\\mu g}{\\mu m^3} \\rightarrow \\frac{oz}{in^3} ](../../../upload.wikimedia.org/math/b/0/5/b05e3bbb56cb122c0cb8ec8d7522bb21.png)

_d)_ ![ 4.18 \\frac{J}{g*oC} \\rightarrow \\frac{kWh}{lb*oF} ](../../../upload.wikimedia.org/math/2/7/3/273ceef0fd25c6f427707bda30995842.png) (note: kWh means kilowatt-hour)

_e)_ ![ 1.00 \\mbox{ m}^3 \\rightarrow L \\rightarrow dm^3 \\rightarrow mL \\rightarrow cm^3 ](../../../upload.wikimedia.org/math/1/3/6/1360f987a8549c40645eb9be40fda31c.png)

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Problem:**  


**2.** Perform a dimensional analysis on the following equations to determine if they are reasonable:

_a)_ ![ v = dt ](../../../upload.wikimedia.org/math/0/f/6/0f6504a996237acd6e7886577b8f79d9.png), where v is velocity, d is distance, and t is time.

_b)_ ![ F = \\frac{m*v^2}{r} ](../../../upload.wikimedia.org/math/f/9/f/f9f6133aed45d3a99af7d4cefcd3aa85.png) where F is force, m is mass, v is velocity, and r is radius (a distance).

_c)_ ![ F_{bouy} = \\rho*V*g ](../../../upload.wikimedia.org/math/7/a/1/7a1d99b069a7acf49c1ec58c00ae8ede.png) where ![ \\rho ](../../../upload.wikimedia.org/math/f/7/f/f7f177957cf064a93e9811df8fe65ed1.png) is density, V is volume, and g is gravitational acceleration.

_d)_ ![ \\dot{m} = \\frac{\\dot{V}}{\\rho} ](../../../upload.wikimedia.org/math/0/f/4/0f4db102c0e1d5942515d40219cee48a.png) where ![ \\dot{m} ](../../../upload.wikimedia.org/math/a/a/b/aab3acd5d09b124c29056f56620db26e.png) is mass flow rate, ![ \\dot{V} ](../../../upload.wikimedia.org/math/5/6/8/568172c8e93d9cddd514d06931fc1239.png) is volumetric flow rate, and ![ \\rho ](../../../upload.wikimedia.org/math/f/7/f/f7f177957cf064a93e9811df8fe65ed1.png) is density.

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Problem:**  


**3.** Recall that the ideal gas law is ![ PV=nRT ](../../../upload.wikimedia.org/math/e/0/4/e0412dbb6e50de76a383b8f6e7d81dce.png) where P is pressure, V is volume, n is number of moles, R is a constant, and T is the temperature.

_a)_ What are the units of R in terms of the base unit types (length, time, mass, and temperature)?

_b)_ Show how these two values of R are equivalent: ![ R=0.0821\\frac{L*atm}{mol*K} = 8.31\\frac{J}{mol*K} ](../../../upload.wikimedia.org/math/d/f/9/df98f850e0d06368644d1d33aa0cf638.png)

_c)_ If an ideal gas exists in a closed container with a molar density of ![ 0.03 \\frac{mol}{L} ](../../../upload.wikimedia.org/math/f/7/a/f7a411ec4c83f4690fc01bfe113a8a5b.png) at a pressure of ![ 0.96*10^5\\mbox{ Pa} ](../../../upload.wikimedia.org/math/2/7/1/2716e74f50256e670d406da3f46e9b20.png), what temperature is the container held at?

_d)_ What is the molar concentration of an ideal gas with a partial pressure of ![ 4.5*10^5\\mbox{ Pa} ](../../../upload.wikimedia.org/math/b/b/8/bb82a62af1bd0f6bccee8a7beca4bb8b.png) if the total pressure in the container is ![ 6\\mbox{ atm} ](../../../upload.wikimedia.org/math/7/b/3/7b34f2505f39fd570d5d2f92f352bc0c.png)?

_e)_ At what temperatures and pressures is a gas most and least likely to be ideal? (hint: you can't use it when you have a liquid)

_f)_ Suppose you want to mix ideal gasses in two separate tanks together. The first tank is held at a pressure of 500 Torr and contains 50 moles of water vapor and 30 moles of water at 70oC. The second is held at 400 Torr and 70oC. The volume of the second tank is the same as that of the first, and the ratio of moles water vapor to moles of water is the same in both tanks.

You recombine the gasses into a single tank the same size as the first two. Assuming that the temperature remains constant, what is the pressure in the final tank? If the tank can withstand 1 atm pressure, will it blow up?

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Problem:**  


**4.** Consider the reaction ![ H_2O_2 <-> H_2O + \\frac{1}{2} O_2 ](../../../upload.wikimedia.org/math/0/6/6/0668a1c653dcb184712d5a83cd87fe4a.png), which is carried out by many organisms as a way to eliminate hydrogen peroxide.

_a)_. What is the standard enthalpy of this reaction? Under what conditions does it hold?

_b)_. What is the standard Gibbs energy change of this reaction? Under what conditions does it hold? In what direction is the reaction spontaneous at standard conditions?

_c)_. What is the Gibbs energy change at biological conditions (1 atm and 37oC) if the initial hydrogen peroxide concentration is 0.01M? Assume oxygen is the only gas present in the cell.

_d)_. What is the equilibrium constant under the conditions in part c? Under the conditions in part b)? What is the constant independent of?

_e)_. Repeat parts a through d for the alternative reaction ![ H_2O_2 \\rightarrow H_2 + O_2 ](../../../upload.wikimedia.org/math/9/d/5/9d5fa813262840c6138da3801128cb2f.png). Why isn't this reaction used instead?

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Problem:**  


**5.** Two ideal gasses A and B combine to form a third ideal gas, C, in the reaction ![ A + B \\rightarrow C ](../../../upload.wikimedia.org/math/a/7/1/a71d4d1b231d038dfb65eaf890821b7d.png). Suppose that the reaction is irreversible and occurs at a constant temperature of 25oC in a 5L container. If you start with 0.2 moles of A and 0.5 moles of B at a total pressure of 1.04 atm, what will the pressure be when the reaction is completed?

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Problem:**  


**6.** How much heat is released when 45 grams of methane are burned in excess air under standard conditions? How about when the same mass of glucose is burned? What is one possible reason why most heterotrophic organisms use glucose instead of methane as a fuel? Assume that the combustion is complete, i.e. no carbon monoxide is formed.

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Problem:**  


**7.** Suppose that you have carbon monoxide and water in a tank in a 1.5:1 ratio.

_a)_ In the literature, find the reaction that these two compounds undergo (hint: look for the _water gas shift_ reaction). Why is it an important reaction?

_b)_ Using a table of Gibbs energies of formation, calculate the equilibrium constant for the reaction.

_c)_ How much hydrogen can be produced from this initial mixture?

_d)_ What are some ways in which the yield of hydrogen can be increased? (hint: recall Le Chatlier's principle for equilibrium).

_e)_ What factors do you think may influence how long it takes for the reaction to reach equilibrium?

Problem 8. A bio-fuel plant converts the sugars (glucose) in corn into ethanol and carbon dioxide in a process called fermentation. The plant produces 100 gpm (gallons per minute) of ethanol and can produce of 2.5 gallons of ethanol per bushel of corn. Jimmy farms a total of 2000 acres, 75% of which are corn. He sells 80% of his corn supply to the bio-fuel plant for $4.00/bushel. (Hint: 120 bushels = 1 acre)

a) How long can the plant run with the supply of corn from Jimmy? (hours)

b) How much money did Jimmy make for his corn?

  
[Solutions](http://en.wikibooks.org/w/index.php?title=Introduction_to_Chemical_Engineering_Processes/Print_Version/Solutions&action=edit&redlink=1)

# **Chapter 2: Elementary mass balances**

## The "Black Box" approach to problem-solving

In this book, all the problems you'll solve will be "black-box" problems. This means that we take a look at a unit operation _from the outside_, looking at what goes into the system and what leaves, and extrapolating data about the properties of the entrance and exit streams from this. This type of analysis is important because it does not depend on the specific type of unit operation that is performed. _When doing a black-box analysis, we don't care about how the unit operation is designed, only what the net result is._ Let's look at an example:

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Suppose that you pour 1L of water into the top end of a funnel, and that funnel leads into a large flask, and you measure that the entire liter of water enters the flask. If the funnel had no water in it to begin with, how much is left over after the process is completed?

**Solution** The answer, of course, is 0, because you only put 1L of water in, and 1L of water came out the other end. The answer to this does not depend on the how large the funnel is, the slope of the sides, or any other design aspect of the funnel, which is why it is a black-box problem.

### Conservation equations

The formal mathematical way of describing the black-box approach is with **conservation equations** which explicitly state that what goes into the system must either come out of the system somewhere else, get used up or generated by the system, or remain in the system and **accumulate**. The relationship between these is simple:

  1. The streams entering the system cause an increase of the substance (mass, energy, momentum, etc.) in the system.
  2. The streams leaving the system decrease the amount of the substance in the system.
  3. Generating or consuming mechanisms (such as chemical reactions) can either increase or decrease the stuff in the system.
  4. What's left over is the amount of stuff in the system

With these four statements we can state the following very important general principle:

  


![ Accumulation = In-Out+Generation-Consumption ](../../../upload.wikimedia.org/math/9/a/d/9ad4dae3636df5f77859e5919091bd82.png)

Its so important, in fact, that you'll see it a million times or so, including a few in this book, and it is used to derive a variety of forms of conservation equations.

### Common assumptions on the conservation equation

The conservation equation is very general and applies to any property a system can have. However, it can also lead to complicated equations, and so in order to simplify calculations when appropriate, it is useful to apply assumptions to the problem.

  * **Closed system**: A closed system is one which does not have flows in or out of the substance. Almost always, when one refers to a closed system, it is implied that the system is closed to _mass flow_ but not to other flows such as energy or momentum. The equation for a closed system is:

    ![ Accumulation = Generation ](../../../upload.wikimedia.org/math/2/e/d/2ed7a198eb80f202bf360ffc2b2d4572.png)
    The opposite of a closed system is an **open system** in which the substance is allowed to enter and/or leave the system. The funnel in the example was an open system because mass flowed in and out of it.

  * **No generation**: Certain quantities are always **conserved** in the strict sense that they are never created or destroyed. These are the most useful quantities to do balances on because then the model does not need to include a generation term:

    ![ Accumulation = In - Out ](../../../upload.wikimedia.org/math/c/a/a/caa02fd623c15510bdc2329f29c321b4.png)
    The most commonly-used conserved quantities in this class are **mass** and **energy** (other conserved quantities include momentum and electric charge). However, it is important to note that though the _total_ mass and _total_ energy in a system are conserved, the mass of a single species is **not** (since it may be changed into something else in a reaction). Neither is the "heat" in a system if a so-called "heat-balance" is performed (since it may be transformed into other forms of energy. Therefore, one must be careful when deciding whether to discard the generation term).

  * **Steady State**: A system which does not accumulate a substance is said to be at steady-state. Often times, this allows the engineer to avoid having to solve differential equations and instead use algebra.

    ![ In - Out + Generation - Consumption = 0 ](../../../upload.wikimedia.org/math/e/c/a/eca8934afe1797e12d2bfa2341be97d4.png)
    _All problems in this text assume steady state but it is not always a valid assumption_. It is mostly valid after a process has been running in a controlled manner for long enough that all the flow rates, temperatures, pressures, and other system parameters have reached reasonably constant values. It is not valid when a process is first warming up (or an operating condition is changed) and the system properties change significantly over time. How they change, and how long it takes to become close enough to steady state, is a subject for another course.

## Conservation of mass

TOTAL mass is a conserved quantity (except in nuclear reactions, let's not go there), as is the mass of any individual species if there is no chemical reaction occurring in the system. Let us write the conservation equation _at steady state_ for such a case (with no reaction):

![ In - Out = 0 ](../../../upload.wikimedia.org/math/b/7/9/b7919f399d989e541237c55823ef5c4d.png)

Now, there are two major ways in which mass can enter or leave a system: diffusion and convection. However, if the velocity entering the unit operations is fairly large and the concentration gradient is fairly small, diffusion can be neglected and the only mass entering or leaving the system is due to convective flow:

![ Mass_{in}  = \\dot{m}_{in} = \\rho*v*A ](../../../upload.wikimedia.org/math/2/a/c/2acf39079517f8ca0ff919952991f24f.png)

A similar equation apply for the mass out.

In this book, we generally use the symbol ![ \\dot{m} ](../../../upload.wikimedia.org/math/a/a/b/aab3acd5d09b124c29056f56620db26e.png) to signify a convective mass flow rate, in units of ![ mass/time ](../../../upload.wikimedia.org/math/0/9/c/09c4e06a90647f65a245eadc08a8a5e2.png). Since the total flow in is the sum of individual flows, and the same with the flow out, the following **steady state mass balance** is obtained for the overall mass in the system:

  


![ \\sum{\\dot{m}_{out}}-\\sum{\\dot{m}_{in}}=0 ](../../../upload.wikimedia.org/math/9/c/b/9cb05b4ba5a48ff13e5347f19866f967.png)

If it is a _batch_ system, or if we're looking at how much has entered and left in a given period of time (rather than instantaneously), we can apply the same mass balance without the time component. In this book, a value without the dot signifies a value without a time component:

![ \\sum{m_{out}}-\\sum{m_{in}} = 0 ](../../../upload.wikimedia.org/math/c/2/2/c229da308de40b5a59b780a63616eeeb.png)

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


  
Let's work out the previous example (the funnel), but explicitly state the mass balance. We're given the following information:

  1. ![ {m_{in}} = 1 L ](../../../upload.wikimedia.org/math/9/e/3/9e3270db95776abf32cdea32109cb5f3.png)
  2. ![ {m_{out}} = 1 L ](../../../upload.wikimedia.org/math/6/a/b/6abbde1155b6ca67288ce7497b99a0dd.png)

From the general balance equation,

![ In - Out = Accumulation ](../../../upload.wikimedia.org/math/f/e/f/fef9e5c6bb0d251220caf96371168d80.png)

Therefore, ![ Accumulation = 1L - 1L = 0 ](../../../upload.wikimedia.org/math/c/e/c/cec58a56e59ad91e22d598ccf3f9e979.png).

Since the accumulation is 0, the system is at steady state.

This is a fairly trivial example, but it gets the concepts of "in", "out", and "accumulation" on a physical basis, which is important for setting up problems. In the next section, it will be shown how to apply the mass balance to solve more complex problems with only one component.

## Converting Information into Mass Flows - Introduction

In any system there will be certain parameters that are easier (often considerably) to measure and/or control than others. When you are solving any problem and trying to use a mass balance or any other equation, _it is important to recognize what pieces of information can be interconverted_. The purpose of this section is to show some of the more common alternative ways that mass flow rates are expressed, mostly because it is easier to, for example, [measure a velocity](http://en.wikipedia.org/wiki/orifice_plate) than it is to measure a mass flow rate directly.

## Volumetric Flow rates

A **volumetric flow rate** is a relation of how much volume of a gas or liquid solution passes through a fixed point in a system (typically the entrance or exit point of a process) in a given amount of time. It is denoted as:

  
![ \\dot{V}_n \\dot= \\frac{Volume}{time} ](../../../upload.wikimedia.org/math/3/c/4/3c4b24807485cbd0c3be2a983d9f1b4a.png) in stream n

Volume in the metric system is typically expressed either in L (dm^3), mL (cm^3), or m^3. Note that a cubic meter is very large; a cubic meter of water weighs about 1000kg (2200 pounds) at room temperature!

### Why they're useful

Volumetric flow rates can be measured directly using **flow meters**. They are especially useful for gases since the volume of a gas is one of the four properties that are needed in order to use an _equation of state_ (discussed later in the book) to calculate the molar flow rate. Of the other three, two (pressure, and temperature) can be specified by the reactor design and control systems, while one (compressibility) is strictly a function of temperature and pressure for any gas or gaseous mixture.

### Limitations

**Volumetric Flowrates are Not Conserved**. We can write a balance on volume like anything else, but the "volume generation" term would be a complex function of system properties. Therefore if we are given a volumetric flow rate we should change it into a mass (or mole) flow rate before applying the balance equations.

Volumetric flowrates also do not lend themselves to splitting into components, since when we speak of volumes in practical terms we generally think of the total solution volume, not the partial volume of each component (the latter is a useful tool for thermodynamics, but that's another course entirely). There **are** some things that are measured in volume fractions, but this is relatively uncommon.

### How to convert volumetric flow rates to mass flow rates

Volumetric flowrates are related to mass flow rates by a relatively easy-to-measure physical property. Since ![ \\dot{m} \\dot= mass/time ](../../../upload.wikimedia.org/math/c/7/b/c7b802dc12850a6ea6c692eaa5e32518.png) and ![ \\dot{V} \\dot= volume/time ](../../../upload.wikimedia.org/math/f/c/5/fc599bca0421675dc19a89cf2ad2a8c9.png), we need a property with units of ![ mass/volume ](../../../upload.wikimedia.org/math/4/6/1/46158653bad2642c07fde35a96c14297.png) in order to convert them. The **density** serves this purpose nicely!

  
![ \\dot{V}_n*{\\rho}_n = \\dot{m}_n ](../../../upload.wikimedia.org/math/7/7/2/772995d7fe793decb975ad58bb711e33.png) in stream n

The "i" indicates that we're talking about one particular flow stream here, since each flow may have a different density, mass flow rate, or volumetric flow rate.

## Velocities

The velocity of a bulk fluid is _how much lateral distance along the system (usually a pipe) it passes per unit time_. The velocity of a bulk fluid, like any other, has units of:

  
![ v_n = \\frac{distance}{time} ](../../../upload.wikimedia.org/math/0/4/0/040bc5309354295e5282a896dbbc084e.png) in stream n

By definition, the bulk velocity of a fluid is related to the volumetric flow rate by:

  
![{v}_n = \\frac{\\dot{V}_n}{A_n} ](../../../upload.wikimedia.org/math/f/1/6/f16b300b60cd313e428aafbd1456ddf9.png) in stream n

This distinguishes it from the velocity of the fluid at a certain point (since fluids flow faster in the center of a pipe). The bulk velocity is about the same as the instantaneous velocity for relatively fast flow, or especially for flow of gasses.

For purposes of this class, all velocities given will be bulk velocities, not instantaneous velocities.

### Why they're useful

(Bulk) Velocities are useful because, like volumetric flow rates, they are relatively easy to measure. They are especially useful for liquids since they have constant density (and therefore a constant pressure drop at steady state) as they pass through the orifice or other similar instruments. This is a necessary prerequisite to use the design equations for these instruments.

### Limitations

Like volumetric flowrates, **velocity is not conserved**. Like volumetric flowrate, velocity changes with temperature and pressure of a gas, though for a liquid velocity is generally constant along the length of a pipe.

Also, velocities can't be split into the flows of individual components, since all of the components will generally flow at the same speed. They need to be converted into something that can be split (mass flow rate, molar flow rate, or pressure for a gas) before concentrations can be applied.

### How to convert velocity into mass flow rate

In order to convert the velocity of a fluid stream into a mass flow rate, you need two pieces of information:

  1. The **cross sectional area** of the pipe.
  2. The **density** of the fluid.

In order to convert, first use the definition of bulk velocity to convert it into a volumetric flow rate:

![ \\dot{V}_n = v_n*A_n ](../../../upload.wikimedia.org/math/2/a/1/2a100167b460fcbf90acb7bb55f1144b.png)

Then use the density to convert the volumetric flow rate into a mass flow rate.

![ \\dot{m}_n = \\dot{V}_n*{\\rho}_n ](../../../upload.wikimedia.org/math/b/5/8/b58fff4b65b11aa89bc6aa9101e3e381.png)

The combination of these two equations is useful:

  
![ \\dot{m}_n = v_n*{\\rho}_n*A_n ](../../../upload.wikimedia.org/math/3/d/3/3d3890f0e794ce420afab7ad42c937f5.png) in stream n

## Molar Flow Rates

The concept of a molar flow rate is similar to that of a mass flow rate, it is the number of moles of a solution (or mixture) that pass a fixed point per unit time:

  
![ \\dot{n}_n \\dot= \\frac{moles}{time} ](../../../upload.wikimedia.org/math/2/9/0/290cbf32acd614bbd9e5fcaff58b9cd1.png) in stream n

### Why they're useful

Molar flow rates are mostly useful because _using moles instead of mass allows you to write material balances in terms of reaction conversion and stoichiometry_. In other words, there are a lot fewer unknowns when you use a mole balance, since the stoichiometry allows you to consolidate all of the changes in the reactant and product concentrations in terms of one variable. This will be discussed more in a later chapter.

### Limitations

Unlike mass, **total moles are not conserved**. Total mass flow rate is conserved whether there is a reaction or not, but the same is not true for the number of moles. For example, consider the reaction between hydrogen and oxygen gasses to form water:

![ H_2 + \\frac{1}{2}O_2 \\rightarrow H_2O ](../../../upload.wikimedia.org/math/f/d/4/fd4002de0adb8225de64cfcf5c196f4d.png)

This reaction consumes 1.5 moles of reactants for every mole of products produced, and therefore the total number of moles entering the reactor will be more than the number leaving it.

However, since neither mass nor moles of individual components is conserved in a reacting system, it's better to use moles so that the stoichiometry can be exploited, as described later.

The molar flows are also somewhat less practical than mass flow rates, since you can't measure moles directly but you can measure the mass of something, and then convert it to moles using the molar flow rate.

### How to Change from Molar Flow Rate to Mass Flow Rate

Molar flow rates and mass flow rates are related by the **molecular weight** (also known as the **molar mass**) of the solution. In order to convert the mass and molar flow rates of the _entire solution_, we need to know the **average molecular weight** of the solution. This can be calculated from the molecular weights and mole fractions of the components using the formula:

![ \\bar{MW}_n = \[\\Sigma\({MW}_i*y_i\)\]_n ](../../../upload.wikimedia.org/math/e/7/d/e7dad084c01b157888dd29de682b04a3.png)

where i is an index of _components_ and n is the _stream_ number. ![ y_i ](../../../upload.wikimedia.org/math/1/8/d/18daef71b5d25ce76b8628a81e4fc76b.png) signifies _mole fraction_ of each component (this will all be defined and derived later).

Once this is known it can be used as you would use a molar mass for a single component to find the total molar flow rate.

  
![ \\dot{m}_n = \\dot{n}_n * \\bar{MW}_n ](../../../upload.wikimedia.org/math/e/e/e/eee2ab9553f76ac7f7615a34847d5241.png) in stream n

## A Typical Type of Problem

Most problems you will face are significantly more complicated than the previous problem and the following one. In the engineering world, problems are presented as so-called "word problems", in which a system is described and the problem must be set up and solved (if possible) from the description. This section will attempt to illustrate through example, step by step, some common techniques and pitfalls in setting up mass balances. Some of the steps may seem somewhat excessive at this point, but if you follow them carefully on this relatively simple problem, you will certainly have an easier time following later steps.

## Single Component in Multiple Processes: a Steam Process

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


  
A feed stream of pure liquid water enters an evaporator at a rate of 0.5 kg/s. Three streams come from the evaporator: a vapor stream and two liquid streams. The flowrate of the vapor stream was measured to be 4*10^6 L/min and its density was 4 g/m^3. The vapor stream enters a turbine, where it loses enough energy to condense fully and leave as a single stream. One of the liquid streams is discharged as waste, the other is fed into a heat exchanger, where it is cooled. This stream leaves the heat exchanger at a rate of 1500 pounds per hour. Calculate the flow rate of the discharge and the efficiency of the evaporator.

Note that one way to define efficiency is in terms of conversion, which is intended here:

  
![ efficiency = \\frac{\\dot{m}_{vapor}}{\\dot{m}_{feed}} ](../../../upload.wikimedia.org/math/2/7/0/270fd26bee346f79b048d049b693f7eb.png)

### Step 1: Draw a Flowchart

The problem as it stands contains an awful lot of text, but it won't mean much until you _draw what is given to you_. First, ask yourself, what processes are in use in this problem? **Make a list** of the processes in the problem:

  1. Evaporator (A)
  2. Heat Exchanger (B)
  3. Turbine (C)

Once you have a list of all the processes, you need to **find out how they are connected** (it'll tell you something like "the vapor stream enters a turbine"). Draw a basic sketch of the processes and their connections, and label the processes. It should look something like this: ![FlowchartStep1.PNG](../../../upload.wikimedia.org/wikibooks/en/8/88/FlowchartStep1.PNG)

Remember, we don't care what the actual processes look like, or how they're designed. At this point, we only really label what they are so that we can go back to the problem and know which process they're talking about.

Once all your processes are connected, **find any streams that are not yet accounted for**. In this case, we have not drawn the feed stream into the evaporator, the waste stream from the evaporator, or the exit streams from the turbine and heat exchanger.

![FlowchartStep2.PNG](../../../upload.wikimedia.org/wikibooks/en/e/ee/FlowchartStep2.PNG)

The third step is to **Label all your flows**. Label them with any information you are given. Any information you are not given, and even information you are given should be given a different variable. It is usually easiest to give them the same variable as is found in the equation you will be using (for example, if you have an unknown flow rate, call it ![ \\dot{m} ](../../../upload.wikimedia.org/math/a/a/b/aab3acd5d09b124c29056f56620db26e.png) so it remains clear what the unknown value is physically. Give each a different subscript corresponding to the number of the feed stream (such as ![ \\dot{m_1} ](../../../upload.wikimedia.org/math/f/a/6/fa66dae4fb5590cb590c680792e2ca40.png) for the feed stream that you call "stream 1"). Make sure you **include all units** on the given values!

In the example problem, the flowchart I drew with all flows labeled looked like this:

![FlowchartStep3.PNG](../../../upload.wikimedia.org/wikibooks/en/3/34/FlowchartStep3.PNG)

Notice that for one of the streams, a _volume_ flow rate is given rather than a _mass_ flow rate, so it is labeled as such. This is very important, so that you avoid using a value in an equation that isn't valid (for example, there's no such thing as "conservation of volume" for most cases)!

The final step in drawing the flowchart is to **write down any additional given information in terms of the variables you have defined**. In this problem, the density of the water in the vapor stream is given, so write this on the side for future reference.

Carefully drawn flowcharts and diagrams are half of the key to solving any mass balance, or really a lot of other types of engineering problems. They are just as important as having the right units to getting the right answer.

### Step 2: Make sure your units are consistent

The second step is to make sure all your units are consistent and, if not, to convert everything so that it is. In this case, since the principle that we'll need to use to solve for the flow rate of the waste stream (![ \\dot{m_3} ](../../../upload.wikimedia.org/math/c/3/2/c328568f2fa2389c8240f35ee13fc501.png)) is conservation of mass, everything will need to be on a mass-flow basis, and also in the _same_ mass-flow units.

In this problem, since two of our flow rates are given in metric units (though one is a volumetric flow rate rather than a mass flow rate, so we'll need to change that) and only one in English units, it would save time and minimize mistakes to convert ![ \\dot{V_2} ](../../../upload.wikimedia.org/math/b/9/6/b966a07b63d2dfbd28e1a1f1ebd0dfac.png) and ![ \\dot{m_5} ](../../../upload.wikimedia.org/math/0/3/9/039d3226e977881c6179a981b9bb02e0.png) to kg/s.

From the previous section, the equation relating volumetric flowrate to mass flow rate is:

![ \\dot{V}_i*{\\rho}_i = \\dot{m}_i ](../../../upload.wikimedia.org/math/6/8/2/682d1d70a7a999bd50bdf9105c312fe9.png)

Therefore, we need the density of water vapor in order to calculate the mass flow rate from the volumetric flow rate. Since the density is provided in the problem statement (if it wasn't, we'd need to calculate it with methods described later), the mass flow rate can be calculated:

![ \\dot{V_2} = \\frac{4*10^6 \\mbox{ L}}{1\\mbox{ min}}*\\frac{1\\mbox{ m}^3}{1000\\mbox{ L}}*\\frac{1\\mbox{ min}}{60 \\mbox{ s}} = 66.67 \\frac{m^3}{s} ](../../../upload.wikimedia.org/math/4/e/4/4e4d7370b4cb712cf62c1863344c7253.png)

![ \\rho_2 = 4 \\frac{g}{m^3} * \\frac{1\\mbox{ kg}}{1000\\mbox{ g}} = 0.004 \\frac{kg}{m^3} ](../../../upload.wikimedia.org/math/8/a/5/8a5a5ad4daf6afdb2816327698644833.png)

![ \\dot{m}_2 = 66.67\\frac{m^3}{s} * 0.004 \\frac{kg}{m^3} = 0.2666 \\frac{kg}{s} ](../../../upload.wikimedia.org/math/c/8/b/c8b5917b0cef828e3683ed26e5274fd0.png)

Note that since the density of a gas is so small, a huge volumetric flow rate is necessary to achieve any significant mass flow rate. This is fairly typical and is a practical problem when dealing with gas-phase processes.

The mass flow rate ![ \\dot{m_5} ](../../../upload.wikimedia.org/math/0/3/9/039d3226e977881c6179a981b9bb02e0.png) can be changed in a similar manner, but since it is already in terms of mass (or weight technically), we don't need to apply a density:

![ \\dot{m_5} = 1500\\frac{lb}{hr} * \\frac{1kg}{2.2lb} * \\frac{1hr}{3600s} = 0.1893 \\frac{kg}{s} ](../../../upload.wikimedia.org/math/4/4/8/448bdb3849ed09e405621bfb851bcac6.png)

Now that everything is in the same system of units, we can proceed to the next step.

### Step 3: Relate your variables

Since we have the mass flow rate of the vapor stream we can calculate the efficiency of the evaporator directly:

![ efficiency = \\frac{\\dot{m}_2}{\\dot{m}_1} = \\frac{0.2666 \\frac{kg}{s}}{0.5 \\frac{kg}{s}} = 53.3% ](../../../upload.wikimedia.org/math/3/d/f/3df9a3c48a64efb8b6f93c8f33c4eb72.png)

Finding ![ \\dot{m_4} ](../../../upload.wikimedia.org/math/8/9/4/8940dfd87e4e1445e3c9c45a59fe7e04.png), as asked for in the problem, will be somewhat more difficult. One place to start is to write the mass balance on the evaporator, since that will certainly contain the unknown we seek. Assuming that the process is steady state we can write:

![ In - Out = 0 ](../../../upload.wikimedia.org/math/b/7/9/b7919f399d989e541237c55823ef5c4d.png)

![ \\dot{m}_1 - \\dot{m}_2 - \\dot{m}_4 - \\dot{m}_6 = 0 ](../../../upload.wikimedia.org/math/4/6/b/46bd52a50bc0866be596c5cce37b210c.png)

Problem: we don't know ![ \\dot{m}_6 ](../../../upload.wikimedia.org/math/9/6/c/96c90b61c38fc281b562cc5048b98ca1.png) so with only this equation we cannot solve for ![ \\dot{m}_4 ](../../../upload.wikimedia.org/math/6/7/d/67d4cbe9bedb850068447e075ca92d15.png). Have no fear, however, because there is another way to figure out what ![ \\dot{m}_6 ](../../../upload.wikimedia.org/math/9/6/c/96c90b61c38fc281b562cc5048b98ca1.png) is... can you figure it out? Try to do so before you move on.

### So you want to check your guess? Alright then read on.

The way to find ![ \\dot{m}_6 ](../../../upload.wikimedia.org/math/9/6/c/96c90b61c38fc281b562cc5048b98ca1.png) is to do a mass balance on the heat exchanger, because the mass balance for the heat exchanger is simply:

![ \\dot{m}_6 - \\dot{m}_5 = 0 ](../../../upload.wikimedia.org/math/2/4/6/2460f7c213af2204b25fbeb79ef136b0.png)

Since we know ![ \\dot{m}_5 ](../../../upload.wikimedia.org/math/6/c/6/6c6dd7f51c28863e2f4db62e4819a18e.png) we can calculate ![ \\dot{m}_6 ](../../../upload.wikimedia.org/math/9/6/c/96c90b61c38fc281b562cc5048b98ca1.png) and thus the waste stream flowrate ![\\dot{m}_4 ](../../../upload.wikimedia.org/math/6/7/d/67d4cbe9bedb850068447e075ca92d15.png).

**Note:**  
Notice the strategy here: we first start with a balance on the operation containing the stream we need information about. Then we move to balances on other operations in order to garner additional information about the unknowns in the process. This takes **practice** to figure out when you have enough information to solve the problem or you need to do more balances or look up information.

It is also of note that **any process has a limited number of independent balances you can perform**. This is not as much of an issue with a relatively simple problem like this, but will become an issue with more complex problems. Therefore, a step-by-step method exists to tell you exactly how many independent mass balances you can write on any given process, and therefore how many total independent equations you can use to help you solve problems.

### Step 4: Calculate your unknowns.

Carrying out the plan on this problem:

![ \\dot{m}_6 - 0.1893\\frac{kg}{s} = 0 ](../../../upload.wikimedia.org/math/7/2/8/728645f6d584832716154f8799c6076c.png)

![ \\dot{m}_6 = 0.1893 \\frac{kg}{s} ](../../../upload.wikimedia.org/math/6/d/1/6d12a73b5f023a12b8ca2d42d1645f72.png)

Hence, from the mass balance on the evaporator:

![ \\dot{m}_4 = \\dot{m}_1 - \\dot{m}_2 - \\dot{m}_6 = \(0.5 - 0.2666 - 0.1893\)\\frac{kg}{s} = 0.0441 \\frac{kg}{s} ](../../../upload.wikimedia.org/math/a/f/9/af9cd4ea032aacb989389cc7eea13dfc.png)

So the final answers are:

  


![\\mbox{Evaporator Efficiency} = 53.3% ](../../../upload.wikimedia.org/math/1/9/5/19596ccc668bbe562c8dcb972e14c51e.png)

![ \\mbox{Waste stream rate} = 0.0441\\frac{kg}{s} ](../../../upload.wikimedia.org/math/c/0/7/c07a5f6636f027d941aec6c9475871bd.png)

### Step 5: Check your work.

Ask: Do these answers make sense? Check for things like negative flow rates, efficiencies higher than 100%, or other physically impossible happenings; if something like this happens (and it will), you did something wrong. Is your exit rate higher than the total inlet rate (since no water is created in the processes, it is impossible for this to occur)?

In this case, the values make physical sense, so they may be right. It's always good to go back and check the math _and_ the setup to make sure you didn't forget to convert any units or anything like that.

## Chapter 2 Practice Problems

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Problem:**  


**1.** _a)_ A salt solution is to be concentrated by evaporating the water in a salt pan, with a condensing surface above it to gather the evaporated water. Suppose 1200g of salt solution are emptied into the pan. Once all the water is evaporated, the salt is weighed and found to weigh 100g. What percent of the original solution was water?

_b)_ Now suppose that 0.1 L of the evaporated water was added back to the salt, to bring it to the desired concentration. How much water remains to be used elsewhere?

_c)_ Do you think the salt solution would be safe to drink? Why or why not?

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Problem:**  


**2.** _a)_ In a stone quarry, limestone is to be crushed and poured into molds for manufacture of floor tiles. Suppose that a limestone company uses three trucks, each of which is capable of carrying 3000 kg of limestone. The quarry itself is 20 miles away from the processing plant, and the trucks get there at an average speed of 30 miles/hour. Once at the plant, the limestone is ground into fine powder and then melted and poured into the molds. If each of the resulting slabs weighs 2 kg and the plant operates 24 hours a day, how many slabs can the company make in a day?

_b)_ How could this plant become more efficient? Plot the number of slabs the company can make as a function of distance from the quarry and capacity of the trucks. What factors might keep the company from simply moving as close to the quarry as possible and using the largest trucks possible?

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Problem:**  


**3.** What is the volumetric flowrate of a solution with density 1.5 kg/m^3 flowing at a velocity of 5 m/s and a mass flow rate of 500 g/min? What is the area of the pipe? If it is circular, what is the radius?

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Problem:**  


**4.** Suppose you have a pipe that constricts halfway through from a radius of 0.5 cm to a radius of 0.2 cm. A liquid approaches the constriction at a velocity of 0.5 m/s. What is the velocity of the fluid after the constriction? (Hint: Apply conservation of mass on both sides of the constriction).

**Challenge:** What kind of energy does the fluid gain? Energy is never created or destroyed, so where does it come from?

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Problem:**  


**5.** Suppose that a river with a molar flow rate of 10000 mol/s meets another, larger river flowing at 500000 m^3/s at room temperature. What is the mass flow rate of the river downstream of the intersection if you assume steady state?

_b}_ Evaluate the feasibility of the steady state assumption in this situation. Also qualitatively evaluate the probability that the flowrates are actually constant.

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Problem:**  


**6.** Suppose that the population of a certain country n years after year 2000 if there is no emigration can be modeled with the equation:

![ P = 2.5*10^8*e^{0.045*n}](../../../upload.wikimedia.org/math/1/c/b/1cb1cb77b9ad0cabc370d54f17fa9b02.png)

Also, suppose that in the country, a net emigration of 100,000 people per month actually occurs. What is the total accumulation of people in this country from year 2000 to 2003?

_b)_ What was the population of people in 2002, according to this model?

_c)_ What are some possible problems with this model? For example, what doesn't it take into account? What happens when n is 100? Where did those constants come from? Would they be the same for every country, or for the same country across generations?

[Solutions](http://en.wikibooks.org/w/index.php?title=Introduction_to_Chemical_Engineering_Processes/Print_Version/Solutions&action=edit&redlink=1)

# **Chapter 3: Mass balances on multicomponent systems**

## Component Mass Balance

Most processes, of course, involve more than one input and/or output, and therefore it must be learned how to perform mass balances on . The basic idea remains the same though. We can write a mass balance in the same form as the overall balance for each component:

![ In - Out + Generation = Accumulation ](../../../upload.wikimedia.org/math/0/b/1/0b1844235a31438006e40205686b0ce8.png)

For **steady state** processes, this becomes:

![ In - Out + generation = 0
 ](../../../upload.wikimedia.org/math/7/4/f/74f39024bc84e702fa7d77adff6f1d04.png)

The **overall** mass balance at steady state, recall, is:

![ \\Sigma\\dot{m}_{in} - \\Sigma\\dot{m}_{out} + m_{gen} = 0 ](../../../upload.wikimedia.org/math/2/4/9/2493cbb92877a2eb3b22f42c55433773.png)

The mass of each component can be described by a similar balance.

![ \\Sigma\\dot{m}_{A,in} - \\Sigma\\dot{m}_{A,out} + {m}_{A,gen} = 0 ](../../../upload.wikimedia.org/math/5/0/3/50311de5efb2ce9c7dd156a2cfb0af24.png)

The biggest difference between these two equations is that **The total generation of mass ![ m_{gen} ](../../../upload.wikimedia.org/math/c/0/0/c00f745d81c72755bf600137c3ea1b65.png) is zero due to conservation of mass, but since individual species can be consumed in a reaction, ![ {m}_{A,gen} \\neq 0 ](../../../upload.wikimedia.org/math/a/f/8/af820a379922b41c8dc54b6c0cc57194.png) for a reacting system**

## Concentration Measurements

You may recall from general chemistry that a **concentration** is a measure of the amount of some species in a mixture relative to the total amount of material, or relative to the amount of another species. Several different measurements of concentration come up over and over, so they were given special names.

  


### Molarity

The first major concentration unit is the **molarity** which relates the moles of one particular species to the total volume of the solution.

![ Molarity\(A\)= \[A\] = \\frac{n_A}{V_{sln}} ](../../../upload.wikimedia.org/math/6/3/7/637a531fda7bd1b09f1b190beb6592c7.png) where ![ n \\dot= mol, V \\dot= L](../../../upload.wikimedia.org/math/7/8/3/78375f3479d1b0f927dc53d09166e571.png)

A more useful definition for flow systems that is equally valid is:

  


![ \[A\] = \\frac{\\dot{n}_A}{\\dot{V}_{n}} ](../../../upload.wikimedia.org/math/6/1/0/610faaf58472c70d1ee2f61c5877b625.png) where ![ \\dot{n}_A \\dot= mol/s, \\dot{V}_{n} \\dot= L/s ](../../../upload.wikimedia.org/math/3/4/7/347d08695ace9b908026db88f1638406.png)

Molarity is a useful measure of concentration because it takes into account the volumetric changes that can occur when one creates a mixture from pure substances. Thus it is a very practical unit of concentration. However, since it involves volume, it can change with temperature so _molarity should always be given at a specific temperature_. Molarity of a gaseous mixture can also change with pressure, so it is not usually used for gasses.

### Mole Fraction

The **mole fraction** is one of the most useful units of concentration, since it allows one to directly determine the molar flow rate of any component from the total flow rate. It also conveniently is _always_ between 0 and 1, which is a good check on your work as well as an additional equation that you can always use to help you solve problems.

The mole fraction of a component A in a mixture is defined as:

![ x_A = \\frac{n_A}{n_{n}} ](../../../upload.wikimedia.org/math/7/a/f/7af1d1734f6d9ef81241b54a61a49145.png)

where ![ n_A ](../../../upload.wikimedia.org/math/1/8/5/185c04daf95b1832a7f47d6a983971ce.png) signifies moles of A. Like molarity, a definition in terms of flowrates is also possible:

  
**Mole Fraction Definition**  
![ x_A = \\frac{\\dot{n}_A}{\\dot{n}_{n}} ](../../../upload.wikimedia.org/math/b/c/7/bc7f3933272686dea4ec001c9b3b6465.png)

If you add up all mole fractions in a mixture, you should always obtain 1 (within calculation and measurement error), because sum of individual component flow rates equals the total flow rate:

  


![ \\Sigma x_i = 1 ](../../../upload.wikimedia.org/math/f/2/2/f22d03b6b6897e2f8352da9094d69b1c.png)

Note that **each stream has its own independent set of concentrations**. This fact will become important when you are performing mass balances.

### Mass Fraction

Since mass is a more practical property to measure than moles, flowrates are often given as _mass_ flowrates rather than _molar_ flowrates. When this occurs, it is convenient to express concentrations in terms of **mass fractions** defined similarly to mole fractions.

In most texts mass fraction is given the same notation as mole fraction, and which one is meant is explicitly stated in the equations that are used or the data given.

**Note:**  
In this book, **assume that a percent concentration has the same units as the total flowrate unless stated otherwise**. So if a flowrate is given in kg/s, and a composition is given as "30%", assume that it is 30% by mass.

The definition of a mass fraction is similar to that of moles:

![ x_A = \\frac{m_A}{m_{n}} ](../../../upload.wikimedia.org/math/6/7/b/67b0b947b087a5665c371d99f621fe72.png) for batch systems

  
**Mass fraction of Continuous Systems**  
![ x_A = \\frac{\\dot{m}_A}{\\dot{m}_{n}} ](../../../upload.wikimedia.org/math/c/1/e/c1e35f30db048c1eb6779aaa14b67f8b.png)

where ![ m_A ](../../../upload.wikimedia.org/math/a/0/0/a0077a7da97c15e51f01e7d6a71c5652.png) is the mass of A. It doesn't matter what the units of the mass are as long as they are the same as the units of the total mass of solution.

Like the mole fraction, the total mass fraction in any stream should always add up to 1.

  


![ \\Sigma x_i = 1 ](../../../upload.wikimedia.org/math/f/2/2/f22d03b6b6897e2f8352da9094d69b1c.png)

## Calculations on Multi-component streams

Various conversions must be done with multiple-component streams just as they must for single-component streams. This section shows some methods to combine the properties of single-component streams into something usable for multiple-component streams(with some assumptions).

### Average Molecular Weight

The _average molecular weight_ of a mixture (gas or liquid) is the multicomponent equivalent to the molecular weight of a pure species. It allows you to convert between the mass of a mixture and the number of moles, which is important for reacting systems especially because balances must usually be done in moles, but measurements are generally in grams.

To find the value of ![ \\bar{MW}_n = \\frac{g\\mbox{ sln}}{mole\\mbox{ sln}} ](../../../upload.wikimedia.org/math/c/c/5/cc5e530a33d292f6d68ca20668aa98e8.png), we split the solution up into its components as follows, for k components:

![ \\frac{g\\mbox{ sln}}{mole\\mbox{ sln}} = \\frac{\\Sigma{m_i}}{n_n} = \\Sigma \\frac{m_i}{n_n} ](../../../upload.wikimedia.org/math/3/d/c/3dc8881b0f0a75c4f2a3f76fd6a26e15.png)

![ = \\Sigma\(\\frac{m_i}{n_i}*\\frac{n_i}{n_n}\) = \\Sigma\(MW_i*x_i\)](../../../upload.wikimedia.org/math/6/b/4/6b4f2cf9f313d94bf52de7523a355f21.png)

where ![ x_i ](../../../upload.wikimedia.org/math/0/5/e/05e42209d67fe1eb15a055e9d3b3770e.png) is the **mole fraction** of component i in the mixture. Therefore, we have the following formula:

  


![ \\bar{MW}_n = \\Sigma\(MW_i*x_i\)_n ](../../../upload.wikimedia.org/math/2/f/2/2f2dbc9da1fc21037736dd41a1defcb8.png)  
where ![ x_i ](../../../upload.wikimedia.org/math/0/5/e/05e42209d67fe1eb15a055e9d3b3770e.png) is the mole fraction of component i in the mixture.

This derivation only assumes that **mass is additive**, which it is, so this equation is valid for _any_ mixture.

### Density of Liquid Mixtures

Let us attempt to calculate the density of a liquid mixture from the density of its components, similar to how we calculated the average molecular weight. This time, however, we will notice one critical difference in the assumptions we have to make. We'll also notice that there are **two** different equations we could come up with, depending on the assumptions we make.

#### First Equation

By definition, the density of a single component i is: ![ {\\rho}_i = \\frac{m_i}{V_i} ](../../../upload.wikimedia.org/math/5/9/d/59d3b47751eefcc462f1de7d29c32b5c.png) The corresponding definition for a solution is ![ \\rho = \\frac{m\\mbox{ sln}}{V\\mbox{ sln}} ](../../../upload.wikimedia.org/math/6/4/5/6458d7a73e571351b30aab6955a9e1a6.png). Following a similar derivation to the above for average molecular weight:

![ \\frac{m\\mbox{ sln}}{V\\mbox{ sln}} = \\frac{\\Sigma{m_i}}{V_{n}} = \\Sigma \\frac{m_i}{V_{n}} ](../../../upload.wikimedia.org/math/2/e/6/2e656995d9e64d287a3c906e9265dbf7.png)

![ = \\Sigma \\frac{m_i}{V_i} * \\frac{V_i}{V_n}  = \\Sigma \({\\rho}_i * \\frac{V_i}{V_n}\) ](../../../upload.wikimedia.org/math/c/2/6/c269395db7a74bc555bc9de62bc45431.png)

Now we make the assumption that **The volume of the solution is proportional to the mass**. This is true for any pure substance (the proportionality constant is the density), but it is further assumed that **the proportionality constant is the same for both pure k and the solution**. This equation is therefore useful for two substances with similar pure densities. If this is true then:

![ \\frac{V_i}{V} = \\frac{m_i}{m_n} = x_i ](../../../upload.wikimedia.org/math/b/4/4/b44004ed50069fc3af764e1f58e0c59a.png), where ![ x_i ](../../../upload.wikimedia.org/math/0/5/e/05e42209d67fe1eb15a055e9d3b3770e.png) is the **mass fraction** of component i. Thus:

  


![ {\\rho}_{n} = \\Sigma{\(x_i*{\\rho}_i\)}_n ](../../../upload.wikimedia.org/math/6/5/4/6543911ee90887978fde6be9c0039edc.png)  
where ![ x_i ](../../../upload.wikimedia.org/math/0/5/e/05e42209d67fe1eb15a055e9d3b3770e.png) is the **mass fraction** (not the mole fraction) of component i in the mixture.

#### Second Equation

This equation is easier to derive if we assume the equation will have a form similar to that of average molar mass. Since density is given in terms of mass, it makes sense to start by using the definition of **mass fractions**:

![ x_i = \\frac{m_i}{m_{n}} ](../../../upload.wikimedia.org/math/f/e/6/fe6e6fb22015bfb542a233031b51a7ed.png)

To get this in terms of only _solution_ properties (and not _component_ properties), we need to get rid of ![ m_i ](../../../upload.wikimedia.org/math/2/1/2/212abcc00be63b38e4cd48603ee59f7d.png). We do this first by dividing by the density:

![ \\frac{x_i}{{\\rho}_i} = \\frac{m_i}{m_{n}}*\\frac{V_i}{m_i} ](../../../upload.wikimedia.org/math/0/e/6/0e6842ea112977515f6e28195e4439f1.png)

![ = \\frac{V_i}{m_{n}} ](../../../upload.wikimedia.org/math/9/8/5/9855e911edbe5ace2582ae78264ea3bf.png)

Now if we add all of these up we obtain:

![ \\Sigma\(\\frac{x_i}{{\\rho}_i}\) = \\frac{\\Sigma{V_i}}{m_{n}} ](../../../upload.wikimedia.org/math/f/f/7/ff78a0c3a12ba4f3e061708a0c06b96c.png)

Now we have to make an assumption, and it's different from that in the first case. This time we assume that the **Volumes are additive**. This is true in two cases:

1\. In an **ideal solution**. The idea of an ideal solution will be explained more later, but for now you need to know that ideal solutions:

  * Tend to involve similar compounds in solution with each other, or when one component is so dilute that it doesn't effect the solution properties much.
  * Include Ideal Gas mixtures at constant temperature and pressure.

2 In a **Completely immiscible nonreacting mixture**. In other words, if two substances don't mix at all (like oil and water, or if you throw a rock into a puddle), the total volume will not change when you mix them. And the total volume in this case will be sum of volume of individual components.

If the solution is ideal, then we can write:

![ \\frac{\\Sigma \\dot{V}_i}{\\dot{m}_{n}} = \\frac{\\dot{V}_{n}}{\\dot{m}_{n}} = \\frac{1}{{\\rho}_{n}} ](../../../upload.wikimedia.org/math/3/0/b/30be955148a5b2f952ab2d1d939ba564.png)

Hence, for an ideal solution,

  


![ \\frac{1}{{\\rho}_{n}} = \\Sigma\(\\frac{x_i}{{\\rho}_i}\)_n ](../../../upload.wikimedia.org/math/0/9/f/09f3491e2496088ecb62f05fa625d1c6.png) where ![ x_i ](../../../upload.wikimedia.org/math/0/5/e/05e42209d67fe1eb15a055e9d3b3770e.png) is the **mass fraction** of component i in the mixture.

Note that this is significantly different from the previous equation! This equation is more accurate for most cases. In all cases, however, it is most accurate to look up the value in a handbook such as Perry's Chemical Engineers Handbook if data is available on the solution of interest.

## General Strategies for Multiple-Component Operations

The most important thing to remember about doing mass balances with multiple components is that _for each component, you can write one independent mass balance._ What do I mean by independent? Well, remember we can write the general, overall mass balance for any steady-state system:

![ \\Sigma\\dot{m}_{in} - \\Sigma\\dot{m}_{out} = 0 ](../../../upload.wikimedia.org/math/f/6/a/f6ac595185dc1b450c0cc76d754f21ca.png)

And we can write a similar mass balance for any _component_ of a stream:

![ \\Sigma\\dot{m}_{a,in} - \\Sigma\\dot{m}_{a,out} + m_{a,gen} = 0 ](../../../upload.wikimedia.org/math/5/5/f/55fde21a7b6a9ee2cf77ca4956008d01.png)

This looks like we have three equations here, but in reality only two of them are independent because:

  1. The sum of the masses of the components equals the total mass
  2. The total mass generation due to reaction is always zero (by the law of mass conservation)

Therefore, if we add up all of the mass balances for the _components_ we obtain the _overall_ mass balance. Therefore, **we can choose any set of n equations we want, where n is the number of components, but if we choose the overall mass balance as one of them we cannot use the mass balance on one of the components**.

The choice of which balances to use depends on two particular criteria:

  1. Which component(s) you have the most information on; if you don't have enough information you won't be able to solve the equations you write.
  2. Which component(s) you can make the most reasonable assumptions about. For example, if you have a process involving oxygen and water at low temperatures and pressures, you may say that there is no oxygen dissolved in a liquid flow stream, so it all leaves by another path. This will simplify the algebra a good deal if you write the mass balance on that component.

## Multiple Components in a Single Operation: Separation of Ethanol and Water

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Suppose a stream containing ethanol and water (two fully miscible compounds) flows into a distillation column at 100 kg/s. Two streams leave the column: the vapor stream contains 80% ethanol by mass and the liquid bottoms has an ethanol concentration of 4M. The total liquid stream flowrate is 20 kg/s. Calculate the composition of the entrance stream.

Following the step-by-step method makes things easier.

### Step 1: Draw a Flowchart

The first step as always is to draw the flowchart, as described previously. If you do that for this system, you may end up with something like this, where x signifies mass fraction, [A] signifies molarity of A, and numbers signify stream numbers.

![EtOH H2O Flowchart.PNG](../../../upload.wikimedia.org/wikibooks/en/8/8c/EtOH_H2O_Flowchart.PNG)

### Step 2: Convert Units

Now, we need to turn to converting the concentrations into appropriate units. Since the total flowrates are given in terms of mass, a unit that expresses the concentration in terms of mass of the components would be most useful. The vapor stream compositions are given as mass percents, which works well with the units of flow. However, the liquid phase concentration given in terms of a molarity is not useful for finding a mass flow rate of ethanol (or of water). Hence we must convert the concentration to something more useful.

**Note:**  
Converting between Concentration Measurements

The easiest way to convert between concentrations is to take a careful look at the _units_ of both what you want and what you have, and ask what physical properties (i.e. molar mass, density) you could use to interchange them. In this example, we want to convert a molarity into a mass fraction. We have from the definitions that:

![ \[A\] = \\frac{mol_A}{L_{sln}} ](../../../upload.wikimedia.org/math/8/f/1/8f1c18cc968ef22b9532dd7b503824ee.png)

![ x_A = \\frac{m_A}{m_{sln}} ](../../../upload.wikimedia.org/math/1/5/1/151534a1c9992ecccaf06d8560b6ab81.png)

To convert the numerators, we need to convert moles of A to mass of A, so we can use the molar mass for this purpose. Similarly, to convert the denominators we need to change Liters to Mass, which means we'll use a density. Hence, the conversion from molarity to mass fraction is:

![ x_A = \[A\]*\\frac{\(MW\)_A}{{\\rho}_{SLN}} ](../../../upload.wikimedia.org/math/9/3/b/93b07be850daa03331b6a4b67943b409.png)

Since we have ways to estimate ![ {\\rho}_{SLN} ](../../../upload.wikimedia.org/math/f/d/2/fd20a38b4896661477db2e8fa887c821.png) (remember them?), we can inter-convert the conversions.

In order to convert the molarity into a mass fraction, then, we need the molecular weight of ethanol and the density of a 4M ethanol solution. The former is easy if you know the chemical formula of ethanol: ![ CH_3CH_2OH ](../../../upload.wikimedia.org/math/8/c/5/8c58752c9ebf3eb1e55a98a6dbc5b9e0.png). Calculating the molecular weight (as you did in chem class) you should come up with about ![ 46 \\frac{g}{mol} ](../../../upload.wikimedia.org/math/5/3/9/53921850197ecc9f334b3b6005f98082.png).

Calculating the density involves plugging in mass fractions in and of itself, so you'll end up with an implicit equation. Recall that one method of estimating a solution density is to assume that the solution is ideal (which it probably is not in this case, but if no data are available or we just want an estimate, assumptions like these are all we have, as long as we realize the values will not be exact):

![ \\frac{1}{{\\rho}_{SLN}} = \\Sigma\(\\frac{x_k}{{\\rho}_k}\) ](../../../upload.wikimedia.org/math/7/c/a/7caed918977097895317119f8447139b.png)

In this case, then,

![ \\frac{1}{{\\rho}_{SLN}} = \\frac{x_{EtOH}}{{\\rho}_{EtOH}} + \\frac{x_{H2O}}{{\\rho}_{H2O}} ](../../../upload.wikimedia.org/math/a/7/2/a7271a72a0a0bf91f9d353502f57e8de.png)

We can look up the densities of pure water and pure ethanol, they are as follows (from Wikipedia's articles [w:Ethanol](http://en.wikipedia.org/wiki/Ethanol) and [w:Water](http://en.wikipedia.org/wiki/Water)):

![ {\\rho}_{EtOH} = 0.789 \\frac{g}{cm^3} = 789 \\frac{g}{L} ](../../../upload.wikimedia.org/math/4/f/e/4fe6d38bb2032ab8333892d457bf3c19.png)

![ {\\rho}_{H2O} = 1.00 \\frac{g}{cm^3} = 1000 \\frac{g}{L} ](../../../upload.wikimedia.org/math/0/3/a/03a35dd09abb7f09d0bb3775a306061a.png)

Therefore, since the mass fractions add to one, our equation for density becomes:

![ \\frac{1}{{\\rho}_{sln}} = \\frac{x_{EtOH}}{789\\frac{g}{L}} + \\frac{1 - x_{EtOH}}{1000 \\frac{g}{L}} ](../../../upload.wikimedia.org/math/9/8/6/9862ce497a4813f042b59bb653706b7c.png)

From the NOTE above, we can now finally convert the molarity into a mass fraction as:

![ x_{EtOH} = \[EtOH\]*\\frac{\(MW\)_{EtOH}}{{\\rho}_{SLN}}

= 4\\frac{mol}{L} * 46\\frac{g}{mol}* \(\\frac{x_{EtOH}}{789\\frac{g}{L}} + \\frac{1 - x_{EtOH}}{1000 \\frac{g}{L}}\)](../../../upload.wikimedia.org/math/5/0/9/509ea80b28b620060440611d290a02f4.png)

Solving this equation yields:

![ x_{EtOH} = 0.194 ](../../../upload.wikimedia.org/math/5/a/d/5add41615f5ab91bef5b3d5663fca6ab.png) (unitless)

### Step 3: Relate your Variables

Since we are seeking properties related to mass flow rates, we will need to relate our variables with mass balances.

Remember that we can do a mass balance on any of the N independent species and one on the overall mass, but since the sum of the individual masses equals the overall only ![ N-1 ](../../../upload.wikimedia.org/math/5/b/1/5b1ba2fe7e642d05d58d3df27466b069.png) of these equations will be independent. It is often easiest mathematically to choose the overall mass balance and ![ N - 1 ](../../../upload.wikimedia.org/math/5/b/1/5b1ba2fe7e642d05d58d3df27466b069.png) individual species balances, since you don't need to deal with concentrations for the overall measurements.

Since our concentrations are now in appropriate units, we can do any two mass balances we want. Lets choose the overall first:

![ \\dot{m}_1 - \\dot{m}_2 - \\dot{m}_3 = 0 ](../../../upload.wikimedia.org/math/2/5/a/25a8c0b625a96b3578feaa2857d82ee4.png)

Plugging in known values:

![ \\dot{m}_2 = 100\\frac{kg}{s} - 20\\frac{kg}{s} ](../../../upload.wikimedia.org/math/a/7/2/a72860bc1069951fa9c24b54bb8411b7.png)

![ \\dot{m}_2 = 80\\frac{kg}{s} ](../../../upload.wikimedia.org/math/6/5/5/65513ad6d7c87b4f5ee41a9800237de4.png)

Now that we know ![ \\dot{m}_2 ](../../../upload.wikimedia.org/math/2/d/3/2d3772bdc14344289299dceaa2c138a4.png) we can do a mass balance on either ethanol or water to find the composition of the input stream. Lets choose ethanol (A):

![ \\dot{m}_{A1} = \\dot{m}_{A2} + \\dot{m}_{A3} ](../../../upload.wikimedia.org/math/4/7/7/4776934f6d57a3852f72acc6b6b0e270.png)

Written in terms of mass fractions this becomes:

![ x_{A1}*\\dot{m}_1 = x_{A2} * \\dot{m}_2 + x_{A3}*\\dot{m}_3 ](../../../upload.wikimedia.org/math/b/1/7/b17363da20a7bed31df6a70ad88fff86.png)

Plugging in what we know:

![ x_{A1}*100\\frac{kg}{s} = 0.8*80\\frac{kg}{s} + 0.194*20\\frac{kg}{s} ](../../../upload.wikimedia.org/math/2/6/9/2690e8a40fa5018a8b2fc92ec872881a.png)

![ x_{A1} = 0.68 ](../../../upload.wikimedia.org/math/3/3/8/338bdd1330ff18fe9c8cdf0c27c9c8c9.png)

Hence, **the feed is 68% Ethanol and 34% Water**.

## Introduction to Problem Solving with Multiple Components and Processes

In the vast majority of chemical processes, in which some raw materials are processed to yield a desired end product or set of end products, there will be more than one raw material entering the system and more than one unit operation through which the product must pass in order to achieve the desired result. The calculations for such processes, as you can probably guess, are considerably more complicated than those either for only a single component, or for a single-operation process. Therefore, several techniques have been developed to aid engineers in their analyses. This section describes these techniques and how to apply them to an example problem.

## Degree of Freedom Analysis

For more complex problems than the single-component or single-operation problems that have been explored, it is essential that you have a method of determining if a problem is even solvable given the information that you have. There are three ways to describe a problem in terms of its solvability:

  1. If the problem has a finite (not necessarily unique!) set of solutions then it is called **well-defined**.
  2. The problem can be **overdetermined** (also known as **overspecified**), which means that you have too much information and it is either redundant or inconsistent. This could possibly be fixed by consolidating multiple data into a single function or, in extreme cases, a single value (such as a slope of a linear correlation), or it could be fixed by removing an assumption about the system that one had made.
  3. The problem can be **underdetermined** (or **underspecified**), which means that you don't have enough information to solve for all your unknowns. There are several ways of dealing with this. The most obvious is to gather additional information, such as measuring additional temperatures, flow rates, and so on until you have a well-defined problem. Another way is to use additional equations or information about what we want out of a process, such as how much conversion you obtain in a reaction, how efficient a separation process is, and so on. Finally, we can **make assumptions** in order to simplify the equations, and perhaps they will simplify enough that they become solvable.

The method of analyzing systems to see whether they are over or under-specified, or if they are well-defined, is called a **degree of freedom analysis**. It works as follows for mass balances on a **single process**:

  1. From your flowchart, determine the number of **unknowns** in the process. What qualifies as an unknown depends on what you're looking for, but in a material balance calculation, masses and concentrations are the most common. In equilibrium and energy balance calculations, temperature and pressure also become important unknowns. In a reactor, you should include the conversion as an unknown unless it is given OR you are doing an atom balance.
  2. Subtract the number of **Equations** you can write on the process. This can include mass balances, energy balances, equilibrium relationships, relations between concentrations, and any equations derived from additional information about the process.
  3. The number you are left with is the degrees of freedom of the process.

If the degrees of freedom are **negative** that means the unit operation is **overspecified**. If it is **positive**, the operation is **underspecified**. If it is **zero** then the unit operation is **well-defined**, meaning that it is theoretically possible to solve for the unknowns with a finite set of solutions.

### Degrees of Freedom in Multiple-Process Systems

Multiple-process systems are tougher but not undoable. Here is how to analyze them to _see if a problem is uniquely solvable_:

  1. Label a flowchart completely with all the relevant unknowns.
  2. Perform a degree of freedom analysis on each unit operation, as described above.
  3. Add the degrees of freedom for each of the operations.
  4. Subtract the number of variables in _intermediate streams_, i.e. streams between two unit operations. This is because each of these was counted twice, once for the operation it leaves and once for the one it enters.

The number you are left with is the **process degrees of freedom**, and this is what will tell you if the process as a _whole_ is overspecified, underspecified, or well-defined.

**Note:**  
If any single process is overspecified, and is found to be inconsistent, then the problem as a whole cannot be solved, regardless of whether the process as a whole is well-defined or not.

## Using Degrees of Freedom to Make a Plan

Once you have determined that your problem is solvable, you still need to figure out how you'll solve for your variables. This is the suggested method.

  1. Find a unit operation or combination of unit operations for which the degrees of freedom are zero.
  2. Calculate all of the unknowns involved in this combination.
  3. **Recalculate** the degrees of freedom for each process, treating the calculated values as known rather than as variables.
  4. Repeat these steps until everything is calculated (or at least that which you seek)

**Note:**  
You must be careful when recalculating the degrees of freedom in a process. You have to be aware of the **sandwich effect**, in which calculations from one unit operation can trivialize balances on another operation. For example, suppose you have three processes lined up like this:

-> A -> B -> C ->

Suppose also that through mass balances on operations A and C, you calculate the exit composition of A and the inlet composition of C. Once these are performed, _the mass balances on B are already completely defined_. The moral of the story is that **before you claim that you can write an equation to solve an unknown, write the equation and make sure that it contains an unknown**. Do not count equations that have no unknowns in your degree of freedom analysis.

## Multiple Components and Multiple Processes: Orange Juice Production

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Consider a process in which raw oranges are processed into orange juice. A possible process description follows:

  1. The oranges enter a crusher, in which all of the water contained within the oranges is released.
  2. The now-crushed oranges enter a strainer. The strainer is able to capture 90% of the solids, the remainder exit with the orange juice as pulp.

The velocity of the orange juice stream was measured to be ![ 30 \\frac{m}{s} ](../../../upload.wikimedia.org/math/4/f/b/4fb57b04f4aa742419a97cc24cae62f6.png) and the radius of the piping was 8 inches. Calculate:

a) The mass flow rate of the orange juice product. b) The number of oranges per year that can be processed with this process if it is run 8 hours a day and 360 days a year. Ignore changes due to unsteady state at startup.

Use the following data: Mass of an orange: 0.4 kg Water content of an orange: 80% Density of the solids: Since its mostly sugars, its about the density of glucose, which is ![ 1.540 \\frac{g}{cm^3} ](../../../upload.wikimedia.org/math/9/5/6/956f9f751a7d96827adeb9221eb7b488.png)

### Step 1: Draw a Flowchart

This time we have multiple processes so it's especially important to label each one as its given in the problem.

![OrangeJuiceFlowchart.PNG](../../../upload.wikimedia.org/wikibooks/en/f/fd/OrangeJuiceFlowchart.PNG)

Notice how I changed the 90% capture of solids into an algebraic equation relating the mass of solids in the solid waste to the mass in the feed. This will be important later, because _it is an additional piece of information that is necessary to solve the problem_.

Also note that from here in, "solids" are referred to as S and "water" as W.

### Step 2: Degree of Freedom analysis

Recall that for each stream there are C independent unknowns, where C is the number of components in the individual stream. These generally are **concentrations** of C-1 species and the **total mass flow rate**, since with C-1 concentrations we can find the last one, but we cannot obtain the total mass flow rate from only concentration.

Let us apply the previously described algorithm to determining if the problem is well-defined.

**On the strainer**:

  * There are **6** unknowns: m2, xS3, m3, xS3, m4, and xS4
  * We can write **2** independent mass balances on the overall system (one for each component).
  * We are given a conversion and enough information to write the mass flow rate in the product in terms of only concentration of one component (which eliminates one unknown). Thus we have **2** additional pieces of information.
  * Thus the degrees of freedom of the strainer are 6-2-2 = **2 DOF**

**Note:**  
We are given the mass of an individual orange, but since we cannot use that information alone to find a total mass flow rate of oranges in the feed, and we already have used up our allotment of C-1 independent concentrations, we cannot count this as "given information". If, however, we were told the number of oranges produced per year, then we could use the two pieces of information in tandem to eliminate a single unknown (because then we can find the mass flow rate)

**On the crusher**:

  * There are **3** unknowns (m1, m2, and xS2).
  * We can write **2** independent mass balances.
  * Thus the crusher has 3-2 = **1 DOF**

Therefore for the **system as a whole**:

  * Sum of DOF for unit operations = 2 + 1 = 3 DOF
  * Number of intermediate variables = 2 (m2 and xS2)
  * Total DOF = 3 - 2 = **1 DOF**.

Hence the problem is **underspecified**.

### So how to we solve it?

In order to solve an underspecified problem, one way we can obtain an additional specification is to **make an assumption**. What assumptions could we make that would reduce the number of unknowns (or equivalently, increase the number of variables we do know)?

**The most common type of assumption is to assume that something that is relatively insignificant is zero**.

In this case, one could ask: will the solid stream from the strainer contain any water? It might, of course, but this amount is probably very small compared to both the amount of solids that are captured and how much is strained, provided that it is cleaned regularly and designed well. If we make this assumption, then _this specifies that the mass fraction of water in the waste stream is zero_ (or equivalently, that the mass fraction of solids is one). Therefore, we know one additional piece of information and the degrees of freedom for the overall system become zero.

### Step 3: Convert Units

This step should be done _after_ the degree of freedom analysis, because that analysis is independent of your unit system, and if you don't have enough information to solve a problem (or worse, you have too much), you shouldn't waste time converting units and should instead spend your time defining the problem more precisely and/or seeking out appropriate assumptions to make.

Here, the most sensible choice is either to convert everything to the cgs system or to the m-kg-s system, since most values are already in metric. Here, the latter route is taken.

![ r_4 = 8\\mbox{ in} * \\frac{2.54 \\mbox{ cm}}{in} * \\frac{1 \\mbox{ m}}{100 \\mbox{ cm}} = 0.2032 \\mbox{ m}](../../../upload.wikimedia.org/math/d/9/0/d9049b56412a6aca6d503ab440635e3b.png)

![ {\\rho}_S = 1.54\\frac{g}{cm^3} = 1540 \\frac{kg}{m^3} ](../../../upload.wikimedia.org/math/9/2/a/92a4a7ecb1d755204a9d7dfd7f246eca.png)

Now that everything is in the same system, we can move on to the next step.

### Step 4: Relate your variables

First we have to relate the velocity and area given to us to the mass flowrate of stream 4, so that we can actually use that information in a mass balance. From chapter 2, we can start with the equation:

![ {\\rho}_n*v_n*A_n = \\dot{m}_n ](../../../upload.wikimedia.org/math/f/d/3/fd3297ec3631f1ac189548b9a8194074.png)

Since the pipe is circular and the area of a circle is ![ \\pi *r^2 ](../../../upload.wikimedia.org/math/5/f/d/5fd102462a51456344fe935ad08ffa59.png), we have:

![ A_4 = \\pi * 0.2032^2 = 0.1297 \\mbox{ m}^2 ](../../../upload.wikimedia.org/math/3/4/c/34c84af1fd0c9c59f5d9b6cd22884129.png)

So we have that:

![ {\\rho}_4 * 30 * 0.1297 = 3.8915*{\\rho}_4 = \\dot{m}_4 ](../../../upload.wikimedia.org/math/7/f/1/7f1ac460528950fbe63939c5d284af7a.png)

Now to find the density of stream 4 we assume that volumes are additive, since the solids and water are essentially immiscible (does an orange dissolve when you wash it?). Hence we can use the ideal-fluid model for density:

![ \\frac{1}{\\rho_4} = \\frac{x_{S4}}{\\rho_S} + \\frac{x_{W4}}{\\rho_W} = \\frac{x_{S4}}{\\rho_S} + \\frac{1-x_{S4}}{\\rho_W }](../../../upload.wikimedia.org/math/5/3/4/5343db480fa85efee77d421bff3feec8.png)

![ = \\frac{x_{S4}}{1540} + \\frac{1-x_{S4}}{1000} ](../../../upload.wikimedia.org/math/7/2/3/7239a1f7ac8bb9530bb0d34250ec552d.png)

Hence, we have the equation we need with only concentrations and mass flowrates:

  
EQUATION 1: ![\\frac{x_{S4}}{1540} + \\frac{1-x_{S4}}{1000}=\\frac{3.8915}{\\dot{m}_4} ](../../../upload.wikimedia.org/math/d/9/3/d93441b4b333a0fb5bfca560604bbcc9.png)

Now we have an equation but we haven't used either of our two (why two?) independent mass balances yet. We of course have a choice on which two to use.

In this particular problem, since we are directly given information concerning the amount of _solid_ in stream 4 (the product stream), it seems to make more sense to do the balance on this component. Since we don't have information on stream 2, and finding it would be pointless in this case (all parts of it are the same as those of stream 1), lets do an overall-system balance on the solids:

![ \\Sigma \\dot{m}_{S,in}  - \\Sigma \\dot{m}_{S,out} = 0 ](../../../upload.wikimedia.org/math/2/e/9/2e9da680d538c27d751590ad874fab7f.png)

**Note:**  
Since there is no reaction, the generation term is 0 even for individual-species balances.

Expanding the mass balance in terms of mass fractions gives:

![ \\dot{m}_1*x_{S1} = \\dot{m}_3*x_{S3} + \\dot{m}_4*x_{S4} ](../../../upload.wikimedia.org/math/3/7/e/37e79022fb15d4df7c8cbddc0c194e7d.png)

Plugging in the known values, **with the assumption that stream 3 is pure solids (no water)** and hence ![ x_{S3} = 1 ](../../../upload.wikimedia.org/math/4/8/b/48b0c3a1f3de2e29b911ea29df8e1ef4.png):

  
EQUATION 2: ![ 0.2*\\dot{m}_1 = \(0.9*0.2*\\dot{m}_1\) * 1 + x_{S4}*\\dot{m}_4 ](../../../upload.wikimedia.org/math/7/d/b/7dba17fd6e815fcefd17847b4ce53f62.png)

Finally, we can utilize one further mass balance, so let's use the easiest one: the overall mass balance. This one again assumes that the total flowrate of stream 3 is equal to the solids flowrate.

  
EQUATION 3: ![ \\dot{m}_1 = 0.9*0.2*\\dot{m}_1 + \\dot{m}_4 ](../../../upload.wikimedia.org/math/d/f/6/df6a15e8b435c4b69fc6d793b0432443.png)

We now have three equations in three unknowns ![ \(\\dot{m}_1, \\dot{m}_4, x_{S4}\) ](../../../upload.wikimedia.org/math/4/c/4/4c4f6a4f3beb6687ba47c302c48e760d.png) so the problem is solvable. This is where all those system-solving skills will come in handy.

If you don't like solving by hand, there are numerous computer programs out there to help you solve equations like this, such as [MATLAB](http://en.wikibooks.org/wiki/MATLAB), POLYMATH, and many others. You'll probably want to learn how to use the one your school prefers eventually so why not now?

Using either method, the results are:

  
![ \\dot{m}_1 = 4786\\frac{kg}{s} ](../../../upload.wikimedia.org/math/0/0/5/0055824f919eebdf7c8638ba7d3e0f44.png)

![ \\dot{m}_4 = 3925.07 ](../../../upload.wikimedia.org/math/b/b/7/bb7729815db66c997ad4299eaab09c0f.png)

![ x_{S4} = 0.0244 ](../../../upload.wikimedia.org/math/c/a/8/ca8ef52a05c222654b9797830c7f7bb2.png)

We're almost done here, now we just have to calculate the number of oranges per year.

![ 4786\\frac{kg}{s} * 1\\frac{orange}{0.4\\mbox{ kg}} * 3600 \\frac{s}{hr} * 8 \\frac{wk\\mbox{ hr}}{day}* 360\\frac{\\mbox{wk day}}{year}](../../../upload.wikimedia.org/math/d/c/f/dcff42b11f1ac5dedd50f5f493743b05.png)

  
Yearly Production: ![1.24*10^{11} \\frac{oranges}{year}](../../../upload.wikimedia.org/math/5/b/4/5b4385fe5b0b94cadbcc9bf8042ccee9.png)

## Chapter 3 Practice Problems

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Problem:**  


**1.** _a)_ Look up the composition of air. Estimate its average molecular weight.

_b)_ Qualitatively describe whether the density of air should be large or small compared to the density of water.

_c)_ Qualitatively describe whether the mass density of air should be large or small compared to that of oxygen if the same number of _moles_ of the two gasses are contained in identical containers.

_d)_ If the density of air under certain conditions is 1.06 g/m^3, how much does a gallon of air weigh?

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Problem:**  


**2.** _a)_ Using both of the formulas for average density, calculate estimates for the density of a 50% by mass solution of toluene and benzene. Comment on the results.

_b)_ Repeat this calculation for varying concentrations of toluene. When does it make the most difference which formula you use? When does it make the least? Show the results graphically. Would the trend be the same for any binary solution?

_c)_ Suppose that a 50% mixture of toluene and benzene is to be separated by crystallization. The solution is cooled until one of the components completely freezes and only the other is left as a liquid. The liquid is then removed. What will the majority of the solid be? What will the liquid be? What temperature should be used to achieve this? (give an estimate)

_d)_ In the crystallization process in part c, suppose that the after separation, the solid crystals contained all of the benzene and 1% of the toluene from the original mixture. Suppose also that after melting the solid, the resulting liquid weighed 1435 g. Calculate the mass of the original solution.

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Problem:**  


**3.**. Consider a publishing company in which books are to be bound, printed, and shipped. At 5 a.m. every morning, a shipment of 10,000 reams of paper comes in, as well as enough materials to make 150,000 books, and 30000 pounds of ink. In this particular plant, the average size of a book is 250 pages and each uses about 0.2 pounds of ink.

_a)_ How many books can be printed for each shipment? (Hint: What is the limiting factor?)

_b)_ Suppose that, on average, 4% of all books printed are misprints and must be destroyed. The remaining books are to be distributed to each of 6 continents in the following proportions:
    
    
    North America  15%
    South America  10%
    Europe         20%
    Africa         20%
    Asia           25%
    Australia      10%
    

Each book that is printed (including those that are destroyed) costs the company US$0.50 to print. Those that are shipped cost the following prices to ship from the US:
    
    
    North America  $0.05
    South America  $0.08
    Europe         $0.10
    Africa         $0.20
    Asia           $0.12
    Australia      $0.15
    

If each book sells for an equivalent of US$1.00, what is the maximum profit that the company can make per day?

_c)_ **Challenge** What is the _minimum_ number of books that the company can sell (from any continent) in order to return a profit? (Hint: what is the total cost of this scheme? Does it matter where the books are sold once they are distributed?)

_d)_ How many pounds of ink per day end up in each continent under the scheme in part b? How many pages of paper?

_e)_ Can you think of any ways you can improve this process? What may be some ways to improve the profit margin? How can inventory be reduced? What are some possible problems with your proposed solutions?

[Solutions](http://en.wikibooks.org/w/index.php?title=Introduction_to_Chemical_Engineering_Processes/Print_Version/Solutions&action=edit&redlink=1)

# **Chapter 4: Mass balances with recycle**

## What Is Recycle?

Recycling is the act of taking one stream in a process and reusing it in an earlier part of the process rather than discarding it. It is used in a wide variety of processes.

### Uses and Benefit of Recycle

The use of recycle makes a great deal of environmental _and_ economic sense, for the following reasons among others:

  * Using recycle lets a company achieve a wider range of separations

This will be demonstrated in the next section. However, there is a tradeoff: the more dilute or concentrated you want your product to be, the lower the flowrate you can achieve in the concentrated or dilute stream.

  * By using recycle, in combination with some sort of separation process, a company can increase the overall conversion of an **equilibrium reaction**.

You may recall from general chemistry that many reactions do not go to completion but only up to a certain point, because they are _reversible_. How far the reaction goes depends on the concentrations (or partial pressures for a gas) of the products and the reactants, which are related by the reaction stoichiometry and the equilibrium constant K. If we want to increase the amount of conversion, one way we can do this is to separate out the products from the product mixture and re-feed the purified reactants in to the reactor. By Le Chatlier's Principle, this will cause the reaction to continue moving towards the products.

  * By using recycle, it is possible to recover expensive **catalysts and reagents**.

Catalysts aren't cheap, and if we don't try to recycle them into the reactor, they may be lost in the product stream. This not only gives us a contaminated product but also wastes a lot of catalyst.

  * Because of the previous three uses, recycle can decrease the amount of equipment needed to get a process meet specifications and consumer demand.

For example, it may improve reaction conversion enough to eliminate the need for a second reactor to achieve an economical conversion.

  * Recycle reduces the amount of waste that a company generates.

Not only is this the most environmentally sounds way to go about it, it also saves the company money in disposal costs.

  * Most importantly, all of these things can save a company money.

By using less equipment, the company saves maintainence as well as capital costs, and probably gets the product faster too, if the proper analysis is made.it is use to stop the wastage of the material

## Differences between Recycle and non-Recycle systems

The biggest difference between recycle and non-recycle systems is that _the extra splitting and recombination points must be taken into account, and the properties of the streams change from before to after these points_. To see what is meant by this, consider any arbitrary process in which a change occurs between two streams:
    
    
    Feed -> Process -> Outlet
    

If we wish to implement a recycle system on this process, we often will do something like this:

![Recycle Schematic.PNG](../../../upload.wikimedia.org/wikibooks/en/9/97/Recycle_Schematic.PNG)

The "extra" stream between the splitting and recombination point must be taken into account, but the way to do this is _not_ to do a mass balance on the process, since _the recycle stream itself does not go into the process, only the recombined stream does_.

Instead, we take it into account by performing a mass balance on the _recombination point_ and one on the _splitting point_.

### Assumptions at the Splitting Point

The recombination point is relatively unpredictable because the composition of the stream leaving depends on both the composition of the feed and the composition of the recycle stream. However, the _spliitng point_ is special because **when a stream is split, it generally is split into two streams with equal composition**. This is a piece of information that counts towards "additional information" when performing a degree of freedom analysis.

As an additional specification, it is common to know the _ratio_ of splitting, i.e. how much of the exit stream from the process will be put into the outlet and how much will be recycled. This also counts as "additional information".

### Assumptions at the Recombination Point

The recombination point is generally not specified like the splitting point, and also the recycle stream and feed stream are very likely to have different compositions. The important thing to remember is that you can generally use the properties of the stream coming from the splitting point for the stream entering the recombination point, unless it goes through another process in between (which is entirely possible).

## Degree of Freedom Analysis of Recycle Systems

Degree of freedom analyses are similar for recycle systems to those for other systems, but with a couple important points that the engineer must keep in mind:

  1. The recombination point and the splitting point must be counted in the degree of freedom analysis as "processes", since they can have unknowns that aren't counted anywhere else.
  2. When doing the degree of freedom analysis on the splitting point, _you should not label the concentrations as the same but leave them as separate unknowns until after you complete the DOF analysis_ in order to avoid confusion, since labeling the concentrations as identical "uses up" one of your pieces of information and then you can't count it.

As an example, let's do a degree of freedom analysis on the hypothetical system above, assuming that all streams have two components.

  * **Recombination Point**: 6 variables (3 concentrations and 3 total flow rates) - 2 mass balances = _4_ DOF
  * **Process**: Assuming it's not a reactor and there's only 2 streams, there's 4 variables and 2 mass balances = _2_ DOF
  * **Splitting Point**: 6 variables - 2 mass balances **\- 1 knowing compositions are the same - 1 splitting ratio = 2 DOF**

So the total is 4 + 2 + 2 - 6 (in-between variables) = **2 DOF**. Therefore, if the feed is specified then this entire system can be solved! Of course the results will be different if the process has more than 2 streams, if the splitting is 3-way, if there are more than two components, and so on.

## Suggested Solving Method

The solving method for recycle systems is similar to those of other systems we have seen so far but as you've likely noticed, they are increasingly complicated. Therefore, the importance of **making a plan** becomes of the utmost importance. The way to make a plan is generally as follows:

  1. Draw a completely labeled flow chart for the process.
  2. Do a DOF analysis to make sure the problem is solvable.
  3. If it is solvable, **a lot of the time, the best place to start with a recycle system is with a set of overall system balances, sometimes in combination with balances on processes on the border**. The reason for this is that the overall system balance cuts out the recycle stream entirely, since the recycle stream does not enter or leave the system as a whole but merely travels between two processes, like any other intermediate stream. Often, the composition of the recycle stream is unknown, so this simplifies the calculations a good deal.
  4. Find a set of independent equations that will yield values for a certain set of unknowns (this is often most difficult the first time; sometimes, one of the unit operations in the system will have 0 DOF so start with that one. Otherwise it'll take some searching.)
  5. Considering those variables as known, do a new DOF balance until something has 0 DOF. Calculate the variables on that process.
  6. Repeat until all processes are specified completely.

## Example problem: Improving a Separation Process

It has been stated that recycle can help to This example helps to show that this is true and also show some limitations of the use of recycle on real processes.

Consider the following proposed system without recycle.

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


A mixture of 50% A and 50% B enters a separation process that is capable of splitting the two components into two streams: one containing 60% of the entering A and half the B, and one with 40% of the A and half the B (all by mass):

![Separator Example Flowchart.PNG](../../../upload.wikimedia.org/wikibooks/en/0/02/Separator_Example_Flowchart.PNG)

If 100 kg/hr of feed containing 50% A by mass enters the separator, what are the concentrations of A in the exit streams?

  
A degree of freedom analysis on this process:

4 unknowns (![\\dot{m}_2,x_{A2},\\dot{m}_3,\\mbox{ and } x_{A3} ](../../../upload.wikimedia.org/math/d/6/9/d694d9f118dcff4fca53361f54cdb01b.png)), 2 mass balances, and **2** pieces of information (knowing that 40% of A and half of B leaves in stream 3 is not independent from knowing that 60% of A and half of B leaves in stream 2) = **0** DOF.

Methods of previous chapters can be used to determine that ![ \\dot{m}_2 = 55 \\frac{kg}{hr}, x_{A2} = 0.545, \\dot{m}_3=45 \\frac{kg}{hr}](../../../upload.wikimedia.org/math/0/6/d/06d09a24eeba6f50bbcee3238afc548b.png) and ![ x_{A3} = 0.444 ](../../../upload.wikimedia.org/math/7/0/4/7041e2237467383470ba2b4ac642015b.png). This is good practice for the interested reader.

If we want to obtain a greater separation than this, one thing that we can do is use a _recycle system_, in which a portion of one of the streams is siphoned off and remixed with the feed stream in order for it to be re-separated. The choice of which stream should be re-siphoned depends on the desired properties of the exit streams. The effects of each choice will now be assessed.

### Implementing Recycle on the Separation Process

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Suppose that in the previous example, a recycle system is set up in which half of stream 3 is siphoned off and recombined with the feed (which is still the same composition as before). Recalculate the concentrations of A in streams 2 and 3. Is the separation more or less effective than that without recycle? Can you see a major limitation of this method? How might this be overcome?

This is a rather involved problem, and must be taken one step at a time. The analyses of the cases for recycling each stream are similar, so the first case will be considered in detail and the second will be left for the reader.

#### Step 1: Draw a Flowchart

You must be careful when drawing the flowchart because the separator separates 60% of all the A that enters it into stream 2, **not 60% of the fresh feed stream**.

Note: there is a mistake in the flow scheme. m6 and xA6 before the process is actually m4 and xA4

![Separator Example Flowchart with Recycle.PNG](../../../upload.wikimedia.org/wikibooks/en/7/77/Separator_Example_Flowchart_with_Recycle.PNG)

#### Step 2: Do a Degree of Freedom Analysis

Recall that you must include the recombination and splitting points in your analysis.

  * **Recombination point**: 4 unknowns - 2 mass balances = **2** degrees of freedom
  * **Separator**: 6 unknowns (nothing is specified) - 2 independent pieces of information - 2 mass balances = **2** DOF
  * **Splitting point**: 6 unknowns (again, nothing is specified) - 2 mass balances - 1 assumption that concentration remains constant - 1 splitting ratio = **2** DOF
  * **Total** = 2 + 2 + 2 - 6 = **0**. Thus the problem is completely specified.

#### Step 3: Devise a Plan and Carry it Out

First, look at the entire system, since none of the original processes individually had 0 DOF.

  * **Overall mass balance on A**: ![ 0.5* 100\\frac{kg}{h} = \\dot{m}_2*x_{A2} + \\dot{m}_6*x_{A6} ](../../../upload.wikimedia.org/math/4/1/5/4156d27230aa9454a9ed9f1f4a3c0e20.png)
  * **Overall mass balance on B**: ![ 50\\frac{kg}{h} = \\dot{m}_2*\(1-x_{A2}\) + \\dot{m}_6*\(1-x_{A6}\) ](../../../upload.wikimedia.org/math/e/f/9/ef9f49691969c1cb130c90aa4286aa6e.png)

We have 4 unknowns and 2 equations at this point. This is where the problem solving requires some ingenuity. First, lets see what happens when we combine this information with the splitting ratio and constant concentration at the splitter:

  * **Splitting Ratio**: ![ {m}_6 = \\frac{\\dot{m}_3}{2} ](../../../upload.wikimedia.org/math/3/f/0/3f0bd25a817c9ac9135ed358e9a83ad1.png)
  * **Constant concentration**: ![ x_{A6} = x_{A3} ](../../../upload.wikimedia.org/math/b/5/4/b5425f9b3d4b4c5cc15c1c6a36989c0e.png)

Plugging these into the overall balances we have:

  * **On A**: ![ 50 = \\dot{m}_2*x_{A2} + \\frac{\\dot{m}_3}{2}*x_{A3} ](../../../upload.wikimedia.org/math/c/5/c/c5c428fcfb5f6c2fa5c4a7268d2ea413.png)
  * **Total**: ![ 50 = \\dot{m}_2*\(1-x_{A2}\) + \\frac{\\dot{m}_3}{2}*\(1-x_{A3}\) ](../../../upload.wikimedia.org/math/1/6/0/160510c5dfcd17689275845416a37045.png)

Again we have more equations than unknowns _but we know how to relate everything in these two equations to the inlet concentrations in the separator_. This is due to the conversions we are given:

  * **60% of entering A goes into stream 2** means ![ \\dot{m}_2*x_{A2} = 0.6*x_{A4}*\\dot{m}_4 ](../../../upload.wikimedia.org/math/c/c/5/cc567658d8fcb3d1ef57fefb2ad1c58c.png)
  * **40% of entering A goes into stream 3** means ![ \\dot{m}_3*x_{A3} = 0.4*x_{A4}*\\dot{m}_4 ](../../../upload.wikimedia.org/math/2/b/6/2b641a3fcb4f566f7a618fda6f3027e7.png)
  * **50% of entering B goes into stream 2** means ![ \\dot{m}_2*\(1-x_{A2}\) = 0.5*\(1-x_{A4}\)*\\dot{m}_4 ](../../../upload.wikimedia.org/math/e/e/8/ee83d1d358c65bd677b8ee2320eddfad.png)
  * **50% of entering B goes into stream 3** means ![ \\dot{m}_3*x_{A3} = 0.5*\(1-x_{A4}\)*\\dot{m}_4 ](../../../upload.wikimedia.org/math/1/a/7/1a7e7dc7f990428922036202504d4770.png)

Spend some time trying to figure out where these equations come from, it's all definition of mass fraction and translating words into algebraic equations.

Plugging in all of these into the existing balances, we finally obtain 2 equations in 2 unknowns:

  
**On A**: ![ 50 = 0.6 \\dot{m}_4*x_{A4} + \\frac{0.4}{2} \\dot{m}_4*x_{A4} ](../../../upload.wikimedia.org/math/6/e/d/6ed020006570276c429a7e5f3eb03c19.png)

**On B**: ![ 50 = 0.5 \\dot{m}_4*\(1-x_{A4}\)+ \\frac{0.5}{2} \\dot{m}_4*\(1-x_{A4}\) ](../../../upload.wikimedia.org/math/8/1/1/811e9eff0b5bf83b8772f02963a53e24.png)

Solving these equations gives:

  
![ \\dot{m}_4 = 129.17\\frac{kg}{h}, x_{A4} = 0.484 ](../../../upload.wikimedia.org/math/6/e/2/6e27576d0019c2410052b3322f6c3ff6.png)

**Note:**  
Notice that two things happened as expected: the concentration of the stream entering the evaporator went down (because the feed is mixing with a more dilute recycle stream), and the total flowrate went up (again due to contribution from the recycle stream). This is always a good rough check to see if your answer makes sense, for example if the flowrate was lower than the feed rate you'd know something went wrong

Once these values are known, you can choose to do a balance either on the separator or on the recombination point, since both now have 0 degrees of freedom. We choose the separator because that leads directly to what we're looking for.

The mass balances on the separator can be solved using the same method as that without a recycle system, the results are:

  
![\\dot{m}_2 = 70.83 \\frac{kg}{hr}, x_{A2} = 0.530, \\dot{m}_3 = 58.33 \\frac{kg}{hr}, x_{A3} = 0.429 ](../../../upload.wikimedia.org/math/5/0/2/502e3141175af4acc64fc05c24e12381.png)

Now since we know the flowrate of stream 3 and the splitting ratio we can find the rate of stream 6:

  
![ \\dot{m}_6 = \\frac{\\dot{m}_3}{2} = 29.165\\frac{kg}{hr}, x_{A6}=x_{A3}= 0.429 ](../../../upload.wikimedia.org/math/e/b/f/ebf3891fc8a30e3ca0335d9e72a37956.png)

**Note:**  
You should check to make sure that m2 and m6 add up to the total feed rate, otherwise you made a mistake.

Now we can assess how effective the recycle is. The concentration of A in the liquid stream _was_ reduced, by a small margin of 0.015 mole fraction. However, this extra reduction came at a pair of costs: the flow rate of dilute stream was significantly reduced: from 45 to 29.165 kg/hr! **This limitation is important to keep in mind and also explains why we bother trying to make very efficient separation processes.**

## Systems with Recycle: a Cleaning Process

### Problem Statement

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Consider a process in which freshly-mined ore is to be cleaned so that later processing units do not get contaminated with dirt. 3000 kg/hr of dirty ore is dumped into a large washer, in which water is allowed to soak the ore on its way to a drain on the bottom of the unit. The amount of dirt remaining on the ore after this process is negligible, but water remains absorbed on the ore surface such that the net mass flow rate of the cleaned ore is 3100 kg/hr.

The dirty water is cleaned in a settler, which is able to remove 90% of the dirt in the stream without removing a significant amount of water. The cleaned stream then is combined with a fresh water stream before re-entering the washer.

The wet, clean ore enters a dryer, in which all of the water is removed. Dry ore is removed from the dryer at 2900 kg/hr.

The design schematic for this process was as follows:

![OreSchematic.PNG](../../../upload.wikimedia.org/wikibooks/en/2/2f/OreSchematic.PNG)

_a)_ Calculate the necessary mass flow rate of fresh water to achieve this removal at steady state.

_b)_ Suppose that the solubility of dirt in water is ![ 0.4 \\frac{g \\mbox{ dirt}}{cm^3 \\mbox{ H}_2O } ](../../../upload.wikimedia.org/math/7/6/1/761167d18fcbbe669892f7bcaf431deb.png). Assuming that the water leaving the washer is saturated with dirt, calculate the mass fraction of dirt in the stream that enters the washer (after it has been mixed with the fresh-water stream).

### First Step: Draw a Flowchart

A schematic is given in the problem statement but it is very incomplete, since it does not contain any of the design specifications (the efficiency of the settler, the solubility of soil in water, and the mass flow rates). Therefore, _it is highly recommended that you draw your own picture even when one is provided for you_. Make sure you label all of the streams, and the unknown concentrations.

![OreFlowchart.PNG](../../../upload.wikimedia.org/wikibooks/en/7/79/OreFlowchart.PNG)

### Second Step: Degree of Freedom Analysis

  * **Around the washer**: 6 independent unknowns (![x_{O1}, \\dot{m}_2, x_{D2}, \\dot{m}_3, x_{D3}, x_{O4}](../../../upload.wikimedia.org/math/8/f/f/8ff69a07c9571b0a071e4e63a0b3bd8f.png)), three independent mass balances (ore, dirt, and water), and one solubility. The washer has **2 DOF**.
  * **Around the dryer**: 2 independent unknowns (![x_{O4}, \\dot{m}_5](../../../upload.wikimedia.org/math/a/c/f/acf1ef0189f5efaeafd9570f626eda54.png)) and two independent equations = **0 DOF**.

**Note:**  
Since the dryer has no degrees of freedom already, we can say that _the system variables behave as if the stream going into the dryer was not going anywhere_, and therefore this stream should not be included in the "in-between variables" calculation.

  * **Around the Settler**:5 independent unknowns (![\\dot{m}_3, x_{D3}, \\dot{m}_7, \\dot{m}_8, x_{D8}](../../../upload.wikimedia.org/math/5/f/2/5f27c341e3bfe719bedd1cd63cbba857.png)), two mass balances (dirt and water), the solubility of saturated dirt, and one additional information (90% removal of dirt), leaving us with **1 DOF**.
  * **At the mixing point**: We need to include this in order to calculate the total degrees of freedom for the process, since otherwise we're not counting m9 anywhere. 5 unknowns (![\\dot{m}_2, x_{D2}, \\dot{m}_8, x_{D8}, \\dot{m}_9](../../../upload.wikimedia.org/math/f/c/e/fce7c23406b16495eb2695747d4d990f.png)) and 2 mass balances leaves us with **3 DOF**.

Therefore, Overall = 3+2+1 - 6 intermediate variables (not including xO4 since that's going to the dryer) = 0

The problem is well-defined.

### Devising a Plan

Recall that the idea is to look for a unit operation or some combination of them with 0 Degrees of Freedom, calculate those variables, and then recalculate the degrees of freedom until everything is accounted for.

From our initial analysis, the dryer had 0 DOF so we can calculate the two unknowns xO4 and m5. Now we can consider xO4 and m5 known and redo the degree of freedom analysis on the unit operations.

  * **Around the washer**: We only have 5 unknowns now (![x_{O1}, \\dot{m}_2, x_{D2}, \\dot{m}_3, x_{D3} ](../../../upload.wikimedia.org/math/a/8/7/a87ec9fab83f5d8543ebe0bed1f96eff.png)), but still only three equations and the solubility. **1 DOF**.
  * **Around the settler**: Nothing has changed here since xO4 and m5 aren't connected to this operation.
  * **Overall System**: We have three unknowns (![x_{O1}, \\dot{m}_7, \\dot{m}_9](../../../upload.wikimedia.org/math/e/0/4/e04c28b3ceafb45374efa50c8492f1b2.png)) since ![\\dot{m}_5](../../../upload.wikimedia.org/math/6/c/6/6c6dd7f51c28863e2f4db62e4819a18e.png) is already determined, and we have three mass balances (ore, dirt, and water). Hence we have **0 DOF** for the overall system.

Now we can say we know ![ x_{O1}, \\dot{m}_7, ](../../../upload.wikimedia.org/math/d/0/1/d0112c8e5e817caa269d2a443ac7c077.png) and ![ \\dot{m}_9 ](../../../upload.wikimedia.org/math/d/1/f/d1f4bc14b64140a9863b5c7905b35479.png).

  * **Around the settler again**: since we know m7 the settler now has **0 DOF** and we can solve for ![ \\dot{m}_3, x_{D3}, \\dot{m}_8, ](../../../upload.wikimedia.org/math/c/1/5/c15a837fbc804e4555d1a508f9f579ab.png) and ![ x_{D8} ](../../../upload.wikimedia.org/math/a/d/1/ad1aceeee47276a2714165250e66a855.png).
  * **Around the washer again**: Now we know m8 and xD8. How many balances can we write?

**Note:**  
If we try to write a balance on the ore, we will find that _the ore is already balanced because of the other balances we've done_. If you try to write an ore balance, you'll see you already know the values of all the unknowns in the equations. Hence we can't count that balance as an equation we can use (I'll show you this when we work out the actual calculation).

The washer therefore has 2 unknowns (m2, xD2) and _2 equations_ (the dirt and water balances) = **0 DOF**

This final step can also be done by balances on the recombination point (as shown below). Once we have m2 and xD2 the system is completely determined.

### Converting Units

The only given information in inconsistent units is the solubility, which is given as ![ 0.4 \\frac{g \\mbox{ dirt}}{cm^3 \\mbox{ H}_2O } ](../../../upload.wikimedia.org/math/7/6/1/761167d18fcbbe669892f7bcaf431deb.png). However, since we know the density of water (or can look it up), we can convert this to ![ \\frac{kg \\mbox{ dirt}}{kg \\mbox{ H}_2O } ](../../../upload.wikimedia.org/math/7/5/b/75bb16634b744df2e43ecffcc1fb7384.png) as follows:

![ 0.4 \\frac{g \\mbox{ dirt}}{cm^3 \\mbox{ H}_2O} * 1 \\frac{ cm^3 \\mbox{ H}_2O}{g \\mbox{ H}_2O} 
= 0.4 \\frac{ g \\mbox{ dirt}}{g \\mbox{ H}_2O} 
= 0.4 \\frac{ kg \\mbox{ dirt}}{kg \\mbox{ H}_2O} ](../../../upload.wikimedia.org/math/a/c/5/ac5efe61e18aa48e7cedab9abc131358.png)

Now that this information is in the same units as the mass flow rates we can proceed to the next step.

### Carrying Out the Plan

First, do any two mass balances on the dryer. I choose total and ore balances. Remember that the third balance is not independent of the first two!

  * **Overall Balance**: ![ \\dot{m}_4 = \\dot{m}_5 + \\dot{m}_6 ](../../../upload.wikimedia.org/math/c/1/e/c1efcbf7c2de88650e40468b0ea9ef76.png)
  * **Ore Balance**: ![ \\dot{m}_4*x_{O4} = \\dot{m}_5*x_{O5} + \\dot{m}_6*x_{O6} ](../../../upload.wikimedia.org/math/c/2/5/c25f2e6908a9df219d650719c6d37db0.png)

Substituting the known values:

  * Overall: ![ 3100 = \\dot{m}_5 + 2900 ](../../../upload.wikimedia.org/math/c/f/f/cff6328c039d379878d8a87cefa13266.png)
  * Ore: ![ x_{O4}*3100 = 1*2900 ](../../../upload.wikimedia.org/math/1/4/6/146301d8b4f1f645ca23af547f5056d8.png)

Solving gives:

  
![ \\dot{m}_5 = 200 \\frac{kg}{hr} ](../../../upload.wikimedia.org/math/6/2/0/620ffc8870e934b878fda4351bc9d487.png)

![ x_{O4} = 0.935 \\frac{kg}{hr} ](../../../upload.wikimedia.org/math/3/f/b/3fb26b45bb0106f0fefff0c27476fde7.png)

Now that we have finished the dryer we do the next step in our plan, which was the overall system balance:

  * **Water Balance**: ![ \\dot{m}_9 = \\dot{m}_5 ](../../../upload.wikimedia.org/math/9/5/5/955d305b75569ffb784cfe16071139ab.png)
  * **Ore Balance**: ![ x_{O1}*\\dot{m}_1 = \\dot{m}_6 ](../../../upload.wikimedia.org/math/8/e/9/8e9f3b2f95b1c21a1b3eb8a04391fa15.png)
  * **Dirt Balance**: ![ \(1-x_{O1}\)*\\dot{m}_1 = \\dot{m}_7 ](../../../upload.wikimedia.org/math/a/e/1/ae1bcace777f910aa998988545ac38f3.png)

  
![ \\dot{m}_9 = 200\\frac{kg}{hr} ](../../../upload.wikimedia.org/math/9/b/6/9b6b365e06a756c42ec61fc6f4413cc4.png), ![ x_{O1} = 0.967 ](../../../upload.wikimedia.org/math/3/d/b/3db250271918cdf3fe42a570ab0f3703.png),

![ \\dot{m}_7 = 100 \\frac{kg}{hr} ](../../../upload.wikimedia.org/math/b/4/6/b467894a79bcb38816544415cab43c33.png)

Next we move to the settler as planned, this one's a bit trickier since the solutions aren't immediately obvious but a system must be solved.

  * **Overall Balance**: ![ \\dot{m}_3 = \\dot{m}_7 + \\dot{m}_8 ](../../../upload.wikimedia.org/math/4/c/3/4c330a5a16472573f39270b0445e685b.png)
  * **Dirt Balance**: ![ \\dot{m}_3*x_{D3} = \\dot{m}_7*x_{D7} + \\dot{m}_8*x_{D8} ](../../../upload.wikimedia.org/math/a/9/2/a927c277c37762b523796b7d8347272b.png)
  * **Efficiency of Removal**: ![ \\dot{m}_7 = 0.9*\\dot{m}_3*x_{D3} ](../../../upload.wikimedia.org/math/c/8/9/c890d9ea9d69c805c8705ca314915b1e.png)

Using the solubility is slightly tricky. You use it by noticing that _the mass of dirt in stream 3 is proportional to the mass of water_, and hence you can write that:

  * mass dirt in stream 3 = 0.4 * mass water in stream 3
  * **Solubility**: ![ \\dot{m}_3*x_{D3} = 0.4*\\dot{m}_3 * \(1-x_{D3}\) ](../../../upload.wikimedia.org/math/8/a/c/8ac9c7666904f835f1fd33120df1d69d.png)

Plugging in known values, the following system of equations is obtained:

  * ![ \\dot{m}_3 = 100 + \\dot{m}_8 ](../../../upload.wikimedia.org/math/8/0/4/8046c71979568ee97cede539263cffc5.png)
  * ![ \\dot{m}_3*x_{D3} = 100 + \\dot{m}_8*x_{D8} ](../../../upload.wikimedia.org/math/d/9/7/d9779871c02172f39ac2f7ab1a86a553.png)
  * ![ \\dot{m}_3*x_{D3} = 111.11 ](../../../upload.wikimedia.org/math/f/0/0/f00fcd2a8bd125af6d575facc0a555c8.png)
  * ![ \\dot{m}_3*x_{D3} = 0.4*\\dot{m}_3*\(1-x_{D3}\) ](../../../upload.wikimedia.org/math/8/a/c/8ac9c7666904f835f1fd33120df1d69d.png)

Solving these equations for the 4 unknowns, the solutions are:

  
![ \\dot{m}_3 = 388.89\\frac{kg}{hr}, \\dot{m}_8 = 288.89\\frac{kg}{hr}, ](../../../upload.wikimedia.org/math/e/9/e/e9e042a6c8c87068d04807839741d610.png)

![ x_{D3} = 0.286, x_{D8} = 0.0385 ](../../../upload.wikimedia.org/math/b/a/f/bafca45a293ae34bbdd02c8de50787cb.png)

Finally, we can go to the mixing point, and say:

  * **Overall**: ![ \\dot{m}_8 + \\dot{m}_9 = \\dot{m}_2 ](../../../upload.wikimedia.org/math/a/8/0/a80189843b833fed787893645298edb8.png)
  * **Dirt**: ![ \\dot{m}_8*x_{D8} = \\dot{m}_2*x_{D2} ](../../../upload.wikimedia.org/math/9/f/b/9fb4952683d3ec9d8999e2edc6b60961.png)

From which the final unknowns are obtained:

  
![ \\dot{m}_2 = 488.89 \\frac{kg}{hr}](../../../upload.wikimedia.org/math/d/8/c/d8c6c3ed6e92f9acd71ab77f348ee697.png)

![ x_{D2} = 0.0229 ](../../../upload.wikimedia.org/math/b/a/a/baa7ec2e3f595ebbbd0238bba4d7310c.png)

Since the problem was asking for ![ \\dot{m}_2 ](../../../upload.wikimedia.org/math/2/d/3/2d3772bdc14344289299dceaa2c138a4.png), we are now finished.

### Check your work

These values should be checked by making a new flowchart with the numerical values, and ensuring that the balances on the washer are satisfied. This is left as an exercise for the reader.

# **Chapter 5: Mass/mole balances in reacting systems**

## Review of Reaction Stoichiometry

Up until now, all of the balances we have done on systems have been in terms of **mass**. However, mass is inconvenient for a reacting system because it does not allow us to take advantage of the _stoichiometry_ of the reaction in relating the relative amounts of reactants and of products.

**Stoichiometry** is the relationship between reactants and products in a balanced reaction as given by the ratio of their coefficients. For example, in the reaction:

![ C_2H_2 + 2H_2 \\rightarrow C_2H_6 ](../../../upload.wikimedia.org/math/0/5/6/0569aecbca37cb1e94b19522266ab775.png)

the reaction stoichiometry would dictate that for every **one** molecule of ![ C_2H_2 ](../../../upload.wikimedia.org/math/7/f/a/7fad1402ade6a272675b7a62273e9ae3.png) (acetylene) that reacts, two molecules of ![ H_2 ](../../../upload.wikimedia.org/math/4/f/6/4f645cef778354b6fc6e9a9fa63ed844.png) (hydrogen) are consumed and one molecule of ![ C_2H_6 ](../../../upload.wikimedia.org/math/4/4/5/445b2b7ba7fdd81b13ed141174094a19.png) are formed. However, **this does not hold for grams of products and reactants**.

Even though the number of molecules in _single substance_ is proportional to the mass of that substance, the constant of proportionality (the molecular mass) is not the same for every molecule. Hence, it is necessary to use the molecular weight of each molecule to convert from grams to _moles_ in order to use the reaction's coefficients.

## Molecular Mole Balances

We can write balances on moles like we can on anything else. We'll start with our ubiquitous general balance equation:

![ Input - Output = Accumulation - Generation ](../../../upload.wikimedia.org/math/8/1/0/810ccdf40ec2ea6b2287d08b2e347fb6.png)

As usual we assume that accumulation = 0 in this book so that:

![ Input - Output + Generation = 0 ](../../../upload.wikimedia.org/math/7/3/7/7370002ec0cd08f4346180af394dfcc4.png)

Let us denote molar flow rates by ![ \\dot{n} ](../../../upload.wikimedia.org/math/2/1/e/21e42b0d3f22d51793979688930cc8a7.png) to distinguish them from mass flow rates. We then have a similar equation to the mass balance equation:

  
![ \\Sigma \\dot{n}_{in} - \\Sigma \\dot{n}_{out} + n_{gen} = 0 ](../../../upload.wikimedia.org/math/a/d/1/ad185c7c3a567fe4034e68a195db8f17.png)

The same equation can be written in terms of each individual species.

There are a couple of important things to note about this type of balance as opposed to a mass balance:

  1. Just like with the mass balance, in a mole balance, a non-reactive system has ![ n_{gen} = 0 ](../../../upload.wikimedia.org/math/1/8/6/186018f6cbc063cc8cc3f98811d99e0c.png) for all species.
  2. **Unlike the mass balance**, the TOTAL generation of moles isn't necessarily 0 even for the overall mole balance! To see this, consider how the total number of moles changes in the above reaction; the final number of moles will not equal the initial number because 3 total moles of molecules are reacting to form 1 mole of products.

Why would we use it if the generation isn't necessarily 0? We use the molecular mole balance because **if we know how much of any one substance is consumed or created in the reaction, we can find all of the others from the reaction stoichiometry**. This is a very powerful tool because _each reaction only creates one new unknown_ if you use this method! The following section is merely a formalization of this concept, which can be used to solve problems involving reactors.

## Extent of Reaction

In order to formalize the previous analysis of reactions in terms of a single variable, let us consider the generic reaction:

![ aA + bB \\rightarrow cC + dD ](../../../upload.wikimedia.org/math/0/6/a/06ac9b691b88f7e453c53f64d052e1a5.png)

The **Molar Extent of Reaction** X is defined as:

  
![ X = -\\frac{\\Delta n_A}{a} = -\\frac{\\Delta n_B}{b} =  \\frac{\\Delta n_C}{c} = \\frac{\\Delta n_D}{d} ](../../../upload.wikimedia.org/math/2/7/8/2787d1bb2a611fa4e2fddd333ab823e2.png)

Since all of these are equivalent, it is possible to find the change in moles of any species in a given reaction if the extent of reaction X is known.

**Note:**  
Though they won't be discussed here, there are other ways in which the extent of reaction can be defined. Some other definitions are dependent on the **percent** change of a particular substrate, and the stoichiometry is used in a different way to determine the change in the others. This definition makes X independent of the substrate you choose.

The following example illustrates the use of the extent of reaction.

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Consider the reaction ![ H_2O_2 + O_3 \\rightarrow H_2O + 2O_2 ](../../../upload.wikimedia.org/math/4/d/1/4d1b61a200cdb475a2ceef57bff0cfc7.png). If you start with 50 g of ![ H_2O_2 ](../../../upload.wikimedia.org/math/6/1/9/619d83426f632d57129490cec592e2a3.png) and 25 grams of ![ O_3 ](../../../upload.wikimedia.org/math/e/a/7/ea7c9aa46ec1ce38d9663d73b8ae6deb.png), and 25% of the moles of ![ O_3 ](../../../upload.wikimedia.org/math/e/a/7/ea7c9aa46ec1ce38d9663d73b8ae6deb.png) are consumed, find the molar extent of reaction and the changes in the other components.

**Solution**: First we need to convert to moles, since stoichiometry is **not** valid when units are in terms of mass.

![ 50 \\mbox{ g H}_2O_2 * \\frac{1 \\mbox{ mol}}{34 \\mbox{ g}} = 1.471 \\mbox{ moles H}_2O_2 ](../../../upload.wikimedia.org/math/1/4/b/14b394174abd93c47aadd5f9b60c8adf.png)

![ 25 \\mbox{ g O}_3 * \\frac{1 \\mbox{ mol}}{48 \\mbox{ g}} = 0.5208 \\mbox{ moles O}_3 ](../../../upload.wikimedia.org/math/f/d/8/fd8911f4dae67c18d8804074873cd191.png)

Clearly ozone is the limiting reactant here. Since 25% is consumed, we have that:

![ \\Delta n\(O_3\) = -0.25*0.5208 = -0.1302 \\mbox{ moles O}_3 ](../../../upload.wikimedia.org/math/9/e/8/9e8d110683c0275a8d59983fe88ec6d5.png)

Hence, by definition, ![ X = \\frac{-0.1302}{1} = 0.1302 ](../../../upload.wikimedia.org/math/0/d/d/0dd932a81024f8d31a7c6e0a551762a2.png)

And then we have ![ \\Delta n\(H_2O_2\) = -0.1302, \\Delta n\(H_2O\) = 0.1302, \\Delta n\(O_2\) = 2*0.1302 = 0.2604 ](../../../upload.wikimedia.org/math/8/e/2/8e28e9bceb12d3e5275aa882ac2f631b.png), all in moles of the appropriate substrate.

## Mole Balances and Extents of Reaction

The mole balance written above can be written in terms of extent of reaction _if we notice that the ![ \\Delta n\(A\) ](../../../upload.wikimedia.org/math/2/4/6/2460db77ef9abf33c847a91915d04be7.png) term defined above is exactly the number of moles of a generated or consumed by the reaction_.

**Note:**  
This is only useful for _individual species_ balances, not the overall mole balance. When doing balances on reactive systems, unlike with non-reactive systems, it is generally easier to use all individual species balances possible, rather than the total mole balance and then all but one of the individual species. This is because the total generation of moles in a reaction is generally not 0, so no algebraic advantage is gained by using the total material balance on the system.

Therefore we can write that:

![ n_{A,gen} = \\Delta n\(A\) = -X*a ](../../../upload.wikimedia.org/math/c/0/6/c06182303019646b41494904fb98de18.png)

where X is the molar extent of reaction and a is the stoichiometric coefficient of A. Plugging this into the mole balance derived earlier, we arrive at the **molecular mole balance** equation:

  
![ \\Sigma \\dot{n}_{A,out} - \\Sigma{n}_{A,in} - X*a = 0 ](../../../upload.wikimedia.org/math/2/8/b/28b24e2ed43f17a2f8b9399db135cb88.png) if A is consumed, or +Xa if it is generated in the reaction

## Degree of Freedom Analysis on Reacting Systems

If we have N different molecules in a system, we can write N mass balances _or_ N mole balances, whether a reaction occurs in the system or not. The only difference is that in a reacting system, we have one additional unknown, the molar extent of reaction, for each reaction taking place in the system. Therefore **each reaction taking place in a process will add one degree of freedom to the process**.

**Note:**  
This will be different from the _atom_ balance which is discussed later.

## Complications

Unfortunately, life is not ideal, and even if we want a single reaction to occur to give us only the desired product, this is either impossible or uneconomical compared to dealing with byproducts, side reactions, equilibrium limitations, and other non-idealities.

### Independent and Dependent Reactions

When you have more than one reaction in a system, you need to make sure that they are **independent**. The idea of independent reactions is similar to the idea of [linear independence](http://en.wikipedia.org/wiki/linear_independence) in mathematics.

Lets consider the following two general parallel competing reactions:

  * ![ aA + bB \\rightarrow cC + dD ](../../../upload.wikimedia.org/math/0/6/a/06ac9b691b88f7e453c53f64d052e1a5.png)
  * ![ a_2A + b_2B \\rightarrow e_2E ](../../../upload.wikimedia.org/math/2/5/7/257de103d77ff84988de29100ec078ec.png)

We can represent each of the reactions by a _vector_ of the coefficients:

  * ![ V  = \[\\mbox{ A coeff, B coeff, C coeff, D coeff, E coeff}\]](../../../upload.wikimedia.org/math/b/9/2/b9204a73333fce2988dc2758af552865.png)
  * ![ v_1 = \[-a, -b, c, d,0\] ](../../../upload.wikimedia.org/math/2/a/5/2a52a9a3f3666181131d230c098a5cab.png)
  * ![ v_2 = \[-a_2,-b_2,0,0,e_2\] ](../../../upload.wikimedia.org/math/a/d/9/ad92124bb028fb1ed6f7269975381e6b.png)

[This site](http://www.math.odu.edu/~bogacki/cgi-bin/lat.cgi?c=li)

**Note:**  
The site above gives a nice tool to tell whether any number of vectors are linearly dependent or not. Lacking such a tool, it is necessary to assess by hand whether the equations are independent. **Only independent equations should be used in your analysis of multiple reactions**, so if you have dependent equations, you can eliminate reactions from consideration until you've obtained an independent set.

![Clipboard](../../../upload.wikimedia.org/wikipedia/commons/thumb/1/1f/Clipboard.svg/45px-Clipboard.svg.png)

**To do:**  
double-check this.

By definition a set of vectors is only linearly independent if the equation:

![ K_1*v_1 + K_2*v_2 = 0 ](../../../upload.wikimedia.org/math/d/2/6/d26d3f14710fb2c549728de124b5e6e9.png)

where K1 and K2 are constants only has one solution: ![ K_1 = K_2 = 0 ](../../../upload.wikimedia.org/math/3/3/7/337ffabc488d5af7ddc93f92111a354e.png).

Lets plug in our vectors:

![ K_1 * \[-a, -b, c, d,0\] + K_2 * \[-a_2,-b_2,0,0,e_2\] = 0 ](../../../upload.wikimedia.org/math/4/a/f/4afa2835b7000210ad00ea71b7b85176.png)

Since _all components_ must add up to 0, the following system follows:

  * ![ -K_1*a - K_2*a_2 = 0 ](../../../upload.wikimedia.org/math/9/2/d/92d9aeb13843a8d96b1b1ce686218d72.png)
  * ![ -K_1*b - K_2*b_2 = 0 ](../../../upload.wikimedia.org/math/b/3/7/b374976e7e497cb55114d995c05c675c.png)
  * ![ K_1*c + 0 = 0 ](../../../upload.wikimedia.org/math/d/f/1/df1fe4e64ab8bdc0ee2666ce860ea2f0.png)
  * ![ K_1*d + 0 = 0 ](../../../upload.wikimedia.org/math/0/f/6/0f668c446fc48b77d20d52928be8a7d8.png)
  * ![ 0 + K_2*e_2 = 0 ](../../../upload.wikimedia.org/math/4/1/1/411966928374524ce95acffd65ebc446.png)

Obviously, the last three equations imply that unless c = d = 0 and e2 = 0, ![ K_1 = K_2 = 0 ](../../../upload.wikimedia.org/math/3/3/7/337ffabc488d5af7ddc93f92111a354e.png) and thus the reactions are independent.

#### Linearly Dependent Reactions

There is one rule to keep in mind whenever you are checking for reaction dependence or independence, which is summarized in the following box.

  
**If any non-zero multiple of one reaction can be added to a multiple of a second reaction to yield a third reaction, then the three reactions are not independent**.

Therefore, if the following reaction could occur in the same system as the two above:

![ \(a+a_2\)A + \(b+b_2\)B \\rightarrow cC + dD + e_2E ](../../../upload.wikimedia.org/math/4/a/f/4af68c53b99501a8dfb33de50b753166.png)

then it would not be possible to analyze all three reactions at once, since this reaction is the sum of the first two. Only two can legitimately be analyzed at the same time.

All degree of freedom analyses in this book assume that the reactions are independent. You should check this by inspection or, for a large number of reactions, with numerical methods.

### Extent of Reaction for Multiple Independent Reactions

When you are setting up extents of reaction in a molecular species balance, you _must_ make sure that you set up one for _each_ reaction, and include both in your mole balance. So really, your mole balance will look like this:

![ \\Sigma n_{A,in} - \\Sigma{n_A,out} + \\Sigma a_k X_k = 0 ](../../../upload.wikimedia.org/math/9/e/2/9e2777df2a3cc6abe37594a59593ea93.png)

for all k reactions. In such cases it is generally easier, if possible, to use an atom balance instead due to the difficulty of solving such equations.

### Equilibrium Reactions

In many cases (actually, the majority of them), a given reaction will be **reversible**, meaning that instead of reacting to completion, it will stop at a certain point and not go any farther. How far the reaction goes is dictated by the value of the _equilibrium coefficient_. Recall from general chemistry that the equilibrium coefficient for the reaction ![ aA + bB \\rightarrow cC + dD ](../../../upload.wikimedia.org/math/0/6/a/06ac9b691b88f7e453c53f64d052e1a5.png) is defined as follows:

  
![K=\\frac{C_{C,eq}^c*C_{D,eq}^d}{C_{A,eq}^a*C_{B,eq}^b} ](../../../upload.wikimedia.org/math/d/d/c/ddc31817067e395b03ae2085c9c9f303.png) with concentration ![ C_i ](../../../upload.wikimedia.org/math/6/8/3/6839c253419b2770304593fac4ef3ce5.png) expressed as molarity for liquid solutes or partial pressure for gasses

Here [A] is the equilibrium concentration of A, usually expressed in molarity for an aqueous solution or partial pressure for a gas. _This equation can be remembered as "products over reactants"_ .

Usually **solids and solvents are omitted** by convention, since their concentrations stay approximately constant throughout a reaction. For example, in an aqueous solution, if water reacts, it is left out of the equilibrium expression.

Often, we are interested in obtaining the extent of reaction of an equilibrium reaction when it is in equilibrium. In order to do this, first recall that:

![ X = \\frac{-\\Delta n_A}{a} ](../../../upload.wikimedia.org/math/7/3/b/73b0366d578bfca33e64dce08a7150a8.png)

and similar for the other species.

#### Liquid-phase Analysis

Rewriting this in terms of molarity (moles per volume) by dividing by volume, we have:

![ \\frac{X}{V} = \\frac{\[A\]_0 - \[A\]_f}{a} ](../../../upload.wikimedia.org/math/5/7/a/57a42b7222864605c06d6209eb4e785d.png)

Or, since the final state we're interested in is the equilibrium state,

![ \\frac{X}{V} = \\frac{\[A\]_0 - \[A\]_{eq}}{a} ](../../../upload.wikimedia.org/math/d/2/9/d29ec9fd1adb2bb0bc0b56221cb639cf.png)

Solving for the desired equilibrium concentration, we obtain the equation for equilibrium concentration of A in terms of conversion:

  
![ \[A\]_{eq} = \[A\]_0 - \\frac{aX}{V} ](../../../upload.wikimedia.org/math/7/c/8/7c81ef38df756d4aff6b3c5b3ff916e8.png)

Similar equations can be written for B, C, and D using the definition of extent of reaction. Plugging in all the equations into the expression for K, we obtain:

  
![ K = \\frac{\(\[C\]_0+\\frac{cX}{V}\)^c \(\[D\]_0 + \\frac{dX}{V}\)^d} {\(\[A\]_0 - \\frac{aX}{V}\)^a\(\[B\]_0 - \\frac{bX}{V}\)^b} ](../../../upload.wikimedia.org/math/0/c/2/0c289997e3f1f29eb3539dbb93acea70.png)

At equilibrium for liquid-phase reactions **only**

Using this equation, knowing the value of K, the reaction stoichiometry, the initial concentrations, and the volume of the system, the equilibrium extent of reaction can be determined.

**Note:**  
**If you know the reaction reaches equilibrium in the reactor, this counts as an additional piece of information in the DOF analysis because it allows you to find X**. This is the same idea as the idea that, if you have an irreversible reaction and know it goes to completion, you can calculate the extent of reaction from that.

#### Gas-phase Analysis

By convention, gas-phase equilibrium constants are given in terms of **partial pressures** which, for ideal gasses, are related to the mole fraction by the equation:

  
![ P_A = y_AP ](../../../upload.wikimedia.org/math/f/b/4/fb4014836673ee97a968c52d285c7516.png) **for ideal gasses only**

If A, B, C, and D were all gases, then, the equilibrium constant would look like this:

  
![ \\frac{P_C^cP_D^d}{P_A^aP_B^b} ](../../../upload.wikimedia.org/math/a/2/d/a2dad9ec5208d1ef36809a83005aa0c0.png) **Gas-Phase Equilibrium Constant**

In order to write the gas equilibrium constant in terms of extent of reaction, let us assume for the moment that we are dealing with ideal gases. You may recall from general chemistry that for an ideal gas, we can write the ideal gas law for _each species_ just as validly as we can on the _whole gas_ (for a non-ideal gas, this is in general not true). Since this is true, we can say that:

![ \\frac{n_A}{V} = \[A\] = \\frac{P_A}{RT} ](../../../upload.wikimedia.org/math/b/1/1/b11823ce21a1b358b9335ba01800643f.png)

Plugging this into the equation for ![ \\frac{X}{V} ](../../../upload.wikimedia.org/math/a/7/3/a73d671edb4df8334fd907f53ec2ff3b.png) above, we obtain:

![ \\frac{aX}{V} = \[A\] - \[A\]_{eq} = \\frac{P_{A0}}{RT} - \\frac{P_{A,eq}}{RT} ](../../../upload.wikimedia.org/math/f/3/4/f34cd853e22c5415ba5f870a1f224af2.png)

Therefore,

  
![ P_{a,eq} = P_{A0} - \\frac{aXRT}{V}](../../../upload.wikimedia.org/math/4/d/9/4d9676f3ff1c5ad368d87cbfc86186ef.png)

Similar equations can be written for the other components. Plugging these into the equilibrium constant expression:

  
![K = \\frac{ \(P_{C0} + \\frac{cXRT}{V} \)^c \(P_{D0} + \\frac{dXRT}{V} \)^d} {\(P_{A0} - \\frac{aXRT}{V} \)^a \(P_{B0} - \\frac{bXRT}{V} \)^b } ](../../../upload.wikimedia.org/math/3/3/d/33ddb4e345ccbba4e8ced182b305eaaa.png)

**Gas Phase Ideal-Gas Equilibrium Reaction at Equilibrium**

Again, if we know we are at equilibrium and we know the equilibrium coefficient (which can often be found in standard tables) we can calculate the extent of reaction.

### Special Notes about Gas Reactions

You need to remember that _In a constant-volume, isothermal gas reaction, the total pressure will change as the reaction goes on_, unless the same number of moles are created as produced. In order to show that this is true, you only need to write the ideal gas law for the total amount of gas, and realize that the total number of moles in the system changes.

This is why we don't want to use _total_ pressure in the above equations for K, we want to use _partial_ pressures, which we can conveniently write in terms of extent of reaction.

### Inert Species

Notice that all of the above equilibrium equations depend on _concentration_ of the substance, in one form or another. Therefore, if there are species present that don't react, they may still have an effect on the equilibrium because they will decrease the concentrations of the reactants and products. Just make sure you take them into account when you're calculating the concentrations or partial pressures of each species in preparation for plugging into the equilibrium constant.

## Example Reactor Solution using Extent of Reaction and the DOF

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Consider the reaction of Phosphene with oxygen: ![ 4PH_3+8O_2 \\rightarrow P_4O_{10} + 6H_2O ](../../../upload.wikimedia.org/math/4/5/c/45c1fead4a7f69d892bdd4ffcf14ec74.png)

Suppose a 100-kg mixture of 50% ![ PH_3 ](../../../upload.wikimedia.org/math/a/b/0/ab0d26fb39998de2680fe410f54b8c70.png) and 50% ![ O_2 ](../../../upload.wikimedia.org/math/6/1/2/612275025e7d8f5e663bf9355bd422b4.png) by mass enters a reactor in a single stream, and the single exit stream contains 25% ![ O_2 ](../../../upload.wikimedia.org/math/6/1/2/612275025e7d8f5e663bf9355bd422b4.png) by mass. Assume that all the reduction in oxygen occurs due to the reaction. How many degrees of freedom does this problem have? If possible, determine mass composition of all the products.

It always helps to draw a flowchart:

![Phosphate Rxn Flowchart.PNG](../../../upload.wikimedia.org/wikibooks/en/4/43/Phosphate_Rxn_Flowchart.PNG)

There are **four** independent unknowns: the total mass (mole) flowrate out of the reactor, the concentrations of two of the exiting species (once they are known, the forth can be calculated), and the extent of reaction.

Additionally, we can write four independent equations, one on each reacting substance. Hence, there are **0** DOF and this problem can be solved.

Let's illustrate how to do it for this relatively simple system, which illustrates some very important things to keep in mind.

First, recall that **total mass is conserved even in a reacting system**. Therefore, we can write that:

![ \\dot{m}_{out} = \\dot{m}_{in} = 100 \\mbox{ kg} ](../../../upload.wikimedia.org/math/6/4/0/64051a511c8b4f0d67330e7ef44e1d00.png)

Now, since component masses aren't conserved, we need to convert as much as we can into moles so we can apply the extent of reaction.

![ \\dot{n}_{PH_3,in}= 0.5*\(100\\mbox{ kg}\)* \\frac{1 \\mbox{ mol}}{0.034 \\mbox{ kg}} = 1470.6 \\mbox{ moles PH}_3 \\mbox{ in} ](../../../upload.wikimedia.org/math/2/f/5/2f57c7cffc3d7955d64419ff5aad4292.png) ![ \\dot{n}_{O_2,in} = 0.5*\(100\\mbox{ kg}\)* \\frac{1 \\mbox{ mol}}{0.032 \\mbox{ kg}} = 1562.5 \\mbox{ moles O}_2 \\mbox{ in} ](../../../upload.wikimedia.org/math/2/d/0/2d018e90983d9301e35aa9f88153b166.png)

![ \\dot{n}_{O_2,out} = 0.25*\(100\\mbox{ kg}\)* \\frac{1 \\mbox{ mol}}{0.032 \\mbox{ kg}} = 781.25 \\mbox{ moles O}_2 \\mbox{ out} ](../../../upload.wikimedia.org/math/9/0/3/9039130f765a878e4a795b514d197109.png)

Let's use the **mole balance on oxygen** to find the extent of reaction, since we know how much enters and how much leaves. Recall that:

![ \\Sigma \\dot{n}_{A,in} - \\Sigma \\dot{n}_{A,out} - a*X = 0 ](../../../upload.wikimedia.org/math/a/9/a/a9ab646505be83b9f33ad410a6d526a0.png)

where a is the stoichiometric coefficient for A. Plugging in known values, including a = 8 (from the reaction written above), we have:

![ 1562.5 - 781.25 - 8X = 0 ](../../../upload.wikimedia.org/math/f/6/8/f685a628015dbdc3b98c6c4d66fface3.png)

Solving gives:

  
![ X = 97.66 \\mbox{ moles} ](../../../upload.wikimedia.org/math/6/7/8/6781ebc9b1e345f646232ea06bbee4d0.png)

Now let's apply the mole balances to the other species to find how much of them is present:

  * ![PH_3: 1470.6 - \\dot{n}_{PH_3,out} - 4\(97.66\) = 0 \\rightarrow \\dot{n}_{PH_3,out} = 1080.0 \\mbox{ moles PH}_3 ](../../../upload.wikimedia.org/math/e/f/e/efecb0b3441575a526eed56bb6c94324.png)
  * ![P_4H_10: 0 - \\dot{n}_{P_4O_{10},out}+ 1\(97.66\) = 0 \\rightarrow \\dot{n}_{P_4H_10,out} = 97.66 \\mbox{ moles P}_4O_{10} ](../../../upload.wikimedia.org/math/a/9/f/a9ff79bfbcff030a28a01f7cdf65e75b.png) (note it's + instead of - because it's being generated rather than consumed by the reaction)
  * ![H_2O: 0 - \\dot{n}_{H_2O,out} + 6\(97.66\) = 0 \\rightarrow \\dot{n}_{H_2O,out} = 586.0 \\mbox{ moles H}_2O ](../../../upload.wikimedia.org/math/9/b/9/9b92cb6e13bddae556725d8237ca337d.png)

Finally, the last step we need to do is find the mass of all of these, and divide by the total mass to obtain the mass percents. As a sanity check, all of these plus 25 kg of oxygen should yield 100 kg total.

  * Mass ![ PH_3 ](../../../upload.wikimedia.org/math/a/b/0/ab0d26fb39998de2680fe410f54b8c70.png) out = 1080 moles * 0.034 kg\mole = 36.72 kg
  * Mass ![ P_4O_{10} ](../../../upload.wikimedia.org/math/a/d/8/ad8a8c9a8ec8361155fa89929983eff1.png) out = 97.66 moles * .284 kg\mole = 27.74 kg
  * Mass ![ H_2O ](../../../upload.wikimedia.org/math/a/8/9/a8970556f82155430c09c99db24d4261.png) out = 586 moles * 0.018 kg\mole = 10.55 kg

Sanity check: 36,72 + 27.74 + 10.55 + 25 (oxygen) = 100 kg (total), so we're still sane.

Hence, we get:

  
![ 36.72% PH_3,27.74% P_4H_{10},10.55% H_2O, 25% O_2 ](../../../upload.wikimedia.org/math/0/7/1/071d97bb159dac3373498d53ca81475f.png) by mass

## Example Reactor with Equilibrium

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Suppose that you are working in an organic chemistry lab in which 10 kg of compound A is added to 100 kg of a 16% aqueous solution of B (which has a density of 57 lb/ft^3) The following reaction occurs:

![ A + 2B \\leftarrow \\rightarrow 3C + D ](../../../upload.wikimedia.org/math/b/f/1/bf176de3f97401a4c6b83af5b46f8c35.png)

A has a molar mass of 25 g/mol and B has a molar mass of 47 g/mol. If the equilibrium constant for this reaction is 187 at 298K, how much of compound C could you obtain from this reaction? Assume that all products and reactants are soluble in water at the design conditions. Adding 10 kg of A to the solution causes the volume to increase by 5 L. Assume that the volume does not change over the course of the reaction.

**Solution**: First, draw a flowchart of what we're given.

![Equilibrium Rxn Flowchart.PNG](../../../upload.wikimedia.org/wikibooks/en/7/7b/Equilibrium_Rxn_Flowchart.PNG)

Since all of the species are dissolved in water, we should write the equilibrium constant in terms of molarity:

![ K = 187 = \\frac{\[C\]^3\[D\]}{\[A\]\[B\]^2} ](../../../upload.wikimedia.org/math/8/b/9/8b920ceda3b62753f9806d099908c3c6.png)

We use initial _molarities_ of A and B, while we are given mass percents, so we need to convert.

Let's first find the number of _moles_ of A and B we have initially:

![ n_{A0} = 10 \\mbox{ kg A} * \\frac{1 \\mbox{ mol A}}{0.025 \\mbox{ kg A}} = 400 \\mbox{ mol A} ](../../../upload.wikimedia.org/math/d/0/3/d03dc89d8fa5d36533435a5db26c8e2b.png)

![ n_{B0} = 100\\mbox{ kg solution} * \\frac{0.16 \\mbox{ kg B}}{\\mbox{kg sln}} = 16 \\mbox{ kg B} * \\frac{1 \\mbox{mol B}}{\\mbox{ 0.047 kg B}} = 340.43 \\mbox{ mol B} ](../../../upload.wikimedia.org/math/3/b/7/3b70e2ddc4a35cf4e9b891e5644ff572.png)

Now, the volume contributed by the 100kg of 16% B solution is:

![ V = \\frac{m}{\\rho} = \\frac{100 \\mbox{ kg}}{57 \\frac{lb}{ft^3} * \\frac{1 \\mbox{ kg}}{2.2 \\mbox{ lb}} * \\frac{1 \\mbox{ ft}^3}{28.317 \\mbox{ L}} } = 109.3 \\mbox{ L}](../../../upload.wikimedia.org/math/4/2/6/426aa5798cc703ec9fb39205ae04cb38.png)

Since adding the A contributes 5L to the volume, the volume after the two are mixed is ![ 109.3\\mbox{ L} + 5\\mbox{ L} = 114.3\\mbox{ L} ](../../../upload.wikimedia.org/math/e/1/0/e1017ddb502c954acc14280ce57ddb20.png).

By definition then, the molarities of A and B before the reaction occurs are:

  * ![ \[A\]_0 = \\frac{400 \\mbox{ moles A}}{114.3 \\mbox{ L}} = 3.500 M ](../../../upload.wikimedia.org/math/1/7/c/17c4266d2cecbda1834daddf28085b6c.png)
  * ![ \[B\]_0 = \\frac{340.42 \\mbox{ moles B}}{114.3 \\mbox{ L}} = 2.978 M ](../../../upload.wikimedia.org/math/e/7/9/e792a4b0f9035860cc758c6552cd2eb3.png)

In addition, there is no C or D in the solution initially:

  * ![ \[C\]_0 = \[D\]_0 = 0 ](../../../upload.wikimedia.org/math/9/9/c/99cfabda757c18efbb57ff6ca3de5f7e.png)

According to the stoichiometry of the reaction, ![ a = 1, b = 2, c = 3, d = 1 ](../../../upload.wikimedia.org/math/5/5/0/5500480755ce83bf7dad999ebba76d17.png). Therefore we now have enough information to solve for the conversion. Plugging all the known values into the equilibrium equation for liquids, the following equation is obtained:

![ 187 = \\frac{ \(\\frac{3X}{114.3}\)^3 \(\\frac{X}{114.3}\) } { \(3.5 - \\frac{X}{114.3} \) \(2.978 - \\frac{2X}{114.3} \) ^2 } ](../../../upload.wikimedia.org/math/d/4/6/d4651f160ca54d90f3ab3c857a28818e.png)

This equation can be solved using Goalseek or one of the numerical methods in appendix 1 to give:

  
![ X = 146.31\\mbox{ moles} ](../../../upload.wikimedia.org/math/d/d/9/dd9770f07ab04a770212c6a59e53f884.png)

Since we seek the amount of compound C that is produced, we have:

  * ![ X = \\frac{\\Delta n_C}{c} ](../../../upload.wikimedia.org/math/4/8/6/486f02d17d0277a2e97f5d83ed3d3789.png)
  * Since ![ c = 3, n_{C0} = 0, \\mbox{ and } X = 146.31 ](../../../upload.wikimedia.org/math/e/0/3/e034a2f47ee8caf508b20a9a409fb506.png), this yields ![ n_C = 3*146.31 = 438.93\\mbox{ moles C} ](../../../upload.wikimedia.org/math/1/2/6/12651572802acbc10e03745789c99b8c.png)

  
438.93 moles of C can be produced by this reaction.

## Introduction to Reactions with Recycle

Reactions with recycle are very useful for a number of reasons, most notably because they can be used to improve the selectivity of multiple reactions, push a reaction beyond its equilibrium conversion, or speed up a catalytic reaction by removing products. A recycle loop coupled with a reactor will generally contain a separation process in which unused reactants are (partially) separated from products. These reactants are then fed back into the reactor along with the fresh feed.

## Example Reactor with Recycle

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Consider a system designed for the hydrogenation of ethylene into ethane:

  * ![ 2H_2 + C_2H_2 \\rightarrow C_2H_6 ](../../../upload.wikimedia.org/math/1/6/4/1644af37ff71dee1f3d1a3da2022a907.png)
  * ![\(2A + B \\rightarrow C\)](../../../upload.wikimedia.org/math/e/c/8/ec85534684bbc8bf0a2d32e85dae4925.png)

The reaction takes too long to go to completion (and releases too much heat) so the designers decided to implement a recycle system in which, after only part of the reaction had finished, the mixture was sent into a membrane separator. There, most of the ethylene was separated out, with little hydrogen or ethylene contamination. After this separation, the cleaned stream entered a splitter, where some of the remaining mixture was returned to the reactor and the remainder discarded.

The system specifications for this process were as follows:

  * Feed: 584 kg/h ethylene, 200 kg/h hydrogen gas
  * Outlet stream from reactor contains 15% hydrogen by mass
  * Mass flows from membrane separator: 100 kg/h, 5% Hydrogen and 93% ethane
  * Splitter: 30% reject and 70% reflux

What was the extent of reaction for this system? What would the extent of reaction be if there was no separation/recycle process after (assume that the mass percent of hydrogen leaving the reactor is the same)? What limits how effective this process can be?

**Solution**:

Let's first draw our flowchart as usual:

![Reactor with Recycle.PNG](../../../upload.wikimedia.org/wikibooks/en/8/82/Reactor_with_Recycle.PNG)

### DOF Analysis

  * On reactor: 6 unknowns ![\(\\dot{m}_5, x_{A5}, x_{B5}, \\dot{m}_3, x_{B3}, X\)](../../../upload.wikimedia.org/math/b/3/a/b3a51a26c0df1b2da7ae3abf73ccd0b9.png) \- 3 equations = 3 DOF
  * On separator: 5 unknowns ![\(\\dot{m}_3, x_{B3}, \\dot{m}_5, x_{A5}, x_{B5}\)](../../../upload.wikimedia.org/math/0/e/2/0e24312cdcf2c3efb6496582c10ec02c.png) \- 3 equations = 2 DOF
  * On splitter: 3 unknowns - 0 equations (we used all of them in labeling the chart) -> 3 DOF
  * Duplicate variables: 8 (![\\dot{m}_5, x_{A5}, x_{B5} ](../../../upload.wikimedia.org/math/9/3/e/93e8ce46a824115050bfb2a122be547b.png) twice each and ![ \\dot{m}_3, x_{B3}](../../../upload.wikimedia.org/math/e/c/1/ec1c6719520350d6a7396ee18699ab92.png) once)
  * Total DOF = 8 - 8 = 0 DOF

### Plan and Solution

Generally, though not always, it is easiest to deal with the reactor itself _last_ because it usually has the most unknowns. Lets begin by looking at the overall system because we can often get some valuable information from that.

**Overall System** DOF(overall system) = 4 unknowns (![ \\dot{m}_5, x_{A5}, X, x_{B5}](../../../upload.wikimedia.org/math/c/5/5/c554263755f6e788c59a051c59279f06.png)) - 3 equations = 1 DOF.

**Note:**  
We CANNOT say that total mass of A and B is conserved because we have a reaction here! Therefore we must include the conversion X in our list of unknowns for both the reactor _and_ the overall system. However, the _total_ mass in the system is conserved so we can solve for ![ \\dot{m}_5 ](../../../upload.wikimedia.org/math/6/c/6/6c6dd7f51c28863e2f4db62e4819a18e.png).

Let's go ahead and solve for m5 though because that'll be useful later.

![784 = 100 + 0.3\(\\dot{m}_5\) ](../../../upload.wikimedia.org/math/4/4/d/44d85764dd7e9d52f086405577b405cf.png)

  
![\\dot{m}_5 = 2280 \\mbox{ kg/h}](../../../upload.wikimedia.org/math/f/a/5/fa5758a77361ca1f74c8eeb846d35fdb.png)

We can't do anything else with the overall system without knowing the conversion so lets look elsewhere.

DOF(separator) = 4 unknowns (![\\dot{m}_3,  x_{B3}, x_{A5}, x_{B5}](../../../upload.wikimedia.org/math/0/a/a/0aa265110d4cff10b5bc215ae1967228.png)) - 3 equations = 1 DOF. Let's solve for those variables we can though.

We can solve for m3 because from the overall material balance on the separator:

  * ![\\dot{m}_3 = \\dot{m}_4 + \\dot{m}_5 ](../../../upload.wikimedia.org/math/e/d/7/ed7839678270119d520b71f8cac507df.png)
  * ![ \\dot{m}_3 = 100 + 2280 ](../../../upload.wikimedia.org/math/2/0/f/20faad8a5234c3e1170a4d512422ad36.png)

  
![ \\dot{m}_3 = 2380\\mbox{ kg/h} ](../../../upload.wikimedia.org/math/4/8/a/48ac62328d91716ddf332cdb9197d8c2.png)

Then we can do a mass balance on A to solve for xA5:

  * ![ \\dot{m}_3x_{A3} = \\dot{m}_4x_{A4} + \\dot{m}_5x_{A5} ](../../../upload.wikimedia.org/math/3/c/4/3c45c62aa75f297e2459eefc76cc896e.png)
  * ![ 2380\(0.15\) = 100\(0.05\) + 2380\(x_{A5}\) ](../../../upload.wikimedia.org/math/1/2/1/121836ecd4b07fa07c1d9849a2317a54.png)

  
![x_{A5} = .1544](../../../upload.wikimedia.org/math/8/4/d/84da61155571e4d82896110d80b1a334.png)

Since we don't know ![ x_{B5} ](../../../upload.wikimedia.org/math/a/6/b/a6bd4ca35ad79cf92f4a52c4143ab8c8.png) or ![ x_{B3} ](../../../upload.wikimedia.org/math/a/3/f/a3fd193ab7ce4f6aa5f22312382d6515.png), we cannot use the mass balance on B or C for the separator, so lets move on. Let's now turn to the reactor:

### Reactor Analysis

DOF: 3 unknowns remaining (![ x_{B3}, x_{B5}, and X](../../../upload.wikimedia.org/math/f/1/4/f1450edc5705fb472a36ac8b75b4f192.png)) - 2 equations (because the overall balance is already solved!) = **1 DOF**. Therefore we still cannot solve the reactor completely. However, we can solve for the conversion and generation terms given what we know at this point. Lets start by writing a _mole_ balance on A in the reactor.

![ \\dot{n}_{A1} + \\dot{n}_{A,recycle} - X*a = \\dot{n}_{A3} ](../../../upload.wikimedia.org/math/2/5/7/25753558dfcf14435a6cc11ce65759cb.png)

To find the three nA terms we need to convert from mass to moles (since A is hydrogen, H2, the molecular weight is ![ \\frac{1\\mbox{ mol}}{0.002016\\mbox{ g}} ](../../../upload.wikimedia.org/math/9/6/8/96833520e2dc0c42e8cedb8d3c245281.png)):

  * ![ \\dot{n}_{A1} = 200\\frac{kg}{h} * \\frac{1 \\mbox{ mol}}{0.002016 \\mbox{ kg}} = 99206 \\frac{\\mbox{mol A}}{h} ](../../../upload.wikimedia.org/math/b/e/4/be423564afdffd3a14b3a1ee5601eb38.png)
  * ![ \\dot{n}_{A,recycle} = 0.7*\\frac{m_5*x_{A5}}{MW_A} = \\frac{0.7\(2280\)\(0.1544\)}{0.002016} = 122000 \\frac{\\mbox{mol A}}{h} ](../../../upload.wikimedia.org/math/7/a/4/7a4b348d26821c73ded06d3ebf4a31e7.png)

Thus the _total_ amount of A entering the reactor is:

  * ![ \\dot{n}_{A,in} = 99206 + 122000 = 221428\\frac{\\mbox{mol A}}{h} ](../../../upload.wikimedia.org/math/e/3/0/e30e778b387daa92327a8d83af98a3b1.png)

The amount exiting is:

  * ![ \\dot{n}_{A,out} = \\frac{ \\dot{m}_3*x_{A3}}{MM_A} = \\frac{2380*0.15}{0.002016} = 177083 \\frac{\\mbox{mol A}}{h} ](../../../upload.wikimedia.org/math/0/0/d/00d85bebffac398c3a1c173fcf14d9f1.png)

Therefore we have the following from the mole balance:

  * ![ 221428 - 2X = 177083 ](../../../upload.wikimedia.org/math/e/e/6/ee614bf4306e2bf5b76d3fe6aae6e7d6.png)

  
![ X = 22173 \\frac{\\mbox{ moles}}{h} ](../../../upload.wikimedia.org/math/f/1/a/f1a08d45734c8fb834e780f1b0b98486.png)

Now that we have this we can calculate the mass of B and C generated:

  * ![ m_{B,gen} = -Xb*MW_B = 22173 \\frac{\\mbox{ mol B}}{h} * 0.026 \\frac{kg}{\\mbox{ mol B}} = -576.5\\frac{\\mbox{ kg B}}{h} ](../../../upload.wikimedia.org/math/4/c/f/4cf94848aa3a89d8acb7a9e8d7168cd8.png)
  * ![ m_{C,gen} = +Xc*MW_C = 22173 \\frac{\\mbox{ mol C}}{h} * 0.030 \\frac{kg}{\\mbox{ mol C}} = +665.2\\frac{\\mbox{ kg C}}{h} ](../../../upload.wikimedia.org/math/1/b/4/1b4f88deb1462784b4999d39840a73d4.png)

At this point you may want to calculate the amount of B and C leaving the reactor with the mass balances on B and C:

  * ![ 584 + 0.7*x_{B5}*2280 - 576.5 = x_{B3}*2380 ](../../../upload.wikimedia.org/math/1/1/0/110f43c17ae6d6116b0f34e0deadea61.png)
  * (1) ![ 0.7*\(1-0.1544-x_{B5}\)*2280 + 665.2 = \(1-0.15-x_{B3}\)*2380 ](../../../upload.wikimedia.org/math/a/0/c/a0c8dc6bf8e9d5130a8694f660b33225.png)

However, _these equations are exactly the same!_ Therefore, we have proven our assertion that there is still 1 DOF in the reactor. So we need to look elsewhere for something to calculate xB5. That place is the separator balance on B:

  * ![ \\dot{m}_3*x_{B3} = \\dot{m}_4*x_{B4} + \\dot{m}_5*x_{B5} ](../../../upload.wikimedia.org/math/7/a/3/7a3e59df1d125a6d45b3a112bbb2a53b.png)
  * (2) ![ 2380x_{B3} = 0.02\(100\) + 2280x_{B5}](../../../upload.wikimedia.org/math/a/2/3/a23b4eee201ac0aeff65b27049a15225.png)

Solving these two equations (1) and (2) yields the final two variables in the system:

  
![ x_{B3} = 0.00856, x_{B5} = 0.008058 ](../../../upload.wikimedia.org/math/8/f/0/8f014a35559e6170ca893962cab76ece.png)

Note that this means the predominant species in stream 5 is also C (![ x_{C5} = 0.838 ](../../../upload.wikimedia.org/math/6/1/5/615decfd173eee173f5b0220d45a2dd9.png)). However, the separator/recycle setup **does** make a big difference, as we'll see next.

### Comparison to the situation without the separator/recycle system

Now that we know how much ethane we can obtain from the reactor after separating, let's compare to what would happen without any of the recycle systems in place. With the same data as in the first part of this problem, the new flowchart looks like this:

![Reactor with NO Recycle.PNG](../../../upload.wikimedia.org/wikibooks/en/c/ca/Reactor_with_NO_Recycle.PNG)

There are three unknowns (![ \\dot{m}_3, x_{B3}, X ](../../../upload.wikimedia.org/math/6/c/d/6cdc6f82d6c8a31dfe0cd37dd672aea0.png)) and three independent material balances, so the problem can be solved. Starting with an overall mass balance because total mass is conserved:

  * ![ \\dot{m}_1 + \\dot{m}_2 = \\dot{m}_3 ](../../../upload.wikimedia.org/math/d/3/6/d36f0ab70194a88adf88b47338573a5b.png)
  * ![ \\dot{m}_3 = 789\\frac{kg}{h} ](../../../upload.wikimedia.org/math/4/6/2/4620d28fbb048d1db1170a28dbd68399.png)

We can carry out the same sort of analysis on the reactor as we did in the previous section to find the conversion and mass percent of C in the exit stream, which is left as an exercise to the reader. The result is that:

  * ![ X = 20250\\mbox{ moles}, x_{C3} = 0.77 ](../../../upload.wikimedia.org/math/2/a/d/2adc5f806ad6cfcd08bd8bb1fe621282.png)

Compare this to the two exit streams in the recycle setup. **Both of the streams were richer in C than 77%, even the reject stream**. This occurred because the unreacted A and B was allowed to re-enter the reactor and form more C, and the separator was able to separate almost all the C that formed from the unreacted A and B.

## The concept of atom balances

Let's begin this section by looking at the reaction of hydrogen with oxygen to form water:

![ H_2 + O_2 \\rightarrow H_2O ](../../../upload.wikimedia.org/math/6/1/6/616bd5351f4d9555f54892443e035631.png)

We may attempt to do our calculations with this reaction, but there is something seriously wrong with this equation! It is not _balanced_; as written, it implies that an atom of oxygen is somehow "lost" in the reaction, but this is in general impossible. Therefore, we must compensate by writing:

![ H_2 + \\frac{1}{2}O_2 \\rightarrow H_2O ](../../../upload.wikimedia.org/math/f/d/4/fd4002de0adb8225de64cfcf5c196f4d.png)

or some multiple thereof.

Notice that in doing this we have made use of the following conservation law, which is actually the basis of the conservation of mass:

  
**The number of atoms of any given element does not change in any reaction** (assuming that it is not a nuclear reaction).

Since by definition the number of moles of an element is proportional to the number of atoms, this implies that ![ \\dot{n}_{A,gen} = 0 ](../../../upload.wikimedia.org/math/8/9/e/89ee5ecfa94e36e6029d0a08b2286fbd.png) where A represents any _element_ in atomic form.

## Mathematical formulation of the atom balance

Now recall the general balance equation:

![ In - Out + Generation = Accumulation ](../../../upload.wikimedia.org/math/0/b/1/0b1844235a31438006e40205686b0ce8.png)

In this course we're assuming ![ Accumulation = 0 ](../../../upload.wikimedia.org/math/8/6/5/865ee08f43c321adbb413b6d6385a09a.png). Since the _moles of atoms_ of any element are conserved, ![ generation = 0 ](../../../upload.wikimedia.org/math/e/8/a/e8a946e59bf7226fea324fee699b35d9.png). So we have the following balance on a given element A:

  
For a given element A,

![ \\Sigma \\dot{n}_{A,in} - \\Sigma \\dot{n}_{A,out} = 0 ](../../../upload.wikimedia.org/math/7/7/c/77cad51799e461282e5425e49053aefa.png)

**Note:**  
When analyzing a reacting system you must choose _either_ an atom balance _or_ a molecular species balance but **not both**. Each has advantages; an atom balance often yields simpler algebra (especially for multiple reactions; the actual reaction that takes place is irrelevant!) but also will not directly tell you the extent(s) of reaction, and will not tell you if the system specifications are actually impossible to achieve for a given set of equilibrium reactions.

## Degree of Freedom Analysis for the atom balance

As before, to do a degree of freedom analysis, it is necessary to count the number of unknowns and the number of equations one can write, and then subtract them. However, there are a couple of important things to be aware of with these balances.

  * **When doing atom balances, the extent of reaction does not count as an unknown, while with a molecular species balance it does**. This is the primary advantage of this method: the extent of reaction does not matter since atoms of elements are conserved regardless of how far the reaction has proceeded.
  * You need to make sure each atom balance will be **independent**. This is difficult to tell unless you write out the equations and look to see if any two are identical.
  * In reactions with **inert species**, each molecular balance on the inert species counts as an _additional equation_. This is because of the following important note:

**Note:**  
When you're doing an atom balance you should **only include reactive species, not inerts**.

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Suppose a mixture of nitrous oxide (![ N_2O ](../../../upload.wikimedia.org/math/2/3/9/239600df75ec65d96088ccc156cd8b06.png)) and oxygen is used in a natural gas burner. The reaction ![ CH_4 + 2O_2 \\rightarrow 2H_2O + CO_2 ](../../../upload.wikimedia.org/math/0/e/7/0e78bee37d5050d09e1c8a9963081a62.png) occurs in it.

There would be _four_ equations that you could write: 3 atom balances (C, H, and O) and a molecular balance on nitrous oxide. You would _not_ include the moles of nitrous oxide in the atom balance on oxygen.

## Example of the use of the atom balance

Let's re-examine a problem from the previous section. In that section it was solved using a molecular species balance, while here it will be solved using atom balances.

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Consider the reaction of Phosphene with oxygen: ![ 4PH_3+8O_2 \\rightarrow P_4O_{10} + 6H_2O ](../../../upload.wikimedia.org/math/4/5/c/45c1fead4a7f69d892bdd4ffcf14ec74.png)

Suppose a 100-kg mixture of 50% ![ PH_3 ](../../../upload.wikimedia.org/math/a/b/0/ab0d26fb39998de2680fe410f54b8c70.png) and 50% ![ O_2 ](../../../upload.wikimedia.org/math/6/1/2/612275025e7d8f5e663bf9355bd422b4.png) by mass enters a reactor in a single stream, and the single exit stream contains 25% ![ O_2 ](../../../upload.wikimedia.org/math/6/1/2/612275025e7d8f5e663bf9355bd422b4.png) by mass. Assume that all the reduction in oxygen occurs due to the reaction. How many degrees of freedom does this problem have? If possible, determine mass composition of all the products.

For purposes of examination, the flowchart is re-displayed here:

![Phosphate Rxn Flowchart.PNG](../../../upload.wikimedia.org/wikibooks/en/4/43/Phosphate_Rxn_Flowchart.PNG)

  


### Degree of Freedom Analysis

There are three elements involved in the system (P, H, and O) so we can write three atom balances on the system.

There are likewise **three** unknowns (since the extent of reaction is NOT an unknown when using the atom balance): the outlet concentrations of ![ PH_3, P_4O_{10}, H_2O ](../../../upload.wikimedia.org/math/f/c/2/fc2d4d7d017b6ccacefe9bb07000214e.png)

Therefore, there are 3 - 3 = 0 unknowns.

### Problem Solution

Let's start the same as we did in the previous section: by finding converting the given information into moles. The calculations of the previous section are repeated here:

  * ![ \\dot{m}_{out} = \\dot{m}_{in} = 100 \\mbox{ kg} ](../../../upload.wikimedia.org/math/6/4/0/64051a511c8b4f0d67330e7ef44e1d00.png)
  * ![ \\dot{n}_{PH_3,in}= 0.5*\(100\\mbox{ kg}\)* \\frac{1 \\mbox{ mol}}{0.034 \\mbox{ kg}} = 1470.6 \\mbox{ moles PH}_3 \\mbox{ in} ](../../../upload.wikimedia.org/math/2/f/5/2f57c7cffc3d7955d64419ff5aad4292.png)
  * ![ \\dot{n}_{O_2,in} = 0.5*\(100\\mbox{ kg}\)* \\frac{1 \\mbox{ mol}}{0.032 \\mbox{ kg}} = 1562.5 \\mbox{ moles O}_2 \\mbox{ in} ](../../../upload.wikimedia.org/math/2/d/0/2d018e90983d9301e35aa9f88153b166.png)
  * ![ \\dot{n}_{O_2,out} = 0.25*\(100\\mbox{ kg}\)* \\frac{1 \\mbox{ mol}}{0.032 \\mbox{ kg}} = 781.25 \\mbox{ moles O}_2 \\mbox{ out} ](../../../upload.wikimedia.org/math/9/0/3/9039130f765a878e4a795b514d197109.png)

Now we start to _diverge_ from the path of molecular balances and instead write atom balances on each of the elements in the reaction. Let's start with Phosphorus. How many moles of Phosphorus _atoms_ are entering?

  * Inlet: Only ![ PH_3 ](../../../upload.wikimedia.org/math/a/b/0/ab0d26fb39998de2680fe410f54b8c70.png) provides P, so the inlet moles of P are just ![ 1*1470.6 = 1470.6 \\mbox{ moles P in} ](../../../upload.wikimedia.org/math/1/2/c/12c6421b04fa5039404d20652e90cad3.png)
  * Outlet: There are _two_ ways phosphorus leaves: as unused ![ PH_3 ](../../../upload.wikimedia.org/math/a/b/0/ab0d26fb39998de2680fe410f54b8c70.png) or as the product ![ P_4O_{10} ](../../../upload.wikimedia.org/math/a/d/8/ad8a8c9a8ec8361155fa89929983eff1.png). Therefore, the moles of ![ PH_3 ](../../../upload.wikimedia.org/math/a/b/0/ab0d26fb39998de2680fe410f54b8c70.png) out are ![ 1*n_{PH_3,out} + 4*n_{P_4O_{10},out} ](../../../upload.wikimedia.org/math/2/f/4/2f42702cc2906a972dbf462c75c579d7.png). Note that the 4 in this equation comes from the fact that there are 4 Phosphorus atoms in every mole of ![ P_4O_{10} ](../../../upload.wikimedia.org/math/a/d/8/ad8a8c9a8ec8361155fa89929983eff1.png).

Therefore the atom balance on Phosphorus becomes:

  
**Phosphorus**

![ 1*n_{PH_3,out} + 4*n_{P_4O_{10},out} = 1470.6 ](../../../upload.wikimedia.org/math/b/b/a/bba395992d47726ce3a219cfe762723d.png)

Similarly, on Oxygen we have:

  * Inlet: ![ 2*n_{O_2,in} = 2*1562.5 = 3125\\mbox{ moles O}_2](../../../upload.wikimedia.org/math/2/1/b/21b423ad722eb561097cae3f4150e9bf.png)
  * Outlet: ![ 2*n_{O_2,out} + 10*n_{P_4O_{10},out} + 1*n_{H_2O,out} = 1562.5 + 10*n_{P_4O_{10},out} + 1*n_{H_2O,out} ](../../../upload.wikimedia.org/math/9/6/8/9681ef332bfde5cd6060a04dfbee74b3.png)

  
**Oxygen**

![ 1562.5 + 10*n_{P_4O_{10},out} + 1*n_{H_2O,out}= 3125 ](../../../upload.wikimedia.org/math/1/2/b/12bfcd23b9fd5aa244dcba53113c67e7.png)

Finally, check to see if you can get the following Hydrogen balance as a practice problem:

  
**Hydrogen**

![ 2*n_{H_2O,out} + 3*n_{PH_3,out} = 4411.8 ](../../../upload.wikimedia.org/math/6/5/6/6562ce85b49b2fde6bde8061d2e0db13.png)

Solving these three linear equations, the solutions are:

  
![ n_{PH_3,out}=1080 , n_{O_2,out}=586, n_{P_4O_{10},out} = 97.66 ](../../../upload.wikimedia.org/math/4/1/a/41a8a837eee6a749a21a216fa2c4a14f.png) moles

All of these answers are identical to those obtained using extents of reaction. Since the remainder of the solution to that problem is identical to that in the previous section, the reader is referred there for its completion.

## Example of balances with inert species

Sometimes it's more difficult to choose which type of balance you want, because both are possible but one is significantly easier than the other. As an example, lets consider a basic pollution control system.

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Suppose that you are running a power plant and your burner releases a lot of pollutants into the air. The flue gas has been analyzed to contain 5% ![ SO_2 ](../../../upload.wikimedia.org/math/2/1/d/21d290c484fb74bf9b2cc46cb73e8fae.png), 3% ![ NO_2 ](../../../upload.wikimedia.org/math/c/e/4/ce4d8406606c5041f69a6358f4e2c0ac.png), 7% ![ O_2 ](../../../upload.wikimedia.org/math/6/1/2/612275025e7d8f5e663bf9355bd422b4.png) and 15% ![ CO_2 ](../../../upload.wikimedia.org/math/6/f/8/6f8247a9b3f148deefbcd1976e311ca1.png) by moles. The remainder was determined to be inert.

Local regulations require that the emissions of sulfur dioxide be less than 200 ppm (by moles) from your plant. They also require you to reduce nitrogen dioxide emissions to less than 50 ppm. You decide that the most economical method for control of these for your plant is to utilize ammonia-based processes. The proposed system is as follows:

  1. Put the flue gas through a _denitrification_ system, into which (pure) ammonia is pumped. The amount of ammonia pumped in is three times as much as would theoretically be needed to use all of the nitrogen dioxide in the flue gas.
  2. Allow it to react a specified amount of time.
  3. Pump it into a _desulfurization_ system. Nothing new is injected here, it just has a different catalyst than the denitrification, and the substrates are at a different temperature and pressure.

The reactions that occur are:

  1. ![ 2NO_2 + 4NH_3 + O_2 \\rightarrow 3 N_2 + 6H_2O ](../../../upload.wikimedia.org/math/e/1/9/e192a524ebbd39d0f69c365c66fe9183.png)
  2. ![ H_2O + 2NH_3 + SO_2 \\rightarrow \(NH_4\)_2SO_3 ](../../../upload.wikimedia.org/math/4/1/1/411345fdcbf41b4a1074c7f86b5563c6.png)

If your plant makes ![ 130\\frac{ft^3}{s} ](../../../upload.wikimedia.org/math/4/5/4/454617af95bc25aaadbdd09330027492.png) of flue gas at ![ T = 900K ](../../../upload.wikimedia.org/math/b/b/6/bb66f365a571021b28636d9d2f4aabf7.png) and ![ P = 2\\mbox{ atm} ](../../../upload.wikimedia.org/math/0/a/6/0a6072ea4d17aa33ee153588df4839bf.png), how must ammonia do you need to purchase for each 8-hour shift? How much of it remains unused? Why do we want to have a significant amount of excess ammonia?

Assume that the flue gas is an ideal gas. Recall the ideal gas law, ![ PV = nRT ](../../../upload.wikimedia.org/math/e/0/4/e0412dbb6e50de76a383b8f6e7d81dce.png), where ![ R = 0.0821 \\frac{L*atm}{mol*K} ](../../../upload.wikimedia.org/math/8/a/7/8a789b06f9012bc1090744f1cb58ea19.png).

### Step 1: Flowchart

Flowcharts are becoming especially important now as means of organizing all of that information!

![Air Pollution Example.PNG](../../../upload.wikimedia.org/wikibooks/en/2/2e/Air_Pollution_Example.PNG)

### Step 2: Degrees of Freedom

Let's consider an atomic balance on each reactor.

  * Denitrification system: 9 unknowns (all concentrations in stream 3, and ![ \\dot{n}_2 ](../../../upload.wikimedia.org/math/5/f/6/5f69f686d690928c6d12f3dfdd6b348c.png).) - 3 atom balances (N, H, and O) - 3 inert species (![ CO_2, SO_2, inerts ](../../../upload.wikimedia.org/math/5/0/d/50dc182e8a892ce1b938841b70fad550.png)) - 1 additional info (3X stoichiometric feed) = **2 DOF**
  * Desulfurization system: 15 unknowns - 4 atom balances (N, H, O, and S) - 5 inerts (![ CO_2, O_2, NO_2, N_2, inerts ](../../../upload.wikimedia.org/math/b/1/b/b1bc8054a9cc966e737c4961847b0d71.png)) = **6 DOF**
  * Total = 2 + 6 - 8 shared = **0 DOF**, hence the problem has a unique solution.

We can also perform the same type of analysis on molecular balances.

  * Denitrification system: 10 unknowns (now the conversion ![ X_1 ](../../../upload.wikimedia.org/math/c/2/6/c264bdb22a754961ef6021b6c1914161.png) is also unknown) - 8 molecular species balances - 1 additional info = **1 DOF**
  * Desulfurization system: 16 unknowns (now the conversion ![ X_2 ](../../../upload.wikimedia.org/math/c/5/5/c5550be7b87cf24698427e4bdb08d21b.png) is unknown) - 9 balances = **7 DOF**.
  * Total = 1 + 7 - 8 shared = **0 DOF**.

Therefore the problem is theoretically solvable by both methods.

### Step 3: Units

The only weird units in this problem (everything is given in moles already so no need to convert) are in the volumetric flowrate, which is given in ![ \\frac{ft^3}{s} ](../../../upload.wikimedia.org/math/1/7/a/17aeabef3e295d492d5fb1680516309c.png). Lets convert this to ![ \\frac{moles}{s} ](../../../upload.wikimedia.org/math/8/c/0/8c02a9615355799ca21106ab24e24139.png) using the ideal gas law. To use the law with the given value of R is is necessary to change the flowrate to units of ![ \\frac{L}{s} ](../../../upload.wikimedia.org/math/b/3/f/b3f156af2dd4f20105bb4469571c7506.png):

![ 130 \\frac{ft^3}{s} * \\frac{28.317\\mbox{ L}}{ft^3} = 3681.2\\frac{L}{s} ](../../../upload.wikimedia.org/math/4/9/6/4963f99f541ee3bacb0fece6c49d5990.png) ![ P\\dot{V} = \\dot{n}RT \\rightarrow 2*3681.2 = \\dot{n}_1\(0.0821\)\(900\) ](../../../upload.wikimedia.org/math/6/1/0/610a63829d1507385acbcf20e9f5ff46.png)

![ \\dot{n}_1 = 99.64\\frac{moles}{s} ](../../../upload.wikimedia.org/math/0/8/4/0841729fb019e3c3b2ac9477fdc66f05.png)

Now that everything is in good units we can move on to the next step.

### Step 4: Devise a plan

We can first determine the value of ![ \\dot{n}_2 ](../../../upload.wikimedia.org/math/5/f/6/5f69f686d690928c6d12f3dfdd6b348c.png) using the additional information. Then, we should look to an _overall system balance_.

Since none of the individual reactors is completely solvable by itself, it is necessary to look to combinations of processes to solve the problem. **The best way to do an overall system balance with multiple reactions is to treat the entire system as if it was a single reactor in which multiple reactions were occurring**. In this case, the flowchart will be revised to look like this:

![Air Pollution Example Overall System.PNG](../../../upload.wikimedia.org/wikibooks/en/4/4b/Air_Pollution_Example_Overall_System.PNG)

Before we try solving anything, we should check to make sure that we still have no degrees of freedom.

**Atom Balance**

There are 8 unknowns (don't count conversions when doing atom balances), 4 types of atoms (H, N, O, and S), 2 species that never react, and 1 additional piece of information (3X stoichiometric), so there is **1 DOF**. This is obviously a problem, which occurs because **when performing atom balances you cannot distinguish between species that react in only ONE reaction and those that take part in more than one**.

In this case, then, it is necessary to look to molecular-species balances.

**Molecular-species balance**

In this case, there are 10 unknowns, but we can do molecular species balances on 9 species ![ \(SO_2, NO_2, NH_3, N_2, O_2, CO_2, H_2O, \(NH_4\)_2SO_3, inerts\)](../../../upload.wikimedia.org/math/c/7/b/c7bb3a9586c45b973ae0bb2bf17de2bb.png) and have the additional information, so there are **0 DOF** when using this method.

Once we have all this information, getting the information about stream 3 is trivial from the definition of extent of reaction.

### Step 5: Carry Out the Plan

First off we can determine ![ \\dot{n}_2 ](../../../upload.wikimedia.org/math/5/f/6/5f69f686d690928c6d12f3dfdd6b348c.png) by using the definition of a stoichiometric feed.

![ \\dot{n}_{NO_2,in} = 0.03*99.64 = 2.9892 \\frac{mol}{s} ](../../../upload.wikimedia.org/math/6/f/1/6f1ff66aa3c11995984d93cc89886860.png)

The stoichiometric amount of ammonia needed to react with this is, from the reaction,

![ \\frac{4 \\mbox{ moles NH}_3}{2 \\mbox{ moles NO}_2} * 2.9892 = 5.96 \\frac{ moles \\mbox{ NH}_3}{s} ](../../../upload.wikimedia.org/math/4/f/5/4f55c083022a0033ec6659525ab5b4f9.png)

Since the problem states that three times this amount is injected into the denitrification system, we have:

  
![ \\dot{n}_2 = 17.88 \\frac{moles}{s} ](../../../upload.wikimedia.org/math/a/c/9/ac95a77bdc1b253a1b79ca1cafcf416b.png)

Now, we are going to have a very complex system of equations with the 9 molecular balances. This may be a good time to invest in some equation-solving software.

See if you can derive the following system of equations from the overall-system flowchart above.

  
![ NH_3: \\dot{n}_4*x_{NH_3,4} = 17.88 - 4*X_1 - 2*X_2 ](../../../upload.wikimedia.org/math/b/d/1/bd18f0603c2a9c7cce942d48758c9f26.png)  
![ SO_2: \\dot{n}_4*2*10^{-4} = 0.05 * 99.64 - X_2 ](../../../upload.wikimedia.org/math/e/e/1/ee1ae7c3259cfe567b59f3f7c6f5bb63.png)  
![ NO_2: \\dot{n}_4*5*10^{-5} = 0.03*99.64 - 2*X_1 ](../../../upload.wikimedia.org/math/d/5/6/d56ab90edd6bbc85685d112682e31930.png)  
![ N_2: \\dot{n}_4*x_{N_2,4} = 3*X_1 ](../../../upload.wikimedia.org/math/f/e/e/fee592f648a43edd1357945bfed77434.png)  
![ O_2: \\dot{n}_4*x_{O_2,4} = 0.07*99.64 - X_1 ](../../../upload.wikimedia.org/math/2/5/0/250c38ca7ff6de6cdf28f1b62ee76872.png)  
![ H_2O: \\dot{n}_4*x_{H_2O,4} = 6*X_1 - X_2 ](../../../upload.wikimedia.org/math/b/9/2/b92c0530d99ec5c0307ccbe19ef8a8d9.png)  
![ CO_2: \\dot{n}_4*x_{CO_2,4} = 0.15*99.64 ](../../../upload.wikimedia.org/math/6/d/5/6d532eef119f6bb6783863e23d0cff6c.png)  
![ \(NH_4\)_2\(SO_3\): \\dot{n}_4*x_{\(NH_4\)_2SO_3,4} = X_2 ](../../../upload.wikimedia.org/math/b/9/7/b97737dca7192fc4a323b03dd3df075f.png)  
![ Inerts: \\dot{n}_4*\(1 - 2*10^{-4} - 5*10^{-5} - x_{NH_3,4} - x_{N_2,4} - x_{O_2,4} - x_{H_2O,4} -  x_{CO_2,4} - x_{\(NH_4\)_2SO_3,4} \)  = 0.7*99.64 ](../../../upload.wikimedia.org/math/8/8/c/88c7696f4788d339722c10f4b2c97014.png)

Using an equation-solving package, the following results were obtained:

  
![ X_1 = 1.492\\mbox{ moles} ](../../../upload.wikimedia.org/math/5/e/6/5e6ab32e48e8e1626373e16192e23a71.png)  
![ X_2 = 4.961\\mbox{ moles} ](../../../upload.wikimedia.org/math/8/a/c/8ac545fb2d753b15c1eb1fd3b6950ec9.png)  
![ \\dot{n}_3 = 105.62 \\frac{mol}{s} ](../../../upload.wikimedia.org/math/0/5/8/05894ee67a3aab65031d0482b6403fd8.png)  
![ x_{NH_3, 4} = 0.01884 ](../../../upload.wikimedia.org/math/9/b/c/9bc77f4a305f1b73bf6d87359ef2ba2f.png)  
![ x_{N_2,4} = 0.04238 ](../../../upload.wikimedia.org/math/e/0/5/e050d86012cd3b9213b7cd65637a3106.png)  
![ x_{O_2,4} = 0.05191 ](../../../upload.wikimedia.org/math/e/9/b/e9b80956b5f5e7f059dbcb75db7aa738.png)  
![ x_{H_2O,4} = 0.03778 ](../../../upload.wikimedia.org/math/d/c/c/dcc270be3d0ac8fffb92446711a84320.png)  
![ x_{CO_2,4} = 0.1415 ](../../../upload.wikimedia.org/math/2/d/d/2dd9168b320b7133ea4464ff1186726d.png)  
![ x_{\(NH_4\)_2SO_3,4} = 0.04697 ](../../../upload.wikimedia.org/math/3/d/6/3d63976b255b1d5a4fdb16c0b8d29296.png)  
![ x_{I} = 1 - \\Sigma \\mbox{ \(other components\) } = 0.6606 ](../../../upload.wikimedia.org/math/f/2/3/f238f58cb0c35ce865e3dd0891ed46e2.png)

#### Stream 3

Now that we have completely specified the composition of stream 4, it is possible to go back and find the compositions of stream 3 using the extents of reaction and feed composition. Although this is not necessary to answer the problem statement, it should be done, so that we can then test to make sure that all of the numbers we have obtained are consistent.

![Clipboard](../../../upload.wikimedia.org/wikipedia/commons/thumb/1/1f/Clipboard.svg/45px-Clipboard.svg.png)

**To do:**  
finish this

# **Chapter 6: Multiple-phase systems, introduction to phase equilibrium**

## What IS an ideal gas?

Recall from general chemistry that the volume, pressure, temperature, and moles of a gas in a closed system can be related by the following equation, which is referred to as the **ideal gas law**:

  
![ PV = nRT ](../../../upload.wikimedia.org/math/e/0/4/e0412dbb6e50de76a383b8f6e7d81dce.png)

R is referred to as the **Universal Gas Constant**, and it has the following values for different units of P, V, n, and T:

![ R = 0.0821 \\frac{L*atm}{mol*K} = 8.31 \\frac{J}{mol*K} = 8.31 \\frac{Pa*m^3}{mol*K} ](../../../upload.wikimedia.org/math/f/c/6/fc657ac74d078d49104125a883194ba4.png)

One thing that may have been de-emphasized in an introductory chemistry course is the fact that _gasses do not always follow this law_. In fact, they only do under very special circumstances.

### Theoretical background on the ideal gas law

The ideal gas law relies on several rather strong assumptions about the nature of gasses, which make up the classical [Kinetic Theory of Gases](http://en.wikibooks.org/wiki/General_Chemistry/Kinetic_Theory_of_Gases). These assumptions are:

  1. That gas molecules do not interact with each other whatsoever.
  2. That all collisions between molecules and each other or with the walls of the container are completely **elastic**, meaning the mean kinetic energy of the molecules stays the same.
  3. That gas molecules are very small compared to the distance between them.

There are several other assumptions as well, but these will suffice to explain why deviations occur.

### Important facts about ideal gasses

Ideal gasses are nice because they have several properties that make them relatively easy and useful to work with:

  1. A mixture of ideal gasses is also an ideal gas. Therefore, you can use the ideal gas law on the entire mixture or on any of its components without loss of validity.
  2. The **Partial Pressure** of a component in an ideal gas mixture is related to the total pressure by the equation ![ P_A = x_A*P](../../../upload.wikimedia.org/math/3/5/f/35f961ea97bca52ac8033d584718e026.png) where ![ x_A ](../../../upload.wikimedia.org/math/3/c/4/3c430fc59ecfcc331a864e17196f7c3b.png) is the mole fraction of component A in the mixture.
  3. The ideal gas equation, if it is valid, is independent of the properties of the gas. Therefore, there is no need to look up or measure gas-specific parameters, all you need to know is 3 of the unknowns (P,V,T, and n) and you can solve for the fourth one.
  4. The enthalpy and entropy of an ideal gas only depend on temperature (not pressure or volume).
  5. Many gasses are close to ideal at low pressures and high temperatures. Therefore it can be used as a realistic **reference state** to which a real gas can be compared.

There are many other useful properties of ideal gasses that will be discussed in thermodynamics.

### Deviations at high pressure

Suppose that you have water vapor in a small water bottle. What happens when you apply pressure to that vapor by shrinking the volume? If you apply more pressure to the bottle, the gas molecules within become closer and closer together. However, the closer molecules of a substance are to one another, the larger the [dispersion forces](http://en.wikipedia.org/wiki/dispersion_forces) (and polar forces, if applicable) are between them. Eventually, the dispersion forces become significant, and the kinetic theory of gasses as stated above is no longer valid. Therefore, the ideal gas law no longer applies, or becomes a rough estimate at best.

At very high pressures, a vapor might even condense into a liquid, which would also invalidate the use of the law.

### Deviations at low temperature

By definition of temperature, when the temperature of a substance decreases, this indicates a lower average kinetic energy of the molecules of that substance. Molecules with less kinetic energy also have less momentum, and therefore are more easily swayed by dispersion forces from other molecules and by gravity. Eventually, the temperature might become low enough that the forces from other molecules cause significant deviations in the path of a molecule, and in this case the ideal gas law becomes less valid.

As with pressure, a very low temperature can cause a gas to condense.

### Rule of thumb for use of the ideal gas law

Due to the above two discussions, we can claim that **the ideal gas law is a decent assumption at high temperatures and low pressures, but should not be used outside that realm**.

## The Idea of Equations of State

The ideal gas law is not the only way to express relationships between system properties. There are an infinite number of possible ways that you _could_ propose to correlate system variables, although thermodynamics provides some guidelines regarding how many variables to correlate and what types of correlations we should look for. In particular, it is most useful to correlate relationships between **[state variables](http://en.wikipedia.org/wiki/state_variables)**, for which changes in the properties do not depend on how the change occurred.

Any equation that relates state variables is called an **[equation of state](http://en.wikipedia.org/wiki/equation_of_state)**. The most commonly-used equations of state relate the variables P, T, V, and n (pressure, temperature, volume, and number of moles) since they are all measurable variables whereas many other possible variables are not directly measurable (such as [enthalpy](http://en.wikipedia.org/wiki/enthalpy), which will be discussed later).

## Compressibility

There is a highly useful quantity for describing how much a gaseous system deviates from ideality, which is called the **compressibility** of the gas. The compressibility Z is defined as:

  
![ Z = \\frac{P*V}{n*R*T} ](../../../upload.wikimedia.org/math/c/8/c/c8ccae211713c61f6f77d2241b58dd93.png)

For an ideal gas, since ![ PV = nRT ](../../../upload.wikimedia.org/math/e/0/4/e0412dbb6e50de76a383b8f6e7d81dce.png), ![ Z = 1 ](../../../upload.wikimedia.org/math/8/9/3/89359dedd2737c9352c958ab6bd7ba2e.png). Therefore, **any deviation of the compressibility from 1 is a nonideality**.

The compressibility of a liquid is very small, due to a small volume per mole of substance compared to that of a gas.

### Ideal Gas Law Extension

One use of the compressibility is that it allows a simple extension of the ideal gas law, which is completely general.

  
**Real-Gas Extension to the Ideal Gas Law**

![ PV = nRTZ ](../../../upload.wikimedia.org/math/5/0/6/50651406a4ba742cd8a169aef7e3366a.png)

Unfortunately, Z is not a constant for any material, but changes with pressure and temperature. However, in a later section you will learn a technique called the **generalized compressibility method** with which you can estimate the value of Z for _any_ substance, given certain data. Once it is known, this extension can be used to calculate an unknown system property.

## Alternatives to the ideal gas law 1: Van der Waals Equation

One of the oldest equations that was developed to take non-ideality into account is called the **[Van der Waals Equation](http://en.wikipedia.org/wiki/Van_der_Waals_equation)**, which has two substance-dependent parameters. One takes into account the interactions between particles, and the other the fact that particles have volume, sometimes substantial.

The Van der Waals equation is as follows:

  
![\\left\(p + \\frac{n^2 a}{V^2}\\right\)\\left\(V-nb\\right\) = nRT](../../../upload.wikimedia.org/math/f/0/2/f028b4bc100cbd2c79e01bcfb80493aa.png) where a and b depend on the gas being analyzed

A list of values for a and b can be found on Wikipedia at [this page](http://en.wikipedia.org/wiki/Van_der_Waals_constants_\(data_page\)).

The Van der Waals equation is significantly more accurate than the ideal gas law and can be used to crudely predict when a gas will condense. However, it is not sufficiently accurate for many industrial purposes, and therefore other methods have been sought since then.

## Alternatives to the ideal gas law 2: Virial Equation

The Virial equation is an equation which has a potentially infinite number of parameters that depend on the properties of the substances involved. It is important because it can be shown to be a valid extension to the ideal gas law using statistical theories (whereas the other equations of state have been derived semi-empirically).

The virial equation can take several forms, depending on the data one has available. The one in terms of molar volume is:

![Z = \\frac{PV_m}{RT} = 1 + \\frac{B}{V_m} + \\frac{C}{V_m^2} + \\frac{D}{V_m^3} + \\dots](../../../upload.wikimedia.org/math/8/0/3/803f347f6983ad21278863f0f643d3f7.png)

where ![ V_m ](../../../upload.wikimedia.org/math/4/a/0/4a0ecf53c4b5b87fedf521d6e82a3d87.png) is the molar volume, which is the same as ![ \\frac{V}{n} ](../../../upload.wikimedia.org/math/2/3/d/23d0248179a5951845c273f2ced06a05.png)

## Alternatives to the ideal gas law 3: Peng-Robinson equation

One of the more modern equations of state is the **Peng-Robinson** equation, which is most useful for describing nonpolar molecules such as hydrocarbons or nitrogen. The Peng-Robinson equation has two parameters like the Van der Waals equation, but unlike the latter, one of the parameters is not constant; it depends on the temperature of the system as well as the properties of the substance inside.

## Phase Equlibrium

Many processes in chemical engineering do not only involve a single phase but a combination of two immiscible liquids, or a stream containing both gas and liquid. It is very important to recognize and be able to calculate when these phases are in **equilibrium** with each other, and how much is in each phase. This knowledge will be especially useful when you study separation processes, for many of these processes work by somehow distorting the equilibrium so that one phase is especially rich in one component, and the other is rich in the other component.

More specifically, there are three important criteria for different phases to be in equilibrium with each other:

  1. The **temperature** of the two phases is the same at equilibrium.
  2. The **partial pressure** of every component in the two phases is the same at equilibrium.
  3. The **Gibbs free energy'** of every component in the two phases is the same at equilibrium.

The third criteria will be explored in more depth in another course; it is a consequence of the first two criteria and the second law of thermodynamics.

## Single-Component Phase Equilibrium

If there is only a single component in a mixture, there is only a single possible temperature (at a given pressure) for which phase equilibrium is possible. For example, water at standard pressure (1 atm) can _only_ remain in equilibrium at 100°C. Below this temperature, all of the water condenses, and above it, all of the water vaporizes into steam.

At a given temperature, the unique _atmospheric_ pressure at which a pure liquid boils is called its **vapor pressure**. If the atmospheric pressure is higher than the vapor pressure, the liquid will not boil. Vapor pressure is strongly temperature-dependent. Water at 100°C has a vapor pressure of 1 atmosphere, which explains why water on Earth (which has an atmosphere of about 1 atm) boils at 100°C. Water at a temperature of 20°C(a typical room temperature) will only boil at pressures under 0.023 atm, which is its vapor pressure at that temperature.

## Multiple-Component Phase Equilibrium: Phase Diagrams

In general, chemical engineers are not dealing with single components; instead they deal with equilibrium of **mixtures**. When a mixture begins to boil, the vapor does **not**, in general, have the same composition as the liquid. Instead, the substance with the lower boiling temperature (or higher vapor pressure) will have a vapor concentration higher than that with the higher boiling temperature, though **both will be present in the vapor**. A similar argument applies when a vapor mixture condenses.

The concentrations of the vapor and liquid when the _overall_ concentration and one of the temperature or pressure are fixed can easily be read off of a **phase diagram**. In order to read and understand a phase diagram, it is necessary to understand the concepts of _bubble point_ and _dew point_ for a mixture.

### Bubble Point and Dew Point

In order to be able to predict the phase behavior of a mixture, scientists and engineers examine the _limits_ of phase changes, and then utilize the laws of thermodynamics to determine what happens in between those limits. The limits in the case of gas-liquid phase changes are called the **bubble point** and the **dew point**.

The names imply which one is which:

  1. The bubble point is the point at which the first drop of a liquid mixture begins to vaporize.
  2. The dew point is the point at which the first drop of a gaseous mixture begins to condense.

If you are able to plot both the bubble and the dew points on the same graph, you come up with what is called a **Pxy** or a **Txy** diagram, depending on whether it is graphed at constant temperature or constant pressure. The "xy" implies that the curve is able to provide information on both liquid _and_ vapor compositions, as we will see when we examine the thermodynamics in more detail.

#### Txy and Pxy diagrams

The easier of the two diagrams to calculate (but sometimes harder to grasp intuitively) is the Pxy diagram, which is shown below for an idealized Benzene-Toluene system:

![Benztol pxy diagram.PNG](../../../upload.wikimedia.org/wikibooks/en/3/3f/Benztol_pxy_diagram.PNG)

In order to avoid getting confused about what you're looking at, think: what causes a liquid to vaporize? Two things should come to mind:

  * Increasing the temperature
  * _Decreasing_ the pressure

Therefore, the region with the _higher_ pressure is the liquid region, and that of _lower_ pressure is vapor, as labeled. The region in between the curves is called the **two-phase region**.

**Note:**  
You may be tempted to try and memorize something like _the dew point line is on the bottom in a Pxy diagram and on the top in a Txy diagram_. This is, however, strongly discouraged, as you will very likely become confused if you depend on this type of memorizing. Instead think: **which half of the graph will contain liquid and which half will be vapor?** Then use the definitions of "dew" and "bubble" points to determine which line is which.

Now that we have this curve, what can we do with it? There are several critical pieces of information we can gather from this graph by simple techniques, which have complete analogies in the Txy diagram. First, note that the two lines intersect at ![ x_{Benzene} = 0 ](../../../upload.wikimedia.org/math/3/1/7/317045b7c0fe7f2c3cf33528d34d2099.png) and at ![ x_{Benzene} = 1 ](../../../upload.wikimedia.org/math/0/3/c/03cbc74d1f8b4a1c1e7a09b0dede5bbd.png). These intersections are the _pure-component vapor pressures_ at ![ T = 20^oC ](../../../upload.wikimedia.org/math/f/5/7/f57389bbbdc0d87e02bd83aee0b5cc1f.png), since a pure component boils at its vapor pressure.

We can determine, given the mole fraction of one component and a pressure, whether the system is gas, liquid, or two-phase, which is critical information from a design standpoint. For example, if the Benzene composition in the Benzene-Toluene system is 40% and the pressure is 25 mmHg, the entire mixture will be vapor, whereas if the pressure is raised to 50 mmHg it will all condense. The design of a flash evaporator at 20oC would require a pressure between about 30 and 40 mmHg (the 2-phase region).

We can also determine the composition of each component in a 2-phase mixture, if we know the overall composition and the vapor pressure. First, start on the x-axis at the overall composition and go up to the pressure you want to know about. Then from this point, go left until you reach the bubble-point curve to find the _liquid_ composition, and go to the right until you reach the dew-point curve to find the _vapor_ composition. See the below diagram.

![Benztol phase calc example.PNG](../../../upload.wikimedia.org/wikibooks/en/8/87/Benztol_phase_calc_example.PNG)

This method "works" because the pressure must be constant between each phase while the two phases are in equilibrium. The bubble and dew compositions are the only liquid and vapor compositions that are stable at a given pressure and temperature, so the system will tend toward those values.

Another useful rule is the **lever rule** which can be used to calculate the _percentage of all the material_ that is in a given phase (as opposed to the composition of the vapor). The lever rule equation is [[1]](http://www.soton.ac.uk/~pasr1/tielines.htm):

  
![ %Liquid = \\frac{D2}{D1 + D2} ](../../../upload.wikimedia.org/math/0/f/2/0f25457e4bd9033aee91c9ea28c07f30.png)

\- Lever Rule

and therefore,

![ %Vapor = \\frac{D1}{D1 + D2} ](../../../upload.wikimedia.org/math/b/e/0/be0c96ae2976183c31cf41ebc16369c4.png).

The phase whose percent you're calculating is simply the one which you are going towards for the line segment in the numerator; for example, D1 is going from the point of interest to the liquid phase, so if D1 is in the numerator then you're calculating percent of liquid.

Txy diagrams have entirely analogous rules, but just be aware that the graph is "reversed" somewhat in shape. It's somewhat harder to calculate even in an ideal case, requiring an iterative solution, but is more useful for isobaric (constant-pressure) systems and is worth the effort. The extreme ends of the txy diagram are the boiling temperatures of pure toluene (xb = 0) and benzene (xb = 1) at 760 mmHg.

![Benztol txy diagram.PNG](../../../upload.wikimedia.org/wikibooks/en/5/52/Benztol_txy_diagram.PNG)

#### VLE phase diagram summary

To summarize, here's the information you can directly garner from a phase diagram. Many of these can be used for _all_ types of phase diagrams, not just VLE.

  1. You can use it to tell you what phase(s) you are in at a given composition, temperature, and/or pressure.
  2. You can use it to tell you what the _composition_ of each phase will be, if you're in a multiphase region.
  3. You can use it to tell you how _much_ of the original solution is in each phase, if you're in a multiphase region.
  4. You can use it to gather some properties of the _pure materials_ from the endpoints (though these are usually the best-known of all the mixture properties).

This data is invaluable in systems design which is why you'll be drilled with it before you graduate.

## Raoult's Law: the Simplest Case

The simplest case (by far) to analyze occurs when an **ideal solution** is in equilibrium with an ideal gas. This is potentially a good approximation when two very similar liquids (the archetypal example is benzene and toluene) are dissolved in each other. It is also a good approximation for **solvent** properties (NOT solute properties; there is another law for that) when a very small amount of a solute is dissolved.

In an ideal liquid, the pressure exerted by a certain component on the gas is proportional to the vapor pressure of the _pure liquid_. The only thing that may prevent the liquid from exerting this much pressure is the fact that another component is present. Therefore, the partial pressure of the _liquid component_ on the _gas component_ is:

![ P_A = x_A*P_A^* ](../../../upload.wikimedia.org/math/b/6/c/b6cb4dc61d4f679a1a1acdf228b90daf.png)

where ![ P_A^* ](../../../upload.wikimedia.org/math/d/b/d/dbdea97c2cdefc3a272fe83190512c2e.png) is the vapor pressure of pure component A.

**Note:**  
For all VLE equations, ![ x_A ](../../../upload.wikimedia.org/math/3/c/4/3c430fc59ecfcc331a864e17196f7c3b.png) denote a **liquid mole fraction** of component A, and ![ y_A ](../../../upload.wikimedia.org/math/4/3/6/436a0e7bbd07b8a7db0da04b2629e63e.png) is the **vapor mole fraction** </math> of component A.

Recall that the partial pressure of an ideal gas in a mixture is given by:

![ P_A = y_A*P ](../../../upload.wikimedia.org/math/d/5/7/d57ed8558bc694a5068949d71ee9c97c.png)

Therefore, since the partial pressures must be equal at equilibrium, we have the Raoult's Law equation for each component:

  
**Raoult's Law for component A**

![ y_A*P = x_A*P_{A}^* ](../../../upload.wikimedia.org/math/4/f/1/4f1ab349b662f595047b44c00497253b.png)

### Vapor Pressure Correlations

Unfortunately, life isn't that simple even when everything is ideal. Vapor pressure is not by any stretch of the imagination a constant. In fact, it has a very strong dependence on temperature. Therefore, people have spent a good deal of time and energy developing correlations with which to predict the vapor pressure of a given substance at any reasonable temperature.

One of the most successful correlations is called the **Antoine Equation**, which uses three coefficients, A, B, and C, which depend on the substance being analyzed. The Antoine Equation is as follows.

  
**Antoine Equation**

![ log\(P_A^*\) = A - \\frac{B}{T+C} ](../../../upload.wikimedia.org/math/8/3/e/83e5c609475c1c37eece4af9cfd7dc61.png)

**Note:**  
In the external link provided in the appendix, the logarithm is to base 10, T is in degrees Celsius, and P* is in mmHg. Other sources use different forms (for example, natural log or P* in bars) so be wary.

### Bubble Point and Dew Point with Raoult's Law

#### Key concept

When calculating either a bubble point or a dew point, one quantity is key, and this is the _overall composition_, denoted with the letter z. This is to distinguish it from the _single-phase composition_ in either the liquid or the gas phase. It is necessary to distinguish between them because the composition of the two phases will almost always be different at equilibrium.

It is important to remember that the dew and bubble points of a multi-component mixture are _limits_. The bubble point is the point at which a very small amount of the liquid has evaporated - so small, in fact, that in essence, _the liquid phase composition remains the same as the overall composition_. Making this assumption, it is possible to calculate the composition of that single bubble of vapor that has formed.

Similarly, the dew point is the point at which a very small amount of the vapor has condensed, so that _the gas phase composition remains the same as the overall composition_, and thus it is possible to calculate the composition of the single bubble of liquid.

#### Bubble Point

Recall that the **dew point** of a solution is the set of conditions (either a temperature at constant pressure or a pressure at constant temperature) at which the first drops of a vapor mixture begin to condense.

Let us first consider how to calculate the bubble point (at a constant temperature) of a mixture of 2 components A and B, assuming that the mixture follows Raoult's Law under all conditions. To begin, write Raoult's Law for each component in the mixture.

![ y_A*P = x_A*P_{A}^* ](../../../upload.wikimedia.org/math/4/f/1/4f1ab349b662f595047b44c00497253b.png)

![ y_B*P = x_B*P_{B}^* ](../../../upload.wikimedia.org/math/0/d/4/0d4cce82aa971afcff8a4ace1e30888e.png)

We can add these two equations together to yield:

![ P\(y_A + y_B\) = P_A^*x_A + P_B^*x_B ](../../../upload.wikimedia.org/math/9/6/5/9658aa7bccc1684da8f7b7f33ffd80d8.png)

Now since ![ y_A ](../../../upload.wikimedia.org/math/4/3/6/436a0e7bbd07b8a7db0da04b2629e63e.png) and ![ y_B ](../../../upload.wikimedia.org/math/7/c/a/7ca44d6bad0d616eeb511727b8bc7766.png) are mole fractions and A and B are the only components of the mixture, ![ y_A + y_B = 1 ](../../../upload.wikimedia.org/math/6/1/7/617dbf2ca26258bfd5ebecdd474ab681.png). In addition, recall that since we are considering the bubble point, the liquid composition is essentially equal to the overall composition. Therefore, ![ x_A = z_A ](../../../upload.wikimedia.org/math/d/d/7/dd7e625422e3d604d600179f2de8e975.png) and ![ x_B = z_B ](../../../upload.wikimedia.org/math/8/4/8/848efc57ab49608ea356e7d12c3431eb.png).

**Note:**  
This is only true at the bubble point, not in general.

Hence we have the following equation valid at the bubble point for an ideal equilibrium:

  
**Bubble point equation for two components under Raoult's Law**  
![ P = z_A*P_A^* + z_B*P_B^*](../../../upload.wikimedia.org/math/f/9/2/f92609f5e854a913d9c9987e7a10c4a2.png)

Therefore, if the temperature and overall composition are known, the bubble pressure can be determined directly.

If the pressure is held constant and the bubble point temperature is required, it is necessary to calculate the temperature by an iterative method. The temperature dependence is contained in the Antoine equation for vapor pressure of each component. One method to solve for the temperature is to:

  1. Guess a temperature
  2. Use the guess and the Antoine equation to calculate the vapor pressure of each component in the mixture.
  3. Calculate an equilibrium pressure using the bubble point pressure equation.
  4. If the calculated pressure does not equal the known pressure, it is necessary to change the temperature and try again.

This process is ideally suited to spreadsheet functions such as Excel's "goalseek" routine. An example calculation will be shown in the next section.

If there is more than one component, a similar derivation yields the following:

  
**Bubble point equation for multiple components under Raoult's Law**  
![ P = \\Sigma \(z_i * P_i^*\) ](../../../upload.wikimedia.org/math/7/f/8/7f899c79193cdb66dd7811219fee558e.png) (summed over all components i)

#### Dew Point

The Dew Point calculation is similar, although the equation that results from the derivation is somewhat more complex. The starting point is the same: assume that Raoult's Law applies to each component.

![ y_A*P = x_A*P_{A}^* ](../../../upload.wikimedia.org/math/4/f/1/4f1ab349b662f595047b44c00497253b.png)

![ y_B*P = x_B*P_{B}^* ](../../../upload.wikimedia.org/math/0/d/4/0d4cce82aa971afcff8a4ace1e30888e.png)

Now we want to eliminate the liquid compositions in a similar manner to how we eliminated the vapor compositions in the previous derivation. To do this we need to divide by the vapor pressures:

![ \\frac{y_A*P}{P_A^*} = x_A ](../../../upload.wikimedia.org/math/d/2/3/d236698a2adbad06f40b52aa2ef23200.png)

![ \\frac{y_B*P}{P_B^*} = x_B ](../../../upload.wikimedia.org/math/7/6/c/76c24036fd3dd3da491e7075ef1bf90b.png)

Adding the equations and recalling that ![ x_A + x_B = 1 ](../../../upload.wikimedia.org/math/9/a/b/9ab071b82bd8b99bca7134f4dce979ca.png), we have:

![ \\frac{y_A*P}{P_A^*} + \\frac{y_B*P}{P_B^*} = 1 ](../../../upload.wikimedia.org/math/2/5/7/2575e1d6fca141a92c69a634132e106c.png).

Since this is the dew point, the gas-phase composition is essentially the overall composition, and therefore we have the following dew point equation:

  
**Dew Point Equation for Two Components under Raoult's Law**  
![\\frac{1}{P} = \\frac{z_A}{P_A^*} + \\frac{z_B}{P_B^*} ](../../../upload.wikimedia.org/math/3/6/1/36175adb5215adae9a5a60e51eb5c93e.png)

**Note:**  
This is only valid at the dew point, just as the other equation was only valid at the bubble point.

For multiple components, the equation is similarly:

  
**Dew Point Equation for Multiple Components under Raoult's Law**  
![ \\frac{1}{P} = \\Sigma \( \\frac{z_A}{P_A^*} \) ](../../../upload.wikimedia.org/math/e/d/d/edd97ca364f747e84a555e21b8ae5c61.png)

### Phase Diagrams Resulting from Raoult's Law

By holding one variable constant, varying a second, and calculating the other two, it is possible to calculate a phase diagram from Raoult's Law. Typical Pxy and Txy diagrams derived from Raoult's Law were shown in the previous section for the benzene-toluene system.

Diagrams for systems that follow Raoult's Law are relatively "nice"; it can be shown that they will never have **azeotropes**, which would be indicated by intersection of the bubble and dew point lines. In addition, since only one parameter in the equation depends on the temperature (the vapor pressure) and the pressure dependence is explicit, the dew and bubble point lines are relatively easy to calculate.

## Non-ideal VLE

Deviations from Raoult's Law occur because not all solutions are ideal, nor are all gas mixtures. Therefore, methods have been developed in order to take these nonidealities into account.

### Henry's Law

The third non-ideal method, Henry's Law, is especially useful for **dilute solutions**, and states that at very low concentrations, the partial pressure of the dilute component over a liquid mixture is proportional to the concentration:

  
**Henry's Law**  
For a dilute component A, ![ P_A = H_A*x_A ](../../../upload.wikimedia.org/math/4/9/9/499d5032d812342d3fa80793f56f332a.png)  
where ![ H_A ](../../../upload.wikimedia.org/math/e/a/6/ea669d9b37c9834cbf336659951fe60d.png) is a constant and ![ x_A ](../../../upload.wikimedia.org/math/3/c/4/3c430fc59ecfcc331a864e17196f7c3b.png) is the liquid-phase mole fraction of A

This law is very similar to Raoult's Law, except that the proportionality constant is not the pure-component vapor pressure but is empirically determined from VLE data. Like the pure-component vapor pressure, the Henry's constant is dependent on temperature and the nature of component A. Unlike the pure-component vapor pressure, _it also depends on the solvent_, so when utilizing tables of Henry's constants, make sure that the solvents match.

**Note:**  
If Henry's Law applies to one component of a two-component mixture, the other component is often concentrated enough for Raoult's Law to apply to a reasonable approximation. Therefore, for a mixture of components A and B, where A is dilute and B is concentrated, a system similar to the following is common:

![ y_A*P = x_A*H_A ](../../../upload.wikimedia.org/math/5/8/7/5871264574f3818128b7aba996d3c8a2.png)  
![ y_B*P = x_B*P_B^* ](../../../upload.wikimedia.org/math/7/e/0/7e069886710fcc0bd44846f13aa1b81b.png)

Henry's Law constants are generally very small, and are most useful when the concentration is less than 10% (depending on how accurate you want it, the concentration may need to be less than this).

### Excess Gibbs Energy

The other two commonly-used correction parameters, the **activity coefficient** and the **fugacity coefficient**, are based on how non-ideal a given phase is. For a gas, the degree of non-ideality present is called the **residual Gibbs energy**, while for a liquid it is called **excess Gibbs energy**. The distinction is made because the Gibbs energy of an ideal gas and the Gibbs energy of an ideal solution are very different, as are the natures of how real solutions and real gasses deviate from ideality.

The residual Gibbs energy is based on the ideal gas and is defined as follows:

  
**Residual Gibbs Energy definition (for a gas phase)**  
![ G^R = G_{real} - G_{\\mbox{ideal gas}} ](../../../upload.wikimedia.org/math/c/6/e/c6edce7e3e692f546c6ab6bddf4a2558.png)

The **excess Gibbs energy** of a _liquid_ phase is based on an ideal solution and is defined as:

  
**Excess Gibbs Energy definition (for a liquid phase)**  
![ G^E = G_{real} - G_{\\mbox{ideal solution}} ](../../../upload.wikimedia.org/math/c/d/d/cdd3c9ea4392aa5f2412d6b9ecb55f44.png)

### Activity Coefficients

The activity coefficient takes into account variation from Raoult's Law due to **liquid** excess Gibbs energy. It may be defined as:

  
![ln \\gamma_i = \\frac{G^E_i}{RT} ](../../../upload.wikimedia.org/math/f/4/2/f4247c448d8b1d139265b49397b90d40.png)

where ![ \\gamma ](../../../upload.wikimedia.org/math/3/3/4/334de1ea38b615839e4ee6b65ee1b103.png) is a composition-dependent value which is also different for each component. It therefore is a measure of the effect of each component in contributing to the nonideality of the mixture.

Raoult's Law can be extended using activity coefficients as follows:

  
**Extended Raoult's Law**  
![y_A*P = x_A*P_A^* * \\gamma_i](../../../upload.wikimedia.org/math/6/b/7/6b796dfdc8c3dde2a8adb916754bf352.png)

The extended Raoult's law may be used to calculate activity coefficients: the vapor pressure and equilibrium composition are measured at a low pressure (to avoid gaseous nonideality). Then, since the activity coefficient is only weakly dependent on pressure (liquid properties often change very little with pressure), the same values of the activity coefficient may be used at higher pressures to aid in determining the change in equilibrium properties.

Once activity coefficients are determined at a wide variety of concentrations, it is often desired to condense the information into one equation. See [this publication](http://www.iupac.org/publications/pac/1995/pdf/6706x0859.pdf) for an interesting read on this topic, though it will probably make more sense after you take thermodynamics, it offers a good description of what you will see.

**Note:**  
The definition of the activity coefficient implies that an ideal solution will have an activity coefficient equal to 1 (since its excess Gibbs energy is 0). Thus for an ideal solution the equilibrium equation reduces back to Raoult's law.

### Fugacity Coefficients

The **fugacity coefficient** of a gas is defined in a similar way to the activity coefficient, but it is based on the _residual_ Gibbs energy:

  
![ln \\phi_i = \\frac{G^R_i}{RT} ](../../../upload.wikimedia.org/math/5/1/0/510ef10425b548448da9c9f031468c3d.png)

The fugacity coefficient of a gas depends on temperature, as can be seen clearly from the definition. It also depends heavily on the pressure. Indeed, if you have data available that relates the compressibility of a pure gas, Z, as a function of pressure at constant temperature, the fugacity can be computed using calculus or estimated (roughly) using the following equation if the change in pressure between each set of points is constant:

![ ln \\phi_i = \\Delta P\\Sigma\( \\frac{Z_k - 1}{P_k} \) ](../../../upload.wikimedia.org/math/2/7/e/27ec9a75d3f903a77134bb5820bd0761.png)

where "k" referring to a data point

**Note:**  
Since the compressibility (![ Z_k ](../../../upload.wikimedia.org/math/3/e/4/3e4bb5b03a72d6b18b81836fd7ac6d6c.png)) of an ideal gas is 1 regardless of what the pressure is, the fugacity coefficient of an ideal gas is 1 as well. Therefore, like the activity coefficient, the fugacity coefficient provides us with a measure of how nonideal a given gas or mixture of gasses is.

To do this calculation, **it is necessary to extrapolate so that the first data point is taken at P = 0.**.

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Given the following data:
    
    
    P (atm)      Z
    0.1          0.98
    0.2          0.96
    0.3          0.95
    

Calculate the fugacity coefficient at 0.2 atm and 0.3 atm.

**Solution**: It is necessary to first extrapolate to zero pressure:

![ P=0 \\rightarrow Z~1.0 ](../../../upload.wikimedia.org/math/2/1/d/21df5d58d9f98be0b828a1eace1551f3.png)

Then insert the data into the formula:

![ ln \\phi_i = \(0.2 - 0.1\)* \( \\frac{0.98 - 1}{0.1} + \\frac{0.96-1}{0.2}\) ](../../../upload.wikimedia.org/math/c/2/e/c2e2c1c4f6fdd304d72d713c1eb05292.png)

![Clipboard](../../../upload.wikimedia.org/wikipedia/commons/thumb/1/1f/Clipboard.svg/45px-Clipboard.svg.png)

**To do:**  
Finish this example

Raoult's law can be modified to account for nonideal gasses in a similar way to its modification for nonideal liquids:

  
**VLE Equation for nonideal gasses and nonideal liquids**  
![ \\phi_i*y_i*P=\\gamma_i*x_i*P_i^* ](../../../upload.wikimedia.org/math/6/d/6/6d66e2a5bc719e88ac441a9bce06b29e.png)

This equation is true except at very high pressures, a case covered in [this paper](http://web.mit.edu/10.213/www/handouts/vle.pdf) briefly.

![Clipboard](../../../upload.wikimedia.org/wikipedia/commons/thumb/1/1f/Clipboard.svg/45px-Clipboard.svg.png)

**To do:**  
revamp to include saturation fugacity (which I forgot to include with liquid)

## Critical Constants

At room temperature (about 298K), it is possible to add enough pressure to carbon dioxide to get it to liquify (some fire extinguishers work by keeping liquid carbon dioxide in them under very high pressure, which rapidly vaporizes when the pressure is relieved [[1]](Print_Version.html#cite_note-1). However, if the temperature is raised to higher than 304.2 K, it will be _impossible_ to keep carbon dioxide in a liquid form, because it has too much kinetic energy to remain in the liquid phase. **No amount of pressure can turn carbon dioxide into a liquid if the temperature is too high**.

This threshold temperature is called a **critical temperature**. Any pure stable substance (not just carbon dioxide) will have a single characteristic critical temperature. Pure stable substances will also have a single characteristic **critical pressure**, which is the pressure needed to achieve a phase transition at the critical temperature, and a **critical specific volume** which is the specific volume (volume per mass) of the fluid at this temperature and pressure.

Critical pressures are typically extremely large, ranging from 2.26 atm for helium to 218.3 atm for water [[2]](Print_Version.html#cite_note-2), and about 40 atm on average. Critical temperatures typically range from 5.26 K (for helium) to the high 600s K for some aromatic compounds.

A substance which is at a temperature higher than the critical temperature and a pressure higher than its critical pressure is called a **supercritical fluid**. Supercritical fluids have some properties in common with gasses and some in common with liquid, as may be expected since it they are not observed to be liquid but would be expected to be liquefied at extreme pressures.

## Law of Corresponding States

Recall from the last section that the _compressibility_ of any substance (but most useful for gasses) is defined as:

![ Z=\\frac{P*\\hat{V}}{RT} ](../../../upload.wikimedia.org/math/9/6/c/96c131069a34df8fe32c56ab8d8434a9.png)

The compressibility of a gas is a measure of how non-ideal it is; an ideal gas has a compressibility of 1. At the critical point, in particular, the compressibility is:

  
![ Z_C = \\frac{ P_c*\\hat{V}_c}{R*T_c} ](../../../upload.wikimedia.org/math/5/3/9/53996de38277fcdf17d2c7713da80d5f.png)

Critical constants are important because it has been found experimentally that the following rule is true for many substances:

  
**The Law of Corresponding States**  
Many substances behave in similar manners to each other depending on how far the system conditions are from the critical temperature and pressure of the substance. In particular, the compressibility of a substance is strongly correlated to its variance from the critical conditions.

It has been found experimentally that many substances have very similar compressibility at their critical point. [[2]](http://www.kayelaby.npl.co.uk/chemistry/3_5/3_5.html). **Most nonpolar substances in particular have a critical compressibility of about 0.27.** The similarity of the critical compressibility between substances is what gives some weight to the law of corresponding states. However, the fact that the critical compressibility is not exactly the same for all substances leads to potential estimation errors if this method is used.

The critical constants are able to effectively predict the properties of a substance without gathering a large amount of data. However, it is necessary to define _how_ the properties of the substance change as the system variables become closer to or farther from the critical point of the substance. These methods are discussed in the following sections.

### Compressibility Charts

Recall that many substances have similar critical compressibility values near 0.27. Therefore, charts have been developed which relate compressibility at other conditions to those at the critical point. In order to use these charts, the system parameters are _normalized_ by dividing by the critical constants to yield **reduced temperature, pressure, and volume**:

  
**Reduced parameters**

  
![ T_r = \\frac{T}{T_c} ](../../../upload.wikimedia.org/math/0/9/1/09145119a6b6e8362abb547b85750ab9.png), ![ P_r = \\frac{P}{P_c} ](../../../upload.wikimedia.org/math/a/c/9/ac9075406fcd41a9355f3675e9298104.png), ![ \\hat{V}_r = \\frac{\\hat{V}}{\\hat{V}_c} ](../../../upload.wikimedia.org/math/b/e/5/be5e9c73723b8ec675010b045ee4d3c4.png)

  1. [↑](Print_Version.html#cite_ref-1) [how fire extinguishers work](http://home.howstuffworks.com/fire-extinguisher2.htm)
  2. [↑](Print_Version.html#cite_ref-2) see Wikipedia article on [critical properties](http://en.wikipedia.org/wiki/Critical_temperature)

## Problem 1

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


A **flash evaporator** is a unit operation in which a mixture enters a chamber at a given temperature and pressure, and the vapor and liquid phases that result are separated. Suppose that a mixture of 40% furan and 60% carbon tetrachloride (which is a nearly ideal mixture [[1]](Print_Version.html#cite_note-3) enters a flash evaporator at 0.7 atm and 30oC. Assuming that the mixture follows Raoult's Law, find the the compositions of the liquid and vapor streams that will exit. How much of the original mixture will be vaporized?

  


## Problem 2

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Consider a process in which you want to separate the furan and carbon tetrachloride as much as possible.

a. Repeat problem 1 at pressures of 0.6 atm, 0.5 atm, and 0.4 atm. What do you notice about the purity of the two streams? What do you notice about the amount of liquid and vapor.

b. Consider a process in which half of the vapor stream is recycled (directly) to the flash evaporator. What is the effect on the composition of the liquid? On the composition of the vapor?

c. What conclusions can you draw about the effectiveness of flash evaporators?

  1. [↑](Print_Version.html#cite_ref-3) Smith, Van Ness, and Abbott. _Introduction to Chemical Engineering Thermodynamics_. New York: McGraw Hill, 1996, p. 433.

# **Chapter 7: Energy balances on non-reacting systems**

## General Balance Equation Revisited

Recall the general balance equation that was derived for _any_ system property:

![ In - Out + Generation - Consumption = Accumulation ](../../../upload.wikimedia.org/math/a/f/9/af98eb8e1f714f5a3233d20d1e59ffbb.png)

When we derived the mass balance, we did so by citing the law of conservation of mass, which states that the total generation of mass is 0, and therefore ![ Accumulation = In - Out ](../../../upload.wikimedia.org/math/c/a/a/caa02fd623c15510bdc2329f29c321b4.png).

There is one other major conservation law which provides an additional equation we can use: the **law of conservation of energy**. This states that if E denotes the entire amount of energy in the system,

  
**Law of Conservation of Energy**  
![ E_{in} - E_{out} = E_{accumulated} ](../../../upload.wikimedia.org/math/d/3/0/d304a01556f1cdd1ac17a87049f64f9c.png)

## Types of Energy

In order to write an energy balance, we need to know what kinds of energy can enter or leave a system. Here are some examples (this is not an exhaustive list by any means) of the types of energy that can be gained or lost.

  1. A system could gain or lose _kinetic energy_, if we're analyzing a moving system.
  2. Again, if the system is moving, there could be _potential energy_ changes.
  3. _Heat_ could enter the system via conduction, convection, or radiation.
  4. _Work_ (either **expansion work** or **shaft work**) could be done on, or by, the system.

The total amount of energy entering the system is the sum of all of the different types entering the system. Here are the expressions for the different types of energy:

  1. From physics, recall that ![ KE = \\frac{1}{2}mv^2 ](../../../upload.wikimedia.org/math/5/8/c/58cca7d0abc5b20514b4bcfdf5b6f244.png). If the _system_ itself is not moving, this is zero.
  2. The _gravitational_ potential energy of a system is ![ GPE = mgh ](../../../upload.wikimedia.org/math/b/7/e/b7ea428e6de1f91947896ab7f2b6ffbd.png) where g is the gravitational constant, m is mass in kg and h is the height of the center of mass of the system. If the system does not change height, there is no change in GPE.
  3. The heat entering the system is denoted by Q, regardless of the mechanism by which it enters (the means of calculating this will be discussed in a course on transport phenomenon). According to this book's conventions, _heat entering a system is positive and heat leaving a system is negative_, because the system in effect gains energy when heat enters.
  4. The work done by or on the system is denoted by W. _Work done BY a system is negative_ because the system has to "give up" energy to do work on its surroundings. For example, if a system expands, it loses energy to account for that expansion. Conversely, _work done ON a system is positve_.

## Energy Flows due to Mass Flows

Accumulation of _anything_ is 0 at steady state, and energy is no exception. If, as we have the entire time, we assume that the system is at steady state, we obtain the energy balance equation:

![ E_{in} = E_{out} ](../../../upload.wikimedia.org/math/5/4/f/54f84773a10b02be527d42690a944672.png)

This is the starting point for all of the energy balances below.

Consider a system in which a mass, such as water, enters a system, such as a cup, like so:

![Stilles Mineralwasser.jpg](../../../upload.wikimedia.org/wikipedia/commons/0/02/Stilles_Mineralwasser.jpg)

The mass flow into (or out of) the system carries a certain amount of energy, associated with how fast it is moving (kinetic energy), how high off the ground it is (potential energy), and its temperature (internal energy). It is possible for it to have other types of energy as well, but for now let's assume that these are the only three types of energy that are important. If this is true, then we can say that the total energy carried _in the flow itself_ is:

![ \\dot{E}_i = \(\\frac{1}{2}\\dot{m} v^2 + \\dot{m} gh + \\dot{U}\)_i ](../../../upload.wikimedia.org/math/f/b/5/fb517061daedae5b09885bc7c0c48316.png)

However, there is one additional factor that must be taken into account. **When a mass stream flows into a system it expands or contracts and therefore performs work on the system**. An expression for work due to this expansion is:

![ W_{exp} = P*\\dot{V}_i ](../../../upload.wikimedia.org/math/5/1/b/51bba25e52958cdb0ad27ecaa777a343.png)

Since this work is done _on_ the system, it enters the energy balance as a positive quantity. Therefore the total energy flow into the system due to mass flow is as follows:

![ \\dot{E}_i = \(\\frac{1}{2}\\dot{m} v^2 + \\dot{m} gh + \\dot{U}\)_i + P*\\dot{V}_i ](../../../upload.wikimedia.org/math/3/3/e/33ecfd1b0f3336dbd2f80d4027a632c4.png)

Now, to simplify the math a little bit, we generally don't use internal energy and the PV term. Instead, we combine these terms and call the result the **enthalpy** of the stream. Enthalpy is just the combination of internal energy and expansion work due to the stream's flow, and is denoted by the letter H:

  
**Definition of enthalpy**  
![ H = U + PV ](../../../upload.wikimedia.org/math/5/0/5/505eae3cb8a58f4f816e15f0eaca820d.png)

Therefore, we obtain the following important equation for energy flow carried by mass:

  
In stream i, if only KE, GPE, internal energy, and expansion work are considered,  
the energy carried by mass flow is:  
![ \\dot{E}_i = \(\\frac{1}{2}\\dot{m} v^2 + \\dot{m} gh + \\dot{H}\)_i ](../../../upload.wikimedia.org/math/2/f/4/2f4f640085fcef6fe2587b456beca630.png)

**Note:**  
Kinetic energy and potential energy are generally very small compared to the enthalpy, except in cases of very rapid flow or when there are no significant temperature changes occurring in the system. Therefore, they are often neglected when performing energy balances.

## Other energy flows into and out of the system

The other types of energy flows that could occur in and out of a system are _heat_ and _work_. Heat is defined as energy flow due to a change in temperature, and always flows from higher temperature to lower temperature. Work is defined as an energy transferred by a force (see [here](http://en.wikipedia.org/wiki/Mechanical_work) for details).

  * If there is no heat flow into or out of a system, it is referred to as **adiabatic**.
  * If there are no mechanical parts connected to a system, and the system is not able to expand, then the work is essentially 0.

Some systems which have mechanical parts that perform work are turbines, mixers, engines, stirred tank reactors, agitators, and many others. The type of work performed by these parts is called **shaft work** to distinguish it from work due to expansion of the system itself (which is called _expansion work_).

An "insulated system" is generally interpreted as being essentially adiabatic, though how good this assumption is depends on the quality of the insulation. A system that cannot expand is sometimes described as "rigid".

The notation for these values are as follows:

  * Heat flows: ![ \\dot{Q}_j ](../../../upload.wikimedia.org/math/6/3/4/63456c3010d44dcc99b0ad584338cf10.png), at the "j"th location.
  * Shaft work: ![ \\dot{W}_s ](../../../upload.wikimedia.org/math/e/c/8/ec851c631adbd6e7917ae42dbdca9e44.png)
  * Expansion work: ![ P*\\frac{\\Delta{V}}{\\Delta{t}} ](../../../upload.wikimedia.org/math/6/8/d/68dbc3746a6a28f5be3bff594e61c435.png)

Note that the above implies that **there is no expansion work at steady state** because at steady state nothing about the system, including the volume, changes with time, i.e. ![\\frac{\\Delta{V}}{\\Delta{t}} = 0 \\mbox{ at steady state}](../../../upload.wikimedia.org/math/9/4/5/945c9a5b945ffa1535084984284d1975.png).

  


## Overall steady - state energy balance

If we combine all of these components together, remembering that heat flow into a system and work done _on_ a system are positive, we obtain the following:

  
**Steady State Energy Balance on an Open System**  
![ \\Sigma\(\\frac{1}{2}\\dot{m}v^2 + \\dot{m}gh + \\dot{H}\)_{i,in} 
- \\Sigma\(\\frac{1}{2}\\dot{m}v^2 + \\dot{m}gh + \\dot{H}\)_{i,out}
+ \\Sigma \\dot{Q}_j + \\dot{W}_s = 0 ](../../../upload.wikimedia.org/math/6/d/9/6d9e7bdad41ede6676787864b8493a20.png)

Some important points:

  1. If the system is **closed AND at steady state** that means the total heat flow must equal the total work done in magnitude, and be opposite in sign. However, according to another law of thermodynamics, the second law, it is impossible to change ALL of the heat flow into work, even in the most ideal case.
  2. In an adiabatic system with no work done, the total amount of energy carried by mass flows is equal between those flowing in and those flowing out. However, that DOES NOT imply that the temperature remains the same, as we will see in a later section. Some substances have a greater capacity to hold heat than others, hence the term **heat capacity**.
  3. If the conditions _inside the system_ change over time, then we CANNOT use this form of the energy balance. The next section has information on what to do in the case that the energetics of the system change.[Introduction to Chemical Engineering Processes/Use of the steady state energy balance](http://en.wikibooks.org/w/index.php?title=Introduction_to_Chemical_Engineering_Processes/Use_of_the_steady_state_energy_balance&action=edit&redlink=1)

## What IS accumulation?

Recall that so far in this text it has been assumed that all systems are at **steady state**, which means that there is no buildup of mass, energy, or other conserved quantities. However, there are many situations, such as whenever operating levels change, that a system will not be at steady state, and mass and energy will be **accumulated** over time.

The most important thing to remember about accumulation is that _it deals with the actual amount of stuff **in the system**, not any sort of flow rate_. If you remember if you're dealing with actual system properties rather than flow rates, it will help keep the terms straight in unsteady-state balances.

### Unsteady-state Mass Balance

Lets begin the derivation of an unsteady-state mass balance with the general balance equation which you should know and love by now:

![ In - Out + Generation = Accumulation ](../../../upload.wikimedia.org/math/0/b/1/0b1844235a31438006e40205686b0ce8.png)

Substituting the terms we usually used for in, out, and generation, we obtain:

![\\Sigma \\dot{m}_{i, in} - \\Sigma \\dot{m}_{i,out} + \\dot{m}_{i,gen} = Accumulation ](../../../upload.wikimedia.org/math/4/c/3/4c3c4632b1f9bc570ce48149c79bc230.png)

Now we have to come up with a mathematical formulation for the accumulation. Unlike all of the other terms in this equation, which deal with energy _flows_ into the system, **the accumulation deals with the amount of energy that is already in the system at a certain point of time**, and more specifically how it _changes_ with time.

The rate of accumulation of energy _will not be constant_ unless it is zero (otherwise every reactor in the world would either blow up from excessive mass and energy buildup or would cease operating because all of the reactants and products would be drained out). Recall that if the accumulation reaches zero, the system is at steady state. Most systems tend to move towards a steady state (it is possible to have more than one set of steady state conditions, but it won't be covered here) over long periods of time, as shown below:

![Clipboard](../../../upload.wikimedia.org/wikipedia/commons/thumb/1/1f/Clipboard.svg/45px-Clipboard.svg.png)

**To do:**  
Diagram of plateau.

Such a system is called **self-regulating** (or **naturally stable**). If a system is not self-regulating then special control techniques (see [Control systems](http://en.wikibooks.org/wiki/Control_systems)) must be utilized to force the system into a steady state.

In order to take into account variation in the accumulation rate, we must consider the rate of change over a _very small_ amount of time, so small in fact that it is practically zero, and the accumulation vs. time curve resembles a straight line. The slope of this line at time t is approximately:

![ Slope = \\mbox{ Accumulation rate at time t} = \\frac{M_{sys, t + \\Delta t} - M_{sys,t}}{\\Delta t} ](../../../upload.wikimedia.org/math/3/f/9/3f9c44af991d7144f896f764110a1fe8.png)

Therefore we could write the following:

![ Accumulation = \\frac{M_{sys, t + \\Delta t} - M_{sys,t}}{ \\Delta t} ](../../../upload.wikimedia.org/math/3/6/0/360ae09af9c93caa7e94a5005f1a974b.png)

We then write our mass balance by substituting this accumulation into the mass balance above:

![ \\frac{ M_{sys, t + \\Delta t} - M_{sys,t}}{ \\Delta t} = \\Sigma \\dot{m}_{i,in} - \\Sigma \\dot{m}_{i,out} ](../../../upload.wikimedia.org/math/1/8/c/18c2d4b5191febccf3239e6154e42dc7.png)

For practical applications, this equation is generally multiplied by ![ \\Delta t ](../../../upload.wikimedia.org/math/5/a/7/5a72f1304af0783657605aed0e38201a.png). Then, rather than dealing with flow rates, a new quantity is defined:

![ \\Delta m_i = \\dot{m}_i * \\Delta t ](../../../upload.wikimedia.org/math/e/a/e/eae0247d7c451ba588c075def6587bec.png)

This quantity is the _total amount of mass that enters the system in a finite amount of time_. Substituting this definition into the mass balance yields the following:

  
**Unsteady State Mass Balance**  
![ \\Sigma \\Delta m_{i,in} - \\Sigma \\Delta m_{i, out} + m_{i,gen} = M_{sys, t + \\Delta t} - M_{sys, t} ](../../../upload.wikimedia.org/math/2/e/0/2e0fb9a51e333a2431edd96c87ffc9b7.png)

  


![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


A feed stream with ![ 50 \\frac{kg}{h} ](../../../upload.wikimedia.org/math/c/a/6/ca6e9ae51b922d3f4ed8759c16c3d3f0.png) of water and ![ 1 \\frac{kg}{h} ](../../../upload.wikimedia.org/math/1/3/c/13cf734a710482b98dcc78211b28eccb.png) of ethanol enters a distillation column. A distillation column generally has two outlet streams called the **bottoms** and the **condensate**. At steady state, the condensate is 12% ethanol by mass and the total condensate flowrate is ![ 9 \\frac{kg}{h} ](../../../upload.wikimedia.org/math/4/5/b/45baf46c2ca484c55d10628279e28a72.png).

One day, the boss calls and says that she needs more production, so you turn up the feed to ![ 60 \\frac{kg}{h} ](../../../upload.wikimedia.org/math/8/7/7/877de039e67a8b7855e6eb1a593c747d.png). Two hours later, the distillation column floods.

**a.** What was the cause of the flooding?

**b.** Assuming that the total outlet mass flow rates remained the same throughout the process, what was the total mass accumulation in the column?

**c.** Describe two methods by which the flow rates may be modified to reach a new steady state. Will the new steady state produce the same outlet concentrations as the old steady state? Explain. (hint: how is the separation effectiveness related to the ratio of the two outlet flowrates? You may need to do some research on this)

![Clipboard](../../../upload.wikimedia.org/wikipedia/commons/thumb/1/1f/Clipboard.svg/45px-Clipboard.svg.png)

**To do:**  
Put in the solution to this

### Unsteady-state Energy Balance

Lets start by examining what we have so far, but with the accumulation term (yet to be defined mathematically) added in the right side, since we're not at steady state any more:

![ \\Sigma\(\\frac{1}{2}\\dot{m}v^2 + \\dot{m}gh + \\dot{H}\)_{i,in} 
- \\Sigma\(\\frac{1}{2}\\dot{m}v^2 + \\dot{m}gh + \\dot{H}\)_{i,out}
+ \\Sigma \\dot{Q}_j + \\dot{W}_s = Accumulation ](../../../upload.wikimedia.org/math/3/d/9/3d9613e12b8e4a02858a2dd657faf792.png)

Following the logic from the mass balance, we obtain for the accumulation:

![ \\frac{E_{sys, t + \\Delta t} - E_{sys,t}}{\\Delta t} ](../../../upload.wikimedia.org/math/e/3/4/e346026f665d738e26f5200a8a2c26aa.png)

Therefore, we have:

![ \\Sigma\(\\frac{1}{2}\\dot{m}v^2 + \\dot{m}gh + \\dot{H}\)_{i,in} 
- \\Sigma\(\\frac{1}{2}\\dot{m}v^2 + \\dot{m}gh + \\dot{H}\)_{i,out}
+ \\Sigma \\dot{Q}_j + \\dot{W}_s = \\frac{E_{sys, t + \\Delta t} - E_{sys,t}}{\\Delta t} ](../../../upload.wikimedia.org/math/2/6/f/26fafa16ce1159a7c595ee75f67ad748.png)

Like in the case of the mass balance, we can only consider the **total** energy change over a **total** amount of time using this equation. To do this, we multiply the entire equation above by the time change from some starting point to the point of interest.

Now we need some definitions:

  1. ![ Q = \\dot{Q}*\\Delta t ](../../../upload.wikimedia.org/math/d/4/3/d432b7740afbc7909de1e10050f72926.png) is the TOTAL heat flow over the time period.
  2. ![ W_s = \\dot{W}_s * \\Delta t ](../../../upload.wikimedia.org/math/5/d/f/5dfbd04d4587f8659cfca3692061ffac.png) is the TOTAL shaft work over the time period.
  3. ![ \\dot{m}_i * \\Delta t = \\Delta m_i ](../../../upload.wikimedia.org/math/b/6/5/b65ba8c82fc6c38b9adb16e3a92a4c95.png) is the TOTAL mass flow into (or out of) the system due to stream i during the time period.
  4. ![ \\dot{H}_i * \\Delta t = H ](../../../upload.wikimedia.org/math/8/2/b/82b54c8908cb93e3acacaafdf7c9d0af.png) is the TOTAL enthalpy carried into (or out of) the system due to stream i during the time period.

The major assumption here is that **the enthalpies, heat flow rates, and shaft work on the left hand side of the equals sign must either be constant, or the average value over the whole time period must be used, in order for this equation to be valid**. Whether this assumption is valid or not depends on the situation (for example, it depends on whether the process feeding mass to your process is itself at steady state or not).

With these in mind, we multiply by delta t in order to obtain the following, **unsteady state** energy balance.

  
**Unsteady State Energy Balance**  
![ \\Sigma\(\\frac{1}{2}\\Delta{m}v^2 + \\Delta{m}gh + {H}\)_{i,in} 
- \\Sigma\(\\frac{1}{2}\\Delta{m}v^2 + \\Delta{m}gh + {H}\)_{i,out}
+ \\Sigma {Q}_j + {W}_s = {E_{sys, t + \\Delta t} - E_{sys,t}} ](../../../upload.wikimedia.org/math/3/2/d/32d823bfac598fb1630b2f129ca209d2.png)

# **Chapter 8: Combining energy and mass balances in non-reacting systems**

    The basic idea

    Several example problems illustrating how to combine these two concepts.

# **Chapter 9: Introduction to energy balances on reacting systems**

    What's different for a reacting system?

# **Appendix 1: Useful Mathematical Methods**

## Mean and Standard Deviation

A lot of the time, when you're conducting an experiment, you will run it more than once, especially if it is inexpensive. Scientists run experiments more than once so that the _random errors_ that result from taking measurements, such as having to guess a length between two hash marks on a ruler, cancel themselves out and leave them with a more precise measurement. However, the question remains: how should you consolidate all of the data into something that's more managable to use?

### Mean

Suppose you have n data points taken **under the same conditions** and you wish to consolidate them to as few as feasibly possible. One thing which could help is is to use some centralized value, which is in some way "between" all of the original data points. This, in fact, is called the **mean** of the data set.

There are many ways of computing the mean of a data set depending on how it is believed to be distributed. One of the most common methods is to use the _arithmetic mean_, which is defined as:

  
![ \\bar{x} = \\frac{\\Sigma \\hat{x}_k}{n}](../../../upload.wikimedia.org/math/d/0/b/d0b259c232ff2c9a85cc18d2d2594d45.png)

Other types of mean include the [w:Geometric mean](http://en.wikipedia.org/wiki/Geometric_mean), which should be used when the data are very widely distributed (ex. an exponential distribution) and the "log-mean" which occurs often in transport phenomena.

### Standard Deviation

Having a value for the mean tells you what value the data points "cluster" around but it does not tell you how spread out they are from the center. A second statistical variable called the **standard deviation** is used for that. The standard deviation is essentially _the average distance between the data points and their mean_. The distance is expressed as a squared distance in order to prevent negative deviations from lessoning the effect of positive deviations.

The mathematical formulation for the standard deviation ![ \\sigma ](../../../upload.wikimedia.org/math/9/d/4/9d43cb8bbcb702e9d5943de477f099e2.png) is:

  
![ \\sigma^2 = \\frac{\\Sigma \(\\hat{x}_k - \\bar{x}\)^2}{n-1} ](../../../upload.wikimedia.org/math/8/4/f/84f3149313e6db907c0dec34e99ad349.png)

The denominator is n-1 instead of n because statisticians found that it gives better results for small numbers of experiments; see [w:Standard deviation](http://en.wikipedia.org/wiki/Standard_deviation) for a more thorough explanation of this.

### Putting it together

**The standard deviation of a data set measured under constant conditions is a measure of how precise the data set is.** Because this is true, the standard deviation of a data set is often used in conjunction with the mean in order to report experimental results. Typically, results are reported as:

![ \\bar{x} \\pm \\sigma ](../../../upload.wikimedia.org/math/f/9/d/f9d84c94f8af80a48098751f1e2b9e1d.png)

If a distribution is assumed, knowing both the mean and standard deviation can help us to estimate the probability that the actual value of the variable is within a certain range, _if there is no systematic bias in the data_. If there is (such as use of broken equipment, negligence, and so on) then no statistics could predict the effects of that.

## Linear Regression

Suppose you have a set of data points (![ \\hat{x}_k, \\hat{y}_k ](../../../upload.wikimedia.org/math/1/3/d/13dff133472fe5dc9aa0a3ba963eb143.png)) taken **under differing conditions** which you suspect, from a graph, can be reasonably estimated by drawing a line through the points. Any line that you could draw will have (or can be written in) the following form:

![ y = mx + b ](../../../upload.wikimedia.org/math/e/e/6/ee668a8b96a7fc367f89135101df6c90.png) where m is the slope of the line and b is the y-intercept.

We seek the _best_ line that we could possibly use to estimate the pattern of the data. This line will be most useful for both interpolating between points that we know, and extrapolating to unknown values (as long as they're close to measured values). In the most usual measure, how "good" the fit is is determined by the vertical distance between the line and the data points (![ R_k ](../../../upload.wikimedia.org/math/8/c/1/8c154f576b6c8a6db8eddf12a8ff12f8.png)), which is called the **residual**:

![ R_k = \(m\\hat{x}_k + b\) - \\hat{y}_k ](../../../upload.wikimedia.org/math/a/b/4/ab46819f20bc80c1420458d9156ecfee.png)

In order to normalize the residuals so that they don't cancel when one's positive and one's negative (and thus helping to avoid statistical bias), we are usually concerned with the _square_ of ![ R_k ](../../../upload.wikimedia.org/math/8/c/1/8c154f576b6c8a6db8eddf12a8ff12f8.png) when doing least-squares regression. We use squared terms and not absolute values so that the function is [differentiable](http://en.wikipedia.org/wiki/differentiable), don't worry about this if you haven't taken calculus yet.

In order to take into account _all_ of the data points, we simply seek to minimize the sum of the squared residuals:

![ \\mbox{ minimize }\\Sigma {R_k}^2 ](../../../upload.wikimedia.org/math/d/d/0/dd0662ea128cb13c83da19a3c1398bb1.png)

Using calculus, we can take the derivative of this with respect to m and with respect to b and solve the equations to come up with the values of m and b that minimize the sum of squares (hence the alternate name of this technique: **least-squares regression**. The formulas are as follows, where n is the total number of data points you are regressing[1]:

  
![ m^* = \\frac{n*\\Sigma\(\\hat{x}_k*\\hat{y}_k\)-\\Sigma\(\\hat{x}_k\)*\\Sigma\(\\hat{y}_k\)} {n*\\Sigma\(\\hat{x_k}^2\) - \(\\Sigma\(\\hat{x}_k\)\)^2} ](../../../upload.wikimedia.org/math/a/7/0/a70eeddf08051b4b23e4832fbe2b5792.png)

  
![ b^* = \\frac{\\Sigma\(\\hat{y}_k\) - m^{*}*\\Sigma\(\\hat{x}_k\)} {n} ](../../../upload.wikimedia.org/math/2/1/1/211575a3bf19bb3d499da0dd53c76a07.png)

### Example of linear regression

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Suppose you wanted to measure how fast you got to school by a less direct route than looking at the speedometer of your car. Instead, you look at a map and read the distances between each intersection, and then you measure how long it takes to go each distance. Suppose the results were as shown in the table below. How far from home did you start, and what is the best estimate for your average speed?

t(min) D (yards)

1.1
559.5

1.9
759.5

3.0
898.2

3.8
1116.3

5.3
1308.7

The first thing we should do with any data like this is to graph it and see if a linear fit would be reasonable. Plotting this data, we can see by inspection that a linear fit appears to be reasonable.

![RegressionData.png](../../../upload.wikimedia.org/wikibooks/en/d/db/RegressionData.png)

Now we need to compute all of the values in our regression formulas, and to do this (by hand) we set up a table:

Trial t t^2 D D^2 t*D

1
1.1
1.21
559.5
313040
615.45

2
1.9
3.61
759.5
576840
1443.05

3
3.0
9.00
898.2
806763
2694.6

4
3.8
14.44
1116.3
1246126
4241.94

5
5.3
28.09
1308.7
1712695
6936.11

TOTAL
15.1
56.35
4642.2
4655464
15931.15

  
Now that we have this data we can plug it into our linear regression equation:

![  m^* = \\frac{n*\\Sigma\(\\hat{x}_k*\\hat{y}_k\)-\\Sigma\(\\hat{x}_k\)*\\Sigma\(\\hat{y}_k\)} {n*\\Sigma\(\\hat{x_k}^2\) - \(\\Sigma\(\\hat{x}_k\)\)^2} ](../../../upload.wikimedia.org/math/a/7/0/a70eeddf08051b4b23e4832fbe2b5792.png)

![ = \\frac{5* 15931.13 - 15.1*4642.2}{5*56.35 - \(15.1\)^2} ](../../../upload.wikimedia.org/math/a/4/a/a4a20a3de6aab754e3df0e3ae02a2450.png)

![ = 177.9 \\mbox{ } \\frac{yard}{min} ](../../../upload.wikimedia.org/math/9/c/1/9c1a386477b782b00bfc1adddd43defc.png)

So ![ b = \\frac{\\Sigma\(\\hat{y}_k\) - m^{*}*\\Sigma\(\\hat{x}_k\)} {n} ](../../../upload.wikimedia.org/math/3/2/9/329cdad77f3464359d1aefac1e25e4ef.png)

![ = \\frac{4642.2 - 177.9*15.1}{5} = 391.2 \\mbox{ yards} ](../../../upload.wikimedia.org/math/b/e/e/bee5e3b3993c5955aa9d81924427c0d3.png)

Hence the equation for the line of best fit is:

  
![D=177.9*t+391.2 ](../../../upload.wikimedia.org/math/d/6/8/d6841319b9786d74e934798c40e1cfe3.png)

The graph of this plotted against the data looks like this:

![RegressionDataWithLine.png](../../../upload.wikimedia.org/wikibooks/en/6/62/RegressionDataWithLine.png)

### How to tell how good your regression is

In the previous example, we visually determined if it would be reasonable to perform a linear fit, but it is certainly possible to have a less clear-cut case! If there is some slight curve to the data, is it still "close enough" to be useful? Though it will always come down to your own judgment after seeing the fit line graph against the data, there is a mathematical tool to help you called a **correlation coefficient**, r, which can be defined in several different ways. One of them is as follows [1]:

  
![ r = \\frac{n*\\Sigma\( \\hat{x}_k * \\hat{y}_k \) - \\Sigma\(\\hat{x}_k\)*\\Sigma\(\\hat{y}_k\)}
{\\sqrt{n*\\Sigma\({\\hat{x}_k}^2\) - \(\\Sigma \\hat{x}_k\)^2} * \\sqrt{n*\\Sigma\({\\hat{y}_k}^2\) - \(\\Sigma \\hat{y}_k\)^2}}](../../../upload.wikimedia.org/math/2/6/e/26e8b605c22f9ddd0d26c1324cb38ab3.png)

It can be shown that this value always lies between -1 and 1. The closer it is to 1 (or -1), the more reasonable the linear fit. In general, the more data points you have, the smaller r needs to be before it's a good fit, but a good rule of thumb is to look for high (higher than 0.85 or 0.9) values and then graph to see if the graph makes sense. Sometimes it will, sometimes it won't, the method is not foolproof.

In the above example we have:

![ r = \\frac{5*15931.13 - 15.1*4642.2}{\\sqrt{5*56.35-\(15.1\)^2}*\\sqrt{5*4655464-\(4642.2\)^2}} ](../../../upload.wikimedia.org/math/3/4/4/3447191e3e754300dc6d94fff8025a3b.png)

![ r = 0.992 ](../../../upload.wikimedia.org/math/2/8/a/28ae4de07dd26a88d2e82e4be8a377d5.png)

Hence the data correlates very well with a linear model.

## Linearization

### In general

Whenever you have to fit a parameter or multiple parameters to data, it is a good idea to try to **linearize** the function first, because linear regression is much less intensive and more accurate than nonlinear regression. The goal with any linearization is to reduce the function to the form:

![ Variable\\mbox{ 1} = constant + constant*\\mbox{ Variable 2} ](../../../upload.wikimedia.org/math/7/a/2/7a26ef1458d21757eaa6cc631b46e118.png)

The difference between this and "standard" linear regression is that Variable 1 and Variable 2 can be _any functions_ of x and y, as long as they are not combined in any way (i.e. you can't have ![ln\(x + y\) ](../../../upload.wikimedia.org/math/c/2/c/c2c45f6effc015b2c2c8b6674e4de27b.png) as one variable). The technique can be extended to more than two variables using a method called [w:multiple linear regression](http://en.wikipedia.org/wiki/multiple_linear_regression) but as that's more difficult to perform, this section will focus on two-dimensional regression.

### Power Law

To see some of the power of linearization, let's suppose that we have two variables, x and y, related by a power law:

![ y = A*x^b ](../../../upload.wikimedia.org/math/9/3/b/93b1132b5ad818b07fef90e3b247eb63.png)

where A and b are constants. If we have data connecting changes in y to changes in x, we would like to know the values of a and b. This is difficult to do if the equation is in its current form but we can change it into a linear-type function!

The trick here is we need to get rid of the exponent b, so in order to do that we take the natural log of both sides:

![ ln\\mbox{ y} = ln\\mbox{ \(A*x}^b\) ](../../../upload.wikimedia.org/math/2/9/f/29fd7ff89a5c70f3b6aa1c17d5f42153.png)

Using laws of logarithms we can simplify the right-hand side to obtain the following:

  
![ y = A*x^b <-> ln\\mbox{ y} = ln\\mbox{ A} + b*ln\\mbox{ x} ](../../../upload.wikimedia.org/math/d/d/6/dd69e2ccff159cc7f2769127d5687d02.png)

The beauty of this equation is that it is, in a sense, linear. **If we graph ln(y) vs. ln(x) obtain a straight line with slope b and y-intercept ln(A)**.

### Exponentials

Another common use of linearization is with exponentials, where x and y are related by an expression of the form:

![ y = A*b^x ](../../../upload.wikimedia.org/math/1/c/6/1c67d00fe7ac760db1b09aa25aa44f78.png)

This works for any base but the most common base encountered in practice is Euler's constant, e. Again, we take the natural log of both sides, in order to get rid of the exponent:

  
![ y = A*b^x <-> ln \\mbox{ y} = \\ln{ A} + x* \\ln{ b} ](../../../upload.wikimedia.org/math/7/2/b/72bebc3c7a2779a8abf2e2b649c64478.png)

This time, **Graph ln y vs. x to obtain a line with slope ln(b) and y-intercept ln(A)**.

## Linear Interpolation

Often, when you look up properties on a chart, you will be looking them up at conditions in _between_ two charted conditions. For example, if you were looking up the specific enthalpy of steam at **10 MPa** and 430oC you would look in the steam tables and see something like this: [2]

T (oC) H ![ \(\\frac{kJ}{kg}\) ](../../../upload.wikimedia.org/math/d/6/1/d615ced5d7c76d8c193f1987d926f154.png)

400
2832.4

450
2943.4

How can you figure out the intermediate value for this? We can't exactly but we can **assume that H(T) is a linear function**. If we assume that it is linear, then we can easily find the intermediate value. First, we set up a table, including the unknown value like this:

T (oC) H ![ \(\\frac{kJ}{kg}\) ](../../../upload.wikimedia.org/math/d/6/1/d615ced5d7c76d8c193f1987d926f154.png)

400
2832.4

430
x

450
2943.4

Then since we're assuming the relationship between T and H is linear, and the slope of a line is a constant _the slope between points 3 and 2 has to equal the slope between points 3 and 1_.

Therefore, we can write that:

![ \\frac{2943.4-x}{450-430} = \\frac{2943.4-2832.4}{450-400} ](../../../upload.wikimedia.org/math/8/d/b/8dba8859adf135cb028a6cd6e26df38a.png)

Solving gives **x = 2899 kJ/kg**

The same method can be used to find an unknown T for a given H between two tabulated values.

### General formula

To derive a more general formula (though I always derive it from scratch anyways, it's nice to have a formula), lets replace the numbers by variables ad give them more generic symbols:

x y

![ x_1 ](../../../upload.wikimedia.org/math/f/9/a/f9a3b8e9e501458e8face47cae8826de.png)
![ y_1 ](../../../upload.wikimedia.org/math/f/7/b/f7b4a9a272539da17df482a540896746.png)

![ x^* ](../../../upload.wikimedia.org/math/d/2/8/d282fd3350bb3ec2d1e940574e8fd42d.png)
![ y^* ](../../../upload.wikimedia.org/math/8/9/b/89b3a69e7c4dd4ee4e7aaf09191ac8e1.png)

![ x_2 ](../../../upload.wikimedia.org/math/8/f/4/8f43fce8dbdf3c4f8d0ac91f0de1d43d.png)
![ y_2 ](../../../upload.wikimedia.org/math/8/9/f/89f771207ffb39300acb88dff8bae241.png)

Setting the slope between points 3 and 2 equal to that between 3 and 1 yields:

  
![ \\frac{y2-y1}{x2-x1} = \\frac{y2 - y^*}{x2 - x^*} ](../../../upload.wikimedia.org/math/3/9/b/39b542d5f40e118657d3dab3ea7467f0.png)

This equation can then be solved for x* or y* as appropriate.

### Limitations of Linear Interpolation

It is important to remember that _linear interpolation is not exact_. How inexact it is depends on two major factors:

  1. What the real relationship between x and y is (the more curved it is, the worse the linear approximation)
  2. The difference between consecutive x values on the table (the smaller the distance, the closer almost any function will resemble a line)

Therefore, it is not recommended to use linear interpolation if the spaces are very widely separated. However, if no other method of approximation is available, linear interpolation is often the only option, or other forms of interpolation (which may be just as inaccurate, depending on what the actual function is).

See also [w:interpolation](http://en.wikipedia.org/wiki/interpolation).

## References

[1]: Smith, Karl J. The Nature of Mathematics. Pacific Grove, California: Brooks/Cole Publishing company, 6e, p. 683

[2]: Sandler, Stanley I. Chemical, Biochemical, and Engineering Thermodynamics. University of Deleware: John Wiley and Sons inc., 4e, p. 923

## Basics of Rootfinding

Rootfinding is the determination of solutions to single-variable equations or to systems of n equations in n unknowns (provided that such solutions exist). The basics of the method revolve around the determination of _roots_

A **root** of a function ![ F\(x_1,x_2,...\) ](../../../upload.wikimedia.org/math/8/e/9/8e9be79870b6a3ec2f4f91e0bbf5badc.png) in any number of variables is defined as the solution to the equation ![ F\(x_1,x_2,...\) = 0 ](../../../upload.wikimedia.org/math/3/b/b/3bb551709fca41805f9edb4febdbb4c7.png). **In order to use any of the numerical methods in this section, the equation should be put in a specific form**, and this is one of the more common ones, used for all methods except the iterative method.

However, it is easy to put a function into this form. If you start with an equation of the form:

![ F_1\(x_1,x_2,...\) = F_2\(x_1,x_2,...\) ](../../../upload.wikimedia.org/math/1/f/c/1fc520ee59b1a4cbd02cef4892188fc4.png)

then subtracting ![ F_2 ](../../../upload.wikimedia.org/math/1/6/2/162b23614f3de15ba9c77440d9b75780.png) will yield the required form. _Do not forget to do this, even if there is only a constant on one side!_

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


If you want to use the bisection method later in this section to find one of the solutions of the equation ![ 1 = x^2 ](../../../upload.wikimedia.org/math/f/e/9/fe93d7e08d2671ff1537ec977e3f426a.png), you should rewrite the equation as ![ 0 = x^2 - 1 ](../../../upload.wikimedia.org/math/4/d/8/4d876d5bf28b93db4f2aa7e490272699.png) so as to put it in the correct form.

Since any equation can be put into this form, the methods can potentially be applied to any function, though they work better for some functions than others.

## Analytical vs. Numerical Solutions

An **analytical** solution to an equation or system is a solution which can be arrived at exactly using some mathematical tools. For example, consider the function ![ y = ln\(x\) ](../../../upload.wikimedia.org/math/7/c/7/7c72906369801c8bfb7668b73c743b5a.png), graphed below.

![Ln .svg](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/99/Ln_.svg/600px-Ln_.svg.png)

The root of this function is, by convention, when ![ y = 0 ](../../../upload.wikimedia.org/math/f/a/b/fab37d6c4a697fe660387d3ff8e889a4.png), or when this function crosses the x-axis. Hence, the root will occur when ![ ln\(x\) = 0 \\rightarrow x = e^0 = 1 ](../../../upload.wikimedia.org/math/b/8/7/b87e8565e2459e46b585cae88f4939ef.png)

The answer **x=1** is an analytical solution because through the use of algebra, we were able to come up with an exact answer.

On the other hand, attempting to solve an equation like:

![ -x = ln\(x\) ](../../../upload.wikimedia.org/math/3/0/6/30622aa4b8253376704374379adcd9bd.png)

analytically is sure to lead to frustration because it is not possible with elementary methods. In such a case it is necessary to seek a **numerical** solution, in which guesses are made until the answer is "close enough", but you'll never know what the _exact_ answer is.

All that the numerical methods discussed below do is give you a systematic method of guessing solutions so that you'll be likely (and in some cases guaranteed) to get closer and closer to the true answer. The problem with numerical methods is that most are not guaranteed to work without a good enough _initial guess_. Therefore it is valuable to try a few points until you get somewhere close and _then_ start with the numerical algorithm to get a more accurate answer. They are roughly in order from the easiest to use to the more difficult but faster-converging algorithms.

## Rootfinding Algorithms

### Iterative solution

Iterative solutions in their purest form will solve the desired function so that it is in the form:

![ x = f\(x\) ](../../../upload.wikimedia.org/math/4/e/a/4ea69b9b692b39dd000e70830b90a77f.png)

Then, a value for x is guessed, and f(x) is calculated. The new value of x is then re-inserted into f(x), and the process is repeated until the value of x changes very little.

The following example illustrates this procedure.

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Use an iterative solution to calculate the root of ![ x + ln\(x\) = 0 ](../../../upload.wikimedia.org/math/c/d/f/cdf5a99ba99ba90460d6821726da16df.png)

**Solution**: Solve the equation for x:

![ e^{-x} = x ](../../../upload.wikimedia.org/math/9/b/c/9bc9e7289f80468f4cd090067bc82db9.png)

First we need to guess an x to get it started. Let's try ![ x = 0.5 ](../../../upload.wikimedia.org/math/5/6/0/5606d8595e685914dd73e078734671cb.png)

Then we have:

![ x = e^{-0.5} = 0.6065 ](../../../upload.wikimedia.org/math/2/c/5/2c5908c5242f00f560653ded10409096.png)

![ x_2 = e^{-0.6065} = 0.5453 ](../../../upload.wikimedia.org/math/2/3/d/23d4d326b17ab69cbff0d5c8424176af.png)

![ x_3 = e^{-0.5453} = 0.5796 ](../../../upload.wikimedia.org/math/d/7/8/d7857ef6acd5a44676c9908003243cc1.png)

![ x_4 = e^{-0.5796} = 0.5601 ](../../../upload.wikimedia.org/math/e/c/7/ec747e5615ec083342947271242e45b5.png)

![ x_5 = e^{-0.5601} = 0.5711 ](../../../upload.wikimedia.org/math/9/d/5/9d5d642f3d2dbf31424bedd13a7bd9f1.png)

![ x_6 = e^{-0.5711} = 0.5649 ](../../../upload.wikimedia.org/math/f/3/0/f300aa6e3e84046a90435a74677a81c7.png)

![ x_7 = e^{-0.5649} = 0.5684 ](../../../upload.wikimedia.org/math/9/5/e/95ec343b0bf0220ffb4e35c05b3bbd80.png)

Thus to two decimal places the root is ![ x = 0.56 ](../../../upload.wikimedia.org/math/1/2/9/129fdd14771c4dd35511e9931e8f5640.png). More iterations could be performed to get a more accurate answer if desired.

This method has some rather severe limitations as we'll see in this example:

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Repeat the above but this time solve for x a different way. What do you find?

**Solution: To illustrate the point, let's start with a guess of ![ x = 0.56 ](../../../upload.wikimedia.org/math/1/2/9/129fdd14771c4dd35511e9931e8f5640.png)**

The other way to solve for x is the more obvious way: ![ x = -ln\(x\) ](../../../upload.wikimedia.org/math/1/8/0/180a6d52a164b62075173c1ba4916f12.png)

![ x = -ln\(0.56\) = 0.5798 ](../../../upload.wikimedia.org/math/b/8/0/b805ac40e82af83973de3a8529063763.png)

![ x_2 = -ln\(0.5798\) = 0.5451 ](../../../upload.wikimedia.org/math/e/b/0/eb0d2c41ac04180a9df1ba4fd680e518.png)

![ x_3 = -ln\(0.5451\) = 0.6068 ](../../../upload.wikimedia.org/math/e/b/2/eb2d8d0bbcbc5034e6dd95f7ff8e99f6.png)

Clearly, even though we started with a very good guess, the solution is diverging!

This example shows that **the success of the iteration method strongly depends on the properties of the function on the right-hand side**. In particular, it has to do with how large the slope of the function is at the root. If the slope is too large, the method will not converge, and even if it is small the method converges slowly. Therefore it is generally undesirable to use this method, though some more useful algorithms are based on it (which is why it is presented here).

### Iterative Solution with Weights

Although the iterative solution method has its downfalls, it can be drastically improved through the use of **averaging**. In this method, the function is still solved for x in the form:

![ x = f\(x\) ](../../../upload.wikimedia.org/math/4/e/a/4ea69b9b692b39dd000e70830b90a77f.png)

From the initial guess ![ x_0 ](../../../upload.wikimedia.org/math/0/b/2/0b21a666a81629962ade8afd967826ed.png), the function f(x) is used to generate the second guess ![ x_1 ](../../../upload.wikimedia.org/math/f/9/a/f9a3b8e9e501458e8face47cae8826de.png). However, rather than simply putting ![ x_1 ](../../../upload.wikimedia.org/math/f/9/a/f9a3b8e9e501458e8face47cae8826de.png) into f(x), a _weighted average_ of ![ x_0 ](../../../upload.wikimedia.org/math/0/b/2/0b21a666a81629962ade8afd967826ed.png) and ![ x_1 ](../../../upload.wikimedia.org/math/f/9/a/f9a3b8e9e501458e8face47cae8826de.png) is made:

![ x_1\(New\)= \\alpha*x_0 + \(1-\\alpha\)*x_1\(old\) , 0 \\le \\alpha \\le 1 ](../../../upload.wikimedia.org/math/e/6/6/e66b2b44a999f8b45323a9310728efdf.png)

The term ![ \\alpha ](../../../upload.wikimedia.org/math/b/c/c/bccfc7022dfb945174d9bcebad2297bb.png) is called the **weight**. The most common value of the weight is one-half, in which case the next value to plug into f(x) is simply the average of ![ x_0 ](../../../upload.wikimedia.org/math/0/b/2/0b21a666a81629962ade8afd967826ed.png) and ![ x_1\(old\) ](../../../upload.wikimedia.org/math/e/f/2/ef22d82e92e8d8a2c1941b0cd2258370.png):

  
![ x_1\(New\) = \\frac{x_0 + x_1\(Old\)}{2} ](../../../upload.wikimedia.org/math/0/2/a/02a4a38bebe8aefc4245c4c7affce91b.png)

This new value is then plugged into f(x), averaged with the result, and this is repeated until convergence.

The following examples show that this method converges faster and with more reliability than normal iterative solution.

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Find the root of ![ x + ln\(x\) = 0 ](../../../upload.wikimedia.org/math/c/d/f/cdf5a99ba99ba90460d6821726da16df.png) using the iterative method with a weight of ![ \\alpha = \\frac{1}{2} ](../../../upload.wikimedia.org/math/0/9/c/09c54fe57ce7dd4f32b3620988a0aea6.png)

**Solution**: Let's start with a guess of 0.5 like last time, and compare what happens this time from what happened with normal iteration.

![ x_1 = e^{-0.5} = 0.6065 ](../../../upload.wikimedia.org/math/6/6/2/6623c6ec5f58403a826458240480dc35.png)

![ x_1\(new\) = \\frac{0.5+0.6065}{2} = 0.5533 ](../../../upload.wikimedia.org/math/5/7/5/5756982370de25b94689ea98f252cd68.png)

![ x_2 = e^{-0.5533} = 0.5751 ](../../../upload.wikimedia.org/math/1/f/f/1ff384ad42edf4a87f1b90ad6744bca3.png)

![ x_2\(new\) = \\frac{0.5533+0.5751}{2} = 0.5642 ](../../../upload.wikimedia.org/math/7/b/d/7bd9d670cf7869e418f47f4238b02304.png)

![ x_3 = e^{-0.5642} = 0.5688 ](../../../upload.wikimedia.org/math/b/3/f/b3fdf4dd3eb036a79bf4516dfa0c7e1a.png)

Here, after only three evaluations of the function (which usually takes the longest time of all the steps), we have the root to the same accuracy as **seven** evaluations with the other method!

The method is not only faster-converging but also more stable, so that it can actually be used solving the equation the other way too.

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Starting with an initial guess of ![ x = 0.5 ](../../../upload.wikimedia.org/math/5/6/0/5606d8595e685914dd73e078734671cb.png) and using ![ x = -ln\(x\) ](../../../upload.wikimedia.org/math/1/8/0/180a6d52a164b62075173c1ba4916f12.png) and the weighted iteration method with ![ \\alpha = \\frac{1}{2} ](../../../upload.wikimedia.org/math/0/9/c/09c54fe57ce7dd4f32b3620988a0aea6.png), find the root of the equation.

**Solution**: Starting with ![ x_0 = 0.5 ](../../../upload.wikimedia.org/math/5/a/5/5a52a1913be66eb01d3e17a33d892846.png) we have:

![ x_1 = -ln\(0.5\) = 0.693 ](../../../upload.wikimedia.org/math/2/a/2/2a297b985810860958516f248bbf6289.png)

![ x_1\(new\) = \\frac{0.693+0.5}{2} = 0.597 ](../../../upload.wikimedia.org/math/1/f/7/1f72ca315935c517a4eaec22aab804f9.png)

![ x_2 = -ln\(0.597\) = 0.517 ](../../../upload.wikimedia.org/math/2/e/c/2ec9d7576895c2b1c956f2c23d4951bf.png)

![ x_2\(new\) = \\frac{0.517+0.597}{2} = 0.557 ](../../../upload.wikimedia.org/math/6/2/6/626f0af6422bea87ba3b0827b514ff70.png)

![ x_3 = -ln\(0.557\) = 0.5856 ](../../../upload.wikimedia.org/math/d/4/c/d4c2a41f7aa1f7d764f8d32abfcee9f3.png)

![ x_3\(new\) = \\frac{0.5856+0.557}{2} = 0.571 ](../../../upload.wikimedia.org/math/f/3/0/f30bd8054da2a7bded5a706e259bbbad.png)

![ x_4 = -ln\(0.571\) = 0.560 ](../../../upload.wikimedia.org/math/0/d/e/0de2c2c25c6f32a2c5638e6ef930a3a5.png)

![ x_4\(new\) = \\frac{0.560 + 0.571}{2} = 0.565 ](../../../upload.wikimedia.org/math/a/8/1/a81677fa82566e0ca2a3a871f0ef067a.png)

![ x_5 = -ln\(0.565\) = 0.570 ](../../../upload.wikimedia.org/math/b/6/2/b62f6be9a46c576b82b1affd09e5f09b.png)

Therefore we can (slowly) converge in this case using the weighted iteration method to the solution.

Notice that in this case, if we use regular iteration the result only converged if the equation was solved in a certain way. Using weighted iteration, it is possible to solve it either way and obtain a solution, but one way is clearly faster than the other. However, weighting will accelerate the algorithm in most cases and is relatively easy to implement, so it is a worthwhile method to use.

### Bisection Method

Let us consider an alternative approach to rootfinding. Consider a function f(x) = 0 which we desire to find the roots of. If we let a second variable ![ y = f\(x\) ](../../../upload.wikimedia.org/math/7/c/1/7c1c9491ba7c6e8d6d2cfa82e39b22ca.png), then y will (almost always) change sign between the left-hand side of the root and the right-hand side. This can be seen in the above picture of ![ y = ln\(x\) ](../../../upload.wikimedia.org/math/7/c/7/7c72906369801c8bfb7668b73c743b5a.png), which changes from negative to the left of the root ![ x = 1 ](../../../upload.wikimedia.org/math/a/2/5/a255512f9d61a6777bd5a304235bd26d.png) to positive to its right.

The bisection method works by taking the observation that a function changes sign between two points, and narrowing the interval in which the sign change occurs until the root contained within is tightly enclosed. This only works for a [continuous](http://en.wikipedia.org/wiki/continuous) function, in which there are no jumps or holes in the graph, but a large number of commonly-used functions are like this including logarithms (for positive numbers), sine and cosine, and polynomials.

As a more formalized explaination, consider a function ![ y = f\(x\) ](../../../upload.wikimedia.org/math/7/c/1/7c1c9491ba7c6e8d6d2cfa82e39b22ca.png) that changes sign between ![ x = a ](../../../upload.wikimedia.org/math/5/0/a/50a20ce04c291ac897290a4f2e3bc9e2.png) and ![ x = b ](../../../upload.wikimedia.org/math/e/8/9/e89da0c0c73b56b18d920816a6df9829.png) We can narrow the interval by:

  1. Evaluating the function at the midpoint
  2. Determining whether the function changes signs or not in each sub-interval
  3. If the continuous function changes sign in a sub-interval, that means it contains a root, so we keep the interval.
  4. If the function does not change sign, we discard it. This can potentially cause problems if there are two roots in the interval,so the bisection method is not guaranteed to find ALL of the roots.

Though the bisection method is not guaranteed to find _all_ roots, it **is** guaranteed to find at least one if the original endpoints had opposite signs.

The process above is repeated until you're as close as you like to the root.

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Find the root of ![ y = x+ln\(x\) ](../../../upload.wikimedia.org/math/f/9/2/f925a6f53eaa8f2a21e29d01e9573ec8.png) using the bisection method

By plugging in some numbers, we can find that the function changes sign between ![ x = 0.5 ](../../../upload.wikimedia.org/math/5/6/0/5606d8595e685914dd73e078734671cb.png) ![\(y = -0.193\)](../../../upload.wikimedia.org/math/5/0/4/5046de0e13d50a72e9da470b7f4bfee4.png) and ![ x = 1 ](../../../upload.wikimedia.org/math/a/2/5/a255512f9d61a6777bd5a304235bd26d.png) ![\(y = 1\) ](../../../upload.wikimedia.org/math/9/1/6/916b1450414b3fc44d2a655f178117b0.png). Therefore, since the function is continuous, there must be at least one root in this interval.

  * First Interval: ![ 0.5\(-\) < x < 1\(+\) ](../../../upload.wikimedia.org/math/3/b/7/3b7feddb26643ea8acbaf0847ed7c86f.png)
  * Midpoint: ![ x = 0.75 ](../../../upload.wikimedia.org/math/d/e/1/de12b6de22decaa35e24694ee335bd6d.png)
  * y at midpoint: ![ y = 0.75 + ln\(0.75\) = 0.462 ](../../../upload.wikimedia.org/math/2/e/0/2e01a1a70f51bb30c3f46df5b1fef350.png) Therefore, the sign changes between 0.5 and 0.75 and does not between 0.75 and 1.
  * New Interval: ![ 0.5\(-\) < x < 0.75\(+\) ](../../../upload.wikimedia.org/math/a/5/f/a5f1b71bd355b1ebc4b900c0db1eed36.png)
  * Midpoint: ![ x = 0.625 ](../../../upload.wikimedia.org/math/b/5/4/b5477997d5d70b6d4fbba3c6fa6f0335.png)
  * y at midpoint: ![ y = 0.155 ](../../../upload.wikimedia.org/math/2/f/1/2f1e104fdb7b4931b6df6c02af1fbddb.png)
  * New Interval: ![ 0.5\(-\) < x < 0.625\(+\) ](../../../upload.wikimedia.org/math/2/a/3/2a3d3fabf682876cfa803b20d19c52d0.png)
  * Midpoint: ![ x = 0.5625 ](../../../upload.wikimedia.org/math/7/3/a/73a244758fc499ce495ecfd77a483450.png)
  * y at midpoint: ![ y = -0.0129 ](../../../upload.wikimedia.org/math/b/8/6/b86ab2ba4eaaa5ea9902e4429e2bbc40.png)

We could keep doing this, but since this result is very close to the root, lets see if there's a number smaller than 0.625 which gives a positive function value and save ourselves some time.

  * x Value: ![ x = 0.57 ](../../../upload.wikimedia.org/math/0/6/9/0691b7edda102bd98f0cb9cc7cdda393.png)
  * y value: ![ y = 0.00788 ](../../../upload.wikimedia.org/math/4/5/1/451fbaa0558b3709221c0df981b7a66c.png)

Hence x lies between 0.5625 and 0.57 (since the function changes sign on this interval).

Note that convergence is slow but steady with this method. It is useful for refining crude approximations to something close enough to use a faster but non-guaranteed method such as weighted iteration.

### Regula Falsi

The Regula Falsi method is similar the bisection method. You must again start with two x values between which the function f(x) you want to find the root of changes. However, this method attempts to find a better place than the midpoint of the interval to split it.It is based on the hypothesis that instead of arbitrarily using the midpoint of the interval as a guide, we should do one extra calculation to try and take into account the _shape_ of the curve. This is done by finding the secant line _between two endpoints_ and using the root of that line as the splitting point.

More formally:

  * Draw or calculate the equation for the line between the two endpoints (a,f(a)) and (b,f(b)).
  * Find where this line intersects the x-axis (or when y = 0), giving you x = c
  * Use this x value to evaluate the function, giving you f(c)
  * The sub-intervals are then treated as in the bisection method. If the sign changes between f(a) and f(c), keep the inteval; otherwise, throw it away. Do the same between f(c) and f(b).
  * Repeat until you're at a desired accuracy.

Use these two formulas to solve for the secant line y = mx + B:

  
![ m = \\frac{f\(b\)-f\(a\)}{b-a} ](../../../upload.wikimedia.org/math/2/6/2/26293f405ff9237e7bd74665f07bb09e.png)

![ B = f\(b\)-m*b = f\(a\) - m*a ](../../../upload.wikimedia.org/math/3/2/1/3216d35ff6575e5e35e333a120cee59f.png) (you can use either)

The regula falsi method is guaranteed to converge to a root, but it may or may not be faster than the bisection method, depending on how long it takes to calculate the slope of the line and the shape of the function.

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Find the root of ![ x + ln\(x\) = 0 ](../../../upload.wikimedia.org/math/c/d/f/cdf5a99ba99ba90460d6821726da16df.png) but this time use the regula falsi method.

**Solution**: Be careful with your bookkeeping with this one! It's more important to keep track of y values than it was with bisection, where all we cared about was the sign of the function, not it's actual value.

For comparison with bisection, let's choose the same initial guesses: ![ a = 0.5 ](../../../upload.wikimedia.org/math/4/6/5/465708e0239e986884321b2cee5a0071.png) and ![ b = 1 ](../../../upload.wikimedia.org/math/3/c/9/3c94d884933477acdc14fc70da4b987a.png), for which ![ f\(a\) = -0.693 ](../../../upload.wikimedia.org/math/d/0/0/d00f89811b94dced4fd8a5bbd8545fde.png) and ![ f\(b\) = 1 ](../../../upload.wikimedia.org/math/4/b/8/4b81a5f4a69659fbf6549e334e759b28.png).

  * First interval: ![ 0.5 < x < 1, -0.193\(-\) < f\(x\) < 1\(+\) ](../../../upload.wikimedia.org/math/4/3/a/43a59ca3fe23bd4f390886d13d10d7e6.png)
  * Secant line: ![ y = 2.386x - 1.386 ](../../../upload.wikimedia.org/math/2/4/a/24add2532174709642dd02701324accd.png)
  * Root of secant line: ![ x = 0.581 ](../../../upload.wikimedia.org/math/c/d/7/cd748d6f79b5516ee85d95a0f784c14e.png)
  * Function value at root: ![ f\(x\) = 0.581 + ln\(0.581\) = 0.038\(+\) ](../../../upload.wikimedia.org/math/1/7/a/17a251eeec3c56f0b79211ce69c5467b.png)

Notice that in this case, we can discard a MUCH larger interval than with the bisection method (which would use ![ x = 0.75 ](../../../upload.wikimedia.org/math/d/e/1/de12b6de22decaa35e24694ee335bd6d.png) as the splitting point)

  * Second interval: ![ 0.5 < x < 0.581, -0.193\(-\) < f\(x\) < 0.038\(+\) ](../../../upload.wikimedia.org/math/c/d/4/cd43a61549c04a9ae3ef1a842da77576.png)
  * Secant line: ![ y = 2.852x - 1.619 ](../../../upload.wikimedia.org/math/e/0/d/e0da3c7285d26407bfb5298ca9bc7692.png)
  * Root of secant line: ![ x = 0.5676 ](../../../upload.wikimedia.org/math/6/3/6/636b121089e61fc77121584b34ccc08b.png)
  * Function value at root: ![ f\(x\) = 0.0013 ](../../../upload.wikimedia.org/math/e/0/7/e0791503aa79ea4debd1610f75d9b3cd.png)

We come up with practically the exact root after only two iterations!

In some cases, the regula falsi method will take longer than the bisection method, depending on the shape of the curve. However, it generally worth trying for a couple of iterations due to the drastic speed increases possible.

### Tangent Method (Newton's Method)

In this method, we attempt to find the root of a function y = f(x) using the _tangent_ lines to functions. This is similar to the secant method, except it "cuts loose" from the old point and only concentrates on the new one, thus hoping to avoid hang-ups such as the one experienced in the example.

Since this class assumes students have not taken calculus, the tangent will be approximated by finding the equation of a line between two very close points, which are denoted (x) and ![ \(x+\\delta x\) ](../../../upload.wikimedia.org/math/f/4/d/f4d23a4a6f0282d2e719f63bd36ecb6b.png). The method works as follows:

  1. Choose **one** initial guess, ![ x_1 ](../../../upload.wikimedia.org/math/f/9/a/f9a3b8e9e501458e8face47cae8826de.png)
  2. Evaluate the function f(x) at ![ x = x_1 ](../../../upload.wikimedia.org/math/7/5/2/7521e73e2bb453fc1fcaf52ce5525af8.png) and at ![ x = x_1 + \\delta x ](../../../upload.wikimedia.org/math/5/b/8/5b881241ec0ff1b09b14cf02c87a6bbe.png) where ![ \\delta x ](../../../upload.wikimedia.org/math/5/8/b/58b1e9f4b9a86690cb106e65a265a34f.png) is a small number. These yield two points on your (approximate) tangent line.
  3. Find the equation for the tangent line using the formulas given above.
  4. Find the root of this line. This is ![ x_2 ](../../../upload.wikimedia.org/math/8/f/4/8f43fce8dbdf3c4f8d0ac91f0de1d43d.png)
  5. Repeat steps 2-4 until you're as close as you like to the root.

This method is **not guaranteed to converge** unless you start off with a good enough first guess, which is why the guaranteed methods are useful for generating one. However, since this method, when it converges, is much faster than any of the others, it is preferable to use if a suitable guess is available.

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Find the root of ![ x + ln\(x\) = y ](../../../upload.wikimedia.org/math/1/f/3/1f39ba6f09f2d862a436c4104b565063.png) using the tangent method.

**Solution**: Let's guess ![ x_1 = 0.5 ](../../../upload.wikimedia.org/math/a/9/7/a976958a5c1e1a5fc00b8f9a8bcf33b0.png) for comparison with iteration. Choose ![ \\delta\(x\) = 0.001 ](../../../upload.wikimedia.org/math/a/8/c/a8c9dabb501f2fa06f6553851e3404d1.png)

  * ![ f\(x_1\) = f\(0.5\) = -0.193 ](../../../upload.wikimedia.org/math/9/c/e/9cece855e3128792d70fc47a633b3f49.png)
  * ![ f\(x_1 + \\delta x\) = f\(0.501\) = -0.190 ](../../../upload.wikimedia.org/math/a/6/1/a61a95d325de64c9123f9b8e80aa25ff.png)
  * Tangent line: ![ y = 2.85x - 1.618 ](../../../upload.wikimedia.org/math/6/7/a/67afe96ac48063fda5d96feac6f784a5.png)
  * Root of tangent line: ![ x = 0.5677 ](../../../upload.wikimedia.org/math/0/0/a/00ac32004b3c49050208e62f50b4baae.png)

Already we're as accurate as any other method we've used so far after only **one** calculation!

## What is a System of Equations?

A **system** of equations is any number of equations with more than one total unknown, such that the same unknown must have the same value in every equation. You have probably dealt a great deal, in the past, with _linear systems of equations_, for which many solution methods exist. A linear system is a system of the form:

  
**Linear Systems**

![ C_1 = a_1x_1 + a_2x_2 + ... ](../../../upload.wikimedia.org/math/9/b/a/9bab407e4c69c813feea9aecc2fbda1d.png)

![ C_2 = b_1x_1 + b_2x_2 + ... ](../../../upload.wikimedia.org/math/9/4/0/940870d183a894c094b0374c4a60b36b.png)

And so on, where the a's and b's are constant.

Any system that is not linear is **nonlinear**. Nonlinear equations are, generally, far more difficult to solve than linear equations but there are techniques by which some special cases can be solved for an exact answer. For other cases, there may not be any solutions (which is even true about linear systems!), or those solutions may only be obtainable using a _numerical method_ similar to those for single-variable equations. As you might imagine, these will be considerably more complicated on a multiple-variable system than on a single equation, so it is recommended that you use a computer program if the equations get too nasty.

## Solvability

A system is **solvable** if and only if there are only a finite number of solutions. This is, of course, what you usually want, since you want the results to be somewhat predictable of whatever you're designing.

Here is how you can tell if it will _definitely_ be impossible to solve a set of equations, or if it merely _may_ be impossible.

**Solvability of systems**:

  1. If a set of n **independent** equations has n unknowns, then the system has a finite (possibly 0) number of solutions.
  2. If a set of n **independent** equations has _less than_ n unknowns then the system has an infinite number of solutions.
  3. If a set of n **independent or dependent** equations has _more than_ n unknowns then the system has no solutions.
  4. Any dependent equations in a system do not count towards n.

Note that even if a system is solvable it doesn't mean it has solutions, it just means that there's not an infinite number.

## Methods to Solve Systems

As you may recall there are many ways to solve systems of _linear_ equations. These include:

  * **Linear Combination**: Add multiples of one equation to the others in order to get rid of one variable. This is the basis for [Gaussian elimination](http://en.wikipedia.org/wiki/Gaussian_elimination) which is one of the faster techniques to use with a computer.
  * **[Cramer's rule](http://en.wikipedia.org/wiki/Cramer%27s_rule)** which involves determinants of coefficient matrices.
  * **Substitution**: Solve one equation for one variable and then substitute the resulting expression into all other equations, thus eliminating the variable you solved for.

The last one, substitution, is most useful when you have to solve a set of **nonlinear** equations. Linear combination can only be employed if the same type of term appears in all equations (which is unlikely except for a linear system), and no general analogue for Cramer's rule exists for nonlinear systems. However, substitution is still equally valid. Let's look at a simple example.

### Example of the Substitution Method for Nonlinear Systems

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  

    
    
    Solve the following system of equations for X and Y
    

  1. ![ X + Y^2 = 4 ](../../../upload.wikimedia.org/math/7/4/2/7428e7489283f73ab2637e20af489e26.png)
  2. ![ X^2 - Y^2 = 22 ](../../../upload.wikimedia.org/math/5/4/9/549d757449b0856f0fa4cd8130b5a543.png)

Solution: We want to employ substitution, so we should ask: **which variable is easier to solve for?**. In this case, X (in the top equation) is easiest to solve for so we do that to obtain:

![ X = 4 - Y^2 ](../../../upload.wikimedia.org/math/7/d/6/7d6593815a5be89b5871eaa059af8759.png)

Substituting into the bottom equation gives:

![ \(4 - Y^2\)^2 - Y^2 = 22 ](../../../upload.wikimedia.org/math/e/3/4/e346ed1425d68bdd8940bc9d68b0a59d.png)

![ 4 - 8Y^2 + Y^4 - Y^2 = 22 ](../../../upload.wikimedia.org/math/d/d/b/ddb5b8ca1c832e14cf31107b20778455.png)

![ Y^4 - 9Y^2 - 18 = 0 ](../../../upload.wikimedia.org/math/0/6/9/069e12248e2ba89e90d9fc5430d83e44.png)

This can be solved by the **method of substitution**:

Let ![ U = Y^2 ](../../../upload.wikimedia.org/math/8/8/b/88b0b296cbb2b4589c155b7844221466.png). Plugging this in:

![ U^2 - 9U - 18 = 0 ](../../../upload.wikimedia.org/math/a/7/b/a7bbeaf877337d8a60fb93465eb3e590.png)

**Note:**  
All Ys must be eliminated for this method to be valid. Do something like this when the same variable (or set of variables) appears _in the same for every time_. If it's not in the same form **every time**, i.e. if the equation was something like ![ Y^4 - 9Y^2 - 18 + e^Y ](../../../upload.wikimedia.org/math/7/d/a/7da67a170d5332e9a43a39762b86bf52.png), then the method would not simplify your calculations enough to make it worth doing.

Solving by factoring:

![ \(U - 6\)\(U - 3\) = 0 ](../../../upload.wikimedia.org/math/9/f/a/9fa9216373d0d80fce9cf366a226ac35.png)

![ U = 3,6 ](../../../upload.wikimedia.org/math/b/d/8/bd857ac82970ed573a2c7c2b6d35007f.png)

Thus since ![ U = Y^2 ](../../../upload.wikimedia.org/math/8/8/b/88b0b296cbb2b4589c155b7844221466.png) we obtain **four** solutions for Y!

  
![ Y = \\pm \\sqrt{3}, Y=\\pm \\sqrt{6} ](../../../upload.wikimedia.org/math/d/e/8/de8dbef886db29d5b1ff4a10573e8642.png)

Notice, however, that depending on where this system _came_ from, the negative solutions may not make sense, so think before you continue!

Let's take into account all of them for now. Since we have Y we can now solve for X:

![ X = 4 - Y^2 = 4 - U ](../../../upload.wikimedia.org/math/d/2/0/d20ae502c4312d9d0543817f1e179196.png)

  

    
    
    ![X = 1,-2](../../../upload.wikimedia.org/math/4/c/2/4c21f7f54bff7dd44a83f37e5d73e661.png)
    

**Note:**  
Again, it may be true that only positive values of X make sense If only positive values of X and Y make sense then the ONLY solution that is of any worth to us is the solution ![ \(X,Y\) = \(1,\\sqrt{3}\) ](../../../upload.wikimedia.org/math/0/a/c/0ac3cb323fd6ca706c1ef8675631fe84.png) since ![ Y = \\sqrt{6} ](../../../upload.wikimedia.org/math/b/9/5/b95fc115e722e052f25cd0225900bc7a.png) results in a negative value for X.

Notice that even a small system like this has a large number of solutions and, indeed, some systems will have an infinite number, such as:

  1. ![ y = sin\(x\) ](../../../upload.wikimedia.org/math/6/b/b/6bbf43da34d7a862191f1e1aa733f97c.png)
  2. ![ y = cos\(x\) ](../../../upload.wikimedia.org/math/e/0/4/e047a06fcdef4077b763eb2d88d4eedc.png)

## Numerical Methods to Solve Systems

There are numerical equivalents in multiple variables to _some_ of the methods demonstrated in the previous section. Many of them in their purest forms involve the use of calculus (in fact, the Taylor method does as well), but as before, they can be reduced to approximate algebraic forms at the expense of some accuracy.

### Shots in the Dark

If you can solve all of the equations explicitly for the same variable (say, y) you can guess all but one and then compare how different the resulting values of y are in each equation. This method is entirely brute-force, because **if there are more than two equations, it is necessary to guess all of the variables but one using this method**, and there is no way to tell what the next guess should be. Trying to guess multiple variables at once from thin air gets to be a hastle even with a computer.

Since there are so many problems with this method, it will not be discussed further

### Fixed-point iteration

Again, the multivariate form of fixed-point iteration is so unstable that it generally can be assumed that it will not work. Weighted iteration is also significantly more difficult.

### Looping method

This is one method that _does_ work, and that is somewhat different from any single-variable method. In the looping method technique, it is necessary to be able to solve _each equation for a unique variable_, and then you'll go around in a loop essentially, starting with an initial guess on (ideally) a _single_ variable, say y, and then evaluating all equations until you return to your original variable with a new value y'. If the result is not the same as the guess(es) you started with, you need to make a new guess based on the _trends in the results_.

**Note:**  
What kind of trends am I talking about? If you have a well-behaved system, an increase in y will consistently lead to either an increase or a decrease in y', so you can take advantage of this to see which way you need to adjust your original guess. DO NOT attempt to use the value for y' as a new guess!

More specifically, here is an algorithm you can use:

  1. Solve all equations for a **different variable**.
  2. Make a guess on one variable (or as many as necessary to evaluate a second one, if it's more than one it gets harder though, so it is recommended to use another method)
  3. Go through all of the equations until you end up recalculating the variable (or all of the variables) which you had originally guessed. Note whether the result is higher or lower than your guess.
  4. Make another guess on the variable(s). Go through the loop again.
  5. After these two guesses, we know whether increasing or guess will increase or decrease the recalculated value. Therefore, we can deduce whether we need to increase or decrease our guess to get a recalculated value equal to the guess.
  6. Keep guessing appropriately until the recalculated value equals the guess.

This technique is often necessary in engineering calculations because they are based on data, not on explicit equations for quantities. As we'll see, however, it can be difficult to get it to converge, and this method isn't that convenient to do by hand (though it is the most reliable one to do realistically). It is great, however, for inputting guesses into a computer or spreadsheet until it works.

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Solve this system:

  1. ![ y = e^{-x} ](../../../upload.wikimedia.org/math/6/c/4/6c4f276e90ca0ebf24e7ced07209d12b.png)
  2. ![ y = ln\(x\) ](../../../upload.wikimedia.org/math/7/c/7/7c72906369801c8bfb7668b73c743b5a.png)

First we need to solve one of them for x, let's choose the first one:

![ x = -ln\(y\) ](../../../upload.wikimedia.org/math/f/8/1/f81167e1eaf3645f87dfb79dbcf8ecbe.png)

  * To start off, we make a guess: y = 0.1 Then from the first equation, x = 2.303
  * Plug this back into the second equation and you'll come out with y' = 0.834. The recalculated value is **too high**.
  * Now make a new guess on y: say, y = 0.5. This results in x = 0.6931
  * Plugging back into the second equation gives y' = -0.3665. The recalculated value is **too low**.

**Note:**  
Now we know that increasing the guess decreases the recalculated value y' and vice versa. Since the second value of y' is too low this means that we need the guess to be smaller than 0.5; likewise, since the first y' was too high we need it to be greater than 0.1.

  * Lets now try y = 0.25.
  * This results in x = 1.386 from the first equation and y' = 0.326 from the second. **Too high** so we need to increase our guess.
  * Let's guess y = 0.3
  * This yields x = 1.204 and thus y' = 0.185, which is **too low** indicating the guessed value was too high.
  * Guess y = 0.28, hence x = 1.273 and y'= 0.241. The guess is therefore still too high.
  * Guess y = 0.27, hence x = 1.309 and y' = 0.269. Therefore we have now converged:

  
![ x = 1.309, y = 0.27 ](../../../upload.wikimedia.org/math/b/b/1/bb1a5986fa6f9164ece3e51c543bd93f.png)

#### Looping Method with Spreadsheets

We can do the guessing procedure more easily by programming it into a spreadsheet. First set up three rows like so:
    
    
         A         B      C
    1  y guess     x      y'
    2           =-ln(A2) =ln(B2)
    

In B2 we put the first function solved for x, and in C2 we have the second function solved for y. Now all we need to do is type in guesses in A2 until the value in C2 is the same as our guess (the spreadsheet will automatically calculate B2 and C2 for you). To make things even easier, put the line ![ =A2 - C2 ](../../../upload.wikimedia.org/math/2/d/c/2dc34aa6c9d43457fd48c70b67ae6010.png) into cell D2. Since we want y' to equal y, just keep guessing until the value in D2 is as close to zero as you like.

As a more in-depth example (which would be significantly more difficult to do by hand), consider the system:

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Example:**  


Solve:

  * ![ T = \\frac{2P^2X^2 - 3e^{-X/T}}{T-2} ](../../../upload.wikimedia.org/math/8/9/d/89d2775eb8d83c7cfe374add2f57e523.png)
  * ![ X^2 = T^3 - P ](../../../upload.wikimedia.org/math/0/f/9/0f9d3fe29767faa0506186ed921fea4c.png)
  * ![ 10P = T ](../../../upload.wikimedia.org/math/b/e/a/beadd3d2577e702896d440778cbc8d4e.png)

In order for this to work, we only need to solve each equation for a unique variable, the expression need not be explicit! The following will work (assuming that X is a positive quantity), and this will be evident shortly:

  * ![ T = \\frac{2*P^2*X^2 - 3*e^{-X/T}}{T-2} ](../../../upload.wikimedia.org/math/c/3/3/c33ace5681fa4c8a215fbed3a874c703.png)
  * ![ X = \\sqrt{T^3 - P} ](../../../upload.wikimedia.org/math/6/0/c/60c5791e084d9e79396c377c7d0e18d9.png)
  * ![ P = 0.1T ](../../../upload.wikimedia.org/math/0/5/f/05f8aa5307b26ceb8a1e62d1acceb37f.png)

Now we need to ask: which variable would be the best to guess to start the iteration procedure? In this case the best answer is T because from this guess, we can calculate P from equation 3, then X from equation 2, and finally a new guess on T from equation 1, and use this new value as a gauge of our old guess.

**Note:**  
Generally you want to start the loop with a variable that allows you to calculate a second value with only that one guess. Try to algebraically manipulate your equations so that this is the case before solving, because we want to avoid guessing on multiple variables if at all possible.

Lets program this into the spreadsheet:
    
    
          A                B           C                        D                                        E
    1   T guess            P           X                        T'                                    T' - T guess
    2                   =0.1*A2    =sqrt(A2^3 - B2) =(2*B2^2*C2^2 - 3*exp(-C2/A2))/(A2 - 2)          =D2 - A2
    

Once all this is programmed in, you can just input guesses as before, with the eventual result that:

  
![ P = 0.2453, X = 3.8098, T = 2.453 ](../../../upload.wikimedia.org/math/1/f/5/1f55d01c02f1afbcc17fc25b1e162589.png)

### Multivariable Newton Method

**Note:**  
You may want to skip this section if you don't know how to invert matrices, add them, or multiply them.

There is a multivariate extension to [Newton's method](http://en.wikipedia.org/wiki/Newtons_method) which is highly useful. It converges quickly, like the single-variable version, with the downside that, at least by hand, it is tedious. However, a computer can be programmed to do this with little difficulty, and the method is not limited only to systems which can be explicitly solved like the looping method is. In addition, unlike the looping method, the Newton method will actually give you the next set of values to use as a guess.

The method works as follows:

1\. Solve all of the equations for 0, i.e. let ![ 0 = F\(x_1, x_2, ...\) ](../../../upload.wikimedia.org/math/0/d/b/0db19d5e7dbb3dc8e2e1856fb8463815.png) for all functions F in the system.

2\. Guess a value for all variables, and put them into a matrix (X). Calculate the value of all functions F at this guess, and put them into a matrix (F).

3\. We need to find estimates for all the **partial derivatives** of the function at the guessed values, which is described later.

4\. Construct a matrix (to become the Jacobian) as follows: make an empty matrix with n rows and n columns, where n is the number of equations or the number of variables (remember, a solvable system generally has the same number of equations as variables. Then label the columns with the names of variables and the rows with the names of your functions. It should look something like this:

![ \\begin{bmatrix} 
 --& x_1& y_2 & ... \\\\
F_1 &  & &  \\\\
 F_2&  & & \\\\
... & & & \\\\                    
\\end{bmatrix}. ](../../../upload.wikimedia.org/math/6/0/3/60398bd2b89423fbf0bd2d18b284a57a.png) 5\. Put the appropriate partial derivative in the labeled spot. For example, put the partial derivative with respect to x1 from function 1 in the first spot.

6\. Once the Jacobian matrix is completely constructed, find the [inverse](http://en.wikipedia.org/wiki/Inverse) of the matrix. There are multiple computer programs that can do this including [this one](http://www.allworldsoft.com/software/4-087-matrix-inverse-calculator.htm) (WARNING:Not tested software, use at your own risk!). Or you can do it by hand if you know how.

7\. Matrix-multiply the inverse Jacobian with the transpose function matrix F (to make it a column matrix), then subtract this from the the transposition of X (again, make it a column matrix):

  
![ X_{n+1}^T = X{n}^T - J^{-1}*F_n^T](../../../upload.wikimedia.org/math/7/5/6/7568e73cdc9de70819be5e62fae49659.png)

**Multivariable Newton Method Formula**

8\. The result is your next guess. Repeat until convergence.

#### Estimating Partial Derivatives

![warning](../../../upload.wikimedia.org/wikipedia/commons/thumb/d/dc/Nuvola_apps_important_yellow.svg/40px-Nuvola_apps_important_yellow.svg.png)

You MUST make sure you carry out quite a few decimal places when doing this, because changing the variables by a very small amount may not change the function values too much, but even small changes are important!

A **Partial derivative** is, in its most basic sense, the slope of the tangent line of a function with more than one variable when all variables except one are held constant. The way to calculate it is:

Now we need to stay organized, so let's introduce some notation:

  
![ \\frac{\\delta F_i}{\\delta x_j} ](../../../upload.wikimedia.org/math/c/d/7/cd749e3d2983fafef10980193aded338.png) is the partial derivative of function i with respect to variable j.

To calculate it:

  1. Calculate one function F at your guess.
  2. Increase _one_ variable, x, by a very small amount ![ \\delta ](../../../upload.wikimedia.org/math/f/1/0/f10f03c9836c36537d2539196058bfa2.png). **Leave all other variables constant**.
  3. Recalculate F at the modified guess to give you F'.

The partial derivative of the function F with respect to x is then ![ \\frac{\\delta F_1}{\\delta x} = \\frac{F_1\(x+\\delta,y\) - F_1\(x,y\)}{\\delta} ](../../../upload.wikimedia.org/math/8/0/9/8099f0625a2bb31dcf64262a05137e02.png).

#### Example of Use of Newton Method

Let's go back to our archetypal example:

  * ![ y = e^{-x} ](../../../upload.wikimedia.org/math/6/c/4/6c4f276e90ca0ebf24e7ced07209d12b.png)
  * ![ y = ln\(x\) ](../../../upload.wikimedia.org/math/7/c/7/7c72906369801c8bfb7668b73c743b5a.png)

**Step 1**: We need to solve each for zero:

  * ![ F_1 = 0 = e^{-x} - y ](../../../upload.wikimedia.org/math/f/1/e/f1ed80bc9106a119572ddce2fb357478.png)
  * ![ F_2 = 0 = ln\(x\) - y ](../../../upload.wikimedia.org/math/f/3/9/f39c6a3d7dd65462e7a2c527690d6035.png)

**Step 2**: Lets guess that x = 2.303 and y = 0.1 (it's a good idea to choose guesses that satisfy one of the equations). Then:

![ X = \[2.303, 0.1\] ](../../../upload.wikimedia.org/math/e/5/2/e520bc31ca63a8108d5a637501d60d00.png)

The values of F at this guess are ![ F_1 = 0, F_2 = 0.734213 ](../../../upload.wikimedia.org/math/0/8/a/08aa9d1da4ec09cdead2f240dcb96dc6.png), and hence by definition:

![ F = \[0, 0.724213\] ](../../../upload.wikimedia.org/math/c/d/c/cdc7910cd67b9b17e0a01edc9b957145.png)

**Step 3-5**: Calculate the partial derivatives Lets choose ![ \\delta = 0.01 ](../../../upload.wikimedia.org/math/b/c/3/bc3c2458cb142b29258c429c2c6e4ad9.png). Then:

  * ![ \\frac{\\delta F_1}{\\delta x} = \\frac{F_1\(x+\\delta,y\) - F_1\(x,y\)}{\\delta} ](../../../upload.wikimedia.org/math/8/0/9/8099f0625a2bb31dcf64262a05137e02.png)
  * ![ = \\frac{\(e^{-2.303+0.01}-0.1\) - \(e^{-2.303}-0.1\)}{0.01} = -0.1036 ](../../../upload.wikimedia.org/math/1/5/e/15e4dbe766600e4d09ab4031085e4bec.png)
  * ![ \\frac{\\delta F_1}{\\delta y}= \\frac{F_1\(x,y+\\delta\) - F_1\(x,y\)}{\\delta} ](../../../upload.wikimedia.org/math/7/b/d/7bdf4ed3ca27e832d07ab0bdc4d1347a.png)
  * ![ = \\frac{\(e^{-2.303}-\(0.1+0.01\)\) - \(e^{-2.303}-0.1\)}{0.01} = -1 ](../../../upload.wikimedia.org/math/a/d/7/ad734b8952c5d09810a5e3bd997493ea.png)

The partial derivatives of F2 can be similarly calculated to be ![ {\\delta F_2}{\\delta x} = 0.433 ](../../../upload.wikimedia.org/math/5/3/7/537e306da6b494931fd2cc4d71165811.png) and ![ {\\delta F_2}{\\delta y} = -1 ](../../../upload.wikimedia.org/math/4/4/3/44337156353b052bf7ce1e312d3c444a.png)

Therefore, the Jacobian of the system is:

![ \\begin{bmatrix} 
 --& x& y \\\\
F_1 & -0.1036 & -1 \\\\
 F_2& +0.433 & -1 \\\\                    
\\end{bmatrix}. ](../../../upload.wikimedia.org/math/c/b/2/cb29701c6efc847e61d142d8d7093873.png)

**Step 6**: Using any method you know how to do, you can come up with the inverse of the matrix:

![ J^{-1} = \\begin{bmatrix} 
 --& x& y \\\\
F_1 & -1.3636 & 1.8636 \\\\
 F_2& -0.8069 & -0.1931 \\\\                    
\\end{bmatrix}. ](../../../upload.wikimedia.org/math/b/b/9/bb91f4bcbccc6068db00d60fc7b31b0a.png)

**Step 7**: The transposition of F is simply:

![F^T = \\begin{bmatrix} 
 F \\\\
 0\\\\
 0.734213 \\\\                    
\\end{bmatrix}. ](../../../upload.wikimedia.org/math/4/1/e/41ee6fb47c56a7c57224692d5cefe6a8.png)

Therefore by doing matrix multiplication you can come up with the following _modifying matrix_:

![ J^{-1} * F^T = \\begin{bmatrix} 
 \\Delta \\\\
 1.3682\\\\
 0.1418 \\\\                    
\\end{bmatrix}.](../../../upload.wikimedia.org/math/9/c/3/9c3e28645284763033c4847191eeee01.png)

Therefore, we should subtract 1.3682 from x and 0.1418 from y to get the next guess:

![ x = 0.9373, y = 0.2418 ](../../../upload.wikimedia.org/math/3/c/1/3c152b613fad8a5e8c82add435f4eb7b.png)

Notice how much closer this is to the true answer than what we started with. However, this method is generally better suited to a computer due to all of the tedious matrix algebra.

## Linearization

In the [statistics section](http://en.wikibooks.org/wiki/Introduction_to_Chemical_Engineering_Processes/Basic_Statistics_and_Data_Analysis), the concept of **linearization** was introduced as an extension to linear regression. It is discussed here in terms of plotting. Linearization is particularly useful because it allows an engineer to easily tell whether a simple model (such as an exponential model) is a good fit to data, and to locate outliers.

In order to linearize nonlinear data, it is necessary to **assume** a model that can be linearized. Some requirements for a linearizable function are:

  1. The equation must be _separable_ (i.e. you must be able to put all instances of y or functions of y on one side of the equation, and all instances of x or functions of x on the other)
  2. You must be able to express the function in the form ![ f\(y\) = A* f\(x\) + B ](../../../upload.wikimedia.org/math/9/7/3/973b69c4fa75c65e7998ea1fca9118a4.png)
  3. Simple linearization can only provide a maximum of two constant values (A and B).

Some examples of functions that can be linearized and their linear forms are given in the statistics section. Here is a summary:

    ![ y = A*b^x \\rightarrow ln\(y\) = ln\(A\) + x*ln\(b\) ](../../../upload.wikimedia.org/math/5/e/6/5e6791161f8d7dc594bf461da1f726a6.png) (exponential model, ![ f\(y\) = ln\(y\); f\(x\) = x ](../../../upload.wikimedia.org/math/e/d/3/ed39babd49bf08414136b1b3bb72b5a0.png))
    ![ y = A*x^b \\rightarrow ln\(y\) = ln\(A\) + b*ln\(x\) ](../../../upload.wikimedia.org/math/d/a/1/da1faad521ec29c2941d1dfcbce2cadc.png) (power-law model, ![ f\(y\) = ln\(y\); f\(x\) = ln\(x\) ](../../../upload.wikimedia.org/math/8/d/0/8d0368f68764c487daf698cff19c1b40.png))
    ![ y = \\frac{k_1*x}{k_2 + x} \\rightarrow \\frac{1}{y} = \(\\frac{k_2}{k_1}\)*\\frac{1}{x} + \\frac{1}{k_1}](../../../upload.wikimedia.org/math/3/4/a/34af926e4f3b5516f6dafaee5585bd95.png) (Michaelis-Menton)

![Clipboard](../../../upload.wikimedia.org/wikipedia/commons/thumb/1/1f/Clipboard.svg/45px-Clipboard.svg.png)

**To do:**  
Show some more examples of linearizable functions

An effective method to check the validity of an assumed model is to first evaluate f(y) and f(x) at all data points and then plot f(y) vs. f(x). Spreadsheets are ideal for this computation. If the model assumed is correct, the plot of f(y) vs. f(x) should be linear and randomly scattered around the plot. If the assumed model is _known_ to be correct, the linear plot can be used to quickly identify outlying data points.

![Clipboard](../../../upload.wikimedia.org/wikipedia/commons/thumb/1/1f/Clipboard.svg/45px-Clipboard.svg.png)

**To do:**  
Show a diagram of this

## Log-log and semi-log plots

In engineering, exponential and power-law models show up so often that they have their own special paper that can be used to graph them easily. They are called **semilog** paper and **log-log** paper, respectively, because of the linearized forms of the exponential and power-law functions. These types of paper use a base-10 logarithm (which differs from a natural logarithm by a constant) because it is easier to visualize a base-10 than it is to visualize a base-e.

![Clipboard](../../../upload.wikimedia.org/wikipedia/commons/thumb/1/1f/Clipboard.svg/45px-Clipboard.svg.png)

**To do:**  
Explain more about the log-log and x-log plots, show some diagrams

## Parity Plots

Often, in engineering analysis, there will be a theoretical value of a parameter (for example, the outlet temperature of a reactor calculated from an energy balance), and there will be an actually-measured value. It is often desirable to compare them. One easy graphical way to do this is with a **parity plot**. In a parity plot, one plots the measured values against the experimental values (for the same trial). The y=x line is also plotted as a reference. If the theoretical and experimental values agree, they should lie close to the y=x line and be randomly scattered around it. If they do not (due to either a problematic assumption in the theory, errors in measurement, or both), then the data will be skewed away from the y=x line. This is also useful for identifying outlying measurements.

In addition to checking for actual agreement, a parity plot can be used to tell if the theoretical and experimental values are at least correlated (in which case the plot would be linear, even if not near the y=x line).

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Problem:**  


**1.** In enzyme kinetics, one common form of a rate law is [Michaelis-Menten kinetics](http://en.wikipedia.org/wiki/Michaelis-Menten_kinetics), which is of the form:

![ -r_S = \\frac{V_{max}*\[S\]}{K_m + \[S\]} ](../../../upload.wikimedia.org/math/a/c/8/ac88cac35eab90c8495d98bb78dca278.png)

where ![ V_{max} ](../../../upload.wikimedia.org/math/b/2/3/b236079a9dbb1032f7de25af5a3fb61b.png) and ![ K_m ](../../../upload.wikimedia.org/math/b/7/5/b754d16bfd50c76edbefd48bc88fbfa6.png) are constants.

**a.** Write this equation in a linearized form. What should you plot to get a line? What will the slope be? How about the y-intercept?

**b.** Given the following data and the linearized form of the equation, predict the values of ![ V_{max} ](../../../upload.wikimedia.org/math/b/2/3/b236079a9dbb1032f7de25af5a3fb61b.png) and ![ K_m ](../../../upload.wikimedia.org/math/b/7/5/b754d16bfd50c76edbefd48bc88fbfa6.png)
    
    
    [S], M   rS, M/s
    0.02     0.0006
    0.05     0.0010
    0.08     0.0014
    0.20     0.0026
    0.30     0.0028
    0.50     0.0030
    0.80     0.0036
    1.40     0.0037
    2.00     0.0038
    

Also, calculate the R value and comment on how good the fit is.

**c.** Plot the rate expression in its nonlinear form with the parameters from part b. What might ![ V_max ](../../../upload.wikimedia.org/math/4/8/5/48535627e4f95ddca217ed83f835383f.png) represent?

**d.** Find the value of -rS when [S] is 1.0 M in three ways:

  1. Plug 1.0 into your expression for -rS with the best-fit parameters.
  2. Perform a linear interpolation between the appropriate points nearby.
  3. Perform a linear extrapolation from the line between points (0.5, 0.0030) and (0.8, 0.0036).

Which is probably the most accurate? Why?

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Problem:**  


**2.** Find the standard deviation of the following set of arbitrary data. Write the data in ![ \\mu \\pm \\sigma ](../../../upload.wikimedia.org/math/4/8/0/480cc119400471fd711bf1a9a391cc2a.png) form. Are the data very precise?
    
    
    1.01  1.00    0.86   0.93   0.95   
    1.1   1.04    1.02   1.08   1.12
    0.97  0.93    0.92   0.89   1.15
    

Which data points are most likely to be erroneous? How can you tell?

![Example](../../../upload.wikimedia.org/wikipedia/commons/thumb/9/93/Evolution-tasks.png/40px-Evolution-tasks.png)

**Problem:**  


**3.** Solve the following equations for x using one of the rootfinding methods discussed earlier. Note that some equations have multiple real solutions (the number of solutions is written next to the equation)

**a.** ![ x^2 - 14x + 15 = 0 ](../../../upload.wikimedia.org/math/d/1/2/d122a198ced8d6e092cc76452fc44217.png) (2 solutions). Use the quadratic formula to check your technique before moving on to the next problems.

**b.** ![ x^2 - 14x + 15 - ln\(x\) = 0 ](../../../upload.wikimedia.org/math/9/d/c/9dcbc770ba48ed8c26f29ba8f65381be.png) (1 solution)

**c.** ![ e^{3x} = -x ](../../../upload.wikimedia.org/math/8/e/9/8e9d4c87eb732049a9595526b7b6f99e.png) (1 solution)

**d.** ![ \\frac{x}{2x^2 - 3} - \\frac{2x^3-x^2}{2x-x^2} = 10 ](../../../upload.wikimedia.org/math/d/d/2/dd27d63cff95f7deec16bb3d72c7f3eb.png) (2 solutions)

# **Appendix 2: Problem Solving using Computers**

## Introduction to Spreadsheets

This tutorial probably works with other spreadsheets (such as [w:open office](http://en.wikipedia.org/wiki/open_office)) with minor modifications.

A **spreadsheet** such as Excel is a program that lets you analyze moderately large amounts of data by placing each data point in a **cell** and then performing the same operation on groups of cells at once. One of the nice things about spreadsheets is that data input and manipulation is relatively intuitive and hence easier than doing the same tasks in a programming language like MATLAB (discussed next). This section shows how to do some of these manipulations so that you don't have to by hand.

## Anatomy of a spreadsheet

A spreadsheet has a number of parts that you should be familiar with. When you first open up the spreadsheet program, you will see something that looks like this (the image is from the German version of open office)

![OOo-2.0-Calc-KDE-german.png](../../../upload.wikimedia.org/wikipedia/commons/3/38/OOo-2.0-Calc-KDE-german.png)

First off, notice that the entire page is split up into boxes, and each one is labeled. _Rows are labeled with numbers and columns with letters_. Also, try typing something in, and notice that the box above the spreadsheet (to the right of ![ \\Sigma = ](../../../upload.wikimedia.org/math/1/c/a/1cadcc88a4dae91af9f48887fd47f9b7.png)) will change automatically as you type. When you're just putting in numbers, this **info box** will just have the same number in there. But when you're putting in formulas, the cell will display the value calculated from the formula, while the info box will display what the formula was.

## Inputting and Manipulating Data in Excel

The first step in any spreadsheet analysis is to input the raw data you want to analyze. It is most effective if you put it in columns, with one column for each variable. It lets you see more data at once, and it also is less limited because the maximum number of rows is much larger than the maximum number of columns.

It is good practice to use the first row for the names of the variables, and the remaining for the data points. Make sure you include units. In this section, the following data will be used as illustration:

(Rownumber) Column A Column B

1
t(min)
D (yards)

2
1.1
559.5

3
1.9
759.5

4
3.0
898.2

5
3.8
1116.3

6
5.3
1308.7

  


### Using formulas

In order to tell the spreadsheet that you want to use a formula rather than just enter a number, you have to start the entry with an equal sign (=). You can then use combinations of decimal values and **cell designations**. A cell designation is simply the column letter followed by the row number containing the value you wish to manipulate. For example, if you wanted to find the product of the distance traveled and the time spent traveling, you could put in the formula:
    
    
    = A2*B2
    

into any empty cell and it would give you the answer. From here out it will be assumed that this value is in cell C2. You should label the column with the type of calculation you're performing.

### Performing Operations on Groups of Cells

The question may arise: why not just put in the numbers themselves instead of referencing the cell? There are two major reasons for this:

  1. If you change the value in the referenced cell, the value calculated in the formula will automatically change.
  2. The built-in **dragging** capability of most spreadsheets.

The dragging capability is a simple concept. If you have put a formula into a spreadsheet, you can have it _copied_ to any number of cells you want. To do this, select the cell with the formula and bring the mouse pointer to its lower-right hand corner. You should see a dark **+** icon:

**Info Bar**
"=A2*B2"

(rownum)
Column A
Column B
Column C

1
t(min)
D(yards)
t*D

2
1.1
559.5
  
\-------------  
| 625.45 |  
\------------**+**

3
1.9
759.5

4
3.0
898.2

5
3.8
1116.3

6
5.3
1308.7

Click on the + and drag it down. **This will cause the formulas to change according to how you drag the box.** In this case, if you drag it down to row 6, the spreadsheet will produce the following:
    
    
        A            B              C
    1  t (min)      D (yards)      t*D
    2  1.1         559.5         615.45
    3  1.9         759.5         1443.05
    4  3.0         898.2         2694.6
    5  3.8         1116.3         4241.94
    6  5.3         1308.7         6936.11
    

If you click on the last value in column C (6936.11) the info bar will display:
    
    
    =A6*B6
    

This is very useful for performing the same operations on multiple sets of data at once; rather than having to do the multiplication 5 separate times here, we just do it once and drag down the box.

### Special Functions in Excel

In order to do many mathematical operations in Excel (or at least the easiest way), it is necessary to use _functions_ (not to be confused with formulas). A function is simply an implementation someone already wrote for the mathematical operation, so all you have to do is know how to tell it to do the operation and where to put it when it's done. In excel, you can call a function named "function" by typing the following into a cell:
    
    
    =function(inputs)
    

The function will then execute, and the cell containing the call will display the answer. The necessary inputs are sometimes numbers but are more often the cell addresses. For example, in the data above, say you wanted to take the exponential (![ e^x ](../../../upload.wikimedia.org/math/5/c/f/5cffa5d7a0c145a80dee9dc2295d5cdf.png)) of all the time points in column A, and place the result in column D. The function for exponential is **exp**, and it can only accept one input at a time, but due to the dragging capability of Excel this will not matter much, you can just call it once and then drag the cell _as you would with any formula containing cell addresses_. So to do this you would type into cell D2:
    
    
    =exp(A2)
    

Hit enter, then click the + in the bottom right and drag the cell down. You should end up with something like this after labeling the D column appropriately:
    
    
             A          B            C          D
    1       t (min)   D(yards)     t*D       e^t
    2       1.1      559.5       615.45    3.004166024
    3       1.9      759.5       1443.05   6.685894442
    4       3        898.2       2694.6    20.08553692
    5       3.8      1116.3      4241.94   44.70118449
    6       5.3      1308.7      6936.11   200.33681
    

All excel functions output only one value at a time, though some can accept multiple cells at a time as input (mostly statistical functions).

Following is a brief synopsis of the functions available. For a complete list, see the help files for your spreadsheet, as the availability of each function may vary depending on which one you are using. CELL signifies either the row/column designation of the cell you want to pass to the function as input, or some numerical value you enter manually.

#### Mathematics Functions

Generally these only take one input at a time.
    
    
    abs(CELL): [Absolute value](http://en.wikipedia.org/wiki/Absolute_value) of CELL
    sqrt(CELL): [Square root](http://en.wikipedia.org/wiki/Square_root) of CELL [to do nth roots, use CELL^(1/n)]
    ln(CELL): [Natural log](http://en.wikipedia.org/wiki/Natural_logarithm) of CELL
    log10(CELL): Log of CELL to base 10
    log(CELL, NUM): Log of CELL to the base NUM (use for all bases except e and 10)
    exp(CELL): [Exponential](http://en.wikipedia.org/wiki/Exponential_function)(e^x) of CELL. Use since Excel doesn't have a built-in constant "e".
    sin(CELL), cos(CELL), tan(CELL): [Trigonometric functions](http://en.wikipedia.org/wiki/Trigonometric_functions) sine, cosine, and tangent of CELL. CELL must be in **radians**
    asin(CELL), acos(CELL), atan(CELL): [Inverse trigonometric functions](http://en.wikipedia.org/wiki/Inverse_trigonometric_functions) (returns values in **radians**)
    sinh(CELL), cosh(CELL), tanh(CELL): [Hyperbolic functions](http://en.wikipedia.org/wiki/Hyperbolic_function)
    asinh(CELL), acosh(CELL), atanh(CELL): Inverse hyperbolic functions
    
    

#### Statistics Functions

These are examples of useful statistics functions in Excel, they are not by any means the only ones.

GROUP means a group of cells that are directly next to each other. Define a group by the syntax FIRSTCELL:LASTCELL, for example, using GROUP = A2:A5 passes all the cells between A2 and A5 (inclusive) to the function. If a function requires two different groups (for example, a y and an x), both groups must be within continuous groups of cells.
    
    
    average(CELL1, CELL2, ...) OR average(GROUP): Computes the arithmetic average of all inputs.
    intercept(GROUP1, GROUP2): Calculates the y-intercept (b) of the regression line where y = GROUP1 and x = GROUP2.
       GROUP1 and GROUP2 must have the same size.
    pearson(GROUP1, GROUP2): Calculates the Pearson correlation coefficient (R) between GROUP1 and GROUP2.
    stdev(CELL1, CELL2, ...) OR stdev(GROUP): Computes the _sample_ standard deviation (divides by n-1) of all inputs.
    slope(GROUP1, GROUP2): Calculates the slope (m) of the regression line where y = GROUP1 and x = GROUP2. 
       GROUP1 and GROUP2 must have the same size.
    

#### Programming Functions

## Solving Equations in Spreadsheets: Goal Seek

Excel and possibly other spreadsheets have a very useful tool called _goalseek_ which allows the user to solve single-variable equations (and can be used as an aid in guess-and-check for systems of algebraic equations). Let's suppose for the purposes of this tutorial that you wish to find a solution to the equation:

![ 0 = X^3 + 2X^2 - X + 1 ](../../../upload.wikimedia.org/math/f/c/3/fc34c5de0d861a934985e558be5cbe3e.png)

In order to set up the problem in Goalseek, it is necessary to define a cell for the variable you want to change (X) and a cell for the function you want to evaluate.

**Note:**  
Goalseek will **only work** if you tell it to evaluate some function until it reaches a _constant value_. You cannot tell it to equal something that can change, so for example you cannot do something like this:

![ X - X^3 = 2X^2 + 1 ](../../../upload.wikimedia.org/math/c/4/8/c4850e0bf6e01556d6058998097c4241.png)

because neither side is a constant. The easiest way around this is generally to solve the function for zero and then use that as the evaluating function.

Here, we could set up the cells as follows:
    
    
       A    B          
    1  X    f(X)
    2  -1    =A2^3 + 2*A2^2 - A2 + 1
    

To solve this one, go to
    
    
    Tools > Goalseek...
    

It'll give you three boxes: "Set Cell", "To Value", and "By Changing Cell". Since we want the value in cell B2 to equal 0, enter B2 into the "Set Cell" box and 0 into the "To Value" box. Since cell B2 depends on cell A2, we want to change A2 so that B2 equals 0. Hence, the "By Changing Cell" box should contain A2. Put that in and click "OK", and Goalseek will converge to an answer:
    
    
       A           B          
    1  X          f(X)
    2  -2.54683   -0.00013
    

Notice that _the success of the goalseek depends on what your initial guess was_. If you try to put in an initial guess of 0 in this example (instead of -1), goalseek will diverge. It will tell you so, saying "Goal Seeking with Cell B2 May Not have Found a Solution". However, the algorithm is generally fairly robust so it shouldn't take too many guesses to obtain convergence.

**Note:**  
You can only enter _one cell_ into each of the "Set Cell" and "By Changing Cell" boxes, and the value in "To Value" must be a constant

.

## Graphing Data in Excel

In Excel, there are a variety of ways to graph the data you have inserted, such as bar graphs, pie charts, and many others. The most commonly-used in my experience is the **scatterplot**, which is the name Excel uses for the typical x-y "line graph" plot that you probably think of first when you think of a graph.

### Scatterplots

Scatterplots can be made relating any one independent variable to any number of dependent variables, though if you try to graph too many it will get crowded and hard to read. Excel will automatically give each different dependent variable a different color and a different shape, so that you can distinguish between them. You can also name each "series" of data differently and Excel will automatically set up a legend for you.

This is how to make a scatterplot:

  1. Put the data into columns just like it was given in the problem statement.
  2. Now we need to set up the graph. Go to: ![ Insert \\rightarrow Chart ](../../../upload.wikimedia.org/math/b/c/c/bccd57f841e93e5426f4185b93ccc073.png).
  3. Select "XY (scatter)" and click "next".
  4. Click the "series" tab (on top). If there are any series present, remove them with the remove button (since it usually guesses wrong what you want to graph).

Now we can add a series for each dependent variable we want to graph as follows:

  1. Click "add".
  2. Next to "X values" click the funky arrow symbol to the right of the text box. A small box will pop up.
  3. Click on the first value for the _independent variable_ and drag the mouse down to the last value. Click the funky symbol again to bring you back to the main window.
  4. Do the same thing with the "Y values" but this time you want to select the values of the _dependent variable_.
  5. Click next, and give the graph a title and labels if you want. Then click next and "finish" to generate your graph.

### Performing Regressions of the Data from a Scatterplot

Once you have a scatterplot of your data, you can do one of several types of regression: logarithmic, exponential, polynomial (up to 6th degree), linear, or moving-average. Excel will plot the regression curve against your data automatically, and (except for moving average) you can tell it to give you an equation for the curve. To do this:

  1. Right click on one of the data points (it doesn't matter which). Click "add trendline..."
  2. A new window will come up, asking you for the type of regression. Choose the type of regression you want to use.
  3. Click on the "options" tab, and check the "Display Equation on Chart" box (and, if you want, the "Display R-squared value on Chart" box). Click OK.

If you chose a "linear" regression with the sample data above, the equation and ![ R^2](../../../upload.wikimedia.org/math/c/7/6/c7655279e138e783ceaa224a6a52815a.png) value appear on the graph as ![ y = 177.87x + 391.28, R^2=0.9843 ](../../../upload.wikimedia.org/math/3/4/2/34218646d2e9eca0b307d27caa1e7b2f.png). Note Excel displays ![ R^2](../../../upload.wikimedia.org/math/c/7/6/c7655279e138e783ceaa224a6a52815a.png) rather than R (so that we don't need to worry about negative vs. positive values); if you want R just take the square root, which is 0.9921 as we calculated in the section on linear regressions.

## Further resources for Spreadsheets

Excel and other spreadsheets can do far more than what is described here. For additional information, see [Microsoft Office](http://en.wikibooks.org/wiki/Microsoft_Office), [w:Excel](http://en.wikipedia.org/wiki/Excel), or the help files for the program you are using.

## Introduction to MATLAB

## Inserting and Manipulating Data in MATLAB

### Importing Data from Excel

### Performing Operations on Entire Data Sets

## Graphing Data in MATLAB

### Polynomial Regressions

[MATLAB](http://en.wikibooks.org/wiki/MATLAB) is able to do regressions up to very large polynomial orders, using the "polyfit" function. The syntax for this function is:
    
    
    polyfit(XDATA, YDATA, Order)
    

The x data and y data must be in the form of _arrays_, which for the purposes of this application are simply comma-separated lists separated by brackets. For example, suppose you want to perform the same linear regression that had been performed in the "linear regression" section. The first step is to define the two variables:
    
    
    >> XDATA = [1.1,1.9,3.0,3.8,5.3];
    >> YDATA = [559.5,759.4,898.2,1116.3,1308.7];
    

Then just call polyfit with order '1' since we want a linear regression.
    
    
    >> polyfit(XDATA, YDATA, 1)
    ans = 1.0e+002 *
      1.77876628209900   3.91232582806103
    

The way to interpret this answer is that the first number is the slope of the line (1.778*10^2) and the second is the y-intercept (3.912*10^2).

### Nonlinear Regressions (fminsearch)

# **Appendix 3: Miscellaneous Useful Information**

## Standard vs. Actual Volume

When specifying the volume of a gas, the pressure and temperature _must_ be specified because the volume of a gas depends strongly on both temperature and pressure (assuming that it is in an expandable container). In order to avoid specifying a different set of conditions for each measurement, an engineer can convert to _standard_ temperature and pressure (typically 1 atm and 0oC) which is common to all measurements. This allows direct comparisons of volume measurements, but also requires that one convert back to the actual conditions present in the system before the value can be used.

The conversion that is used assumes that the gas is ideal, so that:

![ PV = nRT ](../../../upload.wikimedia.org/math/e/0/4/e0412dbb6e50de76a383b8f6e7d81dce.png)

### Conversion of volume to volume

We wish to compare a _standard_ state to the _actual_ state in the system. Let us consider the standard state (state "s") first. We have the ideal gas law for the standard state:

![ P_s*V_s = n_sRT_s ](../../../upload.wikimedia.org/math/3/1/c/31c4e0e83b59a93b773263ba15c986b2.png)

Now let us compare the standard state to the actual conditions in the system. The standard state conditions have been completely specified. The ideal gas law is assumed to hold in the actual system conditions as well:

![ P_aV_a = n_aRT_a ](../../../upload.wikimedia.org/math/f/f/7/ff71c7ea066597cf0e3914fd065bc889.png)

Divide this equation by the standard-state ideal gas law gives:

![ \\frac{P_aV_a}{P_s{V}_s} = \\frac{n_aRT_a}{n_sRT_s} ](../../../upload.wikimedia.org/math/c/4/c/c4ce4678eec132e7dd5142826d6b0bd4.png)

where _a_ is actual and _s_ is standard. If we assume that we want to compare the **same number of moles** of the substance between the standard and actual states, the following conversion between the states is obtained:

  
**Conversion from standard to actual volume**  
![ V_a = \\frac{P_sV_s}{T_s}*\\frac{T_a}{P_a} ](../../../upload.wikimedia.org/math/5/c/1/5c1f9b67de1e122e7f396c338e9de5b7.png)  


## "Gauge" Pressure vs. "Absolute" Pressure

## Different types of moles

## "Pound-mass" vs. "Pound-force"

## What is a "Unit Operation"?

A **unit operation** is any part of potentially multiple-step process which can be considered to have a single function. Examples of unit operations include:

  * Separation Processes
  * Purification Processes
  * Mixing Processes
  * Reaction Processes
  * Power Generation Processes
  * Heat Exchangers

In general the ductwork between the processes is not explicitly included, though a single pipe can be analyzed for purposes of determining friction loss, heat losses, pressure drop, and so on.

Large processes are broken into unit operations in order to make them easier to analyze. The key thing to remember about them is that _the conservation laws apply not only to the process as a whole but also to each individual unit operation_.

The purpose of this section is not to show how to design these operations (that's a whole other course) but to give a general idea of how they work.

## Separation Processes

There are a large number of types of separation processes, including distillation, extraction, absorption, membrane filtration, and so on. Each of these can also be used for purification, to varying degrees.

### Separation by Flashing

A mixture of two liquids or a liquid and vapor can be separated by passing it to a **flash drum** at a fixed temperature and pressure. The mixture is allowed to reach equilibrium (or near it), and then the vapor exits the top and the liquid exits the bottom of the drum. This separates the components somewhat, provided that the temperature is chosen between the boiling temperatures of the components of the mixture at the pressure of the drum. The degree of separation depends on the composition of the mixture, the concentrations of the species in the mixture, and the temperature and pressure. Having data such as fugacity data or even vapor pressures for simple modelling like Raoult's Law is invaluable when choosing the operating conditions.

When a solution boils, the resulting gas is **still a mixture**, but the gaseous mixture will in general have more of the lower-boiling compound than the higher-boiling compound. Therefore, a higher-boiling compound can be separated from the lower-boiling compound by simply allowing part of the solution to boil and part to remain as liquid.

### Distillation

Distillation, like flashing, is a process which is generally used to separate a mixture of two or more liquids based on their boiling points. However, what happens in a distillation column is essentially a series of flashes, which are connected with recycle loops. The liquid from each tray comes to equilibrium (ideally) with the vapor, and the vapor rises up to the next tray and the liquid falls to the tray beneath it. Each tray has a different temperature because a reboiler on the bottom and a condenser at the top maintain a temperature gradient across the column (in certain separation setups one of these components is omitted).

Distillation is an unit operation, in which two constituent is separated by different boiling point.

The net result is, like flashing, more of the lower-boiling compound(s) will exit at the top of the column, and more of the higher-boiling compound(s) will fall to the bottoms. Since distillation is multiple flashes in a row, it is typically more effective than a single flash, although the latter may be sufficient depending on the purpose. Distillation columns are standard for many types of separations because it is relatively inexpensive for its efficacy. Distillation has a limit, however: non-ideal mixtures can form **azeotropes**. An azeotrope is a point at which when the solution boils, the vapor has the same composition as the liquid. Therefore no further separation can be done without another method or without using some special tricks.

Two examples of distillation processes are petroleum distillation and the production of alcoholic beverages. In the first case, oil is separated into its many components, with the lightest at the top and the heaviest on the bottom. In the latter, the gas is enriched in ethanol, which is later recondensed.

### Gravitational Separation

Gravitational separation takes advantage of the well-known effect of **density differences**: something that is less dense will float on something that is more dense. Therefore, if two immiscible liquids have significantly different densities, they can be separated by simply letting them settle, then draining the denser liquid out the bottom. Note that the key word here is _immiscible_; if the liquids are soluble in each other, then it is impossible to separate them by this method.

This method can also be used to separate out solids from a liquid mixture, but again the solids must not be soluble in the liquid (or must be less soluble than they are as present in the solution).

### Extraction

Extraction is the general practice of taking something dissolved in one liquid and forcing it to become dissolved in another liquid. This is done by taking advantage of the **relative solubility** of a compound between two liquids. For example, caffeine must be extracted from coffee beans or tea leaves in order to be used in beverages such as coffee or soda. The common method for doing this is to use _supercritical carbon dioxide_, which is able to dissolve caffeine as if it were a liquid. Then, in order to take the caffeine out, the temperature is lowered (lowering the "solubility" in carbon dioxide) and water is injected. The system is then allowed to reach equilibrium. Since caffeine is more soluble in water than it is in carbon dioxide, the majority of it goes into the water.

Extraction is also used for purification, if some solution is contaminated with a pollutant, the pollutant can be extracted with another, clean stream. Even if it is not very soluble, it will still extract _some_ of the pollutant.

Another type of extraction is _acid-base extraction_, which is useful for moving a basic or acidic compound from a polar solvent (such as water) to a non-polar one. Often, the ionized form of the acid or base is soluble in a polar solvent, but the non-ionized form is not as soluble. The reverse is true for the non-ionized form. Therefore, in order to manipulate where the majority of the compound will end up, we alter the pH of the solution by adding acid or base.

For example, suppose you wanted to extract Fluoride (F-) from water into benzene. First, you would add _acid_, because when a strong acid is added to the solution it undergoes the following reaction with fluoride, which is practically irreversible:

![ F^- + H_3O^+ \\rightarrow HF + H_2O ](../../../upload.wikimedia.org/math/c/0/f/c0fe417ff1863b722f4f531abe338661.png)

The hydrogen fluoride is more soluble in benzene than fluoride itself, so it would move into the benzene. The benzene and water fluoride solutions could then be separated by density since they're immiscible.

The term **absorption** is a generalization of extraction that can involve different phases (gas-liquid instead of liquid-liquid). However, the ideas are still the same.

### Membrane Filtration

A membrane is any barrier which allows one substance to pass through it more than another. There are two general types of membrane separators: those which separate based on the **size of the molecules** and those which separate based on **diffusivity**.

An example of the first type of membrane separator is your everyday vacuum cleaner. Vacuum cleaners work by taking in air laden with dust from your carpet. A filter inside the vacuum then traps the dust particles (which are relatively large) and allows the air to pass through it (since air particles are relatively small). A larger-scale operation that works on the same principle is called a **fabric filter** or "Baghouse", which is used in air pollution control or other applications where a solid must be removed from a gas.

Some fancy membranes exist which are able to separate hydrogen from a gaseous mixture by size. These membranes have very small pores which allow hydrogen (the smallest possible molecule, by molecular weight) to pass through by convection, but other molecules cannot pass through the pores and must resort to diffusion (which is comparatively slow). Hence a purified hydrogen mixture results on the other side.

Membranes can separate substances by their diffusivity as well, for example water may diffuse through a certain type of filter faster than ethanol, so if such a filter existed it could be used to enrich the original solution with ethanol.

## Purification Methods

In order to bring any product to market, it is necessary to purify it adequately. Without purification, people could get sick from eating foods, side-reactions could occur in industry which would cause safety concerns, or a scientist's research could be invalidated. Fortunately there are several methods used to purify things. The separation processes mentioned above are often used for this purpose, as are the following two processes:

### Adsorption

Not to be confused with _absorption_, adsorption is a process which separates components by their relative **adhesiveness** to a surface. An adsorption column is essentially a pipe filled with a certain material. When the contaminant flows by, it will bind to the material, and in this way the fluid flowing by is cleaned.

A major disadvantage to this method is that the material will always have a _saturation point_ after which no more contaminant can latch on to it. At this point cleaning stops and therefore the spent material must be replaced by new material.

### Recrystallization

Recrystallization is the purification of substances by taking advantage of changes in solubility with respect to _temperature_.We take advantage of this by dissolving an impure compound and then lowering the temperature _slowly_. The solubility of most solid substances increases with temperature [[3]](http://en.wikipedia.org/wiki/Solubility), so decreasing the temperature will cause the solubility of both the impurities and the substance to be purified to decrease. However, since there is likely much more of the impure substance present than impurities, the impure substance will crystallize out long before the impurities will. Therefore, as long as the temperature is not lowered too quickly, the impure substance will crystallize out in a purer form, while most impurities will remain in solution.

A disadvantage to this method is that it takes a long time to perform, but it is often the most effective method for obtaining a pure sample of a product.

## Reaction Processes

### Plug flow reactors (PFRs) and Packed Bed Reactors (PBRs)

A **plug flow reactor** is a (idealized) reactor in which the reacting fluid flows through a tube at a rapid pace, but without the formation of eddies characteristic of rapid flow. Plug flow reactors tend to be relatively easy to construct (they're essentially pipes) but are problematic in reactions which work better when reactants (or products!) are dilute.

Plug flow reactors can be combined with membrane separators in order to increase the yield of a reactor. The products are selectively pulled out of the reactor as they are made so that the equilibrium in the reactor itself continues to shift towards making more product.

![PFR Membrane diagram.PNG](../../../upload.wikimedia.org/wikibooks/en/0/0f/PFR_Membrane_diagram.PNG)

A packed bed reactor is essentially a plug flow reactor packed with catalyst beads. They are used if, like the majority of reactions in industry, the reaction requires a catalyst to significantly progress at a reasonable temperature.

### Continuous Stirred-Tank Reactors (CSTRs) and Fluidized Bed Reactors (FBs)

A **continuous stirred-tank reactor** is an idealized reactor in which the reactants are dumped in one large tank, allowed to react, and then the products (and unused reactants) are released out of the bottom. In this way the reactants are kept relatively dilute, so the temperatures in the reactor are generally lower. This also can have advantages or disadvantages for the selectivity of the reaction, depending on whether the desired reaction is faster or slower than the undesired one.

![CSTR FB diagram.PNG](../../../upload.wikimedia.org/wikibooks/en/a/a0/CSTR_FB_diagram.PNG)

CSTRs are generally more useful for liquid-phase reactions than PFRs since less transport power is required. However, gas-phase reactions are harder to control in a CSTR.

A **fluidized bed reactor** is, in essence, a CSTR which has been filled with catalyst. The same analogy holds between an FB and CSTR as does between a PFR and a PBR. Unlike CSTRs though, Fluidized beds are commonly used with gases; the gas is pumped in the bottom and bubbles through the catalyst on the way to the top outlet.

### Bioreactors

A **bioreactor** is a reactor that utilizes either a living organism or one or more enzymes from a living organism to accomplish a certain chemical transformation. Bioreactors can be either CSTRs (in which case they are known as **chemostats**) or PFRs.

Certain characteristics of a bioreactor must be more tightly controlled than they must be in a normal CSTR or PFR because cellular enzymes are very complex and have relatively narrow ranges of optimum activity. These include, but are not limited to:

  1. Choice of organism. This is similar to the choice of catalyst for an inorganic reaction.
  2. _Strain_ of the organism. Unlike normal catalysts, organisms are very highly manipulable to produce more of what you're after and less of other products. However, also unlike normal catalysts, they generally require a lot of work to get any significant production at all.
  3. Choice of substrate. Many organisms can utilize many different carbon sources, for example, but may only produce what you want from one of them.
  4. Concentration of substrate and aeration. Two inhibitory effects exist which could prevent you from getting the product you're after. Too much substrate leads to the **glucose effect** in which an organism will ferment regardless of the air supply, while too much air will lead to the **pasteur effect** and a lack of fermentation.
  5. pH and temperature: Bacterial enzymes tend to have a narrow range of optimal pH and temperatures, so these must be carefully controlled.

However, bioreactors have several distinct advantages. One of them is that enzymes tend to be stereospecific, so for example you don't get useless D-sorbose in the production of vitamin C, but you get L-sorbose, which is the active form. In addition, very high production capacities are possible after enough mutations have been induced. Finally, substances which have not been made artificially or which would be very difficult to make artificially (like most antibiotics) can be made relatively easily by a living organism.

## Heat Exchangers

In general, a **heat exchanger** is a device which is used to facilitate the exchange of heat between two mixtures, from the hotter one to the cooler one. Heat exchangers very often involve steam because steam is very good at carrying heat by convection, and it also has a high heat capacity so it won't change temperature as much as another working fluid would. In addition, though steam can be expensive to produce, it is likely to be less expensive than other working fluids since it comes from water.

### Tubular Heat Exchangers

A **tubular heat exchanger** is essentially a jacket around a pipe. The working fluid (often steam) enters the jacket on one side of the heat exchanger and leaves on the other side. Inside the pipe is the mixture which you want to heat or cool. Heat is exchanged through the walls of the device in accordance to the second law of thermodynamics, which requires that heat flow from higher to lower temperatures. Therefore, if it is desired to cool off the fluid in the pipe, the working fluid must be cooler than the fluid in the pipe.

Heat exchangers work because heat naturally flows from higher temperature to lower temperatures. Therefore if a hot fluid and a cold fluid are separated by a heat conducting surface heat can be transferred from the hot fluid to the cold fluid.

Tubular heat exchangers can be set up in two ways: **co-current** or **counter-current**. In a co-current setup, the working fluid and the fluid in the pipe enter on the same side of the heat exchanger. This setup is somewhat inefficient because as heat is exchanged, the temperature of the working fluid will approach that of the fluid in the pipe. The closer the two temperatures become, the less heat can be exchanged. Worse, if the temperatures become equal somewhere in the middle of the heat exchanger, the remaining length is wasted because the two fluids are at thermal equilibrium (no heat is released).

To help counteract these effects, one can use a counter-current setup, in which the working fluid enters the heat exchanger on one end and the fluid in the pipe enters at the _other end_. As an explanation for why this is more efficient, suppose that the working fluid is hotter than the fluid in the pipe, so that the fluid in the pipe is heated up. The fluid in the pipe will be at its highest temperature when it exits the heat exchanger, and at its coolest when it enters. _The working fluid will follow the same trend_ because it cools off as it travels the length of the exchanger. Because it's counter-current, though, the fact that the working fluid cools off has less of an effect because it's exchanging heat with cooler, rather than warmer, fluids in the pipe.

![Clipboard](../../../upload.wikimedia.org/wikipedia/commons/thumb/1/1f/Clipboard.svg/45px-Clipboard.svg.png)

**To do:**  
Draw a diagram to help show how this works.

# **Appendix 4: Notation**

## A Note on Notation

## Base Notation (in alphabetical order)
    
    
    ![ \[i\]_{n} ](../../../upload.wikimedia.org/math/3/f/9/3f9d69d8c3994dc053dd05193f327fd7.png) : Molarity of species i in stream n
    a, b, c, d: Stoichiometric coefficients.
    A: Area
    C: Molar concentration (mol/L)
    K: Equilibrium coefficient
    m: Mass
    MW: Molecular Weight (Molar Mass)
    n: Moles
    n: Number of data points (in statistics section)
    N: Number of components
    P: Pressure
    r: Regression coefficient
    R: Universal gas constant
    T: Temperature
    v: Velocity
    V: Volume
    x: Mole fraction in the _liquid phase_ OR Mass fraction [[1]](Print_Version.html#cite_note-4)
    X: (molar) extent of reaction
    y: Mole fraction in the _gas phase_
    z: Overall composition
    Z: Compressibility
    

  1. [↑](Print_Version.html#cite_ref-4) Unless specified explicitly, **assume that a given percent composition is in terms of the overall flowrate**. So if you're given a flowrate in terms of kg/s and a compositoin of 30%, assume that the 30% is a mass fraction. If a given equation requires one or the other, it will explicitly be stated near the equation which is necessary.

## Greek
    
    
    ![ \\rho ](../../../upload.wikimedia.org/math/f/7/f/f7f177957cf064a93e9811df8fe65ed1.png): Density
    ![ \\Sigma ](../../../upload.wikimedia.org/math/a/6/4/a643a0ef5974b64678111d03125054fc.png): Sum
    

## Subscripts

If a particular component (rather than an arbitrary one) is considered, a specific letter is assigned to it:

  * [A] is the molarity of A
  * ![ x_{A} ](../../../upload.wikimedia.org/math/f/b/e/fbe7005e4a472967fc5e9860109d26a3.png) is the mass fraction of A

Similarly, referring to a specific stream (rather than any old stream you want), each is given a different number.

  * ![ \\dot{n}_1 ](../../../upload.wikimedia.org/math/6/7/e/67e0ff94f21c6647208662f4fdd13545.png) is the molar flowrate in stream 1.
  * ![ \\dot{n}_{A1} ](../../../upload.wikimedia.org/math/6/f/4/6f490dd860eefa566ec04adc9aee4b01.png) is the molar flow rate of component A in stream 1.

**Special subscripts:**

If A is some value denoting a property of an _arbitrary_ component stream, the letter i signifies the arbitrary _component_ and the letter n signifies an arbitrary _stream_, i.e.

  * ![ A_n ](../../../upload.wikimedia.org/math/5/6/1/5614f2f428505eeff293fcebfdc6c8c3.png) is a property of _stream_ n. Note ![ \\dot{n}_n ](../../../upload.wikimedia.org/math/7/0/6/706fc71b924a7252da38a983c066dc5d.png) is the molar flow rate of stream n.
  * ![ A_i ](../../../upload.wikimedia.org/math/e/8/a/e8aaf87d9a5c35b14cfbc370d3fd7b21.png) is a property of _component_ i.

The subscript "gen" signifies generation of something inside the system.

The subscripts "in" and "out" signify flows into and out of the system.

## Embellishments

If A is some value denoting a property then:

![ \\bar{A}_n ](../../../upload.wikimedia.org/math/c/4/f/c4f5dac7530985979d849f04a0f1c58b.png) denotes the _average_ property in stream n

![ \\dot{A}_n ](../../../upload.wikimedia.org/math/1/0/2/1028369d74d11689475e8f93bc8b0fcb.png) denotes a _total flow rate_ in steam n

![ \\dot{A}_{in} ](../../../upload.wikimedia.org/math/4/3/2/43225f0cacd3f49e2d52872d8b1fd71e.png) denotes the flow rate of component i in stream n.

![ \\hat{A} ](../../../upload.wikimedia.org/math/1/d/a/1dab72b4e49f249111b179a8ac68f671.png) indicates a data point in a set.

![ A_i^* ](../../../upload.wikimedia.org/math/2/d/f/2df44d4071a0cba909ca46b6da0428bd.png) is a property of _pure component_ i in a mixture.

## Units Section/Dimensional Analysis

In the units section, the generic variables L, t, m, s, and A are used to demonstrate dimensional analysis. In order to avoid confusing dimensions with units (for example the unit m, meters, is a unit of length, not mass), if this notation is to be used, use the _unit equivalence_ character ![ \\dot= ](../../../upload.wikimedia.org/math/e/0/1/e011dac23072797e8fc73cf575709782.png) rather than a standard equal sign.

## **Appendix 5: Further Reading**

Chapra, S. and Canale, R. 2002. _Numerical Methods for Engineers_, 4th ed. New York: McGraw-Hill.

Felder, R.M. and Rousseau, R.W. 2000. _Elementary Principles of Chemical Processes_, 3rd ed. New York: John Wiley & Sons.

Masterton, W. and Hurley, C. 2001. _Chemistry Principles and Reactions_, 4th ed. New York: Harcourt.

Perry, R.H. and Green, D. 1984. _Perry's Chemical Engineers Handbook_, 6th ed. New York: McGraw-Hill.

Windholz _et al_. 1976. _The Merck Index_, 9th ed. New Jersey: Merck.

[General Chemistry](http://en.wikibooks.org/wiki/General_Chemistry): For a more in-depth analysis of general chemistry

[Matlab](http://en.wikibooks.org/wiki/Matlab): For more information on how to use MATLAB to solve problems.

[Numerical Methods](http://en.wikibooks.org/wiki/Numerical_Methods): For more details on the rootfinding module and other fun math (warning: it's written at a fairly advanced level)

## **Appendix 6: External Links**

**Data Tables**

[Unit conversion table (Wikipedia)](http://en.wikipedia.org/wiki/Conversion_of_units)

[Enthalpies of Formation (Wikipedia)](http://en.wikipedia.org/wiki/Standard_enthalpy_change_of_formation_\(data_table\))

[Periodic Table (Los Alamos National Laboratory)](http://periodic.lanl.gov/default.htm)

[Chemical Sciences Data Tables](http://www.ualberta.ca/~jplambec/che/data/index.htm): Has a fair amount of useful data, including a fairly comprehensive List of Standard Entropies, and Gibbs Energies at 25oC (also a list for ions), a chart with molar masses of the elements, acid equilibrium constants, solubility products, and electric potentials. Definitely one to check out.

[NIST properties](http://webbook.nist.gov/chemistry/fluid/): You can look up properties of many common substances, including water, many light hydrocarbons, and many gases. Data available can include density, enthalpy, entropy, Pitzer accentric factor, surface tension, Joule-Thompson coefficients, and several other variables depending on the substance and conditions selected. To see the data in tabular form, once you enter the temperature and pressure ranges you want, click "view table" and then select the property you want from the pull-down menu. It'll tell you acceptable ranges.

[Generalized compressibility chart](http://www3.me.iastate.edu/me330_colver/new_page_2.htm): This is very useful in the section on gases and liquids, and you should be able to find a copy of this chart in any thermodynamics book or in Perry's handbook. I've linked here so you have some clue what I'm talking about when I write about it.

[Antoine equation coefficients](http://www.engr.umd.edu/~nsw/ench250/antoine.dat), NOTE that these values are for use with a COMMON log, not a NATURAL log.

[Critical constants for various materials](http://www.kayelaby.npl.co.uk/chemistry/3_5/3_5.html)

# Appendix 7: License

![Caution](../../../upload.wikimedia.org/wikipedia/commons/thumb/7/74/Ambox_warning_yellow.svg/40px-Ambox_warning_yellow.svg.png)

As of July 15, 2009 Wikibooks has moved to a dual-licensing system that supersedes the previous GFDL only licensing. In short, this means that text licensed under the GFDL only can no longer be imported to Wikibooks, retroactive to 1 November 2008. Additionally, Wikibooks text might or might not now be exportable under the GFDL depending on whether or not any content was added and not removed since July 15.

Version 1.3, 3 November 2008 Copyright (C) 2000, 2001, 2002, 2007, 2008 Free Software Foundation, Inc. <<http://fsf.org/>>

Everyone is permitted to copy and distribute verbatim copies of this license document, but changing it is not allowed.

## 0\. PREAMBLE

The purpose of this License is to make a manual, textbook, or other functional and useful document "free" in the sense of freedom: to assure everyone the effective freedom to copy and redistribute it, with or without modifying it, either commercially or noncommercially. Secondarily, this License preserves for the author and publisher a way to get credit for their work, while not being considered responsible for modifications made by others.

This License is a kind of "copyleft", which means that derivative works of the document must themselves be free in the same sense. It complements the GNU General Public License, which is a copyleft license designed for free software.

We have designed this License in order to use it for manuals for free software, because free software needs free documentation: a free program should come with manuals providing the same freedoms that the software does. But this License is not limited to software manuals; it can be used for any textual work, regardless of subject matter or whether it is published as a printed book. We recommend this License principally for works whose purpose is instruction or reference.

## 1\. APPLICABILITY AND DEFINITIONS

This License applies to any manual or other work, in any medium, that contains a notice placed by the copyright holder saying it can be distributed under the terms of this License. Such a notice grants a world-wide, royalty-free license, unlimited in duration, to use that work under the conditions stated herein. The "Document", below, refers to any such manual or work. Any member of the public is a licensee, and is addressed as "you". You accept the license if you copy, modify or distribute the work in a way requiring permission under copyright law.

A "Modified Version" of the Document means any work containing the Document or a portion of it, either copied verbatim, or with modifications and/or translated into another language.

A "Secondary Section" is a named appendix or a front-matter section of the Document that deals exclusively with the relationship of the publishers or authors of the Document to the Document's overall subject (or to related matters) and contains nothing that could fall directly within that overall subject. (Thus, if the Document is in part a textbook of mathematics, a Secondary Section may not explain any mathematics.) The relationship could be a matter of historical connection with the subject or with related matters, or of legal, commercial, philosophical, ethical or political position regarding them.

The "Invariant Sections" are certain Secondary Sections whose titles are designated, as being those of Invariant Sections, in the notice that says that the Document is released under this License. If a section does not fit the above definition of Secondary then it is not allowed to be designated as Invariant. The Document may contain zero Invariant Sections. If the Document does not identify any Invariant Sections then there are none.

The "Cover Texts" are certain short passages of text that are listed, as Front-Cover Texts or Back-Cover Texts, in the notice that says that the Document is released under this License. A Front-Cover Text may be at most 5 words, and a Back-Cover Text may be at most 25 words.

A "Transparent" copy of the Document means a machine-readable copy, represented in a format whose specification is available to the general public, that is suitable for revising the document straightforwardly with generic text editors or (for images composed of pixels) generic paint programs or (for drawings) some widely available drawing editor, and that is suitable for input to text formatters or for automatic translation to a variety of formats suitable for input to text formatters. A copy made in an otherwise Transparent file format whose markup, or absence of markup, has been arranged to thwart or discourage subsequent modification by readers is not Transparent. An image format is not Transparent if used for any substantial amount of text. A copy that is not "Transparent" is called "Opaque".

Examples of suitable formats for Transparent copies include plain ASCII without markup, Texinfo input format, LaTeX input format, SGML or XML using a publicly available DTD, and standard-conforming simple HTML, PostScript or PDF designed for human modification. Examples of transparent image formats include PNG, XCF and JPG. Opaque formats include proprietary formats that can be read and edited only by proprietary word processors, SGML or XML for which the DTD and/or processing tools are not generally available, and the machine-generated HTML, PostScript or PDF produced by some word processors for output purposes only.

The "Title Page" means, for a printed book, the title page itself, plus such following pages as are needed to hold, legibly, the material this License requires to appear in the title page. For works in formats which do not have any title page as such, "Title Page" means the text near the most prominent appearance of the work's title, preceding the beginning of the body of the text.

The "publisher" means any person or entity that distributes copies of the Document to the public.

A section "Entitled XYZ" means a named subunit of the Document whose title either is precisely XYZ or contains XYZ in parentheses following text that translates XYZ in another language. (Here XYZ stands for a specific section name mentioned below, such as "Acknowledgements", "Dedications", "Endorsements", or "History".) To "Preserve the Title" of such a section when you modify the Document means that it remains a section "Entitled XYZ" according to this definition.

The Document may include Warranty Disclaimers next to the notice which states that this License applies to the Document. These Warranty Disclaimers are considered to be included by reference in this License, but only as regards disclaiming warranties: any other implication that these Warranty Disclaimers may have is void and has no effect on the meaning of this License.

## 2\. VERBATIM COPYING

You may copy and distribute the Document in any medium, either commercially or noncommercially, provided that this License, the copyright notices, and the license notice saying this License applies to the Document are reproduced in all copies, and that you add no other conditions whatsoever to those of this License. You may not use technical measures to obstruct or control the reading or further copying of the copies you make or distribute. However, you may accept compensation in exchange for copies. If you distribute a large enough number of copies you must also follow the conditions in section 3.

You may also lend copies, under the same conditions stated above, and you may publicly display copies.

## 3\. COPYING IN QUANTITY

If you publish printed copies (or copies in media that commonly have printed covers) of the Document, numbering more than 100, and the Document's license notice requires Cover Texts, you must enclose the copies in covers that carry, clearly and legibly, all these Cover Texts: Front-Cover Texts on the front cover, and Back-Cover Texts on the back cover. Both covers must also clearly and legibly identify you as the publisher of these copies. The front cover must present the full title with all words of the title equally prominent and visible. You may add other material on the covers in addition. Copying with changes limited to the covers, as long as they preserve the title of the Document and satisfy these conditions, can be treated as verbatim copying in other respects.

If the required texts for either cover are too voluminous to fit legibly, you should put the first ones listed (as many as fit reasonably) on the actual cover, and continue the rest onto adjacent pages.

If you publish or distribute Opaque copies of the Document numbering more than 100, you must either include a machine-readable Transparent copy along with each Opaque copy, or state in or with each Opaque copy a computer-network location from which the general network-using public has access to download using public-standard network protocols a complete Transparent copy of the Document, free of added material. If you use the latter option, you must take reasonably prudent steps, when you begin distribution of Opaque copies in quantity, to ensure that this Transparent copy will remain thus accessible at the stated location until at least one year after the last time you distribute an Opaque copy (directly or through your agents or retailers) of that edition to the public.

It is requested, but not required, that you contact the authors of the Document well before redistributing any large number of copies, to give them a chance to provide you with an updated version of the Document.

## 4\. MODIFICATIONS

You may copy and distribute a Modified Version of the Document under the conditions of sections 2 and 3 above, provided that you release the Modified Version under precisely this License, with the Modified Version filling the role of the Document, thus licensing distribution and modification of the Modified Version to whoever possesses a copy of it. In addition, you must do these things in the Modified Version:

  1. Use in the Title Page (and on the covers, if any) a title distinct from that of the Document, and from those of previous versions (which should, if there were any, be listed in the History section of the Document). You may use the same title as a previous version if the original publisher of that version gives permission.
  2. List on the Title Page, as authors, one or more persons or entities responsible for authorship of the modifications in the Modified Version, together with at least five of the principal authors of the Document (all of its principal authors, if it has fewer than five), unless they release you from this requirement.
  3. State on the Title page the name of the publisher of the Modified Version, as the publisher.
  4. Preserve all the copyright notices of the Document.
  5. Add an appropriate copyright notice for your modifications adjacent to the other copyright notices.
  6. Include, immediately after the copyright notices, a license notice giving the public permission to use the Modified Version under the terms of this License, in the form shown in the Addendum below.
  7. Preserve in that license notice the full lists of Invariant Sections and required Cover Texts given in the Document's license notice.
  8. Include an unaltered copy of this License.
  9. Preserve the section Entitled "History", Preserve its Title, and add to it an item stating at least the title, year, new authors, and publisher of the Modified Version as given on the Title Page. If there is no section Entitled "History" in the Document, create one stating the title, year, authors, and publisher of the Document as given on its Title Page, then add an item describing the Modified Version as stated in the previous sentence.
  10. Preserve the network location, if any, given in the Document for public access to a Transparent copy of the Document, and likewise the network locations given in the Document for previous versions it was based on. These may be placed in the "History" section. You may omit a network location for a work that was published at least four years before the Document itself, or if the original publisher of the version it refers to gives permission.
  11. For any section Entitled "Acknowledgements" or "Dedications", Preserve the Title of the section, and preserve in the section all the substance and tone of each of the contributor acknowledgements and/or dedications given therein.
  12. Preserve all the Invariant Sections of the Document, unaltered in their text and in their titles. Section numbers or the equivalent are not considered part of the section titles.
  13. Delete any section Entitled "Endorsements". Such a section may not be included in the Modified version.
  14. Do not retitle any existing section to be Entitled "Endorsements" or to conflict in title with any Invariant Section.
  15. Preserve any Warranty Disclaimers.

If the Modified Version includes new front-matter sections or appendices that qualify as Secondary Sections and contain no material copied from the Document, you may at your option designate some or all of these sections as invariant. To do this, add their titles to the list of Invariant Sections in the Modified Version's license notice. These titles must be distinct from any other section titles.

You may add a section Entitled "Endorsements", provided it contains nothing but endorsements of your Modified Version by various parties—for example, statements of peer review or that the text has been approved by an organization as the authoritative definition of a standard.

You may add a passage of up to five words as a Front-Cover Text, and a passage of up to 25 words as a Back-Cover Text, to the end of the list of Cover Texts in the Modified Version. Only one passage of Front-Cover Text and one of Back-Cover Text may be added by (or through arrangements made by) any one entity. If the Document already includes a cover text for the same cover, previously added by you or by arrangement made by the same entity you are acting on behalf of, you may not add another; but you may replace the old one, on explicit permission from the previous publisher that added the old one.

The author(s) and publisher(s) of the Document do not by this License give permission to use their names for publicity for or to assert or imply endorsement of any Modified Version.

## 5\. COMBINING DOCUMENTS

You may combine the Document with other documents released under this License, under the terms defined in section 4 above for modified versions, provided that you include in the combination all of the Invariant Sections of all of the original documents, unmodified, and list them all as Invariant Sections of your combined work in its license notice, and that you preserve all their Warranty Disclaimers.

The combined work need only contain one copy of this License, and multiple identical Invariant Sections may be replaced with a single copy. If there are multiple Invariant Sections with the same name but different contents, make the title of each such section unique by adding at the end of it, in parentheses, the name of the original author or publisher of that section if known, or else a unique number. Make the same adjustment to the section titles in the list of Invariant Sections in the license notice of the combined work.

In the combination, you must combine any sections Entitled "History" in the various original documents, forming one section Entitled "History"; likewise combine any sections Entitled "Acknowledgements", and any sections Entitled "Dedications". You must delete all sections Entitled "Endorsements".

## 6\. COLLECTIONS OF DOCUMENTS

You may make a collection consisting of the Document and other documents released under this License, and replace the individual copies of this License in the various documents with a single copy that is included in the collection, provided that you follow the rules of this License for verbatim copying of each of the documents in all other respects.

You may extract a single document from such a collection, and distribute it individually under this License, provided you insert a copy of this License into the extracted document, and follow this License in all other respects regarding verbatim copying of that document.

## 7\. AGGREGATION WITH INDEPENDENT WORKS

A compilation of the Document or its derivatives with other separate and independent documents or works, in or on a volume of a storage or distribution medium, is called an "aggregate" if the copyright resulting from the compilation is not used to limit the legal rights of the compilation's users beyond what the individual works permit. When the Document is included in an aggregate, this License does not apply to the other works in the aggregate which are not themselves derivative works of the Document.

If the Cover Text requirement of section 3 is applicable to these copies of the Document, then if the Document is less than one half of the entire aggregate, the Document's Cover Texts may be placed on covers that bracket the Document within the aggregate, or the electronic equivalent of covers if the Document is in electronic form. Otherwise they must appear on printed covers that bracket the whole aggregate.

## 8\. TRANSLATION

Translation is considered a kind of modification, so you may distribute translations of the Document under the terms of section 4. Replacing Invariant Sections with translations requires special permission from their copyright holders, but you may include translations of some or all Invariant Sections in addition to the original versions of these Invariant Sections. You may include a translation of this License, and all the license notices in the Document, and any Warranty Disclaimers, provided that you also include the original English version of this License and the original versions of those notices and disclaimers. In case of a disagreement between the translation and the original version of this License or a notice or disclaimer, the original version will prevail.

If a section in the Document is Entitled "Acknowledgements", "Dedications", or "History", the requirement (section 4) to Preserve its Title (section 1) will typically require changing the actual title.

## 9\. TERMINATION

You may not copy, modify, sublicense, or distribute the Document except as expressly provided under this License. Any attempt otherwise to copy, modify, sublicense, or distribute it is void, and will automatically terminate your rights under this License.

However, if you cease all violation of this License, then your license from a particular copyright holder is reinstated (a) provisionally, unless and until the copyright holder explicitly and finally terminates your license, and (b) permanently, if the copyright holder fails to notify you of the violation by some reasonable means prior to 60 days after the cessation.

Moreover, your license from a particular copyright holder is reinstated permanently if the copyright holder notifies you of the violation by some reasonable means, this is the first time you have received notice of violation of this License (for any work) from that copyright holder, and you cure the violation prior to 30 days after your receipt of the notice.

Termination of your rights under this section does not terminate the licenses of parties who have received copies or rights from you under this License. If your rights have been terminated and not permanently reinstated, receipt of a copy of some or all of the same material does not give you any rights to use it.

## 10\. FUTURE REVISIONS OF THIS LICENSE

The Free Software Foundation may publish new, revised versions of the GNU Free Documentation License from time to time. Such new versions will be similar in spirit to the present version, but may differ in detail to address new problems or concerns. See <http://www.gnu.org/copyleft/>.

Each version of the License is given a distinguishing version number. If the Document specifies that a particular numbered version of this License "or any later version" applies to it, you have the option of following the terms and conditions either of that specified version or of any later version that has been published (not as a draft) by the Free Software Foundation. If the Document does not specify a version number of this License, you may choose any version ever published (not as a draft) by the Free Software Foundation. If the Document specifies that a proxy can decide which future versions of this License can be used, that proxy's public statement of acceptance of a version permanently authorizes you to choose that version for the Document.

## 11\. RELICENSING

"Massive Multiauthor Collaboration Site" (or "MMC Site") means any World Wide Web server that publishes copyrightable works and also provides prominent facilities for anybody to edit those works. A public wiki that anybody can edit is an example of such a server. A "Massive Multiauthor Collaboration" (or "MMC") contained in the site means any set of copyrightable works thus published on the MMC site.

"CC-BY-SA" means the Creative Commons Attribution-Share Alike 3.0 license published by Creative Commons Corporation, a not-for-profit corporation with a principal place of business in San Francisco, California, as well as future copyleft versions of that license published by that same organization.

"Incorporate" means to publish or republish a Document, in whole or in part, as part of another Document.

An MMC is "eligible for relicensing" if it is licensed under this License, and if all works that were first published under this License somewhere other than this MMC, and subsequently incorporated in whole or in part into the MMC, (1) had no cover texts or invariant sections, and (2) were thus incorporated prior to November 1, 2008.

The operator of an MMC Site may republish an MMC contained in the site under CC-BY-SA on the same site at any time before August 1, 2009, provided the MMC is eligible for relicensing.

# How to use this License for your documents

To use this License in a document you have written, include a copy of the License in the document and put the following copyright and license notices just after the title page:

    Copyright (c) YEAR YOUR NAME.
    Permission is granted to copy, distribute and/or modify this document
    under the terms of the GNU Free Documentation License, Version 1.3
    or any later version published by the Free Software Foundation;
    with no Invariant Sections, no Front-Cover Texts, and no Back-Cover Texts.
    A copy of the license is included in the section entitled "GNU
    Free Documentation License".

If you have Invariant Sections, Front-Cover Texts and Back-Cover Texts, replace the "with...Texts." line with this:

    with the Invariant Sections being LIST THEIR TITLES, with the
    Front-Cover Texts being LIST, and with the Back-Cover Texts being LIST.

If you have Invariant Sections without Cover Texts, or some other combination of the three, merge those two alternatives to suit the situation.

If your document contains nontrivial examples of program code, we recommend releasing these examples in parallel under your choice of free software license, such as the GNU General Public License, to permit their use in free software.

![](http://en.wikibooks.org/wiki/Special:CentralAutoLogin/start?type=1x1)

Retrieved from "[http://en.wikibooks.org/w/index.php?title=Introduction_to_Chemical_Engineering_Processes/Print_Version&oldid=2013545](http://en.wikibooks.org/w/index.php?title=Introduction_to_Chemical_Engineering_Processes/Print_Version&oldid=2013545)" 

[Category](http://en.wikibooks.org/wiki/Special:Categories): 

  * [Introduction to Chemical Engineering Processes](http://en.wikibooks.org/wiki/Category:Introduction_to_Chemical_Engineering_Processes)

## Navigation menu

### Personal tools

  * [Create account](http://en.wikibooks.org/w/index.php?title=Special:UserLogin&returnto=Introduction+to+Chemical+Engineering+Processes%2FPrint+Version&type=signup)
  * [Log in](http://en.wikibooks.org/w/index.php?title=Special:UserLogin&returnto=Introduction+to+Chemical+Engineering+Processes%2FPrint+Version)

### Namespaces

  * [Book](Print_Version.html)
  * [Discussion](http://en.wikibooks.org/wiki/Talk:Introduction_to_Chemical_Engineering_Processes/Print_Version)

### 

### Variants

### Views

  * [Read](Print_Version.html)
  * [Edit](http://en.wikibooks.org/w/index.php?title=Introduction_to_Chemical_Engineering_Processes/Print_Version&action=edit)
  * [View history](http://en.wikibooks.org/w/index.php?title=Introduction_to_Chemical_Engineering_Processes/Print_Version&action=history)

### Actions

### Search

### Navigation

  * [Main Page](/wiki/Main_Page)
  * [Help](https://www.mediawiki.org/wiki/Special:MyLanguage/Help:Contents)
  * [Browse](/wiki/Wikibooks:Card_Catalog_Office)
  * [Cookbook](/wiki/Cookbook:Table_of_Contents)
  * [Wikijunior](/wiki/Wikijunior)
  * [Featured books](/wiki/Wikibooks:Featured_books)
  * [Recent changes](/wiki/Special:RecentChanges)
  * [Donations](//donate.wikimedia.org/wiki/Special:FundraiserRedirector?utm_source=donate&utm_medium=sidebar&utm_campaign=C13_en.wikibooks.org&uselang=en)
  * [Random book](/wiki/Special:RandomRootpage)
  * [Using Wikibooks](/wiki/Using_Wikibooks)

### Community

  * [Reading room](/wiki/Wikibooks:Reading_room)
  * ### Navigation

    * [Main Page](http://en.wikibooks.org/wiki/Main_Page)
    * [Help](https://www.mediawiki.org/wiki/Special:MyLanguage/Help:Contents)
    * [Browse](http://en.wikibooks.org/wiki/Wikibooks:Card_Catalog_Office)
    * [Cookbook](http://en.wikibooks.org/wiki/Cookbook:Table_of_Contents)
    * [Wikijunior](http://en.wikibooks.org/wiki/Wikijunior)
    * [Featured books](http://en.wikibooks.org/wiki/Wikibooks:Featured_books)
    * [Recent changes](http://en.wikibooks.org/wiki/Special:RecentChanges)
    * [Donations](http://donate.wikimedia.org/wiki/Special:FundraiserRedirector?utm_source=donate&utm_medium=sidebar&utm_campaign=C13_en.wikibooks.org&uselang=en)
    * [Random book](http://en.wikibooks.org/wiki/Special:RandomRootpage)
    * [Using Wikibooks](http://en.wikibooks.org/wiki/Using_Wikibooks)

### Community

    * [Reading room](http://en.wikibooks.org/wiki/Wikibooks:Reading_room)
    * [Community portal](http://en.wikibooks.org/wiki/Wikibooks:Community_Portal)
    * [Bulletin Board](http://en.wikibooks.org/wiki/Wikibooks:Reading_room/Bulletin_Board)
    * [Help out!](http://en.wikibooks.org/wiki/Wikibooks:Maintenance)
    * [Policies and guidelines](http://en.wikibooks.org/wiki/Wikibooks:Policies_and_guidelines)
    * [Contact us](http://en.wikibooks.org/wiki/Wikibooks:Contact_us)

### Tools

    * [What links here](http://en.wikibooks.org/wiki/Special:WhatLinksHere/Introduction_to_Chemical_Engineering_Processes/Print_Version)
    * [Related changes](http://en.wikibooks.org/wiki/Special:RecentChangesLinked/Introduction_to_Chemical_Engineering_Processes/Print_Version)
    * [Upload file](http://commons.wikimedia.org/wiki/Special:UploadWizard)
    * [Special pages](http://en.wikibooks.org/wiki/Special:SpecialPages)
    * [Permanent link](http://en.wikibooks.org/w/index.php?title=Introduction_to_Chemical_Engineering_Processes/Print_Version&oldid=2013545)
    * [Page information](http://en.wikibooks.org/w/index.php?title=Introduction_to_Chemical_Engineering_Processes/Print_Version&action=info)
    * [Cite this page](http://en.wikibooks.org/w/index.php?title=Special:Cite&page=Introduction_to_Chemical_Engineering_Processes%2FPrint_Version&id=2013545)

### In other languages

    * ### Sister projects

    * [Wikipedia](http://en.wikipedia.org/wiki/Main_Page)
    * [Wikiversity](http://en.wikiversity.org/wiki/Wikiversity:Main_Page)
    * [Wiktionary](http://en.wiktionary.org/wiki/Wiktionary:Main_Page)
    * [Wikiquote](http://en.wikiquote.org/wiki/Main_Page)
    * [Wikisource](http://en.wikisource.org/wiki/Main_Page)
    * [Wikinews](http://en.wikinews.org/wiki/Main_Page)
    * [Wikivoyage](http://en.wikivoyage.org/wiki/Main_Page)
    * [Commons](http://commons.wikimedia.org/wiki/Main_Page)
    * [Wikidata](http://www.wikidata.org/wiki/Wikidata:Main_Page)

### Print/export

    * [Create a collection](http://en.wikibooks.org/w/index.php?title=Special:Book&bookcmd=book_creator&referer=Introduction+to+Chemical+Engineering+Processes%2FPrint+Version)
    * [Download as PDF](http://en.wikibooks.org/w/index.php?title=Special:Book&bookcmd=render_article&arttitle=Introduction+to+Chemical+Engineering+Processes%2FPrint+Version&oldid=2013545&writer=rl)
    * [Printable version](http://en.wikibooks.org/w/index.php?title=Introduction_to_Chemical_Engineering_Processes/Print_Version&printable=yes)

    * This page was last modified on 7 January 2011, at 19:37.
    * Text is available under the [Creative Commons Attribution-ShareAlike License.](http://creativecommons.org/licenses/by-sa/3.0/); additional terms may apply. By using this site, you agree to the [Terms of Use](http://wikimediafoundation.org/wiki/Terms_of_Use) and [Privacy Policy.](http://wikimediafoundation.org/wiki/Privacy_policy)
    * [Privacy policy](http://wikimediafoundation.org/wiki/Privacy_policy)
    * [About Wikibooks](http://en.wikibooks.org/wiki/Wikibooks:Welcome)
    * [Disclaimers](http://en.wikibooks.org/wiki/Wikibooks:General_disclaimer)
    * [Developers](https://www.mediawiki.org/wiki/Special:MyLanguage/How_to_contribute)
    * [Mobile view](http://en.m.wikibooks.org/wiki/Introduction_to_Chemical_Engineering_Processes/Print_Version)
    * ![Wikimedia Foundation](http://bits.wikimedia.org/images/wikimedia-button.png)
    * ![Powered by MediaWiki](http://bits.wikimedia.org/static-1.24wmf1/skins/common/images/poweredby_mediawiki_88x31.png)
